#!/bin/sh

# ******************************************************************************
# * FILE NAME: create_sd.sh
# *
# * DESCRIPTION:
# *  This script file parses the .out executables generated by the diagnostic
# *  makefile into SD card loadable files.
# *
# *  The script expects the board as a parameter. Out2rprc and MulitcoreImageGen
# *  can be set manually by environment variables $OUT2RPRC and 
# *  $MULTICOREIMAGEGEN, respectively. If not set, script will assume default
# *  location in $PDK_INSTALL_PATH/ti/boot/sbl/tools
# *
# *  Syntax:
# *      create_sd.sh [BOARD]
# *
# *  Example:
# *      create_sd.sh idkAM572x
#
# * Copyright (C) 2015, Texas Instruments, Inc.
# *****************************************************************************

board=$1
target=$2
cpu_id=$3
file_ext=*.out

if [ $board = "evmAM335x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "evmAM437x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "idkAM437x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "skAM437x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "bbbAM335x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "icev2AM335x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

if [ $board = "skAM335x" ]
then
    echo Done! create_sd script not needed for $board!
    exit 0
fi

boarddir=$PDK_DIAG_DIR
if [ -d "$boarddir/bin/$board/$target" ]
then
	echo Creating SD files for $board...
else
	echo Missing folder/files under $boarddir/bin/$board/$target
	echo Usage: ./create_sd.sh [BOARD]
	echo     Eg. ./create_sd.sh idkAM572x
	exit
fi

out2rprc=$OUT2RPRC
if [ -z $out2rprc ]
then
	out2rprc=$PDK_INSTALL_PATH/ti/boot/sbl/tools/out2rprc/bin/out2rprc.exe
fi
if [ ! -f $out2rprc ]
then
	echo Missing out2rprc!
	echo     Not found in default location: \$PDK_INSTALL_PATH/ti/boot/sbl/tools/out2rprc/bin/out2rprc.exe
	echo     Override default location by setting OUT2RPRC, eg:
	echo         export OUT2RPRC=$PDK_INSTALL_PATH/ti/boot/sbl/tools/out2rprc/bin/out2rprc.exe
	exit
else
	echo Using out2prc: $out2rprc
fi

multigen=$MULTICOREIMAGEGEN
if [ -z $multigen ]
then
	multigen=$PDK_INSTALL_PATH/ti/boot/sbl/tools/multicoreImageGen/bin/MulticoreImageGen
fi
if [ ! -f $multigen ]
then
	echo Missing multigen!
	echo     Not found in default location: \$PDK_INSTALL_PATH/ti/boot/sbl/tools/multicoreImageGen/bin/MulticoreImageGen
	echo     Override default location by setting MULTICOREIMAGEGEN, eg:
	echo         export MULTICOREIMAGEGEN=$PDK_INSTALL_PATH/ti/boot/sbl/tools/multicoreImageGen/bin/MulticoreImageGen
	exit
else
	echo Using multigen: $multigen
fi

sd_dir=$boarddir/bin/$board/sd

if [ $board = "am65xx_evm" ]
then
    sd_dir=$boarddir/bin/$board/sd/$target
fi

if [ $board = "am65xx_idk" ]
then
    sd_dir=$boarddir/bin/$board/sd/$target
fi

if [ $board = "j721e_evm" ]
then
    sd_dir=$boarddir/bin/$board/sd/$target
	if [ $target = "armv8" ]
	then
		file_ext=*mpu1_0_release.xa72fg
	else
		file_ext=*mcu1_0_release.xer5f
	fi
fi

mkdir -p $sd_dir
outfiles=$boarddir/bin/$board/$target/$file_ext
for out in $outfiles
do
	echo Parsing $out ...
	name=${out##*/}
	name=${name%%_$file_ext}
	rprc=$sd_dir/${name}_rprc
	mono $out2rprc $out $rprc
	$multigen LE 55 $sd_dir/${name}_TEST $cpu_id $rprc
	rm $rprc
done

if [ -e $sd_dir/framework_TEST ]
then
	if [ -e $sd_dir/frameworkLoader_TEST ]
	then
		mv $sd_dir/framework_TEST $sd_dir/framework
		mv $sd_dir/frameworkLoader_TEST $sd_dir/app
	else
		mv $sd_dir/framework_TEST $sd_dir/app
	fi
else
	echo Warning! Diagnostic framework not found!
fi

if [ $board = "am65xx_evm" ] || [ $board = "am65xx_idk" ] || [ $board = "j721e_sim" ] || [ $board = "j721e_evm" ]
then
	x509certgen=$PDK_INSTALL_PATH/ti/build/makerules/x509CertificateGen.sh
	k3devkey=$PDK_INSTALL_PATH/ti/build/makerules/k3_dev_mpk.pem
	echo Signing $sd_dir/app for K3 platform ...
	mv $sd_dir/app $sd_dir/framework_app.unsigned
	$x509certgen -b $sd_dir/framework_app.unsigned -o $sd_dir/app -c R5 -l 0x0 -k $k3devkey
	rm $sd_dir/framework_app.unsigned
fi

echo Done! Check $sd_dir for images to put onto SD card.
