/******************************************************************************
*
* Register header file for USBOTGSS_C2_WRAPPER
*
******************************************************************************
*/
/**
 *  Copyright (c) Texas Instruments Incorporated 2015-2016
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/



#ifndef HW_USBOTGSS_C2_WRAPPER_H_
#define HW_USBOTGSS_C2_WRAPPER_H_

#ifdef __cplusplus
extern "C"
{
#endif


/**************************************************************************
* Register Overlay Structure
**************************************************************************/
typedef struct {
    volatile uint32_t USBOTGSS_REVISION;    // Offset: 0x0
    volatile uint8_t  RSVD0[12];
    volatile uint32_t USBOTGSS_SYSCONFIG;   // Offset: 0x10
    volatile uint8_t  RSVD1[4];
    volatile uint32_t USBOTGSS_IRQ_EOI_MAIN;    // Offset: 0x18
    volatile uint8_t  RSVD2[4];
    volatile uint32_t USBOTGSS_IRQSTS_RAW_MAIN[4];  // Offset: 0x20
    volatile uint32_t USBOTGSS_IRQSTS_MAIN[4];  // Offset: 0x24
    volatile uint32_t USBOTGSS_IRQEN_SET_MAIN[4];   // Offset: 0x28
    volatile uint32_t USBOTGSS_IRQEN_CLR_MAIN[4];   // Offset: 0x2c
    volatile uint8_t  RSVD3[1008];
    volatile uint32_t USBOTGSS_IRQ_EOI_MISC;    // Offset: 0x42c
    volatile uint32_t USBOTGSS_IRQSTS_RAW_MISC; // Offset: 0x430
    volatile uint32_t USBOTGSS_IRQSTS_MISC; // Offset: 0x434
    volatile uint32_t USBOTGSS_IRQEN_SET_MISC;  // Offset: 0x438
    volatile uint32_t USBOTGSS_IRQEN_CLR_MISC;  // Offset: 0x43c
    volatile uint8_t  RSVD4[192];
    volatile uint32_t USBOTGSS_UTMI_CTRL;   // Offset: 0x500
    volatile uint32_t USBOTGSS_UTMI_STS;    // Offset: 0x504
    volatile uint32_t USBOTGSS_TXFIFO_DEPTH;    // Offset: 0x508
    volatile uint32_t USBOTGSS_RXFIFO_DEPTH;    // Offset: 0x50c
    volatile uint8_t  RSVD5[496];
    volatile uint32_t USBOTGSS_MMRAM_OFFSET;    // Offset: 0x700
    volatile uint32_t USBOTGSS_FLADJ;   // Offset: 0x704
    volatile uint32_t USBOTGSS_DBG_CFG; // Offset: 0x708
    volatile uint32_t USBOTGSS_DBG_DATA;    // Offset: 0x70c
    volatile uint32_t USBOTGSS_DEV_EBC_EN;  // Offset: 0x710
} USBOTGSSC2WRAPPERRegs;


/**************************************************************************
* Register Definitions
**************************************************************************/
#define USBOTGSS_C2_WRAPPER_REVISION                            (0x0u)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG                           (0x10u)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN                        (0x18u)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0(n)             (0x20U + ((n-1) * 0x14U))
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0(n)             (0x28U + ((n-1) * 0x14U))
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0(n)             (0x2cU + ((n-1) * 0x14U))
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0(n)                 (0x24U + ((n-1) * 0x14U))
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC                        (0x42Cu)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC                  (0x430U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC                      (0x434U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC                  (0x438U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC                  (0x43cU)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL                       (0x500u)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS                     (0x504u)
#define USBOTGSS_C2_WRAPPER_TXFIFO_DEPTH                        (0x508u)
#define USBOTGSS_C2_WRAPPER_RXFIFO_DEPTH                        (0x50Cu)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET                        (0x700u)
#define USBOTGSS_C2_WRAPPER_FLADJ                               (0x704u)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA                          (0x70Cu)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG                           (0x708u)
#define USBOTGSS_C2_WRAPPER_DEV_EBC_EN                          (0x710u)


/**************************************************************************
* Field Definition Macros
**************************************************************************/

#define USBOTGSS_C2_WRAPPER_REVISION_Y_MINOR_SHIFT              (0U)
#define USBOTGSS_C2_WRAPPER_REVISION_Y_MINOR_MASK               (0x0000003fU)

#define USBOTGSS_C2_WRAPPER_REVISION_CUSTOM_SHIFT               (6U)
#define USBOTGSS_C2_WRAPPER_REVISION_CUSTOM_MASK                (0x000000c0U)

#define USBOTGSS_C2_WRAPPER_REVISION_R_RTL_SHIFT                (11U)
#define USBOTGSS_C2_WRAPPER_REVISION_R_RTL_MASK                 (0x0000f800U)

#define USBOTGSS_C2_WRAPPER_REVISION_FUNC_SHIFT                 (16U)
#define USBOTGSS_C2_WRAPPER_REVISION_FUNC_MASK                  (0x0fff0000U)

#define USBOTGSS_C2_WRAPPER_REVISION_SCHEME_SHIFT               (30U)
#define USBOTGSS_C2_WRAPPER_REVISION_SCHEME_MASK                (0xc0000000U)

#define USBOTGSS_C2_WRAPPER_REVISION_BU_SHIFT                   (28U)
#define USBOTGSS_C2_WRAPPER_REVISION_BU_MASK                    (0x30000000U)

#define USBOTGSS_C2_WRAPPER_REVISION_X_MAJOR_SHIFT              (8U)
#define USBOTGSS_C2_WRAPPER_REVISION_X_MAJOR_MASK               (0x00000700U)

#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_SHIFT            (2U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_MASK             (0x0000000cU)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_FORCE             (0U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_NO                (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_SMART             (2U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_IDLEMODE_SMART_WAKEUP      (3U)

#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_SHIFT         (4U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_MASK          (0x00000030U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_FORCE          (0U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_NO             (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_SMART          (2U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_STANDBYMODE_SMART_WAKEUP   (3U)

#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_SHIFT          (16U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_MASK           (0x00010000U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_DISABLED        (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_ENABLED         (0U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_DISABLE         (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_DMADISABLE_ENABLE          (0U)

#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_SHIFT           (17U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_MASK            (0x00020000U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_DORST            (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_NOOP             (0U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_ACTIVE           (1U)
#define USBOTGSS_C2_WRAPPER_SYSCONFIG_WRAPRESET_DONE             (0U)

#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI0_SHIFT             (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI0_MASK              (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI0_READ0              (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI0                    (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI0_NOOP               (0U)

#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI1_SHIFT             (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI1_MASK              (0x00000002U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI1_READ0              (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI1                    (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI1_NOOP               (0U)

#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI2_SHIFT             (2U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI2_MASK              (0x00000004U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI2_READ0              (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI2                    (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI2_NOOP               (0U)

#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI3_SHIFT             (3U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI3_MASK              (0x00000008U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI3_READ0              (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI3                    (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MAIN_EOI3_NOOP               (0U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MAIN_0_COREIRQ_ST_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN      (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MAIN_0_COREIRQ_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MAIN_0_COREIRQ_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_SHIFT   (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_MASK    (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_NONE     (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_CLR      (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MAIN_0_COREIRQ_ST_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC_EOI_MISC_SHIFT         (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC_EOI_MISC_MASK          (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC_READ0                   (0U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC_IRQ                     (1U)
#define USBOTGSS_C2_WRAPPER_IRQ_EOI_MISC_NOOP                    (0U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_FALL_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_SHIFT  (3U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_MASK  (0x00000008U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_FALL_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_SHIFT  (4U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_MASK  (0x00000010U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_FALL_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_SHIFT  (5U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_MASK  (0x00000020U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_FALL_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_SHIFT  (8U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_MASK  (0x00000100U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_IDPULLUP_RISE_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_SHIFT  (11U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_MASK  (0x00000800U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DISCHRGVBUS_RISE_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_SHIFT  (12U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_MASK  (0x00001000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_CHRGVBUS_RISE_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_SHIFT  (13U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_MASK  (0x00002000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DRVVBUS_RISE_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_SHIFT       (16U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_MASK        (0x00010000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_NONE         (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_NOACTION     (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_SET          (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_OEVT_PENDING      (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_SHIFT  (17U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_MASK  (0x00020000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_SET  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_RAW_MISC_DMADISABLECLR_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_MASK   (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_NONE    (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_FALL_CLEAR   (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_SHIFT  (3U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_MASK  (0x00000008U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_FALL_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_SHIFT  (4U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_MASK   (0x00000010U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_NONE    (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_FALL_CLEAR   (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_SHIFT   (5U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_MASK    (0x00000020U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_NONE     (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_FALL_CLEAR    (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_SHIFT  (8U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_MASK   (0x00000100U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_NONE    (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_IDPULLUP_RISE_CLEAR   (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_SHIFT  (11U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_MASK  (0x00000800U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DISCHRGVBUS_RISE_CLEAR  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_SHIFT           (16U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_MASK            (0x00010000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_NONE             (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_NOACTION         (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_PENDING          (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_OEVT_CLEAR            (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_SHIFT  (17U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_MASK   (0x00020000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_NONE    (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_CLEAR   (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DMADISABLECLR_PENDING  (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_SHIFT  (12U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_MASK   (0x00001000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_NONE    (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_CHRGVBUS_RISE_CLEAR   (1U)

#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_SHIFT   (13U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_MASK    (0x00002000U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_NONE     (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_PENDING  (1U)
#define USBOTGSS_C2_WRAPPER_IRQSTATUS_MISC_DRVVBUS_RISE_CLEAR    (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN_SHIFT  (3U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN_MASK  (0x00000008U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN_SHIFT  (4U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN_MASK  (0x00000010U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN_SHIFT  (5U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN_MASK  (0x00000020U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN   (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN_SHIFT  (8U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN_MASK  (0x00000100U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_IDPULLUP_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN_SHIFT  (11U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN_MASK  (0x00000800U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DISCHRGVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN_SHIFT  (12U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN_MASK  (0x00001000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_CHRGVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN_SHIFT  (13U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN_MASK  (0x00002000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN   (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DRVVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN_SHIFT    (16U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN_MASK     (0x00010000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN           (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_OEVT_EN_ENABLED   (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN_SHIFT  (17U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN_MASK  (0x00020000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_SET_MISC_DMADISABLECLR_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_SHIFT  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_MASK  (0x00000001U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_SHIFT  (3U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_MASK  (0x00000008U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_SHIFT  (4U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_MASK  (0x00000010U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_SHIFT  (5U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_MASK  (0x00000020U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_FALL_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_SHIFT  (8U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_MASK  (0x00000100U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_IDPULLUP_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_SHIFT  (11U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_MASK  (0x00000800U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DISCHRGVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_SHIFT  (12U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_MASK  (0x00001000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_CHRGVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_SHIFT  (13U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_MASK  (0x00002000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DRVVBUS_RISE_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_SHIFT    (16U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_MASK     (0x00010000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_CLEAR     (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_OEVT_EN_ENABLED   (1U)

#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_SHIFT  (17U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_MASK  (0x00020000U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_DISABLED  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_NOACTION  (0U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_CLEAR  (1U)
#define USBOTGSS_C2_WRAPPER_IRQENABLE_CLR_MISC_DMADISABLECLR_EN_ENABLED  (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_IDPULLUP_SHIFT        (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_IDPULLUP_MASK         (0x00000001U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_IDPULLUP_DISABLE       (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_IDPULLUP_ENABLE        (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DISCHRGVBUS_SHIFT     (3U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DISCHRGVBUS_MASK      (0x00000008U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DISCHRGVBUS_DISCHARGE  (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DISCHRGVBUS_NOACTION   (0U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_CHRGVBUS_SHIFT        (4U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_CHRGVBUS_MASK         (0x00000010U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_CHRGVBUS_CHARGE        (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_CHRGVBUS_NOACTION      (0U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DRVVBUS_SHIFT         (5U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DRVVBUS_MASK          (0x00000020U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DRVVBUS_DRIVE          (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_CTRL_DRVVBUS_NOACTION       (0U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SW_MODE_SHIFT       (31U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SW_MODE_MASK        (0x80000000U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SW_MODE_IO           (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SW_MODE              (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_VBUSVALID_SHIFT     (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_VBUSVALID_MASK      (0x00000002U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_VBUSVALID_VALID      (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_VBUSVALID_NOTVALID   (0U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSVALID_SHIFT     (2U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSVALID_MASK      (0x00000004U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSVALID_NOTVALID   (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSVALID_VALID      (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSEND_SHIFT       (3U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSEND_MASK        (0x00000008U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSEND_ENDED        (1U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_SESSEND_NOTENDED     (0U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_IDDIG_SHIFT         (4U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_IDDIG_MASK          (0x00000010U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_IDDIG_IDA            (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_IDDIG_IDB            (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_TXBITSTUFFEN_SHIFT  (8U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_TXBITSTUFFEN_MASK   (0x00000100U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_TXBITSTUFFEN_NOBS    (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_TXBITSTUFFEN_BS      (1U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_POWERPRESENT_SHIFT  (9U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_POWERPRESENT_MASK   (0x00000200U)

#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_PORT_OVERCURRENT_SHIFT  (10U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_PORT_OVERCURRENT_MASK  (0x00000400U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_PORT_OVERCURRENT_NONE  (0U)
#define USBOTGSS_C2_WRAPPER_UTMI_OTG_STATUS_PORT_OVERCURRENT_OC  (1U)

#define USBOTGSS_C2_WRAPPER_TXFIFO_DEPTH_TXFIFO_DEPTH_SHIFT     (0U)
#define USBOTGSS_C2_WRAPPER_TXFIFO_DEPTH_TXFIFO_DEPTH_MASK      (0x0000ffffU)

#define USBOTGSS_C2_WRAPPER_RXFIFO_DEPTH_RXFIFO_DEPTH_SHIFT     (0U)
#define USBOTGSS_C2_WRAPPER_RXFIFO_DEPTH_RXFIFO_DEPTH_MASK      (0x0000ffffU)

#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_OFFSET_MSB_SHIFT       (15U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_OFFSET_MSB_MASK        (0x000f8000U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_MSB_RAM0_BASE           (8U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_MSB_RAM1_BASE           (16U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_MSB_RAM2_BASE           (24U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_MSB_CORE_BOT            (0U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_MSB_CORE_TOP            (1U)

#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_OFFSET_LSB_SHIFT       (0U)
#define USBOTGSS_C2_WRAPPER_MMRAM_OFFSET_OFFSET_LSB_MASK        (0x00007fffU)

#define USBOTGSS_C2_WRAPPER_FLADJ_FLADJ_30MHZ_SHIFT             (21U)
#define USBOTGSS_C2_WRAPPER_FLADJ_FLADJ_30MHZ_MASK              (0x07e00000U)

#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U2_PORT_DISABLE_SHIFT    (27U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U2_PORT_DISABLE_MASK     (0x08000000U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U2_PORT_DISABLE_EN        (0U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U2_PORT_DISABLE_DIS       (1U)

#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U3_PORT_DISABLE_SHIFT    (28U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U3_PORT_DISABLE_MASK     (0x10000000U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U3_PORT_DISABLE_EN        (0U)
#define USBOTGSS_C2_WRAPPER_FLADJ_HOST_U3_PORT_DISABLE_DIS       (1U)

#define USBOTGSS_C2_WRAPPER_FLADJ_XHCI_REVISION_SHIFT           (29U)
#define USBOTGSS_C2_WRAPPER_FLADJ_XHCI_REVISION_MASK            (0x20000000U)
#define USBOTGSS_C2_WRAPPER_FLADJ_XHCI_REVISION_0_96             (0U)
#define USBOTGSS_C2_WRAPPER_FLADJ_XHCI_REVISION_1_0              (1U)

#define USBOTGSS_C2_WRAPPER_FLADJ_CORE_SW_RESET_SHIFT           (31U)
#define USBOTGSS_C2_WRAPPER_FLADJ_CORE_SW_RESET_MASK            (0x80000000U)
#define USBOTGSS_C2_WRAPPER_FLADJ_CORE_SW_RESET_NORESET          (0U)
#define USBOTGSS_C2_WRAPPER_FLADJ_CORE_SW_RESET                  (1U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG0_SHIFT               (0U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG0_MASK                (0x00000001U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG1_SHIFT               (1U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG1_MASK                (0x00000002U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG2_SHIFT               (2U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG2_MASK                (0x00000004U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG3_SHIFT               (3U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG3_MASK                (0x00000008U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG4_SHIFT               (4U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG4_MASK                (0x00000010U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG5_SHIFT               (5U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG5_MASK                (0x00000020U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG6_SHIFT               (6U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG6_MASK                (0x00000040U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG7_SHIFT               (7U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG7_MASK                (0x00000080U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG8_SHIFT               (8U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG8_MASK                (0x00000100U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG9_SHIFT               (9U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG9_MASK                (0x00000200U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG10_SHIFT              (10U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG10_MASK               (0x00000400U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG11_SHIFT              (11U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG11_MASK               (0x00000800U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG12_SHIFT              (12U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG12_MASK               (0x00001000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG13_SHIFT              (13U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG13_MASK               (0x00002000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG14_SHIFT              (14U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG14_MASK               (0x00004000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG15_SHIFT              (15U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG15_MASK               (0x00008000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG16_SHIFT              (16U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG16_MASK               (0x00010000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG17_SHIFT              (17U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG17_MASK               (0x00020000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG18_SHIFT              (18U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG18_MASK               (0x00040000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG19_SHIFT              (19U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG19_MASK               (0x00080000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG20_SHIFT              (20U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG20_MASK               (0x00100000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG21_SHIFT              (21U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG21_MASK               (0x00200000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG22_SHIFT              (22U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG22_MASK               (0x00400000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG23_SHIFT              (23U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG23_MASK               (0x00800000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG24_SHIFT              (24U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG24_MASK               (0x01000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG25_SHIFT              (25U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG25_MASK               (0x02000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG26_SHIFT              (26U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG26_MASK               (0x04000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG27_SHIFT              (27U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG27_MASK               (0x08000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG28_SHIFT              (28U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG28_MASK               (0x10000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG29_SHIFT              (29U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG29_MASK               (0x20000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG30_SHIFT              (30U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG30_MASK               (0x40000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG31_SHIFT              (31U)
#define USBOTGSS_C2_WRAPPER_DEBUG_DATA_DBG31_MASK               (0x80000000U)

#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_SHIFT                 (0U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_MASK                  (0x00000007U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_TIELO                  (0U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_UTMI                   (1U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_PIPE                   (2U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_CORE                   (3U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_TRACE_LO               (4U)
#define USBOTGSS_C2_WRAPPER_DEBUG_CFG_SEL_TRACE_HI               (5U)

#define USBOTGSS_C2_WRAPPER_DEV_EBC_EN_INEP_SHIFT               (14U)
#define USBOTGSS_C2_WRAPPER_DEV_EBC_EN_INEP_MASK                (0x0000c000U)

#define USBOTGSS_C2_WRAPPER_DEV_EBC_EN_OUTEP_SHIFT              (30U)
#define USBOTGSS_C2_WRAPPER_DEV_EBC_EN_OUTEP_MASK               (0xc0000000U)

#ifdef __cplusplus
}
#endif
#endif /* HW_USBOTGSS_C2_WRAPPER_H_ */
