/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2012-2013
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#define mkptr(base,offset)                ((volatile unsigned int *)((base)+(offset)))
#define sleep(ticks)    {volatile int i; for (i=0; i < ticks; i++);}
#define UINT unsigned int

UINT dfe_cfg_base =  0x24000000; //MMR base address of DFE
#define MASK 0x80

static void vbusp_rmw32(UINT addr, UINT data, UINT mask)
{
 UINT rdata;
  rdata= *mkptr(addr,0x0);
	sleep(1);
  *mkptr(addr, 0x0)=((rdata &~mask)|(data & mask));
}

static void vbusp_write32(UINT addr, UINT data)
{
 sleep(1);
*mkptr(addr, 0x0)=data;
}

static UINT vbusp_read32(UINT addr)
{
 sleep(1);
 return (*mkptr(addr, 0x0));
}

#ifdef _TMS320C6X
#pragma CODE_SECTION(dfe_init,".text:dfe");
#endif

void dfe_init()
{
        UINT read_data;
        //UINT dduc_tx =0x0;
        UINT dduc_tx0=0;
        UINT dduc_tx1=0;
        UINT dduc_tx2=0;
        UINT dduc_tx3=0;


        ////////initialization sequence//////

        //First, be sure the bb's frame start logic is set to never sync
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040370, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040374, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040378, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x0004037c, 0x00000000);
        sleep(25);

        //Release mem_mpu_access for all blocks
        vbusp_write32(dfe_cfg_base + 0x01f00000 + 0x00000008, 0x00000000);

        //First, determine which dducs are in tx and which are rx.
        read_data = vbusp_read32 (dfe_cfg_base + 0x00100000 + 0x00000400);
//	dduc_tx |= ((read_data & MASK)>>7);
	dduc_tx0= ((read_data & MASK)>>7);


        read_data = vbusp_read32 (dfe_cfg_base + 0x00180000 + 0x00000400);
//        dduc_tx |=(((read_data & MASK)>>7)<<1);
	dduc_tx1= ((read_data & MASK)>>7);

        read_data = vbusp_read32 (dfe_cfg_base + 0x00200000 + 0x00000400);
//	dduc_tx |= (((read_data & MASK)>>7)<<2);
	dduc_tx2= ((read_data & MASK)>>7);

        read_data = vbusp_read32 (dfe_cfg_base + 0x00280000 + 0x00000400);
	dduc_tx3= ((read_data & MASK)>>7);
//	dduc_tx |= (((read_data & MASK)>>7)<<3);

        //Release each block in signal processing order. Each given block releases init_clk_gate,
        //then init_state, then clear_data, with delays between each release.
        //misc
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000000f, 0x0000007f);
        sleep(25);

        //cb (being done first to support cb sourcing, so sourcing can begin before blocks are initialized)
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000000f, 0x0000007f);
        sleep(25);

        //bb
        //***hmm...all blocks operate in only 1 direction EXCEPT for bb...how to release it? Doing it first for now...
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000000f, 0x0000007f);
        sleep(25);

        //tx dducs (dducs must be released at the same time; use mpu_sync)
        if (dduc_tx0==1) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx1==1) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx2==1) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx3==1) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        if (dduc_tx0==1) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx1==1) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx2==1) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx3==1) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        if (dduc_tx0==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00001111);
        }

        if (dduc_tx1==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00001111);
        }
        if (dduc_tx2==1)
        {
	    vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00001111);
        }

        if (dduc_tx3==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00001111);
        }
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        if (dduc_tx0==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx1==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx2==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx3==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx0==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx1==1)
		vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx2==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx3==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000001, 0x0000007f);

        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //summer (does not have init_clk_gate)
        vbusp_rmw32 (dfe_cfg_base + 0x00900000 + 0x00000014, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00900000 + 0x00000014, 0x0000000f, 0x0000007f);
        sleep(25);

        //acl
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000000f, 0x0000007f);
        sleep(25);

        //cfrs (both cfrs must be released at the same time; use mpu_sync)
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000061, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000041, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000600, 0x00000001); //pre mult sync
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000600, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000604, 0x00000001); //post mult sync
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000604, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000600, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000600, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000604, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000604, 0x00000000);
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000001, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000001, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //cdfr
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000000f, 0x0000007f);
        sleep(25);

        //dpda
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000000f, 0x0000007f);
        sleep(25);

        //dpd
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000000f, 0x0000007f);
        sleep(25);

        //tx
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000000f, 0x0000007f);
        sleep(25);

        //jesdtx
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000000f, 0x00000f7f); //also release per-lane clear_data
        sleep(25);

        //rx (needs to be released BEFORE jesdrx ***I think? This was needed in Appleton RX...)
        //***wait, do the dc accumulator syncs need to be sent AFTER the frames start arriving?
        //***even if rx needs to have clear_data and init_state released before frames arrive, does it have to be AFTER cken arrives?
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000310, 0x00000010, 0x000000f0); //dc accumulator syncs
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00001111);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000310, 0x00000000, 0x000000f0);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000000);
        vbusp_rmw32 (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000000f, 0x0000007f);
        sleep(25);

        //fb (needs to be released BEFORE jesdrx ***I think? This was needed in Appleton RX...)
        //***wait, do the dc accumulator syncs need to be sent AFTER the frames start arriving?
        //***even if fb needs to have clear_data and init_state released before frames arrive, does it have to be AFTER cken arrives?
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000388, 0x00000001); //dc accumulator syncs
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000388, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000000);
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000000f, 0x0000007f);
        sleep(25);

        //jesdrx
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000000f, 0x00000f7f); //also release per-lane clear_data
        sleep(25);

        //rx dducs (dducs must be released at the same time; use mpu_sync)
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        if (!(dduc_tx0==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000001);
        } 
        if (!(dduc_tx1==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000001);
        }
        if (!(dduc_tx2==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000001);
        }
        if (!(dduc_tx3==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000001);
        }
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        if (!(dduc_tx0==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx1==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx2==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx3==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000001, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //Finally, set the bb's frame start logic to look at the IQN2's 5/10/20ms
        //(depending on the standard) frame.
        //***also, for now the mux for WHICH strobe is used here is assumed to be done in the .reg file...
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040370, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040374, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040378, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x0004037c, 0x00002222);
        sleep(25);

}

UINT addrTable[18][2] =
{
		{0x00000000,  0x0006C0F4},
		{0x00100000,  0x001438BC},
		{0x00180000,  0x001C38BC},
		{0x00200000,  0x002438BC},
		{0x00280000,  0x002C38BC},
		{0x00900000,  0x00900074},
		{0x00B00000,  0x00B187FC},
		{0x00B80000,  0x00B987FC},
		{0x00F00000,  0x00F00000},
		{0x01000000,  0x01001118},
		{0x01300000,  0x01363FFC},
		{0x01500000,  0x01500000},
		{0x01600000,  0x01640BFC},
		{0x01A00000,  0x01A423FC},
		{0x01C00000,  0x01C00488},
		{0x01D00000,  0x01D42DD0},
		{0x01E80000,  0x01EC09FC},
		{0x01F00000,  0x01F02520}
};


#ifdef _TMS320C6X
#pragma CODE_SECTION(preloadTgtcfg_local,".text:dfe");
#endif
void preloadTgtcfg_local()
{
    UINT ui, i;

    for(i = 0; i < 18; i++)
		if(addrTable[i][1]>addrTable[i][0])
			for(ui = addrTable[i][0]; ui<=addrTable[i][1]; ui+=4)
			{
				*mkptr( dfe_cfg_base, ui ) = 0x0;
			}

    return;
}

#if DFE_BB_LOOPBACK == 1
static void iqn2_bb_loopback_config_split_1();
static void iqn2_bb_loopback_config_split_2();
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config,".text:dfe");
#endif
void iqn2_bb_loopback_config()
{
	// base on P1_5_LTE5_WCDMA_spDL_bufrlpbk.reg
	*mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
	*mkptr( dfe_cfg_base, 0x00040208 ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x0004020C ) = 0x00004080;
	*mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x00040214 ) = 0x00004008;
	*mkptr( dfe_cfg_base, 0x00040218 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00040228 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00040240 ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x00040244 ) = 0x00008088;
	*mkptr( dfe_cfg_base, 0x00040248 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00040250 ) = 0x00002052;
	*mkptr( dfe_cfg_base, 0x00040254 ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x0004027C ) = 0x00004020;
	*mkptr( dfe_cfg_base, 0x00040280 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x00040288 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x000402C0 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x000402C4 ) = 0x00000022;
	*mkptr( dfe_cfg_base, 0x000402D0 ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x000402D4 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00040344 ) = 0x0000000A;      //0x00000005;
	*mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x0004036C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00040370 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00040374 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00040388 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00040394 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00040440 ) = 0x000001D2;
	*mkptr( dfe_cfg_base, 0x00040448 ) = 0x0000017C;
	*mkptr( dfe_cfg_base, 0x00040450 ) = 0x00000180;
	*mkptr( dfe_cfg_base, 0x00040480 ) = 0x000001D4;
	*mkptr( dfe_cfg_base, 0x00040488 ) = 0x000005FC;
	*mkptr( dfe_cfg_base, 0x00040490 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x000404A0 ) = 0x000001D5;
	*mkptr( dfe_cfg_base, 0x000404A8 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x000404B0 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x00040800 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000D8B;
	*mkptr( dfe_cfg_base, 0x00040840 ) = 0x000000D2;
	*mkptr( dfe_cfg_base, 0x00040848 ) = 0x0000017C;
	*mkptr( dfe_cfg_base, 0x00040850 ) = 0x0000017C;
	*mkptr( dfe_cfg_base, 0x00040858 ) = 0x00000D11;
	*mkptr( dfe_cfg_base, 0x000408C0 ) = 0x000000D5;
	*mkptr( dfe_cfg_base, 0x000408C8 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x000408D0 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x000408D8 ) = 0x00000CD6;
	*mkptr( dfe_cfg_base, 0x00041200 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x0004120C ) = 0x0000017C;
	*mkptr( dfe_cfg_base, 0x00041210 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00041220 ) = 0x00000551;
	*mkptr( dfe_cfg_base, 0x0004122C ) = 0x0000017C;
	*mkptr( dfe_cfg_base, 0x00041230 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00040600 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00040A00 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00060000 ) = 0x0000D200;
	*mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060008 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060010 ) = 0x0000D201;
	*mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060018 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060020 ) = 0x0000D202;
	*mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060028 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060030 ) = 0x0000D203;
	*mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060038 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060040 ) = 0x0000D204;
	*mkptr( dfe_cfg_base, 0x00060044 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060048 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060050 ) = 0x0000D205;
	*mkptr( dfe_cfg_base, 0x00060054 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060058 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060060 ) = 0x0000D206;
	*mkptr( dfe_cfg_base, 0x00060064 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060068 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060070 ) = 0x0000D207;
	*mkptr( dfe_cfg_base, 0x00060074 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060078 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00060080 ) = 0x0000D510;
	*mkptr( dfe_cfg_base, 0x00060084 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060088 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x00060090 ) = 0x0000D511;
	*mkptr( dfe_cfg_base, 0x00060094 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060098 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600A0 ) = 0x0000D512;
	*mkptr( dfe_cfg_base, 0x000600A4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600A8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600B0 ) = 0x0000D513;
	*mkptr( dfe_cfg_base, 0x000600B4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600B8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600C0 ) = 0x0000D514;
	*mkptr( dfe_cfg_base, 0x000600C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600C8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600D0 ) = 0x0000D515;
	*mkptr( dfe_cfg_base, 0x000600D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600D8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600E0 ) = 0x0000D516;
	*mkptr( dfe_cfg_base, 0x000600E4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600E8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x000600F0 ) = 0x0000D517;
	*mkptr( dfe_cfg_base, 0x000600F4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x000600F8 ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008230;
	*mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008231;
	*mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008232;
	*mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008233;
	*mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x00060880 ) = 0x00008234;
	*mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x000608A0 ) = 0x00008235;
	*mkptr( dfe_cfg_base, 0x000608A4 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x000608C0 ) = 0x00008236;
	*mkptr( dfe_cfg_base, 0x000608C4 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x000608E0 ) = 0x00008237;
	*mkptr( dfe_cfg_base, 0x000608E4 ) = 0x0000F012;
	*mkptr( dfe_cfg_base, 0x00060900 ) = 0x00008560;
	*mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x00060920 ) = 0x00008561;
	*mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x00060940 ) = 0x00008562;
	*mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x00060960 ) = 0x00008563;
	*mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x00060980 ) = 0x00008564;
	*mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x000609A0 ) = 0x00008565;
	*mkptr( dfe_cfg_base, 0x000609A4 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x000609C0 ) = 0x00008566;
	*mkptr( dfe_cfg_base, 0x000609C4 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x000609E0 ) = 0x00008567;
	*mkptr( dfe_cfg_base, 0x000609E4 ) = 0x0000F016;
	*mkptr( dfe_cfg_base, 0x00060A04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x0006100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00061010 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00061014 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00061018 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x0006101C ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00061020 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00061024 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000000A;
	*mkptr( dfe_cfg_base, 0x0006102C ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x0006103C ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006104C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006105C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006106C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006107C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061084 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061088 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x0006108C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00061090 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00061094 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00061098 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x0006109C ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x000610A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006110C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006111C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006112C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006113C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006114C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006115C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006116C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006117C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006118C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006119C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006120C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006121C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006122C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006123C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006124C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006125C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006126C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006127C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006128C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006129C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006130C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006131C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006132C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006133C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006134C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006135C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006136C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006137C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006138C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006139C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614A0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614A8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614B0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614B8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614C0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614C8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614D0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614D8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614E0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614E8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614F8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061800 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061808 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061810 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061818 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618A0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618A8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618B0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618B8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618E0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618E8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618F0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618F8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619A0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619A8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619B0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619B8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619E0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619E8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619F0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619F8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A00 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A08 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A10 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A18 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A20 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A28 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A30 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A38 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061C00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ECC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006200C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006201C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006202C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006203C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006204C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006205C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006206C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006207C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006208C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006209C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x0006240C ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x00062410 ) = 0x00000204;
	*mkptr( dfe_cfg_base, 0x00062414 ) = 0x00000205;
	*mkptr( dfe_cfg_base, 0x00062418 ) = 0x00000206;
	*mkptr( dfe_cfg_base, 0x0006241C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x00062420 ) = 0x00000508;
	*mkptr( dfe_cfg_base, 0x00062424 ) = 0x00000509;
	*mkptr( dfe_cfg_base, 0x00062428 ) = 0x0000050A;
	*mkptr( dfe_cfg_base, 0x0006242C ) = 0x0000050B;
	*mkptr( dfe_cfg_base, 0x00062430 ) = 0x0000050C;
	*mkptr( dfe_cfg_base, 0x00062434 ) = 0x0000050D;
	*mkptr( dfe_cfg_base, 0x00062438 ) = 0x0000050E;
	*mkptr( dfe_cfg_base, 0x0006243C ) = 0x0000050F;
	*mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006244C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006245C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006246C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006247C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062480 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00062484 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x00062488 ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x0006248C ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x00062490 ) = 0x00000204;
	*mkptr( dfe_cfg_base, 0x00062494 ) = 0x00000205;
	*mkptr( dfe_cfg_base, 0x00062498 ) = 0x00000206;
	*mkptr( dfe_cfg_base, 0x0006249C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x000624A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006250C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006251C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006252C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006253C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006254C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006255C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006256C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006257C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006258C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006259C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006260C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006261C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006262C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006263C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006264C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006265C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006266C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006267C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006268C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006269C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006270C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006271C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006272C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006273C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006274C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006275C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006276C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006277C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006278C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006279C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006280C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006281C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006282C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006283C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006284C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006285C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006286C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006287C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006288C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006289C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006290C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006291C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006292C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006293C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006294C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006295C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006296C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006297C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006298C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006299C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ABC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ACC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ADC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062C00 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C04 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C08 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C0C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C10 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C14 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C18 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C1C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C20 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C24 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C28 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C2C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C30 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C34 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C38 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C3C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C40 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C44 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C48 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C4C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C50 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C54 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C58 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C5C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C60 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C64 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C68 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C6C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C70 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C74 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C78 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C7C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C80 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C84 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C88 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C8C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C90 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C94 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C98 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C9C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CAC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CBC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006300C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006301C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006302C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006303C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006304C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006305C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006306C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006307C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006308C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006309C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006310C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006311C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006312C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006313C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006314C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006315C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006316C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006317C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006318C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006319C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006320C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006321C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006322C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006323C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006324C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006325C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006326C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006327C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006328C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006329C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006330C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006331C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006332C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006333C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006334C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006335C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006336C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006337C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006338C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006339C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006340C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006341C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006342C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006343C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006344C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006345C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006346C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006347C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006348C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006349C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006350C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006351C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006352C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006353C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006354C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006355C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006356C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006357C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006358C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006359C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x0006400C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x0006401C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x0006402C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x0006403C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
	*mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
	*mkptr( dfe_cfg_base, 0x0006404C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
	*mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
	*mkptr( dfe_cfg_base, 0x0006405C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
	*mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001A00;
	*mkptr( dfe_cfg_base, 0x0006406C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001C00;
	*mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001E00;
	*mkptr( dfe_cfg_base, 0x0006407C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
	*mkptr( dfe_cfg_base, 0x0006408C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
	*mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
	*mkptr( dfe_cfg_base, 0x0006409C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640A0 ) = 0x00002800;
	*mkptr( dfe_cfg_base, 0x000640A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640A8 ) = 0x00002A00;
	*mkptr( dfe_cfg_base, 0x000640AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640B0 ) = 0x00002C00;
	*mkptr( dfe_cfg_base, 0x000640B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640B8 ) = 0x00002E00;
	*mkptr( dfe_cfg_base, 0x000640BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640C0 ) = 0x00003000;
	*mkptr( dfe_cfg_base, 0x000640C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640C8 ) = 0x00003200;
	*mkptr( dfe_cfg_base, 0x000640CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640D0 ) = 0x00003400;
	*mkptr( dfe_cfg_base, 0x000640D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640D8 ) = 0x00003600;
	*mkptr( dfe_cfg_base, 0x000640DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640E0 ) = 0x00003800;
	*mkptr( dfe_cfg_base, 0x000640E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640E8 ) = 0x00003A00;
	*mkptr( dfe_cfg_base, 0x000640EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640F0 ) = 0x00003C00;
	*mkptr( dfe_cfg_base, 0x000640F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640F8 ) = 0x00003E00;
	*mkptr( dfe_cfg_base, 0x000640FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
	*mkptr( dfe_cfg_base, 0x0006410C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
	*mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
	*mkptr( dfe_cfg_base, 0x0006411C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
	*mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004A00;
	*mkptr( dfe_cfg_base, 0x0006412C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004C00;
	*mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004E00;
	*mkptr( dfe_cfg_base, 0x0006413C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
	*mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
	*mkptr( dfe_cfg_base, 0x0006414C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
	*mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
	*mkptr( dfe_cfg_base, 0x0006415C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
	*mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005A00;
	*mkptr( dfe_cfg_base, 0x0006416C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005C00;
	*mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005E00;
	*mkptr( dfe_cfg_base, 0x0006417C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
	*mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
	*mkptr( dfe_cfg_base, 0x0006418C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
	*mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
	*mkptr( dfe_cfg_base, 0x0006419C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641A0 ) = 0x00006800;
	*mkptr( dfe_cfg_base, 0x000641A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641A8 ) = 0x00006A00;
	*mkptr( dfe_cfg_base, 0x000641AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641B0 ) = 0x00006C00;
	*mkptr( dfe_cfg_base, 0x000641B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641B8 ) = 0x00006E00;
	*mkptr( dfe_cfg_base, 0x000641BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641C0 ) = 0x00007000;
	*mkptr( dfe_cfg_base, 0x000641C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641C8 ) = 0x00007200;
	*mkptr( dfe_cfg_base, 0x000641CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641D0 ) = 0x00007400;
	*mkptr( dfe_cfg_base, 0x000641D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641D8 ) = 0x00007600;
	*mkptr( dfe_cfg_base, 0x000641DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641E0 ) = 0x00007800;
	*mkptr( dfe_cfg_base, 0x000641E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641E8 ) = 0x00007A00;
	*mkptr( dfe_cfg_base, 0x000641EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641F0 ) = 0x00007C00;
	*mkptr( dfe_cfg_base, 0x000641F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641F8 ) = 0x00007E00;
	*mkptr( dfe_cfg_base, 0x000641FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
	*mkptr( dfe_cfg_base, 0x0006420C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
	*mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
	*mkptr( dfe_cfg_base, 0x0006421C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
	*mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008A00;
	*mkptr( dfe_cfg_base, 0x0006422C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008C00;
	*mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008E00;
	*mkptr( dfe_cfg_base, 0x0006423C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
	*mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
	*mkptr( dfe_cfg_base, 0x0006424C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
	*mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
	*mkptr( dfe_cfg_base, 0x0006425C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
	*mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009A00;
	*mkptr( dfe_cfg_base, 0x0006426C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009C00;
	*mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009E00;
	*mkptr( dfe_cfg_base, 0x0006427C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000A000;
	*mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000A200;
	*mkptr( dfe_cfg_base, 0x0006428C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000A400;
	*mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000A600;
	*mkptr( dfe_cfg_base, 0x0006429C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642A0 ) = 0x0000A800;
	*mkptr( dfe_cfg_base, 0x000642A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642A8 ) = 0x0000AA00;
	*mkptr( dfe_cfg_base, 0x000642AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642B0 ) = 0x0000AC00;
	*mkptr( dfe_cfg_base, 0x000642B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642B8 ) = 0x0000AE00;
	*mkptr( dfe_cfg_base, 0x000642BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642C0 ) = 0x0000B000;
	*mkptr( dfe_cfg_base, 0x000642C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642C8 ) = 0x0000B200;
	*mkptr( dfe_cfg_base, 0x000642CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642D0 ) = 0x0000B400;
	*mkptr( dfe_cfg_base, 0x000642D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642D8 ) = 0x0000B600;
	*mkptr( dfe_cfg_base, 0x000642DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642E0 ) = 0x0000B800;
	*mkptr( dfe_cfg_base, 0x000642E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642E8 ) = 0x0000BA00;
	*mkptr( dfe_cfg_base, 0x000642EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642F0 ) = 0x0000BC00;
	*mkptr( dfe_cfg_base, 0x000642F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642F8 ) = 0x0000BE00;
	*mkptr( dfe_cfg_base, 0x000642FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000C000;
	*mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000C200;
	*mkptr( dfe_cfg_base, 0x0006430C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000C400;
	*mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000C600;
	*mkptr( dfe_cfg_base, 0x0006431C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000C800;
	*mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000CA00;
	*mkptr( dfe_cfg_base, 0x0006432C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000CC00;
	*mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000CE00;
	*mkptr( dfe_cfg_base, 0x0006433C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000D000;
	*mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000D200;
	*mkptr( dfe_cfg_base, 0x0006434C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000D400;
	*mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000D600;
	*mkptr( dfe_cfg_base, 0x0006435C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000D800;
	*mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000DA00;
	*mkptr( dfe_cfg_base, 0x0006436C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000DC00;
	*mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000DE00;
	*mkptr( dfe_cfg_base, 0x0006437C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000E000;
	*mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000E200;
	*mkptr( dfe_cfg_base, 0x0006438C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000E400;
	*mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000E600;
	*mkptr( dfe_cfg_base, 0x0006439C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643A0 ) = 0x0000E800;
	*mkptr( dfe_cfg_base, 0x000643A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643A8 ) = 0x0000EA00;
	*mkptr( dfe_cfg_base, 0x000643AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643B0 ) = 0x0000EC00;
	*mkptr( dfe_cfg_base, 0x000643B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643B8 ) = 0x0000EE00;
	*mkptr( dfe_cfg_base, 0x000643BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643C0 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000643C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643C8 ) = 0x0000F200;
	*mkptr( dfe_cfg_base, 0x000643CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643D0 ) = 0x0000F400;
	*mkptr( dfe_cfg_base, 0x000643D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643D8 ) = 0x0000F600;
	*mkptr( dfe_cfg_base, 0x000643DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643E0 ) = 0x0000F800;
	*mkptr( dfe_cfg_base, 0x000643E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643E8 ) = 0x0000FA00;
	*mkptr( dfe_cfg_base, 0x000643EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643F0 ) = 0x0000FC00;
	*mkptr( dfe_cfg_base, 0x000643F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643F8 ) = 0x0000FE00;
	*mkptr( dfe_cfg_base, 0x000643FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0006C000 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C004 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C008 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C00C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C010 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C020 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C024 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C028 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C02C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C030 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C040 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C044 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C048 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C04C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C050 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C060 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C064 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C068 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C06C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C070 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C080 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C084 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C088 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C08C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C090 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0A4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0A8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0AC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0B0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0C4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0C8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0D0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0E4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0E8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0EC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0F0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00101300 ) = 0x0000031F;
	*mkptr( dfe_cfg_base, 0x00101304 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x00101308 ) = 0x000000B7;
	*mkptr( dfe_cfg_base, 0x00101318 ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x0010131C ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x00101324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00140000 ) = 0x0000F07F;
	*mkptr( dfe_cfg_base, 0x00140004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140008 ) = 0x0000BC20;
	*mkptr( dfe_cfg_base, 0x0014000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140010 ) = 0x000088B3;
	*mkptr( dfe_cfg_base, 0x00140018 ) = 0x000038FE;
	*mkptr( dfe_cfg_base, 0x0014001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00140200 ) = 0x0000166A;
	*mkptr( dfe_cfg_base, 0x00140208 ) = 0x0000F0B6;
	*mkptr( dfe_cfg_base, 0x0014020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140210 ) = 0x0000EEB1;
	*mkptr( dfe_cfg_base, 0x00140214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140218 ) = 0x0000299C;
	*mkptr( dfe_cfg_base, 0x00140400 ) = 0x000000E9;
	*mkptr( dfe_cfg_base, 0x00140408 ) = 0x00000D17;
	*mkptr( dfe_cfg_base, 0x00140410 ) = 0x0000F2A2;
	*mkptr( dfe_cfg_base, 0x00140414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140418 ) = 0x0000FB5B;
	*mkptr( dfe_cfg_base, 0x0014041C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140600 ) = 0x0000F7BC;
	*mkptr( dfe_cfg_base, 0x00140604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140608 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00140610 ) = 0x00000734;
	*mkptr( dfe_cfg_base, 0x00140618 ) = 0x0000F517;
	*mkptr( dfe_cfg_base, 0x0014061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140800 ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x00140808 ) = 0x0000FA45;
	*mkptr( dfe_cfg_base, 0x0014080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140810 ) = 0x00000443;
	*mkptr( dfe_cfg_base, 0x00140818 ) = 0x00000344;
	*mkptr( dfe_cfg_base, 0x00140A00 ) = 0x00000351;
	*mkptr( dfe_cfg_base, 0x00140A08 ) = 0x0000FF4C;
	*mkptr( dfe_cfg_base, 0x00140A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140A10 ) = 0x0000FC78;
	*mkptr( dfe_cfg_base, 0x00140A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140A18 ) = 0x00000411;
	*mkptr( dfe_cfg_base, 0x00140C00 ) = 0x0000FF60;
	*mkptr( dfe_cfg_base, 0x00140C04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140C08 ) = 0x0000025C;
	*mkptr( dfe_cfg_base, 0x00140C10 ) = 0x0000FEAC;
	*mkptr( dfe_cfg_base, 0x00140C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140C18 ) = 0x0000FE30;
	*mkptr( dfe_cfg_base, 0x00140C1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140E00 ) = 0x0000FEDC;
	*mkptr( dfe_cfg_base, 0x00140E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140E08 ) = 0x00000012;
	*mkptr( dfe_cfg_base, 0x00140E10 ) = 0x00000172;
	*mkptr( dfe_cfg_base, 0x00140E18 ) = 0x0000FE9B;
	*mkptr( dfe_cfg_base, 0x00140E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141000 ) = 0x00000047;
	*mkptr( dfe_cfg_base, 0x00141008 ) = 0x0000FF36;
	*mkptr( dfe_cfg_base, 0x0014100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141010 ) = 0x0000005D;
	*mkptr( dfe_cfg_base, 0x00141018 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x00141200 ) = 0x0000FFFC;
	*mkptr( dfe_cfg_base, 0x00141204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141208 ) = 0x0000FF55;
	*mkptr( dfe_cfg_base, 0x0014120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141210 ) = 0x0000FF48;
	*mkptr( dfe_cfg_base, 0x00141214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141218 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x0010126C ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00101344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00101348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0010134C ) = 0x00004F00;
	*mkptr( dfe_cfg_base, 0x00101350 ) = 0x0000048D;
	*mkptr( dfe_cfg_base, 0x00143800 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143804 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143808 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014380C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143810 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143814 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143818 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014381C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143820 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143824 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143828 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014382C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143830 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143834 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143838 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014383C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143840 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143844 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143848 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014384C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143850 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143854 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143858 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014385C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143860 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143864 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143868 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014386C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143870 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143874 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x00143878 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014387C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001012C0 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00143488 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00143490 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x0014348C ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00143494 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001434A8 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001434B0 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001434AC ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001434B4 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001434C0 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001434C8 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001434D0 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001434D8 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x001434C4 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001434CC ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001434D4 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001434DC ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x001434E8 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001434F0 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001434EC ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001434F4 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x00143508 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00143510 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x0014350C ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00143514 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x00143520 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x00143528 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x00143530 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x00143538 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x00143524 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x0014352C ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x00143534 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x0014353C ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x00101200 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00101204 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101208 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x0010120C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101210 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x00101214 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101218 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x0010121C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101220 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00101224 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101228 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x0010122C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00101230 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00101234 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x0010123C ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x00101240 ) = 0x0000100F;
	*mkptr( dfe_cfg_base, 0x00101244 ) = 0x0000100F;
	*mkptr( dfe_cfg_base, 0x00101248 ) = 0x0000100E;
	*mkptr( dfe_cfg_base, 0x00101250 ) = 0x0000100C;
	*mkptr( dfe_cfg_base, 0x00101254 ) = 0x00001004;
	*mkptr( dfe_cfg_base, 0x00101258 ) = 0x00001010;
	*mkptr( dfe_cfg_base, 0x0010125C ) = 0x0000100D;
	*mkptr( dfe_cfg_base, 0x00101260 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00101264 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00101268 ) = 0x00002717;
	*mkptr( dfe_cfg_base, 0x001012DC ) = 0x0000A800;
	*mkptr( dfe_cfg_base, 0x00100400 ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x00181300 ) = 0x00000517;
	*mkptr( dfe_cfg_base, 0x00181304 ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x00181308 ) = 0x000010B7;
	*mkptr( dfe_cfg_base, 0x00181318 ) = 0x00008888;
	*mkptr( dfe_cfg_base, 0x0018131C ) = 0x00008888;
	*mkptr( dfe_cfg_base, 0x00181324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x001C0000 ) = 0x00002A00;
	*mkptr( dfe_cfg_base, 0x001C0008 ) = 0x00001AD7;
	*mkptr( dfe_cfg_base, 0x001C0010 ) = 0x0000A1D9;
	*mkptr( dfe_cfg_base, 0x001C0014 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0018 ) = 0x0000D8DA;
	*mkptr( dfe_cfg_base, 0x001C001C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0020 ) = 0x0000290B;
	*mkptr( dfe_cfg_base, 0x001C0024 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x001C0028 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x001C002C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x001C0200 ) = 0x00000243;
	*mkptr( dfe_cfg_base, 0x001C0208 ) = 0x0000F70F;
	*mkptr( dfe_cfg_base, 0x001C020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0210 ) = 0x0000044E;
	*mkptr( dfe_cfg_base, 0x001C0218 ) = 0x00000EA3;
	*mkptr( dfe_cfg_base, 0x001C0220 ) = 0x0000EE88;
	*mkptr( dfe_cfg_base, 0x001C0224 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0228 ) = 0x0000EB5A;
	*mkptr( dfe_cfg_base, 0x001C022C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0400 ) = 0x0000FD1D;
	*mkptr( dfe_cfg_base, 0x001C0404 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0408 ) = 0x0000FEA6;
	*mkptr( dfe_cfg_base, 0x001C040C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0410 ) = 0x0000045D;
	*mkptr( dfe_cfg_base, 0x001C0418 ) = 0x0000FF56;
	*mkptr( dfe_cfg_base, 0x001C041C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0420 ) = 0x0000FB67;
	*mkptr( dfe_cfg_base, 0x001C0424 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0428 ) = 0x00000424;
	*mkptr( dfe_cfg_base, 0x001C0600 ) = 0x0000FF46;
	*mkptr( dfe_cfg_base, 0x001C0604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0608 ) = 0x00000158;
	*mkptr( dfe_cfg_base, 0x001C0610 ) = 0x0000FFF6;
	*mkptr( dfe_cfg_base, 0x001C0614 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0618 ) = 0x0000FD99;
	*mkptr( dfe_cfg_base, 0x001C061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0620 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x001C0628 ) = 0x000001F6;
	*mkptr( dfe_cfg_base, 0x001C0800 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x001C0808 ) = 0x0000006A;
	*mkptr( dfe_cfg_base, 0x001C0810 ) = 0x0000FF5E;
	*mkptr( dfe_cfg_base, 0x001C0814 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0818 ) = 0x0000FFE0;
	*mkptr( dfe_cfg_base, 0x001C081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0820 ) = 0x000000E5;
	*mkptr( dfe_cfg_base, 0x001C0828 ) = 0x0000FF84;
	*mkptr( dfe_cfg_base, 0x001C082C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0A00 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x001C0A08 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x001C0A10 ) = 0x0000FFA4;
	*mkptr( dfe_cfg_base, 0x001C0A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0A18 ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x001C0A20 ) = 0x0000002A;
	*mkptr( dfe_cfg_base, 0x001C0A28 ) = 0x0000FF97;
	*mkptr( dfe_cfg_base, 0x001C0A2C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0C00 ) = 0x0000FFFE;
	*mkptr( dfe_cfg_base, 0x001C0C04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0C08 ) = 0x0000FFB1;
	*mkptr( dfe_cfg_base, 0x001C0C0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0C10 ) = 0x00000012;
	*mkptr( dfe_cfg_base, 0x001C0C18 ) = 0x00000049;
	*mkptr( dfe_cfg_base, 0x001C0C20 ) = 0x0000FFC5;
	*mkptr( dfe_cfg_base, 0x001C0C24 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0C28 ) = 0x0000FFD0;
	*mkptr( dfe_cfg_base, 0x001C0C2C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0E00 ) = 0x0000FFF1;
	*mkptr( dfe_cfg_base, 0x001C0E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0E10 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x001C0E18 ) = 0x0000FFED;
	*mkptr( dfe_cfg_base, 0x001C0E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0E20 ) = 0x0000FFED;
	*mkptr( dfe_cfg_base, 0x001C0E24 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0E28 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x001C1020 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x001C1028 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0018126C ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00181344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00181348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0018134C ) = 0x00004F00;
	*mkptr( dfe_cfg_base, 0x00181350 ) = 0x0000048D;
	*mkptr( dfe_cfg_base, 0x001C3800 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3804 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3808 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C380C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3810 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3814 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3818 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C381C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3820 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3824 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3828 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C382C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3830 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3834 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3838 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C383C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3840 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3844 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3848 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C384C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3850 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3854 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3858 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C385C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3860 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3864 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3868 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C386C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3870 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3874 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001C3878 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C387C ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x001812C0 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x001C3488 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C3490 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001C348C ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C3494 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001C34A8 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C34B0 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001C34AC ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C34B4 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001C34C0 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001C34C8 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001C34D0 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001C34D8 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x001C34C4 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001C34CC ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001C34D4 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001C34DC ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x001C34E8 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C34F0 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001C34EC ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C34F4 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x001C3508 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C3510 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001C350C ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x001C3514 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x001C3520 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001C3528 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001C3530 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001C3538 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x001C3524 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x001C352C ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x001C3534 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x001C353C ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x00181200 ) = 0x0000003F;
	*mkptr( dfe_cfg_base, 0x00181204 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00181208 ) = 0x00000037;
	*mkptr( dfe_cfg_base, 0x0018120C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00181210 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00181214 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00181218 ) = 0x00000037;
	*mkptr( dfe_cfg_base, 0x0018121C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00181220 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00181224 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x00181228 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x0018122C ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00181230 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00181234 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x0018123C ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x00181240 ) = 0x0000100F;
	*mkptr( dfe_cfg_base, 0x00181244 ) = 0x0000100F;
	*mkptr( dfe_cfg_base, 0x00181248 ) = 0x0000100E;
	*mkptr( dfe_cfg_base, 0x00181250 ) = 0x0000100C;
	*mkptr( dfe_cfg_base, 0x00181254 ) = 0x00001004;
	*mkptr( dfe_cfg_base, 0x00181258 ) = 0x00001010;
	*mkptr( dfe_cfg_base, 0x0018125C ) = 0x0000100D;
	*mkptr( dfe_cfg_base, 0x00181260 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00181264 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00181268 ) = 0x00002717;
	*mkptr( dfe_cfg_base, 0x001812DC ) = 0x0000A800;
	*mkptr( dfe_cfg_base, 0x00180400 ) = 0x00008080;
	*mkptr( dfe_cfg_base, 0x00201300 ) = 0x0000031F;
	*mkptr( dfe_cfg_base, 0x00201304 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x00201308 ) = 0x00002037;
	*mkptr( dfe_cfg_base, 0x00201318 ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x0020131C ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x00201324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00240000 ) = 0x0000A3F3;
	*mkptr( dfe_cfg_base, 0x00240004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240008 ) = 0x0000D6E3;
	*mkptr( dfe_cfg_base, 0x0024000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240010 ) = 0x00002703;
	*mkptr( dfe_cfg_base, 0x00240014 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00240018 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0024001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00240200 ) = 0x0000F0E7;
	*mkptr( dfe_cfg_base, 0x00240204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240208 ) = 0x0000EB42;
	*mkptr( dfe_cfg_base, 0x0024020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240210 ) = 0x0000278A;
	*mkptr( dfe_cfg_base, 0x00240218 ) = 0x00001C08;
	*mkptr( dfe_cfg_base, 0x00240400 ) = 0x00000316;
	*mkptr( dfe_cfg_base, 0x00240408 ) = 0x0000F8D4;
	*mkptr( dfe_cfg_base, 0x0024040C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240410 ) = 0x00000289;
	*mkptr( dfe_cfg_base, 0x00240418 ) = 0x00000DA6;
	*mkptr( dfe_cfg_base, 0x00240600 ) = 0x0000034D;
	*mkptr( dfe_cfg_base, 0x00240608 ) = 0x00000126;
	*mkptr( dfe_cfg_base, 0x00240610 ) = 0x0000FB9E;
	*mkptr( dfe_cfg_base, 0x00240614 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240618 ) = 0x00000216;
	*mkptr( dfe_cfg_base, 0x00240800 ) = 0x0000FF9E;
	*mkptr( dfe_cfg_base, 0x00240804 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240808 ) = 0x0000026B;
	*mkptr( dfe_cfg_base, 0x00240810 ) = 0x0000FEA4;
	*mkptr( dfe_cfg_base, 0x00240814 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240818 ) = 0x0000FD7A;
	*mkptr( dfe_cfg_base, 0x0024081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240A00 ) = 0x0000FE8A;
	*mkptr( dfe_cfg_base, 0x00240A04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240A08 ) = 0x0000003D;
	*mkptr( dfe_cfg_base, 0x00240A10 ) = 0x0000015B;
	*mkptr( dfe_cfg_base, 0x00240A18 ) = 0x0000FE8F;
	*mkptr( dfe_cfg_base, 0x00240A1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240C00 ) = 0x0000FFD3;
	*mkptr( dfe_cfg_base, 0x00240C04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240C08 ) = 0x0000FEED;
	*mkptr( dfe_cfg_base, 0x00240C0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240C10 ) = 0x000000F1;
	*mkptr( dfe_cfg_base, 0x00240C18 ) = 0x000000AF;
	*mkptr( dfe_cfg_base, 0x00240E00 ) = 0x000000C1;
	*mkptr( dfe_cfg_base, 0x00240E08 ) = 0x0000FF8D;
	*mkptr( dfe_cfg_base, 0x00240E0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240E10 ) = 0x0000FF8A;
	*mkptr( dfe_cfg_base, 0x00240E14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240E18 ) = 0x000000EE;
	*mkptr( dfe_cfg_base, 0x00241000 ) = 0x00000051;
	*mkptr( dfe_cfg_base, 0x00241008 ) = 0x0000007D;
	*mkptr( dfe_cfg_base, 0x00241010 ) = 0x0000FF56;
	*mkptr( dfe_cfg_base, 0x00241014 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241018 ) = 0x0000FFE6;
	*mkptr( dfe_cfg_base, 0x0024101C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241200 ) = 0x0000FF9E;
	*mkptr( dfe_cfg_base, 0x00241204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241208 ) = 0x0000006E;
	*mkptr( dfe_cfg_base, 0x00241210 ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x00241218 ) = 0x0000FF68;
	*mkptr( dfe_cfg_base, 0x0024121C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x0020126C ) = 0x0000021E;
	*mkptr( dfe_cfg_base, 0x00201344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00201348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0020134C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00201350 ) = 0x0000048D;
	*mkptr( dfe_cfg_base, 0x00243800 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243804 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243808 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024380C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243810 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243814 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243818 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024381C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243820 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243824 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243828 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024382C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243830 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243834 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243838 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024383C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243840 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243844 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243848 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024384C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243850 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243854 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243858 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024385C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243860 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243864 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243868 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024386C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243870 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243874 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x00243878 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024387C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002012C0 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00243488 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x00243490 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x0024348C ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x00243494 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002434A8 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002434B0 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002434AC ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002434B4 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002434C0 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002434C8 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002434D0 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002434D8 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x002434C4 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002434CC ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002434D4 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002434DC ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x002434E8 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002434F0 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002434EC ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002434F4 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00243508 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x00243510 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x0024350C ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x00243514 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x00243520 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x00243528 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x00243530 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x00243538 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x00243524 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x0024352C ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x00243534 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x0024353C ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x00201200 ) = 0x0000003F;
	*mkptr( dfe_cfg_base, 0x00201204 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00201208 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x0020120C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00201210 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00201214 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00201218 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x0020121C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00201220 ) = 0x0000003F;
	*mkptr( dfe_cfg_base, 0x00201224 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00201228 ) = 0x0000002F;
	*mkptr( dfe_cfg_base, 0x0020122C ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00201230 ) = 0x0000003E;
	*mkptr( dfe_cfg_base, 0x00201234 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x0020123C ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x00201240 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00201244 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00201248 ) = 0x00001009;
	*mkptr( dfe_cfg_base, 0x00201250 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00201254 ) = 0x00001002;
	*mkptr( dfe_cfg_base, 0x00201258 ) = 0x00001010;
	*mkptr( dfe_cfg_base, 0x0020125C ) = 0x00001007;
	*mkptr( dfe_cfg_base, 0x00201260 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00201264 ) = 0x00001F1F;
	*mkptr( dfe_cfg_base, 0x00201268 ) = 0x00001707;
	*mkptr( dfe_cfg_base, 0x002012DC ) = 0x00007800;
	*mkptr( dfe_cfg_base, 0x002012E4 ) = 0x00002222;
	*mkptr( dfe_cfg_base, 0x002012E8 ) = 0x00003333;
	*mkptr( dfe_cfg_base, 0x002012EC ) = 0x00001111;
	*mkptr( dfe_cfg_base, 0x00281300 ) = 0x0000031F;
	*mkptr( dfe_cfg_base, 0x00281304 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x00281308 ) = 0x00000037;
	*mkptr( dfe_cfg_base, 0x00281318 ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x0028131C ) = 0x00009999;
	*mkptr( dfe_cfg_base, 0x00281324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x002C0000 ) = 0x0000F07F;
	*mkptr( dfe_cfg_base, 0x002C0004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0008 ) = 0x0000BC20;
	*mkptr( dfe_cfg_base, 0x002C000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0010 ) = 0x000088B3;
	*mkptr( dfe_cfg_base, 0x002C0018 ) = 0x000038FE;
	*mkptr( dfe_cfg_base, 0x002C001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x002C0200 ) = 0x0000166A;
	*mkptr( dfe_cfg_base, 0x002C0208 ) = 0x0000F0B6;
	*mkptr( dfe_cfg_base, 0x002C020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0210 ) = 0x0000EEB1;
	*mkptr( dfe_cfg_base, 0x002C0214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0218 ) = 0x0000299C;
	*mkptr( dfe_cfg_base, 0x002C0400 ) = 0x000000E9;
	*mkptr( dfe_cfg_base, 0x002C0408 ) = 0x00000D17;
	*mkptr( dfe_cfg_base, 0x002C0410 ) = 0x0000F2A2;
	*mkptr( dfe_cfg_base, 0x002C0414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0418 ) = 0x0000FB5B;
	*mkptr( dfe_cfg_base, 0x002C041C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0600 ) = 0x0000F7BC;
	*mkptr( dfe_cfg_base, 0x002C0604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0608 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x002C0610 ) = 0x00000734;
	*mkptr( dfe_cfg_base, 0x002C0618 ) = 0x0000F517;
	*mkptr( dfe_cfg_base, 0x002C061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0800 ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x002C0808 ) = 0x0000FA45;
	*mkptr( dfe_cfg_base, 0x002C080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0810 ) = 0x00000443;
	*mkptr( dfe_cfg_base, 0x002C0818 ) = 0x00000344;
	*mkptr( dfe_cfg_base, 0x002C0A00 ) = 0x00000351;
	*mkptr( dfe_cfg_base, 0x002C0A08 ) = 0x0000FF4C;
	*mkptr( dfe_cfg_base, 0x002C0A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0A10 ) = 0x0000FC78;
	*mkptr( dfe_cfg_base, 0x002C0A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0A18 ) = 0x00000411;
	*mkptr( dfe_cfg_base, 0x002C0C00 ) = 0x0000FF60;
	*mkptr( dfe_cfg_base, 0x002C0C04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0C08 ) = 0x0000025C;
	*mkptr( dfe_cfg_base, 0x002C0C10 ) = 0x0000FEAC;
	*mkptr( dfe_cfg_base, 0x002C0C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0C18 ) = 0x0000FE30;
	*mkptr( dfe_cfg_base, 0x002C0C1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0E00 ) = 0x0000FEDC;
	*mkptr( dfe_cfg_base, 0x002C0E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0E08 ) = 0x00000012;
	*mkptr( dfe_cfg_base, 0x002C0E10 ) = 0x00000172;
	*mkptr( dfe_cfg_base, 0x002C0E18 ) = 0x0000FE9B;
	*mkptr( dfe_cfg_base, 0x002C0E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1000 ) = 0x00000047;
	*mkptr( dfe_cfg_base, 0x002C1008 ) = 0x0000FF36;
	*mkptr( dfe_cfg_base, 0x002C100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1010 ) = 0x0000005D;
	*mkptr( dfe_cfg_base, 0x002C1018 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x002C1200 ) = 0x0000FFFC;
	*mkptr( dfe_cfg_base, 0x002C1204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1208 ) = 0x0000FF55;
	*mkptr( dfe_cfg_base, 0x002C120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1210 ) = 0x0000FF48;
	*mkptr( dfe_cfg_base, 0x002C1214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1218 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x0028126C ) = 0x0000021E;
	*mkptr( dfe_cfg_base, 0x00281344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00281348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0028134C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00281350 ) = 0x0000048D;
	*mkptr( dfe_cfg_base, 0x002C3800 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3804 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3808 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C380C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3810 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3814 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3818 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C381C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3820 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3824 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3828 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C382C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3830 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3834 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3838 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C383C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3840 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3844 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3848 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C384C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3850 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3854 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3858 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C385C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3860 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3864 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3868 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C386C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3870 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3874 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002C3878 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C387C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x002812C0 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x002C3488 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002C3490 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C348C ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002C3494 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C34A8 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002C34B0 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C34AC ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002C34B4 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C34C0 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002C34C8 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002C34D0 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002C34D8 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x002C34C4 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002C34CC ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002C34D4 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002C34DC ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x002C34E8 ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002C34F0 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C34EC ) = 0x0000AAAB;
	*mkptr( dfe_cfg_base, 0x002C34F4 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C3508 ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002C3510 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C350C ) = 0x0000AAAA;
	*mkptr( dfe_cfg_base, 0x002C3514 ) = 0x00005555;
	*mkptr( dfe_cfg_base, 0x002C3520 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002C3528 ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002C3530 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002C3538 ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x002C3524 ) = 0x0000C180;
	*mkptr( dfe_cfg_base, 0x002C352C ) = 0x0000EB2A;
	*mkptr( dfe_cfg_base, 0x002C3534 ) = 0x000014D5;
	*mkptr( dfe_cfg_base, 0x002C353C ) = 0x00003E80;
	*mkptr( dfe_cfg_base, 0x00281200 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00281204 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00281208 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x0028120C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00281210 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00281214 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00281218 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x0028121C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00281220 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00281224 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00281228 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x0028122C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00281230 ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00281234 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x0028123C ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x00281240 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00281244 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00281248 ) = 0x00001009;
	*mkptr( dfe_cfg_base, 0x00281250 ) = 0x0000100A;
	*mkptr( dfe_cfg_base, 0x00281254 ) = 0x00001002;
	*mkptr( dfe_cfg_base, 0x00281258 ) = 0x00001010;
	*mkptr( dfe_cfg_base, 0x0028125C ) = 0x00001007;
	*mkptr( dfe_cfg_base, 0x00281260 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00281264 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00281268 ) = 0x00001707;
	*mkptr( dfe_cfg_base, 0x002812DC ) = 0x00007805;
	*mkptr( dfe_cfg_base, 0x002812E8 ) = 0x00004111;
	*mkptr( dfe_cfg_base, 0x00900004 ) = 0x00005544;
	*mkptr( dfe_cfg_base, 0x00900008 ) = 0x00004444;
	*mkptr( dfe_cfg_base, 0x0090000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00900018 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00900024 ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x00900030 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00900040 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00B003CC ) = 0x0000800D;
	*mkptr( dfe_cfg_base, 0x00B00208 ) = 0x00001180;
	*mkptr( dfe_cfg_base, 0x00B00288 ) = 0x00001180;
	*mkptr( dfe_cfg_base, 0x00B00360 ) = 0x0000CADE;
	*mkptr( dfe_cfg_base, 0x00B00370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B00200 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00280 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00210 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B00218 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00230 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B00290 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B00298 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B002B0 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B00300 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B00308 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B00310 ) = 0x00003F5C;
	*mkptr( dfe_cfg_base, 0x00B00318 ) = 0x00003F5C;
	*mkptr( dfe_cfg_base, 0x00B00320 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B00328 ) = 0x0000013C;
	*mkptr( dfe_cfg_base, 0x00B00338 ) = 0x00000BB8;
	*mkptr( dfe_cfg_base, 0x00B00340 ) = 0x00000363;
	*mkptr( dfe_cfg_base, 0x00B00348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00B00378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B18004 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1800C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18014 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1801C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18024 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1802C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18034 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1803C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18044 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1804C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18054 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1805C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18064 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1806C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18074 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1807C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18080 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x00B18084 ) = 0x00007FE8;
	*mkptr( dfe_cfg_base, 0x00B18088 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x00B1808C ) = 0x00007C17;
	*mkptr( dfe_cfg_base, 0x00B18090 ) = 0x00000334;
	*mkptr( dfe_cfg_base, 0x00B18094 ) = 0x00007899;
	*mkptr( dfe_cfg_base, 0x00B18098 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00B1809C ) = 0x00007562;
	*mkptr( dfe_cfg_base, 0x00B180A0 ) = 0x00000350;
	*mkptr( dfe_cfg_base, 0x00B180A4 ) = 0x0000726A;
	*mkptr( dfe_cfg_base, 0x00B180A8 ) = 0x0000035C;
	*mkptr( dfe_cfg_base, 0x00B180AC ) = 0x00006FA8;
	*mkptr( dfe_cfg_base, 0x00B180B0 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x00B180B4 ) = 0x00006D17;
	*mkptr( dfe_cfg_base, 0x00B180B8 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00B180BC ) = 0x00006AB2;
	*mkptr( dfe_cfg_base, 0x00B180C0 ) = 0x00000378;
	*mkptr( dfe_cfg_base, 0x00B180C4 ) = 0x00006873;
	*mkptr( dfe_cfg_base, 0x00B180C8 ) = 0x00000380;
	*mkptr( dfe_cfg_base, 0x00B180CC ) = 0x00006657;
	*mkptr( dfe_cfg_base, 0x00B180D0 ) = 0x00000388;
	*mkptr( dfe_cfg_base, 0x00B180D4 ) = 0x0000645A;
	*mkptr( dfe_cfg_base, 0x00B180D8 ) = 0x00000390;
	*mkptr( dfe_cfg_base, 0x00B180DC ) = 0x0000627A;
	*mkptr( dfe_cfg_base, 0x00B180E0 ) = 0x00000394;
	*mkptr( dfe_cfg_base, 0x00B180E4 ) = 0x000060B4;
	*mkptr( dfe_cfg_base, 0x00B180E8 ) = 0x00000398;
	*mkptr( dfe_cfg_base, 0x00B180EC ) = 0x00005F06;
	*mkptr( dfe_cfg_base, 0x00B180F0 ) = 0x000003A0;
	*mkptr( dfe_cfg_base, 0x00B180F4 ) = 0x00005D6D;
	*mkptr( dfe_cfg_base, 0x00B180F8 ) = 0x000003A4;
	*mkptr( dfe_cfg_base, 0x00B180FC ) = 0x00005BE8;
	*mkptr( dfe_cfg_base, 0x00B18100 ) = 0x000003A8;
	*mkptr( dfe_cfg_base, 0x00B18104 ) = 0x00005A76;
	*mkptr( dfe_cfg_base, 0x00B18108 ) = 0x000003AC;
	*mkptr( dfe_cfg_base, 0x00B1810C ) = 0x00005914;
	*mkptr( dfe_cfg_base, 0x00B18110 ) = 0x000003B0;
	*mkptr( dfe_cfg_base, 0x00B18114 ) = 0x000057C2;
	*mkptr( dfe_cfg_base, 0x00B18118 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B1811C ) = 0x0000567F;
	*mkptr( dfe_cfg_base, 0x00B18120 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B18124 ) = 0x00005549;
	*mkptr( dfe_cfg_base, 0x00B18128 ) = 0x000003B8;
	*mkptr( dfe_cfg_base, 0x00B1812C ) = 0x00005420;
	*mkptr( dfe_cfg_base, 0x00B18130 ) = 0x000003BC;
	*mkptr( dfe_cfg_base, 0x00B18134 ) = 0x00005303;
	*mkptr( dfe_cfg_base, 0x00B18138 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B1813C ) = 0x000051F1;
	*mkptr( dfe_cfg_base, 0x00B18140 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B18144 ) = 0x000050E9;
	*mkptr( dfe_cfg_base, 0x00B18148 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B1814C ) = 0x00004FEB;
	*mkptr( dfe_cfg_base, 0x00B18150 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B18154 ) = 0x00004EF6;
	*mkptr( dfe_cfg_base, 0x00B18158 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B1815C ) = 0x00004E0A;
	*mkptr( dfe_cfg_base, 0x00B18160 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B18164 ) = 0x00004D25;
	*mkptr( dfe_cfg_base, 0x00B18168 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1816C ) = 0x00004C49;
	*mkptr( dfe_cfg_base, 0x00B18170 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B18174 ) = 0x00004B73;
	*mkptr( dfe_cfg_base, 0x00B18178 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1817C ) = 0x00004AA5;
	*mkptr( dfe_cfg_base, 0x00B18180 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B18184 ) = 0x000049DD;
	*mkptr( dfe_cfg_base, 0x00B18188 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B1818C ) = 0x0000491B;
	*mkptr( dfe_cfg_base, 0x00B18190 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B18194 ) = 0x0000485F;
	*mkptr( dfe_cfg_base, 0x00B18198 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B1819C ) = 0x000047A8;
	*mkptr( dfe_cfg_base, 0x00B181A0 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B181A4 ) = 0x000046F7;
	*mkptr( dfe_cfg_base, 0x00B181A8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181AC ) = 0x0000464B;
	*mkptr( dfe_cfg_base, 0x00B181B0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181B4 ) = 0x000045A3;
	*mkptr( dfe_cfg_base, 0x00B181B8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181BC ) = 0x00004501;
	*mkptr( dfe_cfg_base, 0x00B181C0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181C4 ) = 0x00004462;
	*mkptr( dfe_cfg_base, 0x00B181C8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181CC ) = 0x000043C8;
	*mkptr( dfe_cfg_base, 0x00B181D0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181D4 ) = 0x00004332;
	*mkptr( dfe_cfg_base, 0x00B181D8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181DC ) = 0x0000429F;
	*mkptr( dfe_cfg_base, 0x00B181E0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181E4 ) = 0x00004211;
	*mkptr( dfe_cfg_base, 0x00B181E8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181EC ) = 0x00004185;
	*mkptr( dfe_cfg_base, 0x00B181F0 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181F4 ) = 0x000040FE;
	*mkptr( dfe_cfg_base, 0x00B181F8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181FC ) = 0x00004079;
	*mkptr( dfe_cfg_base, 0x00B18200 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18204 ) = 0x00003FF8;
	*mkptr( dfe_cfg_base, 0x00B18208 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B1820C ) = 0x00003F79;
	*mkptr( dfe_cfg_base, 0x00B18210 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18214 ) = 0x00003EFE;
	*mkptr( dfe_cfg_base, 0x00B18218 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1821C ) = 0x00003E85;
	*mkptr( dfe_cfg_base, 0x00B18220 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18224 ) = 0x00003E0F;
	*mkptr( dfe_cfg_base, 0x00B18228 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1822C ) = 0x00003D9B;
	*mkptr( dfe_cfg_base, 0x00B18230 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18234 ) = 0x00003D2A;
	*mkptr( dfe_cfg_base, 0x00B18238 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1823C ) = 0x00003CBB;
	*mkptr( dfe_cfg_base, 0x00B18240 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18244 ) = 0x00003C4F;
	*mkptr( dfe_cfg_base, 0x00B18248 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1824C ) = 0x00003BE5;
	*mkptr( dfe_cfg_base, 0x00B18250 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18254 ) = 0x00003B7D;
	*mkptr( dfe_cfg_base, 0x00B18258 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1825C ) = 0x00003B17;
	*mkptr( dfe_cfg_base, 0x00B18260 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18264 ) = 0x00003AB3;
	*mkptr( dfe_cfg_base, 0x00B18268 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1826C ) = 0x00003A51;
	*mkptr( dfe_cfg_base, 0x00B18270 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18274 ) = 0x000039F1;
	*mkptr( dfe_cfg_base, 0x00B18278 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1827C ) = 0x00003993;
	*mkptr( dfe_cfg_base, 0x00B18280 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18284 ) = 0x00003937;
	*mkptr( dfe_cfg_base, 0x00B18288 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1828C ) = 0x000038DC;
	*mkptr( dfe_cfg_base, 0x00B18290 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B18294 ) = 0x00003883;
	*mkptr( dfe_cfg_base, 0x00B18298 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B1829C ) = 0x0000382C;
	*mkptr( dfe_cfg_base, 0x00B182A0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182A4 ) = 0x000037D6;
	*mkptr( dfe_cfg_base, 0x00B182A8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182AC ) = 0x00003782;
	*mkptr( dfe_cfg_base, 0x00B182B0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182B4 ) = 0x0000372F;
	*mkptr( dfe_cfg_base, 0x00B182B8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182BC ) = 0x000036DD;
	*mkptr( dfe_cfg_base, 0x00B182C0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182C4 ) = 0x0000368D;
	*mkptr( dfe_cfg_base, 0x00B182C8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182CC ) = 0x0000363F;
	*mkptr( dfe_cfg_base, 0x00B182D0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182D4 ) = 0x000035F1;
	*mkptr( dfe_cfg_base, 0x00B182D8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182DC ) = 0x000035A5;
	*mkptr( dfe_cfg_base, 0x00B182E0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182E4 ) = 0x0000355A;
	*mkptr( dfe_cfg_base, 0x00B182E8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182EC ) = 0x00003511;
	*mkptr( dfe_cfg_base, 0x00B182F0 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B182F4 ) = 0x000034C8;
	*mkptr( dfe_cfg_base, 0x00B182F8 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B182FC ) = 0x00003481;
	*mkptr( dfe_cfg_base, 0x00B18300 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18304 ) = 0x0000343B;
	*mkptr( dfe_cfg_base, 0x00B18308 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1830C ) = 0x000033F6;
	*mkptr( dfe_cfg_base, 0x00B18310 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18314 ) = 0x000033B2;
	*mkptr( dfe_cfg_base, 0x00B18318 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1831C ) = 0x0000336F;
	*mkptr( dfe_cfg_base, 0x00B18320 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18324 ) = 0x0000332D;
	*mkptr( dfe_cfg_base, 0x00B18328 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1832C ) = 0x000032EC;
	*mkptr( dfe_cfg_base, 0x00B18330 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18334 ) = 0x000032AC;
	*mkptr( dfe_cfg_base, 0x00B18338 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1833C ) = 0x0000326C;
	*mkptr( dfe_cfg_base, 0x00B18340 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18344 ) = 0x0000322E;
	*mkptr( dfe_cfg_base, 0x00B18348 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1834C ) = 0x000031F1;
	*mkptr( dfe_cfg_base, 0x00B18350 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18354 ) = 0x000031B4;
	*mkptr( dfe_cfg_base, 0x00B18358 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1835C ) = 0x00003179;
	*mkptr( dfe_cfg_base, 0x00B18360 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18364 ) = 0x0000313E;
	*mkptr( dfe_cfg_base, 0x00B18368 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1836C ) = 0x00003104;
	*mkptr( dfe_cfg_base, 0x00B18370 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18374 ) = 0x000030CB;
	*mkptr( dfe_cfg_base, 0x00B18378 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1837C ) = 0x00003093;
	*mkptr( dfe_cfg_base, 0x00B18380 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18384 ) = 0x0000305B;
	*mkptr( dfe_cfg_base, 0x00B18388 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1838C ) = 0x00003024;
	*mkptr( dfe_cfg_base, 0x00B18390 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18394 ) = 0x00002FEE;
	*mkptr( dfe_cfg_base, 0x00B18398 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1839C ) = 0x00002FB8;
	*mkptr( dfe_cfg_base, 0x00B183A0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183A4 ) = 0x00002F84;
	*mkptr( dfe_cfg_base, 0x00B183A8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183AC ) = 0x00002F50;
	*mkptr( dfe_cfg_base, 0x00B183B0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183B4 ) = 0x00002F1C;
	*mkptr( dfe_cfg_base, 0x00B183B8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183BC ) = 0x00002EE9;
	*mkptr( dfe_cfg_base, 0x00B183C0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183C4 ) = 0x00002EB7;
	*mkptr( dfe_cfg_base, 0x00B183C8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183CC ) = 0x00002E86;
	*mkptr( dfe_cfg_base, 0x00B183D0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183D4 ) = 0x00002E55;
	*mkptr( dfe_cfg_base, 0x00B183D8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183DC ) = 0x00002E24;
	*mkptr( dfe_cfg_base, 0x00B183E0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183E4 ) = 0x00002DF5;
	*mkptr( dfe_cfg_base, 0x00B183E8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183EC ) = 0x00002DC6;
	*mkptr( dfe_cfg_base, 0x00B183F0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183F4 ) = 0x00002D97;
	*mkptr( dfe_cfg_base, 0x00B183F8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183FC ) = 0x00002D69;
	*mkptr( dfe_cfg_base, 0x00B10100 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10108 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1010C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10110 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10118 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1011C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10120 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10128 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1012C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10130 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10138 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1013C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B1500C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x00B15014 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00B1501C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00B15024 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00B1502C ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x00B15034 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00B1503C ) = 0x00000700;
	*mkptr( dfe_cfg_base, 0x00B15044 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00B1504C ) = 0x00000900;
	*mkptr( dfe_cfg_base, 0x00B15054 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x00B1505C ) = 0x00000B00;
	*mkptr( dfe_cfg_base, 0x00B15064 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00B1506C ) = 0x00000D00;
	*mkptr( dfe_cfg_base, 0x00B15074 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x00B1507C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00B14000 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x00B14004 ) = 0x00004130;
	*mkptr( dfe_cfg_base, 0x00B14008 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x00B1400C ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B14010 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B14014 ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B14018 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B1401C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14020 ) = 0x0000080C;
	*mkptr( dfe_cfg_base, 0x00B14024 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x00B14040 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14044 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14048 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1404C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14050 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14054 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14058 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1405C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14060 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14064 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14080 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14084 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14088 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1408C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14090 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14094 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14098 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1409C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14100 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14104 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14108 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1410C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14110 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14114 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14118 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1411C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14120 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14124 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14140 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14144 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14148 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1414C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14150 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14154 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14158 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1415C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14160 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14164 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14180 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14184 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14188 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1418C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14190 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14194 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14198 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1419C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14200 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14204 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14208 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1420C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14210 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14214 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14218 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1421C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14220 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14224 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14240 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14244 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14248 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1424C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14250 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14254 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14258 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1425C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14260 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14264 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14280 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14284 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14288 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1428C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14290 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14294 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14298 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1429C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14300 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14304 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14308 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1430C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14310 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14314 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14318 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1431C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14320 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14324 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14340 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14344 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14348 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1434C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14350 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14354 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14358 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1435C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14360 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14364 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14380 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14384 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14388 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1438C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14390 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14394 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14398 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1439C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14400 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14404 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14408 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1440C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14410 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14414 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14418 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1441C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14420 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14424 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14440 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14444 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14448 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1444C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14450 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14454 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14458 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1445C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14460 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14464 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14480 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14484 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14488 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1448C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14490 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14494 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14498 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1449C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14500 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14504 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14508 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1450C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14510 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14514 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14518 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1451C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14520 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14524 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14540 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14544 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14548 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1454C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14550 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14554 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14558 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1455C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14560 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14564 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14580 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14584 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14588 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1458C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14590 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14594 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14598 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1459C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14600 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14604 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14608 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1460C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14610 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14614 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14618 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1461C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14620 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14624 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14640 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14644 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1464C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14650 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14654 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14658 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1465C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14660 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14664 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14680 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14684 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14688 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1468C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14690 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14694 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14698 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1469C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14700 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14704 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14708 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1470C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14710 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14714 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14718 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1471C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14720 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14724 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14740 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14744 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14748 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1474C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14750 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14754 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14758 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1475C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14760 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14764 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14780 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14784 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14788 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1478C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14790 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14794 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14798 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1479C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B04000 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00B04004 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B04404 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B04008 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04408 ) = 0x00000FFD;
	*mkptr( dfe_cfg_base, 0x00B0400C ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B0440C ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B04010 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B04410 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B04014 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B04414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04018 ) = 0x00000041;
	*mkptr( dfe_cfg_base, 0x00B04418 ) = 0x00000032;
	*mkptr( dfe_cfg_base, 0x00B0401C ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B0441C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B04020 ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B04420 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B04024 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B04424 ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B04028 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B04428 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B0402C ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B0442C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B04030 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B04430 ) = 0x00000FF9;
	*mkptr( dfe_cfg_base, 0x00B04034 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B04434 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B04038 ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B04438 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B0403C ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B0443C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B04040 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x00B04440 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B04044 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B04444 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B04048 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B04448 ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B0404C ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B0444C ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B04050 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B04450 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04054 ) = 0x00000FF2;
	*mkptr( dfe_cfg_base, 0x00B04454 ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B04058 ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00B04458 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B0405C ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B0445C ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B04060 ) = 0x00000F8E;
	*mkptr( dfe_cfg_base, 0x00B04460 ) = 0x00000FB9;
	*mkptr( dfe_cfg_base, 0x00B04064 ) = 0x00000FB2;
	*mkptr( dfe_cfg_base, 0x00B04464 ) = 0x00000F8B;
	*mkptr( dfe_cfg_base, 0x00B04068 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B04468 ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B0406C ) = 0x00000FFA;
	*mkptr( dfe_cfg_base, 0x00B0446C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04070 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B04470 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B04074 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04474 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B04078 ) = 0x00000F56;
	*mkptr( dfe_cfg_base, 0x00B04478 ) = 0x00000FB3;
	*mkptr( dfe_cfg_base, 0x00B0407C ) = 0x00000F3B;
	*mkptr( dfe_cfg_base, 0x00B0447C ) = 0x00000F20;
	*mkptr( dfe_cfg_base, 0x00B04080 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B04480 ) = 0x00000F92;
	*mkptr( dfe_cfg_base, 0x00B04084 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00B04484 ) = 0x00000025;
	*mkptr( dfe_cfg_base, 0x00B04088 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B04488 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0408C ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B0448C ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B04090 ) = 0x00000F54;
	*mkptr( dfe_cfg_base, 0x00B04490 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B04094 ) = 0x00000EB1;
	*mkptr( dfe_cfg_base, 0x00B04494 ) = 0x00000EDF;
	*mkptr( dfe_cfg_base, 0x00B04098 ) = 0x00000FAE;
	*mkptr( dfe_cfg_base, 0x00B04498 ) = 0x00000F08;
	*mkptr( dfe_cfg_base, 0x00B0409C ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x00B0449C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00B040A0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B044A0 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00B040A4 ) = 0x00000FE6;
	*mkptr( dfe_cfg_base, 0x00B044A4 ) = 0x00000FC1;
	*mkptr( dfe_cfg_base, 0x00B040A8 ) = 0x00000FB7;
	*mkptr( dfe_cfg_base, 0x00B044A8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B040AC ) = 0x00000E40;
	*mkptr( dfe_cfg_base, 0x00B044AC ) = 0x00000EFA;
	*mkptr( dfe_cfg_base, 0x00B040B0 ) = 0x00000EF5;
	*mkptr( dfe_cfg_base, 0x00B044B0 ) = 0x00000E24;
	*mkptr( dfe_cfg_base, 0x00B040B4 ) = 0x0000013E;
	*mkptr( dfe_cfg_base, 0x00B044B4 ) = 0x0000004A;
	*mkptr( dfe_cfg_base, 0x00B040B8 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B044B8 ) = 0x0000010E;
	*mkptr( dfe_cfg_base, 0x00B040BC ) = 0x00000E51;
	*mkptr( dfe_cfg_base, 0x00B044BC ) = 0x00000E6C;
	*mkptr( dfe_cfg_base, 0x00B040C0 ) = 0x00000386;
	*mkptr( dfe_cfg_base, 0x00B044C0 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B040C4 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B044C4 ) = 0x000006B1;
	*mkptr( dfe_cfg_base, 0x00B08000 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00B08004 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B08404 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B08008 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08408 ) = 0x00000FFD;
	*mkptr( dfe_cfg_base, 0x00B0800C ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B0840C ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B08010 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B08410 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B08014 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B08414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08018 ) = 0x00000041;
	*mkptr( dfe_cfg_base, 0x00B08418 ) = 0x00000032;
	*mkptr( dfe_cfg_base, 0x00B0801C ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B0841C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B08020 ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B08420 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B08024 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B08424 ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B08028 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B08428 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B0802C ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B0842C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B08030 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B08430 ) = 0x00000FF9;
	*mkptr( dfe_cfg_base, 0x00B08034 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B08434 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B08038 ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B08438 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B0803C ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B0843C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B08040 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x00B08440 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B08044 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B08444 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B08048 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B08448 ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B0804C ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B0844C ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B08050 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B08450 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08054 ) = 0x00000FF2;
	*mkptr( dfe_cfg_base, 0x00B08454 ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B08058 ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00B08458 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B0805C ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B0845C ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B08060 ) = 0x00000F8E;
	*mkptr( dfe_cfg_base, 0x00B08460 ) = 0x00000FB9;
	*mkptr( dfe_cfg_base, 0x00B08064 ) = 0x00000FB2;
	*mkptr( dfe_cfg_base, 0x00B08464 ) = 0x00000F8B;
	*mkptr( dfe_cfg_base, 0x00B08068 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B08468 ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B0806C ) = 0x00000FFA;
	*mkptr( dfe_cfg_base, 0x00B0846C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08070 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B08470 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B08074 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08474 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B08078 ) = 0x00000F56;
	*mkptr( dfe_cfg_base, 0x00B08478 ) = 0x00000FB3;
	*mkptr( dfe_cfg_base, 0x00B0807C ) = 0x00000F3B;
	*mkptr( dfe_cfg_base, 0x00B0847C ) = 0x00000F20;
	*mkptr( dfe_cfg_base, 0x00B08080 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B08480 ) = 0x00000F92;
	*mkptr( dfe_cfg_base, 0x00B08084 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00B08484 ) = 0x00000025;
	*mkptr( dfe_cfg_base, 0x00B08088 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B08488 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0808C ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B0848C ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B08090 ) = 0x00000F54;
	*mkptr( dfe_cfg_base, 0x00B08490 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B08094 ) = 0x00000EB1;
	*mkptr( dfe_cfg_base, 0x00B08494 ) = 0x00000EDF;
	*mkptr( dfe_cfg_base, 0x00B08098 ) = 0x00000FAE;
	*mkptr( dfe_cfg_base, 0x00B08498 ) = 0x00000F08;
	*mkptr( dfe_cfg_base, 0x00B0809C ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x00B0849C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00B080A0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B084A0 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00B080A4 ) = 0x00000FE6;
	*mkptr( dfe_cfg_base, 0x00B084A4 ) = 0x00000FC1;
	*mkptr( dfe_cfg_base, 0x00B080A8 ) = 0x00000FB7;
	*mkptr( dfe_cfg_base, 0x00B084A8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B080AC ) = 0x00000E40;
	*mkptr( dfe_cfg_base, 0x00B084AC ) = 0x00000EFA;
	*mkptr( dfe_cfg_base, 0x00B080B0 ) = 0x00000EF5;
	*mkptr( dfe_cfg_base, 0x00B084B0 ) = 0x00000E24;
	*mkptr( dfe_cfg_base, 0x00B080B4 ) = 0x0000013E;
	*mkptr( dfe_cfg_base, 0x00B084B4 ) = 0x0000004A;
	*mkptr( dfe_cfg_base, 0x00B080B8 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B084B8 ) = 0x0000010E;
	*mkptr( dfe_cfg_base, 0x00B080BC ) = 0x00000E51;
	*mkptr( dfe_cfg_base, 0x00B084BC ) = 0x00000E6C;
	*mkptr( dfe_cfg_base, 0x00B080C0 ) = 0x00000386;
	*mkptr( dfe_cfg_base, 0x00B084C0 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B080C4 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B084C4 ) = 0x000006B1;
	*mkptr( dfe_cfg_base, 0x00B00364 ) = 0x0000CADE;
	*mkptr( dfe_cfg_base, 0x00B00374 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B00204 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00284 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00214 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B0021C ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00224 ) = 0x00003232;
	*mkptr( dfe_cfg_base, 0x00B0022C ) = 0x00003232;
	*mkptr( dfe_cfg_base, 0x00B00234 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B00294 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B0029C ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B002A4 ) = 0x00003232;
	*mkptr( dfe_cfg_base, 0x00B002AC ) = 0x00003232;
	*mkptr( dfe_cfg_base, 0x00B002B4 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B00304 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B0030C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B00314 ) = 0x00007F5C;
	*mkptr( dfe_cfg_base, 0x00B0031C ) = 0x00007F5C;
	*mkptr( dfe_cfg_base, 0x00B00324 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B0032C ) = 0x0000013C;
	*mkptr( dfe_cfg_base, 0x00B0033C ) = 0x00000BB8;
	*mkptr( dfe_cfg_base, 0x00B00344 ) = 0x00000363;
	*mkptr( dfe_cfg_base, 0x00B0034C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00B0037C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B18404 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1840C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18414 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1841C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18424 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1842C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18434 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1843C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18444 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1844C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18454 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1845C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18464 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1846C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18474 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1847C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18480 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x00B18484 ) = 0x00007FE8;
	*mkptr( dfe_cfg_base, 0x00B18488 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x00B1848C ) = 0x00007C17;
	*mkptr( dfe_cfg_base, 0x00B18490 ) = 0x00000334;
	*mkptr( dfe_cfg_base, 0x00B18494 ) = 0x00007899;
	*mkptr( dfe_cfg_base, 0x00B18498 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00B1849C ) = 0x00007562;
	*mkptr( dfe_cfg_base, 0x00B184A0 ) = 0x00000350;
	*mkptr( dfe_cfg_base, 0x00B184A4 ) = 0x0000726A;
	*mkptr( dfe_cfg_base, 0x00B184A8 ) = 0x0000035C;
	*mkptr( dfe_cfg_base, 0x00B184AC ) = 0x00006FA8;
	*mkptr( dfe_cfg_base, 0x00B184B0 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x00B184B4 ) = 0x00006D17;
	*mkptr( dfe_cfg_base, 0x00B184B8 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00B184BC ) = 0x00006AB2;
	*mkptr( dfe_cfg_base, 0x00B184C0 ) = 0x00000378;
	*mkptr( dfe_cfg_base, 0x00B184C4 ) = 0x00006873;
	*mkptr( dfe_cfg_base, 0x00B184C8 ) = 0x00000380;
	*mkptr( dfe_cfg_base, 0x00B184CC ) = 0x00006657;
	*mkptr( dfe_cfg_base, 0x00B184D0 ) = 0x00000388;
	*mkptr( dfe_cfg_base, 0x00B184D4 ) = 0x0000645A;
	*mkptr( dfe_cfg_base, 0x00B184D8 ) = 0x00000390;
	*mkptr( dfe_cfg_base, 0x00B184DC ) = 0x0000627A;
	*mkptr( dfe_cfg_base, 0x00B184E0 ) = 0x00000394;
	*mkptr( dfe_cfg_base, 0x00B184E4 ) = 0x000060B4;
	*mkptr( dfe_cfg_base, 0x00B184E8 ) = 0x00000398;
	*mkptr( dfe_cfg_base, 0x00B184EC ) = 0x00005F06;
	*mkptr( dfe_cfg_base, 0x00B184F0 ) = 0x000003A0;
	*mkptr( dfe_cfg_base, 0x00B184F4 ) = 0x00005D6D;
	*mkptr( dfe_cfg_base, 0x00B184F8 ) = 0x000003A4;
	*mkptr( dfe_cfg_base, 0x00B184FC ) = 0x00005BE8;
	*mkptr( dfe_cfg_base, 0x00B18500 ) = 0x000003A8;
	*mkptr( dfe_cfg_base, 0x00B18504 ) = 0x00005A76;
	*mkptr( dfe_cfg_base, 0x00B18508 ) = 0x000003AC;
	*mkptr( dfe_cfg_base, 0x00B1850C ) = 0x00005914;
	*mkptr( dfe_cfg_base, 0x00B18510 ) = 0x000003B0;
	*mkptr( dfe_cfg_base, 0x00B18514 ) = 0x000057C2;
	*mkptr( dfe_cfg_base, 0x00B18518 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B1851C ) = 0x0000567F;
	*mkptr( dfe_cfg_base, 0x00B18520 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B18524 ) = 0x00005549;
	*mkptr( dfe_cfg_base, 0x00B18528 ) = 0x000003B8;
	*mkptr( dfe_cfg_base, 0x00B1852C ) = 0x00005420;
	*mkptr( dfe_cfg_base, 0x00B18530 ) = 0x000003BC;
	*mkptr( dfe_cfg_base, 0x00B18534 ) = 0x00005303;
	*mkptr( dfe_cfg_base, 0x00B18538 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B1853C ) = 0x000051F1;
	*mkptr( dfe_cfg_base, 0x00B18540 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B18544 ) = 0x000050E9;
	*mkptr( dfe_cfg_base, 0x00B18548 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B1854C ) = 0x00004FEB;
	*mkptr( dfe_cfg_base, 0x00B18550 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B18554 ) = 0x00004EF6;
	*mkptr( dfe_cfg_base, 0x00B18558 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B1855C ) = 0x00004E0A;
	*mkptr( dfe_cfg_base, 0x00B18560 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B18564 ) = 0x00004D25;
	*mkptr( dfe_cfg_base, 0x00B18568 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1856C ) = 0x00004C49;
	*mkptr( dfe_cfg_base, 0x00B18570 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B18574 ) = 0x00004B73;
	*mkptr( dfe_cfg_base, 0x00B18578 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1857C ) = 0x00004AA5;
	*mkptr( dfe_cfg_base, 0x00B18580 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B18584 ) = 0x000049DD;
	*mkptr( dfe_cfg_base, 0x00B18588 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B1858C ) = 0x0000491B;
	*mkptr( dfe_cfg_base, 0x00B18590 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B18594 ) = 0x0000485F;
	*mkptr( dfe_cfg_base, 0x00B18598 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B1859C ) = 0x000047A8;
	*mkptr( dfe_cfg_base, 0x00B185A0 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B185A4 ) = 0x000046F7;
	*mkptr( dfe_cfg_base, 0x00B185A8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B185AC ) = 0x0000464B;
	*mkptr( dfe_cfg_base, 0x00B185B0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B185B4 ) = 0x000045A3;
	*mkptr( dfe_cfg_base, 0x00B185B8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B185BC ) = 0x00004501;
	*mkptr( dfe_cfg_base, 0x00B185C0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B185C4 ) = 0x00004462;
	*mkptr( dfe_cfg_base, 0x00B185C8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B185CC ) = 0x000043C8;
	*mkptr( dfe_cfg_base, 0x00B185D0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B185D4 ) = 0x00004332;
	*mkptr( dfe_cfg_base, 0x00B185D8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B185DC ) = 0x0000429F;
	*mkptr( dfe_cfg_base, 0x00B185E0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B185E4 ) = 0x00004211;
	*mkptr( dfe_cfg_base, 0x00B185E8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B185EC ) = 0x00004185;
	*mkptr( dfe_cfg_base, 0x00B185F0 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B185F4 ) = 0x000040FE;
	*mkptr( dfe_cfg_base, 0x00B185F8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B185FC ) = 0x00004079;
	*mkptr( dfe_cfg_base, 0x00B18600 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18604 ) = 0x00003FF8;
	*mkptr( dfe_cfg_base, 0x00B18608 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B1860C ) = 0x00003F79;
	*mkptr( dfe_cfg_base, 0x00B18610 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18614 ) = 0x00003EFE;
	*mkptr( dfe_cfg_base, 0x00B18618 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1861C ) = 0x00003E85;
	*mkptr( dfe_cfg_base, 0x00B18620 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18624 ) = 0x00003E0F;
	*mkptr( dfe_cfg_base, 0x00B18628 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1862C ) = 0x00003D9B;
	*mkptr( dfe_cfg_base, 0x00B18630 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18634 ) = 0x00003D2A;
	*mkptr( dfe_cfg_base, 0x00B18638 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1863C ) = 0x00003CBB;
	*mkptr( dfe_cfg_base, 0x00B18640 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18644 ) = 0x00003C4F;
	*mkptr( dfe_cfg_base, 0x00B18648 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1864C ) = 0x00003BE5;
	*mkptr( dfe_cfg_base, 0x00B18650 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18654 ) = 0x00003B7D;
	*mkptr( dfe_cfg_base, 0x00B18658 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1865C ) = 0x00003B17;
	*mkptr( dfe_cfg_base, 0x00B18660 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18664 ) = 0x00003AB3;
	*mkptr( dfe_cfg_base, 0x00B18668 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1866C ) = 0x00003A51;
	*mkptr( dfe_cfg_base, 0x00B18670 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18674 ) = 0x000039F1;
	*mkptr( dfe_cfg_base, 0x00B18678 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1867C ) = 0x00003993;
	*mkptr( dfe_cfg_base, 0x00B18680 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18684 ) = 0x00003937;
	*mkptr( dfe_cfg_base, 0x00B18688 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1868C ) = 0x000038DC;
	*mkptr( dfe_cfg_base, 0x00B18690 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B18694 ) = 0x00003883;
	*mkptr( dfe_cfg_base, 0x00B18698 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B1869C ) = 0x0000382C;
	*mkptr( dfe_cfg_base, 0x00B186A0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186A4 ) = 0x000037D6;
	*mkptr( dfe_cfg_base, 0x00B186A8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186AC ) = 0x00003782;
	*mkptr( dfe_cfg_base, 0x00B186B0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186B4 ) = 0x0000372F;
	*mkptr( dfe_cfg_base, 0x00B186B8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186BC ) = 0x000036DD;
	*mkptr( dfe_cfg_base, 0x00B186C0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186C4 ) = 0x0000368D;
	*mkptr( dfe_cfg_base, 0x00B186C8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186CC ) = 0x0000363F;
	*mkptr( dfe_cfg_base, 0x00B186D0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186D4 ) = 0x000035F1;
	*mkptr( dfe_cfg_base, 0x00B186D8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186DC ) = 0x000035A5;
	*mkptr( dfe_cfg_base, 0x00B186E0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186E4 ) = 0x0000355A;
	*mkptr( dfe_cfg_base, 0x00B186E8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B186EC ) = 0x00003511;
	*mkptr( dfe_cfg_base, 0x00B186F0 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B186F4 ) = 0x000034C8;
	*mkptr( dfe_cfg_base, 0x00B186F8 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B186FC ) = 0x00003481;
	*mkptr( dfe_cfg_base, 0x00B18700 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18704 ) = 0x0000343B;
	*mkptr( dfe_cfg_base, 0x00B18708 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1870C ) = 0x000033F6;
	*mkptr( dfe_cfg_base, 0x00B18710 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18714 ) = 0x000033B2;
	*mkptr( dfe_cfg_base, 0x00B18718 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1871C ) = 0x0000336F;
	*mkptr( dfe_cfg_base, 0x00B18720 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18724 ) = 0x0000332D;
	*mkptr( dfe_cfg_base, 0x00B18728 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1872C ) = 0x000032EC;
	*mkptr( dfe_cfg_base, 0x00B18730 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18734 ) = 0x000032AC;
	*mkptr( dfe_cfg_base, 0x00B18738 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1873C ) = 0x0000326C;
	*mkptr( dfe_cfg_base, 0x00B18740 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18744 ) = 0x0000322E;
	*mkptr( dfe_cfg_base, 0x00B18748 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1874C ) = 0x000031F1;
	*mkptr( dfe_cfg_base, 0x00B18750 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18754 ) = 0x000031B4;
	*mkptr( dfe_cfg_base, 0x00B18758 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1875C ) = 0x00003179;
	*mkptr( dfe_cfg_base, 0x00B18760 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18764 ) = 0x0000313E;
	*mkptr( dfe_cfg_base, 0x00B18768 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1876C ) = 0x00003104;
	*mkptr( dfe_cfg_base, 0x00B18770 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18774 ) = 0x000030CB;
	*mkptr( dfe_cfg_base, 0x00B18778 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1877C ) = 0x00003093;
	*mkptr( dfe_cfg_base, 0x00B18780 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18784 ) = 0x0000305B;
	*mkptr( dfe_cfg_base, 0x00B18788 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1878C ) = 0x00003024;
	*mkptr( dfe_cfg_base, 0x00B18790 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18794 ) = 0x00002FEE;
	*mkptr( dfe_cfg_base, 0x00B18798 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1879C ) = 0x00002FB8;
	*mkptr( dfe_cfg_base, 0x00B187A0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187A4 ) = 0x00002F84;
	*mkptr( dfe_cfg_base, 0x00B187A8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187AC ) = 0x00002F50;
	*mkptr( dfe_cfg_base, 0x00B187B0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187B4 ) = 0x00002F1C;
	*mkptr( dfe_cfg_base, 0x00B187B8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187BC ) = 0x00002EE9;
	*mkptr( dfe_cfg_base, 0x00B187C0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187C4 ) = 0x00002EB7;
	*mkptr( dfe_cfg_base, 0x00B187C8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187CC ) = 0x00002E86;
	*mkptr( dfe_cfg_base, 0x00B187D0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187D4 ) = 0x00002E55;
	*mkptr( dfe_cfg_base, 0x00B187D8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187DC ) = 0x00002E24;
	*mkptr( dfe_cfg_base, 0x00B187E0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187E4 ) = 0x00002DF5;
	*mkptr( dfe_cfg_base, 0x00B187E8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187EC ) = 0x00002DC6;
	*mkptr( dfe_cfg_base, 0x00B187F0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187F4 ) = 0x00002D97;
	*mkptr( dfe_cfg_base, 0x00B187F8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B187FC ) = 0x00002D69;
	*mkptr( dfe_cfg_base, 0x00B10140 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10144 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10148 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1014C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10150 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10154 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10158 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1015C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10160 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10164 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10168 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1016C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10170 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10174 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10178 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1017C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B1508C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x00B15094 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00B1509C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00B150A4 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00B150AC ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x00B150B4 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00B150BC ) = 0x00000700;
	*mkptr( dfe_cfg_base, 0x00B150C4 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00B150CC ) = 0x00000900;
	*mkptr( dfe_cfg_base, 0x00B150D4 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x00B150DC ) = 0x00000B00;
	*mkptr( dfe_cfg_base, 0x00B150E4 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00B150EC ) = 0x00000D00;
	*mkptr( dfe_cfg_base, 0x00B150F4 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x00B150FC ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00B14800 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x00B14804 ) = 0x00004130;
	*mkptr( dfe_cfg_base, 0x00B14808 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x00B1480C ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B14810 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B14814 ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B14818 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B1481C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14820 ) = 0x0000080C;
	*mkptr( dfe_cfg_base, 0x00B14824 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x00B14840 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14844 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14848 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1484C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14850 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14854 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14858 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1485C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14860 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14864 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14880 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14884 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14888 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1488C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14890 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14894 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14898 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1489C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B148A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B148C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B148E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B148E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14900 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14904 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14908 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1490C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14910 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14914 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14918 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1491C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14920 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14924 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14940 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14944 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14948 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1494C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14950 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14954 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14958 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1495C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14960 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14964 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14980 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14984 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14988 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1498C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14990 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14994 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14998 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1499C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B149A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B149C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B149E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B149E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14A00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14A24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14A40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14A64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14A80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14A9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14AA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14AC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ACC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AD0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AD4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AD8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ADC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14AE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14AE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14B00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14B24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14B40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14B64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14B80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14B9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14BA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14BC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BCC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BD0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BD4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BD8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BDC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14BE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14BE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14C00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14C24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14C40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14C64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14C80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14C9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14CA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14CC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CCC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CD0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CD4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CD8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CDC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14CE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14CE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14D00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14D24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14D40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14D64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14D80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14D9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14DA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14DC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DCC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DD0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DD4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DD8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DDC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14DE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14DE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14E00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14E24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14E40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14E64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14E80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14E9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14EA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14EA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14EC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14EC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14EC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ECC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ED0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ED4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14ED8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14EDC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14EE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14EE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14F00 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F04 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F10 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F14 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F18 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F1C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F20 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14F24 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14F40 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F44 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F48 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F4C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F50 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F54 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F58 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F5C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F60 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14F64 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14F80 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F84 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F90 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F94 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F98 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14F9C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FA0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14FA4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14FC0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FC4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FC8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FCC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FD0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FD4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FD8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FDC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14FE0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14FE4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B040C8 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00B040CC ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B044CC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B040D0 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B044D0 ) = 0x00000FFD;
	*mkptr( dfe_cfg_base, 0x00B040D4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B044D4 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B040D8 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B044D8 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B040DC ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B044DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B040E0 ) = 0x00000041;
	*mkptr( dfe_cfg_base, 0x00B044E0 ) = 0x00000032;
	*mkptr( dfe_cfg_base, 0x00B040E4 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B044E4 ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B040E8 ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B044E8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B040EC ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B044EC ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B040F0 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B044F0 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B040F4 ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B044F4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B040F8 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B044F8 ) = 0x00000FF9;
	*mkptr( dfe_cfg_base, 0x00B040FC ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B044FC ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B04100 ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B04500 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B04104 ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B04504 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B04108 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x00B04508 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B0410C ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B0450C ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B04110 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B04510 ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B04114 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B04514 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B04118 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B04518 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0411C ) = 0x00000FF2;
	*mkptr( dfe_cfg_base, 0x00B0451C ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B04120 ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00B04520 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B04124 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B04524 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B04128 ) = 0x00000F8E;
	*mkptr( dfe_cfg_base, 0x00B04528 ) = 0x00000FB9;
	*mkptr( dfe_cfg_base, 0x00B0412C ) = 0x00000FB2;
	*mkptr( dfe_cfg_base, 0x00B0452C ) = 0x00000F8B;
	*mkptr( dfe_cfg_base, 0x00B04130 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B04530 ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B04134 ) = 0x00000FFA;
	*mkptr( dfe_cfg_base, 0x00B04534 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04138 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B04538 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B0413C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0453C ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B04140 ) = 0x00000F56;
	*mkptr( dfe_cfg_base, 0x00B04540 ) = 0x00000FB3;
	*mkptr( dfe_cfg_base, 0x00B04144 ) = 0x00000F3B;
	*mkptr( dfe_cfg_base, 0x00B04544 ) = 0x00000F20;
	*mkptr( dfe_cfg_base, 0x00B04148 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B04548 ) = 0x00000F92;
	*mkptr( dfe_cfg_base, 0x00B0414C ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00B0454C ) = 0x00000025;
	*mkptr( dfe_cfg_base, 0x00B04150 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B04550 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04154 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B04554 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B04158 ) = 0x00000F54;
	*mkptr( dfe_cfg_base, 0x00B04558 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B0415C ) = 0x00000EB1;
	*mkptr( dfe_cfg_base, 0x00B0455C ) = 0x00000EDF;
	*mkptr( dfe_cfg_base, 0x00B04160 ) = 0x00000FAE;
	*mkptr( dfe_cfg_base, 0x00B04560 ) = 0x00000F08;
	*mkptr( dfe_cfg_base, 0x00B04164 ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x00B04564 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00B04168 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B04568 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00B0416C ) = 0x00000FE6;
	*mkptr( dfe_cfg_base, 0x00B0456C ) = 0x00000FC1;
	*mkptr( dfe_cfg_base, 0x00B04170 ) = 0x00000FB7;
	*mkptr( dfe_cfg_base, 0x00B04570 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B04174 ) = 0x00000E40;
	*mkptr( dfe_cfg_base, 0x00B04574 ) = 0x00000EFA;
	*mkptr( dfe_cfg_base, 0x00B04178 ) = 0x00000EF5;
	*mkptr( dfe_cfg_base, 0x00B04578 ) = 0x00000E24;
	*mkptr( dfe_cfg_base, 0x00B0417C ) = 0x0000013E;
	*mkptr( dfe_cfg_base, 0x00B0457C ) = 0x0000004A;
	*mkptr( dfe_cfg_base, 0x00B04180 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B04580 ) = 0x0000010E;
	*mkptr( dfe_cfg_base, 0x00B04184 ) = 0x00000E51;
	*mkptr( dfe_cfg_base, 0x00B04584 ) = 0x00000E6C;
	*mkptr( dfe_cfg_base, 0x00B04188 ) = 0x00000386;
	*mkptr( dfe_cfg_base, 0x00B04588 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B0418C ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B0458C ) = 0x000006B1;
	*mkptr( dfe_cfg_base, 0x00B080C8 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x00B080CC ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B084CC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B080D0 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B084D0 ) = 0x00000FFD;
	*mkptr( dfe_cfg_base, 0x00B080D4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B084D4 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B080D8 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B084D8 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B080DC ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B084DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B080E0 ) = 0x00000041;
	*mkptr( dfe_cfg_base, 0x00B084E0 ) = 0x00000032;
	*mkptr( dfe_cfg_base, 0x00B080E4 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B084E4 ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B080E8 ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B084E8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B080EC ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B084EC ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B080F0 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B084F0 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B080F4 ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B084F4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B080F8 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B084F8 ) = 0x00000FF9;
	*mkptr( dfe_cfg_base, 0x00B080FC ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B084FC ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B08100 ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B08500 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B08104 ) = 0x00000FF8;
	*mkptr( dfe_cfg_base, 0x00B08504 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B08108 ) = 0x00000017;
	*mkptr( dfe_cfg_base, 0x00B08508 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B0810C ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B0850C ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00B08110 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B08510 ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B08114 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B08514 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B08118 ) = 0x00000FFB;
	*mkptr( dfe_cfg_base, 0x00B08518 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0811C ) = 0x00000FF2;
	*mkptr( dfe_cfg_base, 0x00B0851C ) = 0x00000FEF;
	*mkptr( dfe_cfg_base, 0x00B08120 ) = 0x0000001E;
	*mkptr( dfe_cfg_base, 0x00B08520 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00B08124 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B08524 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B08128 ) = 0x00000F8E;
	*mkptr( dfe_cfg_base, 0x00B08528 ) = 0x00000FB9;
	*mkptr( dfe_cfg_base, 0x00B0812C ) = 0x00000FB2;
	*mkptr( dfe_cfg_base, 0x00B0852C ) = 0x00000F8B;
	*mkptr( dfe_cfg_base, 0x00B08130 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B08530 ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B08134 ) = 0x00000FFA;
	*mkptr( dfe_cfg_base, 0x00B08534 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08138 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B08538 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x00B0813C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0853C ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B08140 ) = 0x00000F56;
	*mkptr( dfe_cfg_base, 0x00B08540 ) = 0x00000FB3;
	*mkptr( dfe_cfg_base, 0x00B08144 ) = 0x00000F3B;
	*mkptr( dfe_cfg_base, 0x00B08544 ) = 0x00000F20;
	*mkptr( dfe_cfg_base, 0x00B08148 ) = 0x00000FF1;
	*mkptr( dfe_cfg_base, 0x00B08548 ) = 0x00000F92;
	*mkptr( dfe_cfg_base, 0x00B0814C ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x00B0854C ) = 0x00000025;
	*mkptr( dfe_cfg_base, 0x00B08150 ) = 0x00000FFE;
	*mkptr( dfe_cfg_base, 0x00B08550 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08154 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B08554 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x00B08158 ) = 0x00000F54;
	*mkptr( dfe_cfg_base, 0x00B08558 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B0815C ) = 0x00000EB1;
	*mkptr( dfe_cfg_base, 0x00B0855C ) = 0x00000EDF;
	*mkptr( dfe_cfg_base, 0x00B08160 ) = 0x00000FAE;
	*mkptr( dfe_cfg_base, 0x00B08560 ) = 0x00000F08;
	*mkptr( dfe_cfg_base, 0x00B08164 ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x00B08564 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00B08168 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B08568 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00B0816C ) = 0x00000FE6;
	*mkptr( dfe_cfg_base, 0x00B0856C ) = 0x00000FC1;
	*mkptr( dfe_cfg_base, 0x00B08170 ) = 0x00000FB7;
	*mkptr( dfe_cfg_base, 0x00B08570 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B08174 ) = 0x00000E40;
	*mkptr( dfe_cfg_base, 0x00B08574 ) = 0x00000EFA;
	*mkptr( dfe_cfg_base, 0x00B08178 ) = 0x00000EF5;
	*mkptr( dfe_cfg_base, 0x00B08578 ) = 0x00000E24;
	*mkptr( dfe_cfg_base, 0x00B0817C ) = 0x0000013E;
	*mkptr( dfe_cfg_base, 0x00B0857C ) = 0x0000004A;
	*mkptr( dfe_cfg_base, 0x00B08180 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B08580 ) = 0x0000010E;
	*mkptr( dfe_cfg_base, 0x00B08184 ) = 0x00000E51;
	*mkptr( dfe_cfg_base, 0x00B08584 ) = 0x00000E6C;
	*mkptr( dfe_cfg_base, 0x00B08188 ) = 0x00000386;
	*mkptr( dfe_cfg_base, 0x00B08588 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B0818C ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B0858C ) = 0x000006B1;
	*mkptr( dfe_cfg_base, 0x00B803CC ) = 0x0000800D;
	*mkptr( dfe_cfg_base, 0x00B80208 ) = 0x00001180;
	*mkptr( dfe_cfg_base, 0x00B80288 ) = 0x00001180;
	*mkptr( dfe_cfg_base, 0x00B80360 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x00B80370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B80200 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80280 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80210 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B80218 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80230 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B80290 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B80298 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B802B0 ) = 0x00003131;
	*mkptr( dfe_cfg_base, 0x00B80300 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B80308 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B80310 ) = 0x0000BF5C;
	*mkptr( dfe_cfg_base, 0x00B80318 ) = 0x0000BF5C;
	*mkptr( dfe_cfg_base, 0x00B80320 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B80328 ) = 0x0000013C;
	*mkptr( dfe_cfg_base, 0x00B80338 ) = 0x00000BB8;
	*mkptr( dfe_cfg_base, 0x00B80340 ) = 0x00000363;
	*mkptr( dfe_cfg_base, 0x00B80348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00B80378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B98004 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9800C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98014 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9801C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98024 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9802C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98034 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9803C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98044 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9804C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98054 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9805C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98064 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9806C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98074 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9807C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98080 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x00B98084 ) = 0x00007FE8;
	*mkptr( dfe_cfg_base, 0x00B98088 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x00B9808C ) = 0x00007C17;
	*mkptr( dfe_cfg_base, 0x00B98090 ) = 0x00000334;
	*mkptr( dfe_cfg_base, 0x00B98094 ) = 0x00007899;
	*mkptr( dfe_cfg_base, 0x00B98098 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00B9809C ) = 0x00007562;
	*mkptr( dfe_cfg_base, 0x00B980A0 ) = 0x00000350;
	*mkptr( dfe_cfg_base, 0x00B980A4 ) = 0x0000726A;
	*mkptr( dfe_cfg_base, 0x00B980A8 ) = 0x0000035C;
	*mkptr( dfe_cfg_base, 0x00B980AC ) = 0x00006FA8;
	*mkptr( dfe_cfg_base, 0x00B980B0 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x00B980B4 ) = 0x00006D17;
	*mkptr( dfe_cfg_base, 0x00B980B8 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00B980BC ) = 0x00006AB2;
	*mkptr( dfe_cfg_base, 0x00B980C0 ) = 0x00000378;
	*mkptr( dfe_cfg_base, 0x00B980C4 ) = 0x00006873;
	*mkptr( dfe_cfg_base, 0x00B980C8 ) = 0x00000380;
	*mkptr( dfe_cfg_base, 0x00B980CC ) = 0x00006657;
	*mkptr( dfe_cfg_base, 0x00B980D0 ) = 0x00000388;
	*mkptr( dfe_cfg_base, 0x00B980D4 ) = 0x0000645A;
	*mkptr( dfe_cfg_base, 0x00B980D8 ) = 0x00000390;
	*mkptr( dfe_cfg_base, 0x00B980DC ) = 0x0000627A;
	*mkptr( dfe_cfg_base, 0x00B980E0 ) = 0x00000394;
	*mkptr( dfe_cfg_base, 0x00B980E4 ) = 0x000060B4;
	*mkptr( dfe_cfg_base, 0x00B980E8 ) = 0x00000398;
	*mkptr( dfe_cfg_base, 0x00B980EC ) = 0x00005F06;
	*mkptr( dfe_cfg_base, 0x00B980F0 ) = 0x000003A0;
	*mkptr( dfe_cfg_base, 0x00B980F4 ) = 0x00005D6D;
	*mkptr( dfe_cfg_base, 0x00B980F8 ) = 0x000003A4;
	*mkptr( dfe_cfg_base, 0x00B980FC ) = 0x00005BE8;
	*mkptr( dfe_cfg_base, 0x00B98100 ) = 0x000003A8;
	*mkptr( dfe_cfg_base, 0x00B98104 ) = 0x00005A76;
	*mkptr( dfe_cfg_base, 0x00B98108 ) = 0x000003AC;
	*mkptr( dfe_cfg_base, 0x00B9810C ) = 0x00005914;
	*mkptr( dfe_cfg_base, 0x00B98110 ) = 0x000003B0;
	*mkptr( dfe_cfg_base, 0x00B98114 ) = 0x000057C2;
	*mkptr( dfe_cfg_base, 0x00B98118 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B9811C ) = 0x0000567F;
	*mkptr( dfe_cfg_base, 0x00B98120 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B98124 ) = 0x00005549;
	*mkptr( dfe_cfg_base, 0x00B98128 ) = 0x000003B8;
	*mkptr( dfe_cfg_base, 0x00B9812C ) = 0x00005420;
	*mkptr( dfe_cfg_base, 0x00B98130 ) = 0x000003BC;
	*mkptr( dfe_cfg_base, 0x00B98134 ) = 0x00005303;
	*mkptr( dfe_cfg_base, 0x00B98138 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B9813C ) = 0x000051F1;
	*mkptr( dfe_cfg_base, 0x00B98140 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B98144 ) = 0x000050E9;
	*mkptr( dfe_cfg_base, 0x00B98148 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B9814C ) = 0x00004FEB;
	*mkptr( dfe_cfg_base, 0x00B98150 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B98154 ) = 0x00004EF6;
	*mkptr( dfe_cfg_base, 0x00B98158 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B9815C ) = 0x00004E0A;
	*mkptr( dfe_cfg_base, 0x00B98160 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B98164 ) = 0x00004D25;
	*mkptr( dfe_cfg_base, 0x00B98168 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B9816C ) = 0x00004C49;
	*mkptr( dfe_cfg_base, 0x00B98170 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B98174 ) = 0x00004B73;
	*mkptr( dfe_cfg_base, 0x00B98178 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B9817C ) = 0x00004AA5;
	*mkptr( dfe_cfg_base, 0x00B98180 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B98184 ) = 0x000049DD;
	*mkptr( dfe_cfg_base, 0x00B98188 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B9818C ) = 0x0000491B;
	*mkptr( dfe_cfg_base, 0x00B98190 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B98194 ) = 0x0000485F;
	*mkptr( dfe_cfg_base, 0x00B98198 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B9819C ) = 0x000047A8;
	*mkptr( dfe_cfg_base, 0x00B981A0 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B981A4 ) = 0x000046F7;
	*mkptr( dfe_cfg_base, 0x00B981A8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981AC ) = 0x0000464B;
	*mkptr( dfe_cfg_base, 0x00B981B0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981B4 ) = 0x000045A3;
	*mkptr( dfe_cfg_base, 0x00B981B8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981BC ) = 0x00004501;
	*mkptr( dfe_cfg_base, 0x00B981C0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981C4 ) = 0x00004462;
	*mkptr( dfe_cfg_base, 0x00B981C8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981CC ) = 0x000043C8;
	*mkptr( dfe_cfg_base, 0x00B981D0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981D4 ) = 0x00004332;
	*mkptr( dfe_cfg_base, 0x00B981D8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981DC ) = 0x0000429F;
	*mkptr( dfe_cfg_base, 0x00B981E0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981E4 ) = 0x00004211;
	*mkptr( dfe_cfg_base, 0x00B981E8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981EC ) = 0x00004185;
	*mkptr( dfe_cfg_base, 0x00B981F0 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981F4 ) = 0x000040FE;
	*mkptr( dfe_cfg_base, 0x00B981F8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981FC ) = 0x00004079;
	*mkptr( dfe_cfg_base, 0x00B98200 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B98204 ) = 0x00003FF8;
	*mkptr( dfe_cfg_base, 0x00B98208 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B9820C ) = 0x00003F79;
	*mkptr( dfe_cfg_base, 0x00B98210 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B98214 ) = 0x00003EFE;
	*mkptr( dfe_cfg_base, 0x00B98218 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9821C ) = 0x00003E85;
	*mkptr( dfe_cfg_base, 0x00B98220 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98224 ) = 0x00003E0F;
	*mkptr( dfe_cfg_base, 0x00B98228 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9822C ) = 0x00003D9B;
	*mkptr( dfe_cfg_base, 0x00B98230 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98234 ) = 0x00003D2A;
	*mkptr( dfe_cfg_base, 0x00B98238 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9823C ) = 0x00003CBB;
	*mkptr( dfe_cfg_base, 0x00B98240 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98244 ) = 0x00003C4F;
	*mkptr( dfe_cfg_base, 0x00B98248 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9824C ) = 0x00003BE5;
	*mkptr( dfe_cfg_base, 0x00B98250 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98254 ) = 0x00003B7D;
	*mkptr( dfe_cfg_base, 0x00B98258 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9825C ) = 0x00003B17;
	*mkptr( dfe_cfg_base, 0x00B98260 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98264 ) = 0x00003AB3;
	*mkptr( dfe_cfg_base, 0x00B98268 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9826C ) = 0x00003A51;
	*mkptr( dfe_cfg_base, 0x00B98270 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98274 ) = 0x000039F1;
	*mkptr( dfe_cfg_base, 0x00B98278 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9827C ) = 0x00003993;
	*mkptr( dfe_cfg_base, 0x00B98280 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98284 ) = 0x00003937;
	*mkptr( dfe_cfg_base, 0x00B98288 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9828C ) = 0x000038DC;
	*mkptr( dfe_cfg_base, 0x00B98290 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B98294 ) = 0x00003883;
	*mkptr( dfe_cfg_base, 0x00B98298 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B9829C ) = 0x0000382C;
	*mkptr( dfe_cfg_base, 0x00B982A0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982A4 ) = 0x000037D6;
	*mkptr( dfe_cfg_base, 0x00B982A8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982AC ) = 0x00003782;
	*mkptr( dfe_cfg_base, 0x00B982B0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982B4 ) = 0x0000372F;
	*mkptr( dfe_cfg_base, 0x00B982B8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982BC ) = 0x000036DD;
	*mkptr( dfe_cfg_base, 0x00B982C0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982C4 ) = 0x0000368D;
	*mkptr( dfe_cfg_base, 0x00B982C8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982CC ) = 0x0000363F;
	*mkptr( dfe_cfg_base, 0x00B982D0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982D4 ) = 0x000035F1;
	*mkptr( dfe_cfg_base, 0x00B982D8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982DC ) = 0x000035A5;
	*mkptr( dfe_cfg_base, 0x00B982E0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982E4 ) = 0x0000355A;
	*mkptr( dfe_cfg_base, 0x00B982E8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982EC ) = 0x00003511;
	*mkptr( dfe_cfg_base, 0x00B982F0 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B982F4 ) = 0x000034C8;
	*mkptr( dfe_cfg_base, 0x00B982F8 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B982FC ) = 0x00003481;
	*mkptr( dfe_cfg_base, 0x00B98300 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98304 ) = 0x0000343B;
	*mkptr( dfe_cfg_base, 0x00B98308 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9830C ) = 0x000033F6;
	*mkptr( dfe_cfg_base, 0x00B98310 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98314 ) = 0x000033B2;
	*mkptr( dfe_cfg_base, 0x00B98318 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9831C ) = 0x0000336F;
	*mkptr( dfe_cfg_base, 0x00B98320 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98324 ) = 0x0000332D;
	*mkptr( dfe_cfg_base, 0x00B98328 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9832C ) = 0x000032EC;
	*mkptr( dfe_cfg_base, 0x00B98330 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98334 ) = 0x000032AC;
	*mkptr( dfe_cfg_base, 0x00B98338 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9833C ) = 0x0000326C;
	*mkptr( dfe_cfg_base, 0x00B98340 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98344 ) = 0x0000322E;
	*mkptr( dfe_cfg_base, 0x00B98348 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9834C ) = 0x000031F1;
	*mkptr( dfe_cfg_base, 0x00B98350 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98354 ) = 0x000031B4;
	*mkptr( dfe_cfg_base, 0x00B98358 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9835C ) = 0x00003179;
	*mkptr( dfe_cfg_base, 0x00B98360 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98364 ) = 0x0000313E;
	*mkptr( dfe_cfg_base, 0x00B98368 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9836C ) = 0x00003104;
	*mkptr( dfe_cfg_base, 0x00B98370 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98374 ) = 0x000030CB;
	*mkptr( dfe_cfg_base, 0x00B98378 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9837C ) = 0x00003093;
	*mkptr( dfe_cfg_base, 0x00B98380 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B98384 ) = 0x0000305B;
	*mkptr( dfe_cfg_base, 0x00B98388 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9838C ) = 0x00003024;
	*mkptr( dfe_cfg_base, 0x00B98390 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B98394 ) = 0x00002FEE;
	*mkptr( dfe_cfg_base, 0x00B98398 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9839C ) = 0x00002FB8;
	*mkptr( dfe_cfg_base, 0x00B983A0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983A4 ) = 0x00002F84;
	*mkptr( dfe_cfg_base, 0x00B983A8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983AC ) = 0x00002F50;
	*mkptr( dfe_cfg_base, 0x00B983B0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983B4 ) = 0x00002F1C;
	*mkptr( dfe_cfg_base, 0x00B983B8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983BC ) = 0x00002EE9;
	*mkptr( dfe_cfg_base, 0x00B983C0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983C4 ) = 0x00002EB7;
	*mkptr( dfe_cfg_base, 0x00B983C8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983CC ) = 0x00002E86;
	*mkptr( dfe_cfg_base, 0x00B983D0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983D4 ) = 0x00002E55;
	*mkptr( dfe_cfg_base, 0x00B983D8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983DC ) = 0x00002E24;
	*mkptr( dfe_cfg_base, 0x00B983E0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983E4 ) = 0x00002DF5;
	*mkptr( dfe_cfg_base, 0x00B983E8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983EC ) = 0x00002DC6;
	*mkptr( dfe_cfg_base, 0x00B983F0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983F4 ) = 0x00002D97;
	*mkptr( dfe_cfg_base, 0x00B983F8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983FC ) = 0x00002D69;
	*mkptr( dfe_cfg_base, 0x00B90100 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90108 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9010C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90110 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90118 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9011C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90120 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90128 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9012C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90130 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90138 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9013C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B9500C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x00B95014 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00B9501C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00B95024 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00B9502C ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x00B95034 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00B9503C ) = 0x00000700;
	*mkptr( dfe_cfg_base, 0x00B95044 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00B9504C ) = 0x00000900;
	*mkptr( dfe_cfg_base, 0x00B95054 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x00B9505C ) = 0x00000B00;
	*mkptr( dfe_cfg_base, 0x00B95064 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00B9506C ) = 0x00000D00;
	*mkptr( dfe_cfg_base, 0x00B95074 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x00B9507C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00B94000 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x00B94004 ) = 0x00004130;
	*mkptr( dfe_cfg_base, 0x00B94008 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x00B9400C ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B94010 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B94014 ) = 0x00003821;
	*mkptr( dfe_cfg_base, 0x00B94018 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B9401C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B94020 ) = 0x0000080C;
	*mkptr( dfe_cfg_base, 0x00B94024 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x00B94040 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94044 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94048 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9404C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94050 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94054 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94058 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9405C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94060 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94064 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94080 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94084 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94088 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9408C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94090 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94094 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94098 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9409C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94100 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94104 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94108 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9410C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94110 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94114 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94118 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9411C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94120 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94124 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94140 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94144 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94148 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9414C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94150 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94154 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94158 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9415C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94160 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94164 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94180 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94184 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94188 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9418C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94190 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94194 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94198 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9419C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94200 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94204 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94208 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9420C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94210 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94214 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94218 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9421C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94220 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94224 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94240 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94244 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94248 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9424C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94250 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94254 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94258 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9425C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94260 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94264 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94280 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94284 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94288 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9428C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94290 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94294 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94298 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9429C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94300 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94304 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94308 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9430C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94310 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94314 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94318 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9431C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94320 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94324 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94340 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94344 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94348 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9434C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94350 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94354 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94358 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9435C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94360 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94364 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94380 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94384 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94388 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9438C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94390 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94394 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94398 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9439C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94400 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94404 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94408 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9440C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94410 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94414 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94418 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9441C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94420 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94424 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94440 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94444 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94448 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9444C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94450 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94454 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94458 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9445C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94460 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94464 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94480 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94484 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94488 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9448C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94490 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94494 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94498 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9449C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94500 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94504 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94508 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9450C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94510 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94514 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94518 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9451C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94520 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94524 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94540 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94544 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94548 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9454C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94550 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94554 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94558 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9455C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94560 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94564 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94580 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94584 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94588 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9458C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94590 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94594 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94598 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9459C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94600 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94604 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94608 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9460C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94610 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94614 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94618 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9461C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94620 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94624 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94640 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94644 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9464C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94650 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94654 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94658 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9465C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94660 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94664 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94680 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94684 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94688 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9468C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94690 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94694 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94698 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9469C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94700 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94704 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94708 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9470C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94710 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94714 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94718 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9471C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94720 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94724 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94740 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94744 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94748 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9474C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94750 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94754 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94758 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9475C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94760 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94764 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94780 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94784 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94788 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9478C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94790 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94794 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94798 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9479C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;


	iqn2_bb_loopback_config_split_1();
	iqn2_bb_loopback_config_split_2();
}

#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config_split_1,".text:dfe");
#endif
static void iqn2_bb_loopback_config_split_1() {
    *mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
        *mkptr( dfe_cfg_base, 0x00B84000 ) = 0x0000000F;
        *mkptr( dfe_cfg_base, 0x00B84004 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B84404 ) = 0x00000008;
        *mkptr( dfe_cfg_base, 0x00B84008 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B84408 ) = 0x00000FFD;
        *mkptr( dfe_cfg_base, 0x00B8400C ) = 0x00000FFF;
        *mkptr( dfe_cfg_base, 0x00B8440C ) = 0x00000006;
        *mkptr( dfe_cfg_base, 0x00B84010 ) = 0x00000FF7;
        *mkptr( dfe_cfg_base, 0x00B84410 ) = 0x00000FF7;
        *mkptr( dfe_cfg_base, 0x00B84014 ) = 0x00000019;
        *mkptr( dfe_cfg_base, 0x00B84414 ) = 0x00000003;
        *mkptr( dfe_cfg_base, 0x00B84018 ) = 0x00000041;
        *mkptr( dfe_cfg_base, 0x00B84418 ) = 0x00000032;
        *mkptr( dfe_cfg_base, 0x00B8401C ) = 0x00000021;
        *mkptr( dfe_cfg_base, 0x00B8441C ) = 0x0000003B;
        *mkptr( dfe_cfg_base, 0x00B84020 ) = 0x00000FEE;
        *mkptr( dfe_cfg_base, 0x00B84420 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B84024 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B84424 ) = 0x00000FEF;
        *mkptr( dfe_cfg_base, 0x00B84028 ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B84428 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B8402C ) = 0x00000FF8;
        *mkptr( dfe_cfg_base, 0x00B8442C ) = 0x00000002;
        *mkptr( dfe_cfg_base, 0x00B84030 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B84430 ) = 0x00000FF9;
        *mkptr( dfe_cfg_base, 0x00B84034 ) = 0x00000015;
        *mkptr( dfe_cfg_base, 0x00B84434 ) = 0x00000014;
        *mkptr( dfe_cfg_base, 0x00B84038 ) = 0x00000FF4;
        *mkptr( dfe_cfg_base, 0x00B84438 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B8403C ) = 0x00000FF8;
        *mkptr( dfe_cfg_base, 0x00B8443C ) = 0x00000FED;
        *mkptr( dfe_cfg_base, 0x00B84040 ) = 0x00000017;
        *mkptr( dfe_cfg_base, 0x00B84440 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B84044 ) = 0x00000FF1;
        *mkptr( dfe_cfg_base, 0x00B84444 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B84048 ) = 0x00000FD4;
        *mkptr( dfe_cfg_base, 0x00B84448 ) = 0x00000FD8;
        *mkptr( dfe_cfg_base, 0x00B8404C ) = 0x00000FFB;
        *mkptr( dfe_cfg_base, 0x00B8444C ) = 0x00000FE5;
        *mkptr( dfe_cfg_base, 0x00B84050 ) = 0x00000FFB;
        *mkptr( dfe_cfg_base, 0x00B84450 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B84054 ) = 0x00000FF2;
        *mkptr( dfe_cfg_base, 0x00B84454 ) = 0x00000FEF;
        *mkptr( dfe_cfg_base, 0x00B84058 ) = 0x0000001E;
        *mkptr( dfe_cfg_base, 0x00B84458 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B8405C ) = 0x00000FF5;
        *mkptr( dfe_cfg_base, 0x00B8445C ) = 0x0000001C;
        *mkptr( dfe_cfg_base, 0x00B84060 ) = 0x00000F8E;
        *mkptr( dfe_cfg_base, 0x00B84460 ) = 0x00000FB9;
        *mkptr( dfe_cfg_base, 0x00B84064 ) = 0x00000FB2;
        *mkptr( dfe_cfg_base, 0x00B84464 ) = 0x00000F8B;
        *mkptr( dfe_cfg_base, 0x00B84068 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B84468 ) = 0x00000FE4;
        *mkptr( dfe_cfg_base, 0x00B8406C ) = 0x00000FFA;
        *mkptr( dfe_cfg_base, 0x00B8446C ) = 0x00000003;
        *mkptr( dfe_cfg_base, 0x00B84070 ) = 0x00000016;
        *mkptr( dfe_cfg_base, 0x00B84470 ) = 0x00000FFF;
        *mkptr( dfe_cfg_base, 0x00B84074 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B84474 ) = 0x00000023;
        *mkptr( dfe_cfg_base, 0x00B84078 ) = 0x00000F56;
        *mkptr( dfe_cfg_base, 0x00B84478 ) = 0x00000FB3;
        *mkptr( dfe_cfg_base, 0x00B8407C ) = 0x00000F3B;
        *mkptr( dfe_cfg_base, 0x00B8447C ) = 0x00000F20;
        *mkptr( dfe_cfg_base, 0x00B84080 ) = 0x00000FF1;
        *mkptr( dfe_cfg_base, 0x00B84480 ) = 0x00000F92;
        *mkptr( dfe_cfg_base, 0x00B84084 ) = 0x0000001F;
        *mkptr( dfe_cfg_base, 0x00B84484 ) = 0x00000025;
        *mkptr( dfe_cfg_base, 0x00B84088 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B84488 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B8408C ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B8448C ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B84090 ) = 0x00000F54;
        *mkptr( dfe_cfg_base, 0x00B84490 ) = 0x00000FD5;
        *mkptr( dfe_cfg_base, 0x00B84094 ) = 0x00000EB1;
        *mkptr( dfe_cfg_base, 0x00B84494 ) = 0x00000EDF;
        *mkptr( dfe_cfg_base, 0x00B84098 ) = 0x00000FAE;
        *mkptr( dfe_cfg_base, 0x00B84498 ) = 0x00000F08;
        *mkptr( dfe_cfg_base, 0x00B8409C ) = 0x0000006F;
        *mkptr( dfe_cfg_base, 0x00B8449C ) = 0x00000040;
        *mkptr( dfe_cfg_base, 0x00B840A0 ) = 0x00000FDE;
        *mkptr( dfe_cfg_base, 0x00B844A0 ) = 0x00000035;
        *mkptr( dfe_cfg_base, 0x00B840A4 ) = 0x00000FE6;
        *mkptr( dfe_cfg_base, 0x00B844A4 ) = 0x00000FC1;
        *mkptr( dfe_cfg_base, 0x00B840A8 ) = 0x00000FB7;
        *mkptr( dfe_cfg_base, 0x00B844A8 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B840AC ) = 0x00000E40;
        *mkptr( dfe_cfg_base, 0x00B844AC ) = 0x00000EFA;
        *mkptr( dfe_cfg_base, 0x00B840B0 ) = 0x00000EF5;
        *mkptr( dfe_cfg_base, 0x00B844B0 ) = 0x00000E24;
        *mkptr( dfe_cfg_base, 0x00B840B4 ) = 0x0000013E;
        *mkptr( dfe_cfg_base, 0x00B844B4 ) = 0x0000004A;
        *mkptr( dfe_cfg_base, 0x00B840B8 ) = 0x00000FC8;
        *mkptr( dfe_cfg_base, 0x00B844B8 ) = 0x0000010E;
        *mkptr( dfe_cfg_base, 0x00B840BC ) = 0x00000E51;
        *mkptr( dfe_cfg_base, 0x00B844BC ) = 0x00000E6C;
        *mkptr( dfe_cfg_base, 0x00B840C0 ) = 0x00000386;
        *mkptr( dfe_cfg_base, 0x00B844C0 ) = 0x00000031;
        *mkptr( dfe_cfg_base, 0x00B840C4 ) = 0x000007FF;
        *mkptr( dfe_cfg_base, 0x00B844C4 ) = 0x000006B1;
        *mkptr( dfe_cfg_base, 0x00B88000 ) = 0x0000000F;
        *mkptr( dfe_cfg_base, 0x00B88004 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B88404 ) = 0x00000008;
        *mkptr( dfe_cfg_base, 0x00B88008 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B88408 ) = 0x00000FFD;
        *mkptr( dfe_cfg_base, 0x00B8800C ) = 0x00000FFF;
        *mkptr( dfe_cfg_base, 0x00B8840C ) = 0x00000006;
        *mkptr( dfe_cfg_base, 0x00B88010 ) = 0x00000FF7;
        *mkptr( dfe_cfg_base, 0x00B88410 ) = 0x00000FF7;
        *mkptr( dfe_cfg_base, 0x00B88014 ) = 0x00000019;
        *mkptr( dfe_cfg_base, 0x00B88414 ) = 0x00000003;
        *mkptr( dfe_cfg_base, 0x00B88018 ) = 0x00000041;
        *mkptr( dfe_cfg_base, 0x00B88418 ) = 0x00000032;
        *mkptr( dfe_cfg_base, 0x00B8801C ) = 0x00000021;
        *mkptr( dfe_cfg_base, 0x00B8841C ) = 0x0000003B;
        *mkptr( dfe_cfg_base, 0x00B88020 ) = 0x00000FEE;
        *mkptr( dfe_cfg_base, 0x00B88420 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B88024 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B88424 ) = 0x00000FEF;
        *mkptr( dfe_cfg_base, 0x00B88028 ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B88428 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B8802C ) = 0x00000FF8;
        *mkptr( dfe_cfg_base, 0x00B8842C ) = 0x00000002;
        *mkptr( dfe_cfg_base, 0x00B88030 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B88430 ) = 0x00000FF9;
        *mkptr( dfe_cfg_base, 0x00B88034 ) = 0x00000015;
        *mkptr( dfe_cfg_base, 0x00B88434 ) = 0x00000014;
        *mkptr( dfe_cfg_base, 0x00B88038 ) = 0x00000FF4;
        *mkptr( dfe_cfg_base, 0x00B88438 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B8803C ) = 0x00000FF8;
        *mkptr( dfe_cfg_base, 0x00B8843C ) = 0x00000FED;
        *mkptr( dfe_cfg_base, 0x00B88040 ) = 0x00000017;
        *mkptr( dfe_cfg_base, 0x00B88440 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B88044 ) = 0x00000FF1;
        *mkptr( dfe_cfg_base, 0x00B88444 ) = 0x0000000C;
        *mkptr( dfe_cfg_base, 0x00B88048 ) = 0x00000FD4;
        *mkptr( dfe_cfg_base, 0x00B88448 ) = 0x00000FD8;
        *mkptr( dfe_cfg_base, 0x00B8804C ) = 0x00000FFB;
        *mkptr( dfe_cfg_base, 0x00B8844C ) = 0x00000FE5;
        *mkptr( dfe_cfg_base, 0x00B88050 ) = 0x00000FFB;
        *mkptr( dfe_cfg_base, 0x00B88450 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B88054 ) = 0x00000FF2;
        *mkptr( dfe_cfg_base, 0x00B88454 ) = 0x00000FEF;
        *mkptr( dfe_cfg_base, 0x00B88058 ) = 0x0000001E;
        *mkptr( dfe_cfg_base, 0x00B88458 ) = 0x00000007;
        *mkptr( dfe_cfg_base, 0x00B8805C ) = 0x00000FF5;
        *mkptr( dfe_cfg_base, 0x00B8845C ) = 0x0000001C;
        *mkptr( dfe_cfg_base, 0x00B88060 ) = 0x00000F8E;
        *mkptr( dfe_cfg_base, 0x00B88460 ) = 0x00000FB9;
        *mkptr( dfe_cfg_base, 0x00B88064 ) = 0x00000FB2;
        *mkptr( dfe_cfg_base, 0x00B88464 ) = 0x00000F8B;
        *mkptr( dfe_cfg_base, 0x00B88068 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B88468 ) = 0x00000FE4;
        *mkptr( dfe_cfg_base, 0x00B8806C ) = 0x00000FFA;
        *mkptr( dfe_cfg_base, 0x00B8846C ) = 0x00000003;
        *mkptr( dfe_cfg_base, 0x00B88070 ) = 0x00000016;
        *mkptr( dfe_cfg_base, 0x00B88470 ) = 0x00000FFF;
        *mkptr( dfe_cfg_base, 0x00B88074 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B88474 ) = 0x00000023;
        *mkptr( dfe_cfg_base, 0x00B88078 ) = 0x00000F56;
        *mkptr( dfe_cfg_base, 0x00B88478 ) = 0x00000FB3;
        *mkptr( dfe_cfg_base, 0x00B8807C ) = 0x00000F3B;
        *mkptr( dfe_cfg_base, 0x00B8847C ) = 0x00000F20;
        *mkptr( dfe_cfg_base, 0x00B88080 ) = 0x00000FF1;
        *mkptr( dfe_cfg_base, 0x00B88480 ) = 0x00000F92;
        *mkptr( dfe_cfg_base, 0x00B88084 ) = 0x0000001F;
        *mkptr( dfe_cfg_base, 0x00B88484 ) = 0x00000025;
        *mkptr( dfe_cfg_base, 0x00B88088 ) = 0x00000FFE;
        *mkptr( dfe_cfg_base, 0x00B88488 ) = 0x00000004;
        *mkptr( dfe_cfg_base, 0x00B8808C ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B8848C ) = 0x0000000D;
        *mkptr( dfe_cfg_base, 0x00B88090 ) = 0x00000F54;
        *mkptr( dfe_cfg_base, 0x00B88490 ) = 0x00000FD5;
        *mkptr( dfe_cfg_base, 0x00B88094 ) = 0x00000EB1;
        *mkptr( dfe_cfg_base, 0x00B88494 ) = 0x00000EDF;
        *mkptr( dfe_cfg_base, 0x00B88098 ) = 0x00000FAE;
        *mkptr( dfe_cfg_base, 0x00B88498 ) = 0x00000F08;
        *mkptr( dfe_cfg_base, 0x00B8809C ) = 0x0000006F;
        *mkptr( dfe_cfg_base, 0x00B8849C ) = 0x00000040;
        *mkptr( dfe_cfg_base, 0x00B880A0 ) = 0x00000FDE;
        *mkptr( dfe_cfg_base, 0x00B884A0 ) = 0x00000035;
        *mkptr( dfe_cfg_base, 0x00B880A4 ) = 0x00000FE6;
        *mkptr( dfe_cfg_base, 0x00B884A4 ) = 0x00000FC1;
        *mkptr( dfe_cfg_base, 0x00B880A8 ) = 0x00000FB7;
        *mkptr( dfe_cfg_base, 0x00B884A8 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B880AC ) = 0x00000E40;
        *mkptr( dfe_cfg_base, 0x00B884AC ) = 0x00000EFA;
        *mkptr( dfe_cfg_base, 0x00B880B0 ) = 0x00000EF5;
        *mkptr( dfe_cfg_base, 0x00B884B0 ) = 0x00000E24;
        *mkptr( dfe_cfg_base, 0x00B880B4 ) = 0x0000013E;
        *mkptr( dfe_cfg_base, 0x00B884B4 ) = 0x0000004A;
        *mkptr( dfe_cfg_base, 0x00B880B8 ) = 0x00000FC8;
        *mkptr( dfe_cfg_base, 0x00B884B8 ) = 0x0000010E;
        *mkptr( dfe_cfg_base, 0x00B880BC ) = 0x00000E51;
        *mkptr( dfe_cfg_base, 0x00B884BC ) = 0x00000E6C;
        *mkptr( dfe_cfg_base, 0x00B880C0 ) = 0x00000386;
        *mkptr( dfe_cfg_base, 0x00B884C0 ) = 0x00000031;
        *mkptr( dfe_cfg_base, 0x00B880C4 ) = 0x000007FF;
        *mkptr( dfe_cfg_base, 0x00B884C4 ) = 0x000006B1;
        *mkptr( dfe_cfg_base, 0x00B80364 ) = 0x00007215;
        *mkptr( dfe_cfg_base, 0x00B80374 ) = 0x00008000;
        *mkptr( dfe_cfg_base, 0x00B80204 ) = 0x00000F0F;
        *mkptr( dfe_cfg_base, 0x00B80284 ) = 0x00000F0F;
        *mkptr( dfe_cfg_base, 0x00B80214 ) = 0x00008686;
        *mkptr( dfe_cfg_base, 0x00B8021C ) = 0x00000F0F;
        *mkptr( dfe_cfg_base, 0x00B80224 ) = 0x00003232;
        *mkptr( dfe_cfg_base, 0x00B8022C ) = 0x00003232;
        *mkptr( dfe_cfg_base, 0x00B80234 ) = 0x00003131;
        *mkptr( dfe_cfg_base, 0x00B80294 ) = 0x00008686;
        *mkptr( dfe_cfg_base, 0x00B8029C ) = 0x00000F0F;
        *mkptr( dfe_cfg_base, 0x00B802A4 ) = 0x00003232;
        *mkptr( dfe_cfg_base, 0x00B802AC ) = 0x00003232;
        *mkptr( dfe_cfg_base, 0x00B802B4 ) = 0x00003131;
        *mkptr( dfe_cfg_base, 0x00B80304 ) = 0x00000008;
        *mkptr( dfe_cfg_base, 0x00B8030C ) = 0x00000008;
        *mkptr( dfe_cfg_base, 0x00B80314 ) = 0x0000FF5C;
        *mkptr( dfe_cfg_base, 0x00B8031C ) = 0x0000FF5C;
        *mkptr( dfe_cfg_base, 0x00B80324 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B8032C ) = 0x0000013C;
        *mkptr( dfe_cfg_base, 0x00B8033C ) = 0x00000BB8;
        *mkptr( dfe_cfg_base, 0x00B80344 ) = 0x00000363;
        *mkptr( dfe_cfg_base, 0x00B8034C ) = 0x000000FF;
        *mkptr( dfe_cfg_base, 0x00B8037C ) = 0x00008000;
        *mkptr( dfe_cfg_base, 0x00B98404 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9840C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98414 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9841C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98424 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9842C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98434 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9843C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98444 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9844C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98454 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9845C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98464 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9846C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98474 ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B9847C ) = 0x00007FFF;
        *mkptr( dfe_cfg_base, 0x00B98480 ) = 0x0000030C;
        *mkptr( dfe_cfg_base, 0x00B98484 ) = 0x00007FE8;
        *mkptr( dfe_cfg_base, 0x00B98488 ) = 0x00000320;
        *mkptr( dfe_cfg_base, 0x00B9848C ) = 0x00007C17;
        *mkptr( dfe_cfg_base, 0x00B98490 ) = 0x00000334;
        *mkptr( dfe_cfg_base, 0x00B98494 ) = 0x00007899;
        *mkptr( dfe_cfg_base, 0x00B98498 ) = 0x00000340;
        *mkptr( dfe_cfg_base, 0x00B9849C ) = 0x00007562;
        *mkptr( dfe_cfg_base, 0x00B984A0 ) = 0x00000350;
        *mkptr( dfe_cfg_base, 0x00B984A4 ) = 0x0000726A;
        *mkptr( dfe_cfg_base, 0x00B984A8 ) = 0x0000035C;
        *mkptr( dfe_cfg_base, 0x00B984AC ) = 0x00006FA8;
        *mkptr( dfe_cfg_base, 0x00B984B0 ) = 0x00000368;
        *mkptr( dfe_cfg_base, 0x00B984B4 ) = 0x00006D17;
        *mkptr( dfe_cfg_base, 0x00B984B8 ) = 0x00000370;
        *mkptr( dfe_cfg_base, 0x00B984BC ) = 0x00006AB2;
        *mkptr( dfe_cfg_base, 0x00B984C0 ) = 0x00000378;
        *mkptr( dfe_cfg_base, 0x00B984C4 ) = 0x00006873;
        *mkptr( dfe_cfg_base, 0x00B984C8 ) = 0x00000380;
        *mkptr( dfe_cfg_base, 0x00B984CC ) = 0x00006657;
        *mkptr( dfe_cfg_base, 0x00B984D0 ) = 0x00000388;
        *mkptr( dfe_cfg_base, 0x00B984D4 ) = 0x0000645A;
        *mkptr( dfe_cfg_base, 0x00B984D8 ) = 0x00000390;
        *mkptr( dfe_cfg_base, 0x00B984DC ) = 0x0000627A;
        *mkptr( dfe_cfg_base, 0x00B984E0 ) = 0x00000394;
        *mkptr( dfe_cfg_base, 0x00B984E4 ) = 0x000060B4;
        *mkptr( dfe_cfg_base, 0x00B984E8 ) = 0x00000398;
        *mkptr( dfe_cfg_base, 0x00B984EC ) = 0x00005F06;
        *mkptr( dfe_cfg_base, 0x00B984F0 ) = 0x000003A0;
        *mkptr( dfe_cfg_base, 0x00B984F4 ) = 0x00005D6D;
        *mkptr( dfe_cfg_base, 0x00B984F8 ) = 0x000003A4;
        *mkptr( dfe_cfg_base, 0x00B984FC ) = 0x00005BE8;
        *mkptr( dfe_cfg_base, 0x00B98500 ) = 0x000003A8;
        *mkptr( dfe_cfg_base, 0x00B98504 ) = 0x00005A76;
        *mkptr( dfe_cfg_base, 0x00B98508 ) = 0x000003AC;
        *mkptr( dfe_cfg_base, 0x00B9850C ) = 0x00005914;
        *mkptr( dfe_cfg_base, 0x00B98510 ) = 0x000003B0;
        *mkptr( dfe_cfg_base, 0x00B98514 ) = 0x000057C2;
        *mkptr( dfe_cfg_base, 0x00B98518 ) = 0x000003B4;
        *mkptr( dfe_cfg_base, 0x00B9851C ) = 0x0000567F;
        *mkptr( dfe_cfg_base, 0x00B98520 ) = 0x000003B4;
        *mkptr( dfe_cfg_base, 0x00B98524 ) = 0x00005549;
        *mkptr( dfe_cfg_base, 0x00B98528 ) = 0x000003B8;
        *mkptr( dfe_cfg_base, 0x00B9852C ) = 0x00005420;
        *mkptr( dfe_cfg_base, 0x00B98530 ) = 0x000003BC;
        *mkptr( dfe_cfg_base, 0x00B98534 ) = 0x00005303;
        *mkptr( dfe_cfg_base, 0x00B98538 ) = 0x000003C0;
        *mkptr( dfe_cfg_base, 0x00B9853C ) = 0x000051F1;
        *mkptr( dfe_cfg_base, 0x00B98540 ) = 0x000003C0;
        *mkptr( dfe_cfg_base, 0x00B98544 ) = 0x000050E9;
        *mkptr( dfe_cfg_base, 0x00B98548 ) = 0x000003C4;
        *mkptr( dfe_cfg_base, 0x00B9854C ) = 0x00004FEB;
        *mkptr( dfe_cfg_base, 0x00B98550 ) = 0x000003C4;
        *mkptr( dfe_cfg_base, 0x00B98554 ) = 0x00004EF6;
        *mkptr( dfe_cfg_base, 0x00B98558 ) = 0x000003C8;
        *mkptr( dfe_cfg_base, 0x00B9855C ) = 0x00004E0A;
        *mkptr( dfe_cfg_base, 0x00B98560 ) = 0x000003C8;
        *mkptr( dfe_cfg_base, 0x00B98564 ) = 0x00004D25;
        *mkptr( dfe_cfg_base, 0x00B98568 ) = 0x000003CC;
        *mkptr( dfe_cfg_base, 0x00B9856C ) = 0x00004C49;
        *mkptr( dfe_cfg_base, 0x00B98570 ) = 0x000003CC;
        *mkptr( dfe_cfg_base, 0x00B98574 ) = 0x00004B73;
        *mkptr( dfe_cfg_base, 0x00B98578 ) = 0x000003CC;
        *mkptr( dfe_cfg_base, 0x00B9857C ) = 0x00004AA5;
        *mkptr( dfe_cfg_base, 0x00B98580 ) = 0x000003D0;
        *mkptr( dfe_cfg_base, 0x00B98584 ) = 0x000049DD;
        *mkptr( dfe_cfg_base, 0x00B98588 ) = 0x000003D0;
        *mkptr( dfe_cfg_base, 0x00B9858C ) = 0x0000491B;
        *mkptr( dfe_cfg_base, 0x00B98590 ) = 0x000003D4;
        *mkptr( dfe_cfg_base, 0x00B98594 ) = 0x0000485F;
        *mkptr( dfe_cfg_base, 0x00B98598 ) = 0x000003D4;
        *mkptr( dfe_cfg_base, 0x00B9859C ) = 0x000047A8;
        *mkptr( dfe_cfg_base, 0x00B985A0 ) = 0x000003D4;
        *mkptr( dfe_cfg_base, 0x00B985A4 ) = 0x000046F7;
        *mkptr( dfe_cfg_base, 0x00B985A8 ) = 0x000003D8;
        *mkptr( dfe_cfg_base, 0x00B985AC ) = 0x0000464B;
        *mkptr( dfe_cfg_base, 0x00B985B0 ) = 0x000003D8;
        *mkptr( dfe_cfg_base, 0x00B985B4 ) = 0x000045A3;
        *mkptr( dfe_cfg_base, 0x00B985B8 ) = 0x000003D8;
        *mkptr( dfe_cfg_base, 0x00B985BC ) = 0x00004501;
        *mkptr( dfe_cfg_base, 0x00B985C0 ) = 0x000003D8;
        *mkptr( dfe_cfg_base, 0x00B985C4 ) = 0x00004462;
        *mkptr( dfe_cfg_base, 0x00B985C8 ) = 0x000003DC;
        *mkptr( dfe_cfg_base, 0x00B985CC ) = 0x000043C8;
        *mkptr( dfe_cfg_base, 0x00B985D0 ) = 0x000003DC;
        *mkptr( dfe_cfg_base, 0x00B985D4 ) = 0x00004332;
        *mkptr( dfe_cfg_base, 0x00B985D8 ) = 0x000003DC;
        *mkptr( dfe_cfg_base, 0x00B985DC ) = 0x0000429F;
        *mkptr( dfe_cfg_base, 0x00B985E0 ) = 0x000003DC;
        *mkptr( dfe_cfg_base, 0x00B985E4 ) = 0x00004211;
        *mkptr( dfe_cfg_base, 0x00B985E8 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B985EC ) = 0x00004185;
        *mkptr( dfe_cfg_base, 0x00B985F0 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B985F4 ) = 0x000040FE;
        *mkptr( dfe_cfg_base, 0x00B985F8 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B985FC ) = 0x00004079;
        *mkptr( dfe_cfg_base, 0x00B98600 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B98604 ) = 0x00003FF8;
        *mkptr( dfe_cfg_base, 0x00B98608 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B9860C ) = 0x00003F79;
        *mkptr( dfe_cfg_base, 0x00B98610 ) = 0x000003E0;
        *mkptr( dfe_cfg_base, 0x00B98614 ) = 0x00003EFE;
        *mkptr( dfe_cfg_base, 0x00B98618 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B9861C ) = 0x00003E85;
        *mkptr( dfe_cfg_base, 0x00B98620 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B98624 ) = 0x00003E0F;
        *mkptr( dfe_cfg_base, 0x00B98628 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B9862C ) = 0x00003D9B;
        *mkptr( dfe_cfg_base, 0x00B98630 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B98634 ) = 0x00003D2A;
        *mkptr( dfe_cfg_base, 0x00B98638 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B9863C ) = 0x00003CBB;
        *mkptr( dfe_cfg_base, 0x00B98640 ) = 0x000003E4;
        *mkptr( dfe_cfg_base, 0x00B98644 ) = 0x00003C4F;
        *mkptr( dfe_cfg_base, 0x00B98648 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B9864C ) = 0x00003BE5;
        *mkptr( dfe_cfg_base, 0x00B98650 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B98654 ) = 0x00003B7D;
        *mkptr( dfe_cfg_base, 0x00B98658 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B9865C ) = 0x00003B17;
        *mkptr( dfe_cfg_base, 0x00B98660 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B98664 ) = 0x00003AB3;
        *mkptr( dfe_cfg_base, 0x00B98668 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B9866C ) = 0x00003A51;
        *mkptr( dfe_cfg_base, 0x00B98670 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B98674 ) = 0x000039F1;
        *mkptr( dfe_cfg_base, 0x00B98678 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B9867C ) = 0x00003993;
        *mkptr( dfe_cfg_base, 0x00B98680 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B98684 ) = 0x00003937;
        *mkptr( dfe_cfg_base, 0x00B98688 ) = 0x000003E8;
        *mkptr( dfe_cfg_base, 0x00B9868C ) = 0x000038DC;
        *mkptr( dfe_cfg_base, 0x00B98690 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B98694 ) = 0x00003883;
        *mkptr( dfe_cfg_base, 0x00B98698 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B9869C ) = 0x0000382C;
        *mkptr( dfe_cfg_base, 0x00B986A0 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986A4 ) = 0x000037D6;
        *mkptr( dfe_cfg_base, 0x00B986A8 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986AC ) = 0x00003782;
        *mkptr( dfe_cfg_base, 0x00B986B0 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986B4 ) = 0x0000372F;
        *mkptr( dfe_cfg_base, 0x00B986B8 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986BC ) = 0x000036DD;
        *mkptr( dfe_cfg_base, 0x00B986C0 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986C4 ) = 0x0000368D;
        *mkptr( dfe_cfg_base, 0x00B986C8 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986CC ) = 0x0000363F;
        *mkptr( dfe_cfg_base, 0x00B986D0 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986D4 ) = 0x000035F1;
        *mkptr( dfe_cfg_base, 0x00B986D8 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986DC ) = 0x000035A5;
        *mkptr( dfe_cfg_base, 0x00B986E0 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986E4 ) = 0x0000355A;
        *mkptr( dfe_cfg_base, 0x00B986E8 ) = 0x000003EC;
        *mkptr( dfe_cfg_base, 0x00B986EC ) = 0x00003511;
        *mkptr( dfe_cfg_base, 0x00B986F0 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B986F4 ) = 0x000034C8;
        *mkptr( dfe_cfg_base, 0x00B986F8 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B986FC ) = 0x00003481;
        *mkptr( dfe_cfg_base, 0x00B98700 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98704 ) = 0x0000343B;
        *mkptr( dfe_cfg_base, 0x00B98708 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9870C ) = 0x000033F6;
        *mkptr( dfe_cfg_base, 0x00B98710 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98714 ) = 0x000033B2;
        *mkptr( dfe_cfg_base, 0x00B98718 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9871C ) = 0x0000336F;
        *mkptr( dfe_cfg_base, 0x00B98720 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98724 ) = 0x0000332D;
        *mkptr( dfe_cfg_base, 0x00B98728 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9872C ) = 0x000032EC;
        *mkptr( dfe_cfg_base, 0x00B98730 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98734 ) = 0x000032AC;
        *mkptr( dfe_cfg_base, 0x00B98738 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9873C ) = 0x0000326C;
        *mkptr( dfe_cfg_base, 0x00B98740 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98744 ) = 0x0000322E;
        *mkptr( dfe_cfg_base, 0x00B98748 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9874C ) = 0x000031F1;
        *mkptr( dfe_cfg_base, 0x00B98750 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98754 ) = 0x000031B4;
        *mkptr( dfe_cfg_base, 0x00B98758 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9875C ) = 0x00003179;
        *mkptr( dfe_cfg_base, 0x00B98760 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98764 ) = 0x0000313E;
        *mkptr( dfe_cfg_base, 0x00B98768 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B9876C ) = 0x00003104;
        *mkptr( dfe_cfg_base, 0x00B98770 ) = 0x000003F0;
        *mkptr( dfe_cfg_base, 0x00B98774 ) = 0x000030CB;
        *mkptr( dfe_cfg_base, 0x00B98778 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B9877C ) = 0x00003093;
        *mkptr( dfe_cfg_base, 0x00B98780 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B98784 ) = 0x0000305B;
        *mkptr( dfe_cfg_base, 0x00B98788 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B9878C ) = 0x00003024;
        *mkptr( dfe_cfg_base, 0x00B98790 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B98794 ) = 0x00002FEE;
        *mkptr( dfe_cfg_base, 0x00B98798 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B9879C ) = 0x00002FB8;
        *mkptr( dfe_cfg_base, 0x00B987A0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987A4 ) = 0x00002F84;
        *mkptr( dfe_cfg_base, 0x00B987A8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987AC ) = 0x00002F50;
        *mkptr( dfe_cfg_base, 0x00B987B0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987B4 ) = 0x00002F1C;
        *mkptr( dfe_cfg_base, 0x00B987B8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987BC ) = 0x00002EE9;
        *mkptr( dfe_cfg_base, 0x00B987C0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987C4 ) = 0x00002EB7;
        *mkptr( dfe_cfg_base, 0x00B987C8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987CC ) = 0x00002E86;
        *mkptr( dfe_cfg_base, 0x00B987D0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987D4 ) = 0x00002E55;
        *mkptr( dfe_cfg_base, 0x00B987D8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987DC ) = 0x00002E24;
        *mkptr( dfe_cfg_base, 0x00B987E0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987E4 ) = 0x00002DF5;
        *mkptr( dfe_cfg_base, 0x00B987E8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987EC ) = 0x00002DC6;
        *mkptr( dfe_cfg_base, 0x00B987F0 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987F4 ) = 0x00002D97;
        *mkptr( dfe_cfg_base, 0x00B987F8 ) = 0x000003F4;
        *mkptr( dfe_cfg_base, 0x00B987FC ) = 0x00002D69;
        *mkptr( dfe_cfg_base, 0x00B90140 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B90144 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90148 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B9014C ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90150 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B90154 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90158 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B9015C ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90160 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B90164 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90168 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B9016C ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90170 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B90174 ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B90178 ) = 0x00010005;
        *mkptr( dfe_cfg_base, 0x00B9017C ) = 0x00000001;
        *mkptr( dfe_cfg_base, 0x00B9508C ) = 0x00000100;
        *mkptr( dfe_cfg_base, 0x00B95094 ) = 0x00000200;
        *mkptr( dfe_cfg_base, 0x00B9509C ) = 0x00000300;
        *mkptr( dfe_cfg_base, 0x00B950A4 ) = 0x00000400;
        *mkptr( dfe_cfg_base, 0x00B950AC ) = 0x00000500;
        *mkptr( dfe_cfg_base, 0x00B950B4 ) = 0x00000600;
        *mkptr( dfe_cfg_base, 0x00B950BC ) = 0x00000700;
        *mkptr( dfe_cfg_base, 0x00B950C4 ) = 0x00000800;
        *mkptr( dfe_cfg_base, 0x00B950CC ) = 0x00000900;
        *mkptr( dfe_cfg_base, 0x00B950D4 ) = 0x00000A00;
        *mkptr( dfe_cfg_base, 0x00B950DC ) = 0x00000B00;
        *mkptr( dfe_cfg_base, 0x00B950E4 ) = 0x00000C00;
        *mkptr( dfe_cfg_base, 0x00B950EC ) = 0x00000D00;
        *mkptr( dfe_cfg_base, 0x00B950F4 ) = 0x00000E00;
        *mkptr( dfe_cfg_base, 0x00B950FC ) = 0x00000F00;
        *mkptr( dfe_cfg_base, 0x00B94800 ) = 0x0000392D;
        *mkptr( dfe_cfg_base, 0x00B94804 ) = 0x00004130;
        *mkptr( dfe_cfg_base, 0x00B94808 ) = 0x0000287A;
        *mkptr( dfe_cfg_base, 0x00B9480C ) = 0x00003821;
        *mkptr( dfe_cfg_base, 0x00B94810 ) = 0x00001CA8;
        *mkptr( dfe_cfg_base, 0x00B94814 ) = 0x00003821;
        *mkptr( dfe_cfg_base, 0x00B94818 ) = 0x00001CA8;
        *mkptr( dfe_cfg_base, 0x00B9481C ) = 0x000038C7;
        *mkptr( dfe_cfg_base, 0x00B94820 ) = 0x0000080C;
        *mkptr( dfe_cfg_base, 0x00B94824 ) = 0x00000404;
        *mkptr( dfe_cfg_base, 0x00B94840 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94844 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94848 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9484C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94850 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94854 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94858 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9485C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94860 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94864 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94880 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94884 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94888 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9488C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94890 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94894 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94898 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9489C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948A0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B948A4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B948C0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948C4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948C8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948CC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948D0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948D4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948D8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948DC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B948E0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B948E4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94900 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94904 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94908 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9490C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94910 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94914 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94918 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9491C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94920 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94924 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94940 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94944 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94948 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9494C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94950 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94954 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94958 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9495C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94960 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94964 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94980 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94984 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94988 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9498C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94990 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94994 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94998 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B9499C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949A0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B949A4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B949C0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949C4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949C8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949CC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949D0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949D4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949D8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949DC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B949E0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B949E4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94A00 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A04 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A08 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A0C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A10 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A14 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A18 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A1C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A20 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94A24 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94A40 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A44 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A48 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A4C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A50 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A54 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A58 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A5C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A60 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94A64 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94A80 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A84 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A88 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A8C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A90 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A94 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A98 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94A9C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AA0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94AA4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94AC0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AC4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AC8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94ACC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AD0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AD4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AD8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94ADC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94AE0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94AE4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94B00 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B04 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B08 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B0C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B10 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B14 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B18 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B1C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B20 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94B24 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94B40 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B44 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B48 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B4C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B50 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B54 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B58 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B5C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B60 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94B64 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94B80 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B84 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B88 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B8C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B90 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B94 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B98 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94B9C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BA0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94BA4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94BC0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BC4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BC8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BCC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BD0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BD4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BD8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BDC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94BE0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94BE4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94C00 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C04 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C08 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C0C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C10 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C14 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C18 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C1C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C20 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94C24 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94C40 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C44 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C48 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C4C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C50 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C54 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C58 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C5C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C60 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94C64 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94C80 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C84 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C88 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C8C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C90 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C94 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C98 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94C9C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CA0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94CA4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94CC0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CC4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CC8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CCC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CD0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CD4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CD8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CDC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94CE0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94CE4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94D00 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D04 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D08 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D0C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D10 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D14 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D18 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D1C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D20 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94D24 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94D40 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D44 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D48 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D4C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D50 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D54 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D58 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D5C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D60 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94D64 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94D80 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D84 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D88 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D8C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D90 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D94 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D98 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94D9C ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DA0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94DA4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94DC0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DC4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DC8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DCC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DD0 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DD4 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DD8 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DDC ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94DE0 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94DE4 ) = 0x00000101;
        *mkptr( dfe_cfg_base, 0x00B94E00 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94E04 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94E08 ) = 0x0000FFFF;
        *mkptr( dfe_cfg_base, 0x00B94E0C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E10 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E14 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E18 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E1C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E20 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94E24 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94E40 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E44 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E48 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E4C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E50 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E54 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E58 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E5C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E60 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94E64 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94E80 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E84 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E88 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E8C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E90 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E94 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E98 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94E9C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94EA0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94EA4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94EC0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94EC4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94EC8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94ECC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94ED0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94ED4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94ED8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94EDC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94EE0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94EE4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94F00 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F04 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F08 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F0C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F10 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F14 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F18 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F1C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F20 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94F24 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94F40 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F44 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F48 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F4C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F50 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F54 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F58 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F5C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F60 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94F64 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94F80 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F84 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F88 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F8C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F90 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F94 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F98 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94F9C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FA0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94FA4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94FC0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FC4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FC8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FCC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FD0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FD4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FD8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FDC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94FE0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94FE4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B840C8 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x00B840CC ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B844CC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B840D0 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B844D0 ) = 0x00000FFD;
    *mkptr( dfe_cfg_base, 0x00B840D4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B844D4 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B840D8 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B844D8 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B840DC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B844DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B840E0 ) = 0x00000041;
    *mkptr( dfe_cfg_base, 0x00B844E0 ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x00B840E4 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B844E4 ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B840E8 ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B844E8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B840EC ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B844EC ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B840F0 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B844F0 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B840F4 ) = 0x00000FF8;
    *mkptr( dfe_cfg_base, 0x00B844F4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B840F8 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B844F8 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B840FC ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B844FC ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B84100 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B84500 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B84104 ) = 0x00000FF8;
    *mkptr( dfe_cfg_base, 0x00B84504 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84108 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B84508 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B8410C ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B8450C ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B84110 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B84510 ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B84114 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B84514 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B84118 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B84518 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8411C ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B8451C ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B84120 ) = 0x0000001E;
    *mkptr( dfe_cfg_base, 0x00B84520 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B84124 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B84524 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B84128 ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B84528 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B8412C ) = 0x00000FB2;
    *mkptr( dfe_cfg_base, 0x00B8452C ) = 0x00000F8B;
    *mkptr( dfe_cfg_base, 0x00B84130 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B84530 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B84134 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B84534 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84138 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B84538 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B8413C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8453C ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B84140 ) = 0x00000F56;
    *mkptr( dfe_cfg_base, 0x00B84540 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B84144 ) = 0x00000F3B;
    *mkptr( dfe_cfg_base, 0x00B84544 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B84148 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B84548 ) = 0x00000F92;
    *mkptr( dfe_cfg_base, 0x00B8414C ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B8454C ) = 0x00000025;
    *mkptr( dfe_cfg_base, 0x00B84150 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B84550 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84154 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B84554 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B84158 ) = 0x00000F54;
    *mkptr( dfe_cfg_base, 0x00B84558 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B8415C ) = 0x00000EB1;
    *mkptr( dfe_cfg_base, 0x00B8455C ) = 0x00000EDF;
    *mkptr( dfe_cfg_base, 0x00B84160 ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B84560 ) = 0x00000F08;
    *mkptr( dfe_cfg_base, 0x00B84164 ) = 0x0000006F;
    *mkptr( dfe_cfg_base, 0x00B84564 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x00B84168 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B84568 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00B8416C ) = 0x00000FE6;
    *mkptr( dfe_cfg_base, 0x00B8456C ) = 0x00000FC1;
    *mkptr( dfe_cfg_base, 0x00B84170 ) = 0x00000FB7;
    *mkptr( dfe_cfg_base, 0x00B84570 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B84174 ) = 0x00000E40;
    *mkptr( dfe_cfg_base, 0x00B84574 ) = 0x00000EFA;
    *mkptr( dfe_cfg_base, 0x00B84178 ) = 0x00000EF5;
    *mkptr( dfe_cfg_base, 0x00B84578 ) = 0x00000E24;
    *mkptr( dfe_cfg_base, 0x00B8417C ) = 0x0000013E;
    *mkptr( dfe_cfg_base, 0x00B8457C ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B84180 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B84580 ) = 0x0000010E;
    *mkptr( dfe_cfg_base, 0x00B84184 ) = 0x00000E51;
    *mkptr( dfe_cfg_base, 0x00B84584 ) = 0x00000E6C;
    *mkptr( dfe_cfg_base, 0x00B84188 ) = 0x00000386;
    *mkptr( dfe_cfg_base, 0x00B84588 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B8418C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B8458C ) = 0x000006B1;
    *mkptr( dfe_cfg_base, 0x00B880C8 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x00B880CC ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B884CC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B880D0 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B884D0 ) = 0x00000FFD;
    *mkptr( dfe_cfg_base, 0x00B880D4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B884D4 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B880D8 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B884D8 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B880DC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B884DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B880E0 ) = 0x00000041;
    *mkptr( dfe_cfg_base, 0x00B884E0 ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x00B880E4 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B884E4 ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B880E8 ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B884E8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B880EC ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B884EC ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B880F0 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B884F0 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B880F4 ) = 0x00000FF8;
    *mkptr( dfe_cfg_base, 0x00B884F4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B880F8 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B884F8 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B880FC ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B884FC ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B88100 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B88500 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B88104 ) = 0x00000FF8;
    *mkptr( dfe_cfg_base, 0x00B88504 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88108 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B88508 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B8810C ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B8850C ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B88110 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B88510 ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B88114 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B88514 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B88118 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B88518 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8811C ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B8851C ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B88120 ) = 0x0000001E;
    *mkptr( dfe_cfg_base, 0x00B88520 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00B88124 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B88524 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B88128 ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B88528 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B8812C ) = 0x00000FB2;
    *mkptr( dfe_cfg_base, 0x00B8852C ) = 0x00000F8B;
    *mkptr( dfe_cfg_base, 0x00B88130 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B88530 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B88134 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B88534 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88138 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B88538 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B8813C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8853C ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B88140 ) = 0x00000F56;
    *mkptr( dfe_cfg_base, 0x00B88540 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B88144 ) = 0x00000F3B;
    *mkptr( dfe_cfg_base, 0x00B88544 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B88148 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B88548 ) = 0x00000F92;
    *mkptr( dfe_cfg_base, 0x00B8814C ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B8854C ) = 0x00000025;
    *mkptr( dfe_cfg_base, 0x00B88150 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B88550 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88154 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B88554 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B88158 ) = 0x00000F54;
    *mkptr( dfe_cfg_base, 0x00B88558 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B8815C ) = 0x00000EB1;
    *mkptr( dfe_cfg_base, 0x00B8855C ) = 0x00000EDF;
    *mkptr( dfe_cfg_base, 0x00B88160 ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B88560 ) = 0x00000F08;
    *mkptr( dfe_cfg_base, 0x00B88164 ) = 0x0000006F;
    *mkptr( dfe_cfg_base, 0x00B88564 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x00B88168 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B88568 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00B8816C ) = 0x00000FE6;
    *mkptr( dfe_cfg_base, 0x00B8856C ) = 0x00000FC1;
    *mkptr( dfe_cfg_base, 0x00B88170 ) = 0x00000FB7;
    *mkptr( dfe_cfg_base, 0x00B88570 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B88174 ) = 0x00000E40;
    *mkptr( dfe_cfg_base, 0x00B88574 ) = 0x00000EFA;
    *mkptr( dfe_cfg_base, 0x00B88178 ) = 0x00000EF5;
    *mkptr( dfe_cfg_base, 0x00B88578 ) = 0x00000E24;
    *mkptr( dfe_cfg_base, 0x00B8817C ) = 0x0000013E;
    *mkptr( dfe_cfg_base, 0x00B8857C ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B88180 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B88580 ) = 0x0000010E;
    *mkptr( dfe_cfg_base, 0x00B88184 ) = 0x00000E51;
    *mkptr( dfe_cfg_base, 0x00B88584 ) = 0x00000E6C;
    *mkptr( dfe_cfg_base, 0x00B88188 ) = 0x00000386;
    *mkptr( dfe_cfg_base, 0x00B88588 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B8818C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B8858C ) = 0x000006B1;
    *mkptr( dfe_cfg_base, 0x01000008 ) = 0x41000004;
    *mkptr( dfe_cfg_base, 0x0100000C ) = 0x41000004;
    *mkptr( dfe_cfg_base, 0x01000010 ) = 0x41000004;
    *mkptr( dfe_cfg_base, 0x01000014 ) = 0x41000004;
    *mkptr( dfe_cfg_base, 0x01000038 ) = 0xC0300C03;
    *mkptr( dfe_cfg_base, 0x0100003C ) = 0x002642DB;
    *mkptr( dfe_cfg_base, 0x01000040 ) = 0x00FFBBBB;
    *mkptr( dfe_cfg_base, 0x01000200 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01000204 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01000208 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0100020C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01000210 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01000214 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01000218 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0100021C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01000220 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01000224 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01000228 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0100022C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01000230 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01000234 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01000238 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0100023C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01000400 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01000404 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01000408 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0100040C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01000410 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01000414 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01000418 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0100041C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01000420 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01000424 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01000428 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0100042C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01000430 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01000434 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01000438 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0100043C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01000600 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01000604 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01000608 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0100060C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01000610 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01000614 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01000618 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0100061C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01000620 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01000624 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01000628 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0100062C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01000630 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01000634 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01000638 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0100063C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01000800 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01000804 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01000808 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0100080C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01000810 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01000814 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01000818 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0100081C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01000820 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01000824 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01000828 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0100082C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01000830 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01000834 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01000838 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0100083C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01001100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01001104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01001108 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x0100110C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001110 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001114 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001118 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01300200 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01300204 ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01300208 ) = 0x00000183;
	*mkptr( dfe_cfg_base, 0x0130020C ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01300210 ) = 0x08070100;
	*mkptr( dfe_cfg_base, 0x01300214 ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01300228 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01300400 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300408 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130040C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300410 ) = 0x1500018B;
	*mkptr( dfe_cfg_base, 0x01300414 ) = 0x1500210B;
	*mkptr( dfe_cfg_base, 0x01300418 ) = 0x150002B8;
	*mkptr( dfe_cfg_base, 0x0130041C ) = 0x150012B0;
	*mkptr( dfe_cfg_base, 0x01300420 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300428 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130042C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300430 ) = 0x1500018B;
	*mkptr( dfe_cfg_base, 0x01300434 ) = 0x1500210B;
	*mkptr( dfe_cfg_base, 0x01300438 ) = 0x150002B8;
	*mkptr( dfe_cfg_base, 0x0130043C ) = 0x150012B0;
	*mkptr( dfe_cfg_base, 0x01300440 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300448 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130044C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300450 ) = 0x1500018B;
	*mkptr( dfe_cfg_base, 0x01300454 ) = 0x1500210B;
	*mkptr( dfe_cfg_base, 0x01300458 ) = 0x150002B8;
	*mkptr( dfe_cfg_base, 0x0130045C ) = 0x150012B0;
	*mkptr( dfe_cfg_base, 0x01300460 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300468 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130046C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300470 ) = 0x1500018B;
	*mkptr( dfe_cfg_base, 0x01300474 ) = 0x1500210B;
	*mkptr( dfe_cfg_base, 0x01300478 ) = 0x150002B8;
	*mkptr( dfe_cfg_base, 0x0130047C ) = 0x150012B0;
	*mkptr( dfe_cfg_base, 0x01341800 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341808 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341810 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341818 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341820 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341828 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341830 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341838 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341840 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341848 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341850 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341858 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341860 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341868 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341870 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341878 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341880 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341888 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341890 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341898 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013418F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341900 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341908 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341910 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341918 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341920 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341928 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341930 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341938 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341940 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341948 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341950 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341958 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341960 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341968 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341970 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341978 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341980 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341988 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341990 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341998 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013419F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341A98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341AF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341B98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341BF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341C98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341CF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341D98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341DF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341E98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341ED0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341ED8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341EF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341F98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01341FF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A800 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A808 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A810 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A818 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A820 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A828 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A830 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A838 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A840 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A848 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A850 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A858 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A860 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A868 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A870 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A878 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A880 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A888 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A890 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A898 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A8F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A900 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A908 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A910 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A918 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A920 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A928 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A930 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A938 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A940 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A948 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A950 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A958 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A960 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A968 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A970 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A978 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A980 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A988 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A990 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A998 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134A9F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AA98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AAF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AB98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ABF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AC98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ACF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AD98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134ADF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AE98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AED0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AED8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AEF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AF98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0134AFF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353800 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353808 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353810 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353818 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353820 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353828 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353830 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353838 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353840 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353848 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353850 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353858 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353860 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353868 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353870 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353878 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353880 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353888 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353890 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353898 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013538F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353900 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353908 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353910 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353918 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353920 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353928 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353930 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353938 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353940 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353948 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353950 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353958 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353960 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353968 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353970 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353978 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353980 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353988 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353990 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353998 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013539F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353A98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353AF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353B98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353BF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353C98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353CF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353D98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353DF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353E98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353ED0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353ED8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353EF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353F98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01353FF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C800 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C808 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C810 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C818 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C820 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C828 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C830 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C838 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C840 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C848 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C850 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C858 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C860 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C868 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C870 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C878 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C880 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C888 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C890 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C898 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C8F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C900 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C908 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C910 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C918 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C920 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C928 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C930 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C938 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C940 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C948 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C950 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C958 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C960 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C968 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C970 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C978 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C980 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C988 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C990 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C998 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135C9F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CA98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CAF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CB98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CBF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CC98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CCF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CD98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CDF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CE98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CED0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CED8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CEF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF00 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF08 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF10 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF18 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF20 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF28 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF30 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF38 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF40 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF48 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF50 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF58 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF60 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF68 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF70 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF78 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF80 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF88 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF90 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CF98 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFA0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFA8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFB0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFB8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFC0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFC8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFD0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFD8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFE0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFE8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFF0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x0135CFF8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01600014 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x01600018 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x0160001C ) = 0x77111110;
	*mkptr( dfe_cfg_base, 0x01600020 ) = 0x86860201;
	*mkptr( dfe_cfg_base, 0x0160002C ) = 0x00002FFA;
	*mkptr( dfe_cfg_base, 0x01600054 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x01600058 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x0160005C ) = 0x77111110;
	*mkptr( dfe_cfg_base, 0x01600060 ) = 0x86860403;
	*mkptr( dfe_cfg_base, 0x0160006C ) = 0x00002FFA;
	*mkptr( dfe_cfg_base, 0x016000B8 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x016000F8 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600138 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600178 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x016000BC ) = 0x0001EFED;
	*mkptr( dfe_cfg_base, 0x0160013C ) = 0x0001EFED;
	*mkptr( dfe_cfg_base, 0x01600200 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600220 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600400 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600420 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600600 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600620 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600800 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600820 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01600B08 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01600B0C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B10 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B14 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B18 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B24 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01600B28 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B30 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600A00 ) = 0x77FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A04 ) = 0x37FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A10 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A14 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A18 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A1C ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A28 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A2C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A30 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A34 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A80 ) = 0x77FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A84 ) = 0x37FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A90 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A94 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A98 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A9C ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600AA8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AAC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AB0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AB4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01640000 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640004 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640008 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164000C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640010 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640014 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640018 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164001C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640020 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640024 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640028 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164002C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640030 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640034 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640038 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164003C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640040 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640044 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640048 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164004C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640050 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640054 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640058 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164005C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640060 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640064 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640068 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164006C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640070 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640074 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640078 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164007C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640080 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640084 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640088 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164008C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640090 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640094 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640098 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164009C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640100 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640104 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640108 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164010C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640110 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640114 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640118 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164011C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640120 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640124 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640128 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164012C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640130 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640134 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640138 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164013C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640140 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640144 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640148 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164014C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640150 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640154 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640158 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164015C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640160 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640164 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640168 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164016C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640170 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640174 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640178 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164017C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640180 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640184 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640188 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164018C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640190 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640194 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640198 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164019C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640200 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640204 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640208 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164020C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640210 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640214 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640218 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164021C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640220 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640224 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640228 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164022C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640230 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640234 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640238 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164023C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640240 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640244 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640248 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164024C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640250 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640254 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640258 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164025C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640260 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640264 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640268 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164026C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640270 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640274 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640278 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164027C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640280 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640284 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640288 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164028C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640290 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640294 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640298 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164029C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640300 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640304 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640308 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164030C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640310 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640314 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640318 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164031C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640320 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640324 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640328 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164032C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640330 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640334 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640338 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164033C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640340 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640344 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640348 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164034C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640350 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640354 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640358 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164035C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640360 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640364 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640368 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164036C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640374 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164037C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640380 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640384 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640388 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164038C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640390 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640394 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640398 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164039C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640400 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640404 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640408 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164040C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640410 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640414 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640418 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164041C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640420 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640424 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640428 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164042C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640430 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640434 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640438 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164043C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640440 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640444 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640448 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164044C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640450 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640454 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640458 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164045C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640460 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640464 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640468 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164046C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640470 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640474 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640478 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164047C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640480 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640484 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640488 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164048C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640490 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640494 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640498 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164049C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640500 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640504 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640508 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164050C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640510 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640514 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640518 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164051C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640520 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640524 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640528 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164052C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640530 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640534 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640538 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164053C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640540 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640544 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640548 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164054C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640550 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640554 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640558 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164055C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640560 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640564 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640568 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164056C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640570 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640574 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640578 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164057C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640580 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640584 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640588 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164058C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640590 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640594 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640598 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164059C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640600 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640604 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640608 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164060C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640610 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640614 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640618 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164061C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640620 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640624 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640628 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164062C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640630 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640634 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640638 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164063C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640640 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640644 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640648 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164064C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640650 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640654 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640658 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164065C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640660 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640664 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640668 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164066C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640670 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640674 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640678 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164067C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640680 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640684 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640688 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164068C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640690 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640694 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640698 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164069C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640700 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640704 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640708 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164070C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640710 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640714 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640718 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164071C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640720 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640724 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640728 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164072C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640730 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640734 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640738 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164073C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640740 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640744 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640748 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164074C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640750 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640754 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640758 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164075C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640760 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640764 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640768 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164076C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640770 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640774 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640778 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164077C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640780 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640784 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640788 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164078C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640790 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640794 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640798 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164079C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640800 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640804 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640808 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164080C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640810 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640814 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640818 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164081C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640820 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640824 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640828 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164082C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640830 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640834 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640838 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164083C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640840 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640844 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640848 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164084C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640850 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640854 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640858 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164085C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640860 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640864 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640868 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164086C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640870 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640874 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640878 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164087C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640880 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640884 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640888 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164088C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640890 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640894 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640898 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164089C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640900 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640904 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640908 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164090C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640910 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640914 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640918 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164091C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640920 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640924 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640928 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164092C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640930 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640934 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640938 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164093C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640940 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640944 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640948 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164094C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640950 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640954 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640958 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164095C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640960 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640964 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640968 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164096C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640970 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640974 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640978 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164097C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640980 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640984 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640988 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164098C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640990 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640994 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640998 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164099C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A00 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A04 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A08 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A0C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A10 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A14 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A18 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A1C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A20 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A24 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A28 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A2C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A30 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A34 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A38 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A3C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A40 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A44 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A48 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A4C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A50 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A54 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A58 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A5C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A60 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A64 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A68 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A6C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A70 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A74 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A78 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A7C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A80 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A84 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A88 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A8C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A90 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A94 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A98 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A9C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AAC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ABC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ACC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ADC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AEC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AFC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B00 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B04 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B08 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B0C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B10 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B14 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B18 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B1C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B20 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B24 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B28 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B2C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B30 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B34 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B38 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B3C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B40 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B44 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B48 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B4C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B50 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B54 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B58 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B5C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B60 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B64 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B68 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B6C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B70 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B74 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B78 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B7C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B80 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B84 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B88 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B8C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B90 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B94 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B98 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B9C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BAC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BBC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BCC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BDC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BEC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BFC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01D00014 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D00020 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D00048 ) = 0x0000EEEE;
	*mkptr( dfe_cfg_base, 0x01D00404 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00408 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0040C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00410 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00414 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00418 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0041C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00420 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00444 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00448 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0044C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00450 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00454 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00458 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0045C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00460 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00484 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00488 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0048C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00490 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00494 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00498 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0049C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004A0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D004C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004CC ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004D0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004D4 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004D8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004DC ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004E0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00804 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00808 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0080C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00844 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00848 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0084C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00884 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00888 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0088C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D008C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D008C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D008CC ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00C14 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D00C1C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C24 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C54 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D00C5C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C64 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C6C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C74 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D01404 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01408 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D0140C ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D01410 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D01808 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D0180C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D01810 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01814 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D01820 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D01824 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01848 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D0184C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D01850 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01854 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D01860 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D01864 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01C04 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01C44 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01C84 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01CC4 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D02004 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D02408 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01D0240C ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D02410 ) = 0x00000204;
	*mkptr( dfe_cfg_base, 0x01D02414 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02418 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01D0241C ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D02420 ) = 0x00000205;
	*mkptr( dfe_cfg_base, 0x01D02424 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01D02428 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x01D0242C ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x01D02430 ) = 0x00000206;
	*mkptr( dfe_cfg_base, 0x01D02434 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D02438 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x01D0243C ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x01D02440 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x01D02444 ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01D02448 ) = 0x00000104;
	*mkptr( dfe_cfg_base, 0x01D0244C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D02450 ) = 0x00000304;
	*mkptr( dfe_cfg_base, 0x01D02454 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D02458 ) = 0x00000105;
	*mkptr( dfe_cfg_base, 0x01D0245C ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D02460 ) = 0x00000305;
	*mkptr( dfe_cfg_base, 0x01D02464 ) = 0x00000102;
	*mkptr( dfe_cfg_base, 0x01D02468 ) = 0x00000106;
	*mkptr( dfe_cfg_base, 0x01D0246C ) = 0x00000302;
	*mkptr( dfe_cfg_base, 0x01D02470 ) = 0x00000306;
	*mkptr( dfe_cfg_base, 0x01D02474 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x01D02478 ) = 0x00000107;
	*mkptr( dfe_cfg_base, 0x01D0247C ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x01D02480 ) = 0x00000307;
	*mkptr( dfe_cfg_base, 0x01D02484 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01D02488 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01D0248C ) = 0x00000208;
	*mkptr( dfe_cfg_base, 0x01D02490 ) = 0x0000020C;
	*mkptr( dfe_cfg_base, 0x01D02494 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x01D02498 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x01D0249C ) = 0x00000209;
	*mkptr( dfe_cfg_base, 0x01D024A0 ) = 0x0000020D;
	*mkptr( dfe_cfg_base, 0x01D024A4 ) = 0x0000000A;
	*mkptr( dfe_cfg_base, 0x01D024A8 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x01D024AC ) = 0x0000020A;
	*mkptr( dfe_cfg_base, 0x01D024B0 ) = 0x0000020E;
	*mkptr( dfe_cfg_base, 0x01D024B4 ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x01D024B8 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01D024BC ) = 0x0000020B;
	*mkptr( dfe_cfg_base, 0x01D024C0 ) = 0x0000020F;
	*mkptr( dfe_cfg_base, 0x01D024C4 ) = 0x00000108;
	*mkptr( dfe_cfg_base, 0x01D024C8 ) = 0x0000010C;
	*mkptr( dfe_cfg_base, 0x01D024CC ) = 0x00000308;
	*mkptr( dfe_cfg_base, 0x01D024D0 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x01D024D4 ) = 0x00000109;
	*mkptr( dfe_cfg_base, 0x01D024D8 ) = 0x0000010D;
	*mkptr( dfe_cfg_base, 0x01D024DC ) = 0x00000309;
	*mkptr( dfe_cfg_base, 0x01D024E0 ) = 0x0000030D;
	*mkptr( dfe_cfg_base, 0x01D024E4 ) = 0x0000010A;
	*mkptr( dfe_cfg_base, 0x01D024E8 ) = 0x0000010E;
	*mkptr( dfe_cfg_base, 0x01D024EC ) = 0x0000030A;
	*mkptr( dfe_cfg_base, 0x01D024F0 ) = 0x0000030E;
	*mkptr( dfe_cfg_base, 0x01D024F4 ) = 0x0000010B;
	*mkptr( dfe_cfg_base, 0x01D024F8 ) = 0x0000010F;
	*mkptr( dfe_cfg_base, 0x01D024FC ) = 0x0000030B;
	*mkptr( dfe_cfg_base, 0x01D02500 ) = 0x0000030F;
	*mkptr( dfe_cfg_base, 0x01D02804 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02808 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0280C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02810 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02814 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02818 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0281C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02820 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02824 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D02828 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D0282C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D02830 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D02834 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D02838 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D0283C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D02840 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D40010 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01D40014 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x01D40018 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x01D40044 ) = 0x0000EE00;
	*mkptr( dfe_cfg_base, 0x01D40048 ) = 0x0000EEEE;
	*mkptr( dfe_cfg_base, 0x01D40804 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40808 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4080C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40844 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40848 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4084C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40884 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40888 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4088C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D408C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D408C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D408CC ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40904 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40908 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4090C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40944 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40948 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4094C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40C14 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D40C1C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C24 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C54 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D40C5C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C64 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C6C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C74 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41014 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4101C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41024 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D4102C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41034 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41044 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01D4104C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D41054 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4105C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41064 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D4106C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41074 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41084 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01D4108C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D41094 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4109C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410A4 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410AC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410B4 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41404 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41408 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D4140C ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D41410 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D41808 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4180C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D41810 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D41814 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D41820 ) = 0x00001CE0;
	*mkptr( dfe_cfg_base, 0x01D41824 ) = 0x000003FC;
	*mkptr( dfe_cfg_base, 0x01D41828 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41848 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4184C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D41850 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D41854 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D41860 ) = 0x00001CE0;
	*mkptr( dfe_cfg_base, 0x01D41864 ) = 0x000003FC;
	*mkptr( dfe_cfg_base, 0x01D41868 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41C04 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41C44 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41C84 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41CC4 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D42004 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D42808 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D4280C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01D42810 ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x01D42844 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D42848 ) = 0x00000210;
	*mkptr( dfe_cfg_base, 0x01D4284C ) = 0x00000012;
	*mkptr( dfe_cfg_base, 0x01D42850 ) = 0x00000212;
	*mkptr( dfe_cfg_base, 0x01D42884 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D42888 ) = 0x00000220;
	*mkptr( dfe_cfg_base, 0x01D4288C ) = 0x00000022;
	*mkptr( dfe_cfg_base, 0x01D42890 ) = 0x00000222;
	*mkptr( dfe_cfg_base, 0x01D428C4 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01D428C8 ) = 0x00000230;
	*mkptr( dfe_cfg_base, 0x01D428CC ) = 0x00000032;
	*mkptr( dfe_cfg_base, 0x01D428D0 ) = 0x00000232;
	*mkptr( dfe_cfg_base, 0x01D42904 ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01D42908 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4290C ) = 0x00000102;
	*mkptr( dfe_cfg_base, 0x01D42910 ) = 0x00000302;
	*mkptr( dfe_cfg_base, 0x01D42944 ) = 0x00000110;
	*mkptr( dfe_cfg_base, 0x01D42948 ) = 0x00000310;
	*mkptr( dfe_cfg_base, 0x01D4294C ) = 0x00000112;
	*mkptr( dfe_cfg_base, 0x01D42950 ) = 0x00000312;
	*mkptr( dfe_cfg_base, 0x01D42984 ) = 0x00000120;
	*mkptr( dfe_cfg_base, 0x01D42988 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x01D4298C ) = 0x00000122;
	*mkptr( dfe_cfg_base, 0x01D42990 ) = 0x00000322;
	*mkptr( dfe_cfg_base, 0x01D429C4 ) = 0x00000130;
	*mkptr( dfe_cfg_base, 0x01D429C8 ) = 0x00000330;
	*mkptr( dfe_cfg_base, 0x01D429CC ) = 0x00000132;
	*mkptr( dfe_cfg_base, 0x01D429D0 ) = 0x00000332;
	*mkptr( dfe_cfg_base, 0x01D42A04 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42A08 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42A0C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42A10 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42A44 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42A48 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42A4C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42A50 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42A84 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42A88 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42A8C ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42A90 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42AC4 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42AC8 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42ACC ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42AD0 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42B04 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42B08 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42B0C ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42B10 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42B44 ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42B48 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42B4C ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42B50 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42B84 ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42B88 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42B8C ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42B90 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42BC4 ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42BC8 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01D42BCC ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42BD0 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01D42C04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D42C08 ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x01D42C0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D42C10 ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x01D42C44 ) = 0x00000013;
	*mkptr( dfe_cfg_base, 0x01D42C48 ) = 0x00000213;
	*mkptr( dfe_cfg_base, 0x01D42C4C ) = 0x00000013;
	*mkptr( dfe_cfg_base, 0x01D42C50 ) = 0x00000213;
	*mkptr( dfe_cfg_base, 0x01D42C84 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x01D42C88 ) = 0x00000223;
	*mkptr( dfe_cfg_base, 0x01D42C8C ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x01D42C90 ) = 0x00000223;
	*mkptr( dfe_cfg_base, 0x01D42CC4 ) = 0x00000033;
	*mkptr( dfe_cfg_base, 0x01D42CC8 ) = 0x00000233;
	*mkptr( dfe_cfg_base, 0x01D42CCC ) = 0x00000033;
	*mkptr( dfe_cfg_base, 0x01D42CD0 ) = 0x00000233;
	*mkptr( dfe_cfg_base, 0x01D42D04 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x01D42D08 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x01D42D0C ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x01D42D10 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x01D42D44 ) = 0x00000113;
	*mkptr( dfe_cfg_base, 0x01D42D48 ) = 0x00000313;
	*mkptr( dfe_cfg_base, 0x01D42D4C ) = 0x00000113;
	*mkptr( dfe_cfg_base, 0x01D42D50 ) = 0x00000313;
	*mkptr( dfe_cfg_base, 0x01D42D84 ) = 0x00000123;
	*mkptr( dfe_cfg_base, 0x01D42D88 ) = 0x00000323;
	*mkptr( dfe_cfg_base, 0x01D42D8C ) = 0x00000123;
	*mkptr( dfe_cfg_base, 0x01D42D90 ) = 0x00000323;
	*mkptr( dfe_cfg_base, 0x01D42DC4 ) = 0x00000133;
	*mkptr( dfe_cfg_base, 0x01D42DC8 ) = 0x00000333;
	*mkptr( dfe_cfg_base, 0x01D42DCC ) = 0x00000133;
	*mkptr( dfe_cfg_base, 0x01D42DD0 ) = 0x00000333;
	*mkptr( dfe_cfg_base, 0x01A00E00 ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A00200 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01A00204 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x01A006A0 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x01A0022C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A00230 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A41000 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41080 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A410C0 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41100 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x01A41108 ) = 0x000007D0;
	*mkptr( dfe_cfg_base, 0x01A007EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007F8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007FC ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x01A00604 ) = 0x0000FF00;
	*mkptr( dfe_cfg_base, 0x01A00648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01A41800 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x01A41804 ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x01A41808 ) = 0x00000062;
	*mkptr( dfe_cfg_base, 0x01A4180C ) = 0x00000057;
	*mkptr( dfe_cfg_base, 0x01A41810 ) = 0x0000004F;
	*mkptr( dfe_cfg_base, 0x01A41814 ) = 0x00000047;
	*mkptr( dfe_cfg_base, 0x01A41818 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A4181C ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x01A41820 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x01A41824 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41828 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x01A4182C ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x01A41830 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x01A41834 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41838 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A4183C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x01A41840 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x01A41844 ) = 0x00000013;
	*mkptr( dfe_cfg_base, 0x01A41848 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A4184C ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x01A41850 ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x01A41854 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x01A41874 ) = 0x000000F9;
	*mkptr( dfe_cfg_base, 0x01A41878 ) = 0x000000F7;
	*mkptr( dfe_cfg_base, 0x01A4187C ) = 0x000000F5;
	*mkptr( dfe_cfg_base, 0x01A41880 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41884 ) = 0x000000F2;
	*mkptr( dfe_cfg_base, 0x01A41888 ) = 0x000000F1;
	*mkptr( dfe_cfg_base, 0x01A4188C ) = 0x000000EF;
	*mkptr( dfe_cfg_base, 0x01A41890 ) = 0x000000EE;
	*mkptr( dfe_cfg_base, 0x01A41894 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41898 ) = 0x000000EB;
	*mkptr( dfe_cfg_base, 0x01A4189C ) = 0x000000EA;
	*mkptr( dfe_cfg_base, 0x01A418A0 ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A418A4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x01A418A8 ) = 0x000000E6;
	*mkptr( dfe_cfg_base, 0x01A418AC ) = 0x000000E5;
	*mkptr( dfe_cfg_base, 0x01A418B0 ) = 0x000000E3;
	*mkptr( dfe_cfg_base, 0x01A418B4 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x01A418B8 ) = 0x000000E1;
	*mkptr( dfe_cfg_base, 0x01A418BC ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A418C0 ) = 0x000000DF;
	*mkptr( dfe_cfg_base, 0x01A418C4 ) = 0x000000DE;
	*mkptr( dfe_cfg_base, 0x01A418C8 ) = 0x000000DD;
	*mkptr( dfe_cfg_base, 0x01A418CC ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A418D0 ) = 0x000000DB;
	*mkptr( dfe_cfg_base, 0x01A418D4 ) = 0x000000DA;
	*mkptr( dfe_cfg_base, 0x01A418D8 ) = 0x000000D9;
	*mkptr( dfe_cfg_base, 0x01A418DC ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A418E0 ) = 0x000000D7;
	*mkptr( dfe_cfg_base, 0x01A418E4 ) = 0x000000D6;
	*mkptr( dfe_cfg_base, 0x01A418E8 ) = 0x000000D5;
	*mkptr( dfe_cfg_base, 0x01A418EC ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A418F0 ) = 0x000000D3;
	*mkptr( dfe_cfg_base, 0x01A418F4 ) = 0x000000D2;
	*mkptr( dfe_cfg_base, 0x01A418F8 ) = 0x000000D1;
	*mkptr( dfe_cfg_base, 0x01A418FC ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41900 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41904 ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x01A41908 ) = 0x000000CE;
	*mkptr( dfe_cfg_base, 0x01A4190C ) = 0x000000CD;
	*mkptr( dfe_cfg_base, 0x01A41910 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41914 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41918 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x01A4191C ) = 0x000000CA;
	*mkptr( dfe_cfg_base, 0x01A41920 ) = 0x000000C9;
	*mkptr( dfe_cfg_base, 0x01A41924 ) = 0x000000C9;
	*mkptr( dfe_cfg_base, 0x01A41928 ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A4192C ) = 0x000000C7;
	*mkptr( dfe_cfg_base, 0x01A41930 ) = 0x000000C6;
	*mkptr( dfe_cfg_base, 0x01A41934 ) = 0x000000C6;
	*mkptr( dfe_cfg_base, 0x01A41938 ) = 0x000000C5;
	*mkptr( dfe_cfg_base, 0x01A4193C ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41940 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41944 ) = 0x000000C3;
	*mkptr( dfe_cfg_base, 0x01A41948 ) = 0x000000C2;
	*mkptr( dfe_cfg_base, 0x01A4194C ) = 0x000000C2;
	*mkptr( dfe_cfg_base, 0x01A41950 ) = 0x000000C1;
	*mkptr( dfe_cfg_base, 0x01A41954 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41958 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A4195C ) = 0x000000BF;
	*mkptr( dfe_cfg_base, 0x01A41960 ) = 0x000000BE;
	*mkptr( dfe_cfg_base, 0x01A41964 ) = 0x000000BE;
	*mkptr( dfe_cfg_base, 0x01A41968 ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x01A4196C ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x01A41970 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41974 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41978 ) = 0x000000BB;
	*mkptr( dfe_cfg_base, 0x01A4197C ) = 0x000000BA;
	*mkptr( dfe_cfg_base, 0x01A41980 ) = 0x000000BA;
	*mkptr( dfe_cfg_base, 0x01A41984 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x01A41988 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x01A4198C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41990 ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41994 ) = 0x000000B7;
	*mkptr( dfe_cfg_base, 0x01A41998 ) = 0x000000B7;
	*mkptr( dfe_cfg_base, 0x01A4199C ) = 0x000000B6;
	*mkptr( dfe_cfg_base, 0x01A419A0 ) = 0x000000B5;
	*mkptr( dfe_cfg_base, 0x01A419A4 ) = 0x000000B5;
	*mkptr( dfe_cfg_base, 0x01A419A8 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A419AC ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A419B0 ) = 0x000000B3;
	*mkptr( dfe_cfg_base, 0x01A419B4 ) = 0x000000B3;
	*mkptr( dfe_cfg_base, 0x01A419B8 ) = 0x000000B2;
	*mkptr( dfe_cfg_base, 0x01A419BC ) = 0x000000B2;
	*mkptr( dfe_cfg_base, 0x01A419C0 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x01A419C4 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x01A419C8 ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419CC ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419D0 ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419D4 ) = 0x000000AF;
	*mkptr( dfe_cfg_base, 0x01A419D8 ) = 0x000000AF;
	*mkptr( dfe_cfg_base, 0x01A419DC ) = 0x000000AE;
	*mkptr( dfe_cfg_base, 0x01A419E0 ) = 0x000000AE;
	*mkptr( dfe_cfg_base, 0x01A419E4 ) = 0x000000AD;
	*mkptr( dfe_cfg_base, 0x01A419E8 ) = 0x000000AD;
	*mkptr( dfe_cfg_base, 0x01A419EC ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A419F0 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A419F4 ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A419F8 ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A419FC ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A41A00 ) = 0x000000AA;
	*mkptr( dfe_cfg_base, 0x01A41A04 ) = 0x000000AA;
	*mkptr( dfe_cfg_base, 0x01A41A08 ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A0C ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A10 ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A14 ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41A18 ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41A1C ) = 0x000000A7;
	*mkptr( dfe_cfg_base, 0x01A41A20 ) = 0x000000A7;
	*mkptr( dfe_cfg_base, 0x01A41A24 ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A28 ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A2C ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A30 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A34 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A38 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A3C ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41A40 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41A44 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A48 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A4C ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A50 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A54 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A58 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A5C ) = 0x000000A1;
	*mkptr( dfe_cfg_base, 0x01A41A60 ) = 0x000000A1;
	*mkptr( dfe_cfg_base, 0x01A41A64 ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A68 ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A6C ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A70 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A74 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A78 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A7C ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A80 ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A84 ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A88 ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A8C ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A90 ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A94 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41A98 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41A9C ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41AA0 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AA4 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AA8 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AAC ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB0 ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB4 ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB8 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41ABC ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC0 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC4 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC8 ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41ACC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41AD0 ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41AD4 ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41AD8 ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41ADC ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41AE0 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AE4 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AE8 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AEC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF0 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF4 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF8 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AFC ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B00 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B04 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B08 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B0C ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B10 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B14 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B18 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B1C ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B20 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B24 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B28 ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B2C ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B30 ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B34 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B38 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B3C ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B40 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B44 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B48 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B4C ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B50 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B54 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B58 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B5C ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B60 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B64 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B68 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B6C ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B70 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B74 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B78 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B7C ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B80 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B84 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B88 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B8C ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B90 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B94 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41B98 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41B9C ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41BA0 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41BA4 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BA8 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BAC ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BB0 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BB4 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BB8 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BBC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC0 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC4 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC8 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BCC ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD0 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD4 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD8 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BDC ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE0 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE4 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE8 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BEC ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF0 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF4 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF8 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BFC ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41C00 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A41C04 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A41C08 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A41C0C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A41C10 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A41C14 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41C18 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A41C1C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A41C20 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A41C24 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41C28 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A41C2C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A41C30 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A41C34 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A41C38 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A41C3C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A41C40 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A41C44 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A41C48 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A41C4C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A41C50 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A41C54 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A41C58 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A41C5C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41C60 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A41C64 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41C68 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A41C6C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A41C70 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A41C74 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41C78 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A41C7C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41C80 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A41C84 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41C88 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A41C8C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41C90 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A41C94 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A41C98 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A41C9C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A41CA0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A41CA4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A41CA8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A41CAC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41CB0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A41CB4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41CB8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A41CBC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41CC0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A41CC4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41CC8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A41CCC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41CD0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A41CD4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41CD8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A41CDC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41CE0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A41CE4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41CE8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A41CEC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41CF0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A41CF4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41CF8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A41CFC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A41D00 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A41D04 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A41D08 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A41D0C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A41D10 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A41D14 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A41D18 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A41D1C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A41D20 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A41D24 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A41D28 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A41D2C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A41D30 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A41D34 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A41D38 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A41D3C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A41D40 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A41D44 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A41D48 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A41D4C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A41D50 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A41D54 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A41D58 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A41D5C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A41D60 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A41D64 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A41D68 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A41D6C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A41D70 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A41D74 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A41D78 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A41D7C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A41D80 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A41D84 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A41D88 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A41D8C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A41D90 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A41D94 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A41D98 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A41D9C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41DA0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A41DA4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A41DA8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A41DAC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A41DB0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A41DB4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A41DB8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A41DBC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41DC0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A41DC4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A41DC8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A41DCC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A41DD0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A41DD4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A41DD8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A41DDC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41DE0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A41DE4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A41DE8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A41DEC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A41DF0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A41DF4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A41DF8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A41E00 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A41E04 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A41E08 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A41E0C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A41E10 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A41E14 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41E18 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A41E1C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A41E20 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A41E24 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41E28 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A41E2C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A41E30 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A41E34 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A41E38 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A41E3C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A41E40 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A41E44 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A41E48 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A41E4C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A41E50 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A41E54 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A41E58 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A41E5C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41E60 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A41E64 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41E68 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A41E6C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A41E70 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A41E74 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41E78 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A41E7C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41E80 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A41E84 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41E88 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A41E8C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41E90 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A41E94 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A41E98 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A41E9C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A41EA0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A41EA4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A41EA8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A41EAC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41EB0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A41EB4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41EB8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A41EBC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41EC0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A41EC4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41EC8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A41ECC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41ED0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A41ED4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41ED8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A41EDC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41EE0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A41EE4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41EE8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A41EEC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41EF0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A41EF4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41EF8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A41EFC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A41F00 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A41F04 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A41F08 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A41F0C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A41F10 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A41F14 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A41F18 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A41F1C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A41F20 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A41F24 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A41F28 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A41F2C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A41F30 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A41F34 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A41F38 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A41F3C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A41F40 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A41F44 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A41F48 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A41F4C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A41F50 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A41F54 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A41F58 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A41F5C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A41F60 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A41F64 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A41F68 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A41F6C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A41F70 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A41F74 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A41F78 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A41F7C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A41F80 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A41F84 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A41F88 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A41F8C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A41F90 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A41F94 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A41F98 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A41F9C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41FA0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A41FA4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A41FA8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A41FAC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A41FB0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A41FB4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A41FB8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A41FBC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41FC0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A41FC4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A41FC8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A41FCC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A41FD0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A41FD4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A41FD8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A41FDC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41FE0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A41FE4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A41FE8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A41FEC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A41FF0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A41FF4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A41FF8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A42000 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A42004 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A42008 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A4200C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A42010 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A42014 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A42018 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A4201C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A42020 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A42024 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A42028 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A4202C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A42030 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A42034 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A42038 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A4203C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A42040 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A42044 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A42048 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A4204C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A42050 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A42054 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A42058 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A4205C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A42060 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A42064 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A42068 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A4206C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A42070 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A42074 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A42078 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A4207C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A42080 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A42084 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A42088 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A4208C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A42090 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A42094 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A42098 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A4209C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A420A0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A420A4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A420A8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A420AC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A420B0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A420B4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A420B8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A420BC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A420C0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A420C4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A420C8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A420CC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A420D0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A420D4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A420D8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A420DC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A420E0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A420E4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A420E8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A420EC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A420F0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A420F4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A420F8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A420FC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A42100 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A42104 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A42108 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A4210C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A42110 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A42114 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A42118 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A4211C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A42120 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A42124 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A42128 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A4212C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A42130 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A42134 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A42138 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A4213C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A42140 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A42144 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A42148 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A4214C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A42150 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A42154 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A42158 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A4215C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A42160 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A42164 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A42168 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A4216C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A42170 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A42174 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A42178 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A4217C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A42180 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A42184 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A42188 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A4218C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A42190 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A42194 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A42198 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A4219C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A421A0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A421A4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A421A8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A421AC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A421B0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A421B4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A421B8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A421BC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A421C0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A421C4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A421C8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A421CC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A421D0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A421D4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A421D8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A421DC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A421E0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A421E4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A421E8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A421EC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A421F0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A421F4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A421F8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A42200 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A42204 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A42208 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A4220C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A42210 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A42214 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A42218 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A4221C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A42220 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A42224 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A42228 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A4222C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A42230 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A42234 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A42238 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A4223C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A42240 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A42244 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A42248 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A4224C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A42250 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A42254 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A42258 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A4225C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A42260 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A42264 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A42268 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A4226C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A42270 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A42274 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A42278 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A4227C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A42280 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A42284 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A42288 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A4228C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A42290 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A42294 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A42298 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A4229C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A422A0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A422A4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A422A8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A422AC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A422B0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A422B4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A422B8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A422BC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A422C0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A422C4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A422C8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A422CC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A422D0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A422D4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A422D8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A422DC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A422E0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A422E4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A422E8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A422EC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A422F0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A422F4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A422F8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A422FC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A42300 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A42304 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A42308 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A4230C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A42310 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A42314 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A42318 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A4231C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A42320 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A42324 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A42328 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A4232C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A42330 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A42334 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A42338 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A4233C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A42340 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A42344 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A42348 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A4234C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A42350 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A42354 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A42358 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A4235C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A42360 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A42364 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A42368 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A4236C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A42370 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A42374 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A42378 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A4237C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A42380 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A42384 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A42388 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A4238C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A42390 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A42394 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A42398 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A4239C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A423A0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A423A4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A423A8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A423AC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A423B0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A423B4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A423B8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A423BC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A423C0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A423C4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A423C8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A423CC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A423D0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A423D4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A423D8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A423DC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A423E0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A423E4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A423E8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A423EC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A423F0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A423F4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A423F8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A41400 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41440 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41480 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A414C0 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41500 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x01A41508 ) = 0x000007D0;
	*mkptr( dfe_cfg_base, 0x01A006EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006F8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006FC ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x01A00A0C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A00A14 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A0020C ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01A00214 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A40620 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A406A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40720 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A407A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40420 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A404A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40520 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A405A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40220 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A402A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40320 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A403A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40020 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A400A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40120 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A401A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A0021C ) = 0x00001111;
	*mkptr( dfe_cfg_base, 0x01A00220 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x01A00224 ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A00228 ) = 0x00001280;
	*mkptr( dfe_cfg_base, 0x01A002F0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002F8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002E0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002E8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A40800 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40804 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40808 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A4080C ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40810 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40814 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40818 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A4081C ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40820 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40824 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40828 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4082C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40830 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40834 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40838 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4083C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40840 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40844 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40848 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4084C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40850 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40854 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40858 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4085C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40860 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40864 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40868 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4086C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40870 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40874 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40878 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4087C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40880 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40884 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40888 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4088C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40890 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40894 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40898 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4089C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408AC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408BC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408CC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408EC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408FC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40900 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40904 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40908 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4090C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40910 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40914 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40918 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4091C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40920 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40924 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40928 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4092C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40930 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40934 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40938 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4093C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40940 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40944 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40948 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4094C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40950 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40954 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40958 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4095C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40960 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40964 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40968 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4096C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40970 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40974 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40978 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4097C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40980 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40984 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40988 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4098C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40990 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40994 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40998 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4099C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409AC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409BC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409CC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409EC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409FC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40A00 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A04 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A08 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A0C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A10 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A14 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A18 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A1C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A20 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A24 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A28 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A2C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A30 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A34 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A38 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A3C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A40 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A44 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A48 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A4C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A50 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A54 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A58 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A5C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A60 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A64 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A68 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A6C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A70 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A74 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A78 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A7C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A80 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A84 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A88 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A8C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A90 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A94 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A98 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A9C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AAC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ABC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ACC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ADC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AEC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AFC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B00 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B04 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B08 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B0C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B10 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B14 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B18 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B1C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B20 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B24 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B28 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B2C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B30 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B34 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B38 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B3C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B40 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B44 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B48 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B4C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B50 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B54 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B58 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B5C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B60 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B64 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B68 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B6C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B70 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B74 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B78 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B7C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B80 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B84 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B88 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B8C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B90 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B94 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B98 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B9C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BAC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BBC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BCC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BDC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BEC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BFC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A003EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003F8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003FC ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x01A0023C ) = 0x00000010;
}

#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config_split_2,".text:dfe");
#endif
static void iqn2_bb_loopback_config_split_2() {
	*mkptr( dfe_cfg_base, 0x01E80700 ) = 0x00001100;
	*mkptr( dfe_cfg_base, 0x01E80704 ) = 0x0000300C;
	*mkptr( dfe_cfg_base, 0x01E80708 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01E80728 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01E8073C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x01EC0000 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0040 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC00C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0100 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0140 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0180 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC01C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0200 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0240 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E80258 ) = 0x00007D00;
	*mkptr( dfe_cfg_base, 0x01E80278 ) = 0x0000FA00;
	*mkptr( dfe_cfg_base, 0x01E80288 ) = 0x00003880;
	*mkptr( dfe_cfg_base, 0x01E80210 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x01E80204 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x01E80218 ) = 0x00001F00;
	*mkptr( dfe_cfg_base, 0x01EC0800 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0804 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0808 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC080C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0810 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0814 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0818 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC081C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0820 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0824 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0828 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC082C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0830 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0834 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0838 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC083C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0840 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0844 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0848 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC084C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0850 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0854 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0858 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC085C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0860 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0864 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0868 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC086C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0870 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0874 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0878 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC087C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0880 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0884 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0888 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC088C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0890 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0894 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0898 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC089C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0900 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0904 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0908 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC090C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0910 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0914 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0918 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC091C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0920 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0924 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0928 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC092C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0930 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0934 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0938 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC093C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0940 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0944 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0948 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC094C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0950 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0954 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0958 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC095C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0960 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0964 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0968 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC096C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0970 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0974 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0978 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC097C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0980 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0984 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0988 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC098C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0990 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0994 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0998 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC099C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01E80214 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01E802B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00200 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01C00208 ) = 0x00001111;
	*mkptr( dfe_cfg_base, 0x01C00210 ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01C00214 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00218 ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01C0021C ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01C00220 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01C00224 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00228 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01C0022C ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00234 ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00238 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0023C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01C00240 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00244 ) = 0x00100000;
	*mkptr( dfe_cfg_base, 0x01C00248 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0024C ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00250 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00254 ) = 0x10100000;
	*mkptr( dfe_cfg_base, 0x01C00258 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0025C ) = 0x32100000;
	*mkptr( dfe_cfg_base, 0x01C00260 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00264 ) = 0x10100000;
	*mkptr( dfe_cfg_base, 0x01C00268 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0026C ) = 0x32100000;
	*mkptr( dfe_cfg_base, 0x01C00270 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00274 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01C00278 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01C00280 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01C00328 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01C003E4 ) = 0x01010101;
	*mkptr( dfe_cfg_base, 0x01F00064 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x01F00068 ) = 0x0E030400;
	*mkptr( dfe_cfg_base, 0x01F00080 ) = 0x00009000;
	*mkptr( dfe_cfg_base, 0x01F00084 ) = 0x00006600;
	*mkptr( dfe_cfg_base, 0x01F00088 ) = 0x00006720;
	*mkptr( dfe_cfg_base, 0x01F0008C ) = 0x00006228;
	*mkptr( dfe_cfg_base, 0x01F00090 ) = 0x0000A101;
	*mkptr( dfe_cfg_base, 0x01F00094 ) = 0x00004043;
	*mkptr( dfe_cfg_base, 0x01F00098 ) = 0x00004044;
	*mkptr( dfe_cfg_base, 0x01F0009C ) = 0x00009404;
	*mkptr( dfe_cfg_base, 0x01F000A0 ) = 0x00004008;
	*mkptr( dfe_cfg_base, 0x01F000A4 ) = 0x00004010;
	*mkptr( dfe_cfg_base, 0x01F000A8 ) = 0x00004018;
	*mkptr( dfe_cfg_base, 0x01F000B8 ) = 0x00006305;
	*mkptr( dfe_cfg_base, 0x01F000BC ) = 0x00009707;
	*mkptr( dfe_cfg_base, 0x01F00400 ) = 0x000409FC;
	*mkptr( dfe_cfg_base, 0x01F00404 ) = 0x02800002;
	*mkptr( dfe_cfg_base, 0x01F00408 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01F0040C ) = 0x01000000;
	*mkptr( dfe_cfg_base, 0x01F00410 ) = 0x80069008;
	*mkptr( dfe_cfg_base, 0x01F00414 ) = 0x03080020;
	*mkptr( dfe_cfg_base, 0x01F00418 ) = 0x000409FC;
	*mkptr( dfe_cfg_base, 0x01F0041C ) = 0x03800002;
	*mkptr( dfe_cfg_base, 0x01F00424 ) = 0x04000000;
	*mkptr( dfe_cfg_base, 0x01F00428 ) = 0x9DF01400;
	*mkptr( dfe_cfg_base, 0x01F0042C ) = 0x050A0100;
	*mkptr( dfe_cfg_base, 0x01F00434 ) = 0x06000000;
	*mkptr( dfe_cfg_base, 0x01F0043C ) = 0x07000000;
	*mkptr( dfe_cfg_base, 0x01F00440 ) = 0x91340000;
	*mkptr( dfe_cfg_base, 0x01F00444 ) = 0x09069000;
	*mkptr( dfe_cfg_base, 0x01F00448 ) = 0x91349000;
	*mkptr( dfe_cfg_base, 0x01F0044C ) = 0x0A069000;
	*mkptr( dfe_cfg_base, 0x01F00450 ) = 0x91352000;
	*mkptr( dfe_cfg_base, 0x01F00454 ) = 0x0B069000;
	*mkptr( dfe_cfg_base, 0x01F00458 ) = 0x9135B000;
	*mkptr( dfe_cfg_base, 0x01F0045C ) = 0x0B069000;
	*mkptr( dfe_cfg_base, 0x01F00464 ) = 0x0C000000;
	*mkptr( dfe_cfg_base, 0x01F0046C ) = 0x0D000000;
	*mkptr( dfe_cfg_base, 0x01F00474 ) = 0x0E000000;
	*mkptr( dfe_cfg_base, 0x01F0047C ) = 0x0F000000;
	*mkptr( dfe_cfg_base, 0x01F00480 ) = 0x15C40000;
	*mkptr( dfe_cfg_base, 0x01F00484 ) = 0x11018000;
	*mkptr( dfe_cfg_base, 0x01F00488 ) = 0x15C48000;
	*mkptr( dfe_cfg_base, 0x01F0048C ) = 0x12018000;
	*mkptr( dfe_cfg_base, 0x01F00490 ) = 0x15C50000;
	*mkptr( dfe_cfg_base, 0x01F00494 ) = 0x13018000;
	*mkptr( dfe_cfg_base, 0x01F00498 ) = 0x15C58000;
	*mkptr( dfe_cfg_base, 0x01F0049C ) = 0x13018000;
	*mkptr( dfe_cfg_base, 0x01F004A0 ) = 0x15C60000;
	*mkptr( dfe_cfg_base, 0x01F004A4 ) = 0x15018000;
	*mkptr( dfe_cfg_base, 0x01F004A8 ) = 0x15C68000;
	*mkptr( dfe_cfg_base, 0x01F004AC ) = 0x16018000;
	*mkptr( dfe_cfg_base, 0x01F004B0 ) = 0x15C70000;
	*mkptr( dfe_cfg_base, 0x01F004B4 ) = 0x17018000;
	*mkptr( dfe_cfg_base, 0x01F004B8 ) = 0x15C78000;
	*mkptr( dfe_cfg_base, 0x01F004BC ) = 0x17018000;
	*mkptr( dfe_cfg_base, 0x01F004C0 ) = 0x91C40004;
	*mkptr( dfe_cfg_base, 0x01F004C4 ) = 0x19868000;
	*mkptr( dfe_cfg_base, 0x01F004C8 ) = 0x91C48004;
	*mkptr( dfe_cfg_base, 0x01F004CC ) = 0x1A868000;
	*mkptr( dfe_cfg_base, 0x01F004D0 ) = 0x91C50004;
	*mkptr( dfe_cfg_base, 0x01F004D4 ) = 0x1B868000;
	*mkptr( dfe_cfg_base, 0x01F004D8 ) = 0x91C58004;
	*mkptr( dfe_cfg_base, 0x01F004DC ) = 0x1B068000;
	*mkptr( dfe_cfg_base, 0x01F004E0 ) = 0x91C60004;
	*mkptr( dfe_cfg_base, 0x01F004E4 ) = 0x1D868000;
	*mkptr( dfe_cfg_base, 0x01F004E8 ) = 0x91C68004;
	*mkptr( dfe_cfg_base, 0x01F004EC ) = 0x1E868000;
	*mkptr( dfe_cfg_base, 0x01F004F0 ) = 0x91C70004;
	*mkptr( dfe_cfg_base, 0x01F004F4 ) = 0x1F868000;
	*mkptr( dfe_cfg_base, 0x01F004F8 ) = 0x91C78004;
	*mkptr( dfe_cfg_base, 0x01F004FC ) = 0x1F068000;
	*mkptr( dfe_cfg_base, 0x01F00500 ) = 0x000405FC;
	*mkptr( dfe_cfg_base, 0x01F00504 ) = 0x22800002;
	*mkptr( dfe_cfg_base, 0x01F00508 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01F0050C ) = 0x21000000;
	*mkptr( dfe_cfg_base, 0x01F00510 ) = 0x84068008;
	*mkptr( dfe_cfg_base, 0x01F00514 ) = 0x23080020;
	*mkptr( dfe_cfg_base, 0x01F00518 ) = 0x000405FC;
	*mkptr( dfe_cfg_base, 0x01F0051C ) = 0x24800002;
	*mkptr( dfe_cfg_base, 0x01F00524 ) = 0x24000000;
	*mkptr( dfe_cfg_base, 0x01F0052C ) = 0x25000000;
	*mkptr( dfe_cfg_base, 0x01F00534 ) = 0x26000000;
	*mkptr( dfe_cfg_base, 0x01F0053C ) = 0x27000000;
	*mkptr( dfe_cfg_base, 0x01F00544 ) = 0x28000000;
	*mkptr( dfe_cfg_base, 0x01F0054C ) = 0x29000000;
	*mkptr( dfe_cfg_base, 0x01F00554 ) = 0x2A000000;
	*mkptr( dfe_cfg_base, 0x01F0055C ) = 0x2B000000;
	*mkptr( dfe_cfg_base, 0x01F00564 ) = 0x2C000000;
	*mkptr( dfe_cfg_base, 0x01F0056C ) = 0x2D000000;
	*mkptr( dfe_cfg_base, 0x01F00574 ) = 0x2E000000;
	*mkptr( dfe_cfg_base, 0x01F0057C ) = 0x2F000000;
	*mkptr( dfe_cfg_base, 0x01F00584 ) = 0x30000000;
	*mkptr( dfe_cfg_base, 0x01F0058C ) = 0x31000000;
	*mkptr( dfe_cfg_base, 0x01F00594 ) = 0x32000000;
	*mkptr( dfe_cfg_base, 0x01F0059C ) = 0x33000000;
	*mkptr( dfe_cfg_base, 0x01F005A4 ) = 0x34000000;
	*mkptr( dfe_cfg_base, 0x01F005AC ) = 0x35000000;
	*mkptr( dfe_cfg_base, 0x01F005B4 ) = 0x36000000;
	*mkptr( dfe_cfg_base, 0x01F005BC ) = 0x37000000;
	*mkptr( dfe_cfg_base, 0x01F005C4 ) = 0x38000000;
	*mkptr( dfe_cfg_base, 0x01F005CC ) = 0x39000000;
	*mkptr( dfe_cfg_base, 0x01F005D4 ) = 0x3A000000;
	*mkptr( dfe_cfg_base, 0x01F005DC ) = 0x3B000000;
	*mkptr( dfe_cfg_base, 0x01F005E4 ) = 0x3C000000;
	*mkptr( dfe_cfg_base, 0x01F005EC ) = 0x3D000000;
	*mkptr( dfe_cfg_base, 0x01F005F4 ) = 0x3E000000;
	*mkptr( dfe_cfg_base, 0x01F005FC ) = 0x3F000000;
	*mkptr( dfe_cfg_base, 0x01F00604 ) = 0x40000000;
	*mkptr( dfe_cfg_base, 0x01F0060C ) = 0x41000000;
	*mkptr( dfe_cfg_base, 0x01F00614 ) = 0x42000000;
	*mkptr( dfe_cfg_base, 0x01F0061C ) = 0x43000000;
	*mkptr( dfe_cfg_base, 0x01F00624 ) = 0x44000000;
	*mkptr( dfe_cfg_base, 0x01F0062C ) = 0x45000000;
	*mkptr( dfe_cfg_base, 0x01F00634 ) = 0x46000000;
	*mkptr( dfe_cfg_base, 0x01F0063C ) = 0x47000000;
	*mkptr( dfe_cfg_base, 0x01F00644 ) = 0x48000000;
	*mkptr( dfe_cfg_base, 0x01F0064C ) = 0x49000000;
	*mkptr( dfe_cfg_base, 0x01F00654 ) = 0x4A000000;
	*mkptr( dfe_cfg_base, 0x01F0065C ) = 0x4B000000;
	*mkptr( dfe_cfg_base, 0x01F00664 ) = 0x4C000000;
	*mkptr( dfe_cfg_base, 0x01F0066C ) = 0x4D000000;
	*mkptr( dfe_cfg_base, 0x01F00674 ) = 0x4E000000;
	*mkptr( dfe_cfg_base, 0x01F0067C ) = 0x4F000000;
	*mkptr( dfe_cfg_base, 0x01F00684 ) = 0x50000000;
	*mkptr( dfe_cfg_base, 0x01F0068C ) = 0x51000000;
	*mkptr( dfe_cfg_base, 0x01F00694 ) = 0x52000000;
	*mkptr( dfe_cfg_base, 0x01F0069C ) = 0x53000000;
	*mkptr( dfe_cfg_base, 0x01F006A4 ) = 0x54000000;
	*mkptr( dfe_cfg_base, 0x01F006AC ) = 0x55000000;
	*mkptr( dfe_cfg_base, 0x01F006B4 ) = 0x56000000;
	*mkptr( dfe_cfg_base, 0x01F006BC ) = 0x57000000;
	*mkptr( dfe_cfg_base, 0x01F006C4 ) = 0x58000000;
	*mkptr( dfe_cfg_base, 0x01F006CC ) = 0x59000000;
	*mkptr( dfe_cfg_base, 0x01F006D4 ) = 0x5A000000;
	*mkptr( dfe_cfg_base, 0x01F006DC ) = 0x5B000000;
	*mkptr( dfe_cfg_base, 0x01F006E4 ) = 0x5C000000;
	*mkptr( dfe_cfg_base, 0x01F006EC ) = 0x5D000000;
	*mkptr( dfe_cfg_base, 0x01F006F4 ) = 0x5E000000;
	*mkptr( dfe_cfg_base, 0x01F006FC ) = 0x5F000000;
	*mkptr( dfe_cfg_base, 0x01F00704 ) = 0x60000000;
	*mkptr( dfe_cfg_base, 0x01F0070C ) = 0x61000000;
	*mkptr( dfe_cfg_base, 0x01F00714 ) = 0x62000000;
	*mkptr( dfe_cfg_base, 0x01F0071C ) = 0x63000000;
	*mkptr( dfe_cfg_base, 0x01F00724 ) = 0x64000000;
	*mkptr( dfe_cfg_base, 0x01F0072C ) = 0x65000000;
	*mkptr( dfe_cfg_base, 0x01F00734 ) = 0x66000000;
	*mkptr( dfe_cfg_base, 0x01F0073C ) = 0x67000000;
	*mkptr( dfe_cfg_base, 0x01F00744 ) = 0x68000000;
	*mkptr( dfe_cfg_base, 0x01F0074C ) = 0x69000000;
	*mkptr( dfe_cfg_base, 0x01F00754 ) = 0x6A000000;
	*mkptr( dfe_cfg_base, 0x01F0075C ) = 0x6B000000;
	*mkptr( dfe_cfg_base, 0x01F00764 ) = 0x6C000000;
	*mkptr( dfe_cfg_base, 0x01F0076C ) = 0x6D000000;
	*mkptr( dfe_cfg_base, 0x01F00774 ) = 0x6E000000;
	*mkptr( dfe_cfg_base, 0x01F0077C ) = 0x6F000000;
	*mkptr( dfe_cfg_base, 0x01F00784 ) = 0x70000000;
	*mkptr( dfe_cfg_base, 0x01F0078C ) = 0x71000000;
	*mkptr( dfe_cfg_base, 0x01F00794 ) = 0x72000000;
	*mkptr( dfe_cfg_base, 0x01F0079C ) = 0x73000000;
	*mkptr( dfe_cfg_base, 0x01F007A4 ) = 0x74000000;
	*mkptr( dfe_cfg_base, 0x01F007AC ) = 0x75000000;
	*mkptr( dfe_cfg_base, 0x01F007B4 ) = 0x76000000;
	*mkptr( dfe_cfg_base, 0x01F007BC ) = 0x77000000;
	*mkptr( dfe_cfg_base, 0x01F007C4 ) = 0x78000000;
	*mkptr( dfe_cfg_base, 0x01F007CC ) = 0x79000000;
	*mkptr( dfe_cfg_base, 0x01F007D4 ) = 0x7A000000;
	*mkptr( dfe_cfg_base, 0x01F007DC ) = 0x7B000000;
	*mkptr( dfe_cfg_base, 0x01F007E4 ) = 0x7C000000;
	*mkptr( dfe_cfg_base, 0x01F007EC ) = 0x7D000000;
	*mkptr( dfe_cfg_base, 0x01F007F4 ) = 0x7E000000;
	*mkptr( dfe_cfg_base, 0x01F007FC ) = 0x7F000000;
	*mkptr( dfe_cfg_base, 0x01F00004 ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01F01004 ) = 0x00082011;
	*mkptr( dfe_cfg_base, 0x01F01024 ) = 0x00000012;
}

#endif

