/*  ============================================================================
 *   Copyright (c) Texas Instruments Incorporated 2012-2013
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#define mkptr(base,offset)                ((volatile unsigned int *)((base)+(offset)))
#define sleep(ticks)    {volatile int i; for (i=0; i < ticks; i++);}
#define UINT unsigned int

UINT dfe_cfg_base =  0x24000000; //MMR base address of DFE
#define MASK 0x80

static void vbusp_rmw32(UINT addr, UINT data, UINT mask)
{
 UINT rdata;
  rdata= *mkptr(addr,0x0);
	sleep(1);
  *mkptr(addr, 0x0)=((rdata &~mask)|(data & mask));
}

static void vbusp_write32(UINT addr, UINT data)
{
 sleep(1);
*mkptr(addr, 0x0)=data;
}

static UINT vbusp_read32(UINT addr)
{
 sleep(1);
 return (*mkptr(addr, 0x0));
}

#ifdef _TMS320C6X
#pragma CODE_SECTION(dfe_init,".text:dfe");
#endif

void dfe_init()
{
        UINT read_data;
        //UINT dduc_tx =0x0;
        UINT dduc_tx0=0;
        UINT dduc_tx1=0;
        UINT dduc_tx2=0;
        UINT dduc_tx3=0;


        ////////initialization sequence//////

        //First, be sure the bb's frame start logic is set to never sync
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040370, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040374, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040378, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x0004037c, 0x00000000);
        sleep(25);

        //Release mem_mpu_access for all blocks
        vbusp_write32(dfe_cfg_base + 0x01f00000 + 0x00000008, 0x00000000);

        //First, determine which dducs are in tx and which are rx.
        read_data = vbusp_read32 (dfe_cfg_base + 0x00100000 + 0x00000400);
//	dduc_tx |= ((read_data & MASK)>>7);
	dduc_tx0= ((read_data & MASK)>>7);


        read_data = vbusp_read32 (dfe_cfg_base + 0x00180000 + 0x00000400);
//        dduc_tx |=(((read_data & MASK)>>7)<<1);
	dduc_tx1= ((read_data & MASK)>>7);

        read_data = vbusp_read32 (dfe_cfg_base + 0x00200000 + 0x00000400);
//	dduc_tx |= (((read_data & MASK)>>7)<<2);
	dduc_tx2= ((read_data & MASK)>>7);

        read_data = vbusp_read32 (dfe_cfg_base + 0x00280000 + 0x00000400);
	dduc_tx3= ((read_data & MASK)>>7);
//	dduc_tx |= (((read_data & MASK)>>7)<<3);

        //Release each block in signal processing order. Each given block releases init_clk_gate,
        //then init_state, then clear_data, with delays between each release.
        //misc
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32(dfe_cfg_base + 0x01f00000 + 0x00000004, 0x0000000f, 0x0000007f);
        sleep(25);

        //cb (being done first to support cb sourcing, so sourcing can begin before blocks are initialized)
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01c00000 + 0x00000450, 0x0000000f, 0x0000007f);
        sleep(25);

        //bb
        //***hmm...all blocks operate in only 1 direction EXCEPT for bb...how to release it? Doing it first for now...
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00000000 + 0x00040234, 0x0000000f, 0x0000007f);
        sleep(25);

        //tx dducs (dducs must be released at the same time; use mpu_sync)
        if (dduc_tx0==1) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx1==1) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx2==1) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000061, 0x0000007f);
        if (dduc_tx3==1) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        if (dduc_tx0==1) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx1==1) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx2==1) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000041, 0x0000007f);
        if (dduc_tx3==1) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        if (dduc_tx0==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00001111);
        }

        if (dduc_tx1==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00001111);
        }
        if (dduc_tx2==1)
        {
	    vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00001111);
        }

        if (dduc_tx3==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00001111); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00001111);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00001111);
        }
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        if (dduc_tx0==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx1==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx2==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx3==1)
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000000);
        }
        if (dduc_tx0==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx1==1)
		vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx2==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000001, 0x0000007f);
        if (dduc_tx3==1) 
		vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000001, 0x0000007f);

        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //summer (does not have init_clk_gate)
        vbusp_rmw32 (dfe_cfg_base + 0x00900000 + 0x00000014, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00900000 + 0x00000014, 0x0000000f, 0x0000007f);
        sleep(25);

        //acl
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x00f00000 + 0x00000040, 0x0000000f, 0x0000007f);
        sleep(25);

        //cfrs (both cfrs must be released at the same time; use mpu_sync)
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000061, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000041, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000600, 0x00000001); //pre mult sync
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000600, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000604, 0x00000001); //post mult sync
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000604, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000600, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000600, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b00000 + 0x00000604, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x00b80000 + 0x00000604, 0x00000000);
        vbusp_rmw32   (dfe_cfg_base + 0x00b00000 + 0x00000004, 0x00000001, 0x0000007f);
        vbusp_rmw32   (dfe_cfg_base + 0x00b80000 + 0x00000004, 0x00000001, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu_sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //cdfr
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01000000 + 0x00001000, 0x0000000f, 0x0000007f);
        sleep(25);

        //dpda
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01500000 + 0x00000080, 0x0000000f, 0x0000007f);
        sleep(25);

        //dpd
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01300000 + 0x0000022c, 0x0000000f, 0x0000007f);
        sleep(25);

        //tx
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01600000 + 0x00000b38, 0x0000000f, 0x0000007f);
        sleep(25);

        //jesdtx
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00000004, 0x0000000f, 0x00000f7f); //also release per-lane clear_data
        sleep(25);

        //rx (needs to be released BEFORE jesdrx ***I think? This was needed in Appleton RX...)
        //***wait, do the dc accumulator syncs need to be sent AFTER the frames start arriving?
        //***even if rx needs to have clear_data and init_state released before frames arrive, does it have to be AFTER cken arrives?
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000310, 0x00000010, 0x000000f0); //dc accumulator syncs
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00001111);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_rmw32   (dfe_cfg_base + 0x01a00000 + 0x00000310, 0x00000000, 0x000000f0);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000000);
        vbusp_rmw32 (dfe_cfg_base + 0x01a00000 + 0x00000e00, 0x0000000f, 0x0000007f);
        sleep(25);

        //fb (needs to be released BEFORE jesdrx ***I think? This was needed in Appleton RX...)
        //***wait, do the dc accumulator syncs need to be sent AFTER the frames start arriving?
        //***even if fb needs to have clear_data and init_state released before frames arrive, does it have to be AFTER cken arrives?
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000388, 0x00000001); //dc accumulator syncs
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000001);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000388, 0x00000000);
        vbusp_write32 (dfe_cfg_base + 0x01a00000 + 0x00000318, 0x00000000);
        vbusp_rmw32 (dfe_cfg_base + 0x01e80000 + 0x00000200, 0x0000000f, 0x0000007f);
        sleep(25);

        //jesdrx
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000006f, 0x0000007f);
        sleep(25);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000004f, 0x0000007f);
        sleep(250);
        vbusp_rmw32 (dfe_cfg_base + 0x01d00000 + 0x00040004, 0x0000000f, 0x00000f7f); //also release per-lane clear_data
        sleep(25);

        //rx dducs (dducs must be released at the same time; use mpu_sync)
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000061, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000061, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000041, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000041, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(250);
        if (!(dduc_tx0==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000001);
        } 
        if (!(dduc_tx1==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000001);
        }
        if (!(dduc_tx2==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000001);
        }
        if (!(dduc_tx3==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000001); //nco syncs
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000001);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000001);
        }
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        if (!(dduc_tx0==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00100000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx1==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00180000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx2==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00200000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx3==1))
        {
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d0, 0x00000000); //nco sync back to never
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d4, 0x00000000);
            vbusp_write32 (dfe_cfg_base + 0x00280000 + 0x000012d8, 0x00000000);
        }
        if (!(dduc_tx0==1)) vbusp_rmw32 (dfe_cfg_base + 0x00100000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx1==1)) vbusp_rmw32 (dfe_cfg_base + 0x00180000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx2==1)) vbusp_rmw32 (dfe_cfg_base + 0x00200000 + 0x00000400, 0x00000001, 0x0000007f);
        if (!(dduc_tx3==1)) vbusp_rmw32 (dfe_cfg_base + 0x00280000 + 0x00000400, 0x00000001, 0x0000007f);
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000001); //toggle mpu sync
        vbusp_write32 (dfe_cfg_base + 0x01f00000 + 0x00002400, 0x00000000);
        sleep(25);

        //Finally, set the bb's frame start logic to look at the IQN2's 5/10/20ms
        //(depending on the standard) frame.
        //***also, for now the mux for WHICH strobe is used here is assumed to be done in the .reg file...
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040370, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040374, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x00040378, 0x00002222);
        vbusp_write32 (dfe_cfg_base + 0x00000000 + 0x0004037c, 0x00002222);
        sleep(25);

}

UINT addrTableLocal[18][2] =
{
        {0x00000000,  0x0006C0F4},
        {0x00100000,  0x001438BC},
        {0x00180000,  0x001C38BC},
        {0x00200000,  0x002438BC},
        {0x00280000,  0x002C38BC},
        {0x00900000,  0x00900074},
        {0x00B00000,  0x00B183FC},
        {0x00B80000,  0x00B983FC},
        {0x00F00000,  0x00F00000},
        {0x01000000,  0x01001118},
        {0x01300000,  0x01363FFC},
        {0x01500000,  0x01500000},
        {0x01600000,  0x01640BFC},
        {0x01A00000,  0x01A423FC},
        {0x01C00000,  0x01C00488},
        {0x01D00000,  0x01D42DD0},
        {0x01E80000,  0x01EC09FC},
        {0x01F00000,  0x01F00000}
};

#ifdef _TMS320C6X
#pragma CODE_SECTION(preloadTgtcfg_local,".text:dfe");
#endif
void preloadTgtcfg_local()
{
    UINT ui, i;

    for(i = 0; i < 18; i++)
		if(addrTableLocal[i][1]>addrTableLocal[i][0])
			for(ui = addrTableLocal[i][0]; ui<=addrTableLocal[i][1]; ui+=4)
			{
				*mkptr( dfe_cfg_base, ui ) = 0x0;
			}

    return;
}

#if DFE_BB_LOOPBACK == 1
static void iqn2_bb_loopback_config_split_1();
static void iqn2_bb_loopback_config_split_2();
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config,".text:dfe");
#endif
void iqn2_bb_loopback_config()
{
	// based on P1_3_LTE20_bblpbk.reg
	*mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
	*mkptr( dfe_cfg_base, 0x0004020C ) = 0x00004080;
	*mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x00040214 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00040228 ) = 0x00001000;
	*mkptr( dfe_cfg_base, 0x00040240 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x00040244 ) = 0x00002222;
	*mkptr( dfe_cfg_base, 0x00040250 ) = 0x00004444;
	*mkptr( dfe_cfg_base, 0x0004027C ) = 0x00000808;
	*mkptr( dfe_cfg_base, 0x00040280 ) = 0x00000808;
	*mkptr( dfe_cfg_base, 0x000402C0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x000402D0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x000402E4 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x000402F4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00040344 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000811;
	*mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x0004036C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00040374 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00040400 ) = 0x000001C4;
	*mkptr( dfe_cfg_base, 0x00040408 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00040410 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00040800 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000D67;
	*mkptr( dfe_cfg_base, 0x00060000 ) = 0x0000D000;
	*mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060010 ) = 0x0000D001;
	*mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060020 ) = 0x0000D010;
	*mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060030 ) = 0x0000D011;
	*mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008420;
	*mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008421;
	*mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008430;
	*mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008431;
	*mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000608A4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000608C4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000608E4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000609A4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000609C4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000609E4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060A84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060AE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060B84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060BE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060C84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060CE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D04 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D24 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D44 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D64 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060D84 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DA4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DC4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00060DE4 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x0006100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00061010 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061014 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061018 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006101C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061020 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061024 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006102C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006103C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006104C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006105C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006106C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006107C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061080 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061084 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061088 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006108C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061090 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061094 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061098 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006109C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000610FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006110C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006111C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006112C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006113C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006114C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006115C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006116C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006117C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006118C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006119C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000611FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006120C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006121C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006122C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006123C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006124C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006125C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006126C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006127C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006128C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006129C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000612FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006130C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006131C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006132C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006133C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006134C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006135C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006136C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006137C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006138C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006139C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000613FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614A0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614A8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614B0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614B8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614C0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614C8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614D0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614D8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614E0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614E8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000614F8 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00061800 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061808 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061810 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061818 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618A0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618A8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618B0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618B8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618E0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618E8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618F0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000618F8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619A0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619A8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619B0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619B8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619E0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619E8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619F0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x000619F8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A00 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A08 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A10 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A18 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A20 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A28 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A30 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061A38 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00061C00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061C9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061CFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061D9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061DFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061E9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ECC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061ED8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061EFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061F9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00061FFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006200C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006201C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006202C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006203C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006204C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006205C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006206C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006207C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006208C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006209C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000620BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000401;
	*mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x0006240C ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00062410 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062414 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062418 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006241C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062420 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062424 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062428 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006242C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062430 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062434 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062438 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006243C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006244C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006245C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006246C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006247C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062480 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062484 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062488 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006248C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062490 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062494 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062498 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006249C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000624FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006250C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006251C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006252C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006253C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006254C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006255C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006256C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006257C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006258C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006259C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000625FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006260C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006261C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006262C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006263C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006264C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006265C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006266C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006267C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006268C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006269C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000626FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006270C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006271C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006272C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006273C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006274C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006275C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006276C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006277C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006278C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006279C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000627FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006280C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006281C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006282C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006283C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006284C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006285C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006286C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006287C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006288C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006289C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000628FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006290C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006291C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006292C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006293C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006294C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006295C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006296C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006297C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006298C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006299C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000629FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062A9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ABC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ACC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062ADC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062AFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B00 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B04 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B08 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B0C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B10 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B14 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B1C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B20 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B24 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B28 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B2C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B30 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B34 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B38 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B3C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B40 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B44 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B48 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B4C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B50 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B54 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B58 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B5C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B60 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B64 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B68 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B6C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B70 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B74 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B78 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B7C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B80 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B84 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B88 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B8C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B90 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B94 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B98 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062B9C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BA8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BAC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BB8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BBC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BC8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BCC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BD8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BDC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BE8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BEC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BF8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062BFC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00062C00 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C04 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C08 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C0C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C10 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C14 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C18 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C1C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C20 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C24 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C28 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C2C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C30 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C34 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C38 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C3C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C40 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C44 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C48 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C4C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C50 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C54 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C58 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C5C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C60 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C64 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C68 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C6C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C70 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C74 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C78 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C7C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C80 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C84 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C88 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C8C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C90 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C94 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C98 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062C9C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CA8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CAC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CB8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00062CBC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006300C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006301C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006302C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006303C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006304C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006305C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006306C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006307C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006308C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006309C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000630FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006310C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006311C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006312C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006313C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006314C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006315C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006316C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006317C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006318C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006319C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000631FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006320C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006321C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006322C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006323C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006324C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006325C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006326C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006327C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006328C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006329C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000632FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006330C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006331C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006332C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006333C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006334C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006335C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006336C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006337C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006338C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x0006339C ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633A8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633AC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633B8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633BC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633C8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633CC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633D8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633DC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633E8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633EC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F0 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F4 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633F8 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x000633FC ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006340C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006341C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006342C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006343C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006344C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006345C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006346C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006347C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006348C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006349C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000634FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006350C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006351C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006352C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006353C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006354C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006355C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006356C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006357C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006358C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006359C ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635A8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635AC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635B8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635BC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635C8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635CC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635D8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635DC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635E8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635EC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F4 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635F8 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x000635FC ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x0006400C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x0006401C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x0006402C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x0006403C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
	*mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
	*mkptr( dfe_cfg_base, 0x0006404C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
	*mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
	*mkptr( dfe_cfg_base, 0x0006405C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
	*mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001A00;
	*mkptr( dfe_cfg_base, 0x0006406C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001C00;
	*mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001E00;
	*mkptr( dfe_cfg_base, 0x0006407C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
	*mkptr( dfe_cfg_base, 0x0006408C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
	*mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
	*mkptr( dfe_cfg_base, 0x0006409C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640A0 ) = 0x00002800;
	*mkptr( dfe_cfg_base, 0x000640A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640A8 ) = 0x00002A00;
	*mkptr( dfe_cfg_base, 0x000640AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640B0 ) = 0x00002C00;
	*mkptr( dfe_cfg_base, 0x000640B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640B8 ) = 0x00002E00;
	*mkptr( dfe_cfg_base, 0x000640BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640C0 ) = 0x00003000;
	*mkptr( dfe_cfg_base, 0x000640C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640C8 ) = 0x00003200;
	*mkptr( dfe_cfg_base, 0x000640CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640D0 ) = 0x00003400;
	*mkptr( dfe_cfg_base, 0x000640D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640D8 ) = 0x00003600;
	*mkptr( dfe_cfg_base, 0x000640DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640E0 ) = 0x00003800;
	*mkptr( dfe_cfg_base, 0x000640E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640E8 ) = 0x00003A00;
	*mkptr( dfe_cfg_base, 0x000640EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640F0 ) = 0x00003C00;
	*mkptr( dfe_cfg_base, 0x000640F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000640F8 ) = 0x00003E00;
	*mkptr( dfe_cfg_base, 0x000640FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
	*mkptr( dfe_cfg_base, 0x0006410C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
	*mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
	*mkptr( dfe_cfg_base, 0x0006411C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
	*mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004A00;
	*mkptr( dfe_cfg_base, 0x0006412C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004C00;
	*mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004E00;
	*mkptr( dfe_cfg_base, 0x0006413C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
	*mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
	*mkptr( dfe_cfg_base, 0x0006414C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
	*mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
	*mkptr( dfe_cfg_base, 0x0006415C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
	*mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005A00;
	*mkptr( dfe_cfg_base, 0x0006416C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005C00;
	*mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005E00;
	*mkptr( dfe_cfg_base, 0x0006417C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
	*mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
	*mkptr( dfe_cfg_base, 0x0006418C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
	*mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
	*mkptr( dfe_cfg_base, 0x0006419C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641A0 ) = 0x00006800;
	*mkptr( dfe_cfg_base, 0x000641A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641A8 ) = 0x00006A00;
	*mkptr( dfe_cfg_base, 0x000641AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641B0 ) = 0x00006C00;
	*mkptr( dfe_cfg_base, 0x000641B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641B8 ) = 0x00006E00;
	*mkptr( dfe_cfg_base, 0x000641BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641C0 ) = 0x00007000;
	*mkptr( dfe_cfg_base, 0x000641C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641C8 ) = 0x00007200;
	*mkptr( dfe_cfg_base, 0x000641CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641D0 ) = 0x00007400;
	*mkptr( dfe_cfg_base, 0x000641D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641D8 ) = 0x00007600;
	*mkptr( dfe_cfg_base, 0x000641DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641E0 ) = 0x00007800;
	*mkptr( dfe_cfg_base, 0x000641E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641E8 ) = 0x00007A00;
	*mkptr( dfe_cfg_base, 0x000641EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641F0 ) = 0x00007C00;
	*mkptr( dfe_cfg_base, 0x000641F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000641F8 ) = 0x00007E00;
	*mkptr( dfe_cfg_base, 0x000641FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
	*mkptr( dfe_cfg_base, 0x0006420C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
	*mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
	*mkptr( dfe_cfg_base, 0x0006421C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
	*mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008A00;
	*mkptr( dfe_cfg_base, 0x0006422C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008C00;
	*mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008E00;
	*mkptr( dfe_cfg_base, 0x0006423C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
	*mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
	*mkptr( dfe_cfg_base, 0x0006424C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
	*mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
	*mkptr( dfe_cfg_base, 0x0006425C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
	*mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009A00;
	*mkptr( dfe_cfg_base, 0x0006426C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009C00;
	*mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009E00;
	*mkptr( dfe_cfg_base, 0x0006427C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000A000;
	*mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000A200;
	*mkptr( dfe_cfg_base, 0x0006428C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000A400;
	*mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000A600;
	*mkptr( dfe_cfg_base, 0x0006429C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642A0 ) = 0x0000A800;
	*mkptr( dfe_cfg_base, 0x000642A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642A8 ) = 0x0000AA00;
	*mkptr( dfe_cfg_base, 0x000642AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642B0 ) = 0x0000AC00;
	*mkptr( dfe_cfg_base, 0x000642B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642B8 ) = 0x0000AE00;
	*mkptr( dfe_cfg_base, 0x000642BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642C0 ) = 0x0000B000;
	*mkptr( dfe_cfg_base, 0x000642C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642C8 ) = 0x0000B200;
	*mkptr( dfe_cfg_base, 0x000642CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642D0 ) = 0x0000B400;
	*mkptr( dfe_cfg_base, 0x000642D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642D8 ) = 0x0000B600;
	*mkptr( dfe_cfg_base, 0x000642DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642E0 ) = 0x0000B800;
	*mkptr( dfe_cfg_base, 0x000642E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642E8 ) = 0x0000BA00;
	*mkptr( dfe_cfg_base, 0x000642EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642F0 ) = 0x0000BC00;
	*mkptr( dfe_cfg_base, 0x000642F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000642F8 ) = 0x0000BE00;
	*mkptr( dfe_cfg_base, 0x000642FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000C000;
	*mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000C200;
	*mkptr( dfe_cfg_base, 0x0006430C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000C400;
	*mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000C600;
	*mkptr( dfe_cfg_base, 0x0006431C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000C800;
	*mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000CA00;
	*mkptr( dfe_cfg_base, 0x0006432C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000CC00;
	*mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000CE00;
	*mkptr( dfe_cfg_base, 0x0006433C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000D000;
	*mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000D200;
	*mkptr( dfe_cfg_base, 0x0006434C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000D400;
	*mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000D600;
	*mkptr( dfe_cfg_base, 0x0006435C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000D800;
	*mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000DA00;
	*mkptr( dfe_cfg_base, 0x0006436C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000DC00;
	*mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000DE00;
	*mkptr( dfe_cfg_base, 0x0006437C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000E000;
	*mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000E200;
	*mkptr( dfe_cfg_base, 0x0006438C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000E400;
	*mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000E600;
	*mkptr( dfe_cfg_base, 0x0006439C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643A0 ) = 0x0000E800;
	*mkptr( dfe_cfg_base, 0x000643A4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643A8 ) = 0x0000EA00;
	*mkptr( dfe_cfg_base, 0x000643AC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643B0 ) = 0x0000EC00;
	*mkptr( dfe_cfg_base, 0x000643B4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643B8 ) = 0x0000EE00;
	*mkptr( dfe_cfg_base, 0x000643BC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643C0 ) = 0x0000F000;
	*mkptr( dfe_cfg_base, 0x000643C4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643C8 ) = 0x0000F200;
	*mkptr( dfe_cfg_base, 0x000643CC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643D0 ) = 0x0000F400;
	*mkptr( dfe_cfg_base, 0x000643D4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643D8 ) = 0x0000F600;
	*mkptr( dfe_cfg_base, 0x000643DC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643E0 ) = 0x0000F800;
	*mkptr( dfe_cfg_base, 0x000643E4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643E8 ) = 0x0000FA00;
	*mkptr( dfe_cfg_base, 0x000643EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643F0 ) = 0x0000FC00;
	*mkptr( dfe_cfg_base, 0x000643F4 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x000643F8 ) = 0x0000FE00;
	*mkptr( dfe_cfg_base, 0x000643FC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0006C000 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C004 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C008 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C00C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C010 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C020 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C024 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C028 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C02C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C030 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C040 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C044 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C048 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C04C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C050 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C060 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C064 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C068 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C06C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C070 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C080 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C084 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C088 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C08C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C090 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0A0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0A4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0A8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0AC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0B0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0C0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0C4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0C8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0D0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x0006C0E0 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x0006C0E4 ) = 0x00000FFF;
	*mkptr( dfe_cfg_base, 0x0006C0E8 ) = 0x00000055;
	*mkptr( dfe_cfg_base, 0x0006C0EC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x0006C0F0 ) = 0x0000807F;
	*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000308;
	*mkptr( dfe_cfg_base, 0x00101300 ) = 0x00000307;
	*mkptr( dfe_cfg_base, 0x00101304 ) = 0x00000438;
	*mkptr( dfe_cfg_base, 0x00101308 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x00101318 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x00101324 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140000 ) = 0x0000EAA5;
	*mkptr( dfe_cfg_base, 0x00140004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140008 ) = 0x0000C721;
	*mkptr( dfe_cfg_base, 0x0014000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140010 ) = 0x0000971E;
	*mkptr( dfe_cfg_base, 0x00140018 ) = 0x0000401A;
	*mkptr( dfe_cfg_base, 0x0014001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00140200 ) = 0x000013CE;
	*mkptr( dfe_cfg_base, 0x00140208 ) = 0x0000ED20;
	*mkptr( dfe_cfg_base, 0x0014020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140210 ) = 0x0000F4B0;
	*mkptr( dfe_cfg_base, 0x00140214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140218 ) = 0x000027F2;
	*mkptr( dfe_cfg_base, 0x00140400 ) = 0x0000057A;
	*mkptr( dfe_cfg_base, 0x00140408 ) = 0x00000977;
	*mkptr( dfe_cfg_base, 0x00140410 ) = 0x0000F0C3;
	*mkptr( dfe_cfg_base, 0x00140414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140418 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x00140600 ) = 0x0000F906;
	*mkptr( dfe_cfg_base, 0x00140604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140608 ) = 0x000006B5;
	*mkptr( dfe_cfg_base, 0x00140610 ) = 0x00000318;
	*mkptr( dfe_cfg_base, 0x00140618 ) = 0x0000F4F1;
	*mkptr( dfe_cfg_base, 0x0014061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140800 ) = 0x0000FD84;
	*mkptr( dfe_cfg_base, 0x00140804 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140808 ) = 0x0000FC7B;
	*mkptr( dfe_cfg_base, 0x0014080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140810 ) = 0x0000060C;
	*mkptr( dfe_cfg_base, 0x00140818 ) = 0x0000FF54;
	*mkptr( dfe_cfg_base, 0x0014081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140A00 ) = 0x000002B7;
	*mkptr( dfe_cfg_base, 0x00140A08 ) = 0x0000FD1A;
	*mkptr( dfe_cfg_base, 0x00140A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140A10 ) = 0x0000FEFE;
	*mkptr( dfe_cfg_base, 0x00140A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140A18 ) = 0x0000047B;
	*mkptr( dfe_cfg_base, 0x00140C00 ) = 0x0000011E;
	*mkptr( dfe_cfg_base, 0x00140C08 ) = 0x00000137;
	*mkptr( dfe_cfg_base, 0x00140C10 ) = 0x0000FD8B;
	*mkptr( dfe_cfg_base, 0x00140C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00140E00 ) = 0x0000FF24;
	*mkptr( dfe_cfg_base, 0x00140E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00140E08 ) = 0x0000011F;
	*mkptr( dfe_cfg_base, 0x00140E10 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00140E18 ) = 0x0000FE59;
	*mkptr( dfe_cfg_base, 0x00140E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141000 ) = 0x0000FF97;
	*mkptr( dfe_cfg_base, 0x00141004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141008 ) = 0x0000FFB7;
	*mkptr( dfe_cfg_base, 0x0014100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141010 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x00141018 ) = 0x0000FFB4;
	*mkptr( dfe_cfg_base, 0x0014101C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141200 ) = 0x0000FFC1;
	*mkptr( dfe_cfg_base, 0x00141204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141208 ) = 0x0000FF59;
	*mkptr( dfe_cfg_base, 0x0014120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00141218 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x0010126C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00101344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00101348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0010134C ) = 0x00004300;
	*mkptr( dfe_cfg_base, 0x00101350 ) = 0x00000481;
	*mkptr( dfe_cfg_base, 0x00143800 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143804 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x00143808 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014380C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x00143810 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x00143814 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x00143818 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x0014381C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x001012C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00101200 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00101204 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00101208 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0010120C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00101210 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00101214 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00101218 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0010121C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00101220 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00101224 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0010122C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00101230 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00101234 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0010123C ) = 0x00000243;
	*mkptr( dfe_cfg_base, 0x00101240 ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00101244 ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00101248 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00101250 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00101254 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00101258 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x0010125C ) = 0x00000401;
	*mkptr( dfe_cfg_base, 0x00101260 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00101264 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00101268 ) = 0x00002616;
	*mkptr( dfe_cfg_base, 0x001012DC ) = 0x0000B800;
	*mkptr( dfe_cfg_base, 0x00100400 ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x00181300 ) = 0x00000307;
	*mkptr( dfe_cfg_base, 0x00181304 ) = 0x00000438;
	*mkptr( dfe_cfg_base, 0x00181308 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x00181318 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x00181324 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0000 ) = 0x0000EAA5;
	*mkptr( dfe_cfg_base, 0x001C0004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0008 ) = 0x0000C721;
	*mkptr( dfe_cfg_base, 0x001C000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0010 ) = 0x0000971E;
	*mkptr( dfe_cfg_base, 0x001C0018 ) = 0x0000401A;
	*mkptr( dfe_cfg_base, 0x001C001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x001C0200 ) = 0x000013CE;
	*mkptr( dfe_cfg_base, 0x001C0208 ) = 0x0000ED20;
	*mkptr( dfe_cfg_base, 0x001C020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0210 ) = 0x0000F4B0;
	*mkptr( dfe_cfg_base, 0x001C0214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0218 ) = 0x000027F2;
	*mkptr( dfe_cfg_base, 0x001C0400 ) = 0x0000057A;
	*mkptr( dfe_cfg_base, 0x001C0408 ) = 0x00000977;
	*mkptr( dfe_cfg_base, 0x001C0410 ) = 0x0000F0C3;
	*mkptr( dfe_cfg_base, 0x001C0414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0418 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x001C0600 ) = 0x0000F906;
	*mkptr( dfe_cfg_base, 0x001C0604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0608 ) = 0x000006B5;
	*mkptr( dfe_cfg_base, 0x001C0610 ) = 0x00000318;
	*mkptr( dfe_cfg_base, 0x001C0618 ) = 0x0000F4F1;
	*mkptr( dfe_cfg_base, 0x001C061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0800 ) = 0x0000FD84;
	*mkptr( dfe_cfg_base, 0x001C0804 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0808 ) = 0x0000FC7B;
	*mkptr( dfe_cfg_base, 0x001C080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0810 ) = 0x0000060C;
	*mkptr( dfe_cfg_base, 0x001C0818 ) = 0x0000FF54;
	*mkptr( dfe_cfg_base, 0x001C081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0A00 ) = 0x000002B7;
	*mkptr( dfe_cfg_base, 0x001C0A08 ) = 0x0000FD1A;
	*mkptr( dfe_cfg_base, 0x001C0A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0A10 ) = 0x0000FEFE;
	*mkptr( dfe_cfg_base, 0x001C0A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0A18 ) = 0x0000047B;
	*mkptr( dfe_cfg_base, 0x001C0C00 ) = 0x0000011E;
	*mkptr( dfe_cfg_base, 0x001C0C08 ) = 0x00000137;
	*mkptr( dfe_cfg_base, 0x001C0C10 ) = 0x0000FD8B;
	*mkptr( dfe_cfg_base, 0x001C0C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x001C0E00 ) = 0x0000FF24;
	*mkptr( dfe_cfg_base, 0x001C0E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C0E08 ) = 0x0000011F;
	*mkptr( dfe_cfg_base, 0x001C0E10 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x001C0E18 ) = 0x0000FE59;
	*mkptr( dfe_cfg_base, 0x001C0E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1000 ) = 0x0000FF97;
	*mkptr( dfe_cfg_base, 0x001C1004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1008 ) = 0x0000FFB7;
	*mkptr( dfe_cfg_base, 0x001C100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1010 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x001C1018 ) = 0x0000FFB4;
	*mkptr( dfe_cfg_base, 0x001C101C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1200 ) = 0x0000FFC1;
	*mkptr( dfe_cfg_base, 0x001C1204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1208 ) = 0x0000FF59;
	*mkptr( dfe_cfg_base, 0x001C120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x001C1218 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x0018126C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00181344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00181348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0018134C ) = 0x00004300;
	*mkptr( dfe_cfg_base, 0x00181350 ) = 0x00000481;
	*mkptr( dfe_cfg_base, 0x001C3800 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3804 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x001C3808 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C380C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x001C3810 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C3814 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x001C3818 ) = 0x0000332F;
	*mkptr( dfe_cfg_base, 0x001C381C ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x001812C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00181200 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00181204 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00181208 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0018120C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00181210 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00181214 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00181218 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0018121C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00181220 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00181224 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0018122C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00181230 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00181234 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0018123C ) = 0x00000243;
	*mkptr( dfe_cfg_base, 0x00181240 ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00181244 ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00181248 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00181250 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00181254 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00181258 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x0018125C ) = 0x00000401;
	*mkptr( dfe_cfg_base, 0x00181260 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00181264 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00181268 ) = 0x00002616;
	*mkptr( dfe_cfg_base, 0x001812DC ) = 0x0000B800;
	*mkptr( dfe_cfg_base, 0x00180400 ) = 0x00004080;
	*mkptr( dfe_cfg_base, 0x00201300 ) = 0x00000307;
	*mkptr( dfe_cfg_base, 0x00201304 ) = 0x00000438;
	*mkptr( dfe_cfg_base, 0x00201308 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00201318 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x00201324 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240000 ) = 0x0000EAA5;
	*mkptr( dfe_cfg_base, 0x00240004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240008 ) = 0x0000C721;
	*mkptr( dfe_cfg_base, 0x0024000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240010 ) = 0x0000971E;
	*mkptr( dfe_cfg_base, 0x00240018 ) = 0x0000401A;
	*mkptr( dfe_cfg_base, 0x0024001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00240200 ) = 0x000013CE;
	*mkptr( dfe_cfg_base, 0x00240208 ) = 0x0000ED20;
	*mkptr( dfe_cfg_base, 0x0024020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240210 ) = 0x0000F4B0;
	*mkptr( dfe_cfg_base, 0x00240214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240218 ) = 0x000027F2;
	*mkptr( dfe_cfg_base, 0x00240400 ) = 0x0000057A;
	*mkptr( dfe_cfg_base, 0x00240408 ) = 0x00000977;
	*mkptr( dfe_cfg_base, 0x00240410 ) = 0x0000F0C3;
	*mkptr( dfe_cfg_base, 0x00240414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240418 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x00240600 ) = 0x0000F906;
	*mkptr( dfe_cfg_base, 0x00240604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240608 ) = 0x000006B5;
	*mkptr( dfe_cfg_base, 0x00240610 ) = 0x00000318;
	*mkptr( dfe_cfg_base, 0x00240618 ) = 0x0000F4F1;
	*mkptr( dfe_cfg_base, 0x0024061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240800 ) = 0x0000FD84;
	*mkptr( dfe_cfg_base, 0x00240804 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240808 ) = 0x0000FC7B;
	*mkptr( dfe_cfg_base, 0x0024080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240810 ) = 0x0000060C;
	*mkptr( dfe_cfg_base, 0x00240818 ) = 0x0000FF54;
	*mkptr( dfe_cfg_base, 0x0024081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240A00 ) = 0x000002B7;
	*mkptr( dfe_cfg_base, 0x00240A08 ) = 0x0000FD1A;
	*mkptr( dfe_cfg_base, 0x00240A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240A10 ) = 0x0000FEFE;
	*mkptr( dfe_cfg_base, 0x00240A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240A18 ) = 0x0000047B;
	*mkptr( dfe_cfg_base, 0x00240C00 ) = 0x0000011E;
	*mkptr( dfe_cfg_base, 0x00240C08 ) = 0x00000137;
	*mkptr( dfe_cfg_base, 0x00240C10 ) = 0x0000FD8B;
	*mkptr( dfe_cfg_base, 0x00240C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x00240E00 ) = 0x0000FF24;
	*mkptr( dfe_cfg_base, 0x00240E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00240E08 ) = 0x0000011F;
	*mkptr( dfe_cfg_base, 0x00240E10 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x00240E18 ) = 0x0000FE59;
	*mkptr( dfe_cfg_base, 0x00240E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241000 ) = 0x0000FF97;
	*mkptr( dfe_cfg_base, 0x00241004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241008 ) = 0x0000FFB7;
	*mkptr( dfe_cfg_base, 0x0024100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241010 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x00241018 ) = 0x0000FFB4;
	*mkptr( dfe_cfg_base, 0x0024101C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241200 ) = 0x0000FFC1;
	*mkptr( dfe_cfg_base, 0x00241204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241208 ) = 0x0000FF59;
	*mkptr( dfe_cfg_base, 0x0024120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00241218 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x0020126C ) = 0x00000216;
	*mkptr( dfe_cfg_base, 0x00201344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00201348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0020134C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00201350 ) = 0x00000481;
	*mkptr( dfe_cfg_base, 0x00243800 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243804 ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x00243808 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024380C ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x00243810 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x00243814 ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x00243818 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x0024381C ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x002012C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00201200 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00201204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00201208 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0020120C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00201210 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00201214 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00201218 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x0020121C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00201220 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00201224 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00201228 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x0020122C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00201230 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00201234 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0020123C ) = 0x00000243;
	*mkptr( dfe_cfg_base, 0x00201240 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00201244 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00201248 ) = 0x00000401;
	*mkptr( dfe_cfg_base, 0x00201250 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00201254 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00201258 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x0020125C ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00201260 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00201264 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00201268 ) = 0x00001606;
	*mkptr( dfe_cfg_base, 0x002012DC ) = 0x0000C800;
	*mkptr( dfe_cfg_base, 0x002012E4 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x00281300 ) = 0x00000307;
	*mkptr( dfe_cfg_base, 0x00281304 ) = 0x00000438;
	*mkptr( dfe_cfg_base, 0x00281308 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00281318 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x00281324 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0000 ) = 0x0000EAA5;
	*mkptr( dfe_cfg_base, 0x002C0004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0008 ) = 0x0000C721;
	*mkptr( dfe_cfg_base, 0x002C000C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0010 ) = 0x0000971E;
	*mkptr( dfe_cfg_base, 0x002C0018 ) = 0x0000401A;
	*mkptr( dfe_cfg_base, 0x002C001C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x002C0200 ) = 0x000013CE;
	*mkptr( dfe_cfg_base, 0x002C0208 ) = 0x0000ED20;
	*mkptr( dfe_cfg_base, 0x002C020C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0210 ) = 0x0000F4B0;
	*mkptr( dfe_cfg_base, 0x002C0214 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0218 ) = 0x000027F2;
	*mkptr( dfe_cfg_base, 0x002C0400 ) = 0x0000057A;
	*mkptr( dfe_cfg_base, 0x002C0408 ) = 0x00000977;
	*mkptr( dfe_cfg_base, 0x002C0410 ) = 0x0000F0C3;
	*mkptr( dfe_cfg_base, 0x002C0414 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0418 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x002C0600 ) = 0x0000F906;
	*mkptr( dfe_cfg_base, 0x002C0604 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0608 ) = 0x000006B5;
	*mkptr( dfe_cfg_base, 0x002C0610 ) = 0x00000318;
	*mkptr( dfe_cfg_base, 0x002C0618 ) = 0x0000F4F1;
	*mkptr( dfe_cfg_base, 0x002C061C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0800 ) = 0x0000FD84;
	*mkptr( dfe_cfg_base, 0x002C0804 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0808 ) = 0x0000FC7B;
	*mkptr( dfe_cfg_base, 0x002C080C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0810 ) = 0x0000060C;
	*mkptr( dfe_cfg_base, 0x002C0818 ) = 0x0000FF54;
	*mkptr( dfe_cfg_base, 0x002C081C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0A00 ) = 0x000002B7;
	*mkptr( dfe_cfg_base, 0x002C0A08 ) = 0x0000FD1A;
	*mkptr( dfe_cfg_base, 0x002C0A0C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0A10 ) = 0x0000FEFE;
	*mkptr( dfe_cfg_base, 0x002C0A14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0A18 ) = 0x0000047B;
	*mkptr( dfe_cfg_base, 0x002C0C00 ) = 0x0000011E;
	*mkptr( dfe_cfg_base, 0x002C0C08 ) = 0x00000137;
	*mkptr( dfe_cfg_base, 0x002C0C10 ) = 0x0000FD8B;
	*mkptr( dfe_cfg_base, 0x002C0C14 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0C18 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x002C0E00 ) = 0x0000FF24;
	*mkptr( dfe_cfg_base, 0x002C0E04 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C0E08 ) = 0x0000011F;
	*mkptr( dfe_cfg_base, 0x002C0E10 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x002C0E18 ) = 0x0000FE59;
	*mkptr( dfe_cfg_base, 0x002C0E1C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1000 ) = 0x0000FF97;
	*mkptr( dfe_cfg_base, 0x002C1004 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1008 ) = 0x0000FFB7;
	*mkptr( dfe_cfg_base, 0x002C100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1010 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x002C1018 ) = 0x0000FFB4;
	*mkptr( dfe_cfg_base, 0x002C101C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1200 ) = 0x0000FFC1;
	*mkptr( dfe_cfg_base, 0x002C1204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1208 ) = 0x0000FF59;
	*mkptr( dfe_cfg_base, 0x002C120C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x002C1218 ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x0028126C ) = 0x00000216;
	*mkptr( dfe_cfg_base, 0x00281344 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00281348 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x0028134C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00281350 ) = 0x00000481;
	*mkptr( dfe_cfg_base, 0x002C3800 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3804 ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x002C3808 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C380C ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x002C3810 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C3814 ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x002C3818 ) = 0x0000CCD1;
	*mkptr( dfe_cfg_base, 0x002C381C ) = 0x000001F8;
	*mkptr( dfe_cfg_base, 0x002812C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00281200 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00281204 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00281208 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x0028120C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00281210 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00281214 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00281218 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x0028121C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00281220 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x00281224 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00281228 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x0028122C ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00281230 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00281234 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x0028123C ) = 0x00000243;
	*mkptr( dfe_cfg_base, 0x00281240 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00281244 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00281248 ) = 0x00000401;
	*mkptr( dfe_cfg_base, 0x00281250 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00281254 ) = 0x00000402;
	*mkptr( dfe_cfg_base, 0x00281258 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x0028125C ) = 0x00000403;
	*mkptr( dfe_cfg_base, 0x00281260 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00281264 ) = 0x00000303;
	*mkptr( dfe_cfg_base, 0x00281268 ) = 0x00001606;
	*mkptr( dfe_cfg_base, 0x002812DC ) = 0x0000C800;
	*mkptr( dfe_cfg_base, 0x002812E4 ) = 0x00000041;
	*mkptr( dfe_cfg_base, 0x00900004 ) = 0x00000606;
	*mkptr( dfe_cfg_base, 0x00900018 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00900030 ) = 0x00001000;
	*mkptr( dfe_cfg_base, 0x00B003CC ) = 0x00008006;
	*mkptr( dfe_cfg_base, 0x00B00208 ) = 0x000008C0;
	*mkptr( dfe_cfg_base, 0x00B00288 ) = 0x000008C0;
	*mkptr( dfe_cfg_base, 0x00B00360 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x00B00368 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00B00370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B00200 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00280 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00210 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B00218 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B00230 ) = 0x00004040;
	*mkptr( dfe_cfg_base, 0x00B00290 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B00298 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B002B0 ) = 0x00004040;
	*mkptr( dfe_cfg_base, 0x00B00300 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B00308 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B00310 ) = 0x00003F5C;
	*mkptr( dfe_cfg_base, 0x00B00318 ) = 0x00003F5C;
	*mkptr( dfe_cfg_base, 0x00B00320 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B00328 ) = 0x00000174;
	*mkptr( dfe_cfg_base, 0x00B00338 ) = 0x00000BB8;
	*mkptr( dfe_cfg_base, 0x00B00340 ) = 0x00000363;
	*mkptr( dfe_cfg_base, 0x00B00348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00B00378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B18004 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1800C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18014 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1801C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18024 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1802C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18034 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1803C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18044 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1804C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18054 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1805C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18064 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1806C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18074 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B1807C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B18080 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x00B18084 ) = 0x00007FE8;
	*mkptr( dfe_cfg_base, 0x00B18088 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x00B1808C ) = 0x00007C17;
	*mkptr( dfe_cfg_base, 0x00B18090 ) = 0x00000334;
	*mkptr( dfe_cfg_base, 0x00B18094 ) = 0x00007899;
	*mkptr( dfe_cfg_base, 0x00B18098 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00B1809C ) = 0x00007562;
	*mkptr( dfe_cfg_base, 0x00B180A0 ) = 0x00000350;
	*mkptr( dfe_cfg_base, 0x00B180A4 ) = 0x0000726A;
	*mkptr( dfe_cfg_base, 0x00B180A8 ) = 0x0000035C;
	*mkptr( dfe_cfg_base, 0x00B180AC ) = 0x00006FA8;
	*mkptr( dfe_cfg_base, 0x00B180B0 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x00B180B4 ) = 0x00006D17;
	*mkptr( dfe_cfg_base, 0x00B180B8 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00B180BC ) = 0x00006AB2;
	*mkptr( dfe_cfg_base, 0x00B180C0 ) = 0x00000378;
	*mkptr( dfe_cfg_base, 0x00B180C4 ) = 0x00006873;
	*mkptr( dfe_cfg_base, 0x00B180C8 ) = 0x00000380;
	*mkptr( dfe_cfg_base, 0x00B180CC ) = 0x00006657;
	*mkptr( dfe_cfg_base, 0x00B180D0 ) = 0x00000388;
	*mkptr( dfe_cfg_base, 0x00B180D4 ) = 0x0000645A;
	*mkptr( dfe_cfg_base, 0x00B180D8 ) = 0x00000390;
	*mkptr( dfe_cfg_base, 0x00B180DC ) = 0x0000627A;
	*mkptr( dfe_cfg_base, 0x00B180E0 ) = 0x00000394;
	*mkptr( dfe_cfg_base, 0x00B180E4 ) = 0x000060B4;
	*mkptr( dfe_cfg_base, 0x00B180E8 ) = 0x00000398;
	*mkptr( dfe_cfg_base, 0x00B180EC ) = 0x00005F06;
	*mkptr( dfe_cfg_base, 0x00B180F0 ) = 0x000003A0;
	*mkptr( dfe_cfg_base, 0x00B180F4 ) = 0x00005D6D;
	*mkptr( dfe_cfg_base, 0x00B180F8 ) = 0x000003A4;
	*mkptr( dfe_cfg_base, 0x00B180FC ) = 0x00005BE8;
	*mkptr( dfe_cfg_base, 0x00B18100 ) = 0x000003A8;
	*mkptr( dfe_cfg_base, 0x00B18104 ) = 0x00005A76;
	*mkptr( dfe_cfg_base, 0x00B18108 ) = 0x000003AC;
	*mkptr( dfe_cfg_base, 0x00B1810C ) = 0x00005914;
	*mkptr( dfe_cfg_base, 0x00B18110 ) = 0x000003B0;
	*mkptr( dfe_cfg_base, 0x00B18114 ) = 0x000057C2;
	*mkptr( dfe_cfg_base, 0x00B18118 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B1811C ) = 0x0000567F;
	*mkptr( dfe_cfg_base, 0x00B18120 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B18124 ) = 0x00005549;
	*mkptr( dfe_cfg_base, 0x00B18128 ) = 0x000003B8;
	*mkptr( dfe_cfg_base, 0x00B1812C ) = 0x00005420;
	*mkptr( dfe_cfg_base, 0x00B18130 ) = 0x000003BC;
	*mkptr( dfe_cfg_base, 0x00B18134 ) = 0x00005303;
	*mkptr( dfe_cfg_base, 0x00B18138 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B1813C ) = 0x000051F1;
	*mkptr( dfe_cfg_base, 0x00B18140 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B18144 ) = 0x000050E9;
	*mkptr( dfe_cfg_base, 0x00B18148 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B1814C ) = 0x00004FEB;
	*mkptr( dfe_cfg_base, 0x00B18150 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B18154 ) = 0x00004EF6;
	*mkptr( dfe_cfg_base, 0x00B18158 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B1815C ) = 0x00004E0A;
	*mkptr( dfe_cfg_base, 0x00B18160 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B18164 ) = 0x00004D25;
	*mkptr( dfe_cfg_base, 0x00B18168 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1816C ) = 0x00004C49;
	*mkptr( dfe_cfg_base, 0x00B18170 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B18174 ) = 0x00004B73;
	*mkptr( dfe_cfg_base, 0x00B18178 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B1817C ) = 0x00004AA5;
	*mkptr( dfe_cfg_base, 0x00B18180 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B18184 ) = 0x000049DD;
	*mkptr( dfe_cfg_base, 0x00B18188 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B1818C ) = 0x0000491B;
	*mkptr( dfe_cfg_base, 0x00B18190 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B18194 ) = 0x0000485F;
	*mkptr( dfe_cfg_base, 0x00B18198 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B1819C ) = 0x000047A8;
	*mkptr( dfe_cfg_base, 0x00B181A0 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B181A4 ) = 0x000046F7;
	*mkptr( dfe_cfg_base, 0x00B181A8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181AC ) = 0x0000464B;
	*mkptr( dfe_cfg_base, 0x00B181B0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181B4 ) = 0x000045A3;
	*mkptr( dfe_cfg_base, 0x00B181B8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181BC ) = 0x00004501;
	*mkptr( dfe_cfg_base, 0x00B181C0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B181C4 ) = 0x00004462;
	*mkptr( dfe_cfg_base, 0x00B181C8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181CC ) = 0x000043C8;
	*mkptr( dfe_cfg_base, 0x00B181D0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181D4 ) = 0x00004332;
	*mkptr( dfe_cfg_base, 0x00B181D8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181DC ) = 0x0000429F;
	*mkptr( dfe_cfg_base, 0x00B181E0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B181E4 ) = 0x00004211;
	*mkptr( dfe_cfg_base, 0x00B181E8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181EC ) = 0x00004185;
	*mkptr( dfe_cfg_base, 0x00B181F0 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181F4 ) = 0x000040FE;
	*mkptr( dfe_cfg_base, 0x00B181F8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B181FC ) = 0x00004079;
	*mkptr( dfe_cfg_base, 0x00B18200 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18204 ) = 0x00003FF8;
	*mkptr( dfe_cfg_base, 0x00B18208 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B1820C ) = 0x00003F79;
	*mkptr( dfe_cfg_base, 0x00B18210 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B18214 ) = 0x00003EFE;
	*mkptr( dfe_cfg_base, 0x00B18218 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1821C ) = 0x00003E85;
	*mkptr( dfe_cfg_base, 0x00B18220 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18224 ) = 0x00003E0F;
	*mkptr( dfe_cfg_base, 0x00B18228 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1822C ) = 0x00003D9B;
	*mkptr( dfe_cfg_base, 0x00B18230 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18234 ) = 0x00003D2A;
	*mkptr( dfe_cfg_base, 0x00B18238 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B1823C ) = 0x00003CBB;
	*mkptr( dfe_cfg_base, 0x00B18240 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B18244 ) = 0x00003C4F;
	*mkptr( dfe_cfg_base, 0x00B18248 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1824C ) = 0x00003BE5;
	*mkptr( dfe_cfg_base, 0x00B18250 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18254 ) = 0x00003B7D;
	*mkptr( dfe_cfg_base, 0x00B18258 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1825C ) = 0x00003B17;
	*mkptr( dfe_cfg_base, 0x00B18260 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18264 ) = 0x00003AB3;
	*mkptr( dfe_cfg_base, 0x00B18268 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1826C ) = 0x00003A51;
	*mkptr( dfe_cfg_base, 0x00B18270 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18274 ) = 0x000039F1;
	*mkptr( dfe_cfg_base, 0x00B18278 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1827C ) = 0x00003993;
	*mkptr( dfe_cfg_base, 0x00B18280 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B18284 ) = 0x00003937;
	*mkptr( dfe_cfg_base, 0x00B18288 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B1828C ) = 0x000038DC;
	*mkptr( dfe_cfg_base, 0x00B18290 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B18294 ) = 0x00003883;
	*mkptr( dfe_cfg_base, 0x00B18298 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B1829C ) = 0x0000382C;
	*mkptr( dfe_cfg_base, 0x00B182A0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182A4 ) = 0x000037D6;
	*mkptr( dfe_cfg_base, 0x00B182A8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182AC ) = 0x00003782;
	*mkptr( dfe_cfg_base, 0x00B182B0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182B4 ) = 0x0000372F;
	*mkptr( dfe_cfg_base, 0x00B182B8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182BC ) = 0x000036DD;
	*mkptr( dfe_cfg_base, 0x00B182C0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182C4 ) = 0x0000368D;
	*mkptr( dfe_cfg_base, 0x00B182C8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182CC ) = 0x0000363F;
	*mkptr( dfe_cfg_base, 0x00B182D0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182D4 ) = 0x000035F1;
	*mkptr( dfe_cfg_base, 0x00B182D8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182DC ) = 0x000035A5;
	*mkptr( dfe_cfg_base, 0x00B182E0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182E4 ) = 0x0000355A;
	*mkptr( dfe_cfg_base, 0x00B182E8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B182EC ) = 0x00003511;
	*mkptr( dfe_cfg_base, 0x00B182F0 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B182F4 ) = 0x000034C8;
	*mkptr( dfe_cfg_base, 0x00B182F8 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B182FC ) = 0x00003481;
	*mkptr( dfe_cfg_base, 0x00B18300 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18304 ) = 0x0000343B;
	*mkptr( dfe_cfg_base, 0x00B18308 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1830C ) = 0x000033F6;
	*mkptr( dfe_cfg_base, 0x00B18310 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18314 ) = 0x000033B2;
	*mkptr( dfe_cfg_base, 0x00B18318 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1831C ) = 0x0000336F;
	*mkptr( dfe_cfg_base, 0x00B18320 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18324 ) = 0x0000332D;
	*mkptr( dfe_cfg_base, 0x00B18328 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1832C ) = 0x000032EC;
	*mkptr( dfe_cfg_base, 0x00B18330 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18334 ) = 0x000032AC;
	*mkptr( dfe_cfg_base, 0x00B18338 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1833C ) = 0x0000326C;
	*mkptr( dfe_cfg_base, 0x00B18340 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18344 ) = 0x0000322E;
	*mkptr( dfe_cfg_base, 0x00B18348 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1834C ) = 0x000031F1;
	*mkptr( dfe_cfg_base, 0x00B18350 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18354 ) = 0x000031B4;
	*mkptr( dfe_cfg_base, 0x00B18358 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1835C ) = 0x00003179;
	*mkptr( dfe_cfg_base, 0x00B18360 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18364 ) = 0x0000313E;
	*mkptr( dfe_cfg_base, 0x00B18368 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B1836C ) = 0x00003104;
	*mkptr( dfe_cfg_base, 0x00B18370 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B18374 ) = 0x000030CB;
	*mkptr( dfe_cfg_base, 0x00B18378 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1837C ) = 0x00003093;
	*mkptr( dfe_cfg_base, 0x00B18380 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18384 ) = 0x0000305B;
	*mkptr( dfe_cfg_base, 0x00B18388 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1838C ) = 0x00003024;
	*mkptr( dfe_cfg_base, 0x00B18390 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B18394 ) = 0x00002FEE;
	*mkptr( dfe_cfg_base, 0x00B18398 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B1839C ) = 0x00002FB8;
	*mkptr( dfe_cfg_base, 0x00B183A0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183A4 ) = 0x00002F84;
	*mkptr( dfe_cfg_base, 0x00B183A8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183AC ) = 0x00002F50;
	*mkptr( dfe_cfg_base, 0x00B183B0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183B4 ) = 0x00002F1C;
	*mkptr( dfe_cfg_base, 0x00B183B8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183BC ) = 0x00002EE9;
	*mkptr( dfe_cfg_base, 0x00B183C0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183C4 ) = 0x00002EB7;
	*mkptr( dfe_cfg_base, 0x00B183C8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183CC ) = 0x00002E86;
	*mkptr( dfe_cfg_base, 0x00B183D0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183D4 ) = 0x00002E55;
	*mkptr( dfe_cfg_base, 0x00B183D8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183DC ) = 0x00002E24;
	*mkptr( dfe_cfg_base, 0x00B183E0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183E4 ) = 0x00002DF5;
	*mkptr( dfe_cfg_base, 0x00B183E8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183EC ) = 0x00002DC6;
	*mkptr( dfe_cfg_base, 0x00B183F0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183F4 ) = 0x00002D97;
	*mkptr( dfe_cfg_base, 0x00B183F8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B183FC ) = 0x00002D69;
	*mkptr( dfe_cfg_base, 0x00B10100 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10108 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1010C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10110 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10118 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1011C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10120 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10128 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1012C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10130 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B10134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B10138 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B1013C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B1500C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x00B15014 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00B1501C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00B15024 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00B1502C ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x00B15034 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00B1503C ) = 0x00000700;
	*mkptr( dfe_cfg_base, 0x00B15044 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00B1504C ) = 0x00000900;
	*mkptr( dfe_cfg_base, 0x00B15054 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x00B1505C ) = 0x00000B00;
	*mkptr( dfe_cfg_base, 0x00B15064 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00B1506C ) = 0x00000D00;
	*mkptr( dfe_cfg_base, 0x00B15074 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x00B1507C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00B14000 ) = 0x00002D6A;
	*mkptr( dfe_cfg_base, 0x00B14004 ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14008 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x00B1400C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14010 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B14014 ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14018 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B1401C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B14020 ) = 0x00000808;
	*mkptr( dfe_cfg_base, 0x00B14024 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x00B14040 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14044 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14048 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1404C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14050 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14054 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14058 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1405C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14060 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14064 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14080 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14084 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14088 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1408C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14090 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14094 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14098 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1409C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B140E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B140E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14100 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14104 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14108 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1410C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14110 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14114 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14118 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1411C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14120 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14124 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14140 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14144 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14148 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1414C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14150 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14154 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14158 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1415C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14160 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14164 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14180 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14184 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14188 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1418C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14190 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14194 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14198 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1419C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B141E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B141E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14200 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14204 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14208 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1420C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14210 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14214 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14218 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1421C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14220 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14224 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14240 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14244 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14248 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1424C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14250 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14254 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14258 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1425C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14260 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14264 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14280 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14284 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14288 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1428C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14290 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14294 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14298 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1429C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B142E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B142E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14300 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14304 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14308 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1430C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14310 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14314 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14318 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1431C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14320 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14324 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14340 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14344 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14348 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1434C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14350 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14354 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14358 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1435C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14360 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14364 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14380 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14384 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14388 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1438C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14390 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14394 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14398 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1439C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B143E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B143E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14400 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14404 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14408 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1440C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14410 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14414 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14418 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1441C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14420 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14424 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14440 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14444 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14448 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1444C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14450 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14454 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14458 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1445C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14460 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14464 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14480 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14484 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14488 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1448C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14490 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14494 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14498 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1449C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B144E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B144E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14500 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14504 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14508 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1450C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14510 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14514 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14518 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1451C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14520 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14524 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14540 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14544 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14548 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1454C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14550 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14554 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14558 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1455C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14560 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14564 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14580 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14584 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14588 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1458C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14590 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14594 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14598 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1459C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B145E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B145E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14600 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14604 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14608 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1460C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14610 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14614 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14618 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1461C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14620 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14624 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14640 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14644 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1464C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14650 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14654 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14658 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1465C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14660 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14664 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14680 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14684 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14688 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1468C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14690 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14694 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14698 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1469C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B146E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B146E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14700 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14704 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14708 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1470C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14710 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14714 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14718 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1471C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14720 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14724 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14740 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14744 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14748 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1474C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14750 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14754 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14758 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1475C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14760 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14764 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B14780 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14784 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14788 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1478C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14790 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14794 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B14798 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B1479C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B147E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B147E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B04000 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04004 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B04404 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00B04008 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B04408 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B0400C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B0440C ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B04010 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B04410 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B04014 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B04414 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B04018 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B04418 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0401C ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B0441C ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B04020 ) = 0x00000FEB;
	*mkptr( dfe_cfg_base, 0x00B04420 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B04024 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B04424 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B04028 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B04428 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B0402C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B0442C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B04030 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04430 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B04034 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B04434 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B04038 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B04438 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B0403C ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B0443C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B04040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B04440 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04044 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B04444 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B04048 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B04448 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B0404C ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B0444C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04050 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B04450 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B04054 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B04454 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B04058 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B04458 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B0405C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B0445C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B04060 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B04460 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B04064 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04464 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B04068 ) = 0x00000FE0;
	*mkptr( dfe_cfg_base, 0x00B04468 ) = 0x00000FF0;
	*mkptr( dfe_cfg_base, 0x00B0406C ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B0446C ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B04070 ) = 0x00000FF3;
	*mkptr( dfe_cfg_base, 0x00B04470 ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B04074 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B04474 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B04078 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B04478 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B0407C ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B0447C ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B04080 ) = 0x00000FEA;
	*mkptr( dfe_cfg_base, 0x00B04480 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B04084 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B04484 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B04088 ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B04488 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B0408C ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x00B0448C ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B04090 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B04490 ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x00B04094 ) = 0x00000039;
	*mkptr( dfe_cfg_base, 0x00B04494 ) = 0x00000042;
	*mkptr( dfe_cfg_base, 0x00B04098 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B04498 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B0409C ) = 0x00000FC6;
	*mkptr( dfe_cfg_base, 0x00B0449C ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B040A0 ) = 0x00000FB6;
	*mkptr( dfe_cfg_base, 0x00B044A0 ) = 0x00000FB5;
	*mkptr( dfe_cfg_base, 0x00B040A4 ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B044A4 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B040A8 ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B044A8 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B040AC ) = 0x0000005B;
	*mkptr( dfe_cfg_base, 0x00B044AC ) = 0x00000052;
	*mkptr( dfe_cfg_base, 0x00B040B0 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B044B0 ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x00B040B4 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B044B4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B040B8 ) = 0x00000F94;
	*mkptr( dfe_cfg_base, 0x00B044B8 ) = 0x00000FAA;
	*mkptr( dfe_cfg_base, 0x00B040BC ) = 0x00000FB0;
	*mkptr( dfe_cfg_base, 0x00B044BC ) = 0x00000F93;
	*mkptr( dfe_cfg_base, 0x00B040C0 ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B044C0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B040C4 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x00B044C4 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x00B040C8 ) = 0x00000079;
	*mkptr( dfe_cfg_base, 0x00B044C8 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B040CC ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B044CC ) = 0x00000051;
	*mkptr( dfe_cfg_base, 0x00B040D0 ) = 0x00000F76;
	*mkptr( dfe_cfg_base, 0x00B044D0 ) = 0x00000FBF;
	*mkptr( dfe_cfg_base, 0x00B040D4 ) = 0x00000F50;
	*mkptr( dfe_cfg_base, 0x00B044D4 ) = 0x00000F55;
	*mkptr( dfe_cfg_base, 0x00B040D8 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B044D8 ) = 0x00000F7D;
	*mkptr( dfe_cfg_base, 0x00B040DC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x00B044DC ) = 0x0000002F;
	*mkptr( dfe_cfg_base, 0x00B040E0 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x00B044E0 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x00B040E4 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B044E4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x00B040E8 ) = 0x00000F62;
	*mkptr( dfe_cfg_base, 0x00B044E8 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B040EC ) = 0x00000E5F;
	*mkptr( dfe_cfg_base, 0x00B044EC ) = 0x00000EC9;
	*mkptr( dfe_cfg_base, 0x00B040F0 ) = 0x00000E9F;
	*mkptr( dfe_cfg_base, 0x00B044F0 ) = 0x00000E49;
	*mkptr( dfe_cfg_base, 0x00B040F4 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x00B044F4 ) = 0x00000F6B;
	*mkptr( dfe_cfg_base, 0x00B040F8 ) = 0x000003D9;
	*mkptr( dfe_cfg_base, 0x00B044F8 ) = 0x0000022C;
	*mkptr( dfe_cfg_base, 0x00B040FC ) = 0x000006CD;
	*mkptr( dfe_cfg_base, 0x00B044FC ) = 0x00000576;
	*mkptr( dfe_cfg_base, 0x00B04100 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B04500 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x00B08000 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08004 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B08404 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00B08008 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B08408 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B0800C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B0840C ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B08010 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B08410 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B08014 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B08414 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B08018 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B08418 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B0801C ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B0841C ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B08020 ) = 0x00000FEB;
	*mkptr( dfe_cfg_base, 0x00B08420 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B08024 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B08424 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B08028 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B08428 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B0802C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B0842C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B08030 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08430 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B08034 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B08434 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B08038 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B08438 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B0803C ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B0843C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B08040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B08440 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08044 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B08444 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B08048 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B08448 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B0804C ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B0844C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08050 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B08450 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B08054 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B08454 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B08058 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B08458 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B0805C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B0845C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B08060 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B08460 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B08064 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08464 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B08068 ) = 0x00000FE0;
	*mkptr( dfe_cfg_base, 0x00B08468 ) = 0x00000FF0;
	*mkptr( dfe_cfg_base, 0x00B0806C ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B0846C ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B08070 ) = 0x00000FF3;
	*mkptr( dfe_cfg_base, 0x00B08470 ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B08074 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B08474 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B08078 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B08478 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B0807C ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B0847C ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B08080 ) = 0x00000FEA;
	*mkptr( dfe_cfg_base, 0x00B08480 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B08084 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B08484 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B08088 ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B08488 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B0808C ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x00B0848C ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B08090 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B08490 ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x00B08094 ) = 0x00000039;
	*mkptr( dfe_cfg_base, 0x00B08494 ) = 0x00000042;
	*mkptr( dfe_cfg_base, 0x00B08098 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B08498 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B0809C ) = 0x00000FC6;
	*mkptr( dfe_cfg_base, 0x00B0849C ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B080A0 ) = 0x00000FB6;
	*mkptr( dfe_cfg_base, 0x00B084A0 ) = 0x00000FB5;
	*mkptr( dfe_cfg_base, 0x00B080A4 ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B084A4 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B080A8 ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B084A8 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B080AC ) = 0x0000005B;
	*mkptr( dfe_cfg_base, 0x00B084AC ) = 0x00000052;
	*mkptr( dfe_cfg_base, 0x00B080B0 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B084B0 ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x00B080B4 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B084B4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B080B8 ) = 0x00000F94;
	*mkptr( dfe_cfg_base, 0x00B084B8 ) = 0x00000FAA;
	*mkptr( dfe_cfg_base, 0x00B080BC ) = 0x00000FB0;
	*mkptr( dfe_cfg_base, 0x00B084BC ) = 0x00000F93;
	*mkptr( dfe_cfg_base, 0x00B080C0 ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B084C0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B080C4 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x00B084C4 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x00B080C8 ) = 0x00000079;
	*mkptr( dfe_cfg_base, 0x00B084C8 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B080CC ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B084CC ) = 0x00000051;
	*mkptr( dfe_cfg_base, 0x00B080D0 ) = 0x00000F76;
	*mkptr( dfe_cfg_base, 0x00B084D0 ) = 0x00000FBF;
	*mkptr( dfe_cfg_base, 0x00B080D4 ) = 0x00000F50;
	*mkptr( dfe_cfg_base, 0x00B084D4 ) = 0x00000F55;
	*mkptr( dfe_cfg_base, 0x00B080D8 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B084D8 ) = 0x00000F7D;
	*mkptr( dfe_cfg_base, 0x00B080DC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x00B084DC ) = 0x0000002F;
	*mkptr( dfe_cfg_base, 0x00B080E0 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x00B084E0 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x00B080E4 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B084E4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x00B080E8 ) = 0x00000F62;
	*mkptr( dfe_cfg_base, 0x00B084E8 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B080EC ) = 0x00000E5F;
	*mkptr( dfe_cfg_base, 0x00B084EC ) = 0x00000EC9;
	*mkptr( dfe_cfg_base, 0x00B080F0 ) = 0x00000E9F;
	*mkptr( dfe_cfg_base, 0x00B084F0 ) = 0x00000E49;
	*mkptr( dfe_cfg_base, 0x00B080F4 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x00B084F4 ) = 0x00000F6B;
	*mkptr( dfe_cfg_base, 0x00B080F8 ) = 0x000003D9;
	*mkptr( dfe_cfg_base, 0x00B084F8 ) = 0x0000022C;
	*mkptr( dfe_cfg_base, 0x00B080FC ) = 0x000006CD;
	*mkptr( dfe_cfg_base, 0x00B084FC ) = 0x00000576;
	*mkptr( dfe_cfg_base, 0x00B08100 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B08500 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x00B803CC ) = 0x00008006;
	*mkptr( dfe_cfg_base, 0x00B80208 ) = 0x000008C0;
	*mkptr( dfe_cfg_base, 0x00B80288 ) = 0x000008C0;
	*mkptr( dfe_cfg_base, 0x00B80360 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x00B80368 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x00B80370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B80200 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80280 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80210 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B80218 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B80230 ) = 0x00004040;
	*mkptr( dfe_cfg_base, 0x00B80290 ) = 0x00008686;
	*mkptr( dfe_cfg_base, 0x00B80298 ) = 0x00000F0F;
	*mkptr( dfe_cfg_base, 0x00B802B0 ) = 0x00004040;
	*mkptr( dfe_cfg_base, 0x00B80300 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B80308 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B80310 ) = 0x00007F5C;
	*mkptr( dfe_cfg_base, 0x00B80318 ) = 0x00007F5C;
	*mkptr( dfe_cfg_base, 0x00B80320 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B80328 ) = 0x00000174;
	*mkptr( dfe_cfg_base, 0x00B80338 ) = 0x00000BB8;
	*mkptr( dfe_cfg_base, 0x00B80340 ) = 0x00000363;
	*mkptr( dfe_cfg_base, 0x00B80348 ) = 0x000000FF;
	*mkptr( dfe_cfg_base, 0x00B80378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x00B98004 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9800C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98014 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9801C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98024 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9802C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98034 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9803C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98044 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9804C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98054 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9805C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98064 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9806C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98074 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B9807C ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x00B98080 ) = 0x0000030C;
	*mkptr( dfe_cfg_base, 0x00B98084 ) = 0x00007FE8;
	*mkptr( dfe_cfg_base, 0x00B98088 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x00B9808C ) = 0x00007C17;
	*mkptr( dfe_cfg_base, 0x00B98090 ) = 0x00000334;
	*mkptr( dfe_cfg_base, 0x00B98094 ) = 0x00007899;
	*mkptr( dfe_cfg_base, 0x00B98098 ) = 0x00000340;
	*mkptr( dfe_cfg_base, 0x00B9809C ) = 0x00007562;
	*mkptr( dfe_cfg_base, 0x00B980A0 ) = 0x00000350;
	*mkptr( dfe_cfg_base, 0x00B980A4 ) = 0x0000726A;
	*mkptr( dfe_cfg_base, 0x00B980A8 ) = 0x0000035C;
	*mkptr( dfe_cfg_base, 0x00B980AC ) = 0x00006FA8;
	*mkptr( dfe_cfg_base, 0x00B980B0 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x00B980B4 ) = 0x00006D17;
	*mkptr( dfe_cfg_base, 0x00B980B8 ) = 0x00000370;
	*mkptr( dfe_cfg_base, 0x00B980BC ) = 0x00006AB2;
	*mkptr( dfe_cfg_base, 0x00B980C0 ) = 0x00000378;
	*mkptr( dfe_cfg_base, 0x00B980C4 ) = 0x00006873;
	*mkptr( dfe_cfg_base, 0x00B980C8 ) = 0x00000380;
	*mkptr( dfe_cfg_base, 0x00B980CC ) = 0x00006657;
	*mkptr( dfe_cfg_base, 0x00B980D0 ) = 0x00000388;
	*mkptr( dfe_cfg_base, 0x00B980D4 ) = 0x0000645A;
	*mkptr( dfe_cfg_base, 0x00B980D8 ) = 0x00000390;
	*mkptr( dfe_cfg_base, 0x00B980DC ) = 0x0000627A;
	*mkptr( dfe_cfg_base, 0x00B980E0 ) = 0x00000394;
	*mkptr( dfe_cfg_base, 0x00B980E4 ) = 0x000060B4;
	*mkptr( dfe_cfg_base, 0x00B980E8 ) = 0x00000398;
	*mkptr( dfe_cfg_base, 0x00B980EC ) = 0x00005F06;
	*mkptr( dfe_cfg_base, 0x00B980F0 ) = 0x000003A0;
	*mkptr( dfe_cfg_base, 0x00B980F4 ) = 0x00005D6D;
	*mkptr( dfe_cfg_base, 0x00B980F8 ) = 0x000003A4;
	*mkptr( dfe_cfg_base, 0x00B980FC ) = 0x00005BE8;
	*mkptr( dfe_cfg_base, 0x00B98100 ) = 0x000003A8;
	*mkptr( dfe_cfg_base, 0x00B98104 ) = 0x00005A76;
	*mkptr( dfe_cfg_base, 0x00B98108 ) = 0x000003AC;
	*mkptr( dfe_cfg_base, 0x00B9810C ) = 0x00005914;
	*mkptr( dfe_cfg_base, 0x00B98110 ) = 0x000003B0;
	*mkptr( dfe_cfg_base, 0x00B98114 ) = 0x000057C2;
	*mkptr( dfe_cfg_base, 0x00B98118 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B9811C ) = 0x0000567F;
	*mkptr( dfe_cfg_base, 0x00B98120 ) = 0x000003B4;
	*mkptr( dfe_cfg_base, 0x00B98124 ) = 0x00005549;
	*mkptr( dfe_cfg_base, 0x00B98128 ) = 0x000003B8;
	*mkptr( dfe_cfg_base, 0x00B9812C ) = 0x00005420;
	*mkptr( dfe_cfg_base, 0x00B98130 ) = 0x000003BC;
	*mkptr( dfe_cfg_base, 0x00B98134 ) = 0x00005303;
	*mkptr( dfe_cfg_base, 0x00B98138 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B9813C ) = 0x000051F1;
	*mkptr( dfe_cfg_base, 0x00B98140 ) = 0x000003C0;
	*mkptr( dfe_cfg_base, 0x00B98144 ) = 0x000050E9;
	*mkptr( dfe_cfg_base, 0x00B98148 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B9814C ) = 0x00004FEB;
	*mkptr( dfe_cfg_base, 0x00B98150 ) = 0x000003C4;
	*mkptr( dfe_cfg_base, 0x00B98154 ) = 0x00004EF6;
	*mkptr( dfe_cfg_base, 0x00B98158 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B9815C ) = 0x00004E0A;
	*mkptr( dfe_cfg_base, 0x00B98160 ) = 0x000003C8;
	*mkptr( dfe_cfg_base, 0x00B98164 ) = 0x00004D25;
	*mkptr( dfe_cfg_base, 0x00B98168 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B9816C ) = 0x00004C49;
	*mkptr( dfe_cfg_base, 0x00B98170 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B98174 ) = 0x00004B73;
	*mkptr( dfe_cfg_base, 0x00B98178 ) = 0x000003CC;
	*mkptr( dfe_cfg_base, 0x00B9817C ) = 0x00004AA5;
	*mkptr( dfe_cfg_base, 0x00B98180 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B98184 ) = 0x000049DD;
	*mkptr( dfe_cfg_base, 0x00B98188 ) = 0x000003D0;
	*mkptr( dfe_cfg_base, 0x00B9818C ) = 0x0000491B;
	*mkptr( dfe_cfg_base, 0x00B98190 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B98194 ) = 0x0000485F;
	*mkptr( dfe_cfg_base, 0x00B98198 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B9819C ) = 0x000047A8;
	*mkptr( dfe_cfg_base, 0x00B981A0 ) = 0x000003D4;
	*mkptr( dfe_cfg_base, 0x00B981A4 ) = 0x000046F7;
	*mkptr( dfe_cfg_base, 0x00B981A8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981AC ) = 0x0000464B;
	*mkptr( dfe_cfg_base, 0x00B981B0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981B4 ) = 0x000045A3;
	*mkptr( dfe_cfg_base, 0x00B981B8 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981BC ) = 0x00004501;
	*mkptr( dfe_cfg_base, 0x00B981C0 ) = 0x000003D8;
	*mkptr( dfe_cfg_base, 0x00B981C4 ) = 0x00004462;
	*mkptr( dfe_cfg_base, 0x00B981C8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981CC ) = 0x000043C8;
	*mkptr( dfe_cfg_base, 0x00B981D0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981D4 ) = 0x00004332;
	*mkptr( dfe_cfg_base, 0x00B981D8 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981DC ) = 0x0000429F;
	*mkptr( dfe_cfg_base, 0x00B981E0 ) = 0x000003DC;
	*mkptr( dfe_cfg_base, 0x00B981E4 ) = 0x00004211;
	*mkptr( dfe_cfg_base, 0x00B981E8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981EC ) = 0x00004185;
	*mkptr( dfe_cfg_base, 0x00B981F0 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981F4 ) = 0x000040FE;
	*mkptr( dfe_cfg_base, 0x00B981F8 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B981FC ) = 0x00004079;
	*mkptr( dfe_cfg_base, 0x00B98200 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B98204 ) = 0x00003FF8;
	*mkptr( dfe_cfg_base, 0x00B98208 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B9820C ) = 0x00003F79;
	*mkptr( dfe_cfg_base, 0x00B98210 ) = 0x000003E0;
	*mkptr( dfe_cfg_base, 0x00B98214 ) = 0x00003EFE;
	*mkptr( dfe_cfg_base, 0x00B98218 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9821C ) = 0x00003E85;
	*mkptr( dfe_cfg_base, 0x00B98220 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98224 ) = 0x00003E0F;
	*mkptr( dfe_cfg_base, 0x00B98228 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9822C ) = 0x00003D9B;
	*mkptr( dfe_cfg_base, 0x00B98230 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98234 ) = 0x00003D2A;
	*mkptr( dfe_cfg_base, 0x00B98238 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B9823C ) = 0x00003CBB;
	*mkptr( dfe_cfg_base, 0x00B98240 ) = 0x000003E4;
	*mkptr( dfe_cfg_base, 0x00B98244 ) = 0x00003C4F;
	*mkptr( dfe_cfg_base, 0x00B98248 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9824C ) = 0x00003BE5;
	*mkptr( dfe_cfg_base, 0x00B98250 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98254 ) = 0x00003B7D;
	*mkptr( dfe_cfg_base, 0x00B98258 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9825C ) = 0x00003B17;
	*mkptr( dfe_cfg_base, 0x00B98260 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98264 ) = 0x00003AB3;
	*mkptr( dfe_cfg_base, 0x00B98268 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9826C ) = 0x00003A51;
	*mkptr( dfe_cfg_base, 0x00B98270 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98274 ) = 0x000039F1;
	*mkptr( dfe_cfg_base, 0x00B98278 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9827C ) = 0x00003993;
	*mkptr( dfe_cfg_base, 0x00B98280 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B98284 ) = 0x00003937;
	*mkptr( dfe_cfg_base, 0x00B98288 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x00B9828C ) = 0x000038DC;
	*mkptr( dfe_cfg_base, 0x00B98290 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B98294 ) = 0x00003883;
	*mkptr( dfe_cfg_base, 0x00B98298 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B9829C ) = 0x0000382C;
	*mkptr( dfe_cfg_base, 0x00B982A0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982A4 ) = 0x000037D6;
	*mkptr( dfe_cfg_base, 0x00B982A8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982AC ) = 0x00003782;
	*mkptr( dfe_cfg_base, 0x00B982B0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982B4 ) = 0x0000372F;
	*mkptr( dfe_cfg_base, 0x00B982B8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982BC ) = 0x000036DD;
	*mkptr( dfe_cfg_base, 0x00B982C0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982C4 ) = 0x0000368D;
	*mkptr( dfe_cfg_base, 0x00B982C8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982CC ) = 0x0000363F;
	*mkptr( dfe_cfg_base, 0x00B982D0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982D4 ) = 0x000035F1;
	*mkptr( dfe_cfg_base, 0x00B982D8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982DC ) = 0x000035A5;
	*mkptr( dfe_cfg_base, 0x00B982E0 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982E4 ) = 0x0000355A;
	*mkptr( dfe_cfg_base, 0x00B982E8 ) = 0x000003EC;
	*mkptr( dfe_cfg_base, 0x00B982EC ) = 0x00003511;
	*mkptr( dfe_cfg_base, 0x00B982F0 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B982F4 ) = 0x000034C8;
	*mkptr( dfe_cfg_base, 0x00B982F8 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B982FC ) = 0x00003481;
	*mkptr( dfe_cfg_base, 0x00B98300 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98304 ) = 0x0000343B;
	*mkptr( dfe_cfg_base, 0x00B98308 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9830C ) = 0x000033F6;
	*mkptr( dfe_cfg_base, 0x00B98310 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98314 ) = 0x000033B2;
	*mkptr( dfe_cfg_base, 0x00B98318 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9831C ) = 0x0000336F;
	*mkptr( dfe_cfg_base, 0x00B98320 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98324 ) = 0x0000332D;
	*mkptr( dfe_cfg_base, 0x00B98328 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9832C ) = 0x000032EC;
	*mkptr( dfe_cfg_base, 0x00B98330 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98334 ) = 0x000032AC;
	*mkptr( dfe_cfg_base, 0x00B98338 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9833C ) = 0x0000326C;
	*mkptr( dfe_cfg_base, 0x00B98340 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98344 ) = 0x0000322E;
	*mkptr( dfe_cfg_base, 0x00B98348 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9834C ) = 0x000031F1;
	*mkptr( dfe_cfg_base, 0x00B98350 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98354 ) = 0x000031B4;
	*mkptr( dfe_cfg_base, 0x00B98358 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9835C ) = 0x00003179;
	*mkptr( dfe_cfg_base, 0x00B98360 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98364 ) = 0x0000313E;
	*mkptr( dfe_cfg_base, 0x00B98368 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B9836C ) = 0x00003104;
	*mkptr( dfe_cfg_base, 0x00B98370 ) = 0x000003F0;
	*mkptr( dfe_cfg_base, 0x00B98374 ) = 0x000030CB;
	*mkptr( dfe_cfg_base, 0x00B98378 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9837C ) = 0x00003093;
	*mkptr( dfe_cfg_base, 0x00B98380 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B98384 ) = 0x0000305B;
	*mkptr( dfe_cfg_base, 0x00B98388 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9838C ) = 0x00003024;
	*mkptr( dfe_cfg_base, 0x00B98390 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B98394 ) = 0x00002FEE;
	*mkptr( dfe_cfg_base, 0x00B98398 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B9839C ) = 0x00002FB8;
	*mkptr( dfe_cfg_base, 0x00B983A0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983A4 ) = 0x00002F84;
	*mkptr( dfe_cfg_base, 0x00B983A8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983AC ) = 0x00002F50;
	*mkptr( dfe_cfg_base, 0x00B983B0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983B4 ) = 0x00002F1C;
	*mkptr( dfe_cfg_base, 0x00B983B8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983BC ) = 0x00002EE9;
	*mkptr( dfe_cfg_base, 0x00B983C0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983C4 ) = 0x00002EB7;
	*mkptr( dfe_cfg_base, 0x00B983C8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983CC ) = 0x00002E86;
	*mkptr( dfe_cfg_base, 0x00B983D0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983D4 ) = 0x00002E55;
	*mkptr( dfe_cfg_base, 0x00B983D8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983DC ) = 0x00002E24;
	*mkptr( dfe_cfg_base, 0x00B983E0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983E4 ) = 0x00002DF5;
	*mkptr( dfe_cfg_base, 0x00B983E8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983EC ) = 0x00002DC6;
	*mkptr( dfe_cfg_base, 0x00B983F0 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983F4 ) = 0x00002D97;
	*mkptr( dfe_cfg_base, 0x00B983F8 ) = 0x000003F4;
	*mkptr( dfe_cfg_base, 0x00B983FC ) = 0x00002D69;
	*mkptr( dfe_cfg_base, 0x00B90100 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90108 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9010C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90110 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90114 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90118 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9011C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90120 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90124 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90128 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9012C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90130 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B90134 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B90138 ) = 0x00010005;
	*mkptr( dfe_cfg_base, 0x00B9013C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x00B9500C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x00B95014 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x00B9501C ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x00B95024 ) = 0x00000400;
	*mkptr( dfe_cfg_base, 0x00B9502C ) = 0x00000500;
	*mkptr( dfe_cfg_base, 0x00B95034 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x00B9503C ) = 0x00000700;
	*mkptr( dfe_cfg_base, 0x00B95044 ) = 0x00000800;
	*mkptr( dfe_cfg_base, 0x00B9504C ) = 0x00000900;
	*mkptr( dfe_cfg_base, 0x00B95054 ) = 0x00000A00;
	*mkptr( dfe_cfg_base, 0x00B9505C ) = 0x00000B00;
	*mkptr( dfe_cfg_base, 0x00B95064 ) = 0x00000C00;
	*mkptr( dfe_cfg_base, 0x00B9506C ) = 0x00000D00;
	*mkptr( dfe_cfg_base, 0x00B95074 ) = 0x00000E00;
	*mkptr( dfe_cfg_base, 0x00B9507C ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x00B94000 ) = 0x00002D6A;
	*mkptr( dfe_cfg_base, 0x00B94004 ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B94008 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x00B9400C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B94010 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B94014 ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B94018 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x00B9401C ) = 0x000038C7;
	*mkptr( dfe_cfg_base, 0x00B94020 ) = 0x00000808;
	*mkptr( dfe_cfg_base, 0x00B94024 ) = 0x00000404;
	*mkptr( dfe_cfg_base, 0x00B94040 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94044 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94048 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9404C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94050 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94054 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94058 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9405C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94060 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94064 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94080 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94084 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94088 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9408C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94090 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94094 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94098 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9409C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B940E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B940E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94100 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94104 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94108 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9410C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94110 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94114 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94118 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9411C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94120 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94124 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94140 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94144 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94148 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9414C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94150 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94154 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94158 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9415C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94160 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94164 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94180 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94184 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94188 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9418C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94190 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94194 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94198 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9419C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B941E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B941E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94200 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94204 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94208 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9420C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94210 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94214 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94218 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9421C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94220 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94224 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94240 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94244 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94248 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9424C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94250 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94254 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94258 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9425C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94260 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94264 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94280 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94284 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94288 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9428C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94290 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94294 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94298 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9429C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B942E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B942E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94300 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94304 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94308 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9430C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94310 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94314 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94318 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9431C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94320 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94324 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94340 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94344 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94348 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9434C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94350 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94354 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94358 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9435C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94360 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94364 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94380 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94384 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94388 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9438C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94390 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94394 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94398 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9439C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B943E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B943E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94400 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94404 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94408 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9440C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94410 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94414 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94418 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9441C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94420 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94424 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94440 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94444 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94448 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9444C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94450 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94454 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94458 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9445C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94460 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94464 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94480 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94484 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94488 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9448C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94490 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94494 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94498 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9449C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B944E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B944E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94500 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94504 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94508 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9450C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94510 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94514 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94518 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9451C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94520 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94524 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94540 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94544 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94548 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9454C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94550 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94554 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94558 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9455C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94560 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94564 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94580 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94584 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94588 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9458C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94590 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94594 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94598 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9459C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B945E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B945E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94600 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94604 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94608 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9460C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94610 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94614 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94618 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9461C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94620 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94624 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94640 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94644 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9464C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94650 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94654 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94658 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9465C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94660 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94664 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94680 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94684 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94688 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9468C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94690 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94694 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94698 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9469C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B946E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B946E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94700 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94704 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94708 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9470C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94710 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94714 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94718 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9471C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94720 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94724 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94740 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94744 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94748 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9474C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94750 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94754 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94758 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9475C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94760 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94764 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B94780 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94784 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94788 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9478C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94790 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94794 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B94798 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B9479C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947A0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947A4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947C0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947C4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947C8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947CC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947D8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947DC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x00B947E0 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B947E4 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
	*mkptr( dfe_cfg_base, 0x00B84000 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B84004 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B84404 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00B84008 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B84408 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B8400C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B8440C ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B84010 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B84410 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B84014 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B84414 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B84018 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B84418 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B8401C ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B8441C ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B84020 ) = 0x00000FEB;
	*mkptr( dfe_cfg_base, 0x00B84420 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B84024 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B84424 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B84028 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B84428 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B8402C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B8442C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B84030 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B84430 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B84034 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B84434 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B84038 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B84438 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B8403C ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B8443C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B84040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B84440 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B84044 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B84444 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B84048 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B84448 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B8404C ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B8444C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B84050 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B84450 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B84054 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B84454 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B84058 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B84458 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B8405C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B8445C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B84060 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B84460 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B84064 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B84464 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B84068 ) = 0x00000FE0;
	*mkptr( dfe_cfg_base, 0x00B84468 ) = 0x00000FF0;
	*mkptr( dfe_cfg_base, 0x00B8406C ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B8446C ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B84070 ) = 0x00000FF3;
	*mkptr( dfe_cfg_base, 0x00B84470 ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B84074 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B84474 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B84078 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B84478 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B8407C ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B8447C ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B84080 ) = 0x00000FEA;
	*mkptr( dfe_cfg_base, 0x00B84480 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B84084 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B84484 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B84088 ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B84488 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B8408C ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x00B8448C ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B84090 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B84490 ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x00B84094 ) = 0x00000039;
	*mkptr( dfe_cfg_base, 0x00B84494 ) = 0x00000042;
	*mkptr( dfe_cfg_base, 0x00B84098 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B84498 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B8409C ) = 0x00000FC6;
	*mkptr( dfe_cfg_base, 0x00B8449C ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B840A0 ) = 0x00000FB6;
	*mkptr( dfe_cfg_base, 0x00B844A0 ) = 0x00000FB5;
	*mkptr( dfe_cfg_base, 0x00B840A4 ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B844A4 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B840A8 ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B844A8 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B840AC ) = 0x0000005B;
	*mkptr( dfe_cfg_base, 0x00B844AC ) = 0x00000052;
	*mkptr( dfe_cfg_base, 0x00B840B0 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B844B0 ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x00B840B4 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B844B4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B840B8 ) = 0x00000F94;
	*mkptr( dfe_cfg_base, 0x00B844B8 ) = 0x00000FAA;
	*mkptr( dfe_cfg_base, 0x00B840BC ) = 0x00000FB0;
	*mkptr( dfe_cfg_base, 0x00B844BC ) = 0x00000F93;
	*mkptr( dfe_cfg_base, 0x00B840C0 ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B844C0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B840C4 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x00B844C4 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x00B840C8 ) = 0x00000079;
	*mkptr( dfe_cfg_base, 0x00B844C8 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B840CC ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B844CC ) = 0x00000051;
	*mkptr( dfe_cfg_base, 0x00B840D0 ) = 0x00000F76;
	*mkptr( dfe_cfg_base, 0x00B844D0 ) = 0x00000FBF;
	*mkptr( dfe_cfg_base, 0x00B840D4 ) = 0x00000F50;
	*mkptr( dfe_cfg_base, 0x00B844D4 ) = 0x00000F55;
	*mkptr( dfe_cfg_base, 0x00B840D8 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B844D8 ) = 0x00000F7D;
	*mkptr( dfe_cfg_base, 0x00B840DC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x00B844DC ) = 0x0000002F;
	*mkptr( dfe_cfg_base, 0x00B840E0 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x00B844E0 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x00B840E4 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B844E4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x00B840E8 ) = 0x00000F62;
	*mkptr( dfe_cfg_base, 0x00B844E8 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B840EC ) = 0x00000E5F;
	*mkptr( dfe_cfg_base, 0x00B844EC ) = 0x00000EC9;
	*mkptr( dfe_cfg_base, 0x00B840F0 ) = 0x00000E9F;
	*mkptr( dfe_cfg_base, 0x00B844F0 ) = 0x00000E49;
	*mkptr( dfe_cfg_base, 0x00B840F4 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x00B844F4 ) = 0x00000F6B;
	*mkptr( dfe_cfg_base, 0x00B840F8 ) = 0x000003D9;
	*mkptr( dfe_cfg_base, 0x00B844F8 ) = 0x0000022C;
	*mkptr( dfe_cfg_base, 0x00B840FC ) = 0x000006CD;
	*mkptr( dfe_cfg_base, 0x00B844FC ) = 0x00000576;
	*mkptr( dfe_cfg_base, 0x00B84100 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B84500 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x00B88000 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B88004 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x00B88404 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x00B88008 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B88408 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B8800C ) = 0x0000003B;
	*mkptr( dfe_cfg_base, 0x00B8840C ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B88010 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x00B88410 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B88014 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x00B88414 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x00B88018 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B88418 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B8801C ) = 0x00000FE4;
	*mkptr( dfe_cfg_base, 0x00B8841C ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B88020 ) = 0x00000FEB;
	*mkptr( dfe_cfg_base, 0x00B88420 ) = 0x00000FE5;
	*mkptr( dfe_cfg_base, 0x00B88024 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x00B88424 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B88028 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B88428 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B8802C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B8842C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B88030 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B88430 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x00B88034 ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B88434 ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B88038 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B88438 ) = 0x00000FE7;
	*mkptr( dfe_cfg_base, 0x00B8803C ) = 0x00000FF7;
	*mkptr( dfe_cfg_base, 0x00B8843C ) = 0x00000FED;
	*mkptr( dfe_cfg_base, 0x00B88040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B88440 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B88044 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B88444 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B88048 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B88448 ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x00B8804C ) = 0x00000FF4;
	*mkptr( dfe_cfg_base, 0x00B8844C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B88050 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B88450 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B88054 ) = 0x00000FE8;
	*mkptr( dfe_cfg_base, 0x00B88454 ) = 0x00000FE1;
	*mkptr( dfe_cfg_base, 0x00B88058 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x00B88458 ) = 0x00000FF5;
	*mkptr( dfe_cfg_base, 0x00B8805C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B8845C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x00B88060 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x00B88460 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B88064 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B88464 ) = 0x00000015;
	*mkptr( dfe_cfg_base, 0x00B88068 ) = 0x00000FE0;
	*mkptr( dfe_cfg_base, 0x00B88468 ) = 0x00000FF0;
	*mkptr( dfe_cfg_base, 0x00B8806C ) = 0x00000FD8;
	*mkptr( dfe_cfg_base, 0x00B8846C ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B88070 ) = 0x00000FF3;
	*mkptr( dfe_cfg_base, 0x00B88470 ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B88074 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x00B88474 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x00B88078 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B88478 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B8807C ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B8847C ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x00B88080 ) = 0x00000FEA;
	*mkptr( dfe_cfg_base, 0x00B88480 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x00B88084 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B88484 ) = 0x00000FD5;
	*mkptr( dfe_cfg_base, 0x00B88088 ) = 0x00000FD7;
	*mkptr( dfe_cfg_base, 0x00B88488 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B8808C ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x00B8848C ) = 0x00000FEE;
	*mkptr( dfe_cfg_base, 0x00B88090 ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x00B88490 ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x00B88094 ) = 0x00000039;
	*mkptr( dfe_cfg_base, 0x00B88494 ) = 0x00000042;
	*mkptr( dfe_cfg_base, 0x00B88098 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B88498 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x00B8809C ) = 0x00000FC6;
	*mkptr( dfe_cfg_base, 0x00B8849C ) = 0x00000FE2;
	*mkptr( dfe_cfg_base, 0x00B880A0 ) = 0x00000FB6;
	*mkptr( dfe_cfg_base, 0x00B884A0 ) = 0x00000FB5;
	*mkptr( dfe_cfg_base, 0x00B880A4 ) = 0x00000FE9;
	*mkptr( dfe_cfg_base, 0x00B884A4 ) = 0x00000FC8;
	*mkptr( dfe_cfg_base, 0x00B880A8 ) = 0x00000036;
	*mkptr( dfe_cfg_base, 0x00B884A8 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x00B880AC ) = 0x0000005B;
	*mkptr( dfe_cfg_base, 0x00B884AC ) = 0x00000052;
	*mkptr( dfe_cfg_base, 0x00B880B0 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x00B884B0 ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x00B880B4 ) = 0x00000FD4;
	*mkptr( dfe_cfg_base, 0x00B884B4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x00B880B8 ) = 0x00000F94;
	*mkptr( dfe_cfg_base, 0x00B884B8 ) = 0x00000FAA;
	*mkptr( dfe_cfg_base, 0x00B880BC ) = 0x00000FB0;
	*mkptr( dfe_cfg_base, 0x00B884BC ) = 0x00000F93;
	*mkptr( dfe_cfg_base, 0x00B880C0 ) = 0x0000001A;
	*mkptr( dfe_cfg_base, 0x00B884C0 ) = 0x00000FDE;
	*mkptr( dfe_cfg_base, 0x00B880C4 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x00B884C4 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x00B880C8 ) = 0x00000079;
	*mkptr( dfe_cfg_base, 0x00B884C8 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B880CC ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B884CC ) = 0x00000051;
	*mkptr( dfe_cfg_base, 0x00B880D0 ) = 0x00000F76;
	*mkptr( dfe_cfg_base, 0x00B884D0 ) = 0x00000FBF;
	*mkptr( dfe_cfg_base, 0x00B880D4 ) = 0x00000F50;
	*mkptr( dfe_cfg_base, 0x00B884D4 ) = 0x00000F55;
	*mkptr( dfe_cfg_base, 0x00B880D8 ) = 0x00000FCA;
	*mkptr( dfe_cfg_base, 0x00B884D8 ) = 0x00000F7D;
	*mkptr( dfe_cfg_base, 0x00B880DC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x00B884DC ) = 0x0000002F;
	*mkptr( dfe_cfg_base, 0x00B880E0 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x00B884E0 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x00B880E4 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x00B884E4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x00B880E8 ) = 0x00000F62;
	*mkptr( dfe_cfg_base, 0x00B884E8 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x00B880EC ) = 0x00000E5F;
	*mkptr( dfe_cfg_base, 0x00B884EC ) = 0x00000EC9;
	*mkptr( dfe_cfg_base, 0x00B880F0 ) = 0x00000E9F;
	*mkptr( dfe_cfg_base, 0x00B884F0 ) = 0x00000E49;
	*mkptr( dfe_cfg_base, 0x00B880F4 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x00B884F4 ) = 0x00000F6B;
	*mkptr( dfe_cfg_base, 0x00B880F8 ) = 0x000003D9;
	*mkptr( dfe_cfg_base, 0x00B884F8 ) = 0x0000022C;
	*mkptr( dfe_cfg_base, 0x00B880FC ) = 0x000006CD;
	*mkptr( dfe_cfg_base, 0x00B884FC ) = 0x00000576;
	*mkptr( dfe_cfg_base, 0x00B88100 ) = 0x000007FF;
	*mkptr( dfe_cfg_base, 0x00B88500 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x01000008 ) = 0x40000001;
	*mkptr( dfe_cfg_base, 0x0100000C ) = 0x40000001;
	*mkptr( dfe_cfg_base, 0x01000010 ) = 0x40000001;
	*mkptr( dfe_cfg_base, 0x01000014 ) = 0x40000001;
	*mkptr( dfe_cfg_base, 0x01000038 ) = 0x000C0003;
	*mkptr( dfe_cfg_base, 0x0100003C ) = 0x0000048A;
	*mkptr( dfe_cfg_base, 0x01000040 ) = 0x005300AA;
	*mkptr( dfe_cfg_base, 0x01000200 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01000404 ) = 0x0001FFFF;
	*mkptr( dfe_cfg_base, 0x01001100 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01001104 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01001108 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x0100110C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01001110 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01001114 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01001118 ) = 0x00000010;
	iqn2_bb_loopback_config_split_1();
	iqn2_bb_loopback_config_split_2();
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config_split_1,".text:dfe");
#endif
static void iqn2_bb_loopback_config_split_1() {
	*mkptr( dfe_cfg_base, 0x01300200 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01300204 ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01300208 ) = 0x00000081;
	*mkptr( dfe_cfg_base, 0x0130020C ) = 0x00002200;
	*mkptr( dfe_cfg_base, 0x01300210 ) = 0x07070000;
	*mkptr( dfe_cfg_base, 0x01300214 ) = 0x0000F5F4;
	*mkptr( dfe_cfg_base, 0x01300228 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01300400 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300408 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130040C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300410 ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300414 ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300418 ) = 0x1500018E;
	*mkptr( dfe_cfg_base, 0x0130041C ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x01300420 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300428 ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x0130042C ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x01300430 ) = 0x150002E8;
	*mkptr( dfe_cfg_base, 0x01300434 ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x01300438 ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x0130043C ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x01300440 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300448 ) = 0x15000088;
	*mkptr( dfe_cfg_base, 0x0130044C ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300450 ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300454 ) = 0x15003000;
	*mkptr( dfe_cfg_base, 0x01300458 ) = 0x1500018E;
	*mkptr( dfe_cfg_base, 0x0130045C ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x01300460 ) = 0x00003300;
	*mkptr( dfe_cfg_base, 0x01300468 ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x0130046C ) = 0x1500210E;
	*mkptr( dfe_cfg_base, 0x01300470 ) = 0x150002E8;
	*mkptr( dfe_cfg_base, 0x01300474 ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x01300478 ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x0130047C ) = 0x150012E0;
	*mkptr( dfe_cfg_base, 0x01343000 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343008 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343010 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343018 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343020 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343028 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343030 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343038 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343040 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343048 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343050 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343058 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343060 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343068 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343070 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343078 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343088 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343090 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343098 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013430F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343100 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343108 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343110 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343118 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343120 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343128 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343130 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343138 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343140 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343148 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343150 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343158 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343160 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343168 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343170 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343178 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343180 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343188 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343190 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343198 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013431F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343200 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343208 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343210 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343218 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343220 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343228 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343230 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343238 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343240 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343248 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343250 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343258 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343260 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343268 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343270 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343278 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343280 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343288 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343290 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343298 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013432F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343300 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343308 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343310 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343318 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343320 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343328 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343330 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343338 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343340 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343348 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343350 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343358 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343360 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343368 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343370 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343378 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343380 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343388 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343390 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343398 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013433F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343400 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343408 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343410 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343418 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343420 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343428 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343430 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343438 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343440 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343448 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343450 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343458 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343460 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343468 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343470 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343478 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343480 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343488 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343490 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343498 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013434F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343500 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343508 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343510 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343518 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343520 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343528 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343530 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343538 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343540 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343548 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343550 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343558 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343560 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343568 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343570 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343578 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343580 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343588 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343590 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343598 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013435F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343600 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343608 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343610 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343618 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343620 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343628 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343630 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343638 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343640 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343648 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343650 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343658 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343660 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343668 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343670 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343678 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343680 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343688 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343690 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343698 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013436F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343700 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343708 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343710 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343718 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343720 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343728 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343730 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343738 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343740 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343748 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343750 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343758 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343760 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343768 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343770 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343778 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343780 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343788 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343790 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01343798 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013437F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355000 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355008 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355010 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355018 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355020 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355028 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355030 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355038 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355040 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355048 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355050 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355058 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355060 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355068 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355070 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355078 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355088 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355090 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355098 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013550F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355100 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355108 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355110 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355118 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355120 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355128 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355130 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355138 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355140 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355148 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355150 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355158 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355160 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355168 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355170 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355178 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355180 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355188 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355190 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355198 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013551F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355200 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355208 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355210 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355218 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355220 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355228 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355230 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355238 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355240 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355248 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355250 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355258 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355260 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355268 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355270 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355278 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355280 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355288 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355290 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355298 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013552F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355300 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355308 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355310 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355318 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355320 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355328 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355330 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355338 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355340 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355348 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355350 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355358 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355360 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355368 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355370 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355378 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355380 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355388 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355390 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355398 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013553F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355400 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355408 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355410 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355418 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355420 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355428 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355430 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355438 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355440 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355448 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355450 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355458 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355460 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355468 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355470 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355478 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355480 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355488 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355490 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355498 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013554F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355500 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355508 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355510 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355518 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355520 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355528 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355530 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355538 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355540 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355548 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355550 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355558 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355560 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355568 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355570 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355578 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355580 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355588 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355590 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355598 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013555F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355600 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355608 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355610 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355618 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355620 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355628 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355630 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355638 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355640 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355648 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355650 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355658 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355660 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355668 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355670 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355678 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355680 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355688 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355690 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355698 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013556F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355700 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355708 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355710 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355718 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355720 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355728 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355730 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355738 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355740 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355748 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355750 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355758 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355760 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355768 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355770 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355778 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355780 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355788 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355790 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01355798 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557A0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557A8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557B8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557C8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557D8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557E8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x013557F8 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01600014 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x01600018 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x0160001C ) = 0x98111110;
	*mkptr( dfe_cfg_base, 0x01600020 ) = 0x09090001;
	*mkptr( dfe_cfg_base, 0x0160002C ) = 0x00002FF8;
	*mkptr( dfe_cfg_base, 0x01600054 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x01600058 ) = 0x04000004;
	*mkptr( dfe_cfg_base, 0x0160005C ) = 0x98111110;
	*mkptr( dfe_cfg_base, 0x01600060 ) = 0x09090003;
	*mkptr( dfe_cfg_base, 0x0160006C ) = 0x00002FF8;
	*mkptr( dfe_cfg_base, 0x01600080 ) = 0xFFFFFFCD;
	*mkptr( dfe_cfg_base, 0x01600084 ) = 0xFFFFFF7D;
	*mkptr( dfe_cfg_base, 0x01600088 ) = 0x000000DD;
	*mkptr( dfe_cfg_base, 0x0160008C ) = 0xFFFFFF4B;
	*mkptr( dfe_cfg_base, 0x01600090 ) = 0xFFFFFF71;
	*mkptr( dfe_cfg_base, 0x01600094 ) = 0x000002F3;
	*mkptr( dfe_cfg_base, 0x01600098 ) = 0xFFFFFAA9;
	*mkptr( dfe_cfg_base, 0x0160009C ) = 0x000005A0;
	*mkptr( dfe_cfg_base, 0x016000A0 ) = 0xFFFFFE6D;
	*mkptr( dfe_cfg_base, 0x016000A4 ) = 0xFFFFF80B;
	*mkptr( dfe_cfg_base, 0x016000A8 ) = 0x000015F6;
	*mkptr( dfe_cfg_base, 0x016000AC ) = 0xFFFFDAE4;
	*mkptr( dfe_cfg_base, 0x016000B0 ) = 0x000030DA;
	*mkptr( dfe_cfg_base, 0x016000B4 ) = 0x0000CB69;
	*mkptr( dfe_cfg_base, 0x016000B8 ) = 0x0000CB69;
	*mkptr( dfe_cfg_base, 0x016000C0 ) = 0xFFFFFFF9;
	*mkptr( dfe_cfg_base, 0x016000C4 ) = 0xFFFFFF87;
	*mkptr( dfe_cfg_base, 0x016000C8 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x016000CC ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x016000D0 ) = 0xFFFFFE1F;
	*mkptr( dfe_cfg_base, 0x016000D4 ) = 0x0000026F;
	*mkptr( dfe_cfg_base, 0x016000D8 ) = 0xFFFFFED3;
	*mkptr( dfe_cfg_base, 0x016000DC ) = 0xFFFFFD62;
	*mkptr( dfe_cfg_base, 0x016000E0 ) = 0x0000083C;
	*mkptr( dfe_cfg_base, 0x016000E4 ) = 0xFFFFF2F3;
	*mkptr( dfe_cfg_base, 0x016000E8 ) = 0x00000CC1;
	*mkptr( dfe_cfg_base, 0x016000EC ) = 0xFFFFFE29;
	*mkptr( dfe_cfg_base, 0x016000F0 ) = 0xFFFFE29E;
	*mkptr( dfe_cfg_base, 0x016000F4 ) = 0x00009A36;
	*mkptr( dfe_cfg_base, 0x016000F8 ) = 0x00009A36;
	*mkptr( dfe_cfg_base, 0x01600100 ) = 0xFFFFFFCD;
	*mkptr( dfe_cfg_base, 0x01600104 ) = 0xFFFFFF7D;
	*mkptr( dfe_cfg_base, 0x01600108 ) = 0x000000DD;
	*mkptr( dfe_cfg_base, 0x0160010C ) = 0xFFFFFF4B;
	*mkptr( dfe_cfg_base, 0x01600110 ) = 0xFFFFFF71;
	*mkptr( dfe_cfg_base, 0x01600114 ) = 0x000002F3;
	*mkptr( dfe_cfg_base, 0x01600118 ) = 0xFFFFFAA9;
	*mkptr( dfe_cfg_base, 0x0160011C ) = 0x000005A0;
	*mkptr( dfe_cfg_base, 0x01600120 ) = 0xFFFFFE6D;
	*mkptr( dfe_cfg_base, 0x01600124 ) = 0xFFFFF80B;
	*mkptr( dfe_cfg_base, 0x01600128 ) = 0x000015F6;
	*mkptr( dfe_cfg_base, 0x0160012C ) = 0xFFFFDAE4;
	*mkptr( dfe_cfg_base, 0x01600130 ) = 0x000030DA;
	*mkptr( dfe_cfg_base, 0x01600134 ) = 0x0000CB69;
	*mkptr( dfe_cfg_base, 0x01600138 ) = 0x0000CB69;
	*mkptr( dfe_cfg_base, 0x01600140 ) = 0xFFFFFFF9;
	*mkptr( dfe_cfg_base, 0x01600144 ) = 0xFFFFFF87;
	*mkptr( dfe_cfg_base, 0x01600148 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x0160014C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01600150 ) = 0xFFFFFE1F;
	*mkptr( dfe_cfg_base, 0x01600154 ) = 0x0000026F;
	*mkptr( dfe_cfg_base, 0x01600158 ) = 0xFFFFFED3;
	*mkptr( dfe_cfg_base, 0x0160015C ) = 0xFFFFFD62;
	*mkptr( dfe_cfg_base, 0x01600160 ) = 0x0000083C;
	*mkptr( dfe_cfg_base, 0x01600164 ) = 0xFFFFF2F3;
	*mkptr( dfe_cfg_base, 0x01600168 ) = 0x00000CC1;
	*mkptr( dfe_cfg_base, 0x0160016C ) = 0xFFFFFE29;
	*mkptr( dfe_cfg_base, 0x01600170 ) = 0xFFFFE29E;
	*mkptr( dfe_cfg_base, 0x01600174 ) = 0x00009A36;
	*mkptr( dfe_cfg_base, 0x01600178 ) = 0x00009A36;
	*mkptr( dfe_cfg_base, 0x016000BC ) = 0x00006149;
	*mkptr( dfe_cfg_base, 0x0160013C ) = 0x00006149;
	*mkptr( dfe_cfg_base, 0x01600200 ) = 0xFFFFFFD9;
	*mkptr( dfe_cfg_base, 0x01600204 ) = 0x00000727;
	*mkptr( dfe_cfg_base, 0x01600208 ) = 0xFFFFE5D2;
	*mkptr( dfe_cfg_base, 0x0160020C ) = 0x00003A3E;
	*mkptr( dfe_cfg_base, 0x01600210 ) = 0x0001CDF3;
	*mkptr( dfe_cfg_base, 0x01600214 ) = 0x000017A4;
	*mkptr( dfe_cfg_base, 0x01600218 ) = 0xFFFFEEA9;
	*mkptr( dfe_cfg_base, 0x0160021C ) = 0x0000059A;
	*mkptr( dfe_cfg_base, 0x01600220 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01600224 ) = 0x000009AF;
	*mkptr( dfe_cfg_base, 0x01600228 ) = 0xFFFFC3B5;
	*mkptr( dfe_cfg_base, 0x0160022C ) = 0x000148E2;
	*mkptr( dfe_cfg_base, 0x01600230 ) = 0x00011BC6;
	*mkptr( dfe_cfg_base, 0x01600234 ) = 0xFFFFC300;
	*mkptr( dfe_cfg_base, 0x01600238 ) = 0x00000B1E;
	*mkptr( dfe_cfg_base, 0x0160023C ) = 0x0000002A;
	*mkptr( dfe_cfg_base, 0x01600600 ) = 0xFFFFFFD9;
	*mkptr( dfe_cfg_base, 0x01600604 ) = 0x00000727;
	*mkptr( dfe_cfg_base, 0x01600608 ) = 0xFFFFE5D2;
	*mkptr( dfe_cfg_base, 0x0160060C ) = 0x00003A3E;
	*mkptr( dfe_cfg_base, 0x01600610 ) = 0x0001CDF3;
	*mkptr( dfe_cfg_base, 0x01600614 ) = 0x000017A4;
	*mkptr( dfe_cfg_base, 0x01600618 ) = 0xFFFFEEA9;
	*mkptr( dfe_cfg_base, 0x0160061C ) = 0x0000059A;
	*mkptr( dfe_cfg_base, 0x01600620 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01600624 ) = 0x000009AF;
	*mkptr( dfe_cfg_base, 0x01600628 ) = 0xFFFFC3B5;
	*mkptr( dfe_cfg_base, 0x0160062C ) = 0x000148E2;
	*mkptr( dfe_cfg_base, 0x01600630 ) = 0x00011BC6;
	*mkptr( dfe_cfg_base, 0x01600634 ) = 0xFFFFC300;
	*mkptr( dfe_cfg_base, 0x01600638 ) = 0x00000B1E;
	*mkptr( dfe_cfg_base, 0x0160063C ) = 0x0000002A;
	*mkptr( dfe_cfg_base, 0x01600B08 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01600B0C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B10 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B14 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B18 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B24 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01600B28 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B30 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600B34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01600A00 ) = 0x77FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A04 ) = 0x37FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A08 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A0C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A10 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A14 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A18 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A1C ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A28 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A2C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A30 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A34 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A80 ) = 0x77FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A84 ) = 0x37FFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A88 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A8C ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600A90 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A94 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A98 ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600A9C ) = 0xFFFFFFFF;
	*mkptr( dfe_cfg_base, 0x01600AA8 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AAC ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AB0 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01600AB4 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01640000 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640004 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640008 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164000C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640010 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640014 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640018 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164001C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640020 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640024 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640028 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164002C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640030 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640034 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640038 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164003C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640040 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640044 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640048 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164004C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640050 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640054 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640058 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164005C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640060 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640064 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640068 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164006C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640070 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640074 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640078 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164007C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640080 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640084 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640088 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164008C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640090 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640094 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640098 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164009C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016400FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640100 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640104 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640108 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164010C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640110 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640114 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640118 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164011C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640120 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640124 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640128 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164012C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640130 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640134 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640138 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164013C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640140 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640144 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640148 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164014C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640150 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640154 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640158 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164015C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640160 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640164 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640168 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164016C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640170 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640174 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640178 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164017C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640180 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640184 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640188 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164018C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640190 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640194 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640198 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164019C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016401FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640200 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640204 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640208 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164020C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640210 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640214 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640218 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164021C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640220 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640224 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640228 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164022C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640230 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640234 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640238 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164023C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640240 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640244 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640248 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164024C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640250 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640254 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640258 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164025C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640260 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640264 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640268 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164026C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640270 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640274 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640278 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164027C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640280 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640284 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640288 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164028C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640290 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640294 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640298 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164029C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016402FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640300 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640304 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640308 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164030C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640310 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640314 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640318 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164031C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640320 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640324 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640328 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164032C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640330 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640334 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640338 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164033C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640340 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640344 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640348 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164034C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640350 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640354 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640358 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164035C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640360 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640364 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640368 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164036C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640370 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640374 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640378 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164037C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640380 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640384 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640388 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164038C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640390 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640394 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640398 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164039C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016403FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640400 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640404 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640408 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164040C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640410 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640414 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640418 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164041C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640420 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640424 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640428 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164042C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640430 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640434 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640438 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164043C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640440 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640444 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640448 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164044C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640450 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640454 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640458 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164045C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640460 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640464 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640468 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164046C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640470 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640474 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640478 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164047C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640480 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640484 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640488 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164048C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640490 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640494 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640498 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164049C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016404FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640500 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640504 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640508 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164050C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640510 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640514 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640518 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164051C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640520 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640524 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640528 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164052C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640530 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640534 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640538 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164053C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640540 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640544 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640548 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164054C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640550 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640554 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640558 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164055C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640560 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640564 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640568 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164056C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640570 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640574 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640578 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164057C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640580 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640584 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640588 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164058C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640590 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640594 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640598 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164059C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016405FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640600 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640604 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640608 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164060C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640610 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640614 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640618 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164061C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640620 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640624 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640628 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164062C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640630 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640634 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640638 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164063C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640640 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640644 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640648 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164064C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640650 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640654 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640658 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164065C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640660 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640664 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640668 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164066C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640670 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640674 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640678 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164067C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640680 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640684 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640688 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164068C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640690 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640694 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640698 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164069C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016406FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640700 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640704 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640708 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164070C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640710 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640714 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640718 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164071C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640720 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640724 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640728 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164072C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640730 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640734 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640738 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164073C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640740 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640744 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640748 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164074C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640750 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640754 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640758 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164075C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640760 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640764 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640768 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164076C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640770 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640774 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640778 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164077C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640780 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640784 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640788 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164078C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640790 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640794 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640798 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164079C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016407FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640800 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640804 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640808 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164080C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640810 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640814 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640818 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164081C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640820 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640824 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640828 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164082C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640830 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640834 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640838 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164083C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640840 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640844 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640848 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164084C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640850 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640854 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640858 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164085C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640860 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640864 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640868 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164086C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640870 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640874 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640878 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164087C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640880 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640884 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640888 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164088C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640890 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640894 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640898 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164089C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016408FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640900 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640904 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640908 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164090C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640910 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640914 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640918 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164091C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640920 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640924 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640928 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164092C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640930 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640934 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640938 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164093C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640940 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640944 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640948 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164094C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640950 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640954 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640958 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164095C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640960 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640964 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640968 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164096C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640970 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640974 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640978 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164097C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640980 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640984 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640988 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164098C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640990 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640994 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640998 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x0164099C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x016409FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A00 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A04 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A08 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A0C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A10 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A14 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A18 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A1C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A20 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A24 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A28 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A2C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A30 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A34 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A38 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A3C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A40 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A44 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A48 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A4C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A50 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A54 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A58 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A5C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A60 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A64 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A68 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A6C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A70 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A74 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A78 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A7C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A80 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A84 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A88 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A8C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A90 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A94 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A98 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640A9C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AA8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AAC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AB8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ABC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AC8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ACC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AD8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640ADC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AE8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AEC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AF8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640AFC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B00 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B04 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B08 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B0C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B10 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B14 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B18 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B1C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B20 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B24 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B28 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B2C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B30 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B34 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B38 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B3C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B40 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B44 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B48 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B4C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B50 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B54 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B58 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B5C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B60 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B64 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B68 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B6C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B70 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B74 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B78 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B7C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B80 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B84 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B88 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B8C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B90 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B94 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B98 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640B9C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BA8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BAC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BB8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BBC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BC8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BCC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BD8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BDC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BE8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BEC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BF8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01640BFC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01D00014 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D00020 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D00048 ) = 0x0000EEEE;
	*mkptr( dfe_cfg_base, 0x01D00404 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00408 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0040C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00410 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00414 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00418 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0041C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00420 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00444 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00448 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0044C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00450 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00454 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00458 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0045C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00460 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00484 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00488 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0048C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D00490 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00494 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00498 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0049C ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004A0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D004C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004CC ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004D0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004D4 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D004D8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004DC ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D004E0 ) = 0x00005A5A;
	*mkptr( dfe_cfg_base, 0x01D00804 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00808 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0080C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00844 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00848 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0084C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00884 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D00888 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D0088C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D008C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D008C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D008CC ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D00C14 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D00C1C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C24 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C54 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D00C5C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C64 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C6C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D00C74 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D01404 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01408 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D0140C ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D01410 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D01808 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D0180C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D01810 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01814 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D01820 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D01824 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01848 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D0184C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D01850 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01854 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D01860 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D01864 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D01C04 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01C44 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01C84 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D01CC4 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D02004 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D02408 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01D0240C ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D02410 ) = 0x00000204;
	*mkptr( dfe_cfg_base, 0x01D02414 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02418 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01D0241C ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D02420 ) = 0x00000205;
	*mkptr( dfe_cfg_base, 0x01D02424 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01D02428 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x01D0242C ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x01D02430 ) = 0x00000206;
	*mkptr( dfe_cfg_base, 0x01D02434 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D02438 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x01D0243C ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x01D02440 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x01D02444 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01D02448 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01D0244C ) = 0x00000208;
	*mkptr( dfe_cfg_base, 0x01D02450 ) = 0x0000020C;
	*mkptr( dfe_cfg_base, 0x01D02454 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x01D02458 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x01D0245C ) = 0x00000209;
	*mkptr( dfe_cfg_base, 0x01D02460 ) = 0x0000020D;
	*mkptr( dfe_cfg_base, 0x01D02464 ) = 0x0000000A;
	*mkptr( dfe_cfg_base, 0x01D02468 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x01D0246C ) = 0x0000020A;
	*mkptr( dfe_cfg_base, 0x01D02470 ) = 0x0000020E;
	*mkptr( dfe_cfg_base, 0x01D02474 ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x01D02478 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01D0247C ) = 0x0000020B;
	*mkptr( dfe_cfg_base, 0x01D02480 ) = 0x0000020F;
	*mkptr( dfe_cfg_base, 0x01D02488 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01D0248C ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D02490 ) = 0x00000204;
	*mkptr( dfe_cfg_base, 0x01D02494 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02498 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01D0249C ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D024A0 ) = 0x00000205;
	*mkptr( dfe_cfg_base, 0x01D024A4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01D024A8 ) = 0x00000006;
	*mkptr( dfe_cfg_base, 0x01D024AC ) = 0x00000202;
	*mkptr( dfe_cfg_base, 0x01D024B0 ) = 0x00000206;
	*mkptr( dfe_cfg_base, 0x01D024B4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D024B8 ) = 0x00000007;
	*mkptr( dfe_cfg_base, 0x01D024BC ) = 0x00000203;
	*mkptr( dfe_cfg_base, 0x01D024C0 ) = 0x00000207;
	*mkptr( dfe_cfg_base, 0x01D024C4 ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01D024C8 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01D024CC ) = 0x00000208;
	*mkptr( dfe_cfg_base, 0x01D024D0 ) = 0x0000020C;
	*mkptr( dfe_cfg_base, 0x01D024D4 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x01D024D8 ) = 0x0000000D;
	*mkptr( dfe_cfg_base, 0x01D024DC ) = 0x00000209;
	*mkptr( dfe_cfg_base, 0x01D024E0 ) = 0x0000020D;
	*mkptr( dfe_cfg_base, 0x01D024E4 ) = 0x0000000A;
	*mkptr( dfe_cfg_base, 0x01D024E8 ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x01D024EC ) = 0x0000020A;
	*mkptr( dfe_cfg_base, 0x01D024F0 ) = 0x0000020E;
	*mkptr( dfe_cfg_base, 0x01D024F4 ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x01D024F8 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01D024FC ) = 0x0000020B;
	*mkptr( dfe_cfg_base, 0x01D02500 ) = 0x0000020F;
	*mkptr( dfe_cfg_base, 0x01D02804 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02808 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0280C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02810 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02814 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02818 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0281C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02820 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02824 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02828 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0282C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02830 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02834 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02838 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D0283C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D02840 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D40010 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01D40014 ) = 0x00000600;
	*mkptr( dfe_cfg_base, 0x01D40018 ) = 0x00000103;
	*mkptr( dfe_cfg_base, 0x01D40044 ) = 0x0000EE00;
	*mkptr( dfe_cfg_base, 0x01D40048 ) = 0x0000EEEE;
	*mkptr( dfe_cfg_base, 0x01D40804 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40808 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4080C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40844 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40848 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4084C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40884 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40888 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4088C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D408C4 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D408C8 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D408CC ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40904 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40908 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4090C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40944 ) = 0x0000A5A0;
	*mkptr( dfe_cfg_base, 0x01D40948 ) = 0x0000A5A5;
	*mkptr( dfe_cfg_base, 0x01D4094C ) = 0x0000005A;
	*mkptr( dfe_cfg_base, 0x01D40C14 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D40C1C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C24 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C2C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C34 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C54 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D40C5C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C64 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C6C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D40C74 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41004 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01D4100C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D41014 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4101C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41024 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D4102C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41034 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41044 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01D4104C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D41054 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4105C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41064 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D4106C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41074 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41084 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01D4108C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01D41094 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D4109C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410A4 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410AC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D410B4 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D41404 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41408 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D4140C ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D41410 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D41808 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4180C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D41810 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D41814 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D41820 ) = 0x00001CE0;
	*mkptr( dfe_cfg_base, 0x01D41824 ) = 0x000003FC;
	*mkptr( dfe_cfg_base, 0x01D41828 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41848 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4184C ) = 0x00001F07;
	*mkptr( dfe_cfg_base, 0x01D41850 ) = 0x00000F00;
	*mkptr( dfe_cfg_base, 0x01D41854 ) = 0x0000202F;
	*mkptr( dfe_cfg_base, 0x01D41860 ) = 0x00001CE0;
	*mkptr( dfe_cfg_base, 0x01D41864 ) = 0x000003FC;
	*mkptr( dfe_cfg_base, 0x01D41868 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D41C04 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41C44 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41C84 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D41CC4 ) = 0x00001FFC;
	*mkptr( dfe_cfg_base, 0x01D42004 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D42808 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D42810 ) = 0x00000200;
	*mkptr( dfe_cfg_base, 0x01D42844 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D42848 ) = 0x00000210;
	*mkptr( dfe_cfg_base, 0x01D4284C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01D42850 ) = 0x00000210;
	*mkptr( dfe_cfg_base, 0x01D42884 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D42888 ) = 0x00000220;
	*mkptr( dfe_cfg_base, 0x01D4288C ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01D42890 ) = 0x00000220;
	*mkptr( dfe_cfg_base, 0x01D428C4 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01D428C8 ) = 0x00000230;
	*mkptr( dfe_cfg_base, 0x01D428CC ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01D428D0 ) = 0x00000230;
	*mkptr( dfe_cfg_base, 0x01D42904 ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01D42908 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D4290C ) = 0x00000100;
	*mkptr( dfe_cfg_base, 0x01D42910 ) = 0x00000300;
	*mkptr( dfe_cfg_base, 0x01D42944 ) = 0x00000110;
	*mkptr( dfe_cfg_base, 0x01D42948 ) = 0x00000310;
	*mkptr( dfe_cfg_base, 0x01D4294C ) = 0x00000110;
	*mkptr( dfe_cfg_base, 0x01D42950 ) = 0x00000310;
	*mkptr( dfe_cfg_base, 0x01D42984 ) = 0x00000120;
	*mkptr( dfe_cfg_base, 0x01D42988 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x01D4298C ) = 0x00000120;
	*mkptr( dfe_cfg_base, 0x01D42990 ) = 0x00000320;
	*mkptr( dfe_cfg_base, 0x01D429C4 ) = 0x00000130;
	*mkptr( dfe_cfg_base, 0x01D429C8 ) = 0x00000330;
	*mkptr( dfe_cfg_base, 0x01D429CC ) = 0x00000130;
	*mkptr( dfe_cfg_base, 0x01D429D0 ) = 0x00000330;
	*mkptr( dfe_cfg_base, 0x01D42A04 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42A08 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42A0C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42A10 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42A44 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42A48 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42A4C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42A50 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42A84 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42A88 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42A8C ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42A90 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42AC4 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42AC8 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42ACC ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42AD0 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42B04 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42B08 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42B0C ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42B10 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42B44 ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42B48 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42B4C ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42B50 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42B84 ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42B88 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42B8C ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42B90 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42BC4 ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42BC8 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01D42BCC ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42BD0 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01D42C04 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42C08 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42C0C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01D42C10 ) = 0x00000201;
	*mkptr( dfe_cfg_base, 0x01D42C44 ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42C48 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42C4C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01D42C50 ) = 0x00000211;
	*mkptr( dfe_cfg_base, 0x01D42C84 ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42C88 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42C8C ) = 0x00000021;
	*mkptr( dfe_cfg_base, 0x01D42C90 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01D42CC4 ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42CC8 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42CCC ) = 0x00000031;
	*mkptr( dfe_cfg_base, 0x01D42CD0 ) = 0x00000231;
	*mkptr( dfe_cfg_base, 0x01D42D04 ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42D08 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42D0C ) = 0x00000101;
	*mkptr( dfe_cfg_base, 0x01D42D10 ) = 0x00000301;
	*mkptr( dfe_cfg_base, 0x01D42D44 ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42D48 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42D4C ) = 0x00000111;
	*mkptr( dfe_cfg_base, 0x01D42D50 ) = 0x00000311;
	*mkptr( dfe_cfg_base, 0x01D42D84 ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42D88 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42D8C ) = 0x00000121;
	*mkptr( dfe_cfg_base, 0x01D42D90 ) = 0x00000321;
	*mkptr( dfe_cfg_base, 0x01D42DC4 ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42DC8 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01D42DCC ) = 0x00000131;
	*mkptr( dfe_cfg_base, 0x01D42DD0 ) = 0x00000331;
	*mkptr( dfe_cfg_base, 0x01A00E00 ) = 0x00000470;
	*mkptr( dfe_cfg_base, 0x01A00200 ) = 0x00000221;
	*mkptr( dfe_cfg_base, 0x01A00204 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01A006A0 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A0022C ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A00230 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A41000 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41040 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41080 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A410C0 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41100 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x01A41108 ) = 0x000007D0;
	*mkptr( dfe_cfg_base, 0x01A007EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A007FC ) = 0x00003000;
	*mkptr( dfe_cfg_base, 0x01A00604 ) = 0x0000FF00;
	*mkptr( dfe_cfg_base, 0x01A00648 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01A41800 ) = 0x0000007F;
	*mkptr( dfe_cfg_base, 0x01A41804 ) = 0x0000006F;
	*mkptr( dfe_cfg_base, 0x01A41808 ) = 0x00000062;
	*mkptr( dfe_cfg_base, 0x01A4180C ) = 0x00000057;
	*mkptr( dfe_cfg_base, 0x01A41810 ) = 0x0000004F;
	*mkptr( dfe_cfg_base, 0x01A41814 ) = 0x00000047;
	*mkptr( dfe_cfg_base, 0x01A41818 ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A4181C ) = 0x0000003A;
	*mkptr( dfe_cfg_base, 0x01A41820 ) = 0x00000035;
	*mkptr( dfe_cfg_base, 0x01A41824 ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41828 ) = 0x0000002B;
	*mkptr( dfe_cfg_base, 0x01A4182C ) = 0x00000027;
	*mkptr( dfe_cfg_base, 0x01A41830 ) = 0x00000023;
	*mkptr( dfe_cfg_base, 0x01A41834 ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41838 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A4183C ) = 0x00000019;
	*mkptr( dfe_cfg_base, 0x01A41840 ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x01A41844 ) = 0x00000013;
	*mkptr( dfe_cfg_base, 0x01A41848 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A4184C ) = 0x0000000E;
	*mkptr( dfe_cfg_base, 0x01A41850 ) = 0x0000000B;
	*mkptr( dfe_cfg_base, 0x01A41854 ) = 0x00000009;
	*mkptr( dfe_cfg_base, 0x01A41874 ) = 0x000000F9;
	*mkptr( dfe_cfg_base, 0x01A41878 ) = 0x000000F7;
	*mkptr( dfe_cfg_base, 0x01A4187C ) = 0x000000F5;
	*mkptr( dfe_cfg_base, 0x01A41880 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41884 ) = 0x000000F2;
	*mkptr( dfe_cfg_base, 0x01A41888 ) = 0x000000F1;
	*mkptr( dfe_cfg_base, 0x01A4188C ) = 0x000000EF;
	*mkptr( dfe_cfg_base, 0x01A41890 ) = 0x000000EE;
	*mkptr( dfe_cfg_base, 0x01A41894 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41898 ) = 0x000000EB;
	*mkptr( dfe_cfg_base, 0x01A4189C ) = 0x000000EA;
	*mkptr( dfe_cfg_base, 0x01A418A0 ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A418A4 ) = 0x000000E7;
	*mkptr( dfe_cfg_base, 0x01A418A8 ) = 0x000000E6;
	*mkptr( dfe_cfg_base, 0x01A418AC ) = 0x000000E5;
	*mkptr( dfe_cfg_base, 0x01A418B0 ) = 0x000000E3;
	*mkptr( dfe_cfg_base, 0x01A418B4 ) = 0x000000E2;
	*mkptr( dfe_cfg_base, 0x01A418B8 ) = 0x000000E1;
	*mkptr( dfe_cfg_base, 0x01A418BC ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A418C0 ) = 0x000000DF;
	*mkptr( dfe_cfg_base, 0x01A418C4 ) = 0x000000DE;
	*mkptr( dfe_cfg_base, 0x01A418C8 ) = 0x000000DD;
	*mkptr( dfe_cfg_base, 0x01A418CC ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A418D0 ) = 0x000000DB;
	*mkptr( dfe_cfg_base, 0x01A418D4 ) = 0x000000DA;
	*mkptr( dfe_cfg_base, 0x01A418D8 ) = 0x000000D9;
	*mkptr( dfe_cfg_base, 0x01A418DC ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A418E0 ) = 0x000000D7;
	*mkptr( dfe_cfg_base, 0x01A418E4 ) = 0x000000D6;
	*mkptr( dfe_cfg_base, 0x01A418E8 ) = 0x000000D5;
	*mkptr( dfe_cfg_base, 0x01A418EC ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A418F0 ) = 0x000000D3;
	*mkptr( dfe_cfg_base, 0x01A418F4 ) = 0x000000D2;
	*mkptr( dfe_cfg_base, 0x01A418F8 ) = 0x000000D1;
	*mkptr( dfe_cfg_base, 0x01A418FC ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41900 ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41904 ) = 0x000000CF;
	*mkptr( dfe_cfg_base, 0x01A41908 ) = 0x000000CE;
	*mkptr( dfe_cfg_base, 0x01A4190C ) = 0x000000CD;
	*mkptr( dfe_cfg_base, 0x01A41910 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41914 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41918 ) = 0x000000CB;
	*mkptr( dfe_cfg_base, 0x01A4191C ) = 0x000000CA;
	*mkptr( dfe_cfg_base, 0x01A41920 ) = 0x000000C9;
	*mkptr( dfe_cfg_base, 0x01A41924 ) = 0x000000C9;
	*mkptr( dfe_cfg_base, 0x01A41928 ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A4192C ) = 0x000000C7;
	*mkptr( dfe_cfg_base, 0x01A41930 ) = 0x000000C6;
	*mkptr( dfe_cfg_base, 0x01A41934 ) = 0x000000C6;
	*mkptr( dfe_cfg_base, 0x01A41938 ) = 0x000000C5;
	*mkptr( dfe_cfg_base, 0x01A4193C ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41940 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41944 ) = 0x000000C3;
	*mkptr( dfe_cfg_base, 0x01A41948 ) = 0x000000C2;
	*mkptr( dfe_cfg_base, 0x01A4194C ) = 0x000000C2;
	*mkptr( dfe_cfg_base, 0x01A41950 ) = 0x000000C1;
	*mkptr( dfe_cfg_base, 0x01A41954 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41958 ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A4195C ) = 0x000000BF;
	*mkptr( dfe_cfg_base, 0x01A41960 ) = 0x000000BE;
	*mkptr( dfe_cfg_base, 0x01A41964 ) = 0x000000BE;
	*mkptr( dfe_cfg_base, 0x01A41968 ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x01A4196C ) = 0x000000BD;
	*mkptr( dfe_cfg_base, 0x01A41970 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41974 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41978 ) = 0x000000BB;
	*mkptr( dfe_cfg_base, 0x01A4197C ) = 0x000000BA;
	*mkptr( dfe_cfg_base, 0x01A41980 ) = 0x000000BA;
	*mkptr( dfe_cfg_base, 0x01A41984 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x01A41988 ) = 0x000000B9;
	*mkptr( dfe_cfg_base, 0x01A4198C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41990 ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41994 ) = 0x000000B7;
	*mkptr( dfe_cfg_base, 0x01A41998 ) = 0x000000B7;
	*mkptr( dfe_cfg_base, 0x01A4199C ) = 0x000000B6;
	*mkptr( dfe_cfg_base, 0x01A419A0 ) = 0x000000B5;
	*mkptr( dfe_cfg_base, 0x01A419A4 ) = 0x000000B5;
	*mkptr( dfe_cfg_base, 0x01A419A8 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A419AC ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A419B0 ) = 0x000000B3;
	*mkptr( dfe_cfg_base, 0x01A419B4 ) = 0x000000B3;
	*mkptr( dfe_cfg_base, 0x01A419B8 ) = 0x000000B2;
	*mkptr( dfe_cfg_base, 0x01A419BC ) = 0x000000B2;
	*mkptr( dfe_cfg_base, 0x01A419C0 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x01A419C4 ) = 0x000000B1;
	*mkptr( dfe_cfg_base, 0x01A419C8 ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419CC ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419D0 ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A419D4 ) = 0x000000AF;
	*mkptr( dfe_cfg_base, 0x01A419D8 ) = 0x000000AF;
	*mkptr( dfe_cfg_base, 0x01A419DC ) = 0x000000AE;
	*mkptr( dfe_cfg_base, 0x01A419E0 ) = 0x000000AE;
	*mkptr( dfe_cfg_base, 0x01A419E4 ) = 0x000000AD;
	*mkptr( dfe_cfg_base, 0x01A419E8 ) = 0x000000AD;
	*mkptr( dfe_cfg_base, 0x01A419EC ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A419F0 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A419F4 ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A419F8 ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A419FC ) = 0x000000AB;
	*mkptr( dfe_cfg_base, 0x01A41A00 ) = 0x000000AA;
	*mkptr( dfe_cfg_base, 0x01A41A04 ) = 0x000000AA;
	*mkptr( dfe_cfg_base, 0x01A41A08 ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A0C ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A10 ) = 0x000000A9;
	*mkptr( dfe_cfg_base, 0x01A41A14 ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41A18 ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41A1C ) = 0x000000A7;
	*mkptr( dfe_cfg_base, 0x01A41A20 ) = 0x000000A7;
	*mkptr( dfe_cfg_base, 0x01A41A24 ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A28 ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A2C ) = 0x000000A6;
	*mkptr( dfe_cfg_base, 0x01A41A30 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A34 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A38 ) = 0x000000A5;
	*mkptr( dfe_cfg_base, 0x01A41A3C ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41A40 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41A44 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A48 ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A4C ) = 0x000000A3;
	*mkptr( dfe_cfg_base, 0x01A41A50 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A54 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A58 ) = 0x000000A2;
	*mkptr( dfe_cfg_base, 0x01A41A5C ) = 0x000000A1;
	*mkptr( dfe_cfg_base, 0x01A41A60 ) = 0x000000A1;
	*mkptr( dfe_cfg_base, 0x01A41A64 ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A68 ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A6C ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41A70 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A74 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A78 ) = 0x0000009F;
	*mkptr( dfe_cfg_base, 0x01A41A7C ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A80 ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A84 ) = 0x0000009E;
	*mkptr( dfe_cfg_base, 0x01A41A88 ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A8C ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A90 ) = 0x0000009D;
	*mkptr( dfe_cfg_base, 0x01A41A94 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41A98 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41A9C ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41AA0 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AA4 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AA8 ) = 0x0000009B;
	*mkptr( dfe_cfg_base, 0x01A41AAC ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB0 ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB4 ) = 0x0000009A;
	*mkptr( dfe_cfg_base, 0x01A41AB8 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41ABC ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC0 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC4 ) = 0x00000099;
	*mkptr( dfe_cfg_base, 0x01A41AC8 ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41ACC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41AD0 ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41AD4 ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41AD8 ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41ADC ) = 0x00000097;
	*mkptr( dfe_cfg_base, 0x01A41AE0 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AE4 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AE8 ) = 0x00000096;
	*mkptr( dfe_cfg_base, 0x01A41AEC ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF0 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF4 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AF8 ) = 0x00000095;
	*mkptr( dfe_cfg_base, 0x01A41AFC ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B00 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B04 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41B08 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B0C ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B10 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B14 ) = 0x00000093;
	*mkptr( dfe_cfg_base, 0x01A41B18 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B1C ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B20 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B24 ) = 0x00000092;
	*mkptr( dfe_cfg_base, 0x01A41B28 ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B2C ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B30 ) = 0x00000091;
	*mkptr( dfe_cfg_base, 0x01A41B34 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B38 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B3C ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B40 ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41B44 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B48 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B4C ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B50 ) = 0x0000008F;
	*mkptr( dfe_cfg_base, 0x01A41B54 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B58 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B5C ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B60 ) = 0x0000008E;
	*mkptr( dfe_cfg_base, 0x01A41B64 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B68 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B6C ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B70 ) = 0x0000008D;
	*mkptr( dfe_cfg_base, 0x01A41B74 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B78 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B7C ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B80 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41B84 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B88 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B8C ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B90 ) = 0x0000008B;
	*mkptr( dfe_cfg_base, 0x01A41B94 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41B98 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41B9C ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41BA0 ) = 0x0000008A;
	*mkptr( dfe_cfg_base, 0x01A41BA4 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BA8 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BAC ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BB0 ) = 0x00000089;
	*mkptr( dfe_cfg_base, 0x01A41BB4 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BB8 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BBC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC0 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC4 ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41BC8 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BCC ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD0 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD4 ) = 0x00000087;
	*mkptr( dfe_cfg_base, 0x01A41BD8 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BDC ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE0 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE4 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BE8 ) = 0x00000086;
	*mkptr( dfe_cfg_base, 0x01A41BEC ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF0 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF4 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BF8 ) = 0x00000085;
	*mkptr( dfe_cfg_base, 0x01A41BFC ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41C00 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A41C04 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A41C08 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A41C0C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A41C10 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A41C14 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41C18 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A41C1C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A41C20 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A41C24 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41C28 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A41C2C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A41C30 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A41C34 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A41C38 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A41C3C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A41C40 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A41C44 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A41C48 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A41C4C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A41C50 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A41C54 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A41C58 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A41C5C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41C60 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A41C64 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41C68 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A41C6C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A41C70 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A41C74 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41C78 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A41C7C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41C80 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A41C84 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41C88 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A41C8C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41C90 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A41C94 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A41C98 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A41C9C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A41CA0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A41CA4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A41CA8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A41CAC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41CB0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A41CB4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41CB8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A41CBC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41CC0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A41CC4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41CC8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A41CCC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41CD0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A41CD4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41CD8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A41CDC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41CE0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A41CE4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41CE8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A41CEC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41CF0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A41CF4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41CF8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A41CFC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A41D00 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A41D04 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A41D08 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A41D0C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A41D10 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A41D14 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A41D18 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A41D1C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A41D20 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A41D24 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A41D28 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A41D2C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A41D30 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A41D34 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A41D38 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A41D3C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A41D40 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A41D44 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A41D48 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A41D4C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A41D50 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A41D54 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A41D58 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A41D5C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A41D60 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A41D64 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A41D68 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A41D6C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A41D70 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A41D74 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A41D78 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A41D7C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A41D80 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A41D84 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A41D88 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A41D8C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A41D90 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A41D94 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A41D98 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A41D9C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41DA0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A41DA4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A41DA8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A41DAC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A41DB0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A41DB4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A41DB8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A41DBC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41DC0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A41DC4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A41DC8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A41DCC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A41DD0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A41DD4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A41DD8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A41DDC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41DE0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A41DE4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A41DE8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A41DEC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A41DF0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A41DF4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A41DF8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A41E00 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A41E04 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A41E08 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A41E0C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A41E10 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A41E14 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A41E18 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A41E1C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A41E20 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A41E24 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A41E28 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A41E2C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A41E30 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A41E34 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A41E38 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A41E3C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A41E40 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A41E44 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A41E48 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A41E4C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A41E50 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A41E54 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A41E58 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A41E5C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A41E60 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A41E64 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A41E68 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A41E6C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A41E70 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A41E74 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A41E78 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A41E7C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A41E80 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A41E84 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A41E88 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A41E8C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A41E90 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A41E94 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A41E98 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A41E9C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A41EA0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A41EA4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A41EA8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A41EAC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A41EB0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A41EB4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A41EB8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A41EBC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A41EC0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A41EC4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A41EC8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A41ECC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A41ED0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A41ED4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A41ED8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A41EDC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A41EE0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A41EE4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A41EE8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A41EEC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A41EF0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A41EF4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A41EF8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A41EFC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A41F00 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A41F04 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A41F08 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A41F0C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A41F10 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A41F14 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A41F18 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A41F1C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A41F20 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A41F24 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A41F28 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A41F2C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A41F30 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A41F34 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A41F38 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A41F3C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A41F40 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A41F44 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A41F48 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A41F4C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A41F50 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A41F54 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A41F58 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A41F5C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A41F60 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A41F64 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A41F68 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A41F6C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A41F70 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A41F74 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A41F78 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A41F7C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A41F80 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A41F84 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A41F88 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A41F8C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A41F90 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A41F94 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A41F98 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A41F9C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A41FA0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A41FA4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A41FA8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A41FAC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A41FB0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A41FB4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A41FB8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A41FBC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A41FC0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A41FC4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A41FC8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A41FCC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A41FD0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A41FD4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A41FD8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A41FDC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41FE0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A41FE4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A41FE8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A41FEC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A41FF0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A41FF4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A41FF8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A42000 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A42004 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A42008 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A4200C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A42010 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A42014 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A42018 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A4201C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A42020 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A42024 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A42028 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A4202C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A42030 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A42034 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A42038 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A4203C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A42040 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A42044 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A42048 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A4204C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A42050 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A42054 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A42058 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A4205C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A42060 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A42064 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A42068 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A4206C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A42070 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A42074 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A42078 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A4207C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A42080 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A42084 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A42088 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A4208C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A42090 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A42094 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A42098 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A4209C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A420A0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A420A4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A420A8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A420AC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A420B0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A420B4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A420B8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A420BC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A420C0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A420C4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A420C8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A420CC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A420D0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A420D4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A420D8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A420DC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A420E0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A420E4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A420E8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A420EC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A420F0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A420F4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A420F8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A420FC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A42100 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A42104 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A42108 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A4210C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A42110 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A42114 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A42118 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A4211C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A42120 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A42124 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A42128 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A4212C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A42130 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A42134 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A42138 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A4213C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A42140 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A42144 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A42148 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A4214C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A42150 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A42154 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A42158 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A4215C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A42160 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A42164 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A42168 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A4216C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A42170 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A42174 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A42178 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A4217C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A42180 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A42184 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A42188 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A4218C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A42190 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A42194 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A42198 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A4219C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A421A0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A421A4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A421A8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A421AC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A421B0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A421B4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A421B8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A421BC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A421C0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A421C4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A421C8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A421CC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A421D0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A421D4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A421D8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A421DC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A421E0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A421E4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A421E8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A421EC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A421F0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A421F4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A421F8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A42200 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A42204 ) = 0x000000FC;
	*mkptr( dfe_cfg_base, 0x01A42208 ) = 0x000078D7;
	*mkptr( dfe_cfg_base, 0x01A4220C ) = 0x000000F8;
	*mkptr( dfe_cfg_base, 0x01A42210 ) = 0x00007215;
	*mkptr( dfe_cfg_base, 0x01A42214 ) = 0x000000F4;
	*mkptr( dfe_cfg_base, 0x01A42218 ) = 0x00006BB3;
	*mkptr( dfe_cfg_base, 0x01A4221C ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A42220 ) = 0x000065AD;
	*mkptr( dfe_cfg_base, 0x01A42224 ) = 0x000000EC;
	*mkptr( dfe_cfg_base, 0x01A42228 ) = 0x00005FFD;
	*mkptr( dfe_cfg_base, 0x01A4222C ) = 0x000000E8;
	*mkptr( dfe_cfg_base, 0x01A42230 ) = 0x00005A9E;
	*mkptr( dfe_cfg_base, 0x01A42234 ) = 0x000000E4;
	*mkptr( dfe_cfg_base, 0x01A42238 ) = 0x0000558C;
	*mkptr( dfe_cfg_base, 0x01A4223C ) = 0x000000E0;
	*mkptr( dfe_cfg_base, 0x01A42240 ) = 0x000050C3;
	*mkptr( dfe_cfg_base, 0x01A42244 ) = 0x000000DC;
	*mkptr( dfe_cfg_base, 0x01A42248 ) = 0x00004C3F;
	*mkptr( dfe_cfg_base, 0x01A4224C ) = 0x000000D8;
	*mkptr( dfe_cfg_base, 0x01A42250 ) = 0x000047FB;
	*mkptr( dfe_cfg_base, 0x01A42254 ) = 0x000000D4;
	*mkptr( dfe_cfg_base, 0x01A42258 ) = 0x000043F4;
	*mkptr( dfe_cfg_base, 0x01A4225C ) = 0x000000D0;
	*mkptr( dfe_cfg_base, 0x01A42260 ) = 0x00004027;
	*mkptr( dfe_cfg_base, 0x01A42264 ) = 0x000000CC;
	*mkptr( dfe_cfg_base, 0x01A42268 ) = 0x00003C90;
	*mkptr( dfe_cfg_base, 0x01A4226C ) = 0x000000C8;
	*mkptr( dfe_cfg_base, 0x01A42270 ) = 0x0000392D;
	*mkptr( dfe_cfg_base, 0x01A42274 ) = 0x000000C4;
	*mkptr( dfe_cfg_base, 0x01A42278 ) = 0x000035FA;
	*mkptr( dfe_cfg_base, 0x01A4227C ) = 0x000000C0;
	*mkptr( dfe_cfg_base, 0x01A42280 ) = 0x000032F5;
	*mkptr( dfe_cfg_base, 0x01A42284 ) = 0x000000BC;
	*mkptr( dfe_cfg_base, 0x01A42288 ) = 0x0000301B;
	*mkptr( dfe_cfg_base, 0x01A4228C ) = 0x000000B8;
	*mkptr( dfe_cfg_base, 0x01A42290 ) = 0x00002D6B;
	*mkptr( dfe_cfg_base, 0x01A42294 ) = 0x000000B4;
	*mkptr( dfe_cfg_base, 0x01A42298 ) = 0x00002AE0;
	*mkptr( dfe_cfg_base, 0x01A4229C ) = 0x000000B0;
	*mkptr( dfe_cfg_base, 0x01A422A0 ) = 0x0000287A;
	*mkptr( dfe_cfg_base, 0x01A422A4 ) = 0x000000AC;
	*mkptr( dfe_cfg_base, 0x01A422A8 ) = 0x00002637;
	*mkptr( dfe_cfg_base, 0x01A422AC ) = 0x000000A8;
	*mkptr( dfe_cfg_base, 0x01A422B0 ) = 0x00002413;
	*mkptr( dfe_cfg_base, 0x01A422B4 ) = 0x000000A4;
	*mkptr( dfe_cfg_base, 0x01A422B8 ) = 0x0000220F;
	*mkptr( dfe_cfg_base, 0x01A422BC ) = 0x000000A0;
	*mkptr( dfe_cfg_base, 0x01A422C0 ) = 0x00002027;
	*mkptr( dfe_cfg_base, 0x01A422C4 ) = 0x0000009C;
	*mkptr( dfe_cfg_base, 0x01A422C8 ) = 0x00001E5B;
	*mkptr( dfe_cfg_base, 0x01A422CC ) = 0x00000098;
	*mkptr( dfe_cfg_base, 0x01A422D0 ) = 0x00001CA8;
	*mkptr( dfe_cfg_base, 0x01A422D4 ) = 0x00000094;
	*mkptr( dfe_cfg_base, 0x01A422D8 ) = 0x00001B0D;
	*mkptr( dfe_cfg_base, 0x01A422DC ) = 0x00000090;
	*mkptr( dfe_cfg_base, 0x01A422E0 ) = 0x0000198A;
	*mkptr( dfe_cfg_base, 0x01A422E4 ) = 0x0000008C;
	*mkptr( dfe_cfg_base, 0x01A422E8 ) = 0x0000181C;
	*mkptr( dfe_cfg_base, 0x01A422EC ) = 0x00000088;
	*mkptr( dfe_cfg_base, 0x01A422F0 ) = 0x000016C3;
	*mkptr( dfe_cfg_base, 0x01A422F4 ) = 0x00000084;
	*mkptr( dfe_cfg_base, 0x01A422F8 ) = 0x0000157D;
	*mkptr( dfe_cfg_base, 0x01A422FC ) = 0x00000080;
	*mkptr( dfe_cfg_base, 0x01A42300 ) = 0x00001449;
	*mkptr( dfe_cfg_base, 0x01A42304 ) = 0x0000007C;
	*mkptr( dfe_cfg_base, 0x01A42308 ) = 0x00001327;
	*mkptr( dfe_cfg_base, 0x01A4230C ) = 0x00000078;
	*mkptr( dfe_cfg_base, 0x01A42310 ) = 0x00001215;
	*mkptr( dfe_cfg_base, 0x01A42314 ) = 0x00000074;
	*mkptr( dfe_cfg_base, 0x01A42318 ) = 0x00001112;
	*mkptr( dfe_cfg_base, 0x01A4231C ) = 0x00000070;
	*mkptr( dfe_cfg_base, 0x01A42320 ) = 0x0000101D;
	*mkptr( dfe_cfg_base, 0x01A42324 ) = 0x0000006C;
	*mkptr( dfe_cfg_base, 0x01A42328 ) = 0x00000F36;
	*mkptr( dfe_cfg_base, 0x01A4232C ) = 0x00000068;
	*mkptr( dfe_cfg_base, 0x01A42330 ) = 0x00000E5D;
	*mkptr( dfe_cfg_base, 0x01A42334 ) = 0x00000064;
	*mkptr( dfe_cfg_base, 0x01A42338 ) = 0x00000D8F;
	*mkptr( dfe_cfg_base, 0x01A4233C ) = 0x00000060;
	*mkptr( dfe_cfg_base, 0x01A42340 ) = 0x00000CCD;
	*mkptr( dfe_cfg_base, 0x01A42344 ) = 0x0000005C;
	*mkptr( dfe_cfg_base, 0x01A42348 ) = 0x00000C15;
	*mkptr( dfe_cfg_base, 0x01A4234C ) = 0x00000058;
	*mkptr( dfe_cfg_base, 0x01A42350 ) = 0x00000B68;
	*mkptr( dfe_cfg_base, 0x01A42354 ) = 0x00000054;
	*mkptr( dfe_cfg_base, 0x01A42358 ) = 0x00000AC5;
	*mkptr( dfe_cfg_base, 0x01A4235C ) = 0x00000050;
	*mkptr( dfe_cfg_base, 0x01A42360 ) = 0x00000A2B;
	*mkptr( dfe_cfg_base, 0x01A42364 ) = 0x0000004C;
	*mkptr( dfe_cfg_base, 0x01A42368 ) = 0x00000999;
	*mkptr( dfe_cfg_base, 0x01A4236C ) = 0x00000048;
	*mkptr( dfe_cfg_base, 0x01A42370 ) = 0x00000910;
	*mkptr( dfe_cfg_base, 0x01A42374 ) = 0x00000044;
	*mkptr( dfe_cfg_base, 0x01A42378 ) = 0x0000088E;
	*mkptr( dfe_cfg_base, 0x01A4237C ) = 0x00000040;
	*mkptr( dfe_cfg_base, 0x01A42380 ) = 0x00000814;
	*mkptr( dfe_cfg_base, 0x01A42384 ) = 0x0000003C;
	*mkptr( dfe_cfg_base, 0x01A42388 ) = 0x000007A0;
	*mkptr( dfe_cfg_base, 0x01A4238C ) = 0x00000038;
	*mkptr( dfe_cfg_base, 0x01A42390 ) = 0x00000733;
	*mkptr( dfe_cfg_base, 0x01A42394 ) = 0x00000034;
	*mkptr( dfe_cfg_base, 0x01A42398 ) = 0x000006CC;
	*mkptr( dfe_cfg_base, 0x01A4239C ) = 0x00000030;
	*mkptr( dfe_cfg_base, 0x01A423A0 ) = 0x0000066A;
	*mkptr( dfe_cfg_base, 0x01A423A4 ) = 0x0000002C;
	*mkptr( dfe_cfg_base, 0x01A423A8 ) = 0x0000060E;
	*mkptr( dfe_cfg_base, 0x01A423AC ) = 0x00000028;
	*mkptr( dfe_cfg_base, 0x01A423B0 ) = 0x000005B8;
	*mkptr( dfe_cfg_base, 0x01A423B4 ) = 0x00000024;
	*mkptr( dfe_cfg_base, 0x01A423B8 ) = 0x00000566;
	*mkptr( dfe_cfg_base, 0x01A423BC ) = 0x00000020;
	*mkptr( dfe_cfg_base, 0x01A423C0 ) = 0x00000519;
	*mkptr( dfe_cfg_base, 0x01A423C4 ) = 0x0000001C;
	*mkptr( dfe_cfg_base, 0x01A423C8 ) = 0x000004D0;
	*mkptr( dfe_cfg_base, 0x01A423CC ) = 0x00000018;
	*mkptr( dfe_cfg_base, 0x01A423D0 ) = 0x0000048B;
	*mkptr( dfe_cfg_base, 0x01A423D4 ) = 0x00000014;
	*mkptr( dfe_cfg_base, 0x01A423D8 ) = 0x0000044A;
	*mkptr( dfe_cfg_base, 0x01A423DC ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A423E0 ) = 0x0000040C;
	*mkptr( dfe_cfg_base, 0x01A423E4 ) = 0x0000000C;
	*mkptr( dfe_cfg_base, 0x01A423E8 ) = 0x000003D2;
	*mkptr( dfe_cfg_base, 0x01A423EC ) = 0x00000008;
	*mkptr( dfe_cfg_base, 0x01A423F0 ) = 0x0000039C;
	*mkptr( dfe_cfg_base, 0x01A423F4 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A423F8 ) = 0x00000368;
	*mkptr( dfe_cfg_base, 0x01A41400 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41440 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41480 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A414C0 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A41500 ) = 0x000003E8;
	*mkptr( dfe_cfg_base, 0x01A41508 ) = 0x000007D0;
	*mkptr( dfe_cfg_base, 0x01A006EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A006FC ) = 0x00003000;
	*mkptr( dfe_cfg_base, 0x01A00A0C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A00A14 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A0020C ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01A40220 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A402A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40320 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A403A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40020 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A400A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A40120 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A401A0 ) = 0x00007FFF;
	*mkptr( dfe_cfg_base, 0x01A0021C ) = 0x00000011;
	*mkptr( dfe_cfg_base, 0x01A00220 ) = 0x000007B0;
	*mkptr( dfe_cfg_base, 0x01A00224 ) = 0x000000F0;
	*mkptr( dfe_cfg_base, 0x01A00228 ) = 0x00001280;
	*mkptr( dfe_cfg_base, 0x01A002D0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002D8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002C0 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A002C8 ) = 0x00004000;
	*mkptr( dfe_cfg_base, 0x01A40800 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40804 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40808 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A4080C ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40810 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40814 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40818 ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A4081C ) = 0x00000005;
	*mkptr( dfe_cfg_base, 0x01A40820 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40824 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40828 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4082C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40830 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40834 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40838 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4083C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40840 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40844 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40848 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4084C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40850 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40854 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40858 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4085C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40860 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40864 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40868 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4086C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40870 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40874 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40878 ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A4087C ) = 0x00000004;
	*mkptr( dfe_cfg_base, 0x01A40880 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40884 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40888 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4088C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40890 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40894 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40898 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4089C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408A8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408AC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408B8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408BC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408C8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408CC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408D8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408E8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408EC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408F8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A408FC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40900 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40904 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40908 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4090C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40910 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40914 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40918 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4091C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40920 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40924 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40928 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4092C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40930 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40934 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40938 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4093C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40940 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40944 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40948 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4094C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40950 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40954 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40958 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4095C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40960 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40964 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40968 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4096C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40970 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40974 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40978 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4097C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40980 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40984 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40988 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4098C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40990 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40994 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40998 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A4099C ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409A8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409AC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409B8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409BC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409C8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409CC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409D8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409DC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409E8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409EC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F0 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F4 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409F8 ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A409FC ) = 0x00000003;
	*mkptr( dfe_cfg_base, 0x01A40A00 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A04 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A08 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A0C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A10 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A14 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A18 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A1C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A20 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A24 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A28 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A2C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A30 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A34 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A38 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A3C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A40 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A44 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A48 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A4C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A50 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A54 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A58 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A5C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A60 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A64 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A68 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A6C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A70 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A74 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A78 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A7C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A80 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A84 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A88 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A8C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A90 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A94 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A98 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40A9C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AA8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AAC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AB8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ABC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AC8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ACC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AD8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40ADC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AE8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AEC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AF8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40AFC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B00 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B04 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B08 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B0C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B10 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B14 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B18 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B1C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B20 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B24 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B28 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B2C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B30 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B34 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B38 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B3C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B40 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B44 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B48 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B4C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B50 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B54 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B58 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B5C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B60 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B64 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B68 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B6C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B70 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B74 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B78 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B7C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B80 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B84 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B88 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B8C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B90 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B94 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B98 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40B9C ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BA8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BAC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BB8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BBC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BC8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BCC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BD8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BDC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BE8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BEC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF0 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF4 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BF8 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A40BFC ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01A003EC ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003F0 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01A003FC ) = 0x00003000;
	*mkptr( dfe_cfg_base, 0x01A0023C ) = 0x00005110;
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_bb_loopback_config_split_2,".text:dfe");
#endif
static void iqn2_bb_loopback_config_split_2() {
	*mkptr( dfe_cfg_base, 0x01E80700 ) = 0x00001100;
	*mkptr( dfe_cfg_base, 0x01E80704 ) = 0x0000300C;
	*mkptr( dfe_cfg_base, 0x01E80708 ) = 0x0000300C;
	*mkptr( dfe_cfg_base, 0x01E80728 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01E8073C ) = 0x00000016;
	*mkptr( dfe_cfg_base, 0x01EC0000 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0040 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0080 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC00C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0100 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0140 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0180 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC01C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0200 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01EC0240 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E80248 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01E80210 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x01E80204 ) = 0x0000001F;
	*mkptr( dfe_cfg_base, 0x01E80218 ) = 0x00001F00;
	*mkptr( dfe_cfg_base, 0x01EC0800 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0804 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0808 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC080C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0810 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0814 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0818 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC081C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0820 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0824 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0828 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC082C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0830 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0834 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0838 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC083C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0840 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0844 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0848 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC084C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0850 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0854 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0858 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC085C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0860 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0864 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0868 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC086C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0870 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0874 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0878 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC087C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0880 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0884 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0888 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC088C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0890 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0894 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0898 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC089C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC08FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0900 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0904 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0908 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC090C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0910 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0914 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0918 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC091C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0920 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0924 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0928 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC092C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0930 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0934 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0938 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC093C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0940 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0944 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0948 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC094C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0950 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0954 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0958 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC095C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0960 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0964 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0968 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC096C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0970 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0974 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0978 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC097C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0980 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0984 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0988 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC098C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0990 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0994 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC0998 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC099C ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09A8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09AC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09B8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09BC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09C8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09CC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09D8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09DC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09E8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09EC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F0 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F4 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09F8 ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01EC09FC ) = 0x00008000;
	*mkptr( dfe_cfg_base, 0x01E80214 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01E802B0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802C0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802D0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802E0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01E802F0 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00200 ) = 0x00000002;
	*mkptr( dfe_cfg_base, 0x01C00208 ) = 0x00001111;
	*mkptr( dfe_cfg_base, 0x01C0020C ) = 0x000002FD;
	*mkptr( dfe_cfg_base, 0x01C00214 ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00218 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x01C0021C ) = 0x00002000;
	*mkptr( dfe_cfg_base, 0x01C00220 ) = 0x00000010;
	*mkptr( dfe_cfg_base, 0x01C00224 ) = 0x00002200;
	*mkptr( dfe_cfg_base, 0x01C00228 ) = 0x00001020;
	*mkptr( dfe_cfg_base, 0x01C0022C ) = 0x00002200;
	*mkptr( dfe_cfg_base, 0x01C00234 ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00238 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0023C ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00240 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00244 ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00248 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0024C ) = 0x00003210;
	*mkptr( dfe_cfg_base, 0x01C00250 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00254 ) = 0x10100000;
	*mkptr( dfe_cfg_base, 0x01C00258 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0025C ) = 0x32100000;
	*mkptr( dfe_cfg_base, 0x01C00260 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00264 ) = 0x10100000;
	*mkptr( dfe_cfg_base, 0x01C00268 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C0026C ) = 0x32100000;
	*mkptr( dfe_cfg_base, 0x01C00270 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00274 ) = 0x76541010;
	*mkptr( dfe_cfg_base, 0x01C00278 ) = 0x0000000F;
	*mkptr( dfe_cfg_base, 0x01C00280 ) = 0x00000001;
	*mkptr( dfe_cfg_base, 0x01C00328 ) = 0x0000FFFF;
	*mkptr( dfe_cfg_base, 0x01C003E4 ) = 0x01010101;
}
#if 0
void iqn2_bb_loopback_config()
{
*mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
*mkptr( dfe_cfg_base, 0x00040204 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040208 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004020c ) = 0x00004080;
*mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001020;
*mkptr( dfe_cfg_base, 0x00040214 ) = 0x00000008;
*mkptr( dfe_cfg_base, 0x00040218 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004021c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040220 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040224 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040228 ) = 0x00001000;
*mkptr( dfe_cfg_base, 0x0004022c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040230 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040240 ) = 0x0000000c;
*mkptr( dfe_cfg_base, 0x00040244 ) = 0x00002222;
*mkptr( dfe_cfg_base, 0x00040248 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040250 ) = 0x00004444;
*mkptr( dfe_cfg_base, 0x00040254 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004025c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040264 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040268 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004026c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040270 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004027c ) = 0x00000808;
*mkptr( dfe_cfg_base, 0x00040280 ) = 0x00000808;
*mkptr( dfe_cfg_base, 0x00040284 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040288 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040290 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040294 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040298 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004029c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402c0 ) = 0x00000002;
*mkptr( dfe_cfg_base, 0x000402c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402d0 ) = 0x00000002;
*mkptr( dfe_cfg_base, 0x000402d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402e4 ) = 0x00000005;
*mkptr( dfe_cfg_base, 0x000402e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402f4 ) = 0x00000004;
*mkptr( dfe_cfg_base, 0x000402f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000402fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040300 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040304 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040308 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004030c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040310 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040314 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040318 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004031c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040320 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040324 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040328 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004032c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040330 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040334 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040338 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004033c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040344 ) = 0x00000005;
*mkptr( dfe_cfg_base, 0x00040348 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004034c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000811;
*mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000004;
*mkptr( dfe_cfg_base, 0x00040368 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004036c ) = 0x00000002;
*mkptr( dfe_cfg_base, 0x00040370 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040374 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00040378 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004037c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040380 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040384 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040388 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004038c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040390 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040394 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040398 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004039c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000403fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040400 ) = 0x000001c4;
*mkptr( dfe_cfg_base, 0x00040404 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040408 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x0004040c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040410 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x00040414 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040420 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040440 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040460 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040480 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000404a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000404c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000404e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040500 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040520 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040540 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040560 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040580 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000405a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000405c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000405e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000405fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040800 ) = 0x00000054;
*mkptr( dfe_cfg_base, 0x00040804 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x0004080c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x00040814 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000d67;
*mkptr( dfe_cfg_base, 0x00040820 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040840 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040860 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040880 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000408a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000408c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000408e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040900 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040920 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040940 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040960 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040980 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000409a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000409c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000409e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000409fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c04 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c0c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c24 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c2c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c44 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c4c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c64 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c6c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c84 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c8c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040c9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ca0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ca4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ca8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cb0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cb4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cc0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cc4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cc8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ccc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cd0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cd4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cd8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cdc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ce0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ce4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ce8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cf0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040cf4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d04 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d0c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d24 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d2c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d44 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d4c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d64 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d6c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d84 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d8c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040d9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040da0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040da4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040da8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040db0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040db4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dc0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dc4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dc8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dcc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dd0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dd4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dd8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040ddc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040de0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040de4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040de8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040dec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040df0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040df4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e04 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e0c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e24 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e2c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e44 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e4c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e64 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e6c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040e74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041000 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041004 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041008 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004100c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041010 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041014 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041018 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004101c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041020 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041024 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041028 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004102c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041030 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041034 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041038 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004103c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041040 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041044 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041048 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004104c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041050 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041054 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041058 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004105c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041060 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041064 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041068 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004106c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041070 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041074 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041078 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004107c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041100 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041104 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041108 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004110c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041110 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041114 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041118 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041200 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041204 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041208 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004120c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041210 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041220 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041224 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041228 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004122c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041230 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041240 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041244 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041248 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004124c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041250 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041254 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041258 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004125c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041260 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041264 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041280 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041284 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041288 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004128c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041290 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041294 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041298 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004129c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000412a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000412a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041400 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041404 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041408 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004140c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041410 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041414 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041418 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004141c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041420 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041424 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041428 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004142c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041430 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041434 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041480 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041484 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041488 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004148c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041490 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041494 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041498 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004149c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000414b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041500 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041504 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041508 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004150c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041510 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041514 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041518 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004151c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041520 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041524 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041528 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004152c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041530 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041534 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041580 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041584 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041588 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004158c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041590 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041594 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041598 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004159c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000415b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041600 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041604 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041608 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004160c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041610 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041614 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041618 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004161c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041620 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041624 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041628 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004162c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041630 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041634 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041680 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041684 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041688 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004168c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041690 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041694 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041698 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004169c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000416b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041700 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041704 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041708 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004170c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041710 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041714 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041718 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004171c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041720 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041724 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041728 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004172c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041730 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041734 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041780 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041784 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041788 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004178c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041790 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041794 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041798 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004179c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000417b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040600 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040a00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041800 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041804 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041808 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041840 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041844 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041848 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041880 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041884 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041888 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000418c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000418c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000418c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041900 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041904 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041908 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0004190c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041910 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00041914 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060000 ) = 0x00009000;
*mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x00060008 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060010 ) = 0x00009001;
*mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x00060018 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060020 ) = 0x00009010;
*mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x00060028 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060030 ) = 0x00009011;
*mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x00060038 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060040 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060044 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060048 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060050 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060054 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060058 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060060 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060064 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060068 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060070 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060074 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060078 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060080 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060084 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060088 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060090 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060094 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060098 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000600f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060100 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060104 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060108 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060110 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060114 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060118 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060120 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060124 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060128 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060130 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060134 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060138 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060140 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060144 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060148 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060150 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060154 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060158 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060160 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060164 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060168 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060170 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060174 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060178 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060180 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060184 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060188 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060190 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060194 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060198 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000601f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060200 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060204 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060208 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060210 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060214 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060218 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060220 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060224 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060228 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060230 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060234 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060238 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060240 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060244 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060248 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060250 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060254 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060258 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060260 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060264 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060268 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060270 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060274 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060278 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060280 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060284 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060288 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060290 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060294 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060298 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000602f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060300 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060304 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060308 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060310 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060314 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060318 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060320 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060324 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060328 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060330 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060334 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060338 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060340 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060344 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060348 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060350 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060354 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060358 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060360 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060364 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060368 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060370 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060374 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060378 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060380 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060384 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060388 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060390 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060394 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060398 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000603f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060400 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060404 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060408 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060410 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060414 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060418 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060420 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060424 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060428 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060430 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060434 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060438 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060440 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060444 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060448 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060450 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060454 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060458 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060460 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060464 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060468 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060470 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060474 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060478 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008420;
*mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060808 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060810 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060814 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060818 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006081c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008421;
*mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060828 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060830 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060834 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060838 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006083c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008430;
*mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060848 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060850 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060854 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060858 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006085c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008431;
*mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060868 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060870 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060874 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060878 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006087c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060880 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060888 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060890 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060894 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060898 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006089c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608a4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000608a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608c4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000608c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608e4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000608e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000608fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060900 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060908 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060910 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060914 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060918 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006091c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060920 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060928 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060930 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060934 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060938 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006093c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060940 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060948 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060950 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060954 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060958 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006095c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060960 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060968 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060970 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060974 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060978 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006097c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060980 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060988 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060990 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060994 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060998 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006099c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609a4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000609a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609c4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000609c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609e4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000609e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000609fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a04 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060a08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a24 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060a28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a38 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a44 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060a48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a64 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060a68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a78 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a7c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a84 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060a88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060a9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060aa0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060aa4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060aa8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ab0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ab4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ab8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060abc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ac0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ac4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060ac8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ad0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ad4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ad8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060adc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ae0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ae4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060ae8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060af0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060af4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060af8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060afc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b04 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060b08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b24 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060b28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b38 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b44 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060b48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b64 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060b68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b78 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b7c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b84 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060b88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060b9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ba0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ba4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060ba8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bb0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bb4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bb8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bbc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bc0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bc4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060bc8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bd0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bd4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bd8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bdc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060be0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060be4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060be8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bf0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bf4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bf8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060bfc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c04 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060c08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c24 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060c28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c38 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c44 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060c48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c64 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060c68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c78 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c7c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c84 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060c88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060c9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ca0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ca4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060ca8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cb0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cb4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cb8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cbc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cc0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cc4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060cc8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cd0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cd4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cd8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cdc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ce0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ce4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060ce8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cf0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cf4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cf8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060cfc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d04 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060d08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d24 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060d28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d38 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d44 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060d48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d64 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060d68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d78 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d7c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d84 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060d88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060d9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060da0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060da4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060da8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060db0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060db4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060db8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dbc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dc0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dc4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060dc8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dd0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dd4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dd8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060ddc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060de0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060de4 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x00060de8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060df0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060df4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060df8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00060dfc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061000 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
*mkptr( dfe_cfg_base, 0x0006100c ) = 0x00000003;
*mkptr( dfe_cfg_base, 0x00061010 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061014 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061018 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006101c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061020 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061024 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006102c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006103c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006104c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006105c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006106c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006107c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061080 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061084 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061088 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006108c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061090 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061094 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061098 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006109c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000610fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006110c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006111c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006112c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006113c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006114c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006115c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006116c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006117c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006118c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006119c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000611fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006120c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006121c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006122c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006123c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006124c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006125c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006126c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006127c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006128c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006129c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000612fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006130c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006131c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006132c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006133c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006134c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006135c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006136c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006137c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006138c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006139c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000613fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061404 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006140c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061414 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006141c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061424 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006142c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061434 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006143c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061444 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006144c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061454 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006145c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061464 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006146c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061474 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006147c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061484 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006148c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061494 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006149c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614a0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614a8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614b0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614b8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614c0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614c8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614d0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614d8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614e0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614e8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614f0 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000614f8 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000614fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061504 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006150c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061514 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006151c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061524 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006152c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061534 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006153c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061544 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006154c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061554 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006155c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061564 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006156c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00061574 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x0006157c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061800 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061804 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061808 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006180c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061810 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061814 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061818 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006181c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061824 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006182c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061834 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006183c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061844 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006184c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061854 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006185c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061864 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006186c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061874 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006187c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061884 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006188c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061894 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006189c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618a0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618a8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618b0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618b8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618c0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618c8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618d0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618d8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618e0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618e8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618f0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000618f8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000618fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061904 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006190c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061914 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006191c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061924 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006192c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061934 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006193c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061944 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006194c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061954 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006195c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061964 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006196c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061974 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006197c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061984 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006198c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061994 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x0006199c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619a0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619a8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619b0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619b8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619c0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619c8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619d0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619d8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619e0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619e8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619f0 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000619f8 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x000619fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a00 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a04 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a08 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a0c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a10 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a18 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a20 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a24 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a28 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a2c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a30 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061a38 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00061a3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00061c00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061c9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ca0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ca4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ca8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cb0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cb4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cb8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cbc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cc0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cc4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cc8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ccc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cd0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cd4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cd8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cdc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ce0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ce4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ce8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cf0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cf4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cf8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061cfc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061d9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061da0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061da4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061da8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061db0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061db4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061db8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dbc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dc0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dc4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dc8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dcc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dd0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dd4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dd8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ddc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061de0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061de4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061de8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061df0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061df4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061df8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061dfc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061e9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ea0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ea4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ea8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061eac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061eb0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061eb4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061eb8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ebc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ec0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ec4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ec8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ecc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ed0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ed4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ed8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061edc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ee0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ee4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ee8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061eec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ef0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ef4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ef8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061efc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061f9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fa0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fa4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fa8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fb0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fb4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fb8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fbc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fc0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fc4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fc8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fcc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fd0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fd4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fd8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fdc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fe0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fe4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fe8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061fec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ff0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ff4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ff8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00061ffc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006200c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006201c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006202c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006203c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006204c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006205c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006206c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006207c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006208c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006209c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000620bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000401;
*mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000402;
*mkptr( dfe_cfg_base, 0x0006240c ) = 0x00000403;
*mkptr( dfe_cfg_base, 0x00062410 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062414 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062418 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006241c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062420 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062424 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062428 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006242c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062430 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062434 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062438 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006243c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006244c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006245c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006246c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006247c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062480 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062484 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062488 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006248c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062490 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062494 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062498 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006249c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000624fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006250c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006251c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006252c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006253c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006254c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006255c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006256c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006257c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006258c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006259c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000625fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006260c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006261c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006262c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006263c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006264c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006265c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006266c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006267c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006268c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006269c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000626fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006270c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006271c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006272c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006273c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006274c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006275c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006276c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006277c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006278c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006279c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000627fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006280c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006281c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006282c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006283c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006284c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006285c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006286c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006287c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006288c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006289c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000628fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006290c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006291c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006292c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006293c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006294c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006295c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006296c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006297c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006298c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006299c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000629fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062a9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062aa0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062aa4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062aa8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062aac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ab0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ab4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ab8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062abc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ac0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ac4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ac8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062acc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ad0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ad4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ad8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062adc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ae0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ae4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ae8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062aec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062af0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062af4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062af8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062afc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b00 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b04 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b08 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b0c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b10 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b14 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b18 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b1c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b20 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b24 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b28 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b2c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b30 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b34 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b38 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b3c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b40 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b44 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b48 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b4c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b50 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b54 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b58 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b5c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b60 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b64 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b68 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b6c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b70 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b74 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b78 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b7c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b80 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b84 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b88 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b8c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b90 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b94 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b98 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062b9c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ba0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ba4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062ba8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bb0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bb4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bb8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bbc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bc0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bc4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bc8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bcc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bd0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bd4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bd8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bdc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062be0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062be4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062be8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bf0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bf4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bf8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062bfc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00062c00 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c04 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c08 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c0c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c10 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c14 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c18 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c1c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c20 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c24 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c28 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c2c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c30 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c34 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c38 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c3c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c40 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c44 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c48 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c4c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c50 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c54 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c58 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c5c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c60 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c64 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c68 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c6c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c70 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c74 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c78 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c7c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c80 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c84 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c88 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c8c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c90 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c94 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c98 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062c9c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062ca0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062ca4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062ca8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062cac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062cb0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062cb4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062cb8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00062cbc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006300c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006301c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006302c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006303c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006304c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006305c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006306c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006307c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006308c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006309c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000630fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006310c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006311c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006312c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006313c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006314c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006315c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006316c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006317c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006318c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006319c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000631fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006320c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006321c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006322c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006323c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006324c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006325c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006326c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006327c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006328c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006329c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000632fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006330c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006331c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006332c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006333c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006334c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006335c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006336c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006337c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006338c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x0006339c ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633a0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633a4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633a8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633ac ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633b0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633b4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633b8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633bc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633c0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633c4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633c8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633cc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633d0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633d4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633d8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633dc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633e0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633e4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633e8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633ec ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633f0 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633f4 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633f8 ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x000633fc ) = 0x000000ff;
*mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006340c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006341c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006342c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006343c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006344c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006345c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006346c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006347c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006348c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006349c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000634fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006350c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006351c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006352c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006353c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006354c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006355c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006356c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006357c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006358c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006359c ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635a4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635a8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635ac ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635b0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635b4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635b8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635bc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635c4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635c8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635cc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635d0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635d4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635d8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635dc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635e4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635e8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635ec ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635f0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635f4 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635f8 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x000635fc ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x00064000 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
*mkptr( dfe_cfg_base, 0x0006400c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
*mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
*mkptr( dfe_cfg_base, 0x0006401c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
*mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000a00;
*mkptr( dfe_cfg_base, 0x0006402c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000c00;
*mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000e00;
*mkptr( dfe_cfg_base, 0x0006403c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
*mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
*mkptr( dfe_cfg_base, 0x0006404c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
*mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
*mkptr( dfe_cfg_base, 0x0006405c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
*mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001a00;
*mkptr( dfe_cfg_base, 0x0006406c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001c00;
*mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001e00;
*mkptr( dfe_cfg_base, 0x0006407c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
*mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
*mkptr( dfe_cfg_base, 0x0006408c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
*mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
*mkptr( dfe_cfg_base, 0x0006409c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640a0 ) = 0x00002800;
*mkptr( dfe_cfg_base, 0x000640a4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640a8 ) = 0x00002a00;
*mkptr( dfe_cfg_base, 0x000640ac ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640b0 ) = 0x00002c00;
*mkptr( dfe_cfg_base, 0x000640b4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640b8 ) = 0x00002e00;
*mkptr( dfe_cfg_base, 0x000640bc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640c0 ) = 0x00003000;
*mkptr( dfe_cfg_base, 0x000640c4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640c8 ) = 0x00003200;
*mkptr( dfe_cfg_base, 0x000640cc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640d0 ) = 0x00003400;
*mkptr( dfe_cfg_base, 0x000640d4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640d8 ) = 0x00003600;
*mkptr( dfe_cfg_base, 0x000640dc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640e0 ) = 0x00003800;
*mkptr( dfe_cfg_base, 0x000640e4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640e8 ) = 0x00003a00;
*mkptr( dfe_cfg_base, 0x000640ec ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640f0 ) = 0x00003c00;
*mkptr( dfe_cfg_base, 0x000640f4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000640f8 ) = 0x00003e00;
*mkptr( dfe_cfg_base, 0x000640fc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
*mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
*mkptr( dfe_cfg_base, 0x0006410c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
*mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
*mkptr( dfe_cfg_base, 0x0006411c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
*mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004a00;
*mkptr( dfe_cfg_base, 0x0006412c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004c00;
*mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004e00;
*mkptr( dfe_cfg_base, 0x0006413c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
*mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
*mkptr( dfe_cfg_base, 0x0006414c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
*mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
*mkptr( dfe_cfg_base, 0x0006415c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
*mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005a00;
*mkptr( dfe_cfg_base, 0x0006416c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005c00;
*mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005e00;
*mkptr( dfe_cfg_base, 0x0006417c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
*mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
*mkptr( dfe_cfg_base, 0x0006418c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
*mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
*mkptr( dfe_cfg_base, 0x0006419c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641a0 ) = 0x00006800;
*mkptr( dfe_cfg_base, 0x000641a4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641a8 ) = 0x00006a00;
*mkptr( dfe_cfg_base, 0x000641ac ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641b0 ) = 0x00006c00;
*mkptr( dfe_cfg_base, 0x000641b4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641b8 ) = 0x00006e00;
*mkptr( dfe_cfg_base, 0x000641bc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641c0 ) = 0x00007000;
*mkptr( dfe_cfg_base, 0x000641c4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641c8 ) = 0x00007200;
*mkptr( dfe_cfg_base, 0x000641cc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641d0 ) = 0x00007400;
*mkptr( dfe_cfg_base, 0x000641d4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641d8 ) = 0x00007600;
*mkptr( dfe_cfg_base, 0x000641dc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641e0 ) = 0x00007800;
*mkptr( dfe_cfg_base, 0x000641e4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641e8 ) = 0x00007a00;
*mkptr( dfe_cfg_base, 0x000641ec ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641f0 ) = 0x00007c00;
*mkptr( dfe_cfg_base, 0x000641f4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000641f8 ) = 0x00007e00;
*mkptr( dfe_cfg_base, 0x000641fc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
*mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
*mkptr( dfe_cfg_base, 0x0006420c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
*mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
*mkptr( dfe_cfg_base, 0x0006421c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
*mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008a00;
*mkptr( dfe_cfg_base, 0x0006422c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008c00;
*mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008e00;
*mkptr( dfe_cfg_base, 0x0006423c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
*mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
*mkptr( dfe_cfg_base, 0x0006424c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
*mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
*mkptr( dfe_cfg_base, 0x0006425c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
*mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009a00;
*mkptr( dfe_cfg_base, 0x0006426c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009c00;
*mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009e00;
*mkptr( dfe_cfg_base, 0x0006427c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000a000;
*mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000a200;
*mkptr( dfe_cfg_base, 0x0006428c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000a400;
*mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000a600;
*mkptr( dfe_cfg_base, 0x0006429c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642a0 ) = 0x0000a800;
*mkptr( dfe_cfg_base, 0x000642a4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642a8 ) = 0x0000aa00;
*mkptr( dfe_cfg_base, 0x000642ac ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642b0 ) = 0x0000ac00;
*mkptr( dfe_cfg_base, 0x000642b4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642b8 ) = 0x0000ae00;
*mkptr( dfe_cfg_base, 0x000642bc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642c0 ) = 0x0000b000;
*mkptr( dfe_cfg_base, 0x000642c4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642c8 ) = 0x0000b200;
*mkptr( dfe_cfg_base, 0x000642cc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642d0 ) = 0x0000b400;
*mkptr( dfe_cfg_base, 0x000642d4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642d8 ) = 0x0000b600;
*mkptr( dfe_cfg_base, 0x000642dc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642e0 ) = 0x0000b800;
*mkptr( dfe_cfg_base, 0x000642e4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642e8 ) = 0x0000ba00;
*mkptr( dfe_cfg_base, 0x000642ec ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642f0 ) = 0x0000bc00;
*mkptr( dfe_cfg_base, 0x000642f4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000642f8 ) = 0x0000be00;
*mkptr( dfe_cfg_base, 0x000642fc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000c000;
*mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000c200;
*mkptr( dfe_cfg_base, 0x0006430c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000c400;
*mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000c600;
*mkptr( dfe_cfg_base, 0x0006431c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000c800;
*mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000ca00;
*mkptr( dfe_cfg_base, 0x0006432c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000cc00;
*mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000ce00;
*mkptr( dfe_cfg_base, 0x0006433c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000d000;
*mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000d200;
*mkptr( dfe_cfg_base, 0x0006434c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000d400;
*mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000d600;
*mkptr( dfe_cfg_base, 0x0006435c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000d800;
*mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000da00;
*mkptr( dfe_cfg_base, 0x0006436c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000dc00;
*mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000de00;
*mkptr( dfe_cfg_base, 0x0006437c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000e000;
*mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000e200;
*mkptr( dfe_cfg_base, 0x0006438c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000e400;
*mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000e600;
*mkptr( dfe_cfg_base, 0x0006439c ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643a0 ) = 0x0000e800;
*mkptr( dfe_cfg_base, 0x000643a4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643a8 ) = 0x0000ea00;
*mkptr( dfe_cfg_base, 0x000643ac ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643b0 ) = 0x0000ec00;
*mkptr( dfe_cfg_base, 0x000643b4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643b8 ) = 0x0000ee00;
*mkptr( dfe_cfg_base, 0x000643bc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643c0 ) = 0x0000f000;
*mkptr( dfe_cfg_base, 0x000643c4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643c8 ) = 0x0000f200;
*mkptr( dfe_cfg_base, 0x000643cc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643d0 ) = 0x0000f400;
*mkptr( dfe_cfg_base, 0x000643d4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643d8 ) = 0x0000f600;
*mkptr( dfe_cfg_base, 0x000643dc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643e0 ) = 0x0000f800;
*mkptr( dfe_cfg_base, 0x000643e4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643e8 ) = 0x0000fa00;
*mkptr( dfe_cfg_base, 0x000643ec ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643f0 ) = 0x0000fc00;
*mkptr( dfe_cfg_base, 0x000643f4 ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x000643f8 ) = 0x0000fe00;
*mkptr( dfe_cfg_base, 0x000643fc ) = 0x00000001;
*mkptr( dfe_cfg_base, 0x00065000 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065004 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065008 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006500c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065010 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065014 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065018 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006501c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065020 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065024 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065028 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006502c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065030 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065034 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065038 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006503c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065040 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065044 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065048 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006504c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065050 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065054 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065058 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006505c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065060 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065064 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065068 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006506c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065070 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065074 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065078 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006507c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065080 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065084 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065088 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006508c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065090 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065094 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065098 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006509c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000650fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065100 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065104 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065108 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006510c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065110 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065114 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065118 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006511c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065120 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065124 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065128 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006512c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065130 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065134 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065138 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006513c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065140 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065144 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065148 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006514c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065150 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065154 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065158 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006515c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065160 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065164 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065168 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006516c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065170 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065174 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065178 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006517c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065400 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065404 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065408 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006540c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065410 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065414 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065418 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006541c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065420 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065424 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065428 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006542c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065430 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065434 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065438 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006543c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065440 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065444 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065448 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006544c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065450 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065454 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065458 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006545c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065460 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065464 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065468 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006546c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065470 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065474 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065478 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006547c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065480 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065484 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065488 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006548c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065490 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065494 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065498 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006549c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000654fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065500 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065504 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065508 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006550c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065510 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065514 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065518 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006551c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065520 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065524 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065528 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006552c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065530 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065534 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065538 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006553c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065540 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065544 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065548 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006554c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065550 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065554 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065558 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006555c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065560 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065564 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065568 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006556c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065570 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065574 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065578 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006557c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065580 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065584 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065588 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006558c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065590 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065594 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065598 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006559c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000655fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065600 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065604 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065608 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006560c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065610 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065614 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065618 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006561c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065620 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065624 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065628 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006562c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065630 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065634 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065638 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006563c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065640 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065644 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065648 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006564c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065650 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065654 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065658 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006565c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065660 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065664 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065668 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006566c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065670 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065674 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065678 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006567c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065680 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065684 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065688 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006568c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065690 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065694 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065698 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006569c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000656fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065800 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065804 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065808 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006580c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065810 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065814 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065818 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006581c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065820 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065824 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065828 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006582c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065830 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065834 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065838 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006583c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065840 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065844 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065848 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006584c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065850 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065854 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065858 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006585c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065860 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065864 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065868 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006586c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065870 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065874 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065878 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006587c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065880 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065884 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065888 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006588c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065890 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065894 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065898 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006589c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000658fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065900 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065904 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065908 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006590c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065910 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065914 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065918 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006591c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065920 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065924 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065928 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006592c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065930 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065934 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065938 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006593c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065940 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065944 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065948 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006594c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065950 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065954 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065958 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006595c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065960 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065964 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065968 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006596c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065970 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065974 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065978 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006597c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065980 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065984 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065988 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006598c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065990 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065994 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065998 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006599c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659a0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659a4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659a8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659ac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659b0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659b8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659bc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659c0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659c4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659c8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659cc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659d0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659d8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659dc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659e0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659e4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659e8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659ec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659f0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659f8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x000659fc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a00 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a04 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a08 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a0c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a10 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a14 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a18 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a1c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a20 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a24 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a28 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a2c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a30 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a34 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a38 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a3c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a40 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a44 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a48 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a4c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a50 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a54 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a58 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a5c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a60 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a64 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a68 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a6c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a70 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a74 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a78 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a7c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a80 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a84 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a88 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a8c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a90 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a94 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a98 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065a9c ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065aa0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065aa4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065aa8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065aac ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ab0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ab4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ab8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065abc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ac0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ac4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ac8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065acc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ad0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ad4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ad8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065adc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ae0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ae4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065ae8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065aec ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065af0 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065af4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065af8 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00065afc ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c000 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c004 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c008 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c00c ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c010 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c014 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c020 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c024 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c028 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c02c ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c030 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c034 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c040 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c044 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c048 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c04c ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c050 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c054 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c060 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c064 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c068 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c06c ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c070 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c074 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c080 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c084 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c088 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c08c ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c090 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c094 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c0a0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c0a4 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c0a8 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c0ac ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c0b0 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c0b4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c0c0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c0c4 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c0c8 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c0cc ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c0d0 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c0d4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x0006c0e0 ) = 0x0000007f;
*mkptr( dfe_cfg_base, 0x0006c0e4 ) = 0x00000fff;
*mkptr( dfe_cfg_base, 0x0006c0e8 ) = 0x00000055;
*mkptr( dfe_cfg_base, 0x0006c0ec ) = 0x0000ffff;
*mkptr( dfe_cfg_base, 0x0006c0f0 ) = 0x0000807f;
*mkptr( dfe_cfg_base, 0x0006c0f4 ) = 0x00000000;
*mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000308;
}
#endif
#endif
#if DFE_NORMAL_OPERATION == 1
static void iqn2_normal_operation_config_split_1();
static void iqn2_normal_operation_config_split_2();
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_normal_operation_config,".text:dfe");
#endif
void iqn2_normal_operation_config()
{
    // based on P1_3_LTE20_a.reg
    *mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x0004020C ) = 0x00004080;
    *mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x00040214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00040228 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00040240 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00040244 ) = 0x00002222;
    *mkptr( dfe_cfg_base, 0x00040250 ) = 0x00004444;
    *mkptr( dfe_cfg_base, 0x0004027C ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00040280 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x000402C0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402D0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402E4 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x000402F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00040344 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0004036C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00040374 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00040400 ) = 0x000001C4;
    *mkptr( dfe_cfg_base, 0x00040408 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040410 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040800 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000D67;
    *mkptr( dfe_cfg_base, 0x00060000 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060010 ) = 0x0000D001;
    *mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060020 ) = 0x0000D010;
    *mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060030 ) = 0x0000D011;
    *mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008420;
    *mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008421;
    *mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008430;
    *mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008431;
    *mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x0006100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00061010 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061014 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061018 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006101C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061020 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061024 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006102C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006103C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006104C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006105C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006106C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006107C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061084 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061088 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006108C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061090 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061094 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061098 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006109C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006110C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006111C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006112C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006113C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006114C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006115C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006116C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006117C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006118C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006119C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006120C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006121C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006122C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006123C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006124C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006125C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006126C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006127C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006128C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006129C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006130C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006131C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006132C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006133C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006134C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006135C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006136C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006137C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006138C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006139C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061800 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061808 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061810 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061818 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A00 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A08 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A10 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A18 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A20 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A28 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A30 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A38 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061C00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ECC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006200C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006201C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006202C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006203C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006204C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006205C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006206C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006207C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006208C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006209C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x0006240C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00062410 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062414 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062418 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006241C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062420 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062424 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062428 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006242C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062430 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062434 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062438 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006243C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006244C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006245C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006246C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006247C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062480 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062484 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062488 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006248C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062490 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062494 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062498 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006249C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006250C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006251C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006252C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006253C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006254C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006255C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006256C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006257C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006258C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006259C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006260C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006261C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006262C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006263C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006264C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006265C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006266C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006267C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006268C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006269C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006270C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006271C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006272C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006273C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006274C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006275C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006276C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006277C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006278C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006279C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006280C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006281C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006282C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006283C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006284C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006285C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006286C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006287C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006288C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006289C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006290C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006291C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006292C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006293C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006294C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006295C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006296C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006297C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006298C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006299C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ABC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ACC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ADC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062C00 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C04 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C08 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C0C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C10 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C14 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C18 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C1C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C20 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C24 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C28 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C2C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C30 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C34 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C38 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C3C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C40 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C44 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C48 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C4C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C50 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C54 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C58 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C5C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C60 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C64 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C68 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C6C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C70 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C74 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C78 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C7C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C80 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C84 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C88 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C8C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C90 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C94 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C98 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C9C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CAC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CBC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006300C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006301C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006302C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006303C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006304C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006305C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006306C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006307C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006308C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006309C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006310C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006311C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006312C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006313C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006314C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006315C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006316C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006317C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006318C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006319C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006320C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006321C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006322C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006323C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006324C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006325C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006326C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006327C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006328C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006329C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006330C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006331C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006332C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006333C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006334C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006335C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006336C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006337C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006338C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006339C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006340C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006341C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006342C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006343C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006344C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006345C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006346C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006347C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006348C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006349C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006350C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006351C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006352C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006353C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006354C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006355C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006356C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006357C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006358C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006359C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x0006400C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x0006401C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x0006402C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x0006403C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
    *mkptr( dfe_cfg_base, 0x0006404C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
    *mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
    *mkptr( dfe_cfg_base, 0x0006405C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
    *mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001A00;
    *mkptr( dfe_cfg_base, 0x0006406C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001C00;
    *mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001E00;
    *mkptr( dfe_cfg_base, 0x0006407C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x0006408C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
    *mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
    *mkptr( dfe_cfg_base, 0x0006409C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A0 ) = 0x00002800;
    *mkptr( dfe_cfg_base, 0x000640A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A8 ) = 0x00002A00;
    *mkptr( dfe_cfg_base, 0x000640AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B0 ) = 0x00002C00;
    *mkptr( dfe_cfg_base, 0x000640B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B8 ) = 0x00002E00;
    *mkptr( dfe_cfg_base, 0x000640BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C0 ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x000640C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C8 ) = 0x00003200;
    *mkptr( dfe_cfg_base, 0x000640CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D0 ) = 0x00003400;
    *mkptr( dfe_cfg_base, 0x000640D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D8 ) = 0x00003600;
    *mkptr( dfe_cfg_base, 0x000640DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E0 ) = 0x00003800;
    *mkptr( dfe_cfg_base, 0x000640E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E8 ) = 0x00003A00;
    *mkptr( dfe_cfg_base, 0x000640EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F0 ) = 0x00003C00;
    *mkptr( dfe_cfg_base, 0x000640F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F8 ) = 0x00003E00;
    *mkptr( dfe_cfg_base, 0x000640FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
    *mkptr( dfe_cfg_base, 0x0006410C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
    *mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
    *mkptr( dfe_cfg_base, 0x0006411C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
    *mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004A00;
    *mkptr( dfe_cfg_base, 0x0006412C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004C00;
    *mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004E00;
    *mkptr( dfe_cfg_base, 0x0006413C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
    *mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
    *mkptr( dfe_cfg_base, 0x0006414C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
    *mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
    *mkptr( dfe_cfg_base, 0x0006415C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
    *mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005A00;
    *mkptr( dfe_cfg_base, 0x0006416C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005C00;
    *mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005E00;
    *mkptr( dfe_cfg_base, 0x0006417C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
    *mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
    *mkptr( dfe_cfg_base, 0x0006418C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
    *mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
    *mkptr( dfe_cfg_base, 0x0006419C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A0 ) = 0x00006800;
    *mkptr( dfe_cfg_base, 0x000641A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A8 ) = 0x00006A00;
    *mkptr( dfe_cfg_base, 0x000641AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B0 ) = 0x00006C00;
    *mkptr( dfe_cfg_base, 0x000641B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B8 ) = 0x00006E00;
    *mkptr( dfe_cfg_base, 0x000641BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C0 ) = 0x00007000;
    *mkptr( dfe_cfg_base, 0x000641C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C8 ) = 0x00007200;
    *mkptr( dfe_cfg_base, 0x000641CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D0 ) = 0x00007400;
    *mkptr( dfe_cfg_base, 0x000641D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D8 ) = 0x00007600;
    *mkptr( dfe_cfg_base, 0x000641DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E0 ) = 0x00007800;
    *mkptr( dfe_cfg_base, 0x000641E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E8 ) = 0x00007A00;
    *mkptr( dfe_cfg_base, 0x000641EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F0 ) = 0x00007C00;
    *mkptr( dfe_cfg_base, 0x000641F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F8 ) = 0x00007E00;
    *mkptr( dfe_cfg_base, 0x000641FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
    *mkptr( dfe_cfg_base, 0x0006420C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
    *mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
    *mkptr( dfe_cfg_base, 0x0006421C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
    *mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008A00;
    *mkptr( dfe_cfg_base, 0x0006422C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008C00;
    *mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008E00;
    *mkptr( dfe_cfg_base, 0x0006423C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
    *mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
    *mkptr( dfe_cfg_base, 0x0006424C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
    *mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
    *mkptr( dfe_cfg_base, 0x0006425C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
    *mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009A00;
    *mkptr( dfe_cfg_base, 0x0006426C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009C00;
    *mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009E00;
    *mkptr( dfe_cfg_base, 0x0006427C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000A000;
    *mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000A200;
    *mkptr( dfe_cfg_base, 0x0006428C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000A400;
    *mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000A600;
    *mkptr( dfe_cfg_base, 0x0006429C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A0 ) = 0x0000A800;
    *mkptr( dfe_cfg_base, 0x000642A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A8 ) = 0x0000AA00;
    *mkptr( dfe_cfg_base, 0x000642AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B0 ) = 0x0000AC00;
    *mkptr( dfe_cfg_base, 0x000642B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B8 ) = 0x0000AE00;
    *mkptr( dfe_cfg_base, 0x000642BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C0 ) = 0x0000B000;
    *mkptr( dfe_cfg_base, 0x000642C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C8 ) = 0x0000B200;
    *mkptr( dfe_cfg_base, 0x000642CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D0 ) = 0x0000B400;
    *mkptr( dfe_cfg_base, 0x000642D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D8 ) = 0x0000B600;
    *mkptr( dfe_cfg_base, 0x000642DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E0 ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x000642E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E8 ) = 0x0000BA00;
    *mkptr( dfe_cfg_base, 0x000642EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F0 ) = 0x0000BC00;
    *mkptr( dfe_cfg_base, 0x000642F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F8 ) = 0x0000BE00;
    *mkptr( dfe_cfg_base, 0x000642FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000C000;
    *mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000C200;
    *mkptr( dfe_cfg_base, 0x0006430C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000C400;
    *mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000C600;
    *mkptr( dfe_cfg_base, 0x0006431C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000CA00;
    *mkptr( dfe_cfg_base, 0x0006432C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000CC00;
    *mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000CE00;
    *mkptr( dfe_cfg_base, 0x0006433C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000D200;
    *mkptr( dfe_cfg_base, 0x0006434C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000D400;
    *mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000D600;
    *mkptr( dfe_cfg_base, 0x0006435C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000D800;
    *mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000DA00;
    *mkptr( dfe_cfg_base, 0x0006436C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000DC00;
    *mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000DE00;
    *mkptr( dfe_cfg_base, 0x0006437C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000E000;
    *mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000E200;
    *mkptr( dfe_cfg_base, 0x0006438C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000E400;
    *mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000E600;
    *mkptr( dfe_cfg_base, 0x0006439C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A0 ) = 0x0000E800;
    *mkptr( dfe_cfg_base, 0x000643A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A8 ) = 0x0000EA00;
    *mkptr( dfe_cfg_base, 0x000643AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B0 ) = 0x0000EC00;
    *mkptr( dfe_cfg_base, 0x000643B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B8 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x000643BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C0 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000643C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C8 ) = 0x0000F200;
    *mkptr( dfe_cfg_base, 0x000643CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D0 ) = 0x0000F400;
    *mkptr( dfe_cfg_base, 0x000643D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D8 ) = 0x0000F600;
    *mkptr( dfe_cfg_base, 0x000643DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E0 ) = 0x0000F800;
    *mkptr( dfe_cfg_base, 0x000643E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E8 ) = 0x0000FA00;
    *mkptr( dfe_cfg_base, 0x000643EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F0 ) = 0x0000FC00;
    *mkptr( dfe_cfg_base, 0x000643F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F8 ) = 0x0000FE00;
    *mkptr( dfe_cfg_base, 0x000643FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0006C000 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C004 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C008 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C00C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C010 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C020 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C024 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C028 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C02C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C030 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C044 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C048 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C04C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C050 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C064 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C068 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C06C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C070 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C084 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C088 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C08C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C090 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0A4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0A8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0AC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0B0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0C4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0C8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0D0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0E4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0E8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0EC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0F0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000308;
    *mkptr( dfe_cfg_base, 0x00101300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00101304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00101308 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x00101318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00101324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x00140004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x0014000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x00140018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x0014001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00140200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x00140208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x0014020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x00140214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x00140400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x00140408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x00140410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x00140414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x00140600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x00140604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x00140610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x00140618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x0014061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x00140804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x0014080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x00140818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x0014081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x00140A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x00140A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x00140A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x00140C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x00140C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00140C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x00140C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00140E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x00140E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x00140E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00140E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x00140E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x00141004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x0014100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x00141018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x0014101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x00141204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x0014120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0010126C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00101344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00101348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0010134C ) = 0x00004300;
    *mkptr( dfe_cfg_base, 0x00101350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x00143800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143804 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014380C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143814 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014381C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00101200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00101204 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0010120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00101214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101218 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0010121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101220 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00101224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0010122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101230 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00101234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0010123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00101240 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00101244 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00101248 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00101250 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00101254 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00101258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0010125C ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00101260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00101264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00101268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001012DC ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x00100400 ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x00181300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00181304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00181308 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x00181318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00181324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x001C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x001C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x001C0018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x001C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x001C0200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x001C0208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x001C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x001C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x001C0400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x001C0408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x001C0410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x001C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x001C0600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x001C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x001C0610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x001C0618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x001C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x001C0804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x001C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x001C0818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x001C081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x001C0A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x001C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x001C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x001C0C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x001C0C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x001C0C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x001C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x001C0E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x001C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x001C0E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x001C0E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x001C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x001C1004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x001C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x001C1018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x001C101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x001C1204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x001C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0018126C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00181344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00181348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0018134C ) = 0x00004300;
    *mkptr( dfe_cfg_base, 0x00181350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x001C3800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3804 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C380C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3814 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C381C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00181200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00181204 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0018120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00181214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181218 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0018121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181220 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00181224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0018122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181230 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00181234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0018123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00181240 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00181244 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00181248 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00181250 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00181254 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00181258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0018125C ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00181260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00181264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00181268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001812DC ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x00180400 ) = 0x00004080;
    *mkptr( dfe_cfg_base, 0x00201300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00201304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00201308 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00201318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00201324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x00240004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x0024000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x00240018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x0024001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00240200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x00240208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x0024020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x00240214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x00240400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x00240408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x00240410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x00240414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x00240600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x00240604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x00240610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x00240618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x0024061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x00240804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x0024080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x00240818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x0024081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x00240A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x00240A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x00240A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x00240C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x00240C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00240C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x00240C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00240E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x00240E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x00240E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00240E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x00240E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x00241004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x0024100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x00241018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x0024101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x00241204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x0024120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0020126C ) = 0x00000216;
    *mkptr( dfe_cfg_base, 0x00201344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00201348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0020134C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00201350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x00243800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243804 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024380C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243814 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024381C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00201200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00201204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00201208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0020120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00201214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201218 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0020121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201220 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00201224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201228 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0020122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201230 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00201234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0020123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00201240 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201244 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201248 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00201250 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201254 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0020125C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00201260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00201264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00201268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002012DC ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x002012E4 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x00281300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00281304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00281308 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00281318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00281324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x002C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x002C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x002C0018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x002C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x002C0200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x002C0208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x002C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x002C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x002C0400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x002C0408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x002C0410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x002C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x002C0600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x002C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x002C0610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x002C0618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x002C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x002C0804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x002C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x002C0818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x002C081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x002C0A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x002C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x002C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x002C0C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x002C0C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x002C0C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x002C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x002C0E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x002C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x002C0E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x002C0E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x002C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x002C1004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x002C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x002C1018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x002C101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x002C1204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x002C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0028126C ) = 0x00000216;
    *mkptr( dfe_cfg_base, 0x00281344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00281348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0028134C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00281350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x002C3800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3804 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C380C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3814 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C381C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00281200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00281204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00281208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0028120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00281214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281218 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0028121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281220 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00281224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281228 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0028122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281230 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00281234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0028123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00281240 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281244 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281248 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00281250 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281254 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0028125C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00281260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00281264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00281268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002812DC ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x002812E4 ) = 0x00000041;
    *mkptr( dfe_cfg_base, 0x00900004 ) = 0x00000606;
    *mkptr( dfe_cfg_base, 0x00900018 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00900030 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00B003CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B00208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00360 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x00B00368 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00B00370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B00200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00230 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B00290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B002B0 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B00300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00310 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00318 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B00328 ) = 0x00000174;
    *mkptr( dfe_cfg_base, 0x00B00338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B00340 ) = 0x00000363;
    *mkptr( dfe_cfg_base, 0x00B00348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B00378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B18004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B18084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B18088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B1808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B18090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B18094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B18098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B1809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B180A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B180A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B180A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B180AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B180B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B180B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B180B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B180BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B180C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B180C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B180C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B180CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B180D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B180D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B180D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B180DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B180E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B180E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B180E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B180EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B180F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B180F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B180F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B180FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B18100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B18104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B18108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B1810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B18110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B18114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B18118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B1811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B18120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B18124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B18128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B1812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B18130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B18134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B18138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B1813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B18140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B18144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B18148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B1814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B18150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B18154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B18158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B1815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B18160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B18164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B18168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B18170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B18174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B18178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B18180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B18184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B18188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B1818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B18190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B18194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B18198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B1819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B181A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B181A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B181A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B181B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B181B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B181C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B181C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B181D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B181D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B181E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B181E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B181F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B181F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B18200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B18208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B1820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B18210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B18218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B18220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B18228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B18230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B18238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B18240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B18248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B18250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B18258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B18260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B18268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B18270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B18278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B18280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B18288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B18290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B18294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B18298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B1829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B182A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B182A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B182B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B182B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B182C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B182C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B182D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B182D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B182E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B182E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B182F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B182F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B18300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B18308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B18310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B18318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B18320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B18328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B18330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B18338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B18340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B18348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B18350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B18358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B18360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B18368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B18370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B18378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B18380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B18388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B18390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B18398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B183A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B183A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B183B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B183B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B183C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B183C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B183D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B183D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B183E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B183E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B183F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B183F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B10100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B1500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B15014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B1501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B15024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B1502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B15034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B1503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B15044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B1504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B15054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B1505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B15064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B1506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B15074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B1507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B14000 ) = 0x00002D6A;
    *mkptr( dfe_cfg_base, 0x00B14004 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14008 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x00B1400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B14014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B1401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14020 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00B14024 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x00B14040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B04000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B04404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B04008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B04408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0400C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B0440C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B04010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B04410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B04014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B04414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B04018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B04418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B0401C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B0441C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B04020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B04420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B04024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B04424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B04028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B04428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B0402C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B0442C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B04030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B04034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B04434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B04038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B04438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B0403C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B0443C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B04040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B04444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B04048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B0404C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B0444C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B04450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B04054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B04454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B04058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B04458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0405C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B0445C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B04060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B04460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B04064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B04068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B04468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B0406C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B0446C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B04070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B04470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B04074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B04474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B04078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B04478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0407C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0447C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B04080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B04480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B04484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B04088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B04488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B0408C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B0448C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B04090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B04490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B04094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B04494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B04098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B0409C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B0449C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B040A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B044A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B040A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B044A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B040A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B044A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B040AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B044AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B040B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B044B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B040B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B044B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B040B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B044B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B040BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B044BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B040C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B044C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B040C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B044C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B040C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B044C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B040CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B044CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B040D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B044D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B040D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B044D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B040D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B044D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B040DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B044DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B040E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B044E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B040E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B044E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B040E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B044E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B040EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B044EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B040F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B044F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B040F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B044F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B040F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B044F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B040FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B044FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B04100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B04500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B08000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B08404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B08008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B08408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0800C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B0840C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B08010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B08410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B08014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B08414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B08018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B08418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B0801C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B0841C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B08020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B08420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B08024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B08424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B08028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B08428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B0802C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B0842C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B08030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B08034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B08434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B08038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B08438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B0803C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B0843C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B08040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B08444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B08048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B0804C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B0844C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B08450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B08054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B08454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B08058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B08458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0805C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B0845C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B08060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B08460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B08064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B08068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B08468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B0806C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B0846C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B08070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B08470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B08074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B08474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B08078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B08478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0807C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0847C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B08080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B08480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B08484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B08088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B08488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B0808C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B0848C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B08090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B08490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B08094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B08494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B08098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B0809C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B0849C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B080A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B084A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B080A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B084A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B080A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B084A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B080AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B084AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B080B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B084B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B080B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B084B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B080B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B084B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B080BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B084BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B080C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B084C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B080C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B084C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B080C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B084C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B080CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B084CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B080D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B084D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B080D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B084D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B080D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B084D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B080DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B084DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B080E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B084E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B080E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B084E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B080E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B084E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B080EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B084EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B080F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B084F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B080F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B084F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B080F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B084F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B080FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B084FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B08100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B08500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B803CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B80208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80360 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x00B80368 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00B80370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B80200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80230 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B80290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B802B0 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B80300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80310 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80318 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B80328 ) = 0x00000174;
    *mkptr( dfe_cfg_base, 0x00B80338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B80340 ) = 0x00000363;
    *mkptr( dfe_cfg_base, 0x00B80348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B80378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B98004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B98084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B98088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B9808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B98090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B98094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B98098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B9809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B980A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B980A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B980A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B980AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B980B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B980B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B980B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B980BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B980C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B980C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B980C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B980CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B980D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B980D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B980D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B980DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B980E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B980E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B980E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B980EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B980F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B980F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B980F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B980FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B98100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B98104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B98108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B9810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B98110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B98114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B98118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B9811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B98120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B98124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B98128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B9812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B98130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B98134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B98138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B9813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B98140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B98144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B98148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B9814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B98150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B98154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B98158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B9815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B98160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B98164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B98168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B98170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B98174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B98178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B98180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B98184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B98188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B9818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B98190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B98194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B98198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B9819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B981A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B981A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B981A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B981B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B981B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B981C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B981C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B981D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B981D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B981E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B981E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B981F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B981F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B98200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B98208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B9820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B98210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B98218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B98220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B98228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B98230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B98238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B98240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B98248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B98250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B98258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B98260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B98268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B98270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B98278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B98280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B98288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B98290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B98294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B98298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B9829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B982A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B982A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B982B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B982B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B982C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B982C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B982D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B982D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B982E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B982E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B982F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B982F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B98300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B98308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B98310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B98318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B98320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B98328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B98330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B98338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B98340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B98348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B98350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B98358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B98360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B98368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B98370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B98378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B98380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B98388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B98390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B98398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B983A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B983A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B983B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B983B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B983C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B983C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B983D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B983D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B983E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B983E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B983F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B983F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B90100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B9500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B95014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B9501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B95024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B9502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B95034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B9503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B95044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B9504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B95054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B9505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B95064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B9506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B95074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B9507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B94000 ) = 0x00002D6A;
    *mkptr( dfe_cfg_base, 0x00B94004 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94008 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x00B9400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B94014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B9401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94020 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00B94024 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x00B94040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B84000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B84404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B84008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B84408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8400C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B8440C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B84010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B84410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B84014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B84414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B84018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B84418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8401C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B8441C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B84020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B84420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B84024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B84424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B84028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B84428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B8402C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B8442C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B84030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B84034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B84038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B84438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B8403C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B8443C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B84444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B84048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B8404C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B8444C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B84450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B84054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B84454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B84058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B84458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8405C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B8445C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B84060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B84460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B84064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B84068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B84468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B8406C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B8446C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B84070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B84470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B84074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B84474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B84078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B84478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8407C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8447C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B84080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B84480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B84484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B84088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B84488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B8408C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B8448C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B84090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B84490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B84094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B84494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B84098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B8409C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B8449C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B840A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B844A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B840A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B844A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B840A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B844A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B840AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B844AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B840B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B844B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B840B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B844B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B840B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B844B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B840BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B844BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B840C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B844C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B840C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B844C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B840C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B844C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B840CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B844CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B840D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B844D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B840D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B844D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B840D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B844D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B840DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B844DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B840E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B844E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B840E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B844E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B840E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B844E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B840EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B844EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B840F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B844F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B840F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B844F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B840F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B844F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B840FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B844FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B84100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B84500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B88000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B88404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B88008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B88408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8800C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B8840C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B88010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B88410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B88014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B88414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B88018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B88418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8801C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B8841C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B88020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B88420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B88024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B88424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B88028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B88428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B8802C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B8842C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B88030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B88034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B88038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B88438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B8803C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B8843C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B88444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B88048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B8804C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B8844C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B88450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B88054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B88454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B88058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B88458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8805C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B8845C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B88060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B88460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B88064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B88068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B88468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B8806C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B8846C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B88070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B88470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B88074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B88474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B88078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B88478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8807C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8847C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B88080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B88480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B88484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B88088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B88488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B8808C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B8848C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B88090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B88490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B88094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B88494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B88098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B8809C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B8849C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B880A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B884A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B880A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B884A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B880A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B884A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B880AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B884AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B880B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B884B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B880B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B884B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B880B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B884B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B880BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B884BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B880C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B884C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B880C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B884C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B880C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B884C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B880CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B884CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B880D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B884D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B880D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B884D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B880D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B884D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B880DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B884DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B880E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B884E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B880E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B884E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B880E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B884E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B880EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B884EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B880F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B884F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B880F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B884F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B880F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B884F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B880FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B884FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B88100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B88500 ) = 0x000007B0;
    iqn2_normal_operation_config_split_1();
    iqn2_normal_operation_config_split_2();
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_normal_operation_config_split_1,".text:dfe");
#endif
static void iqn2_normal_operation_config_split_1()
{
    *mkptr( dfe_cfg_base, 0x01000008 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x0100000C ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000010 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000014 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000038 ) = 0x000C0003;
    *mkptr( dfe_cfg_base, 0x0100003C ) = 0x0000048A;
    *mkptr( dfe_cfg_base, 0x01000040 ) = 0x005300AA;
    *mkptr( dfe_cfg_base, 0x01000200 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01000404 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01001100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01001104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01001108 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x0100110C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001110 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001114 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001118 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01300200 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300204 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01300208 ) = 0x00000081;
    *mkptr( dfe_cfg_base, 0x0130020C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01300210 ) = 0x07070000;
    *mkptr( dfe_cfg_base, 0x01300214 ) = 0x0000F5F4;
    *mkptr( dfe_cfg_base, 0x01300228 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300400 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300408 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130040C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300410 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300414 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300418 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130041C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300420 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300428 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130042C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300430 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300434 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300438 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130043C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300440 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300448 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130044C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300450 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300454 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300458 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130045C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300460 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300468 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130046C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300470 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300474 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300478 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130047C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01343000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01600014 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600018 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160001C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600020 ) = 0x09090001;
    *mkptr( dfe_cfg_base, 0x0160002C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600054 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600058 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160005C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600060 ) = 0x09090003;
    *mkptr( dfe_cfg_base, 0x0160006C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600080 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600084 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600088 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160008C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600090 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600094 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600098 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160009C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x016000A0 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x016000A4 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x016000A8 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x016000AC ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x016000B0 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x016000B4 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000B8 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000C0 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x016000C4 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x016000C8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x016000CC ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x016000D0 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x016000D4 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x016000D8 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x016000DC ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x016000E0 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x016000E4 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x016000E8 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x016000EC ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x016000F0 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x016000F4 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000F8 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600100 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600104 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600108 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160010C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600110 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600114 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600118 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160011C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x01600120 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x01600124 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x01600128 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x0160012C ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x01600130 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x01600134 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600138 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600140 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x01600144 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x01600148 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x0160014C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01600150 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x01600154 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x01600158 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x0160015C ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x01600160 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x01600164 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x01600168 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x0160016C ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x01600170 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x01600174 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600178 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000BC ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x0160013C ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x01600200 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600204 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600208 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160020C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600210 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600214 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600218 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160021C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600220 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600224 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600228 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160022C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600230 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600234 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600238 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160023C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600600 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600604 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600608 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160060C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600610 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600614 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600618 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160061C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600620 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600624 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600628 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160062C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600630 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600634 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600638 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160063C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600B08 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B10 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B18 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B24 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B28 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B30 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600A00 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A04 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A08 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A0C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A10 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A14 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A18 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A1C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A28 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A2C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A30 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A34 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A80 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A84 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A88 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A8C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A90 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A94 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A98 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A9C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600AA8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AAC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01640000 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640004 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640008 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164000C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640010 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640014 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640018 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164001C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640020 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640024 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640028 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164002C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640030 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640034 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640038 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164003C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640040 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640044 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640048 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164004C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640050 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640054 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640058 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164005C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640060 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640064 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640068 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164006C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640070 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640074 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640078 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164007C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640080 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640084 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640088 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164008C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640090 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640094 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640098 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164009C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640100 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640104 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640108 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164010C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640110 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640114 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640118 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164011C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640120 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640124 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640128 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164012C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640130 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640134 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640138 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164013C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640140 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640144 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640148 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164014C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640150 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640154 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640158 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164015C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640160 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640164 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640168 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164016C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640170 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640174 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640178 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164017C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640180 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640184 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640188 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164018C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640190 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640194 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640198 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164019C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640204 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640208 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164020C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640210 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640214 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640218 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164021C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640220 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640224 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640228 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164022C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640230 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640234 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640238 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164023C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640240 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640244 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640248 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164024C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640250 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640254 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640258 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164025C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640260 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640264 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640268 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164026C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640270 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640274 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640278 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164027C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640280 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640284 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640288 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164028C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640290 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640294 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640298 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164029C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640300 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640304 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640308 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164030C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640310 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640314 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640318 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164031C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640320 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640324 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640328 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164032C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640330 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640334 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640338 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164033C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640340 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640344 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640348 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164034C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640350 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640354 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640358 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164035C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640360 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640364 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640368 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164036C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640374 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164037C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640380 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640384 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640388 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164038C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640390 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640394 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640398 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164039C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640400 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640404 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640408 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164040C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640410 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640414 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640418 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164041C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640420 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640424 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640428 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164042C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640430 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640434 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640438 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164043C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640440 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640444 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640448 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164044C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640450 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640454 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640458 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164045C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640460 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640464 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640468 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164046C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640470 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640474 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640478 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164047C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640480 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640484 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640488 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164048C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640490 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640494 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640498 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164049C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640500 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640504 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640508 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164050C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640510 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640514 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640518 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164051C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640520 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640524 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640528 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164052C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640530 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640534 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640538 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164053C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640540 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640544 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640548 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164054C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640550 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640554 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640558 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164055C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640560 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640564 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640568 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164056C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640570 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640574 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640578 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164057C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640580 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640584 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640588 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164058C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640590 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640594 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640598 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164059C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640600 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640604 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640608 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164060C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640610 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640614 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640618 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164061C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640620 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640624 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640628 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164062C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640630 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640634 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640638 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164063C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640640 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640644 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640648 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164064C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640650 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640654 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640658 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164065C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640660 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640664 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640668 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164066C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640670 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640674 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640678 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164067C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640680 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640684 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640688 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164068C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640690 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640694 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640698 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164069C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640700 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640704 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640708 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164070C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640710 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640714 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640718 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164071C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640720 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640724 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640728 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164072C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640730 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640734 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640738 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164073C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640740 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640744 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640748 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164074C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640750 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640754 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640758 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164075C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640760 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640764 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640768 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164076C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640770 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640774 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640778 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164077C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640780 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640784 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640788 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164078C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640790 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640794 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640798 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164079C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ABC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ACC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ADC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AFC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BBC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BCC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BDC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BFC ) = 0x00008000;
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_normal_operation_config_split_2,".text:dfe");
#endif
static void iqn2_normal_operation_config_split_2()
{
    *mkptr( dfe_cfg_base, 0x01D00020 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D00404 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00408 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0040C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00410 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00414 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00418 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0041C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00420 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00444 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00448 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0044C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00450 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00454 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00458 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0045C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00460 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00484 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00488 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0048C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00490 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00494 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00498 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0049C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004A0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D004C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004CC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004D0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D4 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004DC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004E0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D008C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D008C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D008CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D01404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D0140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D01410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D01808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0180C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D01810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01820 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01824 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0184C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D01850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01860 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01864 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01C04 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C44 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C84 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01CC4 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D02004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D02408 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D0240C ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02410 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D02414 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02418 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D0241C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D02420 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D02424 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D02428 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D0242C ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D02430 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D02434 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D02438 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D0243C ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D02440 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D02444 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D02448 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D0244C ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D02450 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D02454 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D02458 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D0245C ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D02460 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D02464 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D02468 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D0246C ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D02470 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D02474 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D02478 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D0247C ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D02480 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D02488 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D0248C ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02490 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D02494 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02498 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D0249C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D024A0 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D024A4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D024A8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D024AC ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D024B0 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D024B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D024B8 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D024BC ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D024C0 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D024C4 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D024C8 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D024CC ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D024D0 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D024D4 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D024D8 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D024DC ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D024E0 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D024E4 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D024E8 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D024EC ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D024F0 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D024F4 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D024F8 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D024FC ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D02500 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D02804 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02808 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0280C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02810 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02814 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02818 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0281C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02820 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02824 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02828 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0282C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02830 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02834 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02838 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0283C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02840 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D40014 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x01D40018 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D40044 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x01D40048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D40804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D408C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D408C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D408CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40904 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40908 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4090C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40944 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40948 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4094C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41004 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41014 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4101C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41024 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4102C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41034 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41044 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4104C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41054 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4105C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41064 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4106C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41074 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41084 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4108C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41094 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4109C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410A4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410AC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410B4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D4140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D41410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D41808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4180C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D41810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41820 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41824 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41828 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4184C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D41850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41860 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41864 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41868 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41C04 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C44 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C84 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41CC4 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D42004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D42808 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D42810 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D42844 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42848 ) = 0x00000210;
    *mkptr( dfe_cfg_base, 0x01D4284C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42850 ) = 0x00000210;
    *mkptr( dfe_cfg_base, 0x01D42884 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42888 ) = 0x00000220;
    *mkptr( dfe_cfg_base, 0x01D4288C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42890 ) = 0x00000220;
    *mkptr( dfe_cfg_base, 0x01D428C4 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428C8 ) = 0x00000230;
    *mkptr( dfe_cfg_base, 0x01D428CC ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428D0 ) = 0x00000230;
    *mkptr( dfe_cfg_base, 0x01D42904 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42908 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4290C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42910 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D42944 ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42948 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x01D4294C ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42950 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x01D42984 ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42988 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x01D4298C ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42990 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x01D429C4 ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429C8 ) = 0x00000330;
    *mkptr( dfe_cfg_base, 0x01D429CC ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429D0 ) = 0x00000330;
    *mkptr( dfe_cfg_base, 0x01D42A04 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42A08 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42A0C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42A10 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42A44 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42A48 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42A4C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42A50 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42A84 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42A88 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42A8C ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42A90 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42AC4 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42AC8 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42ACC ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42AD0 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42B04 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42B08 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42B0C ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42B10 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42B44 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42B48 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42B4C ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42B50 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42B84 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42B88 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42B8C ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42B90 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42BC4 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42BC8 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42BCC ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42BD0 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42C04 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42C08 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42C0C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42C10 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42C44 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42C48 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42C4C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42C50 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42C84 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42C88 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42C8C ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42C90 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42CC4 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42CC8 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42CCC ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42CD0 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42D04 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42D08 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42D0C ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42D10 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42D44 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42D48 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42D4C ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42D50 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42D84 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42D88 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42D8C ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42D90 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42DC4 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42DC8 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42DCC ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42DD0 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01A00E00 ) = 0x00000470;
    *mkptr( dfe_cfg_base, 0x01A00200 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01A00204 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01A006A0 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A0022C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A00230 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A41000 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41080 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A410C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41100 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41108 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A007EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00604 ) = 0x0000FF00;
    *mkptr( dfe_cfg_base, 0x01A00648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01A41800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x01A41804 ) = 0x0000006F;
    *mkptr( dfe_cfg_base, 0x01A41808 ) = 0x00000062;
    *mkptr( dfe_cfg_base, 0x01A4180C ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x01A41810 ) = 0x0000004F;
    *mkptr( dfe_cfg_base, 0x01A41814 ) = 0x00000047;
    *mkptr( dfe_cfg_base, 0x01A41818 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A4181C ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x01A41820 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x01A41824 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41828 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x01A4182C ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x01A41830 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01A41834 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41838 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A4183C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x01A41840 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01A41844 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01A41848 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A4184C ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01A41850 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01A41854 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01A41874 ) = 0x000000F9;
    *mkptr( dfe_cfg_base, 0x01A41878 ) = 0x000000F7;
    *mkptr( dfe_cfg_base, 0x01A4187C ) = 0x000000F5;
    *mkptr( dfe_cfg_base, 0x01A41880 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41884 ) = 0x000000F2;
    *mkptr( dfe_cfg_base, 0x01A41888 ) = 0x000000F1;
    *mkptr( dfe_cfg_base, 0x01A4188C ) = 0x000000EF;
    *mkptr( dfe_cfg_base, 0x01A41890 ) = 0x000000EE;
    *mkptr( dfe_cfg_base, 0x01A41894 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41898 ) = 0x000000EB;
    *mkptr( dfe_cfg_base, 0x01A4189C ) = 0x000000EA;
    *mkptr( dfe_cfg_base, 0x01A418A0 ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A418A4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x01A418A8 ) = 0x000000E6;
    *mkptr( dfe_cfg_base, 0x01A418AC ) = 0x000000E5;
    *mkptr( dfe_cfg_base, 0x01A418B0 ) = 0x000000E3;
    *mkptr( dfe_cfg_base, 0x01A418B4 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x01A418B8 ) = 0x000000E1;
    *mkptr( dfe_cfg_base, 0x01A418BC ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A418C0 ) = 0x000000DF;
    *mkptr( dfe_cfg_base, 0x01A418C4 ) = 0x000000DE;
    *mkptr( dfe_cfg_base, 0x01A418C8 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x01A418CC ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A418D0 ) = 0x000000DB;
    *mkptr( dfe_cfg_base, 0x01A418D4 ) = 0x000000DA;
    *mkptr( dfe_cfg_base, 0x01A418D8 ) = 0x000000D9;
    *mkptr( dfe_cfg_base, 0x01A418DC ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A418E0 ) = 0x000000D7;
    *mkptr( dfe_cfg_base, 0x01A418E4 ) = 0x000000D6;
    *mkptr( dfe_cfg_base, 0x01A418E8 ) = 0x000000D5;
    *mkptr( dfe_cfg_base, 0x01A418EC ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A418F0 ) = 0x000000D3;
    *mkptr( dfe_cfg_base, 0x01A418F4 ) = 0x000000D2;
    *mkptr( dfe_cfg_base, 0x01A418F8 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x01A418FC ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41900 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41904 ) = 0x000000CF;
    *mkptr( dfe_cfg_base, 0x01A41908 ) = 0x000000CE;
    *mkptr( dfe_cfg_base, 0x01A4190C ) = 0x000000CD;
    *mkptr( dfe_cfg_base, 0x01A41910 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41914 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41918 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x01A4191C ) = 0x000000CA;
    *mkptr( dfe_cfg_base, 0x01A41920 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41924 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41928 ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A4192C ) = 0x000000C7;
    *mkptr( dfe_cfg_base, 0x01A41930 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41934 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41938 ) = 0x000000C5;
    *mkptr( dfe_cfg_base, 0x01A4193C ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41940 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41944 ) = 0x000000C3;
    *mkptr( dfe_cfg_base, 0x01A41948 ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A4194C ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A41950 ) = 0x000000C1;
    *mkptr( dfe_cfg_base, 0x01A41954 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41958 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A4195C ) = 0x000000BF;
    *mkptr( dfe_cfg_base, 0x01A41960 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41964 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41968 ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A4196C ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A41970 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41974 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41978 ) = 0x000000BB;
    *mkptr( dfe_cfg_base, 0x01A4197C ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41980 ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41984 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A41988 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A4198C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41990 ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41994 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A41998 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A4199C ) = 0x000000B6;
    *mkptr( dfe_cfg_base, 0x01A419A0 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A4 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A8 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419AC ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419B0 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B4 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B8 ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419BC ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419C0 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C4 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C8 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419CC ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D0 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D4 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419D8 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419DC ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E0 ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E4 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419E8 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419EC ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F0 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F4 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419F8 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419FC ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A41A00 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A04 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A08 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A0C ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A10 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A14 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A18 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A1C ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A20 ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A24 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A28 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A2C ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A30 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A34 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A38 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A3C ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A40 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A44 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A48 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A4C ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A50 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A54 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A58 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A5C ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A60 ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A64 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A68 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A6C ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A70 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A74 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A78 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A7C ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A80 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A84 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A88 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A8C ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A90 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A94 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A98 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A9C ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41AA0 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA4 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA8 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AAC ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB0 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB4 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB8 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41ABC ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC0 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC4 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC8 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ACC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD0 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD4 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AD8 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41ADC ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AE0 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE4 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE8 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AEC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF0 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF4 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF8 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AFC ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B00 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B04 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B08 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B0C ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B10 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B14 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B18 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B1C ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B20 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B24 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B28 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B2C ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B30 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B34 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B38 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B3C ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B40 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B44 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B48 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B4C ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B50 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B54 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B58 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B5C ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B60 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B64 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B68 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B6C ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B70 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B74 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B78 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B7C ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B80 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B84 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B88 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B8C ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B90 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B94 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B98 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B9C ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA0 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA4 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BA8 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BAC ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB0 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BB8 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BBC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC0 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC8 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BCC ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD0 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD4 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BDC ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE0 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE4 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BEC ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF0 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF4 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF8 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BFC ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41C00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41C04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41C08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41C0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41C10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41C14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41C18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41C1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41C20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41C24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41C28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41C2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41C30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41C34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41C38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41C3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41C40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41C44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41C48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41C4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41C50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41C54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41C58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41C5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41C60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41C64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41C68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41C6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41C70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41C74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41C78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41C7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41C80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41C84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41C88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41C8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41C90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41C94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41C98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41C9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41CA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41CA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41CA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41CAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41CB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41CB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41CB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41CBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41CC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41CC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41CC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41CCC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41CD0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41CD4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41CD8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41CDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41CE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41CE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41CE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41CEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41CF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41CF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41CF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41CFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41D00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41D04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41D08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41D0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41D10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41D14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41D18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41D1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41D20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41D24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41D28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41D2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41D30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41D34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41D38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41D3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41D40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41D44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41D48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41D4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41D50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41D54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41D58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41D5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41D60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41D64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41D68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41D6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41D70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41D74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41D78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41D7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41D80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41D84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41D88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41D8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41D90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41D94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41D98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41D9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41DA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41DA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41DA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41DAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41DB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41DB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41DB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41DBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41DC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41DC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41DC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41DCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41DD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41DD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41DD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41DDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41DE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41DE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41DE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41DEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41DF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41DF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41DF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41E00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41E04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41E08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41E0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41E10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41E14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41E18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41E1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41E20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41E24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41E28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41E2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41E30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41E34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41E38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41E3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41E40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41E44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41E48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41E4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41E50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41E54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41E58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41E5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41E60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41E64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41E68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41E6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41E70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41E74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41E78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41E7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41E80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41E84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41E88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41E8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41E90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41E94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41E98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41E9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41EA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41EA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41EA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41EAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41EB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41EB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41EB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41EBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41EC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41EC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41EC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41ECC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ED0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41ED4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41ED8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41EDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41EE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41EE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41EE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41EEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41EF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41EF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41EF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41EFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41F00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41F04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41F08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41F0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41F10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41F14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41F18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41F1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41F20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41F24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41F28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41F2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41F30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41F34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41F38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41F3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41F40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41F44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41F48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41F4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41F50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41F54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41F58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41F5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41F60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41F64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41F68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41F6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41F70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41F74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41F78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41F7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41F80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41F84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41F88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41F8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41F90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41F94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41F98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41F9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41FA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41FA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41FA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41FAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41FB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41FB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41FB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41FBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41FC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41FC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41FC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41FCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41FD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41FD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41FD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41FDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41FE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41FE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41FE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41FEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41FF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41FF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41FF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42000 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42004 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42008 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4200C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42010 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42014 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42018 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4201C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42020 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42024 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42028 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4202C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42030 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42034 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42038 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4203C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42040 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42044 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42048 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4204C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42050 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42054 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42058 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4205C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42060 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42064 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42068 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4206C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42070 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42074 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42078 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4207C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42080 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42084 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42088 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4208C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42090 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42094 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42098 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4209C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A420A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A420A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A420A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A420AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A420B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A420B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A420B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A420BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A420C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A420C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A420C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A420CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A420D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A420D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A420D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A420DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A420E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A420E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A420E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A420EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A420F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A420F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A420F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A420FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42100 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42104 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42108 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4210C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42110 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42114 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42118 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4211C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42120 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42124 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42128 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4212C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42130 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42134 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42138 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4213C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42140 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42144 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42148 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4214C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42150 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42154 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42158 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4215C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42160 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42164 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42168 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4216C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42170 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42174 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42178 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4217C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42180 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42184 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42188 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4218C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42190 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42194 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42198 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4219C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A421A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A421A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A421A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A421AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A421B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A421B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A421B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A421BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A421C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A421C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A421C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A421CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A421D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A421D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A421D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A421DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A421E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A421E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A421E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A421EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A421F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A421F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A421F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42200 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42204 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42208 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4220C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42210 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42214 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42218 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4221C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42220 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42224 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42228 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4222C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42230 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42234 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42238 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4223C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42240 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42244 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42248 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4224C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42250 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42254 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42258 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4225C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42260 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42264 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42268 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4226C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42270 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42274 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42278 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4227C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42280 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42284 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42288 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4228C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42290 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42294 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42298 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4229C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A422A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A422A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A422A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A422AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A422B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A422B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A422B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A422BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A422C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A422C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A422C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A422CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A422D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A422D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A422D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A422DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A422E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A422E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A422E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A422EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A422F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A422F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A422F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A422FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42300 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42304 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42308 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4230C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42310 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42314 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42318 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4231C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42320 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42324 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42328 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4232C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42330 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42334 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42338 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4233C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42340 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42344 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42348 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4234C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42350 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42354 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42358 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4235C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42360 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42364 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42368 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4236C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42370 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42374 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42378 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4237C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42380 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42384 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42388 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4238C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42390 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42394 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42398 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4239C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A423A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A423A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A423A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A423AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A423B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A423B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A423B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A423BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A423C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A423C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A423C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A423CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A423D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A423D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A423D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A423DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A423E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A423E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A423E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A423EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A423F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A423F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A423F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41400 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41480 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A414C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41500 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41508 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A006EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00A0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A00A14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A0020C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A40220 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A402A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40320 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A403A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40020 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A400A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40120 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A401A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A0021C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01A00220 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x01A00224 ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A00228 ) = 0x00001280;
    *mkptr( dfe_cfg_base, 0x01A002D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A40800 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40804 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40808 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4080C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40810 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40814 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40818 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4081C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40820 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40824 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40828 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4082C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40830 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40834 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40838 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4083C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40840 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40844 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40848 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4084C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40850 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40854 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40858 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4085C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40860 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40864 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40868 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4086C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40870 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40874 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40878 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4087C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40880 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40884 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40888 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4088C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40890 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40894 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40898 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4089C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40900 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40904 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40908 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4090C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40910 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40914 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40918 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4091C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40920 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40924 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40928 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4092C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40930 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40934 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40938 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4093C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40940 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40944 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40948 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4094C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40950 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40954 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40958 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4095C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40960 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40964 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40968 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4096C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40970 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40974 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40978 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4097C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40980 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40984 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40988 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4098C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40990 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40994 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40998 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4099C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40A00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ABC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ACC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ADC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BBC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BCC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BDC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A003EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A0023C ) = 0x00005110;
    *mkptr( dfe_cfg_base, 0x01E8020C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01E80700 ) = 0x00001100;
    *mkptr( dfe_cfg_base, 0x01E80704 ) = 0x0000300C;
    *mkptr( dfe_cfg_base, 0x01E80708 ) = 0x0000300C;
    *mkptr( dfe_cfg_base, 0x01E80728 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01E8073C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01EC0000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC00C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC01C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E80248 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01E80210 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80204 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80218 ) = 0x00001F00;
    *mkptr( dfe_cfg_base, 0x01EC0800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01E80214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01E802B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00200 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01C00208 ) = 0x00001111;
    *mkptr( dfe_cfg_base, 0x01C0020C ) = 0x000002FD;
    *mkptr( dfe_cfg_base, 0x01C00214 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00218 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0021C ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00220 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01C00224 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00228 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0022C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00234 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00238 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0023C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00240 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00244 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00248 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0024C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00250 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00254 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00258 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0025C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00260 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00264 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00268 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0026C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00270 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00274 ) = 0x76541010;
    *mkptr( dfe_cfg_base, 0x01C00278 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00280 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01C00328 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01C003E4 ) = 0x01010101;
}
#endif
#if DFE_JESD_LOOPBACK == 1
static void iqn2_jesd_loopback_config_split_1();
static void iqn2_jesd_loopback_config_split_2();
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_jesd_loopback_config,".text:dfe");
#endif
void iqn2_jesd_loopback_config()
{
#if DFE_RATE==20
    // based on P1_3_LTE20_a_jesdlpbk.reg
    *mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x0004020C ) = 0x00004080;
    *mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x00040214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00040228 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00040240 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00040244 ) = 0x00002222;
    *mkptr( dfe_cfg_base, 0x00040250 ) = 0x00004444;
    *mkptr( dfe_cfg_base, 0x0004027C ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00040280 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x000402C0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402D0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402E4 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x000402F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00040344 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0004036C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00040374 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00040400 ) = 0x000001C4;
    *mkptr( dfe_cfg_base, 0x00040408 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040410 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040800 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000D67;
    *mkptr( dfe_cfg_base, 0x00060000 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060010 ) = 0x0000D001;
    *mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060020 ) = 0x0000D010;
    *mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060030 ) = 0x0000D011;
    *mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008420;
    *mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008421;
    *mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008430;
    *mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008431;
    *mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x0006100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00061010 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061014 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061018 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006101C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061020 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061024 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006102C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006103C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006104C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006105C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006106C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006107C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061084 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061088 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006108C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061090 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061094 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061098 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006109C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006110C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006111C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006112C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006113C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006114C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006115C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006116C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006117C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006118C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006119C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006120C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006121C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006122C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006123C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006124C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006125C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006126C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006127C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006128C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006129C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006130C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006131C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006132C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006133C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006134C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006135C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006136C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006137C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006138C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006139C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061800 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061808 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061810 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061818 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A00 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A08 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A10 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A18 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A20 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A28 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A30 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A38 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061C00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ECC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006200C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006201C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006202C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006203C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006204C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006205C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006206C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006207C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006208C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006209C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x0006240C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00062410 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062414 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062418 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006241C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062420 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062424 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062428 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006242C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062430 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062434 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062438 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006243C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006244C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006245C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006246C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006247C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062480 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062484 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062488 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006248C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062490 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062494 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062498 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006249C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006250C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006251C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006252C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006253C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006254C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006255C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006256C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006257C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006258C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006259C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006260C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006261C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006262C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006263C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006264C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006265C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006266C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006267C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006268C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006269C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006270C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006271C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006272C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006273C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006274C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006275C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006276C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006277C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006278C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006279C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006280C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006281C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006282C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006283C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006284C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006285C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006286C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006287C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006288C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006289C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006290C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006291C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006292C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006293C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006294C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006295C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006296C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006297C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006298C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006299C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ABC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ACC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ADC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062C00 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C04 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C08 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C0C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C10 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C14 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C18 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C1C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C20 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C24 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C28 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C2C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C30 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C34 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C38 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C3C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C40 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C44 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C48 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C4C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C50 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C54 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C58 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C5C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C60 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C64 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C68 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C6C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C70 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C74 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C78 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C7C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C80 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C84 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C88 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C8C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C90 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C94 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C98 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C9C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CAC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CBC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006300C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006301C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006302C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006303C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006304C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006305C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006306C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006307C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006308C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006309C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006310C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006311C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006312C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006313C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006314C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006315C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006316C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006317C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006318C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006319C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006320C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006321C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006322C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006323C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006324C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006325C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006326C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006327C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006328C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006329C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006330C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006331C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006332C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006333C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006334C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006335C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006336C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006337C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006338C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006339C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006340C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006341C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006342C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006343C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006344C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006345C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006346C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006347C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006348C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006349C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006350C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006351C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006352C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006353C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006354C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006355C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006356C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006357C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006358C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006359C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x0006400C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x0006401C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x0006402C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x0006403C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
    *mkptr( dfe_cfg_base, 0x0006404C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
    *mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
    *mkptr( dfe_cfg_base, 0x0006405C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
    *mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001A00;
    *mkptr( dfe_cfg_base, 0x0006406C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001C00;
    *mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001E00;
    *mkptr( dfe_cfg_base, 0x0006407C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x0006408C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
    *mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
    *mkptr( dfe_cfg_base, 0x0006409C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A0 ) = 0x00002800;
    *mkptr( dfe_cfg_base, 0x000640A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A8 ) = 0x00002A00;
    *mkptr( dfe_cfg_base, 0x000640AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B0 ) = 0x00002C00;
    *mkptr( dfe_cfg_base, 0x000640B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B8 ) = 0x00002E00;
    *mkptr( dfe_cfg_base, 0x000640BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C0 ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x000640C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C8 ) = 0x00003200;
    *mkptr( dfe_cfg_base, 0x000640CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D0 ) = 0x00003400;
    *mkptr( dfe_cfg_base, 0x000640D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D8 ) = 0x00003600;
    *mkptr( dfe_cfg_base, 0x000640DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E0 ) = 0x00003800;
    *mkptr( dfe_cfg_base, 0x000640E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E8 ) = 0x00003A00;
    *mkptr( dfe_cfg_base, 0x000640EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F0 ) = 0x00003C00;
    *mkptr( dfe_cfg_base, 0x000640F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F8 ) = 0x00003E00;
    *mkptr( dfe_cfg_base, 0x000640FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
    *mkptr( dfe_cfg_base, 0x0006410C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
    *mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
    *mkptr( dfe_cfg_base, 0x0006411C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
    *mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004A00;
    *mkptr( dfe_cfg_base, 0x0006412C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004C00;
    *mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004E00;
    *mkptr( dfe_cfg_base, 0x0006413C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
    *mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
    *mkptr( dfe_cfg_base, 0x0006414C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
    *mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
    *mkptr( dfe_cfg_base, 0x0006415C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
    *mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005A00;
    *mkptr( dfe_cfg_base, 0x0006416C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005C00;
    *mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005E00;
    *mkptr( dfe_cfg_base, 0x0006417C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
    *mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
    *mkptr( dfe_cfg_base, 0x0006418C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
    *mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
    *mkptr( dfe_cfg_base, 0x0006419C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A0 ) = 0x00006800;
    *mkptr( dfe_cfg_base, 0x000641A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A8 ) = 0x00006A00;
    *mkptr( dfe_cfg_base, 0x000641AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B0 ) = 0x00006C00;
    *mkptr( dfe_cfg_base, 0x000641B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B8 ) = 0x00006E00;
    *mkptr( dfe_cfg_base, 0x000641BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C0 ) = 0x00007000;
    *mkptr( dfe_cfg_base, 0x000641C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C8 ) = 0x00007200;
    *mkptr( dfe_cfg_base, 0x000641CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D0 ) = 0x00007400;
    *mkptr( dfe_cfg_base, 0x000641D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D8 ) = 0x00007600;
    *mkptr( dfe_cfg_base, 0x000641DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E0 ) = 0x00007800;
    *mkptr( dfe_cfg_base, 0x000641E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E8 ) = 0x00007A00;
    *mkptr( dfe_cfg_base, 0x000641EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F0 ) = 0x00007C00;
    *mkptr( dfe_cfg_base, 0x000641F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F8 ) = 0x00007E00;
    *mkptr( dfe_cfg_base, 0x000641FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
    *mkptr( dfe_cfg_base, 0x0006420C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
    *mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
    *mkptr( dfe_cfg_base, 0x0006421C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
    *mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008A00;
    *mkptr( dfe_cfg_base, 0x0006422C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008C00;
    *mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008E00;
    *mkptr( dfe_cfg_base, 0x0006423C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
    *mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
    *mkptr( dfe_cfg_base, 0x0006424C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
    *mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
    *mkptr( dfe_cfg_base, 0x0006425C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
    *mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009A00;
    *mkptr( dfe_cfg_base, 0x0006426C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009C00;
    *mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009E00;
    *mkptr( dfe_cfg_base, 0x0006427C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000A000;
    *mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000A200;
    *mkptr( dfe_cfg_base, 0x0006428C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000A400;
    *mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000A600;
    *mkptr( dfe_cfg_base, 0x0006429C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A0 ) = 0x0000A800;
    *mkptr( dfe_cfg_base, 0x000642A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A8 ) = 0x0000AA00;
    *mkptr( dfe_cfg_base, 0x000642AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B0 ) = 0x0000AC00;
    *mkptr( dfe_cfg_base, 0x000642B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B8 ) = 0x0000AE00;
    *mkptr( dfe_cfg_base, 0x000642BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C0 ) = 0x0000B000;
    *mkptr( dfe_cfg_base, 0x000642C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C8 ) = 0x0000B200;
    *mkptr( dfe_cfg_base, 0x000642CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D0 ) = 0x0000B400;
    *mkptr( dfe_cfg_base, 0x000642D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D8 ) = 0x0000B600;
    *mkptr( dfe_cfg_base, 0x000642DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E0 ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x000642E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E8 ) = 0x0000BA00;
    *mkptr( dfe_cfg_base, 0x000642EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F0 ) = 0x0000BC00;
    *mkptr( dfe_cfg_base, 0x000642F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F8 ) = 0x0000BE00;
    *mkptr( dfe_cfg_base, 0x000642FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000C000;
    *mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000C200;
    *mkptr( dfe_cfg_base, 0x0006430C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000C400;
    *mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000C600;
    *mkptr( dfe_cfg_base, 0x0006431C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000CA00;
    *mkptr( dfe_cfg_base, 0x0006432C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000CC00;
    *mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000CE00;
    *mkptr( dfe_cfg_base, 0x0006433C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000D200;
    *mkptr( dfe_cfg_base, 0x0006434C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000D400;
    *mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000D600;
    *mkptr( dfe_cfg_base, 0x0006435C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000D800;
    *mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000DA00;
    *mkptr( dfe_cfg_base, 0x0006436C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000DC00;
    *mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000DE00;
    *mkptr( dfe_cfg_base, 0x0006437C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000E000;
    *mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000E200;
    *mkptr( dfe_cfg_base, 0x0006438C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000E400;
    *mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000E600;
    *mkptr( dfe_cfg_base, 0x0006439C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A0 ) = 0x0000E800;
    *mkptr( dfe_cfg_base, 0x000643A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A8 ) = 0x0000EA00;
    *mkptr( dfe_cfg_base, 0x000643AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B0 ) = 0x0000EC00;
    *mkptr( dfe_cfg_base, 0x000643B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B8 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x000643BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C0 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000643C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C8 ) = 0x0000F200;
    *mkptr( dfe_cfg_base, 0x000643CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D0 ) = 0x0000F400;
    *mkptr( dfe_cfg_base, 0x000643D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D8 ) = 0x0000F600;
    *mkptr( dfe_cfg_base, 0x000643DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E0 ) = 0x0000F800;
    *mkptr( dfe_cfg_base, 0x000643E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E8 ) = 0x0000FA00;
    *mkptr( dfe_cfg_base, 0x000643EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F0 ) = 0x0000FC00;
    *mkptr( dfe_cfg_base, 0x000643F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F8 ) = 0x0000FE00;
    *mkptr( dfe_cfg_base, 0x000643FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0006C000 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C004 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C008 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C00C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C010 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C020 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C024 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C028 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C02C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C030 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C044 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C048 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C04C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C050 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C064 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C068 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C06C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C070 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C084 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C088 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C08C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C090 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0A4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0A8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0AC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0B0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0C4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0C8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0D0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0E4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0E8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0EC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0F0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000308;
    *mkptr( dfe_cfg_base, 0x00101300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00101304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00101308 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x00101318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00101324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x00140004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x0014000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x00140018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x0014001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00140200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x00140208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x0014020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x00140214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x00140400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x00140408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x00140410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x00140414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x00140600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x00140604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x00140610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x00140618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x0014061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x00140804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x0014080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x00140818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x0014081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x00140A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x00140A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x00140A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x00140C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x00140C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00140C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x00140C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00140E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x00140E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x00140E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00140E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x00140E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x00141004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x0014100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x00141018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x0014101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x00141204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x0014120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0010126C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00101344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00101348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0010134C ) = 0x00004300;
    *mkptr( dfe_cfg_base, 0x00101350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x00143800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143804 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014380C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143814 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x00143818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014381C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00101200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00101204 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0010120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00101214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101218 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0010121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101220 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00101224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0010122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00101230 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00101234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0010123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00101240 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00101244 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00101248 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00101250 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00101254 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00101258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0010125C ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00101260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00101264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00101268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001012DC ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x00100400 ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x00181300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00181304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00181308 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x00181318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00181324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x001C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x001C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x001C0018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x001C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x001C0200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x001C0208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x001C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x001C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x001C0400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x001C0408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x001C0410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x001C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x001C0600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x001C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x001C0610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x001C0618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x001C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x001C0804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x001C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x001C0818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x001C081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x001C0A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x001C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x001C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x001C0C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x001C0C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x001C0C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x001C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x001C0E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x001C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x001C0E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x001C0E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x001C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x001C1004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x001C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x001C1018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x001C101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x001C1204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x001C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0018126C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00181344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00181348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0018134C ) = 0x00004300;
    *mkptr( dfe_cfg_base, 0x00181350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x001C3800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3804 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C380C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3814 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001C3818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C381C ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x001812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00181200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00181204 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0018120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00181214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181218 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0018121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181220 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00181224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0018122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00181230 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00181234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0018123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00181240 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00181244 ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00181248 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00181250 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00181254 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00181258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0018125C ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00181260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00181264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00181268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001812DC ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x00180400 ) = 0x00004080;
    *mkptr( dfe_cfg_base, 0x00201300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00201304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00201308 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00201318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00201324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x00240004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x0024000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x00240018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x0024001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00240200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x00240208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x0024020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x00240214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x00240400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x00240408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x00240410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x00240414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x00240600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x00240604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x00240610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x00240618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x0024061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x00240804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x0024080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x00240818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x0024081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x00240A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x00240A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x00240A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x00240C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x00240C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00240C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x00240C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00240E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x00240E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x00240E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00240E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x00240E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x00241004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x0024100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x00241018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x0024101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x00241204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x0024120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0020126C ) = 0x00000216;
    *mkptr( dfe_cfg_base, 0x00201344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00201348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0020134C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00201350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x00243800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243804 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024380C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243814 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x00243818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024381C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00201200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00201204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00201208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0020120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00201214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201218 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0020121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201220 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00201224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201228 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0020122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00201230 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00201234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0020123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00201240 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201244 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201248 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00201250 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201254 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00201258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0020125C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00201260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00201264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00201268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002012DC ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x002012E4 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x00281300 ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00281304 ) = 0x00000438;
    *mkptr( dfe_cfg_base, 0x00281308 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00281318 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00281324 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0000 ) = 0x0000EAA5;
    *mkptr( dfe_cfg_base, 0x002C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0008 ) = 0x0000C721;
    *mkptr( dfe_cfg_base, 0x002C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0010 ) = 0x0000971E;
    *mkptr( dfe_cfg_base, 0x002C0018 ) = 0x0000401A;
    *mkptr( dfe_cfg_base, 0x002C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x002C0200 ) = 0x000013CE;
    *mkptr( dfe_cfg_base, 0x002C0208 ) = 0x0000ED20;
    *mkptr( dfe_cfg_base, 0x002C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0210 ) = 0x0000F4B0;
    *mkptr( dfe_cfg_base, 0x002C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0218 ) = 0x000027F2;
    *mkptr( dfe_cfg_base, 0x002C0400 ) = 0x0000057A;
    *mkptr( dfe_cfg_base, 0x002C0408 ) = 0x00000977;
    *mkptr( dfe_cfg_base, 0x002C0410 ) = 0x0000F0C3;
    *mkptr( dfe_cfg_base, 0x002C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0418 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x002C0600 ) = 0x0000F906;
    *mkptr( dfe_cfg_base, 0x002C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0608 ) = 0x000006B5;
    *mkptr( dfe_cfg_base, 0x002C0610 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x002C0618 ) = 0x0000F4F1;
    *mkptr( dfe_cfg_base, 0x002C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0800 ) = 0x0000FD84;
    *mkptr( dfe_cfg_base, 0x002C0804 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0808 ) = 0x0000FC7B;
    *mkptr( dfe_cfg_base, 0x002C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0810 ) = 0x0000060C;
    *mkptr( dfe_cfg_base, 0x002C0818 ) = 0x0000FF54;
    *mkptr( dfe_cfg_base, 0x002C081C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A00 ) = 0x000002B7;
    *mkptr( dfe_cfg_base, 0x002C0A08 ) = 0x0000FD1A;
    *mkptr( dfe_cfg_base, 0x002C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A10 ) = 0x0000FEFE;
    *mkptr( dfe_cfg_base, 0x002C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A18 ) = 0x0000047B;
    *mkptr( dfe_cfg_base, 0x002C0C00 ) = 0x0000011E;
    *mkptr( dfe_cfg_base, 0x002C0C08 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x002C0C10 ) = 0x0000FD8B;
    *mkptr( dfe_cfg_base, 0x002C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x002C0E00 ) = 0x0000FF24;
    *mkptr( dfe_cfg_base, 0x002C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0E08 ) = 0x0000011F;
    *mkptr( dfe_cfg_base, 0x002C0E10 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x002C0E18 ) = 0x0000FE59;
    *mkptr( dfe_cfg_base, 0x002C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1000 ) = 0x0000FF97;
    *mkptr( dfe_cfg_base, 0x002C1004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1008 ) = 0x0000FFB7;
    *mkptr( dfe_cfg_base, 0x002C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1010 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x002C1018 ) = 0x0000FFB4;
    *mkptr( dfe_cfg_base, 0x002C101C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1200 ) = 0x0000FFC1;
    *mkptr( dfe_cfg_base, 0x002C1204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1208 ) = 0x0000FF59;
    *mkptr( dfe_cfg_base, 0x002C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1218 ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x0028126C ) = 0x00000216;
    *mkptr( dfe_cfg_base, 0x00281344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00281348 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0028134C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00281350 ) = 0x00000481;
    *mkptr( dfe_cfg_base, 0x002C3800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3804 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C380C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3814 ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002C3818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C381C ) = 0x000001F8;
    *mkptr( dfe_cfg_base, 0x002812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00281200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00281204 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00281208 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x0028120C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281210 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00281214 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281218 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x0028121C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281220 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00281224 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281228 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0028122C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00281230 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00281234 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x0028123C ) = 0x00000243;
    *mkptr( dfe_cfg_base, 0x00281240 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281244 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281248 ) = 0x00000401;
    *mkptr( dfe_cfg_base, 0x00281250 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281254 ) = 0x00000402;
    *mkptr( dfe_cfg_base, 0x00281258 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x0028125C ) = 0x00000403;
    *mkptr( dfe_cfg_base, 0x00281260 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00281264 ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00281268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002812DC ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x002812E4 ) = 0x00000041;
    *mkptr( dfe_cfg_base, 0x00900004 ) = 0x00000606;
    *mkptr( dfe_cfg_base, 0x00900018 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00900030 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00B003CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B00208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00360 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x00B00368 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00B00370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B00200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00230 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B00290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B002B0 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B00300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00310 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00318 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B00328 ) = 0x00000174;
    *mkptr( dfe_cfg_base, 0x00B00338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B00340 ) = 0x00000363;
    *mkptr( dfe_cfg_base, 0x00B00348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B00378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B18004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B18084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B18088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B1808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B18090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B18094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B18098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B1809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B180A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B180A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B180A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B180AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B180B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B180B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B180B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B180BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B180C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B180C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B180C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B180CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B180D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B180D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B180D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B180DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B180E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B180E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B180E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B180EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B180F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B180F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B180F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B180FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B18100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B18104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B18108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B1810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B18110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B18114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B18118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B1811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B18120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B18124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B18128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B1812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B18130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B18134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B18138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B1813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B18140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B18144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B18148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B1814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B18150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B18154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B18158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B1815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B18160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B18164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B18168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B18170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B18174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B18178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B18180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B18184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B18188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B1818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B18190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B18194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B18198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B1819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B181A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B181A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B181A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B181B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B181B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B181C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B181C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B181D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B181D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B181E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B181E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B181F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B181F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B18200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B18208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B1820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B18210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B18218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B18220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B18228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B18230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B18238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B18240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B18248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B18250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B18258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B18260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B18268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B18270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B18278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B18280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B18288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B18290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B18294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B18298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B1829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B182A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B182A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B182B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B182B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B182C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B182C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B182D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B182D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B182E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B182E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B182F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B182F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B18300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B18308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B18310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B18318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B18320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B18328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B18330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B18338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B18340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B18348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B18350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B18358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B18360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B18368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B18370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B18378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B18380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B18388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B18390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B18398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B183A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B183A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B183B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B183B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B183C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B183C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B183D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B183D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B183E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B183E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B183F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B183F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B10100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B1500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B15014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B1501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B15024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B1502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B15034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B1503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B15044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B1504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B15054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B1505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B15064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B1506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B15074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B1507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B14000 ) = 0x00002D6A;
    *mkptr( dfe_cfg_base, 0x00B14004 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14008 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x00B1400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B14014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B1401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14020 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00B14024 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x00B14040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B04000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B04404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B04008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B04408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0400C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B0440C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B04010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B04410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B04014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B04414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B04018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B04418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B0401C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B0441C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B04020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B04420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B04024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B04424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B04028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B04428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B0402C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B0442C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B04030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B04034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B04434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B04038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B04438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B0403C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B0443C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B04040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B04444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B04048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B0404C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B0444C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B04450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B04054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B04454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B04058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B04458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0405C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B0445C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B04060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B04460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B04064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B04068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B04468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B0406C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B0446C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B04070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B04470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B04074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B04474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B04078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B04478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0407C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0447C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B04080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B04480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B04484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B04088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B04488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B0408C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B0448C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B04090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B04490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B04094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B04494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B04098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B0409C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B0449C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B040A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B044A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B040A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B044A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B040A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B044A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B040AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B044AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B040B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B044B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B040B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B044B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B040B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B044B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B040BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B044BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B040C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B044C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B040C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B044C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B040C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B044C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B040CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B044CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B040D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B044D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B040D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B044D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B040D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B044D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B040DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B044DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B040E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B044E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B040E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B044E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B040E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B044E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B040EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B044EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B040F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B044F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B040F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B044F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B040F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B044F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B040FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B044FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B04100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B04500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B08000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B08404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B08008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B08408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0800C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B0840C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B08010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B08410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B08014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B08414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B08018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B08418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B0801C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B0841C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B08020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B08420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B08024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B08424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B08028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B08428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B0802C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B0842C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B08030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B08034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B08434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B08038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B08438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B0803C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B0843C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B08040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B08444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B08048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B0804C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B0844C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B08450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B08054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B08454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B08058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B08458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0805C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B0845C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B08060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B08460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B08064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B08068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B08468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B0806C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B0846C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B08070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B08470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B08074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B08474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B08078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B08478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B0807C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0847C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B08080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B08480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B08484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B08088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B08488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B0808C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B0848C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B08090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B08490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B08094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B08494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B08098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B0809C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B0849C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B080A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B084A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B080A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B084A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B080A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B084A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B080AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B084AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B080B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B084B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B080B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B084B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B080B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B084B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B080BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B084BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B080C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B084C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B080C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B084C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B080C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B084C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B080CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B084CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B080D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B084D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B080D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B084D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B080D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B084D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B080DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B084DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B080E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B084E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B080E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B084E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B080E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B084E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B080EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B084EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B080F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B084F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B080F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B084F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B080F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B084F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B080FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B084FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B08100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B08500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B803CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B80208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80360 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x00B80368 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00B80370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B80200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80230 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B80290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B802B0 ) = 0x00004040;
    *mkptr( dfe_cfg_base, 0x00B80300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80310 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80318 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B80328 ) = 0x00000174;
    *mkptr( dfe_cfg_base, 0x00B80338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B80340 ) = 0x00000363;
    *mkptr( dfe_cfg_base, 0x00B80348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B80378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B98004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B98084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B98088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B9808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B98090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B98094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B98098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B9809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B980A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B980A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B980A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B980AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B980B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B980B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B980B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B980BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B980C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B980C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B980C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B980CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B980D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B980D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B980D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B980DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B980E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B980E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B980E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B980EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B980F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B980F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B980F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B980FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B98100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B98104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B98108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B9810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B98110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B98114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B98118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B9811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B98120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B98124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B98128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B9812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B98130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B98134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B98138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B9813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B98140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B98144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B98148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B9814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B98150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B98154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B98158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B9815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B98160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B98164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B98168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B98170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B98174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B98178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B98180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B98184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B98188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B9818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B98190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B98194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B98198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B9819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B981A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B981A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B981A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B981B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B981B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B981C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B981C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B981D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B981D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B981E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B981E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B981F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B981F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B98200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B98208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B9820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B98210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B98218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B98220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B98228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B98230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B98238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B98240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B98248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B98250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B98258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B98260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B98268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B98270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B98278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B98280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B98288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B98290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B98294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B98298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B9829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B982A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B982A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B982B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B982B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B982C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B982C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B982D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B982D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B982E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B982E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B982F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B982F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B98300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B98308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B98310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B98318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B98320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B98328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B98330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B98338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B98340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B98348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B98350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B98358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B98360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B98368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B98370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B98378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B98380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B98388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B98390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B98398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B983A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B983A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B983B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B983B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B983C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B983C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B983D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B983D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B983E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B983E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B983F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B983F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B90100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B9500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B95014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B9501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B95024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B9502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B95034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B9503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B95044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B9504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B95054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B9505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B95064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B9506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B95074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B9507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B94000 ) = 0x00002D6A;
    *mkptr( dfe_cfg_base, 0x00B94004 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94008 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x00B9400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B94014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B9401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94020 ) = 0x00000808;
    *mkptr( dfe_cfg_base, 0x00B94024 ) = 0x00000404;
    *mkptr( dfe_cfg_base, 0x00B94040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B84000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B84404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B84008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B84408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8400C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B8440C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B84010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B84410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B84014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B84414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B84018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B84418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8401C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B8441C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B84020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B84420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B84024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B84424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B84028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B84428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B8402C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B8442C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B84030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B84034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B84038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B84438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B8403C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B8443C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B84444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B84048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B8404C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B8444C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B84450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B84054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B84454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B84058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B84458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8405C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B8445C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B84060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B84460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B84064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B84068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B84468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B8406C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B8446C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B84070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B84470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B84074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B84474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B84078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B84478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8407C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8447C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B84080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B84480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B84484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B84088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B84488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B8408C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B8448C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B84090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B84490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B84094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B84494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B84098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B8409C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B8449C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B840A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B844A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B840A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B844A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B840A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B844A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B840AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B844AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B840B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B844B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B840B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B844B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B840B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B844B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B840BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B844BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B840C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B844C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B840C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B844C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B840C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B844C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B840CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B844CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B840D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B844D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B840D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B844D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B840D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B844D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B840DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B844DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B840E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B844E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B840E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B844E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B840E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B844E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B840EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B844EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B840F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B844F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B840F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B844F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B840F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B844F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B840FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B844FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B84100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B84500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x00B88000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88004 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x00B88404 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B88008 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B88408 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8800C ) = 0x0000003B;
    *mkptr( dfe_cfg_base, 0x00B8840C ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B88010 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x00B88410 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B88014 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B88414 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x00B88018 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B88418 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B8801C ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B8841C ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B88020 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B88420 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B88024 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B88424 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B88028 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B88428 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B8802C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B8842C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B88030 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88430 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B88034 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88434 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B88038 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B88438 ) = 0x00000FE7;
    *mkptr( dfe_cfg_base, 0x00B8803C ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B8843C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88440 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88044 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B88444 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B88048 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88448 ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B8804C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B8844C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88050 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B88450 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B88054 ) = 0x00000FE8;
    *mkptr( dfe_cfg_base, 0x00B88454 ) = 0x00000FE1;
    *mkptr( dfe_cfg_base, 0x00B88058 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B88458 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8805C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B8845C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00B88060 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x00B88460 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B88064 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88464 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B88068 ) = 0x00000FE0;
    *mkptr( dfe_cfg_base, 0x00B88468 ) = 0x00000FF0;
    *mkptr( dfe_cfg_base, 0x00B8806C ) = 0x00000FD8;
    *mkptr( dfe_cfg_base, 0x00B8846C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B88070 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B88470 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B88074 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B88474 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B88078 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B88478 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B8807C ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8847C ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x00B88080 ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B88480 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88084 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B88484 ) = 0x00000FD5;
    *mkptr( dfe_cfg_base, 0x00B88088 ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B88488 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B8808C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00B8848C ) = 0x00000FEE;
    *mkptr( dfe_cfg_base, 0x00B88090 ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x00B88490 ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x00B88094 ) = 0x00000039;
    *mkptr( dfe_cfg_base, 0x00B88494 ) = 0x00000042;
    *mkptr( dfe_cfg_base, 0x00B88098 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88498 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x00B8809C ) = 0x00000FC6;
    *mkptr( dfe_cfg_base, 0x00B8849C ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B880A0 ) = 0x00000FB6;
    *mkptr( dfe_cfg_base, 0x00B884A0 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B880A4 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B884A4 ) = 0x00000FC8;
    *mkptr( dfe_cfg_base, 0x00B880A8 ) = 0x00000036;
    *mkptr( dfe_cfg_base, 0x00B884A8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B880AC ) = 0x0000005B;
    *mkptr( dfe_cfg_base, 0x00B884AC ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00B880B0 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x00B884B0 ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x00B880B4 ) = 0x00000FD4;
    *mkptr( dfe_cfg_base, 0x00B884B4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B880B8 ) = 0x00000F94;
    *mkptr( dfe_cfg_base, 0x00B884B8 ) = 0x00000FAA;
    *mkptr( dfe_cfg_base, 0x00B880BC ) = 0x00000FB0;
    *mkptr( dfe_cfg_base, 0x00B884BC ) = 0x00000F93;
    *mkptr( dfe_cfg_base, 0x00B880C0 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B884C0 ) = 0x00000FDE;
    *mkptr( dfe_cfg_base, 0x00B880C4 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x00B884C4 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x00B880C8 ) = 0x00000079;
    *mkptr( dfe_cfg_base, 0x00B884C8 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B880CC ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B884CC ) = 0x00000051;
    *mkptr( dfe_cfg_base, 0x00B880D0 ) = 0x00000F76;
    *mkptr( dfe_cfg_base, 0x00B884D0 ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B880D4 ) = 0x00000F50;
    *mkptr( dfe_cfg_base, 0x00B884D4 ) = 0x00000F55;
    *mkptr( dfe_cfg_base, 0x00B880D8 ) = 0x00000FCA;
    *mkptr( dfe_cfg_base, 0x00B884D8 ) = 0x00000F7D;
    *mkptr( dfe_cfg_base, 0x00B880DC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x00B884DC ) = 0x0000002F;
    *mkptr( dfe_cfg_base, 0x00B880E0 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x00B884E0 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x00B880E4 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x00B884E4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x00B880E8 ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B884E8 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B880EC ) = 0x00000E5F;
    *mkptr( dfe_cfg_base, 0x00B884EC ) = 0x00000EC9;
    *mkptr( dfe_cfg_base, 0x00B880F0 ) = 0x00000E9F;
    *mkptr( dfe_cfg_base, 0x00B884F0 ) = 0x00000E49;
    *mkptr( dfe_cfg_base, 0x00B880F4 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x00B884F4 ) = 0x00000F6B;
    *mkptr( dfe_cfg_base, 0x00B880F8 ) = 0x000003D9;
    *mkptr( dfe_cfg_base, 0x00B884F8 ) = 0x0000022C;
    *mkptr( dfe_cfg_base, 0x00B880FC ) = 0x000006CD;
    *mkptr( dfe_cfg_base, 0x00B884FC ) = 0x00000576;
    *mkptr( dfe_cfg_base, 0x00B88100 ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B88500 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x01000008 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x0100000C ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000010 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000014 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000038 ) = 0x000C0003;
    *mkptr( dfe_cfg_base, 0x0100003C ) = 0x0000048A;
    *mkptr( dfe_cfg_base, 0x01000040 ) = 0x005300AA;
    *mkptr( dfe_cfg_base, 0x01000200 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01000404 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01001100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01001104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01001108 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x0100110C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001110 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001114 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001118 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01300200 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300204 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01300208 ) = 0x00000081;
    *mkptr( dfe_cfg_base, 0x0130020C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01300210 ) = 0x07070000;
    *mkptr( dfe_cfg_base, 0x01300214 ) = 0x0000F5F4;
    *mkptr( dfe_cfg_base, 0x01300228 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300400 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300408 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130040C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300410 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300414 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300418 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130041C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300420 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300428 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130042C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300430 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300434 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300438 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130043C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300440 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300448 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130044C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300450 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300454 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300458 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130045C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300460 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300468 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130046C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300470 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300474 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300478 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130047C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01343000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F8 ) = 0x00002000;
#elif DFE_RATE == 10
    *mkptr( dfe_cfg_base, 0x00040234 ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x0004020C ) = 0x000060C0;
    *mkptr( dfe_cfg_base, 0x00040210 ) = 0x00001830;
    *mkptr( dfe_cfg_base, 0x00040214 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00040228 ) = 0x00001800;
    *mkptr( dfe_cfg_base, 0x00040240 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00040244 ) = 0x00006666;
    *mkptr( dfe_cfg_base, 0x00040250 ) = 0x00003333;
    *mkptr( dfe_cfg_base, 0x0004027C ) = 0x00001818;
    *mkptr( dfe_cfg_base, 0x00040280 ) = 0x00001818;
    *mkptr( dfe_cfg_base, 0x000402C0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402D0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x000402E0 ) = 0x00005000;
    *mkptr( dfe_cfg_base, 0x000402F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00040344 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00040360 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00040364 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0004036C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00040370 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00040400 ) = 0x000001D3;
    *mkptr( dfe_cfg_base, 0x00040408 ) = 0x000005FC;
    *mkptr( dfe_cfg_base, 0x00040410 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00040800 ) = 0x000000D3;
    *mkptr( dfe_cfg_base, 0x00040808 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00040810 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00040818 ) = 0x00000D8B;
    *mkptr( dfe_cfg_base, 0x00041200 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x0004120C ) = 0x000005FC;
    *mkptr( dfe_cfg_base, 0x00041210 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00040600 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00040A00 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00060000 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00060004 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060010 ) = 0x0000D001;
    *mkptr( dfe_cfg_base, 0x00060014 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060020 ) = 0x0000D002;
    *mkptr( dfe_cfg_base, 0x00060024 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060030 ) = 0x0000D003;
    *mkptr( dfe_cfg_base, 0x00060034 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060040 ) = 0x0000D004;
    *mkptr( dfe_cfg_base, 0x00060044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060050 ) = 0x0000D005;
    *mkptr( dfe_cfg_base, 0x00060054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060060 ) = 0x0000D010;
    *mkptr( dfe_cfg_base, 0x00060064 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060070 ) = 0x0000D011;
    *mkptr( dfe_cfg_base, 0x00060074 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060080 ) = 0x0000D012;
    *mkptr( dfe_cfg_base, 0x00060084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060090 ) = 0x0000D013;
    *mkptr( dfe_cfg_base, 0x00060094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x000600A0 ) = 0x0000D014;
    *mkptr( dfe_cfg_base, 0x000600A4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x000600B0 ) = 0x0000D015;
    *mkptr( dfe_cfg_base, 0x000600B4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00060800 ) = 0x00008330;
    *mkptr( dfe_cfg_base, 0x00060804 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060820 ) = 0x00008331;
    *mkptr( dfe_cfg_base, 0x00060824 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060840 ) = 0x00008332;
    *mkptr( dfe_cfg_base, 0x00060844 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060860 ) = 0x00008333;
    *mkptr( dfe_cfg_base, 0x00060864 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060880 ) = 0x00008334;
    *mkptr( dfe_cfg_base, 0x00060884 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608A0 ) = 0x00008335;
    *mkptr( dfe_cfg_base, 0x000608A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608C0 ) = 0x00008320;
    *mkptr( dfe_cfg_base, 0x000608C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000608E0 ) = 0x00008321;
    *mkptr( dfe_cfg_base, 0x000608E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060900 ) = 0x00008322;
    *mkptr( dfe_cfg_base, 0x00060904 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060920 ) = 0x00008323;
    *mkptr( dfe_cfg_base, 0x00060924 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060940 ) = 0x00008324;
    *mkptr( dfe_cfg_base, 0x00060944 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060960 ) = 0x00008325;
    *mkptr( dfe_cfg_base, 0x00060964 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060984 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609A4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609C4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000609E4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060A84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060AE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060B84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060BE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060C84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060CE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D04 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D24 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D44 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D64 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060D84 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DA4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DC4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00060DE4 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x00061004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061008 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x0006100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00061010 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00061014 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00061018 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x0006101C ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00061020 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00061024 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x00061028 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x0006102C ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x00061030 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061034 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061038 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006103C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061044 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061048 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006104C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061050 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061054 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061058 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006105C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061064 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061068 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006106C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061070 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061074 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061078 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006107C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061084 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061088 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006108C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061090 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061094 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061098 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006109C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000610FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061100 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061104 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061108 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006110C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061110 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061114 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061118 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006111C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061120 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061124 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061128 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006112C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061130 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061134 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061138 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006113C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061140 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061144 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061148 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006114C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061150 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061154 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061158 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006115C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061160 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061164 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061168 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006116C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061170 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061174 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061178 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006117C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061180 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061184 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061188 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006118C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061190 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061194 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061198 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006119C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000611FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061200 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061204 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061208 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006120C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061210 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061214 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061218 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006121C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061220 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061224 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061228 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006122C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061230 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061234 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061238 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006123C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061240 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061244 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061248 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006124C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061250 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061254 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061258 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006125C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061260 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061264 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061268 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006126C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061270 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061274 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061278 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006127C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061280 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061284 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061288 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006128C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061290 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061294 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061298 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006129C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000612FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061300 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061304 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061308 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006130C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061310 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061314 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061318 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006131C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061320 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061324 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061328 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006132C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061330 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061334 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061338 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006133C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061340 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061344 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061348 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006134C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061350 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061354 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061358 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006135C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061360 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061364 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061368 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006136C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061370 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061374 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061378 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006137C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061380 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061384 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061388 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006138C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061390 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061394 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061398 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006139C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000613FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061400 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061408 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061410 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061418 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061420 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061428 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061430 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061438 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061440 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061448 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061450 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061458 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061460 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061468 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061470 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061478 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061480 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061488 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061498 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614A8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614B8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614C8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614D8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614E8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000614F8 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061500 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061508 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061510 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061518 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061520 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061528 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061530 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061538 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061540 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061548 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061550 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061558 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061560 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061568 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061570 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061578 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00061800 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061808 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061810 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061818 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061820 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061828 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061830 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061838 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061840 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061848 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061850 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061858 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061860 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061868 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061870 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061878 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061880 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061888 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061890 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061898 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000618F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061900 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061908 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061910 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061918 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061920 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061928 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061930 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061938 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061940 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061948 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061950 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061958 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061960 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061968 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061970 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061978 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061980 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061988 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061990 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061998 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619A8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619B8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619E8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x000619F8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A00 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A08 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A10 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A18 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A20 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A28 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A30 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061A38 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00061C00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061C9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061CFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061D9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061DFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061E9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ECC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061ED8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061EFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061F9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00061FFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006200C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006201C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006202C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006203C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006204C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006205C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006206C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006207C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006208C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006209C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000620BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062400 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00062404 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x00062408 ) = 0x00000302;
    *mkptr( dfe_cfg_base, 0x0006240C ) = 0x00000303;
    *mkptr( dfe_cfg_base, 0x00062410 ) = 0x00000304;
    *mkptr( dfe_cfg_base, 0x00062414 ) = 0x00000305;
    *mkptr( dfe_cfg_base, 0x00062418 ) = 0x00000306;
    *mkptr( dfe_cfg_base, 0x0006241C ) = 0x00000307;
    *mkptr( dfe_cfg_base, 0x00062420 ) = 0x00000308;
    *mkptr( dfe_cfg_base, 0x00062424 ) = 0x00000309;
    *mkptr( dfe_cfg_base, 0x00062428 ) = 0x0000030A;
    *mkptr( dfe_cfg_base, 0x0006242C ) = 0x0000030B;
    *mkptr( dfe_cfg_base, 0x00062430 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062434 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062438 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006243C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062440 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062444 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062448 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006244C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062450 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062454 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062458 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006245C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062460 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062464 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062468 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006246C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062470 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062474 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062478 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006247C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062480 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062484 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062488 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006248C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062490 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062494 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062498 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006249C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000624FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062500 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062504 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062508 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006250C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062510 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062514 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062518 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006251C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062520 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062524 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062528 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006252C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062530 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062534 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062538 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006253C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062540 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062544 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062548 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006254C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062550 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062554 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062558 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006255C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062560 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062564 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062568 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006256C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062570 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062574 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062578 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006257C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062580 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062584 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062588 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006258C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062590 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062594 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062598 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006259C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000625FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062600 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062604 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062608 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006260C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062610 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062614 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062618 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006261C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062620 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062624 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062628 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006262C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062630 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062634 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062638 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006263C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062640 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062644 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062648 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006264C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062650 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062654 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062658 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006265C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062660 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062664 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062668 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006266C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062670 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062674 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062678 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006267C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062680 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062684 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062688 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006268C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062690 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062694 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062698 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006269C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000626FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062700 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062704 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062708 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006270C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062710 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062714 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062718 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006271C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062720 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062724 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062728 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006272C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062730 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062734 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062738 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006273C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062740 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062744 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062748 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006274C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062750 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062754 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062758 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006275C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062760 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062764 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062768 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006276C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062770 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062774 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062778 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006277C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062780 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062784 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062788 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006278C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062790 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062794 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062798 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006279C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000627FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062804 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062808 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006280C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062810 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062814 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062818 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006281C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062820 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062824 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062828 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006282C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062830 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062834 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062838 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006283C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062840 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062844 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062848 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006284C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062850 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062854 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062858 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006285C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062860 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062864 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062868 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006286C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062870 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062874 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062878 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006287C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062880 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062884 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062888 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006288C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062890 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062894 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062898 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006289C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000628FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062900 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062904 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062908 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006290C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062910 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062914 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062918 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006291C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062920 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062924 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062928 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006292C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062930 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062934 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062938 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006293C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062940 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062944 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062948 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006294C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062950 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062954 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062958 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006295C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062960 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062964 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062968 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006296C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062970 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062974 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062978 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006297C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062980 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062984 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062988 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006298C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062990 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062994 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062998 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006299C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000629FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062A9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ABC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ACC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062ADC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062AFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B00 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B04 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B08 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B0C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B10 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B14 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B18 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B1C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B20 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B24 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B28 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B2C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B30 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B34 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B38 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B3C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B40 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B44 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B48 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B4C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B50 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B54 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B58 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B5C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B60 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B64 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B68 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B6C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B70 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B74 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B78 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B7C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B80 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B84 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B88 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B8C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B90 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B94 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B98 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062B9C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BA8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BAC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BB8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BBC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BC8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BCC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BD8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BDC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BE8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BEC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BF8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062BFC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00062C00 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C04 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C08 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C0C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C10 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C14 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C18 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C1C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C20 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C24 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C28 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C2C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C30 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C34 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C38 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C3C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C40 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C44 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C48 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C4C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C50 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C54 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C58 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C5C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C60 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C64 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C68 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C6C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C70 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C74 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C78 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C7C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C80 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C84 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C88 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C8C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C90 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C94 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C98 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062C9C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CA8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CAC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CB8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00062CBC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063000 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063004 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063008 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006300C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063010 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063014 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063018 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006301C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063020 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063024 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063028 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006302C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063030 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063034 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063038 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006303C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063040 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063044 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063048 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006304C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063050 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063054 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063058 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006305C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063060 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063064 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063068 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006306C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063070 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063074 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063078 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006307C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063080 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063084 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063088 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006308C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063090 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063094 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063098 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006309C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000630FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063100 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063104 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063108 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006310C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063110 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063114 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063118 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006311C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063120 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063124 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063128 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006312C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063130 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063134 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063138 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006313C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063140 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063144 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063148 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006314C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063150 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063154 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063158 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006315C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063160 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063164 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063168 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006316C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063170 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063174 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063178 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006317C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063180 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063184 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063188 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006318C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063190 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063194 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063198 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006319C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000631FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063200 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063204 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063208 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006320C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063210 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063214 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063218 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006321C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063220 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063224 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063228 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006322C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063230 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063234 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063238 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006323C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063240 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063244 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063248 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006324C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063250 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063254 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063258 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006325C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063260 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063264 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063268 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006326C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063270 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063274 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063278 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006327C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063280 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063284 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063288 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006328C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063290 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063294 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063298 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006329C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000632FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063300 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063304 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063308 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006330C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063310 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063314 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063318 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006331C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063320 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063324 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063328 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006332C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063330 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063334 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063338 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006333C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063340 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063344 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006334C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063350 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063354 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063358 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006335C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063360 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063364 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063368 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006336C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063370 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063374 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063378 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006337C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063380 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063384 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063388 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006338C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063390 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063394 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063398 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x0006339C ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633A8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633AC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633B8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633BC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633C8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633CC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633D8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633DC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633E8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633EC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F0 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F4 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633F8 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x000633FC ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00063400 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063404 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063408 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006340C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063410 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063414 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063418 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006341C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063420 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063424 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063428 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006342C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063430 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063434 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063438 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006343C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063440 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063444 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063448 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006344C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063450 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063454 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063458 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006345C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063460 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063464 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063468 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006346C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063470 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063474 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063478 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006347C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063480 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063484 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063488 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006348C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063490 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063494 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063498 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006349C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000634FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063500 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063504 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063508 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006350C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063510 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063514 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063518 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006351C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063520 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063524 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063528 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006352C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063530 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063534 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063538 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006353C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063540 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063544 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063548 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006354C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063550 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063554 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063558 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006355C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063560 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063564 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063568 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006356C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063570 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063574 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063578 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006357C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063580 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063584 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063588 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006358C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063590 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063594 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00063598 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006359C ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635A8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635AC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635B8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635BC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635C8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635CC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635D8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635DC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635E8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635EC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F4 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635F8 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x000635FC ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x00064004 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064008 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x0006400C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064010 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00064014 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064018 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x0006401C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064020 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00064024 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064028 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x0006402C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064030 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00064034 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064038 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x0006403C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064040 ) = 0x00001000;
    *mkptr( dfe_cfg_base, 0x00064044 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064048 ) = 0x00001200;
    *mkptr( dfe_cfg_base, 0x0006404C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064050 ) = 0x00001400;
    *mkptr( dfe_cfg_base, 0x00064054 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064058 ) = 0x00001600;
    *mkptr( dfe_cfg_base, 0x0006405C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064060 ) = 0x00001800;
    *mkptr( dfe_cfg_base, 0x00064064 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064068 ) = 0x00001A00;
    *mkptr( dfe_cfg_base, 0x0006406C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064070 ) = 0x00001C00;
    *mkptr( dfe_cfg_base, 0x00064074 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064078 ) = 0x00001E00;
    *mkptr( dfe_cfg_base, 0x0006407C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x00064084 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064088 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x0006408C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064090 ) = 0x00002400;
    *mkptr( dfe_cfg_base, 0x00064094 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064098 ) = 0x00002600;
    *mkptr( dfe_cfg_base, 0x0006409C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A0 ) = 0x00002800;
    *mkptr( dfe_cfg_base, 0x000640A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640A8 ) = 0x00002A00;
    *mkptr( dfe_cfg_base, 0x000640AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B0 ) = 0x00002C00;
    *mkptr( dfe_cfg_base, 0x000640B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640B8 ) = 0x00002E00;
    *mkptr( dfe_cfg_base, 0x000640BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C0 ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x000640C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640C8 ) = 0x00003200;
    *mkptr( dfe_cfg_base, 0x000640CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D0 ) = 0x00003400;
    *mkptr( dfe_cfg_base, 0x000640D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640D8 ) = 0x00003600;
    *mkptr( dfe_cfg_base, 0x000640DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E0 ) = 0x00003800;
    *mkptr( dfe_cfg_base, 0x000640E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640E8 ) = 0x00003A00;
    *mkptr( dfe_cfg_base, 0x000640EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F0 ) = 0x00003C00;
    *mkptr( dfe_cfg_base, 0x000640F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000640F8 ) = 0x00003E00;
    *mkptr( dfe_cfg_base, 0x000640FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00064104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064108 ) = 0x00004200;
    *mkptr( dfe_cfg_base, 0x0006410C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064110 ) = 0x00004400;
    *mkptr( dfe_cfg_base, 0x00064114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064118 ) = 0x00004600;
    *mkptr( dfe_cfg_base, 0x0006411C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064120 ) = 0x00004800;
    *mkptr( dfe_cfg_base, 0x00064124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064128 ) = 0x00004A00;
    *mkptr( dfe_cfg_base, 0x0006412C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064130 ) = 0x00004C00;
    *mkptr( dfe_cfg_base, 0x00064134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064138 ) = 0x00004E00;
    *mkptr( dfe_cfg_base, 0x0006413C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064140 ) = 0x00005000;
    *mkptr( dfe_cfg_base, 0x00064144 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064148 ) = 0x00005200;
    *mkptr( dfe_cfg_base, 0x0006414C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064150 ) = 0x00005400;
    *mkptr( dfe_cfg_base, 0x00064154 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064158 ) = 0x00005600;
    *mkptr( dfe_cfg_base, 0x0006415C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064160 ) = 0x00005800;
    *mkptr( dfe_cfg_base, 0x00064164 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064168 ) = 0x00005A00;
    *mkptr( dfe_cfg_base, 0x0006416C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064170 ) = 0x00005C00;
    *mkptr( dfe_cfg_base, 0x00064174 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064178 ) = 0x00005E00;
    *mkptr( dfe_cfg_base, 0x0006417C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064180 ) = 0x00006000;
    *mkptr( dfe_cfg_base, 0x00064184 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064188 ) = 0x00006200;
    *mkptr( dfe_cfg_base, 0x0006418C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064190 ) = 0x00006400;
    *mkptr( dfe_cfg_base, 0x00064194 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064198 ) = 0x00006600;
    *mkptr( dfe_cfg_base, 0x0006419C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A0 ) = 0x00006800;
    *mkptr( dfe_cfg_base, 0x000641A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641A8 ) = 0x00006A00;
    *mkptr( dfe_cfg_base, 0x000641AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B0 ) = 0x00006C00;
    *mkptr( dfe_cfg_base, 0x000641B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641B8 ) = 0x00006E00;
    *mkptr( dfe_cfg_base, 0x000641BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C0 ) = 0x00007000;
    *mkptr( dfe_cfg_base, 0x000641C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641C8 ) = 0x00007200;
    *mkptr( dfe_cfg_base, 0x000641CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D0 ) = 0x00007400;
    *mkptr( dfe_cfg_base, 0x000641D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641D8 ) = 0x00007600;
    *mkptr( dfe_cfg_base, 0x000641DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E0 ) = 0x00007800;
    *mkptr( dfe_cfg_base, 0x000641E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641E8 ) = 0x00007A00;
    *mkptr( dfe_cfg_base, 0x000641EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F0 ) = 0x00007C00;
    *mkptr( dfe_cfg_base, 0x000641F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000641F8 ) = 0x00007E00;
    *mkptr( dfe_cfg_base, 0x000641FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00064204 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064208 ) = 0x00008200;
    *mkptr( dfe_cfg_base, 0x0006420C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064210 ) = 0x00008400;
    *mkptr( dfe_cfg_base, 0x00064214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064218 ) = 0x00008600;
    *mkptr( dfe_cfg_base, 0x0006421C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064220 ) = 0x00008800;
    *mkptr( dfe_cfg_base, 0x00064224 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064228 ) = 0x00008A00;
    *mkptr( dfe_cfg_base, 0x0006422C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064230 ) = 0x00008C00;
    *mkptr( dfe_cfg_base, 0x00064234 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064238 ) = 0x00008E00;
    *mkptr( dfe_cfg_base, 0x0006423C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064240 ) = 0x00009000;
    *mkptr( dfe_cfg_base, 0x00064244 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064248 ) = 0x00009200;
    *mkptr( dfe_cfg_base, 0x0006424C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064250 ) = 0x00009400;
    *mkptr( dfe_cfg_base, 0x00064254 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064258 ) = 0x00009600;
    *mkptr( dfe_cfg_base, 0x0006425C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064260 ) = 0x00009800;
    *mkptr( dfe_cfg_base, 0x00064264 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064268 ) = 0x00009A00;
    *mkptr( dfe_cfg_base, 0x0006426C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064270 ) = 0x00009C00;
    *mkptr( dfe_cfg_base, 0x00064274 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064278 ) = 0x00009E00;
    *mkptr( dfe_cfg_base, 0x0006427C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064280 ) = 0x0000A000;
    *mkptr( dfe_cfg_base, 0x00064284 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064288 ) = 0x0000A200;
    *mkptr( dfe_cfg_base, 0x0006428C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064290 ) = 0x0000A400;
    *mkptr( dfe_cfg_base, 0x00064294 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064298 ) = 0x0000A600;
    *mkptr( dfe_cfg_base, 0x0006429C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A0 ) = 0x0000A800;
    *mkptr( dfe_cfg_base, 0x000642A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642A8 ) = 0x0000AA00;
    *mkptr( dfe_cfg_base, 0x000642AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B0 ) = 0x0000AC00;
    *mkptr( dfe_cfg_base, 0x000642B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642B8 ) = 0x0000AE00;
    *mkptr( dfe_cfg_base, 0x000642BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C0 ) = 0x0000B000;
    *mkptr( dfe_cfg_base, 0x000642C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642C8 ) = 0x0000B200;
    *mkptr( dfe_cfg_base, 0x000642CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D0 ) = 0x0000B400;
    *mkptr( dfe_cfg_base, 0x000642D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642D8 ) = 0x0000B600;
    *mkptr( dfe_cfg_base, 0x000642DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E0 ) = 0x0000B800;
    *mkptr( dfe_cfg_base, 0x000642E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642E8 ) = 0x0000BA00;
    *mkptr( dfe_cfg_base, 0x000642EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F0 ) = 0x0000BC00;
    *mkptr( dfe_cfg_base, 0x000642F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000642F8 ) = 0x0000BE00;
    *mkptr( dfe_cfg_base, 0x000642FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064300 ) = 0x0000C000;
    *mkptr( dfe_cfg_base, 0x00064304 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064308 ) = 0x0000C200;
    *mkptr( dfe_cfg_base, 0x0006430C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064310 ) = 0x0000C400;
    *mkptr( dfe_cfg_base, 0x00064314 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064318 ) = 0x0000C600;
    *mkptr( dfe_cfg_base, 0x0006431C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064320 ) = 0x0000C800;
    *mkptr( dfe_cfg_base, 0x00064324 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064328 ) = 0x0000CA00;
    *mkptr( dfe_cfg_base, 0x0006432C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064330 ) = 0x0000CC00;
    *mkptr( dfe_cfg_base, 0x00064334 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064338 ) = 0x0000CE00;
    *mkptr( dfe_cfg_base, 0x0006433C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064340 ) = 0x0000D000;
    *mkptr( dfe_cfg_base, 0x00064344 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064348 ) = 0x0000D200;
    *mkptr( dfe_cfg_base, 0x0006434C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064350 ) = 0x0000D400;
    *mkptr( dfe_cfg_base, 0x00064354 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064358 ) = 0x0000D600;
    *mkptr( dfe_cfg_base, 0x0006435C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064360 ) = 0x0000D800;
    *mkptr( dfe_cfg_base, 0x00064364 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064368 ) = 0x0000DA00;
    *mkptr( dfe_cfg_base, 0x0006436C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064370 ) = 0x0000DC00;
    *mkptr( dfe_cfg_base, 0x00064374 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064378 ) = 0x0000DE00;
    *mkptr( dfe_cfg_base, 0x0006437C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064380 ) = 0x0000E000;
    *mkptr( dfe_cfg_base, 0x00064384 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064388 ) = 0x0000E200;
    *mkptr( dfe_cfg_base, 0x0006438C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064390 ) = 0x0000E400;
    *mkptr( dfe_cfg_base, 0x00064394 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00064398 ) = 0x0000E600;
    *mkptr( dfe_cfg_base, 0x0006439C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A0 ) = 0x0000E800;
    *mkptr( dfe_cfg_base, 0x000643A4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643A8 ) = 0x0000EA00;
    *mkptr( dfe_cfg_base, 0x000643AC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B0 ) = 0x0000EC00;
    *mkptr( dfe_cfg_base, 0x000643B4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643B8 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x000643BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C0 ) = 0x0000F000;
    *mkptr( dfe_cfg_base, 0x000643C4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643C8 ) = 0x0000F200;
    *mkptr( dfe_cfg_base, 0x000643CC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D0 ) = 0x0000F400;
    *mkptr( dfe_cfg_base, 0x000643D4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643D8 ) = 0x0000F600;
    *mkptr( dfe_cfg_base, 0x000643DC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E0 ) = 0x0000F800;
    *mkptr( dfe_cfg_base, 0x000643E4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643E8 ) = 0x0000FA00;
    *mkptr( dfe_cfg_base, 0x000643EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F0 ) = 0x0000FC00;
    *mkptr( dfe_cfg_base, 0x000643F4 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x000643F8 ) = 0x0000FE00;
    *mkptr( dfe_cfg_base, 0x000643FC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x0006C000 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C004 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C008 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C00C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C010 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C020 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C024 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C028 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C02C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C030 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C040 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C044 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C048 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C04C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C050 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C060 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C064 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C068 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C06C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C070 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C080 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C084 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C088 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C08C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C090 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0A0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0A4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0A8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0AC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0B0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0C0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0C4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0C8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0D0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x0006C0E0 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x0006C0E4 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x0006C0E8 ) = 0x00000055;
    *mkptr( dfe_cfg_base, 0x0006C0EC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x0006C0F0 ) = 0x0000807F;
    *mkptr( dfe_cfg_base, 0x00040200 ) = 0x00000318;
    *mkptr( dfe_cfg_base, 0x00101300 ) = 0x00000317;
    *mkptr( dfe_cfg_base, 0x00101304 ) = 0x00000C28;
    *mkptr( dfe_cfg_base, 0x00101308 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x00101318 ) = 0x00009999;
    *mkptr( dfe_cfg_base, 0x0010131C ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00101324 ) = 0x0000003F;
    *mkptr( dfe_cfg_base, 0x00140000 ) = 0x0000EFE0;
    *mkptr( dfe_cfg_base, 0x00140004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140008 ) = 0x0000B373;
    *mkptr( dfe_cfg_base, 0x0014000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140010 ) = 0x00008D93;
    *mkptr( dfe_cfg_base, 0x00140018 ) = 0x00004BBD;
    *mkptr( dfe_cfg_base, 0x0014001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00140200 ) = 0x000018A5;
    *mkptr( dfe_cfg_base, 0x00140208 ) = 0x0000EFBE;
    *mkptr( dfe_cfg_base, 0x0014020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140210 ) = 0x0000EC71;
    *mkptr( dfe_cfg_base, 0x00140214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140218 ) = 0x00002D63;
    *mkptr( dfe_cfg_base, 0x00140400 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x00140408 ) = 0x00000E85;
    *mkptr( dfe_cfg_base, 0x00140410 ) = 0x0000F1A4;
    *mkptr( dfe_cfg_base, 0x00140414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140418 ) = 0x0000FA72;
    *mkptr( dfe_cfg_base, 0x0014041C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140600 ) = 0x0000F700;
    *mkptr( dfe_cfg_base, 0x00140604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140608 ) = 0x00000371;
    *mkptr( dfe_cfg_base, 0x00140610 ) = 0x0000081E;
    *mkptr( dfe_cfg_base, 0x00140618 ) = 0x0000F435;
    *mkptr( dfe_cfg_base, 0x0014061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140800 ) = 0x0000010C;
    *mkptr( dfe_cfg_base, 0x00140808 ) = 0x0000F9B3;
    *mkptr( dfe_cfg_base, 0x0014080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140810 ) = 0x0000046B;
    *mkptr( dfe_cfg_base, 0x00140818 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00140A00 ) = 0x00000389;
    *mkptr( dfe_cfg_base, 0x00140A08 ) = 0x0000FF71;
    *mkptr( dfe_cfg_base, 0x00140A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A10 ) = 0x0000FC0E;
    *mkptr( dfe_cfg_base, 0x00140A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140A18 ) = 0x00000448;
    *mkptr( dfe_cfg_base, 0x00140C00 ) = 0x0000FF34;
    *mkptr( dfe_cfg_base, 0x00140C04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140C08 ) = 0x0000028D;
    *mkptr( dfe_cfg_base, 0x00140C10 ) = 0x0000FEB6;
    *mkptr( dfe_cfg_base, 0x00140C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140C18 ) = 0x0000FDE9;
    *mkptr( dfe_cfg_base, 0x00140C1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140E00 ) = 0x0000FED2;
    *mkptr( dfe_cfg_base, 0x00140E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140E08 ) = 0x0000FFFA;
    *mkptr( dfe_cfg_base, 0x00140E0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00140E10 ) = 0x00000197;
    *mkptr( dfe_cfg_base, 0x00140E18 ) = 0x0000FE97;
    *mkptr( dfe_cfg_base, 0x00140E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141000 ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x00141008 ) = 0x0000FF2D;
    *mkptr( dfe_cfg_base, 0x0014100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141010 ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00141018 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x00141200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00141208 ) = 0x0000FF63;
    *mkptr( dfe_cfg_base, 0x0014120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141210 ) = 0x0000FF3E;
    *mkptr( dfe_cfg_base, 0x00141214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00141218 ) = 0x0000006E;
    *mkptr( dfe_cfg_base, 0x0010126C ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00101344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00101348 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0010134C ) = 0x00006B00;
    *mkptr( dfe_cfg_base, 0x00101350 ) = 0x00000489;
    *mkptr( dfe_cfg_base, 0x00143800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143804 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014380C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143814 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014381C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143820 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143824 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143828 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014382C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143830 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143834 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143838 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014383C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143840 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143844 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143848 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014384C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143850 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x00143854 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x00143858 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x0014385C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00143480 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x00143488 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x00143484 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x0014348C ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001434A0 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x001434A8 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001434A4 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x001434AC ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001434C0 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x001434C8 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x001434C4 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x001434CC ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x001434E0 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x001434E8 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x001434E4 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x001434EC ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x00143500 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x00143508 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x00143504 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x0014350C ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x00143520 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x00143528 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x00143524 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x0014352C ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x00101200 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00101204 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00101208 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x0010120C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00101210 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00101214 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00101218 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x0010121C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00101220 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00101224 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0010122C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00101230 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00101234 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0010123C ) = 0x000001CB;
    *mkptr( dfe_cfg_base, 0x00101240 ) = 0x00000C0B;
    *mkptr( dfe_cfg_base, 0x00101244 ) = 0x00000C0B;
    *mkptr( dfe_cfg_base, 0x00101248 ) = 0x00000C0A;
    *mkptr( dfe_cfg_base, 0x00101250 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00101254 ) = 0x00000C08;
    *mkptr( dfe_cfg_base, 0x00101258 ) = 0x00000C0C;
    *mkptr( dfe_cfg_base, 0x0010125C ) = 0x00000C01;
    *mkptr( dfe_cfg_base, 0x00101260 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00101264 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00101268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001012DC ) = 0x00006800;
    *mkptr( dfe_cfg_base, 0x00100400 ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x00181300 ) = 0x00000317;
    *mkptr( dfe_cfg_base, 0x00181304 ) = 0x00000C28;
    *mkptr( dfe_cfg_base, 0x00181308 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x00181318 ) = 0x00009999;
    *mkptr( dfe_cfg_base, 0x0018131C ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00181324 ) = 0x0000003F;
    *mkptr( dfe_cfg_base, 0x001C0000 ) = 0x0000EFE0;
    *mkptr( dfe_cfg_base, 0x001C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0008 ) = 0x0000B373;
    *mkptr( dfe_cfg_base, 0x001C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0010 ) = 0x00008D93;
    *mkptr( dfe_cfg_base, 0x001C0018 ) = 0x00004BBD;
    *mkptr( dfe_cfg_base, 0x001C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x001C0200 ) = 0x000018A5;
    *mkptr( dfe_cfg_base, 0x001C0208 ) = 0x0000EFBE;
    *mkptr( dfe_cfg_base, 0x001C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0210 ) = 0x0000EC71;
    *mkptr( dfe_cfg_base, 0x001C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0218 ) = 0x00002D63;
    *mkptr( dfe_cfg_base, 0x001C0400 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x001C0408 ) = 0x00000E85;
    *mkptr( dfe_cfg_base, 0x001C0410 ) = 0x0000F1A4;
    *mkptr( dfe_cfg_base, 0x001C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0418 ) = 0x0000FA72;
    *mkptr( dfe_cfg_base, 0x001C041C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0600 ) = 0x0000F700;
    *mkptr( dfe_cfg_base, 0x001C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0608 ) = 0x00000371;
    *mkptr( dfe_cfg_base, 0x001C0610 ) = 0x0000081E;
    *mkptr( dfe_cfg_base, 0x001C0618 ) = 0x0000F435;
    *mkptr( dfe_cfg_base, 0x001C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0800 ) = 0x0000010C;
    *mkptr( dfe_cfg_base, 0x001C0808 ) = 0x0000F9B3;
    *mkptr( dfe_cfg_base, 0x001C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0810 ) = 0x0000046B;
    *mkptr( dfe_cfg_base, 0x001C0818 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x001C0A00 ) = 0x00000389;
    *mkptr( dfe_cfg_base, 0x001C0A08 ) = 0x0000FF71;
    *mkptr( dfe_cfg_base, 0x001C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A10 ) = 0x0000FC0E;
    *mkptr( dfe_cfg_base, 0x001C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0A18 ) = 0x00000448;
    *mkptr( dfe_cfg_base, 0x001C0C00 ) = 0x0000FF34;
    *mkptr( dfe_cfg_base, 0x001C0C04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0C08 ) = 0x0000028D;
    *mkptr( dfe_cfg_base, 0x001C0C10 ) = 0x0000FEB6;
    *mkptr( dfe_cfg_base, 0x001C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0C18 ) = 0x0000FDE9;
    *mkptr( dfe_cfg_base, 0x001C0C1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0E00 ) = 0x0000FED2;
    *mkptr( dfe_cfg_base, 0x001C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0E08 ) = 0x0000FFFA;
    *mkptr( dfe_cfg_base, 0x001C0E0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C0E10 ) = 0x00000197;
    *mkptr( dfe_cfg_base, 0x001C0E18 ) = 0x0000FE97;
    *mkptr( dfe_cfg_base, 0x001C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1000 ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x001C1008 ) = 0x0000FF2D;
    *mkptr( dfe_cfg_base, 0x001C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1010 ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x001C1018 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x001C1200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x001C1208 ) = 0x0000FF63;
    *mkptr( dfe_cfg_base, 0x001C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1210 ) = 0x0000FF3E;
    *mkptr( dfe_cfg_base, 0x001C1214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C1218 ) = 0x0000006E;
    *mkptr( dfe_cfg_base, 0x0018126C ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00181344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00181348 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0018134C ) = 0x00006B00;
    *mkptr( dfe_cfg_base, 0x00181350 ) = 0x00000489;
    *mkptr( dfe_cfg_base, 0x001C3800 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3804 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3808 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C380C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3810 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3814 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3818 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C381C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3820 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3824 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3828 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C382C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3830 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3834 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3838 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C383C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3840 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3844 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3848 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C384C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3850 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C3854 ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001C3858 ) = 0x0000332F;
    *mkptr( dfe_cfg_base, 0x001C385C ) = 0x00000107;
    *mkptr( dfe_cfg_base, 0x001812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x001C3480 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x001C3488 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001C3484 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x001C348C ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001C34A0 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x001C34A8 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001C34A4 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x001C34AC ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x001C34C0 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x001C34C8 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x001C34C4 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x001C34CC ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x001C34E0 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x001C34E8 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x001C34E4 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x001C34EC ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x001C3500 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x001C3508 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x001C3504 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x001C350C ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x001C3520 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x001C3528 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x001C3524 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x001C352C ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x00181200 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00181204 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00181208 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x0018120C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00181210 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00181214 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00181218 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x0018121C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00181220 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00181224 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0018122C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00181230 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00181234 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0018123C ) = 0x000001CB;
    *mkptr( dfe_cfg_base, 0x00181240 ) = 0x00000C0B;
    *mkptr( dfe_cfg_base, 0x00181244 ) = 0x00000C0B;
    *mkptr( dfe_cfg_base, 0x00181248 ) = 0x00000C0A;
    *mkptr( dfe_cfg_base, 0x00181250 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00181254 ) = 0x00000C08;
    *mkptr( dfe_cfg_base, 0x00181258 ) = 0x00000C0C;
    *mkptr( dfe_cfg_base, 0x0018125C ) = 0x00000C01;
    *mkptr( dfe_cfg_base, 0x00181260 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00181264 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00181268 ) = 0x00002616;
    *mkptr( dfe_cfg_base, 0x001812DC ) = 0x00006800;
    *mkptr( dfe_cfg_base, 0x00180400 ) = 0x00004080;
    *mkptr( dfe_cfg_base, 0x00201300 ) = 0x00000317;
    *mkptr( dfe_cfg_base, 0x00201304 ) = 0x00000C28;
    *mkptr( dfe_cfg_base, 0x00201308 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00201318 ) = 0x00009999;
    *mkptr( dfe_cfg_base, 0x0020131C ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00201324 ) = 0x0000003F;
    *mkptr( dfe_cfg_base, 0x00240000 ) = 0x0000EFE0;
    *mkptr( dfe_cfg_base, 0x00240004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240008 ) = 0x0000B373;
    *mkptr( dfe_cfg_base, 0x0024000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240010 ) = 0x00008D93;
    *mkptr( dfe_cfg_base, 0x00240018 ) = 0x00004BBD;
    *mkptr( dfe_cfg_base, 0x0024001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00240200 ) = 0x000018A5;
    *mkptr( dfe_cfg_base, 0x00240208 ) = 0x0000EFBE;
    *mkptr( dfe_cfg_base, 0x0024020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240210 ) = 0x0000EC71;
    *mkptr( dfe_cfg_base, 0x00240214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240218 ) = 0x00002D63;
    *mkptr( dfe_cfg_base, 0x00240400 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x00240408 ) = 0x00000E85;
    *mkptr( dfe_cfg_base, 0x00240410 ) = 0x0000F1A4;
    *mkptr( dfe_cfg_base, 0x00240414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240418 ) = 0x0000FA72;
    *mkptr( dfe_cfg_base, 0x0024041C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240600 ) = 0x0000F700;
    *mkptr( dfe_cfg_base, 0x00240604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240608 ) = 0x00000371;
    *mkptr( dfe_cfg_base, 0x00240610 ) = 0x0000081E;
    *mkptr( dfe_cfg_base, 0x00240618 ) = 0x0000F435;
    *mkptr( dfe_cfg_base, 0x0024061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240800 ) = 0x0000010C;
    *mkptr( dfe_cfg_base, 0x00240808 ) = 0x0000F9B3;
    *mkptr( dfe_cfg_base, 0x0024080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240810 ) = 0x0000046B;
    *mkptr( dfe_cfg_base, 0x00240818 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00240A00 ) = 0x00000389;
    *mkptr( dfe_cfg_base, 0x00240A08 ) = 0x0000FF71;
    *mkptr( dfe_cfg_base, 0x00240A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A10 ) = 0x0000FC0E;
    *mkptr( dfe_cfg_base, 0x00240A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240A18 ) = 0x00000448;
    *mkptr( dfe_cfg_base, 0x00240C00 ) = 0x0000FF34;
    *mkptr( dfe_cfg_base, 0x00240C04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240C08 ) = 0x0000028D;
    *mkptr( dfe_cfg_base, 0x00240C10 ) = 0x0000FEB6;
    *mkptr( dfe_cfg_base, 0x00240C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240C18 ) = 0x0000FDE9;
    *mkptr( dfe_cfg_base, 0x00240C1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240E00 ) = 0x0000FED2;
    *mkptr( dfe_cfg_base, 0x00240E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240E08 ) = 0x0000FFFA;
    *mkptr( dfe_cfg_base, 0x00240E0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00240E10 ) = 0x00000197;
    *mkptr( dfe_cfg_base, 0x00240E18 ) = 0x0000FE97;
    *mkptr( dfe_cfg_base, 0x00240E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241000 ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x00241008 ) = 0x0000FF2D;
    *mkptr( dfe_cfg_base, 0x0024100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241010 ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x00241018 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x00241200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00241208 ) = 0x0000FF63;
    *mkptr( dfe_cfg_base, 0x0024120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241210 ) = 0x0000FF3E;
    *mkptr( dfe_cfg_base, 0x00241214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00241218 ) = 0x0000006E;
    *mkptr( dfe_cfg_base, 0x0020126C ) = 0x00000217;
    *mkptr( dfe_cfg_base, 0x00201344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00201348 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0020134C ) = 0x00002B00;
    *mkptr( dfe_cfg_base, 0x00201350 ) = 0x00000489;
    *mkptr( dfe_cfg_base, 0x00243800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243804 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024380C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243814 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024381C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243820 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243824 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243828 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024382C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243830 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243834 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243838 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024383C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243840 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243844 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243848 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024384C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243850 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x00243854 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x00243858 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x0024385C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002012C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00243480 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x00243488 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x00243484 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x0024348C ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x002434A0 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002434A8 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002434A4 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002434AC ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002434C0 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002434C8 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x002434C4 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002434CC ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x002434E0 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002434E8 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x002434E4 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002434EC ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x00243500 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x00243508 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x00243504 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x0024350C ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x00243520 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x00243528 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x00243524 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x0024352C ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x00243540 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x00243548 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x00243544 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x0024354C ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x00243560 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x00243568 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x00243564 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x0024356C ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x00243580 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x00243588 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x00243584 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x0024358C ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x00201200 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00201204 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00201208 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x0020120C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00201210 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00201214 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00201218 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x0020121C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00201220 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00201224 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00201228 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x0020122C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00201230 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00201234 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0020123C ) = 0x000001CB;
    *mkptr( dfe_cfg_base, 0x00201240 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00201244 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00201248 ) = 0x00000C05;
    *mkptr( dfe_cfg_base, 0x00201250 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00201254 ) = 0x00000C02;
    *mkptr( dfe_cfg_base, 0x00201258 ) = 0x00000C0C;
    *mkptr( dfe_cfg_base, 0x0020125C ) = 0x00000C03;
    *mkptr( dfe_cfg_base, 0x00201260 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00201264 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00201268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002012DC ) = 0x00007800;
    *mkptr( dfe_cfg_base, 0x002012EC ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x00281300 ) = 0x00000317;
    *mkptr( dfe_cfg_base, 0x00281304 ) = 0x00000C28;
    *mkptr( dfe_cfg_base, 0x00281308 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x00281318 ) = 0x00009999;
    *mkptr( dfe_cfg_base, 0x0028131C ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x00281324 ) = 0x0000003F;
    *mkptr( dfe_cfg_base, 0x002C0000 ) = 0x0000EFE0;
    *mkptr( dfe_cfg_base, 0x002C0004 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0008 ) = 0x0000B373;
    *mkptr( dfe_cfg_base, 0x002C000C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0010 ) = 0x00008D93;
    *mkptr( dfe_cfg_base, 0x002C0018 ) = 0x00004BBD;
    *mkptr( dfe_cfg_base, 0x002C001C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x002C0200 ) = 0x000018A5;
    *mkptr( dfe_cfg_base, 0x002C0208 ) = 0x0000EFBE;
    *mkptr( dfe_cfg_base, 0x002C020C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0210 ) = 0x0000EC71;
    *mkptr( dfe_cfg_base, 0x002C0214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0218 ) = 0x00002D63;
    *mkptr( dfe_cfg_base, 0x002C0400 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x002C0408 ) = 0x00000E85;
    *mkptr( dfe_cfg_base, 0x002C0410 ) = 0x0000F1A4;
    *mkptr( dfe_cfg_base, 0x002C0414 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0418 ) = 0x0000FA72;
    *mkptr( dfe_cfg_base, 0x002C041C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0600 ) = 0x0000F700;
    *mkptr( dfe_cfg_base, 0x002C0604 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0608 ) = 0x00000371;
    *mkptr( dfe_cfg_base, 0x002C0610 ) = 0x0000081E;
    *mkptr( dfe_cfg_base, 0x002C0618 ) = 0x0000F435;
    *mkptr( dfe_cfg_base, 0x002C061C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0800 ) = 0x0000010C;
    *mkptr( dfe_cfg_base, 0x002C0808 ) = 0x0000F9B3;
    *mkptr( dfe_cfg_base, 0x002C080C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0810 ) = 0x0000046B;
    *mkptr( dfe_cfg_base, 0x002C0818 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x002C0A00 ) = 0x00000389;
    *mkptr( dfe_cfg_base, 0x002C0A08 ) = 0x0000FF71;
    *mkptr( dfe_cfg_base, 0x002C0A0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A10 ) = 0x0000FC0E;
    *mkptr( dfe_cfg_base, 0x002C0A14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0A18 ) = 0x00000448;
    *mkptr( dfe_cfg_base, 0x002C0C00 ) = 0x0000FF34;
    *mkptr( dfe_cfg_base, 0x002C0C04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0C08 ) = 0x0000028D;
    *mkptr( dfe_cfg_base, 0x002C0C10 ) = 0x0000FEB6;
    *mkptr( dfe_cfg_base, 0x002C0C14 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0C18 ) = 0x0000FDE9;
    *mkptr( dfe_cfg_base, 0x002C0C1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0E00 ) = 0x0000FED2;
    *mkptr( dfe_cfg_base, 0x002C0E04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0E08 ) = 0x0000FFFA;
    *mkptr( dfe_cfg_base, 0x002C0E0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C0E10 ) = 0x00000197;
    *mkptr( dfe_cfg_base, 0x002C0E18 ) = 0x0000FE97;
    *mkptr( dfe_cfg_base, 0x002C0E1C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1000 ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x002C1008 ) = 0x0000FF2D;
    *mkptr( dfe_cfg_base, 0x002C100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1010 ) = 0x00000052;
    *mkptr( dfe_cfg_base, 0x002C1018 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x002C1200 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x002C1208 ) = 0x0000FF63;
    *mkptr( dfe_cfg_base, 0x002C120C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1210 ) = 0x0000FF3E;
    *mkptr( dfe_cfg_base, 0x002C1214 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C1218 ) = 0x0000006E;
    *mkptr( dfe_cfg_base, 0x0028126C ) = 0x00000217;
    *mkptr( dfe_cfg_base, 0x00281344 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x00281348 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x0028134C ) = 0x00002B00;
    *mkptr( dfe_cfg_base, 0x00281350 ) = 0x00000489;
    *mkptr( dfe_cfg_base, 0x002C3800 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3804 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3808 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C380C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3810 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3814 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3818 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C381C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3820 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3824 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3828 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C382C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3830 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3834 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3838 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C383C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3840 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3844 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3848 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C384C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3850 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C3854 ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002C3858 ) = 0x0000CCD1;
    *mkptr( dfe_cfg_base, 0x002C385C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x002812C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x002C3480 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x002C3488 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x002C3484 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x002C348C ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x002C34A0 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002C34A8 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002C34A4 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002C34AC ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002C34C0 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002C34C8 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x002C34C4 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002C34CC ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x002C34E0 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002C34E8 ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x002C34E4 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002C34EC ) = 0x00001C72;
    *mkptr( dfe_cfg_base, 0x002C3500 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002C3508 ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x002C3504 ) = 0x00005555;
    *mkptr( dfe_cfg_base, 0x002C350C ) = 0x000071C7;
    *mkptr( dfe_cfg_base, 0x002C3520 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x002C3528 ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x002C3524 ) = 0x0000D655;
    *mkptr( dfe_cfg_base, 0x002C352C ) = 0x0000F21C;
    *mkptr( dfe_cfg_base, 0x002C3540 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x002C3548 ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x002C3544 ) = 0x0000E38E;
    *mkptr( dfe_cfg_base, 0x002C354C ) = 0x0000AAAB;
    *mkptr( dfe_cfg_base, 0x002C3560 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002C3568 ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002C3564 ) = 0x00008E38;
    *mkptr( dfe_cfg_base, 0x002C356C ) = 0x0000AAAA;
    *mkptr( dfe_cfg_base, 0x002C3580 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002C3588 ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x002C3584 ) = 0x00000DE3;
    *mkptr( dfe_cfg_base, 0x002C358C ) = 0x000029AA;
    *mkptr( dfe_cfg_base, 0x00281200 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00281204 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x00281208 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x0028120C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00281210 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00281214 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00281218 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x0028121C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00281220 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00281224 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00281228 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x0028122C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00281230 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x00281234 ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x0028123C ) = 0x000001CB;
    *mkptr( dfe_cfg_base, 0x00281240 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00281244 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00281248 ) = 0x00000C05;
    *mkptr( dfe_cfg_base, 0x00281250 ) = 0x00000C06;
    *mkptr( dfe_cfg_base, 0x00281254 ) = 0x00000C02;
    *mkptr( dfe_cfg_base, 0x00281258 ) = 0x00000C0C;
    *mkptr( dfe_cfg_base, 0x0028125C ) = 0x00000C03;
    *mkptr( dfe_cfg_base, 0x00281260 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00281264 ) = 0x00000B0B;
    *mkptr( dfe_cfg_base, 0x00281268 ) = 0x00001606;
    *mkptr( dfe_cfg_base, 0x002812DC ) = 0x00007800;
    *mkptr( dfe_cfg_base, 0x002812E4 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x002812E8 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x002812EC ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x00900004 ) = 0x00000505;
    *mkptr( dfe_cfg_base, 0x00900018 ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x00900030 ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x00900034 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B003CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B00208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B00360 ) = 0x00008F9E;
    *mkptr( dfe_cfg_base, 0x00B00370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B00200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B00230 ) = 0x00004F4F;
    *mkptr( dfe_cfg_base, 0x00B00290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B00298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B002B0 ) = 0x00004F4F;
    *mkptr( dfe_cfg_base, 0x00B00300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B00310 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00318 ) = 0x00003F5C;
    *mkptr( dfe_cfg_base, 0x00B00320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B00328 ) = 0x000001B8;
    *mkptr( dfe_cfg_base, 0x00B00338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B00340 ) = 0x00000463;
    *mkptr( dfe_cfg_base, 0x00B00348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B00378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B18004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B1807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B18080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B18084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B18088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B1808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B18090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B18094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B18098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B1809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B180A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B180A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B180A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B180AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B180B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B180B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B180B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B180BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B180C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B180C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B180C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B180CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B180D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B180D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B180D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B180DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B180E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B180E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B180E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B180EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B180F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B180F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B180F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B180FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B18100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B18104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B18108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B1810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B18110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B18114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B18118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B1811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B18120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B18124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B18128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B1812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B18130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B18134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B18138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B1813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B18140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B18144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B18148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B1814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B18150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B18154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B18158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B1815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B18160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B18164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B18168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B18170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B18174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B18178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B1817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B18180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B18184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B18188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B1818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B18190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B18194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B18198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B1819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B181A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B181A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B181A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B181B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B181B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B181C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B181C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B181C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B181D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B181D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B181E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B181E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B181E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B181F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B181F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B181FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B18200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B18208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B1820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B18210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B18214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B18218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B18220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B18228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B18230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B18238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B1823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B18240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B18244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B18248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B18250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B18258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B18260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B18268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B18270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B18278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B18280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B18284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B18288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B1828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B18290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B18294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B18298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B1829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B182A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B182A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B182B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B182B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B182C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B182C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B182D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B182D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B182E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B182E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B182EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B182F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B182F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B182FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B18300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B18308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B18310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B18318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B18320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B18328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B18330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B18338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B18340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B18348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B18350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B18358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B18360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B18368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B1836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B18370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B18374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B18378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B18380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B18388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B18390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B18394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B18398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B1839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B183A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B183A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B183B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B183B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B183C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B183C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B183D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B183D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B183E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B183E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B183F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B183F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B183FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B10100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B10134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B10138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B1013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B1500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B15014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B1501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B15024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B1502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B15034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B1503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B15044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B1504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B15054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B1505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B15064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B1506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B15074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B1507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B14000 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x00B14004 ) = 0x00004130;
    *mkptr( dfe_cfg_base, 0x00B14008 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x00B1400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B14014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B1401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B14020 ) = 0x0000090D;
    *mkptr( dfe_cfg_base, 0x00B14024 ) = 0x00000505;
    *mkptr( dfe_cfg_base, 0x00B14040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B140E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B140E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B141E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B141E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B142E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B142E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B143E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B143E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B144E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B144E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B145E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B145E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B146E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B146E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B14780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B14798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B1479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B147E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B147E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B0E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B0ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B04000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04004 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B04404 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B04008 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B04408 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B0400C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B0440C ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B04010 ) = 0x00000026;
    *mkptr( dfe_cfg_base, 0x00B04410 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04014 ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B04414 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x00B04018 ) = 0x00000046;
    *mkptr( dfe_cfg_base, 0x00B04418 ) = 0x0000004E;
    *mkptr( dfe_cfg_base, 0x00B0401C ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B0441C ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x00B04020 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B04420 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04024 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B04424 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B04028 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B04428 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0402C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B0442C ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B04030 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B04430 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B04034 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B04434 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B04038 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B04438 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B0403C ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B04040 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B04440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B04044 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B04444 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B04048 ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B04448 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B0404C ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B0444C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B04050 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B04450 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B04054 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B04454 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B04058 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B04458 ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B0405C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B0445C ) = 0x00000FDD;
    *mkptr( dfe_cfg_base, 0x00B04060 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B04460 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B04064 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B04464 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B04468 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B0406C ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B0446C ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B04070 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B04470 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B04074 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B04474 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B04078 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B04478 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0407C ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B0447C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B04080 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B04480 ) = 0x00000FCB;
    *mkptr( dfe_cfg_base, 0x00B04084 ) = 0x00000FAC;
    *mkptr( dfe_cfg_base, 0x00B04484 ) = 0x00000FA8;
    *mkptr( dfe_cfg_base, 0x00B04088 ) = 0x00000FD6;
    *mkptr( dfe_cfg_base, 0x00B04488 ) = 0x00000FBE;
    *mkptr( dfe_cfg_base, 0x00B0408C ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B0448C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B04090 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B04490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B04094 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B04494 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B04098 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B04498 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B0409C ) = 0x0000001D;
    *mkptr( dfe_cfg_base, 0x00B0449C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00B040A0 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B044A0 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B040A4 ) = 0x00000F9C;
    *mkptr( dfe_cfg_base, 0x00B044A4 ) = 0x00000FC9;
    *mkptr( dfe_cfg_base, 0x00B040A8 ) = 0x00000F68;
    *mkptr( dfe_cfg_base, 0x00B044A8 ) = 0x00000F78;
    *mkptr( dfe_cfg_base, 0x00B040AC ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B044AC ) = 0x00000F70;
    *mkptr( dfe_cfg_base, 0x00B040B0 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B044B0 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B040B4 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B044B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B040B8 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B044B8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B040BC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B044BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B040C0 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B044C0 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x00B040C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B044C4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B040C8 ) = 0x00000FA0;
    *mkptr( dfe_cfg_base, 0x00B044C8 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B040CC ) = 0x00000F32;
    *mkptr( dfe_cfg_base, 0x00B044CC ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B040D0 ) = 0x00000F31;
    *mkptr( dfe_cfg_base, 0x00B044D0 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B040D4 ) = 0x00000FA9;
    *mkptr( dfe_cfg_base, 0x00B044D4 ) = 0x00000F64;
    *mkptr( dfe_cfg_base, 0x00B040D8 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B044D8 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B040DC ) = 0x0000002D;
    *mkptr( dfe_cfg_base, 0x00B044DC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x00B040E0 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B044E0 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B040E4 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B044E4 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B040E8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B040EC ) = 0x00000FC4;
    *mkptr( dfe_cfg_base, 0x00B044EC ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B040F0 ) = 0x00000F29;
    *mkptr( dfe_cfg_base, 0x00B044F0 ) = 0x00000F7B;
    *mkptr( dfe_cfg_base, 0x00B040F4 ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B044F4 ) = 0x00000EE9;
    *mkptr( dfe_cfg_base, 0x00B040F8 ) = 0x00000F42;
    *mkptr( dfe_cfg_base, 0x00B044F8 ) = 0x00000EF1;
    *mkptr( dfe_cfg_base, 0x00B040FC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B044FC ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B04100 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x00B04500 ) = 0x0000005F;
    *mkptr( dfe_cfg_base, 0x00B04104 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B04504 ) = 0x00000056;
    *mkptr( dfe_cfg_base, 0x00B04108 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B04508 ) = 0x00000FDF;
    *mkptr( dfe_cfg_base, 0x00B0410C ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B0450C ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B04110 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B04510 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B04114 ) = 0x00000F80;
    *mkptr( dfe_cfg_base, 0x00B04514 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B04118 ) = 0x00000E89;
    *mkptr( dfe_cfg_base, 0x00B04518 ) = 0x00000EFC;
    *mkptr( dfe_cfg_base, 0x00B0411C ) = 0x00000E7C;
    *mkptr( dfe_cfg_base, 0x00B0451C ) = 0x00000E52;
    *mkptr( dfe_cfg_base, 0x00B04120 ) = 0x00000FD9;
    *mkptr( dfe_cfg_base, 0x00B04520 ) = 0x00000F0B;
    *mkptr( dfe_cfg_base, 0x00B04124 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x00B04524 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x00B04128 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x00B04528 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00B0412C ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B0452C ) = 0x00000FC7;
    *mkptr( dfe_cfg_base, 0x00B04130 ) = 0x00000E55;
    *mkptr( dfe_cfg_base, 0x00B04530 ) = 0x00000E35;
    *mkptr( dfe_cfg_base, 0x00B04134 ) = 0x00000135;
    *mkptr( dfe_cfg_base, 0x00B04534 ) = 0x00000F5C;
    *mkptr( dfe_cfg_base, 0x00B04138 ) = 0x000005C5;
    *mkptr( dfe_cfg_base, 0x00B04538 ) = 0x00000385;
    *mkptr( dfe_cfg_base, 0x00B0413C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B0453C ) = 0x00000766;
    *mkptr( dfe_cfg_base, 0x00B08000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08004 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B08404 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B08008 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B08408 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B0800C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B0840C ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B08010 ) = 0x00000026;
    *mkptr( dfe_cfg_base, 0x00B08410 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08014 ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B08414 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x00B08018 ) = 0x00000046;
    *mkptr( dfe_cfg_base, 0x00B08418 ) = 0x0000004E;
    *mkptr( dfe_cfg_base, 0x00B0801C ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B0841C ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x00B08020 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B08420 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08024 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B08424 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B08028 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B08428 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B0802C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B0842C ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B08030 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B08430 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B08034 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B08434 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B08038 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B08438 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B0803C ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B08040 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B08440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B08044 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B08444 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B08048 ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B08448 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B0804C ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B0844C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B08050 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B08450 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B08054 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B08454 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B08058 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B08458 ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B0805C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B0845C ) = 0x00000FDD;
    *mkptr( dfe_cfg_base, 0x00B08060 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B08460 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B08064 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B08464 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B08468 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B0806C ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B0846C ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B08070 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B08470 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B08074 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B08474 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B08078 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B08478 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B0807C ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B0847C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B08080 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B08480 ) = 0x00000FCB;
    *mkptr( dfe_cfg_base, 0x00B08084 ) = 0x00000FAC;
    *mkptr( dfe_cfg_base, 0x00B08484 ) = 0x00000FA8;
    *mkptr( dfe_cfg_base, 0x00B08088 ) = 0x00000FD6;
    *mkptr( dfe_cfg_base, 0x00B08488 ) = 0x00000FBE;
    *mkptr( dfe_cfg_base, 0x00B0808C ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B0848C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B08090 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B08490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B08094 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B08494 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B08098 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B08498 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B0809C ) = 0x0000001D;
    *mkptr( dfe_cfg_base, 0x00B0849C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00B080A0 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B084A0 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B080A4 ) = 0x00000F9C;
    *mkptr( dfe_cfg_base, 0x00B084A4 ) = 0x00000FC9;
    *mkptr( dfe_cfg_base, 0x00B080A8 ) = 0x00000F68;
    *mkptr( dfe_cfg_base, 0x00B084A8 ) = 0x00000F78;
    *mkptr( dfe_cfg_base, 0x00B080AC ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B084AC ) = 0x00000F70;
    *mkptr( dfe_cfg_base, 0x00B080B0 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B084B0 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B080B4 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B084B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B080B8 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B084B8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B080BC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B084BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B080C0 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B084C0 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x00B080C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B084C4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B080C8 ) = 0x00000FA0;
    *mkptr( dfe_cfg_base, 0x00B084C8 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B080CC ) = 0x00000F32;
    *mkptr( dfe_cfg_base, 0x00B084CC ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B080D0 ) = 0x00000F31;
    *mkptr( dfe_cfg_base, 0x00B084D0 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B080D4 ) = 0x00000FA9;
    *mkptr( dfe_cfg_base, 0x00B084D4 ) = 0x00000F64;
    *mkptr( dfe_cfg_base, 0x00B080D8 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B084D8 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B080DC ) = 0x0000002D;
    *mkptr( dfe_cfg_base, 0x00B084DC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x00B080E0 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B084E0 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B080E4 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B084E4 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B080E8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B080EC ) = 0x00000FC4;
    *mkptr( dfe_cfg_base, 0x00B084EC ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B080F0 ) = 0x00000F29;
    *mkptr( dfe_cfg_base, 0x00B084F0 ) = 0x00000F7B;
    *mkptr( dfe_cfg_base, 0x00B080F4 ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B084F4 ) = 0x00000EE9;
    *mkptr( dfe_cfg_base, 0x00B080F8 ) = 0x00000F42;
    *mkptr( dfe_cfg_base, 0x00B084F8 ) = 0x00000EF1;
    *mkptr( dfe_cfg_base, 0x00B080FC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B084FC ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B08100 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x00B08500 ) = 0x0000005F;
    *mkptr( dfe_cfg_base, 0x00B08104 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B08504 ) = 0x00000056;
    *mkptr( dfe_cfg_base, 0x00B08108 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B08508 ) = 0x00000FDF;
    *mkptr( dfe_cfg_base, 0x00B0810C ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B0850C ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B08110 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B08510 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B08114 ) = 0x00000F80;
    *mkptr( dfe_cfg_base, 0x00B08514 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B08118 ) = 0x00000E89;
    *mkptr( dfe_cfg_base, 0x00B08518 ) = 0x00000EFC;
    *mkptr( dfe_cfg_base, 0x00B0811C ) = 0x00000E7C;
    *mkptr( dfe_cfg_base, 0x00B0851C ) = 0x00000E52;
    *mkptr( dfe_cfg_base, 0x00B08120 ) = 0x00000FD9;
    *mkptr( dfe_cfg_base, 0x00B08520 ) = 0x00000F0B;
    *mkptr( dfe_cfg_base, 0x00B08124 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x00B08524 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x00B08128 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x00B08528 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00B0812C ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B0852C ) = 0x00000FC7;
    *mkptr( dfe_cfg_base, 0x00B08130 ) = 0x00000E55;
    *mkptr( dfe_cfg_base, 0x00B08530 ) = 0x00000E35;
    *mkptr( dfe_cfg_base, 0x00B08134 ) = 0x00000135;
    *mkptr( dfe_cfg_base, 0x00B08534 ) = 0x00000F5C;
    *mkptr( dfe_cfg_base, 0x00B08138 ) = 0x000005C5;
    *mkptr( dfe_cfg_base, 0x00B08538 ) = 0x00000385;
    *mkptr( dfe_cfg_base, 0x00B0813C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B0853C ) = 0x00000766;
    *mkptr( dfe_cfg_base, 0x00B803CC ) = 0x00008006;
    *mkptr( dfe_cfg_base, 0x00B80208 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80288 ) = 0x000008C0;
    *mkptr( dfe_cfg_base, 0x00B80360 ) = 0x00008F9E;
    *mkptr( dfe_cfg_base, 0x00B80370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B80200 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80280 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80210 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80218 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B80230 ) = 0x00004F4F;
    *mkptr( dfe_cfg_base, 0x00B80290 ) = 0x00008686;
    *mkptr( dfe_cfg_base, 0x00B80298 ) = 0x00000F0F;
    *mkptr( dfe_cfg_base, 0x00B802B0 ) = 0x00004F4F;
    *mkptr( dfe_cfg_base, 0x00B80300 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80308 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B80310 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80318 ) = 0x00007F5C;
    *mkptr( dfe_cfg_base, 0x00B80320 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B80328 ) = 0x000001B8;
    *mkptr( dfe_cfg_base, 0x00B80338 ) = 0x00000BB8;
    *mkptr( dfe_cfg_base, 0x00B80340 ) = 0x00000463;
    *mkptr( dfe_cfg_base, 0x00B80348 ) = 0x000000FF;
    *mkptr( dfe_cfg_base, 0x00B80378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x00B98004 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9800C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98014 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9801C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98024 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9802C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98034 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9803C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98044 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9804C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98054 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9805C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98064 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9806C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98074 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B9807C ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x00B98080 ) = 0x0000030C;
    *mkptr( dfe_cfg_base, 0x00B98084 ) = 0x00007FE8;
    *mkptr( dfe_cfg_base, 0x00B98088 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x00B9808C ) = 0x00007C17;
    *mkptr( dfe_cfg_base, 0x00B98090 ) = 0x00000334;
    *mkptr( dfe_cfg_base, 0x00B98094 ) = 0x00007899;
    *mkptr( dfe_cfg_base, 0x00B98098 ) = 0x00000340;
    *mkptr( dfe_cfg_base, 0x00B9809C ) = 0x00007562;
    *mkptr( dfe_cfg_base, 0x00B980A0 ) = 0x00000350;
    *mkptr( dfe_cfg_base, 0x00B980A4 ) = 0x0000726A;
    *mkptr( dfe_cfg_base, 0x00B980A8 ) = 0x0000035C;
    *mkptr( dfe_cfg_base, 0x00B980AC ) = 0x00006FA8;
    *mkptr( dfe_cfg_base, 0x00B980B0 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x00B980B4 ) = 0x00006D17;
    *mkptr( dfe_cfg_base, 0x00B980B8 ) = 0x00000370;
    *mkptr( dfe_cfg_base, 0x00B980BC ) = 0x00006AB2;
    *mkptr( dfe_cfg_base, 0x00B980C0 ) = 0x00000378;
    *mkptr( dfe_cfg_base, 0x00B980C4 ) = 0x00006873;
    *mkptr( dfe_cfg_base, 0x00B980C8 ) = 0x00000380;
    *mkptr( dfe_cfg_base, 0x00B980CC ) = 0x00006657;
    *mkptr( dfe_cfg_base, 0x00B980D0 ) = 0x00000388;
    *mkptr( dfe_cfg_base, 0x00B980D4 ) = 0x0000645A;
    *mkptr( dfe_cfg_base, 0x00B980D8 ) = 0x00000390;
    *mkptr( dfe_cfg_base, 0x00B980DC ) = 0x0000627A;
    *mkptr( dfe_cfg_base, 0x00B980E0 ) = 0x00000394;
    *mkptr( dfe_cfg_base, 0x00B980E4 ) = 0x000060B4;
    *mkptr( dfe_cfg_base, 0x00B980E8 ) = 0x00000398;
    *mkptr( dfe_cfg_base, 0x00B980EC ) = 0x00005F06;
    *mkptr( dfe_cfg_base, 0x00B980F0 ) = 0x000003A0;
    *mkptr( dfe_cfg_base, 0x00B980F4 ) = 0x00005D6D;
    *mkptr( dfe_cfg_base, 0x00B980F8 ) = 0x000003A4;
    *mkptr( dfe_cfg_base, 0x00B980FC ) = 0x00005BE8;
    *mkptr( dfe_cfg_base, 0x00B98100 ) = 0x000003A8;
    *mkptr( dfe_cfg_base, 0x00B98104 ) = 0x00005A76;
    *mkptr( dfe_cfg_base, 0x00B98108 ) = 0x000003AC;
    *mkptr( dfe_cfg_base, 0x00B9810C ) = 0x00005914;
    *mkptr( dfe_cfg_base, 0x00B98110 ) = 0x000003B0;
    *mkptr( dfe_cfg_base, 0x00B98114 ) = 0x000057C2;
    *mkptr( dfe_cfg_base, 0x00B98118 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B9811C ) = 0x0000567F;
    *mkptr( dfe_cfg_base, 0x00B98120 ) = 0x000003B4;
    *mkptr( dfe_cfg_base, 0x00B98124 ) = 0x00005549;
    *mkptr( dfe_cfg_base, 0x00B98128 ) = 0x000003B8;
    *mkptr( dfe_cfg_base, 0x00B9812C ) = 0x00005420;
    *mkptr( dfe_cfg_base, 0x00B98130 ) = 0x000003BC;
    *mkptr( dfe_cfg_base, 0x00B98134 ) = 0x00005303;
    *mkptr( dfe_cfg_base, 0x00B98138 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B9813C ) = 0x000051F1;
    *mkptr( dfe_cfg_base, 0x00B98140 ) = 0x000003C0;
    *mkptr( dfe_cfg_base, 0x00B98144 ) = 0x000050E9;
    *mkptr( dfe_cfg_base, 0x00B98148 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B9814C ) = 0x00004FEB;
    *mkptr( dfe_cfg_base, 0x00B98150 ) = 0x000003C4;
    *mkptr( dfe_cfg_base, 0x00B98154 ) = 0x00004EF6;
    *mkptr( dfe_cfg_base, 0x00B98158 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B9815C ) = 0x00004E0A;
    *mkptr( dfe_cfg_base, 0x00B98160 ) = 0x000003C8;
    *mkptr( dfe_cfg_base, 0x00B98164 ) = 0x00004D25;
    *mkptr( dfe_cfg_base, 0x00B98168 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9816C ) = 0x00004C49;
    *mkptr( dfe_cfg_base, 0x00B98170 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B98174 ) = 0x00004B73;
    *mkptr( dfe_cfg_base, 0x00B98178 ) = 0x000003CC;
    *mkptr( dfe_cfg_base, 0x00B9817C ) = 0x00004AA5;
    *mkptr( dfe_cfg_base, 0x00B98180 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B98184 ) = 0x000049DD;
    *mkptr( dfe_cfg_base, 0x00B98188 ) = 0x000003D0;
    *mkptr( dfe_cfg_base, 0x00B9818C ) = 0x0000491B;
    *mkptr( dfe_cfg_base, 0x00B98190 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B98194 ) = 0x0000485F;
    *mkptr( dfe_cfg_base, 0x00B98198 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B9819C ) = 0x000047A8;
    *mkptr( dfe_cfg_base, 0x00B981A0 ) = 0x000003D4;
    *mkptr( dfe_cfg_base, 0x00B981A4 ) = 0x000046F7;
    *mkptr( dfe_cfg_base, 0x00B981A8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981AC ) = 0x0000464B;
    *mkptr( dfe_cfg_base, 0x00B981B0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981B4 ) = 0x000045A3;
    *mkptr( dfe_cfg_base, 0x00B981B8 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981BC ) = 0x00004501;
    *mkptr( dfe_cfg_base, 0x00B981C0 ) = 0x000003D8;
    *mkptr( dfe_cfg_base, 0x00B981C4 ) = 0x00004462;
    *mkptr( dfe_cfg_base, 0x00B981C8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981CC ) = 0x000043C8;
    *mkptr( dfe_cfg_base, 0x00B981D0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981D4 ) = 0x00004332;
    *mkptr( dfe_cfg_base, 0x00B981D8 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981DC ) = 0x0000429F;
    *mkptr( dfe_cfg_base, 0x00B981E0 ) = 0x000003DC;
    *mkptr( dfe_cfg_base, 0x00B981E4 ) = 0x00004211;
    *mkptr( dfe_cfg_base, 0x00B981E8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981EC ) = 0x00004185;
    *mkptr( dfe_cfg_base, 0x00B981F0 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981F4 ) = 0x000040FE;
    *mkptr( dfe_cfg_base, 0x00B981F8 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B981FC ) = 0x00004079;
    *mkptr( dfe_cfg_base, 0x00B98200 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98204 ) = 0x00003FF8;
    *mkptr( dfe_cfg_base, 0x00B98208 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B9820C ) = 0x00003F79;
    *mkptr( dfe_cfg_base, 0x00B98210 ) = 0x000003E0;
    *mkptr( dfe_cfg_base, 0x00B98214 ) = 0x00003EFE;
    *mkptr( dfe_cfg_base, 0x00B98218 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9821C ) = 0x00003E85;
    *mkptr( dfe_cfg_base, 0x00B98220 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98224 ) = 0x00003E0F;
    *mkptr( dfe_cfg_base, 0x00B98228 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9822C ) = 0x00003D9B;
    *mkptr( dfe_cfg_base, 0x00B98230 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98234 ) = 0x00003D2A;
    *mkptr( dfe_cfg_base, 0x00B98238 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B9823C ) = 0x00003CBB;
    *mkptr( dfe_cfg_base, 0x00B98240 ) = 0x000003E4;
    *mkptr( dfe_cfg_base, 0x00B98244 ) = 0x00003C4F;
    *mkptr( dfe_cfg_base, 0x00B98248 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9824C ) = 0x00003BE5;
    *mkptr( dfe_cfg_base, 0x00B98250 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98254 ) = 0x00003B7D;
    *mkptr( dfe_cfg_base, 0x00B98258 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9825C ) = 0x00003B17;
    *mkptr( dfe_cfg_base, 0x00B98260 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98264 ) = 0x00003AB3;
    *mkptr( dfe_cfg_base, 0x00B98268 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9826C ) = 0x00003A51;
    *mkptr( dfe_cfg_base, 0x00B98270 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98274 ) = 0x000039F1;
    *mkptr( dfe_cfg_base, 0x00B98278 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9827C ) = 0x00003993;
    *mkptr( dfe_cfg_base, 0x00B98280 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B98284 ) = 0x00003937;
    *mkptr( dfe_cfg_base, 0x00B98288 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x00B9828C ) = 0x000038DC;
    *mkptr( dfe_cfg_base, 0x00B98290 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B98294 ) = 0x00003883;
    *mkptr( dfe_cfg_base, 0x00B98298 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B9829C ) = 0x0000382C;
    *mkptr( dfe_cfg_base, 0x00B982A0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982A4 ) = 0x000037D6;
    *mkptr( dfe_cfg_base, 0x00B982A8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982AC ) = 0x00003782;
    *mkptr( dfe_cfg_base, 0x00B982B0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982B4 ) = 0x0000372F;
    *mkptr( dfe_cfg_base, 0x00B982B8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982BC ) = 0x000036DD;
    *mkptr( dfe_cfg_base, 0x00B982C0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982C4 ) = 0x0000368D;
    *mkptr( dfe_cfg_base, 0x00B982C8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982CC ) = 0x0000363F;
    *mkptr( dfe_cfg_base, 0x00B982D0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982D4 ) = 0x000035F1;
    *mkptr( dfe_cfg_base, 0x00B982D8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982DC ) = 0x000035A5;
    *mkptr( dfe_cfg_base, 0x00B982E0 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982E4 ) = 0x0000355A;
    *mkptr( dfe_cfg_base, 0x00B982E8 ) = 0x000003EC;
    *mkptr( dfe_cfg_base, 0x00B982EC ) = 0x00003511;
    *mkptr( dfe_cfg_base, 0x00B982F0 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982F4 ) = 0x000034C8;
    *mkptr( dfe_cfg_base, 0x00B982F8 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B982FC ) = 0x00003481;
    *mkptr( dfe_cfg_base, 0x00B98300 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98304 ) = 0x0000343B;
    *mkptr( dfe_cfg_base, 0x00B98308 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9830C ) = 0x000033F6;
    *mkptr( dfe_cfg_base, 0x00B98310 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98314 ) = 0x000033B2;
    *mkptr( dfe_cfg_base, 0x00B98318 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9831C ) = 0x0000336F;
    *mkptr( dfe_cfg_base, 0x00B98320 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98324 ) = 0x0000332D;
    *mkptr( dfe_cfg_base, 0x00B98328 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9832C ) = 0x000032EC;
    *mkptr( dfe_cfg_base, 0x00B98330 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98334 ) = 0x000032AC;
    *mkptr( dfe_cfg_base, 0x00B98338 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9833C ) = 0x0000326C;
    *mkptr( dfe_cfg_base, 0x00B98340 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98344 ) = 0x0000322E;
    *mkptr( dfe_cfg_base, 0x00B98348 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9834C ) = 0x000031F1;
    *mkptr( dfe_cfg_base, 0x00B98350 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98354 ) = 0x000031B4;
    *mkptr( dfe_cfg_base, 0x00B98358 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9835C ) = 0x00003179;
    *mkptr( dfe_cfg_base, 0x00B98360 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98364 ) = 0x0000313E;
    *mkptr( dfe_cfg_base, 0x00B98368 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B9836C ) = 0x00003104;
    *mkptr( dfe_cfg_base, 0x00B98370 ) = 0x000003F0;
    *mkptr( dfe_cfg_base, 0x00B98374 ) = 0x000030CB;
    *mkptr( dfe_cfg_base, 0x00B98378 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9837C ) = 0x00003093;
    *mkptr( dfe_cfg_base, 0x00B98380 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98384 ) = 0x0000305B;
    *mkptr( dfe_cfg_base, 0x00B98388 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9838C ) = 0x00003024;
    *mkptr( dfe_cfg_base, 0x00B98390 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B98394 ) = 0x00002FEE;
    *mkptr( dfe_cfg_base, 0x00B98398 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B9839C ) = 0x00002FB8;
    *mkptr( dfe_cfg_base, 0x00B983A0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983A4 ) = 0x00002F84;
    *mkptr( dfe_cfg_base, 0x00B983A8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983AC ) = 0x00002F50;
    *mkptr( dfe_cfg_base, 0x00B983B0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983B4 ) = 0x00002F1C;
    *mkptr( dfe_cfg_base, 0x00B983B8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983BC ) = 0x00002EE9;
    *mkptr( dfe_cfg_base, 0x00B983C0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983C4 ) = 0x00002EB7;
    *mkptr( dfe_cfg_base, 0x00B983C8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983CC ) = 0x00002E86;
    *mkptr( dfe_cfg_base, 0x00B983D0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983D4 ) = 0x00002E55;
    *mkptr( dfe_cfg_base, 0x00B983D8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983DC ) = 0x00002E24;
    *mkptr( dfe_cfg_base, 0x00B983E0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983E4 ) = 0x00002DF5;
    *mkptr( dfe_cfg_base, 0x00B983E8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983EC ) = 0x00002DC6;
    *mkptr( dfe_cfg_base, 0x00B983F0 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983F4 ) = 0x00002D97;
    *mkptr( dfe_cfg_base, 0x00B983F8 ) = 0x000003F4;
    *mkptr( dfe_cfg_base, 0x00B983FC ) = 0x00002D69;
    *mkptr( dfe_cfg_base, 0x00B90100 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90108 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9010C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90110 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90114 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90118 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9011C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90120 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90124 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90128 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9012C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90130 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B90134 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B90138 ) = 0x00010005;
    *mkptr( dfe_cfg_base, 0x00B9013C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B9500C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x00B95014 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x00B9501C ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x00B95024 ) = 0x00000400;
    *mkptr( dfe_cfg_base, 0x00B9502C ) = 0x00000500;
    *mkptr( dfe_cfg_base, 0x00B95034 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x00B9503C ) = 0x00000700;
    *mkptr( dfe_cfg_base, 0x00B95044 ) = 0x00000800;
    *mkptr( dfe_cfg_base, 0x00B9504C ) = 0x00000900;
    *mkptr( dfe_cfg_base, 0x00B95054 ) = 0x00000A00;
    *mkptr( dfe_cfg_base, 0x00B9505C ) = 0x00000B00;
    *mkptr( dfe_cfg_base, 0x00B95064 ) = 0x00000C00;
    *mkptr( dfe_cfg_base, 0x00B9506C ) = 0x00000D00;
    *mkptr( dfe_cfg_base, 0x00B95074 ) = 0x00000E00;
    *mkptr( dfe_cfg_base, 0x00B9507C ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x00B94000 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x00B94004 ) = 0x00004130;
    *mkptr( dfe_cfg_base, 0x00B94008 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x00B9400C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94010 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B94014 ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94018 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x00B9401C ) = 0x000038C7;
    *mkptr( dfe_cfg_base, 0x00B94020 ) = 0x0000090D;
    *mkptr( dfe_cfg_base, 0x00B94024 ) = 0x00000505;
    *mkptr( dfe_cfg_base, 0x00B94040 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94044 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94048 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9404C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94050 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94054 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94058 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9405C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94060 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94064 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94080 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94084 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94088 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9408C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94090 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94094 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94098 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9409C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B940E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B940E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94100 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94104 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94108 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9410C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94110 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94114 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94118 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9411C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94120 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94124 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94140 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94144 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94148 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9414C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94150 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94154 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94158 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9415C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94160 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94164 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94180 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94184 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94188 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9418C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94190 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94194 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94198 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9419C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B941E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B941E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94200 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94204 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94208 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9420C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94210 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94214 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94218 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9421C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94220 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94224 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94240 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94244 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94248 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9424C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94250 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94254 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94258 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9425C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94260 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94264 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94280 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94284 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94288 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9428C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94290 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94294 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94298 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9429C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B942E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B942E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94300 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94304 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94308 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9430C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94310 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94314 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94318 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9431C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94320 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94324 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94340 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94344 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94348 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9434C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94350 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94354 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94358 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9435C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94360 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94364 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94380 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94384 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94388 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9438C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94390 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94394 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94398 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9439C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B943E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B943E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94400 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94404 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94408 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9440C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94410 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94414 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94418 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9441C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94420 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94424 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94440 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94444 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94448 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9444C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94450 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94454 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94458 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9445C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94460 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94464 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94480 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94484 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94488 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9448C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94490 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94494 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94498 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9449C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B944E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B944E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94500 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94504 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94508 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9450C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94510 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94514 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94518 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9451C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94520 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94524 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94540 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94544 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94548 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9454C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94550 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94554 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94558 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9455C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94560 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94564 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94580 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94584 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94588 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9458C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94590 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94594 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94598 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9459C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B945E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B945E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94600 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94604 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94608 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9460C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94610 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94614 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94618 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9461C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94620 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94624 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94640 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94644 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9464C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94650 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94654 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94658 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9465C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94660 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94664 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94680 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94684 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94688 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9468C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94690 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94694 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94698 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9469C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B946E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B946E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94700 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94704 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94708 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9470C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94710 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94714 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94718 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9471C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94720 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94724 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94740 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94744 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94748 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9474C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94750 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94754 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94758 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9475C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94760 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94764 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B94780 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94784 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94788 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9478C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94790 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94794 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B94798 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B9479C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947A0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947A4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947C0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947C8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947CC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947D8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947DC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x00B947E0 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B947E4 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x00B8E800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A800 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A804 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A808 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A80C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A810 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A814 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A818 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A81C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A820 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A824 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A828 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A82C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A830 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A834 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A838 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A83C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A840 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A844 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A848 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A84C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A850 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A854 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A858 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A85C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A860 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A864 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A868 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A86C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A870 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A874 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A878 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A87C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A880 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A884 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A888 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A88C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A890 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A894 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A898 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A89C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A8FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A900 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A904 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A908 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A90C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A910 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A914 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A918 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A91C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A920 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A924 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A928 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A92C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A930 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A934 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A938 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A93C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A940 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A944 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A948 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A94C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A950 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A954 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A958 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A95C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A960 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A964 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A968 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A96C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A970 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A974 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A978 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A97C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A980 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A984 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A988 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A98C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A990 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A994 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A998 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A99C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9A8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9AC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9B8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9BC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9C8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9CC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9D8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9DC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9E8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9EC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9F8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8E9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8A9FC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AA9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AABC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AACC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AADC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AAFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB00 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB04 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB08 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB0C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB10 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB14 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB18 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB1C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB20 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB24 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB28 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB2C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB30 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB34 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB38 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB3C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB40 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB44 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB48 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB4C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB50 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB54 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB58 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB5C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB60 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB64 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB68 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB6C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB70 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB74 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB78 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB7C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB80 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB84 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB88 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB8C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB90 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB94 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB98 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8AB9C ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABA8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABAC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABB8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABBC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABC8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABCC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABD8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABDC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABE8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABEC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF0 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF4 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABF8 ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8EBFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B8ABFC ) = 0x00000BAD;
    *mkptr( dfe_cfg_base, 0x00B84000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84004 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B84404 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B84008 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B84408 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B8400C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B8440C ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B84010 ) = 0x00000026;
    *mkptr( dfe_cfg_base, 0x00B84410 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84014 ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B84414 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x00B84018 ) = 0x00000046;
    *mkptr( dfe_cfg_base, 0x00B84418 ) = 0x0000004E;
    *mkptr( dfe_cfg_base, 0x00B8401C ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B8441C ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x00B84020 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B84420 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84024 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B84424 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B84028 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B84428 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8402C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B8442C ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B84030 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B84430 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B84034 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B84434 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B84038 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B84438 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B8403C ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B84040 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B84440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B84044 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B84444 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B84048 ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B84448 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B8404C ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B8444C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B84050 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B84450 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B84054 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B84454 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B84058 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B84458 ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B8405C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B8445C ) = 0x00000FDD;
    *mkptr( dfe_cfg_base, 0x00B84060 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B84460 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B84064 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B84464 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B84468 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B8406C ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B8446C ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B84070 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B84470 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B84074 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B84474 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B84078 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B84478 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8407C ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B8447C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B84080 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B84480 ) = 0x00000FCB;
    *mkptr( dfe_cfg_base, 0x00B84084 ) = 0x00000FAC;
    *mkptr( dfe_cfg_base, 0x00B84484 ) = 0x00000FA8;
    *mkptr( dfe_cfg_base, 0x00B84088 ) = 0x00000FD6;
    *mkptr( dfe_cfg_base, 0x00B84488 ) = 0x00000FBE;
    *mkptr( dfe_cfg_base, 0x00B8408C ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B8448C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B84090 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B84490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B84094 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B84494 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B84098 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B84498 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B8409C ) = 0x0000001D;
    *mkptr( dfe_cfg_base, 0x00B8449C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00B840A0 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B844A0 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B840A4 ) = 0x00000F9C;
    *mkptr( dfe_cfg_base, 0x00B844A4 ) = 0x00000FC9;
    *mkptr( dfe_cfg_base, 0x00B840A8 ) = 0x00000F68;
    *mkptr( dfe_cfg_base, 0x00B844A8 ) = 0x00000F78;
    *mkptr( dfe_cfg_base, 0x00B840AC ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B844AC ) = 0x00000F70;
    *mkptr( dfe_cfg_base, 0x00B840B0 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B844B0 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B840B4 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B844B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B840B8 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B844B8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B840BC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B844BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B840C0 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B844C0 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x00B840C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B844C4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B840C8 ) = 0x00000FA0;
    *mkptr( dfe_cfg_base, 0x00B844C8 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B840CC ) = 0x00000F32;
    *mkptr( dfe_cfg_base, 0x00B844CC ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B840D0 ) = 0x00000F31;
    *mkptr( dfe_cfg_base, 0x00B844D0 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B840D4 ) = 0x00000FA9;
    *mkptr( dfe_cfg_base, 0x00B844D4 ) = 0x00000F64;
    *mkptr( dfe_cfg_base, 0x00B840D8 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B844D8 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B840DC ) = 0x0000002D;
    *mkptr( dfe_cfg_base, 0x00B844DC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x00B840E0 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B844E0 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B840E4 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B844E4 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B840E8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B840EC ) = 0x00000FC4;
    *mkptr( dfe_cfg_base, 0x00B844EC ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B840F0 ) = 0x00000F29;
    *mkptr( dfe_cfg_base, 0x00B844F0 ) = 0x00000F7B;
    *mkptr( dfe_cfg_base, 0x00B840F4 ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B844F4 ) = 0x00000EE9;
    *mkptr( dfe_cfg_base, 0x00B840F8 ) = 0x00000F42;
    *mkptr( dfe_cfg_base, 0x00B844F8 ) = 0x00000EF1;
    *mkptr( dfe_cfg_base, 0x00B840FC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B844FC ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B84100 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x00B84500 ) = 0x0000005F;
    *mkptr( dfe_cfg_base, 0x00B84104 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B84504 ) = 0x00000056;
    *mkptr( dfe_cfg_base, 0x00B84108 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B84508 ) = 0x00000FDF;
    *mkptr( dfe_cfg_base, 0x00B8410C ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B8450C ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B84110 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B84510 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B84114 ) = 0x00000F80;
    *mkptr( dfe_cfg_base, 0x00B84514 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B84118 ) = 0x00000E89;
    *mkptr( dfe_cfg_base, 0x00B84518 ) = 0x00000EFC;
    *mkptr( dfe_cfg_base, 0x00B8411C ) = 0x00000E7C;
    *mkptr( dfe_cfg_base, 0x00B8451C ) = 0x00000E52;
    *mkptr( dfe_cfg_base, 0x00B84120 ) = 0x00000FD9;
    *mkptr( dfe_cfg_base, 0x00B84520 ) = 0x00000F0B;
    *mkptr( dfe_cfg_base, 0x00B84124 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x00B84524 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x00B84128 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x00B84528 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00B8412C ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B8452C ) = 0x00000FC7;
    *mkptr( dfe_cfg_base, 0x00B84130 ) = 0x00000E55;
    *mkptr( dfe_cfg_base, 0x00B84530 ) = 0x00000E35;
    *mkptr( dfe_cfg_base, 0x00B84134 ) = 0x00000135;
    *mkptr( dfe_cfg_base, 0x00B84534 ) = 0x00000F5C;
    *mkptr( dfe_cfg_base, 0x00B84138 ) = 0x000005C5;
    *mkptr( dfe_cfg_base, 0x00B84538 ) = 0x00000385;
    *mkptr( dfe_cfg_base, 0x00B8413C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B8453C ) = 0x00000766;
    *mkptr( dfe_cfg_base, 0x00B88000 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88004 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B88404 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B88008 ) = 0x00000FF7;
    *mkptr( dfe_cfg_base, 0x00B88408 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B8800C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B8840C ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B88010 ) = 0x00000026;
    *mkptr( dfe_cfg_base, 0x00B88410 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88014 ) = 0x0000004A;
    *mkptr( dfe_cfg_base, 0x00B88414 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x00B88018 ) = 0x00000046;
    *mkptr( dfe_cfg_base, 0x00B88418 ) = 0x0000004E;
    *mkptr( dfe_cfg_base, 0x00B8801C ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x00B8841C ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x00B88020 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B88420 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88024 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B88424 ) = 0x00000FE9;
    *mkptr( dfe_cfg_base, 0x00B88028 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B88428 ) = 0x00000FF5;
    *mkptr( dfe_cfg_base, 0x00B8802C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B8842C ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B88030 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x00B88430 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x00B88034 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B88434 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B88038 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B88438 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B8803C ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B88040 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x00B88440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B88044 ) = 0x00000FFB;
    *mkptr( dfe_cfg_base, 0x00B88444 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B88048 ) = 0x00000FEF;
    *mkptr( dfe_cfg_base, 0x00B88448 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B8804C ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B8844C ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B88050 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B88450 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x00B88054 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B88454 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B88058 ) = 0x00000FEB;
    *mkptr( dfe_cfg_base, 0x00B88458 ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B8805C ) = 0x00000FD7;
    *mkptr( dfe_cfg_base, 0x00B8845C ) = 0x00000FDD;
    *mkptr( dfe_cfg_base, 0x00B88060 ) = 0x00000FE5;
    *mkptr( dfe_cfg_base, 0x00B88460 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B88064 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B88464 ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B88468 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B8806C ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B8846C ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B88070 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B88470 ) = 0x00000FF2;
    *mkptr( dfe_cfg_base, 0x00B88074 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x00B88474 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B88078 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B88478 ) = 0x0000001A;
    *mkptr( dfe_cfg_base, 0x00B8807C ) = 0x00000FEA;
    *mkptr( dfe_cfg_base, 0x00B8847C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B88080 ) = 0x00000FB3;
    *mkptr( dfe_cfg_base, 0x00B88480 ) = 0x00000FCB;
    *mkptr( dfe_cfg_base, 0x00B88084 ) = 0x00000FAC;
    *mkptr( dfe_cfg_base, 0x00B88484 ) = 0x00000FA8;
    *mkptr( dfe_cfg_base, 0x00B88088 ) = 0x00000FD6;
    *mkptr( dfe_cfg_base, 0x00B88488 ) = 0x00000FBE;
    *mkptr( dfe_cfg_base, 0x00B8808C ) = 0x00000FFC;
    *mkptr( dfe_cfg_base, 0x00B8848C ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B88090 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B88490 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B88094 ) = 0x00000FF9;
    *mkptr( dfe_cfg_base, 0x00B88494 ) = 0x00000FFA;
    *mkptr( dfe_cfg_base, 0x00B88098 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B88498 ) = 0x00000FFF;
    *mkptr( dfe_cfg_base, 0x00B8809C ) = 0x0000001D;
    *mkptr( dfe_cfg_base, 0x00B8849C ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x00B880A0 ) = 0x00000FF4;
    *mkptr( dfe_cfg_base, 0x00B884A0 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B880A4 ) = 0x00000F9C;
    *mkptr( dfe_cfg_base, 0x00B884A4 ) = 0x00000FC9;
    *mkptr( dfe_cfg_base, 0x00B880A8 ) = 0x00000F68;
    *mkptr( dfe_cfg_base, 0x00B884A8 ) = 0x00000F78;
    *mkptr( dfe_cfg_base, 0x00B880AC ) = 0x00000F8E;
    *mkptr( dfe_cfg_base, 0x00B884AC ) = 0x00000F70;
    *mkptr( dfe_cfg_base, 0x00B880B0 ) = 0x00000FE4;
    *mkptr( dfe_cfg_base, 0x00B884B0 ) = 0x00000FB9;
    *mkptr( dfe_cfg_base, 0x00B880B4 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x00B884B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B880B8 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x00B884B8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x00B880BC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x00B884BC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x00B880C0 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B884C0 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x00B880C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x00B884C4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x00B880C8 ) = 0x00000FA0;
    *mkptr( dfe_cfg_base, 0x00B884C8 ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B880CC ) = 0x00000F32;
    *mkptr( dfe_cfg_base, 0x00B884CC ) = 0x00000F62;
    *mkptr( dfe_cfg_base, 0x00B880D0 ) = 0x00000F31;
    *mkptr( dfe_cfg_base, 0x00B884D0 ) = 0x00000F20;
    *mkptr( dfe_cfg_base, 0x00B880D4 ) = 0x00000FA9;
    *mkptr( dfe_cfg_base, 0x00B884D4 ) = 0x00000F64;
    *mkptr( dfe_cfg_base, 0x00B880D8 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x00B884D8 ) = 0x00000FED;
    *mkptr( dfe_cfg_base, 0x00B880DC ) = 0x0000002D;
    *mkptr( dfe_cfg_base, 0x00B884DC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x00B880E0 ) = 0x00000FFE;
    *mkptr( dfe_cfg_base, 0x00B884E0 ) = 0x00000015;
    *mkptr( dfe_cfg_base, 0x00B880E4 ) = 0x00000FF6;
    *mkptr( dfe_cfg_base, 0x00B884E4 ) = 0x00000FF1;
    *mkptr( dfe_cfg_base, 0x00B880E8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x00B880EC ) = 0x00000FC4;
    *mkptr( dfe_cfg_base, 0x00B884EC ) = 0x00000FF3;
    *mkptr( dfe_cfg_base, 0x00B880F0 ) = 0x00000F29;
    *mkptr( dfe_cfg_base, 0x00B884F0 ) = 0x00000F7B;
    *mkptr( dfe_cfg_base, 0x00B880F4 ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B884F4 ) = 0x00000EE9;
    *mkptr( dfe_cfg_base, 0x00B880F8 ) = 0x00000F42;
    *mkptr( dfe_cfg_base, 0x00B884F8 ) = 0x00000EF1;
    *mkptr( dfe_cfg_base, 0x00B880FC ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x00B884FC ) = 0x00000FAE;
    *mkptr( dfe_cfg_base, 0x00B88100 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x00B88500 ) = 0x0000005F;
    *mkptr( dfe_cfg_base, 0x00B88104 ) = 0x00000017;
    *mkptr( dfe_cfg_base, 0x00B88504 ) = 0x00000056;
    *mkptr( dfe_cfg_base, 0x00B88108 ) = 0x00000FB5;
    *mkptr( dfe_cfg_base, 0x00B88508 ) = 0x00000FDF;
    *mkptr( dfe_cfg_base, 0x00B8810C ) = 0x00000FDA;
    *mkptr( dfe_cfg_base, 0x00B8850C ) = 0x00000FBF;
    *mkptr( dfe_cfg_base, 0x00B88110 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x00B88510 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x00B88114 ) = 0x00000F80;
    *mkptr( dfe_cfg_base, 0x00B88514 ) = 0x00000FE2;
    *mkptr( dfe_cfg_base, 0x00B88118 ) = 0x00000E89;
    *mkptr( dfe_cfg_base, 0x00B88518 ) = 0x00000EFC;
    *mkptr( dfe_cfg_base, 0x00B8811C ) = 0x00000E7C;
    *mkptr( dfe_cfg_base, 0x00B8851C ) = 0x00000E52;
    *mkptr( dfe_cfg_base, 0x00B88120 ) = 0x00000FD9;
    *mkptr( dfe_cfg_base, 0x00B88520 ) = 0x00000F0B;
    *mkptr( dfe_cfg_base, 0x00B88124 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x00B88524 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x00B88128 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x00B88528 ) = 0x00000137;
    *mkptr( dfe_cfg_base, 0x00B8812C ) = 0x00000ED1;
    *mkptr( dfe_cfg_base, 0x00B8852C ) = 0x00000FC7;
    *mkptr( dfe_cfg_base, 0x00B88130 ) = 0x00000E55;
    *mkptr( dfe_cfg_base, 0x00B88530 ) = 0x00000E35;
    *mkptr( dfe_cfg_base, 0x00B88134 ) = 0x00000135;
    *mkptr( dfe_cfg_base, 0x00B88534 ) = 0x00000F5C;
    *mkptr( dfe_cfg_base, 0x00B88138 ) = 0x000005C5;
    *mkptr( dfe_cfg_base, 0x00B88538 ) = 0x00000385;
    *mkptr( dfe_cfg_base, 0x00B8813C ) = 0x000007FF;
    *mkptr( dfe_cfg_base, 0x00B8853C ) = 0x00000766;
    *mkptr( dfe_cfg_base, 0x01000008 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x0100000C ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000010 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000014 ) = 0x40000001;
    *mkptr( dfe_cfg_base, 0x01000038 ) = 0x000C0003;
    *mkptr( dfe_cfg_base, 0x0100003C ) = 0x0000048A;
    *mkptr( dfe_cfg_base, 0x01000040 ) = 0x005300AA;
    *mkptr( dfe_cfg_base, 0x01000200 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01000404 ) = 0x0001FFFF;
    *mkptr( dfe_cfg_base, 0x01001100 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01001104 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01001108 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x0100110C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001110 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001114 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01001118 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01300200 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300204 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01300208 ) = 0x00000081;
    *mkptr( dfe_cfg_base, 0x0130020C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01300210 ) = 0x07070000;
    *mkptr( dfe_cfg_base, 0x01300214 ) = 0x0000F5F4;
    *mkptr( dfe_cfg_base, 0x01300228 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01300400 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300408 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130040C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300410 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300414 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300418 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130041C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300420 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300428 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130042C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300430 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300434 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300438 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130043C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300440 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300448 ) = 0x15000088;
    *mkptr( dfe_cfg_base, 0x0130044C ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300450 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300454 ) = 0x15003000;
    *mkptr( dfe_cfg_base, 0x01300458 ) = 0x1500018E;
    *mkptr( dfe_cfg_base, 0x0130045C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300460 ) = 0x00003300;
    *mkptr( dfe_cfg_base, 0x01300468 ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x0130046C ) = 0x1500210E;
    *mkptr( dfe_cfg_base, 0x01300470 ) = 0x150002E8;
    *mkptr( dfe_cfg_base, 0x01300474 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01300478 ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x0130047C ) = 0x150012E0;
    *mkptr( dfe_cfg_base, 0x01343000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013430F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013431F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013432F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013433F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013434F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013435F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013436F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01343798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013437F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355008 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355010 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355018 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355020 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355028 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355030 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355038 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355048 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355050 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355058 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355060 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355068 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355070 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355078 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355088 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355090 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355098 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013550F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355108 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355110 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355118 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355120 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355128 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355130 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355138 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355148 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355150 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355158 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355160 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355168 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355170 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355178 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355188 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355190 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355198 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013551F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355208 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355210 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355218 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355220 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355228 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355230 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355238 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355248 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355250 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355258 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355260 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355268 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355270 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355278 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355280 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355288 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355290 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355298 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013552F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355300 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355308 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355310 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355318 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355320 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355328 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355330 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355338 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355340 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355348 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355350 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355358 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355360 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355368 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355370 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355378 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355380 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355388 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355390 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355398 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013553F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355400 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355408 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355410 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355418 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355420 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355428 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355430 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355438 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355440 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355448 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355450 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355458 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355460 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355468 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355470 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355478 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355480 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355488 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355490 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355498 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013554F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355500 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355508 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355510 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355518 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355520 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355528 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355530 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355538 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355540 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355548 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355550 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355558 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355560 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355568 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355570 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355578 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355580 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355588 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355590 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355598 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013555F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355600 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355608 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355610 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355618 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355620 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355628 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355630 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355638 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355640 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355648 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355650 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355658 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355660 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355668 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355670 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355678 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355680 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355688 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355690 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355698 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013556F8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355700 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355708 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355710 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355718 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355720 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355728 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355730 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355738 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355740 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355748 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355750 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355758 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355760 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355768 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355770 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355778 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355780 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355788 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355790 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01355798 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557A8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557B8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557C8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557D8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557E8 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x013557F8 ) = 0x00002000;
#endif
    iqn2_jesd_loopback_config_split_1();
    iqn2_jesd_loopback_config_split_2();

}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_jesd_loopback_config_split_1,".text:dfe");
#endif
static void iqn2_jesd_loopback_config_split_1()
{
#if DFE_RATE == 20
    *mkptr( dfe_cfg_base, 0x01600014 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600018 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160001C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600020 ) = 0x09090001;
    *mkptr( dfe_cfg_base, 0x0160002C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600054 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600058 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160005C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600060 ) = 0x09090003;
    *mkptr( dfe_cfg_base, 0x0160006C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600080 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600084 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600088 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160008C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600090 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600094 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600098 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160009C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x016000A0 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x016000A4 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x016000A8 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x016000AC ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x016000B0 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x016000B4 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000B8 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000C0 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x016000C4 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x016000C8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x016000CC ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x016000D0 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x016000D4 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x016000D8 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x016000DC ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x016000E0 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x016000E4 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x016000E8 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x016000EC ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x016000F0 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x016000F4 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000F8 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600100 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600104 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600108 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160010C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600110 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600114 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600118 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160011C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x01600120 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x01600124 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x01600128 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x0160012C ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x01600130 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x01600134 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600138 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600140 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x01600144 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x01600148 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x0160014C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01600150 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x01600154 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x01600158 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x0160015C ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x01600160 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x01600164 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x01600168 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x0160016C ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x01600170 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x01600174 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600178 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000BC ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x0160013C ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x01600200 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600204 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600208 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160020C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600210 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600214 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600218 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160021C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600220 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600224 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600228 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160022C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600230 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600234 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600238 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160023C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600600 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600604 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600608 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160060C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600610 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600614 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600618 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160061C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600620 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600624 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600628 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160062C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600630 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600634 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600638 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160063C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600B08 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B10 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B18 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B24 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B28 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B30 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600A00 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A04 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A08 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A0C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A10 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A14 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A18 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A1C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A28 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A2C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A30 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A34 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A80 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A84 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A88 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A8C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A90 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A94 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A98 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A9C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600AA8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AAC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01640000 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640004 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640008 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164000C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640010 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640014 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640018 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164001C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640020 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640024 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640028 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164002C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640030 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640034 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640038 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164003C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640040 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640044 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640048 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164004C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640050 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640054 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640058 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164005C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640060 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640064 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640068 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164006C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640070 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640074 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640078 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164007C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640080 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640084 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640088 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164008C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640090 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640094 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640098 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164009C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640100 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640104 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640108 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164010C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640110 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640114 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640118 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164011C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640120 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640124 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640128 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164012C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640130 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640134 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640138 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164013C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640140 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640144 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640148 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164014C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640150 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640154 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640158 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164015C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640160 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640164 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640168 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164016C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640170 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640174 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640178 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164017C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640180 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640184 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640188 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164018C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640190 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640194 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640198 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164019C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640204 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640208 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164020C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640210 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640214 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640218 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164021C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640220 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640224 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640228 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164022C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640230 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640234 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640238 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164023C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640240 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640244 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640248 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164024C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640250 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640254 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640258 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164025C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640260 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640264 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640268 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164026C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640270 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640274 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640278 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164027C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640280 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640284 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640288 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164028C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640290 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640294 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640298 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164029C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640300 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640304 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640308 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164030C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640310 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640314 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640318 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164031C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640320 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640324 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640328 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164032C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640330 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640334 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640338 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164033C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640340 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640344 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640348 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164034C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640350 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640354 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640358 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164035C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640360 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640364 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640368 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164036C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640374 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164037C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640380 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640384 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640388 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164038C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640390 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640394 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640398 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164039C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640400 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640404 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640408 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164040C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640410 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640414 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640418 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164041C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640420 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640424 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640428 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164042C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640430 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640434 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640438 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164043C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640440 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640444 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640448 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164044C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640450 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640454 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640458 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164045C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640460 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640464 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640468 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164046C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640470 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640474 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640478 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164047C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640480 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640484 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640488 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164048C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640490 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640494 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640498 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164049C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640500 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640504 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640508 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164050C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640510 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640514 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640518 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164051C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640520 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640524 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640528 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164052C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640530 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640534 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640538 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164053C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640540 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640544 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640548 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164054C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640550 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640554 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640558 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164055C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640560 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640564 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640568 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164056C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640570 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640574 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640578 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164057C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640580 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640584 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640588 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164058C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640590 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640594 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640598 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164059C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640600 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640604 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640608 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164060C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640610 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640614 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640618 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164061C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640620 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640624 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640628 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164062C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640630 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640634 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640638 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164063C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640640 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640644 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640648 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164064C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640650 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640654 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640658 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164065C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640660 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640664 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640668 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164066C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640670 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640674 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640678 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164067C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640680 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640684 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640688 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164068C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640690 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640694 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640698 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164069C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640700 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640704 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640708 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164070C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640710 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640714 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640718 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164071C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640720 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640724 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640728 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164072C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640730 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640734 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640738 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164073C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640740 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640744 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640748 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164074C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640750 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640754 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640758 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164075C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640760 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640764 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640768 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164076C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640770 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640774 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640778 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164077C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640780 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640784 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640788 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164078C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640790 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640794 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640798 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164079C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ABC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ACC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ADC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AFC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BBC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BCC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BDC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BFC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01D00014 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00020 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D00404 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00408 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0040C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00410 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00414 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00418 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0041C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00420 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00444 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00448 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0044C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00450 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00454 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00458 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0045C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00460 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00484 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00488 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0048C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00490 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00494 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00498 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0049C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004A0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D004C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004CC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004D0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D4 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004DC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004E0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D008C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D008C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D008CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D01404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D0140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D01410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D01808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0180C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D01810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01820 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01824 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0184C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D01850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01860 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01864 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01C04 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C44 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C84 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01CC4 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D02004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D02408 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D0240C ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02410 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D02414 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02418 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D0241C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D02420 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D02424 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D02428 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D0242C ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D02430 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D02434 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D02438 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D0243C ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D02440 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D02444 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D02448 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D0244C ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D02450 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D02454 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D02458 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D0245C ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D02460 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D02464 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D02468 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D0246C ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D02470 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D02474 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D02478 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D0247C ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D02480 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D02488 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D0248C ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02490 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D02494 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02498 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D0249C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D024A0 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D024A4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D024A8 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D024AC ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D024B0 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D024B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D024B8 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D024BC ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D024C0 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D024C4 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D024C8 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D024CC ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D024D0 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D024D4 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D024D8 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D024DC ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D024E0 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D024E4 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D024E8 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D024EC ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D024F0 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D024F4 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D024F8 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D024FC ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D02500 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D02804 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02808 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0280C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02810 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02814 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02818 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0281C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02820 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02824 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02828 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0282C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02830 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02834 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02838 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0283C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02840 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D40010 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D40014 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x01D40018 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D40044 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x01D40048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D40804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D408C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D408C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D408CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40904 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40908 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4090C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40944 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40948 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4094C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41004 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41014 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4101C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41024 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4102C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41034 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41044 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4104C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41054 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4105C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41064 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4106C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41074 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41084 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4108C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41094 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4109C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410A4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410AC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410B4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D4140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D41410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D41808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4180C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D41810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41820 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41824 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41828 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4184C ) = 0x00001F07;
    *mkptr( dfe_cfg_base, 0x01D41850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41860 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41864 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41868 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41C04 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C44 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C84 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41CC4 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D42004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D42808 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D42810 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D42844 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42848 ) = 0x00000210;
    *mkptr( dfe_cfg_base, 0x01D4284C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42850 ) = 0x00000210;
    *mkptr( dfe_cfg_base, 0x01D42884 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42888 ) = 0x00000220;
    *mkptr( dfe_cfg_base, 0x01D4288C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42890 ) = 0x00000220;
    *mkptr( dfe_cfg_base, 0x01D428C4 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428C8 ) = 0x00000230;
    *mkptr( dfe_cfg_base, 0x01D428CC ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428D0 ) = 0x00000230;
    *mkptr( dfe_cfg_base, 0x01D42904 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42908 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4290C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42910 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D42944 ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42948 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x01D4294C ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42950 ) = 0x00000310;
    *mkptr( dfe_cfg_base, 0x01D42984 ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42988 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x01D4298C ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42990 ) = 0x00000320;
    *mkptr( dfe_cfg_base, 0x01D429C4 ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429C8 ) = 0x00000330;
    *mkptr( dfe_cfg_base, 0x01D429CC ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429D0 ) = 0x00000330;
    *mkptr( dfe_cfg_base, 0x01D42A04 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42A08 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42A0C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42A10 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42A44 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42A48 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42A4C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42A50 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42A84 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42A88 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42A8C ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42A90 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42AC4 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42AC8 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42ACC ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42AD0 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42B04 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42B08 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42B0C ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42B10 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42B44 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42B48 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42B4C ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42B50 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42B84 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42B88 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42B8C ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42B90 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42BC4 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42BC8 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42BCC ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42BD0 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42C04 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42C08 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42C0C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42C10 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D42C44 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42C48 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42C4C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42C50 ) = 0x00000211;
    *mkptr( dfe_cfg_base, 0x01D42C84 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42C88 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42C8C ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D42C90 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01D42CC4 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42CC8 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42CCC ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42CD0 ) = 0x00000231;
    *mkptr( dfe_cfg_base, 0x01D42D04 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42D08 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42D0C ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42D10 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D42D44 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42D48 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42D4C ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42D50 ) = 0x00000311;
    *mkptr( dfe_cfg_base, 0x01D42D84 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42D88 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42D8C ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D42D90 ) = 0x00000321;
    *mkptr( dfe_cfg_base, 0x01D42DC4 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42DC8 ) = 0x00000331;
    *mkptr( dfe_cfg_base, 0x01D42DCC ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42DD0 ) = 0x00000331;
#elif DFE_RATE == 10
    *mkptr( dfe_cfg_base, 0x01600014 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600018 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160001C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600020 ) = 0x09090001;
    *mkptr( dfe_cfg_base, 0x0160002C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600054 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x01600058 ) = 0x04000004;
    *mkptr( dfe_cfg_base, 0x0160005C ) = 0x98111110;
    *mkptr( dfe_cfg_base, 0x01600060 ) = 0x09090003;
    *mkptr( dfe_cfg_base, 0x0160006C ) = 0x00002FF8;
    *mkptr( dfe_cfg_base, 0x01600080 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600084 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600088 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160008C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600090 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600094 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600098 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160009C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x016000A0 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x016000A4 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x016000A8 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x016000AC ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x016000B0 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x016000B4 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000B8 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x016000C0 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x016000C4 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x016000C8 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x016000CC ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x016000D0 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x016000D4 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x016000D8 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x016000DC ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x016000E0 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x016000E4 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x016000E8 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x016000EC ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x016000F0 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x016000F4 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000F8 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600100 ) = 0xFFFFFFCD;
    *mkptr( dfe_cfg_base, 0x01600104 ) = 0xFFFFFF7D;
    *mkptr( dfe_cfg_base, 0x01600108 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x0160010C ) = 0xFFFFFF4B;
    *mkptr( dfe_cfg_base, 0x01600110 ) = 0xFFFFFF71;
    *mkptr( dfe_cfg_base, 0x01600114 ) = 0x000002F3;
    *mkptr( dfe_cfg_base, 0x01600118 ) = 0xFFFFFAA9;
    *mkptr( dfe_cfg_base, 0x0160011C ) = 0x000005A0;
    *mkptr( dfe_cfg_base, 0x01600120 ) = 0xFFFFFE6D;
    *mkptr( dfe_cfg_base, 0x01600124 ) = 0xFFFFF80B;
    *mkptr( dfe_cfg_base, 0x01600128 ) = 0x000015F6;
    *mkptr( dfe_cfg_base, 0x0160012C ) = 0xFFFFDAE4;
    *mkptr( dfe_cfg_base, 0x01600130 ) = 0x000030DA;
    *mkptr( dfe_cfg_base, 0x01600134 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600138 ) = 0x0000CB69;
    *mkptr( dfe_cfg_base, 0x01600140 ) = 0xFFFFFFF9;
    *mkptr( dfe_cfg_base, 0x01600144 ) = 0xFFFFFF87;
    *mkptr( dfe_cfg_base, 0x01600148 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x0160014C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01600150 ) = 0xFFFFFE1F;
    *mkptr( dfe_cfg_base, 0x01600154 ) = 0x0000026F;
    *mkptr( dfe_cfg_base, 0x01600158 ) = 0xFFFFFED3;
    *mkptr( dfe_cfg_base, 0x0160015C ) = 0xFFFFFD62;
    *mkptr( dfe_cfg_base, 0x01600160 ) = 0x0000083C;
    *mkptr( dfe_cfg_base, 0x01600164 ) = 0xFFFFF2F3;
    *mkptr( dfe_cfg_base, 0x01600168 ) = 0x00000CC1;
    *mkptr( dfe_cfg_base, 0x0160016C ) = 0xFFFFFE29;
    *mkptr( dfe_cfg_base, 0x01600170 ) = 0xFFFFE29E;
    *mkptr( dfe_cfg_base, 0x01600174 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x01600178 ) = 0x00009A36;
    *mkptr( dfe_cfg_base, 0x016000BC ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x0160013C ) = 0x00006149;
    *mkptr( dfe_cfg_base, 0x01600200 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600204 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600208 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160020C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600210 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600214 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600218 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160021C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600220 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600224 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600228 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160022C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600230 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600234 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600238 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160023C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600600 ) = 0xFFFFFFD9;
    *mkptr( dfe_cfg_base, 0x01600604 ) = 0x00000727;
    *mkptr( dfe_cfg_base, 0x01600608 ) = 0xFFFFE5D2;
    *mkptr( dfe_cfg_base, 0x0160060C ) = 0x00003A3E;
    *mkptr( dfe_cfg_base, 0x01600610 ) = 0x0001CDF3;
    *mkptr( dfe_cfg_base, 0x01600614 ) = 0x000017A4;
    *mkptr( dfe_cfg_base, 0x01600618 ) = 0xFFFFEEA9;
    *mkptr( dfe_cfg_base, 0x0160061C ) = 0x0000059A;
    *mkptr( dfe_cfg_base, 0x01600620 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01600624 ) = 0x000009AF;
    *mkptr( dfe_cfg_base, 0x01600628 ) = 0xFFFFC3B5;
    *mkptr( dfe_cfg_base, 0x0160062C ) = 0x000148E2;
    *mkptr( dfe_cfg_base, 0x01600630 ) = 0x00011BC6;
    *mkptr( dfe_cfg_base, 0x01600634 ) = 0xFFFFC300;
    *mkptr( dfe_cfg_base, 0x01600638 ) = 0x00000B1E;
    *mkptr( dfe_cfg_base, 0x0160063C ) = 0x0000002A;
    *mkptr( dfe_cfg_base, 0x01600B08 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B10 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B18 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B24 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01600B28 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B30 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600B34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01600A00 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A04 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A08 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A0C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A10 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A14 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A18 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A1C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A28 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A2C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A30 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A34 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A80 ) = 0x77FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A84 ) = 0x37FFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A88 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A8C ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600A90 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A94 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A98 ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600A9C ) = 0xFFFFFFFF;
    *mkptr( dfe_cfg_base, 0x01600AA8 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AAC ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB0 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01600AB4 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01640000 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640004 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640008 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164000C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640010 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640014 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640018 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164001C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640020 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640024 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640028 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164002C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640030 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640034 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640038 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164003C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640040 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640044 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640048 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164004C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640050 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640054 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640058 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164005C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640060 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640064 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640068 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164006C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640070 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640074 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640078 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164007C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640080 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640084 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640088 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164008C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640090 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640094 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640098 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164009C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016400FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640100 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640104 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640108 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164010C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640110 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640114 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640118 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164011C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640120 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640124 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640128 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164012C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640130 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640134 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640138 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164013C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640140 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640144 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640148 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164014C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640150 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640154 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640158 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164015C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640160 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640164 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640168 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164016C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640170 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640174 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640178 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164017C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640180 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640184 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640188 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164018C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640190 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640194 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640198 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164019C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016401FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640200 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640204 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640208 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164020C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640210 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640214 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640218 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164021C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640220 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640224 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640228 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164022C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640230 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640234 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640238 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164023C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640240 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640244 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640248 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164024C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640250 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640254 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640258 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164025C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640260 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640264 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640268 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164026C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640270 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640274 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640278 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164027C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640280 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640284 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640288 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164028C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640290 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640294 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640298 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164029C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016402FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640300 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640304 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640308 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164030C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640310 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640314 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640318 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164031C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640320 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640324 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640328 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164032C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640330 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640334 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640338 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164033C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640340 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640344 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640348 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164034C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640350 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640354 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640358 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164035C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640360 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640364 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640368 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164036C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640370 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640374 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640378 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164037C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640380 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640384 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640388 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164038C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640390 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640394 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640398 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164039C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016403FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640400 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640404 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640408 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164040C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640410 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640414 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640418 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164041C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640420 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640424 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640428 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164042C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640430 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640434 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640438 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164043C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640440 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640444 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640448 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164044C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640450 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640454 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640458 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164045C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640460 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640464 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640468 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164046C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640470 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640474 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640478 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164047C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640480 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640484 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640488 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164048C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640490 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640494 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640498 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164049C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016404FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640500 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640504 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640508 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164050C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640510 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640514 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640518 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164051C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640520 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640524 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640528 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164052C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640530 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640534 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640538 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164053C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640540 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640544 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640548 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164054C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640550 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640554 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640558 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164055C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640560 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640564 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640568 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164056C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640570 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640574 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640578 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164057C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640580 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640584 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640588 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164058C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640590 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640594 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640598 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164059C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016405FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640600 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640604 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640608 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164060C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640610 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640614 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640618 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164061C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640620 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640624 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640628 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164062C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640630 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640634 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640638 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164063C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640640 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640644 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640648 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164064C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640650 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640654 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640658 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164065C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640660 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640664 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640668 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164066C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640670 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640674 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640678 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164067C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640680 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640684 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640688 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164068C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640690 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640694 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640698 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164069C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016406FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640700 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640704 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640708 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164070C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640710 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640714 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640718 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164071C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640720 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640724 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640728 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164072C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640730 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640734 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640738 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164073C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640740 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640744 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640748 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164074C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640750 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640754 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640758 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164075C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640760 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640764 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640768 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164076C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640770 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640774 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640778 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164077C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640780 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640784 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640788 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164078C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640790 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640794 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640798 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164079C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016407FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016408FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x0164099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x016409FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640A9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ABC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ACC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640ADC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640AFC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B00 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B04 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B08 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B0C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B10 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B14 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B18 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B1C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B20 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B24 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B28 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B2C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B30 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B34 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B38 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B3C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B40 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B44 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B48 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B4C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B50 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B54 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B58 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B5C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B60 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B64 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B68 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B6C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B70 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B74 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B78 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B7C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B80 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B84 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B88 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B8C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B90 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B94 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B98 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640B9C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BA8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BAC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BB8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BBC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BC8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BCC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BD8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BDC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BE8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BEC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BF8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01640BFC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01D00014 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00020 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D00404 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00408 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0040C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00410 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00414 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00418 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0041C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00420 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00444 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00448 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0044C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00450 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00454 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00458 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0045C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00460 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00484 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00488 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0048C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D00490 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00494 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00498 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0049C ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004A0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D004C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004CC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004D0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D4 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D004D8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004DC ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D004E0 ) = 0x00005A5A;
    *mkptr( dfe_cfg_base, 0x01D00804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D00888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D0088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D008C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D008C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D008CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D00C04 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D00C0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C44 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D00C4C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D00C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D00C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D00C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D01404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D01408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D0140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D01410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D01808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0180C ) = 0x00001F03;
    *mkptr( dfe_cfg_base, 0x01D01810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01820 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D0184C ) = 0x00001F03;
    *mkptr( dfe_cfg_base, 0x01D01850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D01860 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D01C04 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C44 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01C84 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D01CC4 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D02004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D02408 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02410 ) = 0x00000200;
    *mkptr( dfe_cfg_base, 0x01D02414 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02418 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D0241C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02420 ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D02424 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D02428 ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D0242C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D02430 ) = 0x00000202;
    *mkptr( dfe_cfg_base, 0x01D02434 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D02438 ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D0243C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D02440 ) = 0x00000203;
    *mkptr( dfe_cfg_base, 0x01D02444 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D02448 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D0244C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01D02450 ) = 0x00000204;
    *mkptr( dfe_cfg_base, 0x01D02454 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D02458 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D0245C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01D02460 ) = 0x00000205;
    *mkptr( dfe_cfg_base, 0x01D02464 ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D02468 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D0246C ) = 0x00000006;
    *mkptr( dfe_cfg_base, 0x01D02470 ) = 0x00000206;
    *mkptr( dfe_cfg_base, 0x01D02474 ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D02478 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D0247C ) = 0x00000007;
    *mkptr( dfe_cfg_base, 0x01D02480 ) = 0x00000207;
    *mkptr( dfe_cfg_base, 0x01D02484 ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D02488 ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D0248C ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01D02490 ) = 0x00000208;
    *mkptr( dfe_cfg_base, 0x01D02494 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D02498 ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D0249C ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01D024A0 ) = 0x00000209;
    *mkptr( dfe_cfg_base, 0x01D024A4 ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D024A8 ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D024AC ) = 0x0000000A;
    *mkptr( dfe_cfg_base, 0x01D024B0 ) = 0x0000020A;
    *mkptr( dfe_cfg_base, 0x01D024B4 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D024B8 ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D024BC ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01D024C0 ) = 0x0000020B;
    *mkptr( dfe_cfg_base, 0x01D024C4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D024C8 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D024CC ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01D024D0 ) = 0x0000020C;
    *mkptr( dfe_cfg_base, 0x01D024D4 ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D024D8 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D024DC ) = 0x0000000D;
    *mkptr( dfe_cfg_base, 0x01D024E0 ) = 0x0000020D;
    *mkptr( dfe_cfg_base, 0x01D024E4 ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D024E8 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D024EC ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01D024F0 ) = 0x0000020E;
    *mkptr( dfe_cfg_base, 0x01D024F4 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D024F8 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D024FC ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D02500 ) = 0x0000020F;
    *mkptr( dfe_cfg_base, 0x01D02804 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02808 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0280C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02810 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02814 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02818 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0281C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02820 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02824 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02828 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0282C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02830 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02834 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02838 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D0283C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D02840 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D40010 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01D40014 ) = 0x00000600;
    *mkptr( dfe_cfg_base, 0x01D40018 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D40044 ) = 0x0000EE00;
    *mkptr( dfe_cfg_base, 0x01D40048 ) = 0x0000EEEE;
    *mkptr( dfe_cfg_base, 0x01D40804 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40808 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4080C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40844 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40848 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4084C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40884 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40888 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4088C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D408C4 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D408C8 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D408CC ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40904 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40908 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4090C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40944 ) = 0x0000A5A0;
    *mkptr( dfe_cfg_base, 0x01D40948 ) = 0x0000A5A5;
    *mkptr( dfe_cfg_base, 0x01D4094C ) = 0x0000005A;
    *mkptr( dfe_cfg_base, 0x01D40C04 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D40C0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D40C14 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C1C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C24 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C2C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C34 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C44 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D40C4C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D40C54 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D40C5C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C64 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C6C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D40C74 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41004 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4100C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41014 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4101C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41024 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4102C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41034 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41044 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4104C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41054 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4105C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41064 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D4106C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41074 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41084 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01D4108C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D41094 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D4109C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410A4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410AC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D410B4 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D41404 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D41408 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D4140C ) = 0x00000201;
    *mkptr( dfe_cfg_base, 0x01D41410 ) = 0x00000301;
    *mkptr( dfe_cfg_base, 0x01D41808 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4180C ) = 0x00001F03;
    *mkptr( dfe_cfg_base, 0x01D41810 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41814 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41820 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41824 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41848 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D4184C ) = 0x00001F03;
    *mkptr( dfe_cfg_base, 0x01D41850 ) = 0x00000F00;
    *mkptr( dfe_cfg_base, 0x01D41854 ) = 0x0000202F;
    *mkptr( dfe_cfg_base, 0x01D41860 ) = 0x00001CE0;
    *mkptr( dfe_cfg_base, 0x01D41864 ) = 0x000003FC;
    *mkptr( dfe_cfg_base, 0x01D41C04 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C44 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41C84 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D41CC4 ) = 0x00001FFC;
    *mkptr( dfe_cfg_base, 0x01D42004 ) = 0x00000300;
    *mkptr( dfe_cfg_base, 0x01D42808 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42810 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01D42844 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42848 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D4284C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01D42850 ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01D42884 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42888 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D4288C ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01D42890 ) = 0x00000021;
    *mkptr( dfe_cfg_base, 0x01D428C4 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428C8 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D428CC ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01D428D0 ) = 0x00000031;
    *mkptr( dfe_cfg_base, 0x01D42904 ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42908 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D4290C ) = 0x00000100;
    *mkptr( dfe_cfg_base, 0x01D42910 ) = 0x00000101;
    *mkptr( dfe_cfg_base, 0x01D42944 ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42948 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D4294C ) = 0x00000110;
    *mkptr( dfe_cfg_base, 0x01D42950 ) = 0x00000111;
    *mkptr( dfe_cfg_base, 0x01D42984 ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42988 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D4298C ) = 0x00000120;
    *mkptr( dfe_cfg_base, 0x01D42990 ) = 0x00000121;
    *mkptr( dfe_cfg_base, 0x01D429C4 ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429C8 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D429CC ) = 0x00000130;
    *mkptr( dfe_cfg_base, 0x01D429D0 ) = 0x00000131;
    *mkptr( dfe_cfg_base, 0x01D42A04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D42A08 ) = 0x00000102;
    *mkptr( dfe_cfg_base, 0x01D42A0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D42A10 ) = 0x00000102;
    *mkptr( dfe_cfg_base, 0x01D42A44 ) = 0x00000012;
    *mkptr( dfe_cfg_base, 0x01D42A48 ) = 0x00000112;
    *mkptr( dfe_cfg_base, 0x01D42A4C ) = 0x00000012;
    *mkptr( dfe_cfg_base, 0x01D42A50 ) = 0x00000112;
    *mkptr( dfe_cfg_base, 0x01D42A84 ) = 0x00000022;
    *mkptr( dfe_cfg_base, 0x01D42A88 ) = 0x00000122;
    *mkptr( dfe_cfg_base, 0x01D42A8C ) = 0x00000022;
    *mkptr( dfe_cfg_base, 0x01D42A90 ) = 0x00000122;
    *mkptr( dfe_cfg_base, 0x01D42AC4 ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x01D42AC8 ) = 0x00000132;
    *mkptr( dfe_cfg_base, 0x01D42ACC ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x01D42AD0 ) = 0x00000132;
    *mkptr( dfe_cfg_base, 0x01D42B04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D42B08 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D42B0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D42B10 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D42B44 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01D42B48 ) = 0x00000113;
    *mkptr( dfe_cfg_base, 0x01D42B4C ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01D42B50 ) = 0x00000113;
    *mkptr( dfe_cfg_base, 0x01D42B84 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01D42B88 ) = 0x00000123;
    *mkptr( dfe_cfg_base, 0x01D42B8C ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01D42B90 ) = 0x00000123;
    *mkptr( dfe_cfg_base, 0x01D42BC4 ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x01D42BC8 ) = 0x00000133;
    *mkptr( dfe_cfg_base, 0x01D42BCC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x01D42BD0 ) = 0x00000133;
    *mkptr( dfe_cfg_base, 0x01D42C04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D42C08 ) = 0x00000102;
    *mkptr( dfe_cfg_base, 0x01D42C0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01D42C10 ) = 0x00000102;
    *mkptr( dfe_cfg_base, 0x01D42C44 ) = 0x00000012;
    *mkptr( dfe_cfg_base, 0x01D42C48 ) = 0x00000112;
    *mkptr( dfe_cfg_base, 0x01D42C4C ) = 0x00000012;
    *mkptr( dfe_cfg_base, 0x01D42C50 ) = 0x00000112;
    *mkptr( dfe_cfg_base, 0x01D42C84 ) = 0x00000022;
    *mkptr( dfe_cfg_base, 0x01D42C88 ) = 0x00000122;
    *mkptr( dfe_cfg_base, 0x01D42C8C ) = 0x00000022;
    *mkptr( dfe_cfg_base, 0x01D42C90 ) = 0x00000122;
    *mkptr( dfe_cfg_base, 0x01D42CC4 ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x01D42CC8 ) = 0x00000132;
    *mkptr( dfe_cfg_base, 0x01D42CCC ) = 0x00000032;
    *mkptr( dfe_cfg_base, 0x01D42CD0 ) = 0x00000132;
    *mkptr( dfe_cfg_base, 0x01D42D04 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D42D08 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D42D0C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01D42D10 ) = 0x00000103;
    *mkptr( dfe_cfg_base, 0x01D42D44 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01D42D48 ) = 0x00000113;
    *mkptr( dfe_cfg_base, 0x01D42D4C ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01D42D50 ) = 0x00000113;
    *mkptr( dfe_cfg_base, 0x01D42D84 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01D42D88 ) = 0x00000123;
    *mkptr( dfe_cfg_base, 0x01D42D8C ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01D42D90 ) = 0x00000123;
    *mkptr( dfe_cfg_base, 0x01D42DC4 ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x01D42DC8 ) = 0x00000133;
    *mkptr( dfe_cfg_base, 0x01D42DCC ) = 0x00000033;
    *mkptr( dfe_cfg_base, 0x01D42DD0 ) = 0x00000133;
#endif
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_jesd_loopback_config_split_2,".text:dfe");
#endif
static void iqn2_jesd_loopback_config_split_2()
{
#if DFE_RATE == 20
    *mkptr( dfe_cfg_base, 0x01A00E00 ) = 0x00000470;
    *mkptr( dfe_cfg_base, 0x01A00200 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01A00204 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01A006A0 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A0022C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A00230 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A41000 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41080 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A410C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41100 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41108 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A007EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00604 ) = 0x0000FF00;
    *mkptr( dfe_cfg_base, 0x01A00648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01A41800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x01A41804 ) = 0x0000006F;
    *mkptr( dfe_cfg_base, 0x01A41808 ) = 0x00000062;
    *mkptr( dfe_cfg_base, 0x01A4180C ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x01A41810 ) = 0x0000004F;
    *mkptr( dfe_cfg_base, 0x01A41814 ) = 0x00000047;
    *mkptr( dfe_cfg_base, 0x01A41818 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A4181C ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x01A41820 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x01A41824 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41828 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x01A4182C ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x01A41830 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01A41834 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41838 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A4183C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x01A41840 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01A41844 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01A41848 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A4184C ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01A41850 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01A41854 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01A41874 ) = 0x000000F9;
    *mkptr( dfe_cfg_base, 0x01A41878 ) = 0x000000F7;
    *mkptr( dfe_cfg_base, 0x01A4187C ) = 0x000000F5;
    *mkptr( dfe_cfg_base, 0x01A41880 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41884 ) = 0x000000F2;
    *mkptr( dfe_cfg_base, 0x01A41888 ) = 0x000000F1;
    *mkptr( dfe_cfg_base, 0x01A4188C ) = 0x000000EF;
    *mkptr( dfe_cfg_base, 0x01A41890 ) = 0x000000EE;
    *mkptr( dfe_cfg_base, 0x01A41894 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41898 ) = 0x000000EB;
    *mkptr( dfe_cfg_base, 0x01A4189C ) = 0x000000EA;
    *mkptr( dfe_cfg_base, 0x01A418A0 ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A418A4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x01A418A8 ) = 0x000000E6;
    *mkptr( dfe_cfg_base, 0x01A418AC ) = 0x000000E5;
    *mkptr( dfe_cfg_base, 0x01A418B0 ) = 0x000000E3;
    *mkptr( dfe_cfg_base, 0x01A418B4 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x01A418B8 ) = 0x000000E1;
    *mkptr( dfe_cfg_base, 0x01A418BC ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A418C0 ) = 0x000000DF;
    *mkptr( dfe_cfg_base, 0x01A418C4 ) = 0x000000DE;
    *mkptr( dfe_cfg_base, 0x01A418C8 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x01A418CC ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A418D0 ) = 0x000000DB;
    *mkptr( dfe_cfg_base, 0x01A418D4 ) = 0x000000DA;
    *mkptr( dfe_cfg_base, 0x01A418D8 ) = 0x000000D9;
    *mkptr( dfe_cfg_base, 0x01A418DC ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A418E0 ) = 0x000000D7;
    *mkptr( dfe_cfg_base, 0x01A418E4 ) = 0x000000D6;
    *mkptr( dfe_cfg_base, 0x01A418E8 ) = 0x000000D5;
    *mkptr( dfe_cfg_base, 0x01A418EC ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A418F0 ) = 0x000000D3;
    *mkptr( dfe_cfg_base, 0x01A418F4 ) = 0x000000D2;
    *mkptr( dfe_cfg_base, 0x01A418F8 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x01A418FC ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41900 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41904 ) = 0x000000CF;
    *mkptr( dfe_cfg_base, 0x01A41908 ) = 0x000000CE;
    *mkptr( dfe_cfg_base, 0x01A4190C ) = 0x000000CD;
    *mkptr( dfe_cfg_base, 0x01A41910 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41914 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41918 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x01A4191C ) = 0x000000CA;
    *mkptr( dfe_cfg_base, 0x01A41920 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41924 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41928 ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A4192C ) = 0x000000C7;
    *mkptr( dfe_cfg_base, 0x01A41930 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41934 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41938 ) = 0x000000C5;
    *mkptr( dfe_cfg_base, 0x01A4193C ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41940 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41944 ) = 0x000000C3;
    *mkptr( dfe_cfg_base, 0x01A41948 ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A4194C ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A41950 ) = 0x000000C1;
    *mkptr( dfe_cfg_base, 0x01A41954 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41958 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A4195C ) = 0x000000BF;
    *mkptr( dfe_cfg_base, 0x01A41960 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41964 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41968 ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A4196C ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A41970 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41974 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41978 ) = 0x000000BB;
    *mkptr( dfe_cfg_base, 0x01A4197C ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41980 ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41984 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A41988 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A4198C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41990 ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41994 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A41998 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A4199C ) = 0x000000B6;
    *mkptr( dfe_cfg_base, 0x01A419A0 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A4 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A8 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419AC ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419B0 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B4 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B8 ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419BC ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419C0 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C4 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C8 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419CC ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D0 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D4 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419D8 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419DC ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E0 ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E4 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419E8 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419EC ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F0 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F4 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419F8 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419FC ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A41A00 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A04 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A08 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A0C ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A10 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A14 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A18 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A1C ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A20 ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A24 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A28 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A2C ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A30 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A34 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A38 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A3C ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A40 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A44 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A48 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A4C ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A50 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A54 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A58 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A5C ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A60 ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A64 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A68 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A6C ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A70 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A74 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A78 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A7C ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A80 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A84 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A88 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A8C ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A90 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A94 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A98 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A9C ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41AA0 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA4 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA8 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AAC ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB0 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB4 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB8 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41ABC ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC0 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC4 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC8 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ACC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD0 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD4 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AD8 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41ADC ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AE0 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE4 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE8 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AEC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF0 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF4 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF8 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AFC ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B00 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B04 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B08 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B0C ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B10 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B14 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B18 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B1C ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B20 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B24 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B28 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B2C ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B30 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B34 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B38 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B3C ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B40 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B44 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B48 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B4C ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B50 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B54 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B58 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B5C ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B60 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B64 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B68 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B6C ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B70 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B74 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B78 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B7C ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B80 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B84 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B88 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B8C ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B90 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B94 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B98 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B9C ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA0 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA4 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BA8 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BAC ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB0 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BB8 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BBC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC0 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC8 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BCC ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD0 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD4 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BDC ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE0 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE4 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BEC ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF0 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF4 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF8 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BFC ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41C00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41C04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41C08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41C0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41C10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41C14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41C18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41C1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41C20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41C24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41C28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41C2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41C30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41C34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41C38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41C3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41C40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41C44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41C48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41C4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41C50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41C54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41C58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41C5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41C60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41C64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41C68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41C6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41C70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41C74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41C78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41C7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41C80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41C84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41C88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41C8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41C90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41C94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41C98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41C9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41CA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41CA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41CA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41CAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41CB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41CB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41CB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41CBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41CC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41CC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41CC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41CCC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41CD0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41CD4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41CD8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41CDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41CE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41CE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41CE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41CEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41CF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41CF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41CF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41CFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41D00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41D04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41D08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41D0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41D10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41D14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41D18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41D1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41D20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41D24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41D28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41D2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41D30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41D34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41D38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41D3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41D40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41D44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41D48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41D4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41D50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41D54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41D58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41D5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41D60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41D64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41D68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41D6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41D70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41D74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41D78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41D7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41D80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41D84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41D88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41D8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41D90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41D94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41D98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41D9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41DA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41DA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41DA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41DAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41DB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41DB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41DB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41DBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41DC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41DC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41DC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41DCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41DD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41DD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41DD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41DDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41DE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41DE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41DE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41DEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41DF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41DF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41DF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41E00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41E04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41E08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41E0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41E10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41E14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41E18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41E1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41E20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41E24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41E28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41E2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41E30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41E34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41E38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41E3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41E40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41E44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41E48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41E4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41E50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41E54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41E58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41E5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41E60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41E64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41E68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41E6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41E70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41E74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41E78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41E7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41E80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41E84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41E88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41E8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41E90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41E94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41E98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41E9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41EA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41EA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41EA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41EAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41EB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41EB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41EB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41EBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41EC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41EC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41EC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41ECC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ED0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41ED4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41ED8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41EDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41EE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41EE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41EE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41EEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41EF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41EF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41EF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41EFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41F00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41F04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41F08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41F0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41F10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41F14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41F18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41F1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41F20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41F24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41F28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41F2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41F30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41F34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41F38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41F3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41F40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41F44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41F48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41F4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41F50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41F54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41F58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41F5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41F60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41F64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41F68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41F6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41F70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41F74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41F78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41F7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41F80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41F84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41F88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41F8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41F90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41F94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41F98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41F9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41FA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41FA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41FA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41FAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41FB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41FB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41FB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41FBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41FC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41FC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41FC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41FCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41FD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41FD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41FD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41FDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41FE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41FE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41FE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41FEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41FF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41FF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41FF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42000 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42004 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42008 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4200C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42010 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42014 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42018 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4201C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42020 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42024 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42028 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4202C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42030 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42034 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42038 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4203C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42040 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42044 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42048 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4204C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42050 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42054 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42058 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4205C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42060 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42064 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42068 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4206C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42070 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42074 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42078 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4207C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42080 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42084 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42088 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4208C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42090 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42094 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42098 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4209C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A420A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A420A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A420A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A420AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A420B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A420B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A420B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A420BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A420C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A420C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A420C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A420CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A420D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A420D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A420D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A420DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A420E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A420E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A420E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A420EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A420F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A420F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A420F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A420FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42100 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42104 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42108 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4210C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42110 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42114 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42118 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4211C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42120 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42124 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42128 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4212C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42130 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42134 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42138 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4213C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42140 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42144 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42148 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4214C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42150 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42154 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42158 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4215C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42160 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42164 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42168 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4216C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42170 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42174 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42178 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4217C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42180 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42184 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42188 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4218C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42190 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42194 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42198 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4219C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A421A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A421A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A421A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A421AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A421B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A421B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A421B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A421BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A421C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A421C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A421C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A421CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A421D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A421D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A421D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A421DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A421E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A421E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A421E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A421EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A421F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A421F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A421F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42200 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42204 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42208 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4220C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42210 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42214 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42218 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4221C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42220 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42224 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42228 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4222C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42230 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42234 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42238 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4223C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42240 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42244 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42248 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4224C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42250 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42254 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42258 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4225C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42260 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42264 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42268 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4226C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42270 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42274 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42278 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4227C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42280 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42284 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42288 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4228C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42290 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42294 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42298 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4229C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A422A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A422A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A422A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A422AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A422B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A422B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A422B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A422BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A422C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A422C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A422C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A422CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A422D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A422D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A422D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A422DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A422E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A422E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A422E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A422EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A422F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A422F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A422F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A422FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42300 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42304 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42308 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4230C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42310 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42314 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42318 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4231C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42320 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42324 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42328 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4232C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42330 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42334 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42338 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4233C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42340 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42344 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42348 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4234C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42350 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42354 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42358 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4235C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42360 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42364 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42368 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4236C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42370 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42374 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42378 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4237C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42380 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42384 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42388 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4238C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42390 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42394 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42398 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4239C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A423A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A423A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A423A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A423AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A423B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A423B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A423B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A423BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A423C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A423C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A423C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A423CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A423D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A423D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A423D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A423DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A423E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A423E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A423E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A423EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A423F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A423F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A423F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41400 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41480 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A414C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41500 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41508 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A006EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00A0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A00A14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A0020C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A40220 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A402A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40320 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A403A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40020 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A400A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40120 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A401A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A0021C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01A00220 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x01A00224 ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A00228 ) = 0x00001280;
    *mkptr( dfe_cfg_base, 0x01A002D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A40800 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40804 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40808 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4080C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40810 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40814 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40818 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4081C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40820 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40824 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40828 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4082C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40830 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40834 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40838 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4083C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40840 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40844 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40848 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4084C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40850 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40854 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40858 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4085C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40860 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40864 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40868 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4086C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40870 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40874 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40878 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4087C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40880 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40884 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40888 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4088C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40890 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40894 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40898 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4089C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40900 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40904 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40908 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4090C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40910 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40914 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40918 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4091C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40920 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40924 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40928 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4092C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40930 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40934 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40938 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4093C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40940 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40944 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40948 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4094C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40950 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40954 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40958 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4095C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40960 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40964 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40968 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4096C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40970 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40974 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40978 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4097C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40980 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40984 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40988 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4098C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40990 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40994 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40998 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4099C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40A00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ABC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ACC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ADC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BBC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BCC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BDC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A003EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A0023C ) = 0x00005110;
    *mkptr( dfe_cfg_base, 0x01E8020C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01E80700 ) = 0x00001100;
    *mkptr( dfe_cfg_base, 0x01E80704 ) = 0x0000300C;
    *mkptr( dfe_cfg_base, 0x01E80708 ) = 0x0000300C;
    *mkptr( dfe_cfg_base, 0x01E80728 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01E8073C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01EC0000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC00C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC01C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E80248 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01E80210 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80204 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80218 ) = 0x00001F00;
    *mkptr( dfe_cfg_base, 0x01EC0800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01E80214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01E802B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00200 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01C00208 ) = 0x00001111;
    *mkptr( dfe_cfg_base, 0x01C0020C ) = 0x000002FD;
    *mkptr( dfe_cfg_base, 0x01C00214 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00218 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0021C ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00220 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01C00224 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00228 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0022C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00234 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00238 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0023C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00240 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00244 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00248 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0024C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00250 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00254 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00258 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0025C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00260 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00264 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00268 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0026C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00270 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00274 ) = 0x76541010;
    *mkptr( dfe_cfg_base, 0x01C00278 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00280 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01C00328 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01C003E4 ) = 0x01010101;
#elif DFE_RATE == 10
    *mkptr( dfe_cfg_base, 0x01A00E00 ) = 0x00000470;
    *mkptr( dfe_cfg_base, 0x01A00200 ) = 0x00000221;
    *mkptr( dfe_cfg_base, 0x01A00204 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01A006A0 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A0022C ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A00230 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A41000 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41040 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41080 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A410C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41100 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41108 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A007EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A007FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00604 ) = 0x0000FF00;
    *mkptr( dfe_cfg_base, 0x01A00648 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01A41800 ) = 0x0000007F;
    *mkptr( dfe_cfg_base, 0x01A41804 ) = 0x0000006F;
    *mkptr( dfe_cfg_base, 0x01A41808 ) = 0x00000062;
    *mkptr( dfe_cfg_base, 0x01A4180C ) = 0x00000057;
    *mkptr( dfe_cfg_base, 0x01A41810 ) = 0x0000004F;
    *mkptr( dfe_cfg_base, 0x01A41814 ) = 0x00000047;
    *mkptr( dfe_cfg_base, 0x01A41818 ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A4181C ) = 0x0000003A;
    *mkptr( dfe_cfg_base, 0x01A41820 ) = 0x00000035;
    *mkptr( dfe_cfg_base, 0x01A41824 ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41828 ) = 0x0000002B;
    *mkptr( dfe_cfg_base, 0x01A4182C ) = 0x00000027;
    *mkptr( dfe_cfg_base, 0x01A41830 ) = 0x00000023;
    *mkptr( dfe_cfg_base, 0x01A41834 ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41838 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A4183C ) = 0x00000019;
    *mkptr( dfe_cfg_base, 0x01A41840 ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01A41844 ) = 0x00000013;
    *mkptr( dfe_cfg_base, 0x01A41848 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A4184C ) = 0x0000000E;
    *mkptr( dfe_cfg_base, 0x01A41850 ) = 0x0000000B;
    *mkptr( dfe_cfg_base, 0x01A41854 ) = 0x00000009;
    *mkptr( dfe_cfg_base, 0x01A41874 ) = 0x000000F9;
    *mkptr( dfe_cfg_base, 0x01A41878 ) = 0x000000F7;
    *mkptr( dfe_cfg_base, 0x01A4187C ) = 0x000000F5;
    *mkptr( dfe_cfg_base, 0x01A41880 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41884 ) = 0x000000F2;
    *mkptr( dfe_cfg_base, 0x01A41888 ) = 0x000000F1;
    *mkptr( dfe_cfg_base, 0x01A4188C ) = 0x000000EF;
    *mkptr( dfe_cfg_base, 0x01A41890 ) = 0x000000EE;
    *mkptr( dfe_cfg_base, 0x01A41894 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41898 ) = 0x000000EB;
    *mkptr( dfe_cfg_base, 0x01A4189C ) = 0x000000EA;
    *mkptr( dfe_cfg_base, 0x01A418A0 ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A418A4 ) = 0x000000E7;
    *mkptr( dfe_cfg_base, 0x01A418A8 ) = 0x000000E6;
    *mkptr( dfe_cfg_base, 0x01A418AC ) = 0x000000E5;
    *mkptr( dfe_cfg_base, 0x01A418B0 ) = 0x000000E3;
    *mkptr( dfe_cfg_base, 0x01A418B4 ) = 0x000000E2;
    *mkptr( dfe_cfg_base, 0x01A418B8 ) = 0x000000E1;
    *mkptr( dfe_cfg_base, 0x01A418BC ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A418C0 ) = 0x000000DF;
    *mkptr( dfe_cfg_base, 0x01A418C4 ) = 0x000000DE;
    *mkptr( dfe_cfg_base, 0x01A418C8 ) = 0x000000DD;
    *mkptr( dfe_cfg_base, 0x01A418CC ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A418D0 ) = 0x000000DB;
    *mkptr( dfe_cfg_base, 0x01A418D4 ) = 0x000000DA;
    *mkptr( dfe_cfg_base, 0x01A418D8 ) = 0x000000D9;
    *mkptr( dfe_cfg_base, 0x01A418DC ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A418E0 ) = 0x000000D7;
    *mkptr( dfe_cfg_base, 0x01A418E4 ) = 0x000000D6;
    *mkptr( dfe_cfg_base, 0x01A418E8 ) = 0x000000D5;
    *mkptr( dfe_cfg_base, 0x01A418EC ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A418F0 ) = 0x000000D3;
    *mkptr( dfe_cfg_base, 0x01A418F4 ) = 0x000000D2;
    *mkptr( dfe_cfg_base, 0x01A418F8 ) = 0x000000D1;
    *mkptr( dfe_cfg_base, 0x01A418FC ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41900 ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41904 ) = 0x000000CF;
    *mkptr( dfe_cfg_base, 0x01A41908 ) = 0x000000CE;
    *mkptr( dfe_cfg_base, 0x01A4190C ) = 0x000000CD;
    *mkptr( dfe_cfg_base, 0x01A41910 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41914 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41918 ) = 0x000000CB;
    *mkptr( dfe_cfg_base, 0x01A4191C ) = 0x000000CA;
    *mkptr( dfe_cfg_base, 0x01A41920 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41924 ) = 0x000000C9;
    *mkptr( dfe_cfg_base, 0x01A41928 ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A4192C ) = 0x000000C7;
    *mkptr( dfe_cfg_base, 0x01A41930 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41934 ) = 0x000000C6;
    *mkptr( dfe_cfg_base, 0x01A41938 ) = 0x000000C5;
    *mkptr( dfe_cfg_base, 0x01A4193C ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41940 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41944 ) = 0x000000C3;
    *mkptr( dfe_cfg_base, 0x01A41948 ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A4194C ) = 0x000000C2;
    *mkptr( dfe_cfg_base, 0x01A41950 ) = 0x000000C1;
    *mkptr( dfe_cfg_base, 0x01A41954 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41958 ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A4195C ) = 0x000000BF;
    *mkptr( dfe_cfg_base, 0x01A41960 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41964 ) = 0x000000BE;
    *mkptr( dfe_cfg_base, 0x01A41968 ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A4196C ) = 0x000000BD;
    *mkptr( dfe_cfg_base, 0x01A41970 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41974 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41978 ) = 0x000000BB;
    *mkptr( dfe_cfg_base, 0x01A4197C ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41980 ) = 0x000000BA;
    *mkptr( dfe_cfg_base, 0x01A41984 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A41988 ) = 0x000000B9;
    *mkptr( dfe_cfg_base, 0x01A4198C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41990 ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41994 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A41998 ) = 0x000000B7;
    *mkptr( dfe_cfg_base, 0x01A4199C ) = 0x000000B6;
    *mkptr( dfe_cfg_base, 0x01A419A0 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A4 ) = 0x000000B5;
    *mkptr( dfe_cfg_base, 0x01A419A8 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419AC ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A419B0 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B4 ) = 0x000000B3;
    *mkptr( dfe_cfg_base, 0x01A419B8 ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419BC ) = 0x000000B2;
    *mkptr( dfe_cfg_base, 0x01A419C0 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C4 ) = 0x000000B1;
    *mkptr( dfe_cfg_base, 0x01A419C8 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419CC ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D0 ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A419D4 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419D8 ) = 0x000000AF;
    *mkptr( dfe_cfg_base, 0x01A419DC ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E0 ) = 0x000000AE;
    *mkptr( dfe_cfg_base, 0x01A419E4 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419E8 ) = 0x000000AD;
    *mkptr( dfe_cfg_base, 0x01A419EC ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F0 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A419F4 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419F8 ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A419FC ) = 0x000000AB;
    *mkptr( dfe_cfg_base, 0x01A41A00 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A04 ) = 0x000000AA;
    *mkptr( dfe_cfg_base, 0x01A41A08 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A0C ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A10 ) = 0x000000A9;
    *mkptr( dfe_cfg_base, 0x01A41A14 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A18 ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41A1C ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A20 ) = 0x000000A7;
    *mkptr( dfe_cfg_base, 0x01A41A24 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A28 ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A2C ) = 0x000000A6;
    *mkptr( dfe_cfg_base, 0x01A41A30 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A34 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A38 ) = 0x000000A5;
    *mkptr( dfe_cfg_base, 0x01A41A3C ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A40 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41A44 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A48 ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A4C ) = 0x000000A3;
    *mkptr( dfe_cfg_base, 0x01A41A50 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A54 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A58 ) = 0x000000A2;
    *mkptr( dfe_cfg_base, 0x01A41A5C ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A60 ) = 0x000000A1;
    *mkptr( dfe_cfg_base, 0x01A41A64 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A68 ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A6C ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41A70 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A74 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A78 ) = 0x0000009F;
    *mkptr( dfe_cfg_base, 0x01A41A7C ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A80 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A84 ) = 0x0000009E;
    *mkptr( dfe_cfg_base, 0x01A41A88 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A8C ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A90 ) = 0x0000009D;
    *mkptr( dfe_cfg_base, 0x01A41A94 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A98 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41A9C ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41AA0 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA4 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AA8 ) = 0x0000009B;
    *mkptr( dfe_cfg_base, 0x01A41AAC ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB0 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB4 ) = 0x0000009A;
    *mkptr( dfe_cfg_base, 0x01A41AB8 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41ABC ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC0 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC4 ) = 0x00000099;
    *mkptr( dfe_cfg_base, 0x01A41AC8 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ACC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD0 ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41AD4 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AD8 ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41ADC ) = 0x00000097;
    *mkptr( dfe_cfg_base, 0x01A41AE0 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE4 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AE8 ) = 0x00000096;
    *mkptr( dfe_cfg_base, 0x01A41AEC ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF0 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF4 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AF8 ) = 0x00000095;
    *mkptr( dfe_cfg_base, 0x01A41AFC ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B00 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B04 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41B08 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B0C ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B10 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B14 ) = 0x00000093;
    *mkptr( dfe_cfg_base, 0x01A41B18 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B1C ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B20 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B24 ) = 0x00000092;
    *mkptr( dfe_cfg_base, 0x01A41B28 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B2C ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B30 ) = 0x00000091;
    *mkptr( dfe_cfg_base, 0x01A41B34 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B38 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B3C ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B40 ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41B44 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B48 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B4C ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B50 ) = 0x0000008F;
    *mkptr( dfe_cfg_base, 0x01A41B54 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B58 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B5C ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B60 ) = 0x0000008E;
    *mkptr( dfe_cfg_base, 0x01A41B64 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B68 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B6C ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B70 ) = 0x0000008D;
    *mkptr( dfe_cfg_base, 0x01A41B74 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B78 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B7C ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B80 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41B84 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B88 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B8C ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B90 ) = 0x0000008B;
    *mkptr( dfe_cfg_base, 0x01A41B94 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B98 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41B9C ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA0 ) = 0x0000008A;
    *mkptr( dfe_cfg_base, 0x01A41BA4 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BA8 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BAC ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB0 ) = 0x00000089;
    *mkptr( dfe_cfg_base, 0x01A41BB4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BB8 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BBC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC0 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC4 ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41BC8 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BCC ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD0 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD4 ) = 0x00000087;
    *mkptr( dfe_cfg_base, 0x01A41BD8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BDC ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE0 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE4 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BE8 ) = 0x00000086;
    *mkptr( dfe_cfg_base, 0x01A41BEC ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF0 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF4 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BF8 ) = 0x00000085;
    *mkptr( dfe_cfg_base, 0x01A41BFC ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41C00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41C04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41C08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41C0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41C10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41C14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41C18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41C1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41C20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41C24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41C28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41C2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41C30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41C34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41C38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41C3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41C40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41C44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41C48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41C4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41C50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41C54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41C58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41C5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41C60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41C64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41C68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41C6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41C70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41C74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41C78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41C7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41C80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41C84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41C88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41C8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41C90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41C94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41C98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41C9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41CA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41CA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41CA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41CAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41CB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41CB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41CB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41CBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41CC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41CC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41CC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41CCC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41CD0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41CD4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41CD8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41CDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41CE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41CE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41CE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41CEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41CF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41CF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41CF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41CFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41D00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41D04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41D08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41D0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41D10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41D14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41D18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41D1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41D20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41D24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41D28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41D2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41D30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41D34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41D38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41D3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41D40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41D44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41D48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41D4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41D50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41D54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41D58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41D5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41D60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41D64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41D68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41D6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41D70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41D74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41D78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41D7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41D80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41D84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41D88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41D8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41D90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41D94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41D98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41D9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41DA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41DA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41DA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41DAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41DB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41DB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41DB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41DBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41DC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41DC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41DC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41DCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41DD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41DD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41DD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41DDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41DE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41DE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41DE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41DEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41DF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41DF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41DF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41E00 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A41E04 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A41E08 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A41E0C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A41E10 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A41E14 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A41E18 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A41E1C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A41E20 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A41E24 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A41E28 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A41E2C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A41E30 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A41E34 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A41E38 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A41E3C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A41E40 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A41E44 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A41E48 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A41E4C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A41E50 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A41E54 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A41E58 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A41E5C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A41E60 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A41E64 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A41E68 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A41E6C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A41E70 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A41E74 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A41E78 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A41E7C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A41E80 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A41E84 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A41E88 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A41E8C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A41E90 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A41E94 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A41E98 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A41E9C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A41EA0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A41EA4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A41EA8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A41EAC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A41EB0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A41EB4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A41EB8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A41EBC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A41EC0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A41EC4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A41EC8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A41ECC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A41ED0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A41ED4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A41ED8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A41EDC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A41EE0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A41EE4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A41EE8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A41EEC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A41EF0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A41EF4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A41EF8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A41EFC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A41F00 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A41F04 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A41F08 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A41F0C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A41F10 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A41F14 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A41F18 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A41F1C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A41F20 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A41F24 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A41F28 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A41F2C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A41F30 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A41F34 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A41F38 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A41F3C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A41F40 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A41F44 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A41F48 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A41F4C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A41F50 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A41F54 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A41F58 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A41F5C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A41F60 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A41F64 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A41F68 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A41F6C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A41F70 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A41F74 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A41F78 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A41F7C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A41F80 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A41F84 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A41F88 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A41F8C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A41F90 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A41F94 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A41F98 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A41F9C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A41FA0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A41FA4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A41FA8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A41FAC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A41FB0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A41FB4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A41FB8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A41FBC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A41FC0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A41FC4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A41FC8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A41FCC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A41FD0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A41FD4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A41FD8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A41FDC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41FE0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A41FE4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A41FE8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A41FEC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A41FF0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A41FF4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A41FF8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42000 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42004 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42008 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4200C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42010 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42014 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42018 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4201C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42020 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42024 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42028 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4202C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42030 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42034 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42038 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4203C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42040 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42044 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42048 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4204C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42050 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42054 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42058 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4205C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42060 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42064 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42068 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4206C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42070 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42074 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42078 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4207C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42080 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42084 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42088 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4208C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42090 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42094 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42098 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4209C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A420A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A420A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A420A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A420AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A420B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A420B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A420B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A420BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A420C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A420C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A420C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A420CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A420D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A420D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A420D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A420DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A420E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A420E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A420E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A420EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A420F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A420F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A420F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A420FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42100 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42104 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42108 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4210C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42110 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42114 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42118 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4211C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42120 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42124 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42128 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4212C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42130 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42134 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42138 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4213C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42140 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42144 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42148 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4214C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42150 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42154 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42158 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4215C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42160 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42164 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42168 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4216C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42170 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42174 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42178 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4217C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42180 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42184 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42188 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4218C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42190 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42194 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42198 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4219C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A421A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A421A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A421A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A421AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A421B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A421B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A421B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A421BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A421C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A421C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A421C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A421CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A421D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A421D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A421D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A421DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A421E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A421E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A421E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A421EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A421F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A421F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A421F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A42200 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A42204 ) = 0x000000FC;
    *mkptr( dfe_cfg_base, 0x01A42208 ) = 0x000078D7;
    *mkptr( dfe_cfg_base, 0x01A4220C ) = 0x000000F8;
    *mkptr( dfe_cfg_base, 0x01A42210 ) = 0x00007215;
    *mkptr( dfe_cfg_base, 0x01A42214 ) = 0x000000F4;
    *mkptr( dfe_cfg_base, 0x01A42218 ) = 0x00006BB3;
    *mkptr( dfe_cfg_base, 0x01A4221C ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A42220 ) = 0x000065AD;
    *mkptr( dfe_cfg_base, 0x01A42224 ) = 0x000000EC;
    *mkptr( dfe_cfg_base, 0x01A42228 ) = 0x00005FFD;
    *mkptr( dfe_cfg_base, 0x01A4222C ) = 0x000000E8;
    *mkptr( dfe_cfg_base, 0x01A42230 ) = 0x00005A9E;
    *mkptr( dfe_cfg_base, 0x01A42234 ) = 0x000000E4;
    *mkptr( dfe_cfg_base, 0x01A42238 ) = 0x0000558C;
    *mkptr( dfe_cfg_base, 0x01A4223C ) = 0x000000E0;
    *mkptr( dfe_cfg_base, 0x01A42240 ) = 0x000050C3;
    *mkptr( dfe_cfg_base, 0x01A42244 ) = 0x000000DC;
    *mkptr( dfe_cfg_base, 0x01A42248 ) = 0x00004C3F;
    *mkptr( dfe_cfg_base, 0x01A4224C ) = 0x000000D8;
    *mkptr( dfe_cfg_base, 0x01A42250 ) = 0x000047FB;
    *mkptr( dfe_cfg_base, 0x01A42254 ) = 0x000000D4;
    *mkptr( dfe_cfg_base, 0x01A42258 ) = 0x000043F4;
    *mkptr( dfe_cfg_base, 0x01A4225C ) = 0x000000D0;
    *mkptr( dfe_cfg_base, 0x01A42260 ) = 0x00004027;
    *mkptr( dfe_cfg_base, 0x01A42264 ) = 0x000000CC;
    *mkptr( dfe_cfg_base, 0x01A42268 ) = 0x00003C90;
    *mkptr( dfe_cfg_base, 0x01A4226C ) = 0x000000C8;
    *mkptr( dfe_cfg_base, 0x01A42270 ) = 0x0000392D;
    *mkptr( dfe_cfg_base, 0x01A42274 ) = 0x000000C4;
    *mkptr( dfe_cfg_base, 0x01A42278 ) = 0x000035FA;
    *mkptr( dfe_cfg_base, 0x01A4227C ) = 0x000000C0;
    *mkptr( dfe_cfg_base, 0x01A42280 ) = 0x000032F5;
    *mkptr( dfe_cfg_base, 0x01A42284 ) = 0x000000BC;
    *mkptr( dfe_cfg_base, 0x01A42288 ) = 0x0000301B;
    *mkptr( dfe_cfg_base, 0x01A4228C ) = 0x000000B8;
    *mkptr( dfe_cfg_base, 0x01A42290 ) = 0x00002D6B;
    *mkptr( dfe_cfg_base, 0x01A42294 ) = 0x000000B4;
    *mkptr( dfe_cfg_base, 0x01A42298 ) = 0x00002AE0;
    *mkptr( dfe_cfg_base, 0x01A4229C ) = 0x000000B0;
    *mkptr( dfe_cfg_base, 0x01A422A0 ) = 0x0000287A;
    *mkptr( dfe_cfg_base, 0x01A422A4 ) = 0x000000AC;
    *mkptr( dfe_cfg_base, 0x01A422A8 ) = 0x00002637;
    *mkptr( dfe_cfg_base, 0x01A422AC ) = 0x000000A8;
    *mkptr( dfe_cfg_base, 0x01A422B0 ) = 0x00002413;
    *mkptr( dfe_cfg_base, 0x01A422B4 ) = 0x000000A4;
    *mkptr( dfe_cfg_base, 0x01A422B8 ) = 0x0000220F;
    *mkptr( dfe_cfg_base, 0x01A422BC ) = 0x000000A0;
    *mkptr( dfe_cfg_base, 0x01A422C0 ) = 0x00002027;
    *mkptr( dfe_cfg_base, 0x01A422C4 ) = 0x0000009C;
    *mkptr( dfe_cfg_base, 0x01A422C8 ) = 0x00001E5B;
    *mkptr( dfe_cfg_base, 0x01A422CC ) = 0x00000098;
    *mkptr( dfe_cfg_base, 0x01A422D0 ) = 0x00001CA8;
    *mkptr( dfe_cfg_base, 0x01A422D4 ) = 0x00000094;
    *mkptr( dfe_cfg_base, 0x01A422D8 ) = 0x00001B0D;
    *mkptr( dfe_cfg_base, 0x01A422DC ) = 0x00000090;
    *mkptr( dfe_cfg_base, 0x01A422E0 ) = 0x0000198A;
    *mkptr( dfe_cfg_base, 0x01A422E4 ) = 0x0000008C;
    *mkptr( dfe_cfg_base, 0x01A422E8 ) = 0x0000181C;
    *mkptr( dfe_cfg_base, 0x01A422EC ) = 0x00000088;
    *mkptr( dfe_cfg_base, 0x01A422F0 ) = 0x000016C3;
    *mkptr( dfe_cfg_base, 0x01A422F4 ) = 0x00000084;
    *mkptr( dfe_cfg_base, 0x01A422F8 ) = 0x0000157D;
    *mkptr( dfe_cfg_base, 0x01A422FC ) = 0x00000080;
    *mkptr( dfe_cfg_base, 0x01A42300 ) = 0x00001449;
    *mkptr( dfe_cfg_base, 0x01A42304 ) = 0x0000007C;
    *mkptr( dfe_cfg_base, 0x01A42308 ) = 0x00001327;
    *mkptr( dfe_cfg_base, 0x01A4230C ) = 0x00000078;
    *mkptr( dfe_cfg_base, 0x01A42310 ) = 0x00001215;
    *mkptr( dfe_cfg_base, 0x01A42314 ) = 0x00000074;
    *mkptr( dfe_cfg_base, 0x01A42318 ) = 0x00001112;
    *mkptr( dfe_cfg_base, 0x01A4231C ) = 0x00000070;
    *mkptr( dfe_cfg_base, 0x01A42320 ) = 0x0000101D;
    *mkptr( dfe_cfg_base, 0x01A42324 ) = 0x0000006C;
    *mkptr( dfe_cfg_base, 0x01A42328 ) = 0x00000F36;
    *mkptr( dfe_cfg_base, 0x01A4232C ) = 0x00000068;
    *mkptr( dfe_cfg_base, 0x01A42330 ) = 0x00000E5D;
    *mkptr( dfe_cfg_base, 0x01A42334 ) = 0x00000064;
    *mkptr( dfe_cfg_base, 0x01A42338 ) = 0x00000D8F;
    *mkptr( dfe_cfg_base, 0x01A4233C ) = 0x00000060;
    *mkptr( dfe_cfg_base, 0x01A42340 ) = 0x00000CCD;
    *mkptr( dfe_cfg_base, 0x01A42344 ) = 0x0000005C;
    *mkptr( dfe_cfg_base, 0x01A42348 ) = 0x00000C15;
    *mkptr( dfe_cfg_base, 0x01A4234C ) = 0x00000058;
    *mkptr( dfe_cfg_base, 0x01A42350 ) = 0x00000B68;
    *mkptr( dfe_cfg_base, 0x01A42354 ) = 0x00000054;
    *mkptr( dfe_cfg_base, 0x01A42358 ) = 0x00000AC5;
    *mkptr( dfe_cfg_base, 0x01A4235C ) = 0x00000050;
    *mkptr( dfe_cfg_base, 0x01A42360 ) = 0x00000A2B;
    *mkptr( dfe_cfg_base, 0x01A42364 ) = 0x0000004C;
    *mkptr( dfe_cfg_base, 0x01A42368 ) = 0x00000999;
    *mkptr( dfe_cfg_base, 0x01A4236C ) = 0x00000048;
    *mkptr( dfe_cfg_base, 0x01A42370 ) = 0x00000910;
    *mkptr( dfe_cfg_base, 0x01A42374 ) = 0x00000044;
    *mkptr( dfe_cfg_base, 0x01A42378 ) = 0x0000088E;
    *mkptr( dfe_cfg_base, 0x01A4237C ) = 0x00000040;
    *mkptr( dfe_cfg_base, 0x01A42380 ) = 0x00000814;
    *mkptr( dfe_cfg_base, 0x01A42384 ) = 0x0000003C;
    *mkptr( dfe_cfg_base, 0x01A42388 ) = 0x000007A0;
    *mkptr( dfe_cfg_base, 0x01A4238C ) = 0x00000038;
    *mkptr( dfe_cfg_base, 0x01A42390 ) = 0x00000733;
    *mkptr( dfe_cfg_base, 0x01A42394 ) = 0x00000034;
    *mkptr( dfe_cfg_base, 0x01A42398 ) = 0x000006CC;
    *mkptr( dfe_cfg_base, 0x01A4239C ) = 0x00000030;
    *mkptr( dfe_cfg_base, 0x01A423A0 ) = 0x0000066A;
    *mkptr( dfe_cfg_base, 0x01A423A4 ) = 0x0000002C;
    *mkptr( dfe_cfg_base, 0x01A423A8 ) = 0x0000060E;
    *mkptr( dfe_cfg_base, 0x01A423AC ) = 0x00000028;
    *mkptr( dfe_cfg_base, 0x01A423B0 ) = 0x000005B8;
    *mkptr( dfe_cfg_base, 0x01A423B4 ) = 0x00000024;
    *mkptr( dfe_cfg_base, 0x01A423B8 ) = 0x00000566;
    *mkptr( dfe_cfg_base, 0x01A423BC ) = 0x00000020;
    *mkptr( dfe_cfg_base, 0x01A423C0 ) = 0x00000519;
    *mkptr( dfe_cfg_base, 0x01A423C4 ) = 0x0000001C;
    *mkptr( dfe_cfg_base, 0x01A423C8 ) = 0x000004D0;
    *mkptr( dfe_cfg_base, 0x01A423CC ) = 0x00000018;
    *mkptr( dfe_cfg_base, 0x01A423D0 ) = 0x0000048B;
    *mkptr( dfe_cfg_base, 0x01A423D4 ) = 0x00000014;
    *mkptr( dfe_cfg_base, 0x01A423D8 ) = 0x0000044A;
    *mkptr( dfe_cfg_base, 0x01A423DC ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A423E0 ) = 0x0000040C;
    *mkptr( dfe_cfg_base, 0x01A423E4 ) = 0x0000000C;
    *mkptr( dfe_cfg_base, 0x01A423E8 ) = 0x000003D2;
    *mkptr( dfe_cfg_base, 0x01A423EC ) = 0x00000008;
    *mkptr( dfe_cfg_base, 0x01A423F0 ) = 0x0000039C;
    *mkptr( dfe_cfg_base, 0x01A423F4 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A423F8 ) = 0x00000368;
    *mkptr( dfe_cfg_base, 0x01A41400 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41440 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41480 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A414C0 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A41500 ) = 0x000003E8;
    *mkptr( dfe_cfg_base, 0x01A41508 ) = 0x000007D0;
    *mkptr( dfe_cfg_base, 0x01A006EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A006FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A00A0C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A00A14 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A0020C ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01A40220 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A402A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40320 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A403A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40020 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A400A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A40120 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A401A0 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01A0021C ) = 0x00000011;
    *mkptr( dfe_cfg_base, 0x01A00220 ) = 0x000007B0;
    *mkptr( dfe_cfg_base, 0x01A00224 ) = 0x000000F0;
    *mkptr( dfe_cfg_base, 0x01A00228 ) = 0x00001280;
    *mkptr( dfe_cfg_base, 0x01A002D0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002D8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C0 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A002C8 ) = 0x00004000;
    *mkptr( dfe_cfg_base, 0x01A40800 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40804 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40808 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4080C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40810 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40814 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40818 ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A4081C ) = 0x00000005;
    *mkptr( dfe_cfg_base, 0x01A40820 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40824 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40828 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4082C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40830 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40834 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40838 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4083C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40840 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40844 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40848 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4084C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40850 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40854 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40858 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4085C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40860 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40864 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40868 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4086C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40870 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40874 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40878 ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A4087C ) = 0x00000004;
    *mkptr( dfe_cfg_base, 0x01A40880 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40884 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40888 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4088C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40890 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40894 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40898 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4089C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A408FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40900 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40904 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40908 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4090C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40910 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40914 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40918 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4091C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40920 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40924 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40928 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4092C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40930 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40934 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40938 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4093C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40940 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40944 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40948 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4094C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40950 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40954 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40958 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4095C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40960 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40964 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40968 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4096C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40970 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40974 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40978 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4097C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40980 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40984 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40988 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4098C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40990 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40994 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40998 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A4099C ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409A8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409AC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409B8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409BC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409C8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409CC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409D8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409DC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409E8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409EC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F0 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F4 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409F8 ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A409FC ) = 0x00000003;
    *mkptr( dfe_cfg_base, 0x01A40A00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40A9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ABC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ACC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40ADC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40AFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B00 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B04 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B08 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B0C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B10 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B14 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B18 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B1C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B20 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B24 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B28 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B2C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B30 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B34 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B38 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B3C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B40 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B44 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B48 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B4C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B50 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B54 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B58 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B5C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B60 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B64 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B68 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B6C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B70 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B74 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B78 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B7C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B80 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B84 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B88 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B8C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B90 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B94 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B98 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40B9C ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BA8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BAC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BB8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BBC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BC8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BCC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BD8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BDC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BE8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BEC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF0 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF4 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BF8 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A40BFC ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01A003EC ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003F0 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01A003FC ) = 0x00003000;
    *mkptr( dfe_cfg_base, 0x01A0023C ) = 0x00005110;
    *mkptr( dfe_cfg_base, 0x01E80700 ) = 0x00001100;
    *mkptr( dfe_cfg_base, 0x01E80704 ) = 0x0000300C;
    *mkptr( dfe_cfg_base, 0x01E80708 ) = 0x00007FFF;
    *mkptr( dfe_cfg_base, 0x01E80728 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01E8073C ) = 0x00000016;
    *mkptr( dfe_cfg_base, 0x01EC0000 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0040 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0080 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC00C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0100 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0140 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0180 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC01C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0200 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01EC0240 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E80210 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80204 ) = 0x0000001F;
    *mkptr( dfe_cfg_base, 0x01E80218 ) = 0x00001F00;
    *mkptr( dfe_cfg_base, 0x01EC0800 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0804 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0808 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC080C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0810 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0814 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0818 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC081C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0820 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0824 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0828 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC082C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0830 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0834 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0838 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC083C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0840 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0844 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0848 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC084C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0850 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0854 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0858 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC085C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0860 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0864 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0868 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC086C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0870 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0874 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0878 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC087C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0880 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0884 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0888 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC088C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0890 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0894 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0898 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC089C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC08FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0900 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0904 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0908 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC090C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0910 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0914 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0918 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC091C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0920 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0924 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0928 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC092C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0930 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0934 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0938 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC093C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0940 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0944 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0948 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC094C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0950 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0954 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0958 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC095C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0960 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0964 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0968 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC096C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0970 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0974 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0978 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC097C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0980 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0984 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0988 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC098C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0990 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0994 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC0998 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC099C ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09A8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09AC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09B8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09BC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09C8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09CC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09D8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09DC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09E8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09EC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F0 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F4 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09F8 ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01EC09FC ) = 0x00008000;
    *mkptr( dfe_cfg_base, 0x01E80214 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01E802B0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802C0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802D0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802E0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01E802F0 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00200 ) = 0x00000002;
    *mkptr( dfe_cfg_base, 0x01C00208 ) = 0x00001111;
    *mkptr( dfe_cfg_base, 0x01C00214 ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00218 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0021C ) = 0x00002000;
    *mkptr( dfe_cfg_base, 0x01C00220 ) = 0x00000010;
    *mkptr( dfe_cfg_base, 0x01C00224 ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00228 ) = 0x00001020;
    *mkptr( dfe_cfg_base, 0x01C0022C ) = 0x00002200;
    *mkptr( dfe_cfg_base, 0x01C00234 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00238 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0023C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00240 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00244 ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00248 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0024C ) = 0x00003210;
    *mkptr( dfe_cfg_base, 0x01C00250 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00254 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00258 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0025C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00260 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00264 ) = 0x10100000;
    *mkptr( dfe_cfg_base, 0x01C00268 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C0026C ) = 0x32100000;
    *mkptr( dfe_cfg_base, 0x01C00270 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00274 ) = 0x76541010;
    *mkptr( dfe_cfg_base, 0x01C00278 ) = 0x0000000F;
    *mkptr( dfe_cfg_base, 0x01C00280 ) = 0x00000001;
    *mkptr( dfe_cfg_base, 0x01C00328 ) = 0x0000FFFF;
    *mkptr( dfe_cfg_base, 0x01C003E4 ) = 0x01010101;
#endif
}
#endif

#ifdef DFE_CTL
static void iqn2_dfe_ctl_jesd_loopback_config_split_1();
static void iqn2_dfe_ctl_jesd_loopback_config_split_2();
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_dfe_ctl_jesd_loopback_config,".text:dfe");
#endif
void iqn2_dfe_ctl_jesd_loopback_config()
{
#if DFE_RATE==20
    // based on P1_3_LTE20_a_jesdlpbk.reg
   		*mkptr( dfe_cfg_base, 0x00040234 ) =  0x00000070;
		*mkptr( dfe_cfg_base, 0x00040200 ) =  0x00000310;
		*mkptr( dfe_cfg_base, 0x00040204 ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x0004020C ) =  0x00004080;
		*mkptr( dfe_cfg_base, 0x00040210 ) =  0x00001020;
		*mkptr( dfe_cfg_base, 0x00040214 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00040228 ) =  0x00001000;
		*mkptr( dfe_cfg_base, 0x00040240 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x00040244 ) =  0x00002222;
		*mkptr( dfe_cfg_base, 0x00040250 ) =  0x00004444;
		*mkptr( dfe_cfg_base, 0x0004027C ) =  0x00000808;
		*mkptr( dfe_cfg_base, 0x00040280 ) =  0x00000808;
		*mkptr( dfe_cfg_base, 0x000402C0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x000402D0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x000402E4 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x000402F4 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00040344 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00040360 ) =  0x00000011;
		*mkptr( dfe_cfg_base, 0x00040364 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x0004036C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00040374 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00040400 ) =  0x000001D4;
		*mkptr( dfe_cfg_base, 0x00040408 ) =  0x000005FC;
		*mkptr( dfe_cfg_base, 0x00040410 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x00040800 ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x00040808 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x00040810 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x00040818 ) =  0x00000D8B;
		*mkptr( dfe_cfg_base, 0x00040E18 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00040E1C ) =  0x0000D8F0;
		*mkptr( dfe_cfg_base, 0x00040E20 ) =  0x00002710;
		*mkptr( dfe_cfg_base, 0x00040E28 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00040E2C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00040E30 ) =  0x0000FFFE;
		*mkptr( dfe_cfg_base, 0x00041200 ) =  0x00000241;
		*mkptr( dfe_cfg_base, 0x0004120C ) =  0x000005FC;
		*mkptr( dfe_cfg_base, 0x00041210 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00040600 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00040A00 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00060000 ) =  0x0000D000;
		*mkptr( dfe_cfg_base, 0x00060004 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00060008 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x00060010 ) =  0x0000D101;
		*mkptr( dfe_cfg_base, 0x00060014 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00060018 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x00060020 ) =  0x0000D210;
		*mkptr( dfe_cfg_base, 0x00060024 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00060028 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x00060030 ) =  0x0000D311;
		*mkptr( dfe_cfg_base, 0x00060034 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00060038 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x00060800 ) =  0x00008420;
		*mkptr( dfe_cfg_base, 0x00060804 ) =  0x0000F010;
		*mkptr( dfe_cfg_base, 0x00060820 ) =  0x00008421;
		*mkptr( dfe_cfg_base, 0x00060824 ) =  0x0000F011;
		*mkptr( dfe_cfg_base, 0x00060840 ) =  0x00008430;
		*mkptr( dfe_cfg_base, 0x00060844 ) =  0x0000F012;
		*mkptr( dfe_cfg_base, 0x00060860 ) =  0x00008431;
		*mkptr( dfe_cfg_base, 0x00060864 ) =  0x0000F013;
		*mkptr( dfe_cfg_base, 0x00060884 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000608A4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000608C4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000608E4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060904 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060924 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060944 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060964 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060984 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000609A4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000609C4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000609E4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060A04 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060A24 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060A44 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060A64 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060A84 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060AA4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060AC4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060AE4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060B04 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060B24 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060B44 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060B64 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060B84 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060BA4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060BC4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060BE4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060C04 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060C24 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060C44 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060C64 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060C84 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060CA4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060CC4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060CE4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060D04 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060D24 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060D44 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060D64 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060D84 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060DA4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060DC4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00060DE4 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x00061004 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061008 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x0006100C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00061010 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061014 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061018 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006101C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061020 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061024 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061028 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006102C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061030 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061034 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061038 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006103C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061040 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061044 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061048 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006104C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061050 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061054 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061058 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006105C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061060 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061064 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061068 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006106C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061070 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061074 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061078 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006107C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061080 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061084 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061088 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006108C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061090 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061094 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061098 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006109C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000610FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061100 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061104 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061108 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006110C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061110 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061114 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061118 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006111C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061120 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061124 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061128 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006112C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061130 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061134 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061138 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006113C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061140 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061144 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061148 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006114C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061150 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061154 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061158 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006115C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061160 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061164 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061168 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006116C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061170 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061174 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061178 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006117C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061180 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061184 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061188 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006118C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061190 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061194 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061198 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006119C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000611FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061200 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061204 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061208 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006120C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061210 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061214 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061218 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006121C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061220 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061224 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061228 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006122C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061230 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061234 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061238 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006123C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061240 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061244 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061248 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006124C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061250 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061254 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061258 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006125C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061260 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061264 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061268 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006126C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061270 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061274 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061278 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006127C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061280 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061284 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061288 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006128C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061290 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061294 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061298 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006129C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000612FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061300 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061304 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061308 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006130C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061310 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061314 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061318 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006131C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061320 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061324 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061328 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006132C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061330 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061334 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061338 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006133C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061340 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061344 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061348 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006134C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061350 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061354 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061358 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006135C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061360 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061364 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061368 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006136C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061370 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061374 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061378 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006137C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061380 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061384 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061388 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006138C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061390 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061394 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061398 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006139C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000613FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061400 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061408 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061410 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061418 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061420 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061428 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061430 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061438 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061440 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061448 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061450 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061458 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061460 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061468 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061470 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061478 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061480 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061488 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061490 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061498 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614A0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614A8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614B0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614B8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614C0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614C8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614D0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614D8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614E0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614E8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614F0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000614F8 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061500 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061508 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061510 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061518 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061520 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061528 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061530 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061538 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061540 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061548 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061550 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061558 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061560 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061568 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061570 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061578 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00061800 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061808 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061810 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061818 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061820 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061828 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061830 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061838 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061840 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061848 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061850 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061858 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061860 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061868 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061870 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061878 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061880 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061888 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061890 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061898 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618A0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618A8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618B0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618B8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618C0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618C8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618D0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618D8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618E0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618E8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618F0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000618F8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061900 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061908 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061910 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061918 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061920 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061928 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061930 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061938 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061940 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061948 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061950 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061958 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061960 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061968 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061970 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061978 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061980 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061988 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061990 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061998 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619A0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619A8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619B0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619B8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619C0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619C8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619D0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619D8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619E0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619E8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619F0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x000619F8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A00 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A08 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A10 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A18 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A20 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A28 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A30 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061A38 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00061C00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061C9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CBC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CCC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CD0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CD4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CD8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CDC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061CFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061D9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DBC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DCC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DD0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DD4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DD8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DDC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061DFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061E9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EBC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061ECC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061ED0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061ED4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061ED8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EDC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061EFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061F9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FBC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FCC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FD0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FD4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FD8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FDC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00061FFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062000 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062004 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062008 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006200C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062010 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062014 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062018 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006201C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062020 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062024 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062028 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006202C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062030 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062034 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062038 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006203C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062040 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062044 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062048 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006204C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062050 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062054 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062058 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006205C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062060 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062064 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062068 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006206C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062070 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062074 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062078 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006207C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062080 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062084 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062088 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006208C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062090 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062094 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062098 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006209C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000620BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062400 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00062404 ) =  0x00000401;
		*mkptr( dfe_cfg_base, 0x00062408 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x0006240C ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00062410 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062414 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062418 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006241C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062420 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062424 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062428 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006242C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062430 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062434 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062438 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006243C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062440 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062444 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062448 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006244C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062450 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062454 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062458 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006245C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062460 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062464 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062468 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006246C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062470 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062474 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062478 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006247C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062480 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062484 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062488 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006248C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062490 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062494 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062498 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006249C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000624FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062500 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062504 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062508 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006250C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062510 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062514 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062518 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006251C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062520 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062524 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062528 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006252C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062530 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062534 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062538 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006253C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062540 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062544 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062548 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006254C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062550 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062554 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062558 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006255C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062560 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062564 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062568 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006256C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062570 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062574 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062578 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006257C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062580 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062584 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062588 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006258C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062590 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062594 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062598 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006259C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000625FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062600 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062604 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062608 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006260C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062610 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062614 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062618 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006261C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062620 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062624 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062628 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006262C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062630 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062634 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062638 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006263C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062640 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062644 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062648 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006264C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062650 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062654 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062658 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006265C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062660 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062664 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062668 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006266C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062670 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062674 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062678 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006267C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062680 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062684 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062688 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006268C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062690 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062694 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062698 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006269C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000626FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062700 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062704 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062708 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006270C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062710 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062714 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062718 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006271C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062720 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062724 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062728 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006272C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062730 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062734 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062738 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006273C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062740 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062744 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062748 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006274C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062750 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062754 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062758 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006275C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062760 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062764 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062768 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006276C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062770 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062774 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062778 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006277C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062780 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062784 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062788 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006278C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062790 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062794 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062798 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006279C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000627FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062800 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062804 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062808 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006280C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062810 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062814 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062818 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006281C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062820 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062824 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062828 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006282C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062830 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062834 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062838 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006283C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062840 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062844 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062848 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006284C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062850 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062854 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062858 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006285C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062860 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062864 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062868 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006286C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062870 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062874 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062878 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006287C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062880 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062884 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062888 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006288C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062890 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062894 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062898 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006289C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000628FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062900 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062904 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062908 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006290C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062910 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062914 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062918 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006291C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062920 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062924 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062928 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006292C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062930 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062934 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062938 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006293C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062940 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062944 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062948 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006294C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062950 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062954 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062958 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006295C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062960 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062964 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062968 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006296C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062970 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062974 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062978 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006297C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062980 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062984 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062988 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006298C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062990 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062994 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062998 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006299C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000629FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062A9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062ABC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062ACC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AD0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AD4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AD8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062ADC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062AFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B00 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B04 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B08 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B0C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B10 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B14 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B18 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B1C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B20 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B24 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B28 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B2C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B30 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B34 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B38 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B3C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B40 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B44 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B48 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B4C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B50 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B54 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B58 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B5C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B60 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B64 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B68 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B6C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B70 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B74 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B78 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B7C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B80 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B84 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B88 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B8C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B90 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B94 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B98 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062B9C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BA0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BA4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BA8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BAC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BB0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BB4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BB8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BBC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BC0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BC4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BC8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BCC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BD0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BD4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BD8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BDC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BE0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BE4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BE8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BEC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BF0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BF4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BF8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062BFC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00062C00 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C04 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C08 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C0C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C10 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C14 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C18 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C1C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C20 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C24 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C28 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C2C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C30 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C34 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C38 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C3C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C40 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C44 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C48 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C4C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C50 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C54 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C58 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C5C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C60 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C64 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C68 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C6C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C70 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C74 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C78 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C7C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C80 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C84 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C88 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C8C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C90 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C94 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C98 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062C9C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CA0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CA4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CA8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CAC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CB0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CB4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CB8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00062CBC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063000 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063004 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063008 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006300C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063010 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063014 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063018 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006301C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063020 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063024 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063028 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006302C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063030 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063034 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063038 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006303C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063040 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063044 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063048 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006304C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063050 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063054 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063058 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006305C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063060 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063064 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063068 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006306C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063070 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063074 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063078 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006307C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063080 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063084 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063088 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006308C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063090 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063094 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063098 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006309C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000630FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063100 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063104 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063108 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006310C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063110 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063114 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063118 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006311C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063120 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063124 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063128 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006312C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063130 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063134 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063138 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006313C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063140 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063144 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063148 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006314C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063150 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063154 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063158 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006315C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063160 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063164 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063168 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006316C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063170 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063174 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063178 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006317C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063180 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063184 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063188 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006318C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063190 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063194 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063198 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006319C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000631FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063200 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063204 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063208 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006320C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063210 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063214 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063218 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006321C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063220 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063224 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063228 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006322C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063230 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063234 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063238 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006323C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063240 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063244 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063248 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006324C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063250 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063254 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063258 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006325C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063260 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063264 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063268 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006326C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063270 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063274 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063278 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006327C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063280 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063284 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063288 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006328C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063290 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063294 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063298 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006329C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000632FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063300 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063304 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063308 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006330C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063310 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063314 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063318 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006331C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063320 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063324 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063328 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006332C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063330 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063334 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063338 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006333C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063340 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063344 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063348 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006334C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063350 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063354 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063358 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006335C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063360 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063364 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063368 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006336C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063370 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063374 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063378 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006337C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063380 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063384 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063388 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006338C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063390 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063394 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063398 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x0006339C ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633A0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633A4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633A8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633AC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633B0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633B4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633B8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633BC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633C0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633C4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633C8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633CC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633D0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633D4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633D8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633DC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633E0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633E4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633E8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633EC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633F0 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633F4 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633F8 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x000633FC ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00063400 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063404 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063408 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006340C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063410 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063414 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063418 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006341C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063420 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063424 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063428 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006342C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063430 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063434 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063438 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006343C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063440 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063444 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063448 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006344C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063450 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063454 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063458 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006345C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063460 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063464 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063468 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006346C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063470 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063474 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063478 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006347C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063480 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063484 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063488 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006348C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063490 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063494 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063498 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006349C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000634FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063500 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063504 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063508 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006350C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063510 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063514 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063518 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006351C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063520 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063524 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063528 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006352C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063530 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063534 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063538 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006353C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063540 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063544 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063548 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006354C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063550 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063554 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063558 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006355C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063560 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063564 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063568 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006356C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063570 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063574 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063578 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006357C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063580 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063584 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063588 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006358C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063590 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063594 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00063598 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006359C ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635A4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635A8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635AC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635B0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635B4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635B8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635BC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635C4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635C8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635CC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635D0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635D4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635D8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635DC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635E4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635E8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635EC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635F0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635F4 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635F8 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x000635FC ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x00064004 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064008 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x0006400C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064010 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00064014 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064018 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x0006401C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064020 ) =  0x00000800;
		*mkptr( dfe_cfg_base, 0x00064024 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064028 ) =  0x00000A00;
		*mkptr( dfe_cfg_base, 0x0006402C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064030 ) =  0x00000C00;
		*mkptr( dfe_cfg_base, 0x00064034 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064038 ) =  0x00000E00;
		*mkptr( dfe_cfg_base, 0x0006403C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064040 ) =  0x00001000;
		*mkptr( dfe_cfg_base, 0x00064044 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064048 ) =  0x00001200;
		*mkptr( dfe_cfg_base, 0x0006404C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064050 ) =  0x00001400;
		*mkptr( dfe_cfg_base, 0x00064054 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064058 ) =  0x00001600;
		*mkptr( dfe_cfg_base, 0x0006405C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064060 ) =  0x00001800;
		*mkptr( dfe_cfg_base, 0x00064064 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064068 ) =  0x00001A00;
		*mkptr( dfe_cfg_base, 0x0006406C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064070 ) =  0x00001C00;
		*mkptr( dfe_cfg_base, 0x00064074 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064078 ) =  0x00001E00;
		*mkptr( dfe_cfg_base, 0x0006407C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064080 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x00064084 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064088 ) =  0x00002200;
		*mkptr( dfe_cfg_base, 0x0006408C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064090 ) =  0x00002400;
		*mkptr( dfe_cfg_base, 0x00064094 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064098 ) =  0x00002600;
		*mkptr( dfe_cfg_base, 0x0006409C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640A0 ) =  0x00002800;
		*mkptr( dfe_cfg_base, 0x000640A4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640A8 ) =  0x00002A00;
		*mkptr( dfe_cfg_base, 0x000640AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640B0 ) =  0x00002C00;
		*mkptr( dfe_cfg_base, 0x000640B4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640B8 ) =  0x00002E00;
		*mkptr( dfe_cfg_base, 0x000640BC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640C0 ) =  0x00003000;
		*mkptr( dfe_cfg_base, 0x000640C4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640C8 ) =  0x00003200;
		*mkptr( dfe_cfg_base, 0x000640CC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640D0 ) =  0x00003400;
		*mkptr( dfe_cfg_base, 0x000640D4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640D8 ) =  0x00003600;
		*mkptr( dfe_cfg_base, 0x000640DC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640E0 ) =  0x00003800;
		*mkptr( dfe_cfg_base, 0x000640E4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640E8 ) =  0x00003A00;
		*mkptr( dfe_cfg_base, 0x000640EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640F0 ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x000640F4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000640F8 ) =  0x00003E00;
		*mkptr( dfe_cfg_base, 0x000640FC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064100 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00064104 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064108 ) =  0x00004200;
		*mkptr( dfe_cfg_base, 0x0006410C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064110 ) =  0x00004400;
		*mkptr( dfe_cfg_base, 0x00064114 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064118 ) =  0x00004600;
		*mkptr( dfe_cfg_base, 0x0006411C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064120 ) =  0x00004800;
		*mkptr( dfe_cfg_base, 0x00064124 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064128 ) =  0x00004A00;
		*mkptr( dfe_cfg_base, 0x0006412C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064130 ) =  0x00004C00;
		*mkptr( dfe_cfg_base, 0x00064134 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064138 ) =  0x00004E00;
		*mkptr( dfe_cfg_base, 0x0006413C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064140 ) =  0x00005000;
		*mkptr( dfe_cfg_base, 0x00064144 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064148 ) =  0x00005200;
		*mkptr( dfe_cfg_base, 0x0006414C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064150 ) =  0x00005400;
		*mkptr( dfe_cfg_base, 0x00064154 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064158 ) =  0x00005600;
		*mkptr( dfe_cfg_base, 0x0006415C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064160 ) =  0x00005800;
		*mkptr( dfe_cfg_base, 0x00064164 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064168 ) =  0x00005A00;
		*mkptr( dfe_cfg_base, 0x0006416C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064170 ) =  0x00005C00;
		*mkptr( dfe_cfg_base, 0x00064174 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064178 ) =  0x00005E00;
		*mkptr( dfe_cfg_base, 0x0006417C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064180 ) =  0x00006000;
		*mkptr( dfe_cfg_base, 0x00064184 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064188 ) =  0x00006200;
		*mkptr( dfe_cfg_base, 0x0006418C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064190 ) =  0x00006400;
		*mkptr( dfe_cfg_base, 0x00064194 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064198 ) =  0x00006600;
		*mkptr( dfe_cfg_base, 0x0006419C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641A0 ) =  0x00006800;
		*mkptr( dfe_cfg_base, 0x000641A4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641A8 ) =  0x00006A00;
		*mkptr( dfe_cfg_base, 0x000641AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641B0 ) =  0x00006C00;
		*mkptr( dfe_cfg_base, 0x000641B4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641B8 ) =  0x00006E00;
		*mkptr( dfe_cfg_base, 0x000641BC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641C0 ) =  0x00007000;
		*mkptr( dfe_cfg_base, 0x000641C4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641C8 ) =  0x00007200;
		*mkptr( dfe_cfg_base, 0x000641CC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641D0 ) =  0x00007400;
		*mkptr( dfe_cfg_base, 0x000641D4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641D8 ) =  0x00007600;
		*mkptr( dfe_cfg_base, 0x000641DC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641E0 ) =  0x00007800;
		*mkptr( dfe_cfg_base, 0x000641E4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641E8 ) =  0x00007A00;
		*mkptr( dfe_cfg_base, 0x000641EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641F0 ) =  0x00007C00;
		*mkptr( dfe_cfg_base, 0x000641F4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000641F8 ) =  0x00007E00;
		*mkptr( dfe_cfg_base, 0x000641FC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064200 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00064204 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064208 ) =  0x00008200;
		*mkptr( dfe_cfg_base, 0x0006420C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064210 ) =  0x00008400;
		*mkptr( dfe_cfg_base, 0x00064214 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064218 ) =  0x00008600;
		*mkptr( dfe_cfg_base, 0x0006421C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064220 ) =  0x00008800;
		*mkptr( dfe_cfg_base, 0x00064224 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064228 ) =  0x00008A00;
		*mkptr( dfe_cfg_base, 0x0006422C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064230 ) =  0x00008C00;
		*mkptr( dfe_cfg_base, 0x00064234 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064238 ) =  0x00008E00;
		*mkptr( dfe_cfg_base, 0x0006423C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064240 ) =  0x00009000;
		*mkptr( dfe_cfg_base, 0x00064244 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064248 ) =  0x00009200;
		*mkptr( dfe_cfg_base, 0x0006424C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064250 ) =  0x00009400;
		*mkptr( dfe_cfg_base, 0x00064254 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064258 ) =  0x00009600;
		*mkptr( dfe_cfg_base, 0x0006425C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064260 ) =  0x00009800;
		*mkptr( dfe_cfg_base, 0x00064264 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064268 ) =  0x00009A00;
		*mkptr( dfe_cfg_base, 0x0006426C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064270 ) =  0x00009C00;
		*mkptr( dfe_cfg_base, 0x00064274 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064278 ) =  0x00009E00;
		*mkptr( dfe_cfg_base, 0x0006427C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064280 ) =  0x0000A000;
		*mkptr( dfe_cfg_base, 0x00064284 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064288 ) =  0x0000A200;
		*mkptr( dfe_cfg_base, 0x0006428C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064290 ) =  0x0000A400;
		*mkptr( dfe_cfg_base, 0x00064294 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064298 ) =  0x0000A600;
		*mkptr( dfe_cfg_base, 0x0006429C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642A0 ) =  0x0000A800;
		*mkptr( dfe_cfg_base, 0x000642A4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642A8 ) =  0x0000AA00;
		*mkptr( dfe_cfg_base, 0x000642AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642B0 ) =  0x0000AC00;
		*mkptr( dfe_cfg_base, 0x000642B4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642B8 ) =  0x0000AE00;
		*mkptr( dfe_cfg_base, 0x000642BC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642C0 ) =  0x0000B000;
		*mkptr( dfe_cfg_base, 0x000642C4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642C8 ) =  0x0000B200;
		*mkptr( dfe_cfg_base, 0x000642CC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642D0 ) =  0x0000B400;
		*mkptr( dfe_cfg_base, 0x000642D4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642D8 ) =  0x0000B600;
		*mkptr( dfe_cfg_base, 0x000642DC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642E0 ) =  0x0000B800;
		*mkptr( dfe_cfg_base, 0x000642E4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642E8 ) =  0x0000BA00;
		*mkptr( dfe_cfg_base, 0x000642EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642F0 ) =  0x0000BC00;
		*mkptr( dfe_cfg_base, 0x000642F4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000642F8 ) =  0x0000BE00;
		*mkptr( dfe_cfg_base, 0x000642FC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064300 ) =  0x0000C000;
		*mkptr( dfe_cfg_base, 0x00064304 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064308 ) =  0x0000C200;
		*mkptr( dfe_cfg_base, 0x0006430C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064310 ) =  0x0000C400;
		*mkptr( dfe_cfg_base, 0x00064314 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064318 ) =  0x0000C600;
		*mkptr( dfe_cfg_base, 0x0006431C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064320 ) =  0x0000C800;
		*mkptr( dfe_cfg_base, 0x00064324 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064328 ) =  0x0000CA00;
		*mkptr( dfe_cfg_base, 0x0006432C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064330 ) =  0x0000CC00;
		*mkptr( dfe_cfg_base, 0x00064334 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064338 ) =  0x0000CE00;
		*mkptr( dfe_cfg_base, 0x0006433C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064340 ) =  0x0000D000;
		*mkptr( dfe_cfg_base, 0x00064344 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064348 ) =  0x0000D200;
		*mkptr( dfe_cfg_base, 0x0006434C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064350 ) =  0x0000D400;
		*mkptr( dfe_cfg_base, 0x00064354 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064358 ) =  0x0000D600;
		*mkptr( dfe_cfg_base, 0x0006435C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064360 ) =  0x0000D800;
		*mkptr( dfe_cfg_base, 0x00064364 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064368 ) =  0x0000DA00;
		*mkptr( dfe_cfg_base, 0x0006436C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064370 ) =  0x0000DC00;
		*mkptr( dfe_cfg_base, 0x00064374 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064378 ) =  0x0000DE00;
		*mkptr( dfe_cfg_base, 0x0006437C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064380 ) =  0x0000E000;
		*mkptr( dfe_cfg_base, 0x00064384 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064388 ) =  0x0000E200;
		*mkptr( dfe_cfg_base, 0x0006438C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064390 ) =  0x0000E400;
		*mkptr( dfe_cfg_base, 0x00064394 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00064398 ) =  0x0000E600;
		*mkptr( dfe_cfg_base, 0x0006439C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643A0 ) =  0x0000E800;
		*mkptr( dfe_cfg_base, 0x000643A4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643A8 ) =  0x0000EA00;
		*mkptr( dfe_cfg_base, 0x000643AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643B0 ) =  0x0000EC00;
		*mkptr( dfe_cfg_base, 0x000643B4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643B8 ) =  0x0000EE00;
		*mkptr( dfe_cfg_base, 0x000643BC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643C0 ) =  0x0000F000;
		*mkptr( dfe_cfg_base, 0x000643C4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643C8 ) =  0x0000F200;
		*mkptr( dfe_cfg_base, 0x000643CC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643D0 ) =  0x0000F400;
		*mkptr( dfe_cfg_base, 0x000643D4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643D8 ) =  0x0000F600;
		*mkptr( dfe_cfg_base, 0x000643DC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643E0 ) =  0x0000F800;
		*mkptr( dfe_cfg_base, 0x000643E4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643E8 ) =  0x0000FA00;
		*mkptr( dfe_cfg_base, 0x000643EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643F0 ) =  0x0000FC00;
		*mkptr( dfe_cfg_base, 0x000643F4 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x000643F8 ) =  0x0000FE00;
		*mkptr( dfe_cfg_base, 0x000643FC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x0006C000 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C004 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C008 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C00C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C010 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C020 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C024 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C028 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C02C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C030 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C040 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C044 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C048 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C04C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C050 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C060 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C064 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C068 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C06C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C070 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C080 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C084 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C088 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C08C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C090 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C0A0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C0A4 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C0A8 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C0AC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C0B0 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C0C0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C0C4 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C0C8 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C0CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C0D0 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x0006C0E0 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x0006C0E4 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x0006C0E8 ) =  0x00000055;
		*mkptr( dfe_cfg_base, 0x0006C0EC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x0006C0F0 ) =  0x0000807F;
		*mkptr( dfe_cfg_base, 0x00040200 ) =  0x00000308;
		*mkptr( dfe_cfg_base, 0x00101300 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00101304 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x00101308 ) =  0x000010B1;
		*mkptr( dfe_cfg_base, 0x00101318 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x00101324 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140000 ) =  0x0000DC5C;
		*mkptr( dfe_cfg_base, 0x00140004 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140008 ) =  0x000075EE;
		*mkptr( dfe_cfg_base, 0x00140010 ) =  0x000029B7;
		*mkptr( dfe_cfg_base, 0x00140014 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00140018 ) =  0x00007997;
		*mkptr( dfe_cfg_base, 0x0014001C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00140200 ) =  0x000028AA;
		*mkptr( dfe_cfg_base, 0x00140208 ) =  0x00001FCC;
		*mkptr( dfe_cfg_base, 0x00140210 ) =  0x0000E371;
		*mkptr( dfe_cfg_base, 0x00140214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140218 ) =  0x0000B163;
		*mkptr( dfe_cfg_base, 0x0014021C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140400 ) =  0x00000477;
		*mkptr( dfe_cfg_base, 0x00140408 ) =  0x0000EA10;
		*mkptr( dfe_cfg_base, 0x0014040C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140410 ) =  0x0000E5CA;
		*mkptr( dfe_cfg_base, 0x00140414 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140418 ) =  0x0000050B;
		*mkptr( dfe_cfg_base, 0x00140600 ) =  0x0000F249;
		*mkptr( dfe_cfg_base, 0x00140604 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140608 ) =  0x0000F82E;
		*mkptr( dfe_cfg_base, 0x0014060C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140610 ) =  0x00000A95;
		*mkptr( dfe_cfg_base, 0x00140618 ) =  0x000013DC;
		*mkptr( dfe_cfg_base, 0x00140800 ) =  0x0000FFFB;
		*mkptr( dfe_cfg_base, 0x00140804 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140808 ) =  0x00000883;
		*mkptr( dfe_cfg_base, 0x00140810 ) =  0x000007F1;
		*mkptr( dfe_cfg_base, 0x00140818 ) =  0x0000FC4B;
		*mkptr( dfe_cfg_base, 0x0014081C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140A00 ) =  0x0000049E;
		*mkptr( dfe_cfg_base, 0x00140A08 ) =  0x0000019A;
		*mkptr( dfe_cfg_base, 0x00140A10 ) =  0x0000FB5D;
		*mkptr( dfe_cfg_base, 0x00140A14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140A18 ) =  0x0000F97D;
		*mkptr( dfe_cfg_base, 0x00140A1C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140C00 ) =  0x0000FF48;
		*mkptr( dfe_cfg_base, 0x00140C04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140C08 ) =  0x0000FD1B;
		*mkptr( dfe_cfg_base, 0x00140C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140C10 ) =  0x0000FE1E;
		*mkptr( dfe_cfg_base, 0x00140C14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140C18 ) =  0x0000021E;
		*mkptr( dfe_cfg_base, 0x00140E00 ) =  0x0000FEF7;
		*mkptr( dfe_cfg_base, 0x00140E04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00140E08 ) =  0x0000001B;
		*mkptr( dfe_cfg_base, 0x00140E10 ) =  0x000001A1;
		*mkptr( dfe_cfg_base, 0x00140E18 ) =  0x0000018D;
		*mkptr( dfe_cfg_base, 0x00141000 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x00141008 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x00141010 ) =  0x0000002B;
		*mkptr( dfe_cfg_base, 0x00141018 ) =  0x0000FF32;
		*mkptr( dfe_cfg_base, 0x0014101C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00141200 ) =  0x0000FFF6;
		*mkptr( dfe_cfg_base, 0x00141204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00141208 ) =  0x0000FFE9;
		*mkptr( dfe_cfg_base, 0x0014120C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00141210 ) =  0x0000FFF9;
		*mkptr( dfe_cfg_base, 0x00141214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00141218 ) =  0x00000047;
		*mkptr( dfe_cfg_base, 0x0010126C ) =  0x00000016;
		*mkptr( dfe_cfg_base, 0x00101344 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00101348 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x0010134C ) =  0x00004300;
		*mkptr( dfe_cfg_base, 0x00101350 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00143804 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x0014380C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x00143814 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x0014381C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x001012C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00101200 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00101204 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00101208 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0010120C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00101210 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00101214 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00101218 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0010121C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00101224 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00101228 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00101234 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x0010123C ) =  0x00000643;
		*mkptr( dfe_cfg_base, 0x00101240 ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00101244 ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00101248 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00101250 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00101254 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00101258 ) =  0x00000404;
		*mkptr( dfe_cfg_base, 0x0010125C ) =  0x00000401;
		*mkptr( dfe_cfg_base, 0x00101260 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00101264 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00101268 ) =  0x00002616;
		*mkptr( dfe_cfg_base, 0x001012DC ) =  0x0000B800;
		*mkptr( dfe_cfg_base, 0x00100400 ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x00181300 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00181304 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x00181308 ) =  0x000010B1;
		*mkptr( dfe_cfg_base, 0x00181318 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x00181324 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0000 ) =  0x0000DC5C;
		*mkptr( dfe_cfg_base, 0x001C0004 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0008 ) =  0x000075EE;
		*mkptr( dfe_cfg_base, 0x001C0010 ) =  0x000029B7;
		*mkptr( dfe_cfg_base, 0x001C0014 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x001C0018 ) =  0x00007997;
		*mkptr( dfe_cfg_base, 0x001C001C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x001C0200 ) =  0x000028AA;
		*mkptr( dfe_cfg_base, 0x001C0208 ) =  0x00001FCC;
		*mkptr( dfe_cfg_base, 0x001C0210 ) =  0x0000E371;
		*mkptr( dfe_cfg_base, 0x001C0214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0218 ) =  0x0000B163;
		*mkptr( dfe_cfg_base, 0x001C021C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0400 ) =  0x00000477;
		*mkptr( dfe_cfg_base, 0x001C0408 ) =  0x0000EA10;
		*mkptr( dfe_cfg_base, 0x001C040C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0410 ) =  0x0000E5CA;
		*mkptr( dfe_cfg_base, 0x001C0414 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0418 ) =  0x0000050B;
		*mkptr( dfe_cfg_base, 0x001C0600 ) =  0x0000F249;
		*mkptr( dfe_cfg_base, 0x001C0604 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0608 ) =  0x0000F82E;
		*mkptr( dfe_cfg_base, 0x001C060C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0610 ) =  0x00000A95;
		*mkptr( dfe_cfg_base, 0x001C0618 ) =  0x000013DC;
		*mkptr( dfe_cfg_base, 0x001C0800 ) =  0x0000FFFB;
		*mkptr( dfe_cfg_base, 0x001C0804 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0808 ) =  0x00000883;
		*mkptr( dfe_cfg_base, 0x001C0810 ) =  0x000007F1;
		*mkptr( dfe_cfg_base, 0x001C0818 ) =  0x0000FC4B;
		*mkptr( dfe_cfg_base, 0x001C081C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0A00 ) =  0x0000049E;
		*mkptr( dfe_cfg_base, 0x001C0A08 ) =  0x0000019A;
		*mkptr( dfe_cfg_base, 0x001C0A10 ) =  0x0000FB5D;
		*mkptr( dfe_cfg_base, 0x001C0A14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0A18 ) =  0x0000F97D;
		*mkptr( dfe_cfg_base, 0x001C0A1C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0C00 ) =  0x0000FF48;
		*mkptr( dfe_cfg_base, 0x001C0C04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0C08 ) =  0x0000FD1B;
		*mkptr( dfe_cfg_base, 0x001C0C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0C10 ) =  0x0000FE1E;
		*mkptr( dfe_cfg_base, 0x001C0C14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0C18 ) =  0x0000021E;
		*mkptr( dfe_cfg_base, 0x001C0E00 ) =  0x0000FEF7;
		*mkptr( dfe_cfg_base, 0x001C0E04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C0E08 ) =  0x0000001B;
		*mkptr( dfe_cfg_base, 0x001C0E10 ) =  0x000001A1;
		*mkptr( dfe_cfg_base, 0x001C0E18 ) =  0x0000018D;
		*mkptr( dfe_cfg_base, 0x001C1000 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x001C1008 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x001C1010 ) =  0x0000002B;
		*mkptr( dfe_cfg_base, 0x001C1018 ) =  0x0000FF32;
		*mkptr( dfe_cfg_base, 0x001C101C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C1200 ) =  0x0000FFF6;
		*mkptr( dfe_cfg_base, 0x001C1204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C1208 ) =  0x0000FFE9;
		*mkptr( dfe_cfg_base, 0x001C120C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C1210 ) =  0x0000FFF9;
		*mkptr( dfe_cfg_base, 0x001C1214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x001C1218 ) =  0x00000047;
		*mkptr( dfe_cfg_base, 0x0018126C ) =  0x00000016;
		*mkptr( dfe_cfg_base, 0x00181344 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00181348 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x0018134C ) =  0x00004300;
		*mkptr( dfe_cfg_base, 0x00181350 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x001C3804 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x001C380C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x001C3814 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x001C381C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x001812C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00181200 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00181204 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00181208 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0018120C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00181210 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00181214 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00181218 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0018121C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00181224 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00181228 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x00181234 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x0018123C ) =  0x00000643;
		*mkptr( dfe_cfg_base, 0x00181240 ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00181244 ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00181248 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00181250 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00181254 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00181258 ) =  0x00000404;
		*mkptr( dfe_cfg_base, 0x0018125C ) =  0x00000401;
		*mkptr( dfe_cfg_base, 0x00181260 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00181264 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00181268 ) =  0x00002616;
		*mkptr( dfe_cfg_base, 0x001812DC ) =  0x0000B800;
		*mkptr( dfe_cfg_base, 0x00180400 ) =  0x00004080;
		*mkptr( dfe_cfg_base, 0x00201300 ) =  0x00000307;
		*mkptr( dfe_cfg_base, 0x00201304 ) =  0x00000438;
		*mkptr( dfe_cfg_base, 0x00201308 ) =  0x00002031;
		*mkptr( dfe_cfg_base, 0x00201318 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x00201324 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240000 ) =  0x0000DC5C;
		*mkptr( dfe_cfg_base, 0x00240004 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240008 ) =  0x000075EE;
		*mkptr( dfe_cfg_base, 0x00240010 ) =  0x000029B7;
		*mkptr( dfe_cfg_base, 0x00240014 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00240018 ) =  0x00007997;
		*mkptr( dfe_cfg_base, 0x0024001C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00240200 ) =  0x000028AA;
		*mkptr( dfe_cfg_base, 0x00240208 ) =  0x00001FCC;
		*mkptr( dfe_cfg_base, 0x00240210 ) =  0x0000E371;
		*mkptr( dfe_cfg_base, 0x00240214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240218 ) =  0x0000B163;
		*mkptr( dfe_cfg_base, 0x0024021C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240400 ) =  0x00000477;
		*mkptr( dfe_cfg_base, 0x00240408 ) =  0x0000EA10;
		*mkptr( dfe_cfg_base, 0x0024040C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240410 ) =  0x0000E5CA;
		*mkptr( dfe_cfg_base, 0x00240414 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240418 ) =  0x0000050B;
		*mkptr( dfe_cfg_base, 0x00240600 ) =  0x0000F249;
		*mkptr( dfe_cfg_base, 0x00240604 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240608 ) =  0x0000F82E;
		*mkptr( dfe_cfg_base, 0x0024060C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240610 ) =  0x00000A95;
		*mkptr( dfe_cfg_base, 0x00240618 ) =  0x000013DC;
		*mkptr( dfe_cfg_base, 0x00240800 ) =  0x0000FFFB;
		*mkptr( dfe_cfg_base, 0x00240804 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240808 ) =  0x00000883;
		*mkptr( dfe_cfg_base, 0x00240810 ) =  0x000007F1;
		*mkptr( dfe_cfg_base, 0x00240818 ) =  0x0000FC4B;
		*mkptr( dfe_cfg_base, 0x0024081C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240A00 ) =  0x0000049E;
		*mkptr( dfe_cfg_base, 0x00240A08 ) =  0x0000019A;
		*mkptr( dfe_cfg_base, 0x00240A10 ) =  0x0000FB5D;
		*mkptr( dfe_cfg_base, 0x00240A14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240A18 ) =  0x0000F97D;
		*mkptr( dfe_cfg_base, 0x00240A1C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240C00 ) =  0x0000FF48;
		*mkptr( dfe_cfg_base, 0x00240C04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240C08 ) =  0x0000FD1B;
		*mkptr( dfe_cfg_base, 0x00240C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240C10 ) =  0x0000FE1E;
		*mkptr( dfe_cfg_base, 0x00240C14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240C18 ) =  0x0000021E;
		*mkptr( dfe_cfg_base, 0x00240E00 ) =  0x0000FEF7;
		*mkptr( dfe_cfg_base, 0x00240E04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00240E08 ) =  0x0000001B;
		*mkptr( dfe_cfg_base, 0x00240E10 ) =  0x000001A1;
		*mkptr( dfe_cfg_base, 0x00240E18 ) =  0x0000018D;
		*mkptr( dfe_cfg_base, 0x00241000 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x00241008 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x00241010 ) =  0x0000002B;
		*mkptr( dfe_cfg_base, 0x00241018 ) =  0x0000FF32;
		*mkptr( dfe_cfg_base, 0x0024101C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00241200 ) =  0x0000FFF6;
		*mkptr( dfe_cfg_base, 0x00241204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00241208 ) =  0x0000FFE9;
		*mkptr( dfe_cfg_base, 0x0024120C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00241210 ) =  0x0000FFF9;
		*mkptr( dfe_cfg_base, 0x00241214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00241218 ) =  0x00000047;
		*mkptr( dfe_cfg_base, 0x0020126C ) =  0x00000216;
		*mkptr( dfe_cfg_base, 0x00201344 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00201348 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x0020134C ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x00201350 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00243804 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x0024380C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x00243814 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x0024381C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x002012C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00201200 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00201204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00201208 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0020120C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00201210 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00201214 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00201218 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x0020121C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00201220 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00201224 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00201228 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x0020122C ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00201230 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x00201234 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x0020123C ) =  0x00000643;
		*mkptr( dfe_cfg_base, 0x00201240 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00201244 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00201248 ) =  0x00000401;
		*mkptr( dfe_cfg_base, 0x00201250 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00201254 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00201258 ) =  0x00000404;
		*mkptr( dfe_cfg_base, 0x0020125C ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00201260 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00201264 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00201268 ) =  0x00001606;
		*mkptr( dfe_cfg_base, 0x002012DC ) =  0x0000C800;
		*mkptr( dfe_cfg_base, 0x002012E4 ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x00281300 ) =  0x00000307;
		*mkptr( dfe_cfg_base, 0x00281304 ) =  0x00000438;
		*mkptr( dfe_cfg_base, 0x00281308 ) =  0x00002031;
		*mkptr( dfe_cfg_base, 0x00281318 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x00281324 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0000 ) =  0x0000DC5C;
		*mkptr( dfe_cfg_base, 0x002C0004 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0008 ) =  0x000075EE;
		*mkptr( dfe_cfg_base, 0x002C0010 ) =  0x000029B7;
		*mkptr( dfe_cfg_base, 0x002C0014 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x002C0018 ) =  0x00007997;
		*mkptr( dfe_cfg_base, 0x002C001C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x002C0200 ) =  0x000028AA;
		*mkptr( dfe_cfg_base, 0x002C0208 ) =  0x00001FCC;
		*mkptr( dfe_cfg_base, 0x002C0210 ) =  0x0000E371;
		*mkptr( dfe_cfg_base, 0x002C0214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0218 ) =  0x0000B163;
		*mkptr( dfe_cfg_base, 0x002C021C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0400 ) =  0x00000477;
		*mkptr( dfe_cfg_base, 0x002C0408 ) =  0x0000EA10;
		*mkptr( dfe_cfg_base, 0x002C040C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0410 ) =  0x0000E5CA;
		*mkptr( dfe_cfg_base, 0x002C0414 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0418 ) =  0x0000050B;
		*mkptr( dfe_cfg_base, 0x002C0600 ) =  0x0000F249;
		*mkptr( dfe_cfg_base, 0x002C0604 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0608 ) =  0x0000F82E;
		*mkptr( dfe_cfg_base, 0x002C060C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0610 ) =  0x00000A95;
		*mkptr( dfe_cfg_base, 0x002C0618 ) =  0x000013DC;
		*mkptr( dfe_cfg_base, 0x002C0800 ) =  0x0000FFFB;
		*mkptr( dfe_cfg_base, 0x002C0804 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0808 ) =  0x00000883;
		*mkptr( dfe_cfg_base, 0x002C0810 ) =  0x000007F1;
		*mkptr( dfe_cfg_base, 0x002C0818 ) =  0x0000FC4B;
		*mkptr( dfe_cfg_base, 0x002C081C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0A00 ) =  0x0000049E;
		*mkptr( dfe_cfg_base, 0x002C0A08 ) =  0x0000019A;
		*mkptr( dfe_cfg_base, 0x002C0A10 ) =  0x0000FB5D;
		*mkptr( dfe_cfg_base, 0x002C0A14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0A18 ) =  0x0000F97D;
		*mkptr( dfe_cfg_base, 0x002C0A1C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0C00 ) =  0x0000FF48;
		*mkptr( dfe_cfg_base, 0x002C0C04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0C08 ) =  0x0000FD1B;
		*mkptr( dfe_cfg_base, 0x002C0C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0C10 ) =  0x0000FE1E;
		*mkptr( dfe_cfg_base, 0x002C0C14 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0C18 ) =  0x0000021E;
		*mkptr( dfe_cfg_base, 0x002C0E00 ) =  0x0000FEF7;
		*mkptr( dfe_cfg_base, 0x002C0E04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C0E08 ) =  0x0000001B;
		*mkptr( dfe_cfg_base, 0x002C0E10 ) =  0x000001A1;
		*mkptr( dfe_cfg_base, 0x002C0E18 ) =  0x0000018D;
		*mkptr( dfe_cfg_base, 0x002C1000 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x002C1008 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x002C1010 ) =  0x0000002B;
		*mkptr( dfe_cfg_base, 0x002C1018 ) =  0x0000FF32;
		*mkptr( dfe_cfg_base, 0x002C101C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C1200 ) =  0x0000FFF6;
		*mkptr( dfe_cfg_base, 0x002C1204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C1208 ) =  0x0000FFE9;
		*mkptr( dfe_cfg_base, 0x002C120C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C1210 ) =  0x0000FFF9;
		*mkptr( dfe_cfg_base, 0x002C1214 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x002C1218 ) =  0x00000047;
		*mkptr( dfe_cfg_base, 0x0028126C ) =  0x00000216;
		*mkptr( dfe_cfg_base, 0x00281344 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00281348 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x0028134C ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x00281350 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x002C3804 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x002C380C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x002C3814 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x002C381C ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x002812C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00281200 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00281204 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00281208 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x0028120C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00281210 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00281214 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00281218 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x0028121C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00281220 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00281224 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x00281228 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x0028122C ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00281230 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x00281234 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x0028123C ) =  0x00000643;
		*mkptr( dfe_cfg_base, 0x00281240 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00281244 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00281248 ) =  0x00000401;
		*mkptr( dfe_cfg_base, 0x00281250 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00281254 ) =  0x00000402;
		*mkptr( dfe_cfg_base, 0x00281258 ) =  0x00000404;
		*mkptr( dfe_cfg_base, 0x0028125C ) =  0x00000403;
		*mkptr( dfe_cfg_base, 0x00281260 ) =  0x00000303;
		*mkptr( dfe_cfg_base, 0x00281264 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00281268 ) =  0x00001606;
		*mkptr( dfe_cfg_base, 0x002812DC ) =  0x0000C800;
		*mkptr( dfe_cfg_base, 0x002812E4 ) =  0x00000041;
		*mkptr( dfe_cfg_base, 0x00900004 ) =  0x00000505;
		*mkptr( dfe_cfg_base, 0x00900018 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00900030 ) =  0x00001000;
		*mkptr( dfe_cfg_base, 0x00B003CC ) =  0x00008006;
		*mkptr( dfe_cfg_base, 0x00B00208 ) =  0x000008C0;
		*mkptr( dfe_cfg_base, 0x00B00288 ) =  0x000008C0;
		*mkptr( dfe_cfg_base, 0x00B00360 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B00368 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00B00370 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B00200 ) =  0x00000F0F;
		*mkptr( dfe_cfg_base, 0x00B00280 ) =  0x00000F0F;
		*mkptr( dfe_cfg_base, 0x00B00210 ) =  0x00000606;
		*mkptr( dfe_cfg_base, 0x00B00218 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00B00230 ) =  0x00004040;
		*mkptr( dfe_cfg_base, 0x00B00290 ) =  0x00000606;
		*mkptr( dfe_cfg_base, 0x00B00298 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00B002B0 ) =  0x00004040;
		*mkptr( dfe_cfg_base, 0x00B00300 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B00308 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B00310 ) =  0x00003F5C;
		*mkptr( dfe_cfg_base, 0x00B00318 ) =  0x00003F5C;
		*mkptr( dfe_cfg_base, 0x00B00320 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B00328 ) =  0x00000170;
		*mkptr( dfe_cfg_base, 0x00B00338 ) =  0x00000BB8;
		*mkptr( dfe_cfg_base, 0x00B00340 ) =  0x00000331;
		*mkptr( dfe_cfg_base, 0x00B00348 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00B00378 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B18004 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1800C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18014 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1801C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18024 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1802C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18034 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1803C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18044 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1804C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18054 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1805C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18064 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1806C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18074 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1807C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B18080 ) =  0x0000030C;
		*mkptr( dfe_cfg_base, 0x00B18084 ) =  0x00007FE8;
		*mkptr( dfe_cfg_base, 0x00B18088 ) =  0x00000320;
		*mkptr( dfe_cfg_base, 0x00B1808C ) =  0x00007C17;
		*mkptr( dfe_cfg_base, 0x00B18090 ) =  0x00000334;
		*mkptr( dfe_cfg_base, 0x00B18094 ) =  0x00007899;
		*mkptr( dfe_cfg_base, 0x00B18098 ) =  0x00000340;
		*mkptr( dfe_cfg_base, 0x00B1809C ) =  0x00007562;
		*mkptr( dfe_cfg_base, 0x00B180A0 ) =  0x00000350;
		*mkptr( dfe_cfg_base, 0x00B180A4 ) =  0x0000726A;
		*mkptr( dfe_cfg_base, 0x00B180A8 ) =  0x0000035C;
		*mkptr( dfe_cfg_base, 0x00B180AC ) =  0x00006FA8;
		*mkptr( dfe_cfg_base, 0x00B180B0 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x00B180B4 ) =  0x00006D17;
		*mkptr( dfe_cfg_base, 0x00B180B8 ) =  0x00000370;
		*mkptr( dfe_cfg_base, 0x00B180BC ) =  0x00006AB2;
		*mkptr( dfe_cfg_base, 0x00B180C0 ) =  0x00000378;
		*mkptr( dfe_cfg_base, 0x00B180C4 ) =  0x00006873;
		*mkptr( dfe_cfg_base, 0x00B180C8 ) =  0x00000380;
		*mkptr( dfe_cfg_base, 0x00B180CC ) =  0x00006657;
		*mkptr( dfe_cfg_base, 0x00B180D0 ) =  0x00000388;
		*mkptr( dfe_cfg_base, 0x00B180D4 ) =  0x0000645A;
		*mkptr( dfe_cfg_base, 0x00B180D8 ) =  0x00000390;
		*mkptr( dfe_cfg_base, 0x00B180DC ) =  0x0000627A;
		*mkptr( dfe_cfg_base, 0x00B180E0 ) =  0x00000394;
		*mkptr( dfe_cfg_base, 0x00B180E4 ) =  0x000060B4;
		*mkptr( dfe_cfg_base, 0x00B180E8 ) =  0x00000398;
		*mkptr( dfe_cfg_base, 0x00B180EC ) =  0x00005F06;
		*mkptr( dfe_cfg_base, 0x00B180F0 ) =  0x000003A0;
		*mkptr( dfe_cfg_base, 0x00B180F4 ) =  0x00005D6D;
		*mkptr( dfe_cfg_base, 0x00B180F8 ) =  0x000003A4;
		*mkptr( dfe_cfg_base, 0x00B180FC ) =  0x00005BE8;
		*mkptr( dfe_cfg_base, 0x00B18100 ) =  0x000003A8;
		*mkptr( dfe_cfg_base, 0x00B18104 ) =  0x00005A76;
		*mkptr( dfe_cfg_base, 0x00B18108 ) =  0x000003AC;
		*mkptr( dfe_cfg_base, 0x00B1810C ) =  0x00005914;
		*mkptr( dfe_cfg_base, 0x00B18110 ) =  0x000003B0;
		*mkptr( dfe_cfg_base, 0x00B18114 ) =  0x000057C2;
		*mkptr( dfe_cfg_base, 0x00B18118 ) =  0x000003B4;
		*mkptr( dfe_cfg_base, 0x00B1811C ) =  0x0000567F;
		*mkptr( dfe_cfg_base, 0x00B18120 ) =  0x000003B4;
		*mkptr( dfe_cfg_base, 0x00B18124 ) =  0x00005549;
		*mkptr( dfe_cfg_base, 0x00B18128 ) =  0x000003B8;
		*mkptr( dfe_cfg_base, 0x00B1812C ) =  0x00005420;
		*mkptr( dfe_cfg_base, 0x00B18130 ) =  0x000003BC;
		*mkptr( dfe_cfg_base, 0x00B18134 ) =  0x00005303;
		*mkptr( dfe_cfg_base, 0x00B18138 ) =  0x000003C0;
		*mkptr( dfe_cfg_base, 0x00B1813C ) =  0x000051F1;
		*mkptr( dfe_cfg_base, 0x00B18140 ) =  0x000003C0;
		*mkptr( dfe_cfg_base, 0x00B18144 ) =  0x000050E9;
		*mkptr( dfe_cfg_base, 0x00B18148 ) =  0x000003C4;
		*mkptr( dfe_cfg_base, 0x00B1814C ) =  0x00004FEB;
		*mkptr( dfe_cfg_base, 0x00B18150 ) =  0x000003C4;
		*mkptr( dfe_cfg_base, 0x00B18154 ) =  0x00004EF6;
		*mkptr( dfe_cfg_base, 0x00B18158 ) =  0x000003C8;
		*mkptr( dfe_cfg_base, 0x00B1815C ) =  0x00004E0A;
		*mkptr( dfe_cfg_base, 0x00B18160 ) =  0x000003C8;
		*mkptr( dfe_cfg_base, 0x00B18164 ) =  0x00004D25;
		*mkptr( dfe_cfg_base, 0x00B18168 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B1816C ) =  0x00004C49;
		*mkptr( dfe_cfg_base, 0x00B18170 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B18174 ) =  0x00004B73;
		*mkptr( dfe_cfg_base, 0x00B18178 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B1817C ) =  0x00004AA5;
		*mkptr( dfe_cfg_base, 0x00B18180 ) =  0x000003D0;
		*mkptr( dfe_cfg_base, 0x00B18184 ) =  0x000049DD;
		*mkptr( dfe_cfg_base, 0x00B18188 ) =  0x000003D0;
		*mkptr( dfe_cfg_base, 0x00B1818C ) =  0x0000491B;
		*mkptr( dfe_cfg_base, 0x00B18190 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B18194 ) =  0x0000485F;
		*mkptr( dfe_cfg_base, 0x00B18198 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B1819C ) =  0x000047A8;
		*mkptr( dfe_cfg_base, 0x00B181A0 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B181A4 ) =  0x000046F7;
		*mkptr( dfe_cfg_base, 0x00B181A8 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B181AC ) =  0x0000464B;
		*mkptr( dfe_cfg_base, 0x00B181B0 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B181B4 ) =  0x000045A3;
		*mkptr( dfe_cfg_base, 0x00B181B8 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B181BC ) =  0x00004501;
		*mkptr( dfe_cfg_base, 0x00B181C0 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B181C4 ) =  0x00004462;
		*mkptr( dfe_cfg_base, 0x00B181C8 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B181CC ) =  0x000043C8;
		*mkptr( dfe_cfg_base, 0x00B181D0 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B181D4 ) =  0x00004332;
		*mkptr( dfe_cfg_base, 0x00B181D8 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B181DC ) =  0x0000429F;
		*mkptr( dfe_cfg_base, 0x00B181E0 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B181E4 ) =  0x00004211;
		*mkptr( dfe_cfg_base, 0x00B181E8 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B181EC ) =  0x00004185;
		*mkptr( dfe_cfg_base, 0x00B181F0 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B181F4 ) =  0x000040FE;
		*mkptr( dfe_cfg_base, 0x00B181F8 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B181FC ) =  0x00004079;
		*mkptr( dfe_cfg_base, 0x00B18200 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B18204 ) =  0x00003FF8;
		*mkptr( dfe_cfg_base, 0x00B18208 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B1820C ) =  0x00003F79;
		*mkptr( dfe_cfg_base, 0x00B18210 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B18214 ) =  0x00003EFE;
		*mkptr( dfe_cfg_base, 0x00B18218 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B1821C ) =  0x00003E85;
		*mkptr( dfe_cfg_base, 0x00B18220 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B18224 ) =  0x00003E0F;
		*mkptr( dfe_cfg_base, 0x00B18228 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B1822C ) =  0x00003D9B;
		*mkptr( dfe_cfg_base, 0x00B18230 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B18234 ) =  0x00003D2A;
		*mkptr( dfe_cfg_base, 0x00B18238 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B1823C ) =  0x00003CBB;
		*mkptr( dfe_cfg_base, 0x00B18240 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B18244 ) =  0x00003C4F;
		*mkptr( dfe_cfg_base, 0x00B18248 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B1824C ) =  0x00003BE5;
		*mkptr( dfe_cfg_base, 0x00B18250 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B18254 ) =  0x00003B7D;
		*mkptr( dfe_cfg_base, 0x00B18258 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B1825C ) =  0x00003B17;
		*mkptr( dfe_cfg_base, 0x00B18260 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B18264 ) =  0x00003AB3;
		*mkptr( dfe_cfg_base, 0x00B18268 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B1826C ) =  0x00003A51;
		*mkptr( dfe_cfg_base, 0x00B18270 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B18274 ) =  0x000039F1;
		*mkptr( dfe_cfg_base, 0x00B18278 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B1827C ) =  0x00003993;
		*mkptr( dfe_cfg_base, 0x00B18280 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B18284 ) =  0x00003937;
		*mkptr( dfe_cfg_base, 0x00B18288 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B1828C ) =  0x000038DC;
		*mkptr( dfe_cfg_base, 0x00B18290 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B18294 ) =  0x00003883;
		*mkptr( dfe_cfg_base, 0x00B18298 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B1829C ) =  0x0000382C;
		*mkptr( dfe_cfg_base, 0x00B182A0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182A4 ) =  0x000037D6;
		*mkptr( dfe_cfg_base, 0x00B182A8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182AC ) =  0x00003782;
		*mkptr( dfe_cfg_base, 0x00B182B0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182B4 ) =  0x0000372F;
		*mkptr( dfe_cfg_base, 0x00B182B8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182BC ) =  0x000036DD;
		*mkptr( dfe_cfg_base, 0x00B182C0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182C4 ) =  0x0000368D;
		*mkptr( dfe_cfg_base, 0x00B182C8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182CC ) =  0x0000363F;
		*mkptr( dfe_cfg_base, 0x00B182D0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182D4 ) =  0x000035F1;
		*mkptr( dfe_cfg_base, 0x00B182D8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182DC ) =  0x000035A5;
		*mkptr( dfe_cfg_base, 0x00B182E0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182E4 ) =  0x0000355A;
		*mkptr( dfe_cfg_base, 0x00B182E8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B182EC ) =  0x00003511;
		*mkptr( dfe_cfg_base, 0x00B182F0 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B182F4 ) =  0x000034C8;
		*mkptr( dfe_cfg_base, 0x00B182F8 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B182FC ) =  0x00003481;
		*mkptr( dfe_cfg_base, 0x00B18300 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18304 ) =  0x0000343B;
		*mkptr( dfe_cfg_base, 0x00B18308 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1830C ) =  0x000033F6;
		*mkptr( dfe_cfg_base, 0x00B18310 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18314 ) =  0x000033B2;
		*mkptr( dfe_cfg_base, 0x00B18318 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1831C ) =  0x0000336F;
		*mkptr( dfe_cfg_base, 0x00B18320 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18324 ) =  0x0000332D;
		*mkptr( dfe_cfg_base, 0x00B18328 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1832C ) =  0x000032EC;
		*mkptr( dfe_cfg_base, 0x00B18330 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18334 ) =  0x000032AC;
		*mkptr( dfe_cfg_base, 0x00B18338 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1833C ) =  0x0000326C;
		*mkptr( dfe_cfg_base, 0x00B18340 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18344 ) =  0x0000322E;
		*mkptr( dfe_cfg_base, 0x00B18348 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1834C ) =  0x000031F1;
		*mkptr( dfe_cfg_base, 0x00B18350 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18354 ) =  0x000031B4;
		*mkptr( dfe_cfg_base, 0x00B18358 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1835C ) =  0x00003179;
		*mkptr( dfe_cfg_base, 0x00B18360 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18364 ) =  0x0000313E;
		*mkptr( dfe_cfg_base, 0x00B18368 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B1836C ) =  0x00003104;
		*mkptr( dfe_cfg_base, 0x00B18370 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B18374 ) =  0x000030CB;
		*mkptr( dfe_cfg_base, 0x00B18378 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B1837C ) =  0x00003093;
		*mkptr( dfe_cfg_base, 0x00B18380 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B18384 ) =  0x0000305B;
		*mkptr( dfe_cfg_base, 0x00B18388 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B1838C ) =  0x00003024;
		*mkptr( dfe_cfg_base, 0x00B18390 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B18394 ) =  0x00002FEE;
		*mkptr( dfe_cfg_base, 0x00B18398 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B1839C ) =  0x00002FB8;
		*mkptr( dfe_cfg_base, 0x00B183A0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183A4 ) =  0x00002F84;
		*mkptr( dfe_cfg_base, 0x00B183A8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183AC ) =  0x00002F50;
		*mkptr( dfe_cfg_base, 0x00B183B0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183B4 ) =  0x00002F1C;
		*mkptr( dfe_cfg_base, 0x00B183B8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183BC ) =  0x00002EE9;
		*mkptr( dfe_cfg_base, 0x00B183C0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183C4 ) =  0x00002EB7;
		*mkptr( dfe_cfg_base, 0x00B183C8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183CC ) =  0x00002E86;
		*mkptr( dfe_cfg_base, 0x00B183D0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183D4 ) =  0x00002E55;
		*mkptr( dfe_cfg_base, 0x00B183D8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183DC ) =  0x00002E24;
		*mkptr( dfe_cfg_base, 0x00B183E0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183E4 ) =  0x00002DF5;
		*mkptr( dfe_cfg_base, 0x00B183E8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183EC ) =  0x00002DC6;
		*mkptr( dfe_cfg_base, 0x00B183F0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183F4 ) =  0x00002D97;
		*mkptr( dfe_cfg_base, 0x00B183F8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B183FC ) =  0x00002D69;
		*mkptr( dfe_cfg_base, 0x00B10100 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B10104 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10108 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B1010C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10110 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B10114 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10118 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B1011C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10120 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B10124 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10128 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B1012C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10130 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B10134 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B10138 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B1013C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B1500C ) =  0x00000100;
		*mkptr( dfe_cfg_base, 0x00B15014 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x00B1501C ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x00B15024 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00B1502C ) =  0x00000500;
		*mkptr( dfe_cfg_base, 0x00B15034 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x00B1503C ) =  0x00000700;
		*mkptr( dfe_cfg_base, 0x00B15044 ) =  0x00000800;
		*mkptr( dfe_cfg_base, 0x00B1504C ) =  0x00000900;
		*mkptr( dfe_cfg_base, 0x00B15054 ) =  0x00000A00;
		*mkptr( dfe_cfg_base, 0x00B1505C ) =  0x00000B00;
		*mkptr( dfe_cfg_base, 0x00B15064 ) =  0x00000C00;
		*mkptr( dfe_cfg_base, 0x00B1506C ) =  0x00000D00;
		*mkptr( dfe_cfg_base, 0x00B15074 ) =  0x00000E00;
		*mkptr( dfe_cfg_base, 0x00B1507C ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x00B14000 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B14004 ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B14008 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1400C ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B14010 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B14014 ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B14018 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B1401C ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B14020 ) =  0x00000505;
		*mkptr( dfe_cfg_base, 0x00B14024 ) =  0x00000505;
		*mkptr( dfe_cfg_base, 0x00B14040 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14044 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14048 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1404C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14050 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14054 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14058 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1405C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14060 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14064 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14080 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14084 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14088 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1408C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14090 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14094 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14098 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1409C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B140A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B140C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B140E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B140E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14100 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14104 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14108 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1410C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14110 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14114 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14118 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1411C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14120 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14124 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14140 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14144 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14148 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1414C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14150 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14154 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14158 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1415C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14160 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14164 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14180 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14184 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14188 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1418C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14190 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14194 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14198 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1419C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B141A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B141C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B141E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B141E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14200 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14204 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14208 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1420C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14210 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14214 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14218 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1421C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14220 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14224 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14240 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14244 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14248 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1424C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14250 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14254 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14258 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1425C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14260 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14264 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14280 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14284 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14288 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1428C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14290 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14294 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14298 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1429C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B142A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B142C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B142E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B142E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14300 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14304 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14308 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1430C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14310 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14314 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14318 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1431C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14320 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14324 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14340 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14344 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14348 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1434C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14350 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14354 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14358 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1435C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14360 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14364 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14380 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14384 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14388 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1438C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14390 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14394 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14398 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1439C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B143A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B143C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B143E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B143E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14400 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14404 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14408 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1440C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14410 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14414 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14418 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1441C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14420 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14424 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14440 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14444 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14448 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1444C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14450 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14454 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14458 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1445C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14460 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14464 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14480 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14484 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14488 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1448C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14490 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14494 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14498 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1449C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B144A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B144C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B144E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B144E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14500 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14504 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14508 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1450C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14510 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14514 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14518 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1451C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14520 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14524 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14540 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14544 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14548 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1454C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14550 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14554 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14558 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1455C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14560 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14564 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14580 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14584 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14588 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1458C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14590 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14594 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14598 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1459C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B145A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B145C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B145E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B145E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14600 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14604 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14608 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1460C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14610 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14614 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14618 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1461C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14620 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14624 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14640 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14644 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14648 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1464C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14650 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14654 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14658 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1465C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14660 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14664 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14680 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14684 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14688 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1468C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14690 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14694 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14698 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1469C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B146A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B146C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B146E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B146E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14700 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14704 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14708 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1470C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14710 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14714 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14718 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1471C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14720 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14724 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14740 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14744 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14748 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1474C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14750 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14754 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14758 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1475C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14760 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14764 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B14780 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14784 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14788 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1478C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14790 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14794 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B14798 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B1479C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B147A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B147C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B147E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B147E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B0E800 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A800 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E804 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A804 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E808 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A808 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E80C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A80C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E810 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A810 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E814 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A814 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E818 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A818 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E81C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A81C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E820 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A820 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E824 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A824 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E828 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A828 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E82C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A82C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E830 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A830 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E834 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A834 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E838 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A838 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E83C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A83C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E840 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A840 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E844 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A844 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E848 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A848 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E84C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A84C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E850 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A850 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E854 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A854 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E858 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A858 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E85C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A85C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E860 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A860 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E864 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A864 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E868 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A868 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E86C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A86C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E870 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A870 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E874 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A874 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E878 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A878 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E87C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A87C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E880 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A880 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E884 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A884 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E888 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A888 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E88C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A88C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E890 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A890 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E894 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A894 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E898 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A898 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E89C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A89C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E8FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A8FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E900 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A900 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E904 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A904 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E908 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A908 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E90C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A90C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E910 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A910 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E914 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A914 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E918 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A918 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E91C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A91C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E920 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A920 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E924 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A924 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E928 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A928 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E92C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A92C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E930 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A930 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E934 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A934 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E938 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A938 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E93C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A93C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E940 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A940 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E944 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A944 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E948 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A948 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E94C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A94C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E950 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A950 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E954 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A954 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E958 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A958 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E95C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A95C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E960 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A960 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E964 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A964 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E968 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A968 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E96C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A96C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E970 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A970 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E974 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A974 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E978 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A978 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E97C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A97C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E980 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A980 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E984 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A984 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E988 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A988 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E98C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A98C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E990 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A990 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E994 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A994 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E998 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A998 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E99C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A99C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0E9FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0A9FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EA9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AA9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EABC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AABC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EACC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AACC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EADC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AADC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EAFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AAFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EB9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0AB9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBBC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABBC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBCC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABCC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBDC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABDC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0EBFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B0ABFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B04000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B04004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B04404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B04008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B04408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B0400C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B0440C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B04010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B04410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B04014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B04414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B04018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B04418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0401C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B0441C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B04020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B04420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B04024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B04424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B04028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B04428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B0442C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B04030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B04430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B04034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B04434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B04038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B04438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B0403C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B0443C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B04040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B04440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B04044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B04444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B04448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B0404C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B0444C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B04050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B04450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B04054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B04454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B04058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B04458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B0405C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B0445C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B04060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B04460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B04064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B04464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B04068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B04468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B0406C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B0446C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B04070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B04470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B04074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B04474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B04078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B04478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B0407C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0447C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B04080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B04480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B04084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B04484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B04088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B04488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B0408C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B0448C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B04090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B04490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B04094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B04494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B04098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B04498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B0409C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B0449C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B040A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B044A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B040A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B044A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B040A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B044A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B040AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B044AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B040B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B044B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B040B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B044B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B040B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B044B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B040BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B044BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B040C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B044C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B040C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B044C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B040C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B044C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B040CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B044CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B040D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B044D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B040D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B044D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B040D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B044D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B040DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B044DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B040E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B044E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B040E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B044E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B040E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B044E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B040EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B044EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B040F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B044F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B040F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B044F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B040F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B044F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B040FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B044FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B04100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B04500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B05000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B05004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B05404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B05008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B05408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B0500C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B0540C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B05010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B05410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B05014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B05414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B05018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B05418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0501C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B0541C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B05020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B05420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B05024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B05424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B05028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B05428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B0542C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B05030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B05430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B05034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B05434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B05038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B05438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B0503C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B0543C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B05040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B05440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B05044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B05444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B05448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B0504C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B0544C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B05050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B05450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B05054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B05454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B05058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B05458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B0505C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B0545C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B05060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B05460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B05064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B05464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B05068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B05468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B0506C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B0546C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B05070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B05470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B05074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B05474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B05078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B05478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B0507C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0547C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B05080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B05480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B05084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B05484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B05088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B05488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B0508C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B0548C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B05090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B05490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B05094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B05494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B05098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B05498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B0509C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B0549C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B050A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B054A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B050A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B054A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B050A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B054A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B050AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B054AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B050B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B054B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B050B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B054B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B050B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B054B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B050BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B054BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B050C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B054C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B050C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B054C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B050C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B054C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B050CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B054CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B050D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B054D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B050D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B054D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B050D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B054D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B050DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B054DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B050E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B054E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B050E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B054E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B050E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B054E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B050EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B054EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B050F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B054F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B050F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B054F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B050F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B054F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B050FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B054FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B05100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B05500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B08000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B08004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B08404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B08008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B08408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B0800C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B0840C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B08010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B08410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B08014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B08414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B08018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B08418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0801C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B0841C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B08020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B08420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B08024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B08424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B08028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B08428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B0842C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B08030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B08430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B08034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B08434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B08038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B08438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B0803C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B0843C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B08040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B08440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B08044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B08444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B08448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B0804C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B0844C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B08050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B08450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B08054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B08454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B08058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B08458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B0805C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B0845C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B08060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B08460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B08064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B08464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B08068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B08468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B0806C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B0846C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B08070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B08470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B08074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B08474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B08078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B08478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B0807C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0847C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B08080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B08480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B08084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B08484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B08088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B08488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B0808C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B0848C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B08090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B08490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B08094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B08494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B08098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B08498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B0809C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B0849C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B080A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B084A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B080A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B084A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B080A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B084A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B080AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B084AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B080B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B084B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B080B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B084B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B080B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B084B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B080BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B084BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B080C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B084C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B080C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B084C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B080C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B084C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B080CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B084CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B080D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B084D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B080D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B084D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B080D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B084D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B080DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B084DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B080E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B084E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B080E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B084E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B080E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B084E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B080EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B084EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B080F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B084F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B080F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B084F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B080F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B084F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B080FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B084FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B08100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B08500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B09000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B09004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B09404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B09008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B09408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B0900C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B0940C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B09010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B09410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B09014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B09414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B09018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B09418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0901C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B0941C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B09020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B09420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B09024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B09424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B09028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B09428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B0942C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B09030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B09430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B09034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B09434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B09038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B09438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B0903C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B0943C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B09040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B09440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B09044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B09444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B09448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B0904C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B0944C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B09050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B09450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B09054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B09454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B09058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B09458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B0905C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B0945C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B09060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B09460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B09064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B09464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B09068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B09468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B0906C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B0946C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B09070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B09470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B09074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B09474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B09078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B09478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B0907C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B0947C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B09080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B09480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B09084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B09484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B09088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B09488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B0908C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B0948C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B09090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B09490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B09094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B09494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B09098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B09498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B0909C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B0949C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B090A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B094A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B090A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B094A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B090A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B094A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B090AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B094AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B090B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B094B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B090B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B094B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B090B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B094B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B090BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B094BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B090C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B094C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B090C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B094C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B090C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B094C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B090CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B094CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B090D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B094D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B090D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B094D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B090D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B094D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B090DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B094DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B090E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B094E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B090E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B094E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B090E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B094E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B090EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B094EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B090F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B094F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B090F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B094F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B090F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B094F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B090FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B094FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B09100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B09500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B803CC ) =  0x00008006;
		*mkptr( dfe_cfg_base, 0x00B80208 ) =  0x000008C0;
		*mkptr( dfe_cfg_base, 0x00B80288 ) =  0x000008C0;
		*mkptr( dfe_cfg_base, 0x00B80360 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B80368 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x00B80370 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B80200 ) =  0x00000F0F;
		*mkptr( dfe_cfg_base, 0x00B80280 ) =  0x00000F0F;
		*mkptr( dfe_cfg_base, 0x00B80210 ) =  0x00000606;
		*mkptr( dfe_cfg_base, 0x00B80218 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00B80230 ) =  0x00004040;
		*mkptr( dfe_cfg_base, 0x00B80290 ) =  0x00000606;
		*mkptr( dfe_cfg_base, 0x00B80298 ) =  0x00000707;
		*mkptr( dfe_cfg_base, 0x00B802B0 ) =  0x00004040;
		*mkptr( dfe_cfg_base, 0x00B80300 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B80308 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B80310 ) =  0x00007F5C;
		*mkptr( dfe_cfg_base, 0x00B80318 ) =  0x00007F5C;
		*mkptr( dfe_cfg_base, 0x00B80320 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B80328 ) =  0x00000170;
		*mkptr( dfe_cfg_base, 0x00B80338 ) =  0x00000BB8;
		*mkptr( dfe_cfg_base, 0x00B80340 ) =  0x00000331;
		*mkptr( dfe_cfg_base, 0x00B80348 ) =  0x000000FF;
		*mkptr( dfe_cfg_base, 0x00B80378 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x00B98004 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9800C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98014 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9801C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98024 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9802C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98034 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9803C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98044 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9804C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98054 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9805C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98064 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9806C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98074 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9807C ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B98080 ) =  0x0000030C;
		*mkptr( dfe_cfg_base, 0x00B98084 ) =  0x00007FE8;
		*mkptr( dfe_cfg_base, 0x00B98088 ) =  0x00000320;
		*mkptr( dfe_cfg_base, 0x00B9808C ) =  0x00007C17;
		*mkptr( dfe_cfg_base, 0x00B98090 ) =  0x00000334;
		*mkptr( dfe_cfg_base, 0x00B98094 ) =  0x00007899;
		*mkptr( dfe_cfg_base, 0x00B98098 ) =  0x00000340;
		*mkptr( dfe_cfg_base, 0x00B9809C ) =  0x00007562;
		*mkptr( dfe_cfg_base, 0x00B980A0 ) =  0x00000350;
		*mkptr( dfe_cfg_base, 0x00B980A4 ) =  0x0000726A;
		*mkptr( dfe_cfg_base, 0x00B980A8 ) =  0x0000035C;
		*mkptr( dfe_cfg_base, 0x00B980AC ) =  0x00006FA8;
		*mkptr( dfe_cfg_base, 0x00B980B0 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x00B980B4 ) =  0x00006D17;
		*mkptr( dfe_cfg_base, 0x00B980B8 ) =  0x00000370;
		*mkptr( dfe_cfg_base, 0x00B980BC ) =  0x00006AB2;
		*mkptr( dfe_cfg_base, 0x00B980C0 ) =  0x00000378;
		*mkptr( dfe_cfg_base, 0x00B980C4 ) =  0x00006873;
		*mkptr( dfe_cfg_base, 0x00B980C8 ) =  0x00000380;
		*mkptr( dfe_cfg_base, 0x00B980CC ) =  0x00006657;
		*mkptr( dfe_cfg_base, 0x00B980D0 ) =  0x00000388;
		*mkptr( dfe_cfg_base, 0x00B980D4 ) =  0x0000645A;
		*mkptr( dfe_cfg_base, 0x00B980D8 ) =  0x00000390;
		*mkptr( dfe_cfg_base, 0x00B980DC ) =  0x0000627A;
		*mkptr( dfe_cfg_base, 0x00B980E0 ) =  0x00000394;
		*mkptr( dfe_cfg_base, 0x00B980E4 ) =  0x000060B4;
		*mkptr( dfe_cfg_base, 0x00B980E8 ) =  0x00000398;
		*mkptr( dfe_cfg_base, 0x00B980EC ) =  0x00005F06;
		*mkptr( dfe_cfg_base, 0x00B980F0 ) =  0x000003A0;
		*mkptr( dfe_cfg_base, 0x00B980F4 ) =  0x00005D6D;
		*mkptr( dfe_cfg_base, 0x00B980F8 ) =  0x000003A4;
		*mkptr( dfe_cfg_base, 0x00B980FC ) =  0x00005BE8;
		*mkptr( dfe_cfg_base, 0x00B98100 ) =  0x000003A8;
		*mkptr( dfe_cfg_base, 0x00B98104 ) =  0x00005A76;
		*mkptr( dfe_cfg_base, 0x00B98108 ) =  0x000003AC;
		*mkptr( dfe_cfg_base, 0x00B9810C ) =  0x00005914;
		*mkptr( dfe_cfg_base, 0x00B98110 ) =  0x000003B0;
		*mkptr( dfe_cfg_base, 0x00B98114 ) =  0x000057C2;
		*mkptr( dfe_cfg_base, 0x00B98118 ) =  0x000003B4;
		*mkptr( dfe_cfg_base, 0x00B9811C ) =  0x0000567F;
		*mkptr( dfe_cfg_base, 0x00B98120 ) =  0x000003B4;
		*mkptr( dfe_cfg_base, 0x00B98124 ) =  0x00005549;
		*mkptr( dfe_cfg_base, 0x00B98128 ) =  0x000003B8;
		*mkptr( dfe_cfg_base, 0x00B9812C ) =  0x00005420;
		*mkptr( dfe_cfg_base, 0x00B98130 ) =  0x000003BC;
		*mkptr( dfe_cfg_base, 0x00B98134 ) =  0x00005303;
		*mkptr( dfe_cfg_base, 0x00B98138 ) =  0x000003C0;
		*mkptr( dfe_cfg_base, 0x00B9813C ) =  0x000051F1;
		*mkptr( dfe_cfg_base, 0x00B98140 ) =  0x000003C0;
		*mkptr( dfe_cfg_base, 0x00B98144 ) =  0x000050E9;
		*mkptr( dfe_cfg_base, 0x00B98148 ) =  0x000003C4;
		*mkptr( dfe_cfg_base, 0x00B9814C ) =  0x00004FEB;
		*mkptr( dfe_cfg_base, 0x00B98150 ) =  0x000003C4;
		*mkptr( dfe_cfg_base, 0x00B98154 ) =  0x00004EF6;
		*mkptr( dfe_cfg_base, 0x00B98158 ) =  0x000003C8;
		*mkptr( dfe_cfg_base, 0x00B9815C ) =  0x00004E0A;
		*mkptr( dfe_cfg_base, 0x00B98160 ) =  0x000003C8;
		*mkptr( dfe_cfg_base, 0x00B98164 ) =  0x00004D25;
		*mkptr( dfe_cfg_base, 0x00B98168 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B9816C ) =  0x00004C49;
		*mkptr( dfe_cfg_base, 0x00B98170 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B98174 ) =  0x00004B73;
		*mkptr( dfe_cfg_base, 0x00B98178 ) =  0x000003CC;
		*mkptr( dfe_cfg_base, 0x00B9817C ) =  0x00004AA5;
		*mkptr( dfe_cfg_base, 0x00B98180 ) =  0x000003D0;
		*mkptr( dfe_cfg_base, 0x00B98184 ) =  0x000049DD;
		*mkptr( dfe_cfg_base, 0x00B98188 ) =  0x000003D0;
		*mkptr( dfe_cfg_base, 0x00B9818C ) =  0x0000491B;
		*mkptr( dfe_cfg_base, 0x00B98190 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B98194 ) =  0x0000485F;
		*mkptr( dfe_cfg_base, 0x00B98198 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B9819C ) =  0x000047A8;
		*mkptr( dfe_cfg_base, 0x00B981A0 ) =  0x000003D4;
		*mkptr( dfe_cfg_base, 0x00B981A4 ) =  0x000046F7;
		*mkptr( dfe_cfg_base, 0x00B981A8 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B981AC ) =  0x0000464B;
		*mkptr( dfe_cfg_base, 0x00B981B0 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B981B4 ) =  0x000045A3;
		*mkptr( dfe_cfg_base, 0x00B981B8 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B981BC ) =  0x00004501;
		*mkptr( dfe_cfg_base, 0x00B981C0 ) =  0x000003D8;
		*mkptr( dfe_cfg_base, 0x00B981C4 ) =  0x00004462;
		*mkptr( dfe_cfg_base, 0x00B981C8 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B981CC ) =  0x000043C8;
		*mkptr( dfe_cfg_base, 0x00B981D0 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B981D4 ) =  0x00004332;
		*mkptr( dfe_cfg_base, 0x00B981D8 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B981DC ) =  0x0000429F;
		*mkptr( dfe_cfg_base, 0x00B981E0 ) =  0x000003DC;
		*mkptr( dfe_cfg_base, 0x00B981E4 ) =  0x00004211;
		*mkptr( dfe_cfg_base, 0x00B981E8 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B981EC ) =  0x00004185;
		*mkptr( dfe_cfg_base, 0x00B981F0 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B981F4 ) =  0x000040FE;
		*mkptr( dfe_cfg_base, 0x00B981F8 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B981FC ) =  0x00004079;
		*mkptr( dfe_cfg_base, 0x00B98200 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B98204 ) =  0x00003FF8;
		*mkptr( dfe_cfg_base, 0x00B98208 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B9820C ) =  0x00003F79;
		*mkptr( dfe_cfg_base, 0x00B98210 ) =  0x000003E0;
		*mkptr( dfe_cfg_base, 0x00B98214 ) =  0x00003EFE;
		*mkptr( dfe_cfg_base, 0x00B98218 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B9821C ) =  0x00003E85;
		*mkptr( dfe_cfg_base, 0x00B98220 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B98224 ) =  0x00003E0F;
		*mkptr( dfe_cfg_base, 0x00B98228 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B9822C ) =  0x00003D9B;
		*mkptr( dfe_cfg_base, 0x00B98230 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B98234 ) =  0x00003D2A;
		*mkptr( dfe_cfg_base, 0x00B98238 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B9823C ) =  0x00003CBB;
		*mkptr( dfe_cfg_base, 0x00B98240 ) =  0x000003E4;
		*mkptr( dfe_cfg_base, 0x00B98244 ) =  0x00003C4F;
		*mkptr( dfe_cfg_base, 0x00B98248 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B9824C ) =  0x00003BE5;
		*mkptr( dfe_cfg_base, 0x00B98250 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B98254 ) =  0x00003B7D;
		*mkptr( dfe_cfg_base, 0x00B98258 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B9825C ) =  0x00003B17;
		*mkptr( dfe_cfg_base, 0x00B98260 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B98264 ) =  0x00003AB3;
		*mkptr( dfe_cfg_base, 0x00B98268 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B9826C ) =  0x00003A51;
		*mkptr( dfe_cfg_base, 0x00B98270 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B98274 ) =  0x000039F1;
		*mkptr( dfe_cfg_base, 0x00B98278 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B9827C ) =  0x00003993;
		*mkptr( dfe_cfg_base, 0x00B98280 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B98284 ) =  0x00003937;
		*mkptr( dfe_cfg_base, 0x00B98288 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x00B9828C ) =  0x000038DC;
		*mkptr( dfe_cfg_base, 0x00B98290 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B98294 ) =  0x00003883;
		*mkptr( dfe_cfg_base, 0x00B98298 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B9829C ) =  0x0000382C;
		*mkptr( dfe_cfg_base, 0x00B982A0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982A4 ) =  0x000037D6;
		*mkptr( dfe_cfg_base, 0x00B982A8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982AC ) =  0x00003782;
		*mkptr( dfe_cfg_base, 0x00B982B0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982B4 ) =  0x0000372F;
		*mkptr( dfe_cfg_base, 0x00B982B8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982BC ) =  0x000036DD;
		*mkptr( dfe_cfg_base, 0x00B982C0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982C4 ) =  0x0000368D;
		*mkptr( dfe_cfg_base, 0x00B982C8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982CC ) =  0x0000363F;
		*mkptr( dfe_cfg_base, 0x00B982D0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982D4 ) =  0x000035F1;
		*mkptr( dfe_cfg_base, 0x00B982D8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982DC ) =  0x000035A5;
		*mkptr( dfe_cfg_base, 0x00B982E0 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982E4 ) =  0x0000355A;
		*mkptr( dfe_cfg_base, 0x00B982E8 ) =  0x000003EC;
		*mkptr( dfe_cfg_base, 0x00B982EC ) =  0x00003511;
		*mkptr( dfe_cfg_base, 0x00B982F0 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B982F4 ) =  0x000034C8;
		*mkptr( dfe_cfg_base, 0x00B982F8 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B982FC ) =  0x00003481;
		*mkptr( dfe_cfg_base, 0x00B98300 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98304 ) =  0x0000343B;
		*mkptr( dfe_cfg_base, 0x00B98308 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9830C ) =  0x000033F6;
		*mkptr( dfe_cfg_base, 0x00B98310 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98314 ) =  0x000033B2;
		*mkptr( dfe_cfg_base, 0x00B98318 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9831C ) =  0x0000336F;
		*mkptr( dfe_cfg_base, 0x00B98320 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98324 ) =  0x0000332D;
		*mkptr( dfe_cfg_base, 0x00B98328 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9832C ) =  0x000032EC;
		*mkptr( dfe_cfg_base, 0x00B98330 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98334 ) =  0x000032AC;
		*mkptr( dfe_cfg_base, 0x00B98338 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9833C ) =  0x0000326C;
		*mkptr( dfe_cfg_base, 0x00B98340 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98344 ) =  0x0000322E;
		*mkptr( dfe_cfg_base, 0x00B98348 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9834C ) =  0x000031F1;
		*mkptr( dfe_cfg_base, 0x00B98350 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98354 ) =  0x000031B4;
		*mkptr( dfe_cfg_base, 0x00B98358 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9835C ) =  0x00003179;
		*mkptr( dfe_cfg_base, 0x00B98360 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98364 ) =  0x0000313E;
		*mkptr( dfe_cfg_base, 0x00B98368 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B9836C ) =  0x00003104;
		*mkptr( dfe_cfg_base, 0x00B98370 ) =  0x000003F0;
		*mkptr( dfe_cfg_base, 0x00B98374 ) =  0x000030CB;
		*mkptr( dfe_cfg_base, 0x00B98378 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B9837C ) =  0x00003093;
		*mkptr( dfe_cfg_base, 0x00B98380 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B98384 ) =  0x0000305B;
		*mkptr( dfe_cfg_base, 0x00B98388 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B9838C ) =  0x00003024;
		*mkptr( dfe_cfg_base, 0x00B98390 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B98394 ) =  0x00002FEE;
		*mkptr( dfe_cfg_base, 0x00B98398 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B9839C ) =  0x00002FB8;
		*mkptr( dfe_cfg_base, 0x00B983A0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983A4 ) =  0x00002F84;
		*mkptr( dfe_cfg_base, 0x00B983A8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983AC ) =  0x00002F50;
		*mkptr( dfe_cfg_base, 0x00B983B0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983B4 ) =  0x00002F1C;
		*mkptr( dfe_cfg_base, 0x00B983B8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983BC ) =  0x00002EE9;
		*mkptr( dfe_cfg_base, 0x00B983C0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983C4 ) =  0x00002EB7;
		*mkptr( dfe_cfg_base, 0x00B983C8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983CC ) =  0x00002E86;
		*mkptr( dfe_cfg_base, 0x00B983D0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983D4 ) =  0x00002E55;
		*mkptr( dfe_cfg_base, 0x00B983D8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983DC ) =  0x00002E24;
		*mkptr( dfe_cfg_base, 0x00B983E0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983E4 ) =  0x00002DF5;
		*mkptr( dfe_cfg_base, 0x00B983E8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983EC ) =  0x00002DC6;
		*mkptr( dfe_cfg_base, 0x00B983F0 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983F4 ) =  0x00002D97;
		*mkptr( dfe_cfg_base, 0x00B983F8 ) =  0x000003F4;
		*mkptr( dfe_cfg_base, 0x00B983FC ) =  0x00002D69;
		*mkptr( dfe_cfg_base, 0x00B90100 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B90104 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90108 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B9010C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90110 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B90114 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90118 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B9011C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90120 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B90124 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90128 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B9012C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90130 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B90134 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B90138 ) =  0x00010005;
		*mkptr( dfe_cfg_base, 0x00B9013C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B9500C ) =  0x00000100;
		*mkptr( dfe_cfg_base, 0x00B95014 ) =  0x00000200;
		*mkptr( dfe_cfg_base, 0x00B9501C ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x00B95024 ) =  0x00000400;
		*mkptr( dfe_cfg_base, 0x00B9502C ) =  0x00000500;
		*mkptr( dfe_cfg_base, 0x00B95034 ) =  0x00000600;
		*mkptr( dfe_cfg_base, 0x00B9503C ) =  0x00000700;
		*mkptr( dfe_cfg_base, 0x00B95044 ) =  0x00000800;
		*mkptr( dfe_cfg_base, 0x00B9504C ) =  0x00000900;
		*mkptr( dfe_cfg_base, 0x00B95054 ) =  0x00000A00;
		*mkptr( dfe_cfg_base, 0x00B9505C ) =  0x00000B00;
		*mkptr( dfe_cfg_base, 0x00B95064 ) =  0x00000C00;
		*mkptr( dfe_cfg_base, 0x00B9506C ) =  0x00000D00;
		*mkptr( dfe_cfg_base, 0x00B95074 ) =  0x00000E00;
		*mkptr( dfe_cfg_base, 0x00B9507C ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x00B94000 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B94004 ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B94008 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9400C ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B94010 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B94014 ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B94018 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x00B9401C ) =  0x00003C00;
		*mkptr( dfe_cfg_base, 0x00B94020 ) =  0x00000505;
		*mkptr( dfe_cfg_base, 0x00B94024 ) =  0x00000505;
		*mkptr( dfe_cfg_base, 0x00B94040 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94044 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94048 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9404C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94050 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94054 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94058 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9405C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94060 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94064 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94080 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94084 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94088 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9408C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94090 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94094 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94098 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9409C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B940A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B940C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B940E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B940E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94100 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94104 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94108 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9410C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94110 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94114 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94118 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9411C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94120 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94124 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94140 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94144 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94148 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9414C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94150 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94154 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94158 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9415C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94160 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94164 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94180 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94184 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94188 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9418C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94190 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94194 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94198 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9419C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B941A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B941C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B941E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B941E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94200 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94204 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94208 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9420C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94210 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94214 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94218 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9421C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94220 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94224 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94240 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94244 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94248 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9424C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94250 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94254 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94258 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9425C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94260 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94264 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94280 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94284 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94288 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9428C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94290 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94294 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94298 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9429C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B942A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B942C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B942E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B942E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94300 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94304 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94308 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9430C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94310 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94314 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94318 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9431C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94320 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94324 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94340 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94344 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94348 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9434C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94350 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94354 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94358 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9435C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94360 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94364 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94380 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94384 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94388 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9438C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94390 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94394 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94398 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9439C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B943A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B943C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B943E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B943E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94400 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94404 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94408 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9440C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94410 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94414 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94418 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9441C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94420 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94424 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94440 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94444 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94448 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9444C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94450 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94454 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94458 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9445C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94460 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94464 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94480 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94484 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94488 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9448C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94490 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94494 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94498 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9449C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B944A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B944C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B944E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B944E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94500 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94504 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94508 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9450C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94510 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94514 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94518 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9451C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94520 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94524 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94540 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94544 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94548 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9454C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94550 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94554 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94558 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9455C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94560 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94564 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94580 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94584 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94588 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9458C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94590 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94594 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94598 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9459C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B945A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B945C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B945E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B945E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94600 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94604 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94608 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9460C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94610 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94614 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94618 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9461C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94620 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94624 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94640 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94644 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94648 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9464C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94650 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94654 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94658 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9465C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94660 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94664 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94680 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94684 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94688 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9468C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94690 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94694 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94698 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9469C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B946A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B946C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B946E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B946E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94700 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94704 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94708 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9470C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94710 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94714 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94718 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9471C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94720 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94724 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94740 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94744 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94748 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9474C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94750 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94754 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94758 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9475C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94760 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94764 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B94780 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94784 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94788 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9478C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94790 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94794 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B94798 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B9479C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947A0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B947A4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B947C0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947C4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947C8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947CC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947D0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947D4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947D8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947DC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x00B947E0 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B947E4 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x00B8E800 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A800 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E804 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A804 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E808 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A808 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E80C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A80C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E810 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A810 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E814 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A814 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E818 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A818 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E81C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A81C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E820 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A820 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E824 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A824 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E828 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A828 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E82C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A82C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E830 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A830 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E834 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A834 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E838 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A838 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E83C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A83C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E840 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A840 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E844 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A844 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E848 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A848 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E84C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A84C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E850 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A850 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E854 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A854 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E858 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A858 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E85C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A85C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E860 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A860 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E864 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A864 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E868 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A868 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E86C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A86C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E870 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A870 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E874 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A874 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E878 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A878 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E87C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A87C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E880 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A880 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E884 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A884 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E888 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A888 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E88C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A88C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E890 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A890 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E894 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A894 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E898 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A898 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E89C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A89C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E8FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A8FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E900 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A900 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E904 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A904 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E908 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A908 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E90C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A90C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E910 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A910 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E914 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A914 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E918 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A918 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E91C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A91C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E920 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A920 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E924 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A924 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E928 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A928 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E92C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A92C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E930 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A930 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E934 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A934 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E938 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A938 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E93C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A93C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E940 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A940 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E944 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A944 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E948 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A948 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E94C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A94C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E950 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A950 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E954 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A954 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E958 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A958 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E95C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A95C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E960 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A960 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E964 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A964 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E968 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A968 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E96C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A96C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E970 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A970 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E974 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A974 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E978 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A978 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E97C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A97C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E980 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A980 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E984 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A984 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E988 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A988 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E98C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A98C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E990 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A990 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E994 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A994 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E998 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A998 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E99C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A99C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9A0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9A4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9A8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9AC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9B0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9B4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9B8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9BC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9C0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9C4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9C8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9CC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9D0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9D4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9D8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9DC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9E0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9E4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9E8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9EC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9F0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9F4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9F8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8E9FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8A9FC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EA9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AA9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EABC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AABC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EACC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AACC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EADC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AADC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EAFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AAFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB00 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB04 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB08 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB0C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB10 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB14 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB18 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB1C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB20 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB24 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB28 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB2C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB30 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB34 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB38 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB3C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB40 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB44 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB48 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB4C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB50 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB54 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB58 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB5C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB60 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB64 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB68 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB6C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB70 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB74 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB78 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB7C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB80 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB84 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB88 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB8C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB90 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB94 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB98 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EB9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8AB9C ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABA0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABA4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABA8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABAC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABB0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABB4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABB8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBBC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABBC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABC0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABC4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABC8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBCC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABCC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABD0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABD4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABD8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBDC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABDC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABE0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABE4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABE8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABEC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABF0 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABF4 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABF8 ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8EBFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B8ABFC ) =  0x00000BAD;
		*mkptr( dfe_cfg_base, 0x00B84000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B84004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B84404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B84008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B84408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B8400C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B8440C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B84010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B84410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B84014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B84414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B84018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B84418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8401C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B8441C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B84020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B84420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B84024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B84424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B84028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B84428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B8442C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B84030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B84430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B84034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B84434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B84038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B84438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B8403C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B8443C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B84040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B84440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B84044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B84444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B84448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B8404C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B8444C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B84050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B84450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B84054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B84454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B84058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B84458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B8405C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B8445C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B84060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B84460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B84064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B84464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B84068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B84468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B8406C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B8446C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B84070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B84470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B84074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B84474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B84078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B84478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B8407C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8447C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B84080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B84480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B84084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B84484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B84088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B84488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B8408C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B8448C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B84090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B84490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B84094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B84494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B84098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B84498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B8409C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B8449C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B840A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B844A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B840A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B844A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B840A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B844A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B840AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B844AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B840B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B844B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B840B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B844B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B840B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B844B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B840BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B844BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B840C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B844C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B840C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B844C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B840C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B844C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B840CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B844CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B840D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B844D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B840D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B844D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B840D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B844D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B840DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B844DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B840E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B844E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B840E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B844E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B840E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B844E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B840EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B844EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B840F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B844F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B840F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B844F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B840F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B844F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B840FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B844FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B84100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B84500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B85000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B85004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B85404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B85008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B85408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B8500C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B8540C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B85010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B85410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B85014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B85414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B85018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B85418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8501C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B8541C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B85020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B85420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B85024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B85424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B85028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B85428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B8542C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B85030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B85430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B85034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B85434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B85038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B85438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B8503C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B8543C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B85040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B85440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B85044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B85444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B85448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B8504C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B8544C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B85050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B85450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B85054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B85454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B85058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B85458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B8505C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B8545C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B85060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B85460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B85064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B85464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B85068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B85468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B8506C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B8546C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B85070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B85470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B85074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B85474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B85078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B85478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B8507C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8547C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B85080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B85480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B85084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B85484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B85088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B85488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B8508C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B8548C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B85090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B85490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B85094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B85494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B85098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B85498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B8509C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B8549C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B850A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B854A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B850A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B854A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B850A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B854A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B850AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B854AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B850B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B854B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B850B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B854B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B850B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B854B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B850BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B854BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B850C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B854C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B850C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B854C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B850C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B854C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B850CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B854CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B850D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B854D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B850D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B854D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B850D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B854D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B850DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B854DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B850E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B854E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B850E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B854E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B850E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B854E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B850EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B854EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B850F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B854F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B850F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B854F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B850F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B854F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B850FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B854FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B85100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B85500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B88000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B88004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B88404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B88008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B88408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B8800C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B8840C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B88010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B88410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B88014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B88414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B88018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B88418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8801C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B8841C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B88020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B88420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B88024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B88424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B88028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B88428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B8842C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B88030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B88430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B88034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B88434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B88038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B88438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B8803C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B8843C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B88040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B88440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B88044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B88444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B88448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B8804C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B8844C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B88050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B88450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B88054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B88454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B88058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B88458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B8805C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B8845C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B88060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B88460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B88064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B88464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B88068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B88468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B8806C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B8846C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B88070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B88470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B88074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B88474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B88078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B88478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B8807C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8847C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B88080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B88480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B88084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B88484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B88088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B88488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B8808C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B8848C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B88090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B88490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B88094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B88494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B88098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B88498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B8809C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B8849C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B880A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B884A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B880A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B884A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B880A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B884A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B880AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B884AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B880B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B884B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B880B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B884B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B880B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B884B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B880BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B884BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B880C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B884C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B880C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B884C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B880C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B884C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B880CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B884CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B880D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B884D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B880D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B884D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B880D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B884D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B880DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B884DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B880E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B884E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B880E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B884E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B880E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B884E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B880EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B884EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B880F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B884F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B880F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B884F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B880F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B884F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B880FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B884FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B88100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B88500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x00B89000 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B89004 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B89404 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B89008 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B89408 ) =  0x00000FE7;
		*mkptr( dfe_cfg_base, 0x00B8900C ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B8940C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B89010 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B89410 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B89014 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B89414 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B89018 ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B89418 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8901C ) =  0x00000FE0;
		*mkptr( dfe_cfg_base, 0x00B8941C ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B89020 ) =  0x00000FCF;
		*mkptr( dfe_cfg_base, 0x00B89420 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B89024 ) =  0x00000FD4;
		*mkptr( dfe_cfg_base, 0x00B89424 ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B89028 ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B89428 ) =  0x00000FDE;
		*mkptr( dfe_cfg_base, 0x00B8942C ) =  0x00000FF7;
		*mkptr( dfe_cfg_base, 0x00B89030 ) =  0x00000FFD;
		*mkptr( dfe_cfg_base, 0x00B89430 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B89034 ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B89434 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B89038 ) =  0x00000FC6;
		*mkptr( dfe_cfg_base, 0x00B89438 ) =  0x00000FD3;
		*mkptr( dfe_cfg_base, 0x00B8903C ) =  0x00000FBF;
		*mkptr( dfe_cfg_base, 0x00B8943C ) =  0x00000FBE;
		*mkptr( dfe_cfg_base, 0x00B89040 ) =  0x00000FD5;
		*mkptr( dfe_cfg_base, 0x00B89440 ) =  0x00000FC7;
		*mkptr( dfe_cfg_base, 0x00B89044 ) =  0x00000FF4;
		*mkptr( dfe_cfg_base, 0x00B89444 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B89448 ) =  0x00000FFE;
		*mkptr( dfe_cfg_base, 0x00B8904C ) =  0x00000FEB;
		*mkptr( dfe_cfg_base, 0x00B8944C ) =  0x00000FF9;
		*mkptr( dfe_cfg_base, 0x00B89050 ) =  0x00000FC4;
		*mkptr( dfe_cfg_base, 0x00B89450 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B89054 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B89454 ) =  0x00000FB4;
		*mkptr( dfe_cfg_base, 0x00B89058 ) =  0x00000FBC;
		*mkptr( dfe_cfg_base, 0x00B89458 ) =  0x00000FB0;
		*mkptr( dfe_cfg_base, 0x00B8905C ) =  0x00000FE3;
		*mkptr( dfe_cfg_base, 0x00B8945C ) =  0x00000FCE;
		*mkptr( dfe_cfg_base, 0x00B89060 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B89460 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B89064 ) =  0x00000FF5;
		*mkptr( dfe_cfg_base, 0x00B89464 ) =  0x00000FFF;
		*mkptr( dfe_cfg_base, 0x00B89068 ) =  0x00000FCA;
		*mkptr( dfe_cfg_base, 0x00B89468 ) =  0x00000FE2;
		*mkptr( dfe_cfg_base, 0x00B8906C ) =  0x00000FA3;
		*mkptr( dfe_cfg_base, 0x00B8946C ) =  0x00000FB3;
		*mkptr( dfe_cfg_base, 0x00B89070 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B89470 ) =  0x00000F9C;
		*mkptr( dfe_cfg_base, 0x00B89074 ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B89474 ) =  0x00000FB2;
		*mkptr( dfe_cfg_base, 0x00B89078 ) =  0x00000FFA;
		*mkptr( dfe_cfg_base, 0x00B89478 ) =  0x00000FE5;
		*mkptr( dfe_cfg_base, 0x00B8907C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x00B8947C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x00B89080 ) =  0x00000FD8;
		*mkptr( dfe_cfg_base, 0x00B89480 ) =  0x00000FF2;
		*mkptr( dfe_cfg_base, 0x00B89084 ) =  0x00000F9E;
		*mkptr( dfe_cfg_base, 0x00B89484 ) =  0x00000FB9;
		*mkptr( dfe_cfg_base, 0x00B89088 ) =  0x00000F88;
		*mkptr( dfe_cfg_base, 0x00B89488 ) =  0x00000F8B;
		*mkptr( dfe_cfg_base, 0x00B8908C ) =  0x00000FAD;
		*mkptr( dfe_cfg_base, 0x00B8948C ) =  0x00000F93;
		*mkptr( dfe_cfg_base, 0x00B89090 ) =  0x00000FEE;
		*mkptr( dfe_cfg_base, 0x00B89490 ) =  0x00000FCD;
		*mkptr( dfe_cfg_base, 0x00B89094 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x00B89494 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x00B89098 ) =  0x00000FEF;
		*mkptr( dfe_cfg_base, 0x00B89498 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x00B8909C ) =  0x00000FA4;
		*mkptr( dfe_cfg_base, 0x00B8949C ) =  0x00000FCB;
		*mkptr( dfe_cfg_base, 0x00B890A0 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B894A0 ) =  0x00000F82;
		*mkptr( dfe_cfg_base, 0x00B890A4 ) =  0x00000F86;
		*mkptr( dfe_cfg_base, 0x00B894A4 ) =  0x00000F70;
		*mkptr( dfe_cfg_base, 0x00B890A8 ) =  0x00000FD7;
		*mkptr( dfe_cfg_base, 0x00B894A8 ) =  0x00000FAA;
		*mkptr( dfe_cfg_base, 0x00B890AC ) =  0x0000001D;
		*mkptr( dfe_cfg_base, 0x00B894AC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x00B890B0 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x00B894B0 ) =  0x00000025;
		*mkptr( dfe_cfg_base, 0x00B890B4 ) =  0x00000FBA;
		*mkptr( dfe_cfg_base, 0x00B894B4 ) =  0x00000FED;
		*mkptr( dfe_cfg_base, 0x00B890B8 ) =  0x00000F5C;
		*mkptr( dfe_cfg_base, 0x00B894B8 ) =  0x00000F84;
		*mkptr( dfe_cfg_base, 0x00B890BC ) =  0x00000F51;
		*mkptr( dfe_cfg_base, 0x00B894BC ) =  0x00000F46;
		*mkptr( dfe_cfg_base, 0x00B890C0 ) =  0x00000FAE;
		*mkptr( dfe_cfg_base, 0x00B894C0 ) =  0x00000F73;
		*mkptr( dfe_cfg_base, 0x00B890C4 ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x00B894C4 ) =  0x00000FF0;
		*mkptr( dfe_cfg_base, 0x00B890C8 ) =  0x0000004D;
		*mkptr( dfe_cfg_base, 0x00B894C8 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x00B890CC ) =  0x00000FEC;
		*mkptr( dfe_cfg_base, 0x00B894CC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x00B890D0 ) =  0x00000F4B;
		*mkptr( dfe_cfg_base, 0x00B894D0 ) =  0x00000FA2;
		*mkptr( dfe_cfg_base, 0x00B890D4 ) =  0x00000EFC;
		*mkptr( dfe_cfg_base, 0x00B894D4 ) =  0x00000F16;
		*mkptr( dfe_cfg_base, 0x00B890D8 ) =  0x00000F5B;
		*mkptr( dfe_cfg_base, 0x00B894D8 ) =  0x00000F17;
		*mkptr( dfe_cfg_base, 0x00B890DC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x00B894DC ) =  0x00000FC0;
		*mkptr( dfe_cfg_base, 0x00B890E0 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x00B894E0 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x00B890E4 ) =  0x00000071;
		*mkptr( dfe_cfg_base, 0x00B894E4 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x00B890E8 ) =  0x00000F41;
		*mkptr( dfe_cfg_base, 0x00B894E8 ) =  0x00000FEA;
		*mkptr( dfe_cfg_base, 0x00B890EC ) =  0x00000E1A;
		*mkptr( dfe_cfg_base, 0x00B894EC ) =  0x00000E98;
		*mkptr( dfe_cfg_base, 0x00B890F0 ) =  0x00000E34;
		*mkptr( dfe_cfg_base, 0x00B894F0 ) =  0x00000DEE;
		*mkptr( dfe_cfg_base, 0x00B890F4 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x00B894F4 ) =  0x00000EF8;
		*mkptr( dfe_cfg_base, 0x00B890F8 ) =  0x0000038F;
		*mkptr( dfe_cfg_base, 0x00B894F8 ) =  0x000001CA;
		*mkptr( dfe_cfg_base, 0x00B890FC ) =  0x000006B7;
		*mkptr( dfe_cfg_base, 0x00B894FC ) =  0x00000547;
		*mkptr( dfe_cfg_base, 0x00B89100 ) =  0x000007FF;
		*mkptr( dfe_cfg_base, 0x00B89500 ) =  0x000007AA;
		*mkptr( dfe_cfg_base, 0x01000008 ) =  0x40000001;
		*mkptr( dfe_cfg_base, 0x0100000C ) =  0x40000001;
		*mkptr( dfe_cfg_base, 0x01000010 ) =  0x40000001;
		*mkptr( dfe_cfg_base, 0x01000014 ) =  0x40000001;
		*mkptr( dfe_cfg_base, 0x01000038 ) =  0x000C0003;
		*mkptr( dfe_cfg_base, 0x0100003C ) =  0x0000048A;
		*mkptr( dfe_cfg_base, 0x01000040 ) =  0x005300AA;
		*mkptr( dfe_cfg_base, 0x01000200 ) =  0x0001FFFF;
		*mkptr( dfe_cfg_base, 0x01000404 ) =  0x0001FFFF;
		*mkptr( dfe_cfg_base, 0x01001100 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01001104 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01001108 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x0100110C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01001110 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01001114 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01001118 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01300200 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01300204 ) =  0x00000100;
		*mkptr( dfe_cfg_base, 0x01300208 ) =  0x00000081;
		*mkptr( dfe_cfg_base, 0x0130020C ) =  0x00002200;
		*mkptr( dfe_cfg_base, 0x01300210 ) =  0x07070000;
		*mkptr( dfe_cfg_base, 0x01300214 ) =  0x0000F5F4;
		*mkptr( dfe_cfg_base, 0x01300228 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01300400 ) =  0x00003300;
		*mkptr( dfe_cfg_base, 0x01300408 ) =  0x15000088;
		*mkptr( dfe_cfg_base, 0x0130040C ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300410 ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300414 ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300418 ) =  0x1500018E;
		*mkptr( dfe_cfg_base, 0x0130041C ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x01300420 ) =  0x00003300;
		*mkptr( dfe_cfg_base, 0x01300428 ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x0130042C ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x01300430 ) =  0x150002E8;
		*mkptr( dfe_cfg_base, 0x01300434 ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x01300438 ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x0130043C ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x01300440 ) =  0x00003300;
		*mkptr( dfe_cfg_base, 0x01300448 ) =  0x15000088;
		*mkptr( dfe_cfg_base, 0x0130044C ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300450 ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300454 ) =  0x15003000;
		*mkptr( dfe_cfg_base, 0x01300458 ) =  0x1500018E;
		*mkptr( dfe_cfg_base, 0x0130045C ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x01300460 ) =  0x00003300;
		*mkptr( dfe_cfg_base, 0x01300468 ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x0130046C ) =  0x1500210E;
		*mkptr( dfe_cfg_base, 0x01300470 ) =  0x150002E8;
		*mkptr( dfe_cfg_base, 0x01300474 ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x01300478 ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x0130047C ) =  0x150012E0;
		*mkptr( dfe_cfg_base, 0x01343000 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343008 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343010 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343018 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343020 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343028 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343030 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343038 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343040 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343048 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343050 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343058 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343060 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343068 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343070 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343078 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343080 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343088 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343090 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343098 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013430F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343100 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343108 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343110 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343118 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343120 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343128 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343130 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343138 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343140 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343148 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343150 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343158 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343160 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343168 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343170 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343178 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343180 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343188 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343190 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343198 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013431F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343200 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343208 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343210 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343218 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343220 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343228 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343230 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343238 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343240 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343248 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343250 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343258 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343260 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343268 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343270 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343278 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343280 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343288 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343290 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343298 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013432F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343300 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343308 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343310 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343318 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343320 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343328 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343330 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343338 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343340 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343348 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343350 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343358 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343360 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343368 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343370 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343378 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343380 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343388 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343390 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343398 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013433F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343400 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343408 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343410 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343418 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343420 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343428 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343430 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343438 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343440 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343448 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343450 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343458 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343460 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343468 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343470 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343478 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343480 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343488 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343490 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343498 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013434F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343500 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343508 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343510 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343518 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343520 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343528 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343530 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343538 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343540 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343548 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343550 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343558 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343560 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343568 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343570 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343578 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343580 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343588 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343590 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343598 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013435F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343600 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343608 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343610 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343618 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343620 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343628 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343630 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343638 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343640 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343648 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343650 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343658 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343660 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343668 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343670 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343678 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343680 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343688 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343690 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343698 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013436F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343700 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343708 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343710 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343718 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343720 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343728 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343730 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343738 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343740 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343748 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343750 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343758 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343760 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343768 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343770 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343778 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343780 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343788 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343790 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01343798 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013437F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355000 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355008 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355010 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355018 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355020 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355028 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355030 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355038 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355040 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355048 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355050 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355058 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355060 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355068 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355070 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355078 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355080 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355088 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355090 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355098 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013550F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355100 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355108 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355110 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355118 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355120 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355128 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355130 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355138 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355140 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355148 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355150 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355158 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355160 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355168 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355170 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355178 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355180 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355188 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355190 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355198 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013551F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355200 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355208 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355210 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355218 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355220 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355228 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355230 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355238 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355240 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355248 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355250 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355258 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355260 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355268 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355270 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355278 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355280 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355288 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355290 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355298 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013552F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355300 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355308 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355310 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355318 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355320 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355328 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355330 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355338 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355340 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355348 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355350 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355358 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355360 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355368 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355370 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355378 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355380 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355388 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355390 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355398 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013553F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355400 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355408 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355410 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355418 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355420 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355428 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355430 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355438 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355440 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355448 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355450 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355458 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355460 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355468 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355470 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355478 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355480 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355488 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355490 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355498 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013554F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355500 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355508 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355510 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355518 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355520 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355528 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355530 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355538 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355540 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355548 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355550 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355558 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355560 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355568 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355570 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355578 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355580 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355588 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355590 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355598 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013555F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355600 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355608 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355610 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355618 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355620 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355628 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355630 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355638 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355640 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355648 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355650 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355658 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355660 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355668 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355670 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355678 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355680 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355688 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355690 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355698 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013556F8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355700 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355708 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355710 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355718 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355720 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355728 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355730 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355738 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355740 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355748 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355750 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355758 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355760 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355768 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355770 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355778 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355780 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355788 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355790 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01355798 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557A0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557A8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557B8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557C8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557D8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557E8 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x013557F8 ) =  0x00002000;
    iqn2_dfe_ctl_jesd_loopback_config_split_1();
    iqn2_dfe_ctl_jesd_loopback_config_split_2();

}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_dfe_ctl_jesd_loopback_config_split_1,".text:dfe");
#endif
static void iqn2_dfe_ctl_jesd_loopback_config_split_1()
{
#if DFE_RATE == 20
 		*mkptr( dfe_cfg_base, 0x01600014 ) =  0x04000001;
		*mkptr( dfe_cfg_base, 0x01600018 ) =  0x04000001;
		*mkptr( dfe_cfg_base, 0x0160001C ) =  0x98111100;
		*mkptr( dfe_cfg_base, 0x01600020 ) =  0x09090001;
		*mkptr( dfe_cfg_base, 0x0160002C ) =  0x00002FF8;
		*mkptr( dfe_cfg_base, 0x01600054 ) =  0x04000001;
		*mkptr( dfe_cfg_base, 0x01600058 ) =  0x04000001;
		*mkptr( dfe_cfg_base, 0x0160005C ) =  0x98111100;
		*mkptr( dfe_cfg_base, 0x01600060 ) =  0x09090003;
		*mkptr( dfe_cfg_base, 0x0160006C ) =  0x00002FF8;
		*mkptr( dfe_cfg_base, 0x01600080 ) =  0xFFFFFFCD;
		*mkptr( dfe_cfg_base, 0x01600084 ) =  0xFFFFFF7D;
		*mkptr( dfe_cfg_base, 0x01600088 ) =  0x000000DD;
		*mkptr( dfe_cfg_base, 0x0160008C ) =  0xFFFFFF4B;
		*mkptr( dfe_cfg_base, 0x01600090 ) =  0xFFFFFF71;
		*mkptr( dfe_cfg_base, 0x01600094 ) =  0x000002F3;
		*mkptr( dfe_cfg_base, 0x01600098 ) =  0xFFFFFAA9;
		*mkptr( dfe_cfg_base, 0x0160009C ) =  0x000005A0;
		*mkptr( dfe_cfg_base, 0x016000A0 ) =  0xFFFFFE6D;
		*mkptr( dfe_cfg_base, 0x016000A4 ) =  0xFFFFF80B;
		*mkptr( dfe_cfg_base, 0x016000A8 ) =  0x000015F6;
		*mkptr( dfe_cfg_base, 0x016000AC ) =  0xFFFFDAE4;
		*mkptr( dfe_cfg_base, 0x016000B0 ) =  0x000030DA;
		*mkptr( dfe_cfg_base, 0x016000B4 ) =  0x0000CB69;
		*mkptr( dfe_cfg_base, 0x016000B8 ) =  0x0000CB69;
		*mkptr( dfe_cfg_base, 0x016000C0 ) =  0xFFFFFFF9;
		*mkptr( dfe_cfg_base, 0x016000C4 ) =  0xFFFFFF87;
		*mkptr( dfe_cfg_base, 0x016000C8 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x016000CC ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x016000D0 ) =  0xFFFFFE1F;
		*mkptr( dfe_cfg_base, 0x016000D4 ) =  0x0000026F;
		*mkptr( dfe_cfg_base, 0x016000D8 ) =  0xFFFFFED3;
		*mkptr( dfe_cfg_base, 0x016000DC ) =  0xFFFFFD62;
		*mkptr( dfe_cfg_base, 0x016000E0 ) =  0x0000083C;
		*mkptr( dfe_cfg_base, 0x016000E4 ) =  0xFFFFF2F3;
		*mkptr( dfe_cfg_base, 0x016000E8 ) =  0x00000CC1;
		*mkptr( dfe_cfg_base, 0x016000EC ) =  0xFFFFFE29;
		*mkptr( dfe_cfg_base, 0x016000F0 ) =  0xFFFFE29E;
		*mkptr( dfe_cfg_base, 0x016000F4 ) =  0x00009A36;
		*mkptr( dfe_cfg_base, 0x016000F8 ) =  0x00009A36;
		*mkptr( dfe_cfg_base, 0x01600100 ) =  0xFFFFFFCD;
		*mkptr( dfe_cfg_base, 0x01600104 ) =  0xFFFFFF7D;
		*mkptr( dfe_cfg_base, 0x01600108 ) =  0x000000DD;
		*mkptr( dfe_cfg_base, 0x0160010C ) =  0xFFFFFF4B;
		*mkptr( dfe_cfg_base, 0x01600110 ) =  0xFFFFFF71;
		*mkptr( dfe_cfg_base, 0x01600114 ) =  0x000002F3;
		*mkptr( dfe_cfg_base, 0x01600118 ) =  0xFFFFFAA9;
		*mkptr( dfe_cfg_base, 0x0160011C ) =  0x000005A0;
		*mkptr( dfe_cfg_base, 0x01600120 ) =  0xFFFFFE6D;
		*mkptr( dfe_cfg_base, 0x01600124 ) =  0xFFFFF80B;
		*mkptr( dfe_cfg_base, 0x01600128 ) =  0x000015F6;
		*mkptr( dfe_cfg_base, 0x0160012C ) =  0xFFFFDAE4;
		*mkptr( dfe_cfg_base, 0x01600130 ) =  0x000030DA;
		*mkptr( dfe_cfg_base, 0x01600134 ) =  0x0000CB69;
		*mkptr( dfe_cfg_base, 0x01600138 ) =  0x0000CB69;
		*mkptr( dfe_cfg_base, 0x01600140 ) =  0xFFFFFFF9;
		*mkptr( dfe_cfg_base, 0x01600144 ) =  0xFFFFFF87;
		*mkptr( dfe_cfg_base, 0x01600148 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x0160014C ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01600150 ) =  0xFFFFFE1F;
		*mkptr( dfe_cfg_base, 0x01600154 ) =  0x0000026F;
		*mkptr( dfe_cfg_base, 0x01600158 ) =  0xFFFFFED3;
		*mkptr( dfe_cfg_base, 0x0160015C ) =  0xFFFFFD62;
		*mkptr( dfe_cfg_base, 0x01600160 ) =  0x0000083C;
		*mkptr( dfe_cfg_base, 0x01600164 ) =  0xFFFFF2F3;
		*mkptr( dfe_cfg_base, 0x01600168 ) =  0x00000CC1;
		*mkptr( dfe_cfg_base, 0x0160016C ) =  0xFFFFFE29;
		*mkptr( dfe_cfg_base, 0x01600170 ) =  0xFFFFE29E;
		*mkptr( dfe_cfg_base, 0x01600174 ) =  0x00009A36;
		*mkptr( dfe_cfg_base, 0x01600178 ) =  0x00009A36;
		*mkptr( dfe_cfg_base, 0x016000BC ) =  0x00006149;
		*mkptr( dfe_cfg_base, 0x0160013C ) =  0x00006149;
		*mkptr( dfe_cfg_base, 0x01600200 ) =  0x0001FFFF;
		*mkptr( dfe_cfg_base, 0x01600604 ) =  0x0001FFFF;
		*mkptr( dfe_cfg_base, 0x01600B00 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01600B04 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01600B08 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01600B0C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B10 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B14 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B18 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B1C ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01600B20 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01600B24 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01600B28 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B2C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B30 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600B34 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01600A00 ) =  0x77FFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A04 ) =  0x37FFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A08 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A0C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A10 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A14 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A18 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A1C ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A28 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A2C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A30 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A34 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A80 ) =  0x77FFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A84 ) =  0x37FFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A88 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A8C ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600A90 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A94 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A98 ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600A9C ) =  0xFFFFFFFF;
		*mkptr( dfe_cfg_base, 0x01600AA8 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600AAC ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600AB0 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01600AB4 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01640000 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640004 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640008 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164000C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640010 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640014 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640018 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164001C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640020 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640024 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640028 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164002C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640030 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640034 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640038 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164003C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640040 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640044 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640048 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164004C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640050 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640054 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640058 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164005C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640060 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640064 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640068 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164006C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640070 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640074 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640078 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164007C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640080 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640084 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640088 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164008C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640090 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640094 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640098 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164009C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016400FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640100 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640104 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640108 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164010C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640110 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640114 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640118 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164011C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640120 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640124 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640128 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164012C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640130 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640134 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640138 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164013C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640140 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640144 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640148 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164014C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640150 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640154 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640158 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164015C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640160 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640164 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640168 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164016C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640170 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640174 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640178 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164017C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640180 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640184 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640188 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164018C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640190 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640194 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640198 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164019C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016401FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640200 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640204 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640208 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164020C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640210 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640214 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640218 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164021C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640220 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640224 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640228 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164022C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640230 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640234 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640238 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164023C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640240 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640244 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640248 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164024C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640250 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640254 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640258 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164025C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640260 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640264 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640268 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164026C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640270 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640274 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640278 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164027C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640280 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640284 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640288 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164028C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640290 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640294 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640298 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164029C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016402FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640300 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640304 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640308 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164030C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640310 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640314 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640318 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164031C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640320 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640324 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640328 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164032C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640330 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640334 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640338 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164033C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640340 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640344 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640348 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164034C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640350 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640354 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640358 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164035C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640360 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640364 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640368 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164036C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640370 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640374 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640378 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164037C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640380 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640384 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640388 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164038C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640390 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640394 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640398 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164039C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016403FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640400 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640404 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640408 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164040C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640410 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640414 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640418 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164041C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640420 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640424 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640428 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164042C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640430 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640434 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640438 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164043C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640440 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640444 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640448 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164044C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640450 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640454 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640458 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164045C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640460 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640464 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640468 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164046C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640470 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640474 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640478 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164047C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640480 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640484 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640488 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164048C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640490 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640494 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640498 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164049C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016404FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640500 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640504 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640508 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164050C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640510 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640514 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640518 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164051C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640520 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640524 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640528 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164052C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640530 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640534 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640538 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164053C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640540 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640544 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640548 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164054C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640550 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640554 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640558 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164055C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640560 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640564 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640568 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164056C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640570 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640574 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640578 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164057C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640580 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640584 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640588 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164058C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640590 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640594 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640598 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164059C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016405FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640600 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640604 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640608 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164060C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640610 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640614 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640618 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164061C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640620 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640624 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640628 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164062C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640630 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640634 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640638 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164063C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640640 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640644 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640648 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164064C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640650 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640654 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640658 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164065C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640660 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640664 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640668 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164066C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640670 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640674 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640678 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164067C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640680 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640684 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640688 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164068C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640690 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640694 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640698 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164069C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016406FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640700 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640704 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640708 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164070C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640710 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640714 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640718 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164071C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640720 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640724 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640728 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164072C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640730 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640734 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640738 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164073C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640740 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640744 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640748 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164074C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640750 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640754 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640758 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164075C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640760 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640764 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640768 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164076C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640770 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640774 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640778 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164077C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640780 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640784 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640788 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164078C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640790 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640794 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640798 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164079C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016407FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640800 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640804 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640808 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164080C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640810 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640814 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640818 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164081C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640820 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640824 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640828 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164082C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640830 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640834 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640838 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164083C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640840 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640844 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640848 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164084C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640850 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640854 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640858 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164085C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640860 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640864 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640868 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164086C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640870 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640874 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640878 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164087C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640880 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640884 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640888 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164088C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640890 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640894 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640898 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164089C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016408FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640900 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640904 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640908 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164090C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640910 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640914 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640918 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164091C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640920 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640924 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640928 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164092C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640930 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640934 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640938 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164093C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640940 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640944 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640948 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164094C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640950 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640954 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640958 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164095C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640960 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640964 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640968 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164096C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640970 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640974 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640978 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164097C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640980 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640984 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640988 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164098C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640990 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640994 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640998 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x0164099C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x016409FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A00 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A04 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A08 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A0C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A10 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A14 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A18 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A1C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A20 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A24 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A28 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A2C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A30 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A34 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A38 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A3C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A40 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A44 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A48 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A4C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A50 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A54 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A58 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A5C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A60 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A64 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A68 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A6C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A70 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A74 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A78 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A7C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A80 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A84 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A88 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A8C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A90 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A94 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A98 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640A9C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AA0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AA4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AA8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AAC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AB0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AB4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AB8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640ABC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AC0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AC4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AC8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640ACC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AD0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AD4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AD8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640ADC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AE0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AE4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AE8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AEC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AF0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AF4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AF8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640AFC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B00 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B04 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B08 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B0C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B10 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B14 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B18 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B1C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B20 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B24 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B28 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B2C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B30 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B34 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B38 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B3C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B40 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B44 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B48 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B4C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B50 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B54 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B58 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B5C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B60 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B64 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B68 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B6C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B70 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B74 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B78 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B7C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B80 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B84 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B88 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B8C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B90 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B94 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B98 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640B9C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BA0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BA4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BA8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BAC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BB0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BB4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BB8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BBC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BC0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BC4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BC8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BCC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BD0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BD4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BD8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BDC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BE0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BE4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BE8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BEC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BF0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BF4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BF8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01640BFC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01D00014 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D00020 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D00048 ) =  0x0000EEEE;
		*mkptr( dfe_cfg_base, 0x01D00404 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00408 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0040C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D00410 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00414 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00418 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0041C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D00420 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00444 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00448 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0044C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D00450 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00454 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00458 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0045C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D00460 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00484 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00488 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0048C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D00490 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00494 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00498 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0049C ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D004A0 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D004C4 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D004C8 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D004CC ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D004D0 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D004D4 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D004D8 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D004DC ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D004E0 ) =  0x00005A5A;
		*mkptr( dfe_cfg_base, 0x01D00804 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00808 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0080C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D00844 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00848 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0084C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D00884 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D00888 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D0088C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D008C4 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D008C8 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D008CC ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D00C04 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01D00C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D00C14 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D00C1C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C24 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C2C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C34 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C44 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01D00C4C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D00C54 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D00C5C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C64 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C6C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D00C74 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D01404 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D01408 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x01D0140C ) =  0x00000201;
		*mkptr( dfe_cfg_base, 0x01D01410 ) =  0x00000301;
		*mkptr( dfe_cfg_base, 0x01D01808 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D0180C ) =  0x00001F03;
		*mkptr( dfe_cfg_base, 0x01D01810 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D01814 ) =  0x0000202F;
		*mkptr( dfe_cfg_base, 0x01D01820 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D01824 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D01848 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D0184C ) =  0x00001F03;
		*mkptr( dfe_cfg_base, 0x01D01850 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D01854 ) =  0x0000202F;
		*mkptr( dfe_cfg_base, 0x01D01860 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D01864 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D01C04 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D01C44 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D01C84 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D01CC4 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D02004 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D02408 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01D02410 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01D02414 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D02418 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01D0241C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D02420 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01D02424 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D02428 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x01D0242C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D02430 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x01D02434 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D02438 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x01D0243C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D02440 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x01D02444 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01D02448 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01D0244C ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01D02450 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01D02454 ) =  0x00000009;
		*mkptr( dfe_cfg_base, 0x01D02458 ) =  0x0000000D;
		*mkptr( dfe_cfg_base, 0x01D0245C ) =  0x00000009;
		*mkptr( dfe_cfg_base, 0x01D02460 ) =  0x0000000D;
		*mkptr( dfe_cfg_base, 0x01D02464 ) =  0x0000000A;
		*mkptr( dfe_cfg_base, 0x01D02468 ) =  0x0000000E;
		*mkptr( dfe_cfg_base, 0x01D0246C ) =  0x0000000A;
		*mkptr( dfe_cfg_base, 0x01D02470 ) =  0x0000000E;
		*mkptr( dfe_cfg_base, 0x01D02474 ) =  0x0000000B;
		*mkptr( dfe_cfg_base, 0x01D02478 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01D0247C ) =  0x0000000B;
		*mkptr( dfe_cfg_base, 0x01D02480 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01D02488 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01D02490 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01D02494 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D02498 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01D0249C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D024A0 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01D024A4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D024A8 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x01D024AC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D024B0 ) =  0x00000006;
		*mkptr( dfe_cfg_base, 0x01D024B4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D024B8 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x01D024BC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D024C0 ) =  0x00000007;
		*mkptr( dfe_cfg_base, 0x01D024C4 ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01D024C8 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01D024CC ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01D024D0 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01D024D4 ) =  0x00000009;
		*mkptr( dfe_cfg_base, 0x01D024D8 ) =  0x0000000D;
		*mkptr( dfe_cfg_base, 0x01D024DC ) =  0x00000009;
		*mkptr( dfe_cfg_base, 0x01D024E0 ) =  0x0000000D;
		*mkptr( dfe_cfg_base, 0x01D024E4 ) =  0x0000000A;
		*mkptr( dfe_cfg_base, 0x01D024E8 ) =  0x0000000E;
		*mkptr( dfe_cfg_base, 0x01D024EC ) =  0x0000000A;
		*mkptr( dfe_cfg_base, 0x01D024F0 ) =  0x0000000E;
		*mkptr( dfe_cfg_base, 0x01D024F4 ) =  0x0000000B;
		*mkptr( dfe_cfg_base, 0x01D024F8 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01D024FC ) =  0x0000000B;
		*mkptr( dfe_cfg_base, 0x01D02500 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01D40010 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01D40018 ) =  0x00000103;
		*mkptr( dfe_cfg_base, 0x01D40044 ) =  0x0000EE00;
		*mkptr( dfe_cfg_base, 0x01D40048 ) =  0x0000EEEE;
		*mkptr( dfe_cfg_base, 0x01D40804 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D40808 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D4080C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D40844 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D40848 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D4084C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D40884 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D40888 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D4088C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D408C4 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D408C8 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D408CC ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D40904 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D40908 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D4090C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D40944 ) =  0x0000A5A0;
		*mkptr( dfe_cfg_base, 0x01D40948 ) =  0x0000A5A5;
		*mkptr( dfe_cfg_base, 0x01D4094C ) =  0x0000005A;
		*mkptr( dfe_cfg_base, 0x01D40C04 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01D40C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D40C14 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D40C1C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C24 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C2C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C34 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C44 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01D40C4C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D40C54 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D40C5C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C64 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C6C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D40C74 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41004 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01D4100C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D41014 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D4101C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41024 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D4102C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41034 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41044 ) =  0x00006000;
		*mkptr( dfe_cfg_base, 0x01D4104C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D41054 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D4105C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41064 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D4106C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41074 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41084 ) =  0x00006000;
		*mkptr( dfe_cfg_base, 0x01D4108C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D41094 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D4109C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D410A4 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D410AC ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D410B4 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D41404 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D41408 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x01D4140C ) =  0x00000201;
		*mkptr( dfe_cfg_base, 0x01D41410 ) =  0x00000301;
		*mkptr( dfe_cfg_base, 0x01D41808 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D4180C ) =  0x00001F03;
		*mkptr( dfe_cfg_base, 0x01D41810 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D41814 ) =  0x0000202F;
		*mkptr( dfe_cfg_base, 0x01D41820 ) =  0x00001CE0;
		*mkptr( dfe_cfg_base, 0x01D41824 ) =  0x000003FC;
		*mkptr( dfe_cfg_base, 0x01D41828 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D41848 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D4184C ) =  0x00001F03;
		*mkptr( dfe_cfg_base, 0x01D41850 ) =  0x00000F00;
		*mkptr( dfe_cfg_base, 0x01D41854 ) =  0x0000202F;
		*mkptr( dfe_cfg_base, 0x01D41860 ) =  0x00001CE0;
		*mkptr( dfe_cfg_base, 0x01D41864 ) =  0x000003FC;
		*mkptr( dfe_cfg_base, 0x01D41868 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D41C04 ) =  0x00001FFC;
		*mkptr( dfe_cfg_base, 0x01D41C44 ) =  0x00001FFC;
		*mkptr( dfe_cfg_base, 0x01D41C84 ) =  0x00001FFC;
		*mkptr( dfe_cfg_base, 0x01D41CC4 ) =  0x00001FFC;
		*mkptr( dfe_cfg_base, 0x01D42004 ) =  0x00000300;
		*mkptr( dfe_cfg_base, 0x01D42808 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D42810 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01D42844 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D42848 ) =  0x00000011;
		*mkptr( dfe_cfg_base, 0x01D4284C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01D42850 ) =  0x00000011;
		*mkptr( dfe_cfg_base, 0x01D42884 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D42888 ) =  0x00000021;
		*mkptr( dfe_cfg_base, 0x01D4288C ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01D42890 ) =  0x00000021;
		*mkptr( dfe_cfg_base, 0x01D428C4 ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01D428C8 ) =  0x00000031;
		*mkptr( dfe_cfg_base, 0x01D428CC ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01D428D0 ) =  0x00000031;
		*mkptr( dfe_cfg_base, 0x01D42904 ) =  0x00000100;
		*mkptr( dfe_cfg_base, 0x01D42908 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x01D4290C ) =  0x00000100;
		*mkptr( dfe_cfg_base, 0x01D42910 ) =  0x00000101;
		*mkptr( dfe_cfg_base, 0x01D42944 ) =  0x00000110;
		*mkptr( dfe_cfg_base, 0x01D42948 ) =  0x00000111;
		*mkptr( dfe_cfg_base, 0x01D4294C ) =  0x00000110;
		*mkptr( dfe_cfg_base, 0x01D42950 ) =  0x00000111;
		*mkptr( dfe_cfg_base, 0x01D42984 ) =  0x00000120;
		*mkptr( dfe_cfg_base, 0x01D42988 ) =  0x00000121;
		*mkptr( dfe_cfg_base, 0x01D4298C ) =  0x00000120;
		*mkptr( dfe_cfg_base, 0x01D42990 ) =  0x00000121;
		*mkptr( dfe_cfg_base, 0x01D429C4 ) =  0x00000130;
		*mkptr( dfe_cfg_base, 0x01D429C8 ) =  0x00000131;
		*mkptr( dfe_cfg_base, 0x01D429CC ) =  0x00000130;
		*mkptr( dfe_cfg_base, 0x01D429D0 ) =  0x00000131;
		*mkptr( dfe_cfg_base, 0x01D42A04 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D42A08 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D42A0C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D42A10 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01D42A44 ) =  0x00000012;
		*mkptr( dfe_cfg_base, 0x01D42A48 ) =  0x00000012;
		*mkptr( dfe_cfg_base, 0x01D42A4C ) =  0x00000012;
		*mkptr( dfe_cfg_base, 0x01D42A50 ) =  0x00000012;
		*mkptr( dfe_cfg_base, 0x01D42A84 ) =  0x00000022;
		*mkptr( dfe_cfg_base, 0x01D42A88 ) =  0x00000022;
		*mkptr( dfe_cfg_base, 0x01D42A8C ) =  0x00000022;
		*mkptr( dfe_cfg_base, 0x01D42A90 ) =  0x00000022;
		*mkptr( dfe_cfg_base, 0x01D42AC4 ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x01D42AC8 ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x01D42ACC ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x01D42AD0 ) =  0x00000032;
		*mkptr( dfe_cfg_base, 0x01D42B04 ) =  0x00000102;
		*mkptr( dfe_cfg_base, 0x01D42B08 ) =  0x00000102;
		*mkptr( dfe_cfg_base, 0x01D42B0C ) =  0x00000102;
		*mkptr( dfe_cfg_base, 0x01D42B10 ) =  0x00000102;
		*mkptr( dfe_cfg_base, 0x01D42B44 ) =  0x00000112;
		*mkptr( dfe_cfg_base, 0x01D42B48 ) =  0x00000112;
		*mkptr( dfe_cfg_base, 0x01D42B4C ) =  0x00000112;
		*mkptr( dfe_cfg_base, 0x01D42B50 ) =  0x00000112;
		*mkptr( dfe_cfg_base, 0x01D42B84 ) =  0x00000122;
		*mkptr( dfe_cfg_base, 0x01D42B88 ) =  0x00000122;
		*mkptr( dfe_cfg_base, 0x01D42B8C ) =  0x00000122;
		*mkptr( dfe_cfg_base, 0x01D42B90 ) =  0x00000122;
		*mkptr( dfe_cfg_base, 0x01D42BC4 ) =  0x00000132;
		*mkptr( dfe_cfg_base, 0x01D42BC8 ) =  0x00000132;
		*mkptr( dfe_cfg_base, 0x01D42BCC ) =  0x00000132;
		*mkptr( dfe_cfg_base, 0x01D42BD0 ) =  0x00000132;
		*mkptr( dfe_cfg_base, 0x01D42C04 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D42C08 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D42C0C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D42C10 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01D42C44 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x01D42C48 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x01D42C4C ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x01D42C50 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x01D42C84 ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x01D42C88 ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x01D42C8C ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x01D42C90 ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x01D42CC4 ) =  0x00000033;
		*mkptr( dfe_cfg_base, 0x01D42CC8 ) =  0x00000033;
		*mkptr( dfe_cfg_base, 0x01D42CCC ) =  0x00000033;
		*mkptr( dfe_cfg_base, 0x01D42CD0 ) =  0x00000033;
		*mkptr( dfe_cfg_base, 0x01D42D04 ) =  0x00000103;
		*mkptr( dfe_cfg_base, 0x01D42D08 ) =  0x00000103;
		*mkptr( dfe_cfg_base, 0x01D42D0C ) =  0x00000103;
		*mkptr( dfe_cfg_base, 0x01D42D10 ) =  0x00000103;
		*mkptr( dfe_cfg_base, 0x01D42D44 ) =  0x00000113;
		*mkptr( dfe_cfg_base, 0x01D42D48 ) =  0x00000113;
		*mkptr( dfe_cfg_base, 0x01D42D4C ) =  0x00000113;
		*mkptr( dfe_cfg_base, 0x01D42D50 ) =  0x00000113;
		*mkptr( dfe_cfg_base, 0x01D42D84 ) =  0x00000123;
		*mkptr( dfe_cfg_base, 0x01D42D88 ) =  0x00000123;
		*mkptr( dfe_cfg_base, 0x01D42D8C ) =  0x00000123;
		*mkptr( dfe_cfg_base, 0x01D42D90 ) =  0x00000123;
		*mkptr( dfe_cfg_base, 0x01D42DC4 ) =  0x00000133;
		*mkptr( dfe_cfg_base, 0x01D42DC8 ) =  0x00000133;
		*mkptr( dfe_cfg_base, 0x01D42DCC ) =  0x00000133;
		*mkptr( dfe_cfg_base, 0x01D42DD0 ) =  0x00000133;
#endif
}
#ifdef _TMS320C6X
#pragma CODE_SECTION(iqn2_dfe_ctl_jesd_loopback_config_split_2,".text:dfe");
#endif
static void iqn2_dfe_ctl_jesd_loopback_config_split_2()
{
#if DFE_RATE == 20
   		*mkptr( dfe_cfg_base, 0x01A00E00 ) =  0x00000470;
		*mkptr( dfe_cfg_base, 0x01A00200 ) =  0x00000221;
		*mkptr( dfe_cfg_base, 0x01A00204 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01A006A0 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A0022C ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A00230 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A41000 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41040 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41080 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A410C0 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41100 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x01A41108 ) =  0x000007D0;
		*mkptr( dfe_cfg_base, 0x01A007EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A007F0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A007FC ) =  0x00003000;
		*mkptr( dfe_cfg_base, 0x01A00604 ) =  0x0000FF00;
		*mkptr( dfe_cfg_base, 0x01A00648 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01A41800 ) =  0x0000007F;
		*mkptr( dfe_cfg_base, 0x01A41804 ) =  0x0000006F;
		*mkptr( dfe_cfg_base, 0x01A41808 ) =  0x00000062;
		*mkptr( dfe_cfg_base, 0x01A4180C ) =  0x00000057;
		*mkptr( dfe_cfg_base, 0x01A41810 ) =  0x0000004F;
		*mkptr( dfe_cfg_base, 0x01A41814 ) =  0x00000047;
		*mkptr( dfe_cfg_base, 0x01A41818 ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x01A4181C ) =  0x0000003A;
		*mkptr( dfe_cfg_base, 0x01A41820 ) =  0x00000035;
		*mkptr( dfe_cfg_base, 0x01A41824 ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01A41828 ) =  0x0000002B;
		*mkptr( dfe_cfg_base, 0x01A4182C ) =  0x00000027;
		*mkptr( dfe_cfg_base, 0x01A41830 ) =  0x00000023;
		*mkptr( dfe_cfg_base, 0x01A41834 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01A41838 ) =  0x0000001C;
		*mkptr( dfe_cfg_base, 0x01A4183C ) =  0x00000019;
		*mkptr( dfe_cfg_base, 0x01A41840 ) =  0x00000016;
		*mkptr( dfe_cfg_base, 0x01A41844 ) =  0x00000013;
		*mkptr( dfe_cfg_base, 0x01A41848 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A4184C ) =  0x0000000E;
		*mkptr( dfe_cfg_base, 0x01A41850 ) =  0x0000000B;
		*mkptr( dfe_cfg_base, 0x01A41854 ) =  0x00000009;
		*mkptr( dfe_cfg_base, 0x01A41874 ) =  0x000000F9;
		*mkptr( dfe_cfg_base, 0x01A41878 ) =  0x000000F7;
		*mkptr( dfe_cfg_base, 0x01A4187C ) =  0x000000F5;
		*mkptr( dfe_cfg_base, 0x01A41880 ) =  0x000000F4;
		*mkptr( dfe_cfg_base, 0x01A41884 ) =  0x000000F2;
		*mkptr( dfe_cfg_base, 0x01A41888 ) =  0x000000F1;
		*mkptr( dfe_cfg_base, 0x01A4188C ) =  0x000000EF;
		*mkptr( dfe_cfg_base, 0x01A41890 ) =  0x000000EE;
		*mkptr( dfe_cfg_base, 0x01A41894 ) =  0x000000EC;
		*mkptr( dfe_cfg_base, 0x01A41898 ) =  0x000000EB;
		*mkptr( dfe_cfg_base, 0x01A4189C ) =  0x000000EA;
		*mkptr( dfe_cfg_base, 0x01A418A0 ) =  0x000000E8;
		*mkptr( dfe_cfg_base, 0x01A418A4 ) =  0x000000E7;
		*mkptr( dfe_cfg_base, 0x01A418A8 ) =  0x000000E6;
		*mkptr( dfe_cfg_base, 0x01A418AC ) =  0x000000E5;
		*mkptr( dfe_cfg_base, 0x01A418B0 ) =  0x000000E3;
		*mkptr( dfe_cfg_base, 0x01A418B4 ) =  0x000000E2;
		*mkptr( dfe_cfg_base, 0x01A418B8 ) =  0x000000E1;
		*mkptr( dfe_cfg_base, 0x01A418BC ) =  0x000000E0;
		*mkptr( dfe_cfg_base, 0x01A418C0 ) =  0x000000DF;
		*mkptr( dfe_cfg_base, 0x01A418C4 ) =  0x000000DE;
		*mkptr( dfe_cfg_base, 0x01A418C8 ) =  0x000000DD;
		*mkptr( dfe_cfg_base, 0x01A418CC ) =  0x000000DC;
		*mkptr( dfe_cfg_base, 0x01A418D0 ) =  0x000000DB;
		*mkptr( dfe_cfg_base, 0x01A418D4 ) =  0x000000DA;
		*mkptr( dfe_cfg_base, 0x01A418D8 ) =  0x000000D9;
		*mkptr( dfe_cfg_base, 0x01A418DC ) =  0x000000D8;
		*mkptr( dfe_cfg_base, 0x01A418E0 ) =  0x000000D7;
		*mkptr( dfe_cfg_base, 0x01A418E4 ) =  0x000000D6;
		*mkptr( dfe_cfg_base, 0x01A418E8 ) =  0x000000D5;
		*mkptr( dfe_cfg_base, 0x01A418EC ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x01A418F0 ) =  0x000000D3;
		*mkptr( dfe_cfg_base, 0x01A418F4 ) =  0x000000D2;
		*mkptr( dfe_cfg_base, 0x01A418F8 ) =  0x000000D1;
		*mkptr( dfe_cfg_base, 0x01A418FC ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A41900 ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A41904 ) =  0x000000CF;
		*mkptr( dfe_cfg_base, 0x01A41908 ) =  0x000000CE;
		*mkptr( dfe_cfg_base, 0x01A4190C ) =  0x000000CD;
		*mkptr( dfe_cfg_base, 0x01A41910 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A41914 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A41918 ) =  0x000000CB;
		*mkptr( dfe_cfg_base, 0x01A4191C ) =  0x000000CA;
		*mkptr( dfe_cfg_base, 0x01A41920 ) =  0x000000C9;
		*mkptr( dfe_cfg_base, 0x01A41924 ) =  0x000000C9;
		*mkptr( dfe_cfg_base, 0x01A41928 ) =  0x000000C8;
		*mkptr( dfe_cfg_base, 0x01A4192C ) =  0x000000C7;
		*mkptr( dfe_cfg_base, 0x01A41930 ) =  0x000000C6;
		*mkptr( dfe_cfg_base, 0x01A41934 ) =  0x000000C6;
		*mkptr( dfe_cfg_base, 0x01A41938 ) =  0x000000C5;
		*mkptr( dfe_cfg_base, 0x01A4193C ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A41940 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A41944 ) =  0x000000C3;
		*mkptr( dfe_cfg_base, 0x01A41948 ) =  0x000000C2;
		*mkptr( dfe_cfg_base, 0x01A4194C ) =  0x000000C2;
		*mkptr( dfe_cfg_base, 0x01A41950 ) =  0x000000C1;
		*mkptr( dfe_cfg_base, 0x01A41954 ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A41958 ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A4195C ) =  0x000000BF;
		*mkptr( dfe_cfg_base, 0x01A41960 ) =  0x000000BE;
		*mkptr( dfe_cfg_base, 0x01A41964 ) =  0x000000BE;
		*mkptr( dfe_cfg_base, 0x01A41968 ) =  0x000000BD;
		*mkptr( dfe_cfg_base, 0x01A4196C ) =  0x000000BD;
		*mkptr( dfe_cfg_base, 0x01A41970 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A41974 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A41978 ) =  0x000000BB;
		*mkptr( dfe_cfg_base, 0x01A4197C ) =  0x000000BA;
		*mkptr( dfe_cfg_base, 0x01A41980 ) =  0x000000BA;
		*mkptr( dfe_cfg_base, 0x01A41984 ) =  0x000000B9;
		*mkptr( dfe_cfg_base, 0x01A41988 ) =  0x000000B9;
		*mkptr( dfe_cfg_base, 0x01A4198C ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A41990 ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A41994 ) =  0x000000B7;
		*mkptr( dfe_cfg_base, 0x01A41998 ) =  0x000000B7;
		*mkptr( dfe_cfg_base, 0x01A4199C ) =  0x000000B6;
		*mkptr( dfe_cfg_base, 0x01A419A0 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x01A419A4 ) =  0x000000B5;
		*mkptr( dfe_cfg_base, 0x01A419A8 ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A419AC ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A419B0 ) =  0x000000B3;
		*mkptr( dfe_cfg_base, 0x01A419B4 ) =  0x000000B3;
		*mkptr( dfe_cfg_base, 0x01A419B8 ) =  0x000000B2;
		*mkptr( dfe_cfg_base, 0x01A419BC ) =  0x000000B2;
		*mkptr( dfe_cfg_base, 0x01A419C0 ) =  0x000000B1;
		*mkptr( dfe_cfg_base, 0x01A419C4 ) =  0x000000B1;
		*mkptr( dfe_cfg_base, 0x01A419C8 ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A419CC ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A419D0 ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A419D4 ) =  0x000000AF;
		*mkptr( dfe_cfg_base, 0x01A419D8 ) =  0x000000AF;
		*mkptr( dfe_cfg_base, 0x01A419DC ) =  0x000000AE;
		*mkptr( dfe_cfg_base, 0x01A419E0 ) =  0x000000AE;
		*mkptr( dfe_cfg_base, 0x01A419E4 ) =  0x000000AD;
		*mkptr( dfe_cfg_base, 0x01A419E8 ) =  0x000000AD;
		*mkptr( dfe_cfg_base, 0x01A419EC ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A419F0 ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A419F4 ) =  0x000000AB;
		*mkptr( dfe_cfg_base, 0x01A419F8 ) =  0x000000AB;
		*mkptr( dfe_cfg_base, 0x01A419FC ) =  0x000000AB;
		*mkptr( dfe_cfg_base, 0x01A41A00 ) =  0x000000AA;
		*mkptr( dfe_cfg_base, 0x01A41A04 ) =  0x000000AA;
		*mkptr( dfe_cfg_base, 0x01A41A08 ) =  0x000000A9;
		*mkptr( dfe_cfg_base, 0x01A41A0C ) =  0x000000A9;
		*mkptr( dfe_cfg_base, 0x01A41A10 ) =  0x000000A9;
		*mkptr( dfe_cfg_base, 0x01A41A14 ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A41A18 ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A41A1C ) =  0x000000A7;
		*mkptr( dfe_cfg_base, 0x01A41A20 ) =  0x000000A7;
		*mkptr( dfe_cfg_base, 0x01A41A24 ) =  0x000000A6;
		*mkptr( dfe_cfg_base, 0x01A41A28 ) =  0x000000A6;
		*mkptr( dfe_cfg_base, 0x01A41A2C ) =  0x000000A6;
		*mkptr( dfe_cfg_base, 0x01A41A30 ) =  0x000000A5;
		*mkptr( dfe_cfg_base, 0x01A41A34 ) =  0x000000A5;
		*mkptr( dfe_cfg_base, 0x01A41A38 ) =  0x000000A5;
		*mkptr( dfe_cfg_base, 0x01A41A3C ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A41A40 ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A41A44 ) =  0x000000A3;
		*mkptr( dfe_cfg_base, 0x01A41A48 ) =  0x000000A3;
		*mkptr( dfe_cfg_base, 0x01A41A4C ) =  0x000000A3;
		*mkptr( dfe_cfg_base, 0x01A41A50 ) =  0x000000A2;
		*mkptr( dfe_cfg_base, 0x01A41A54 ) =  0x000000A2;
		*mkptr( dfe_cfg_base, 0x01A41A58 ) =  0x000000A2;
		*mkptr( dfe_cfg_base, 0x01A41A5C ) =  0x000000A1;
		*mkptr( dfe_cfg_base, 0x01A41A60 ) =  0x000000A1;
		*mkptr( dfe_cfg_base, 0x01A41A64 ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A41A68 ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A41A6C ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A41A70 ) =  0x0000009F;
		*mkptr( dfe_cfg_base, 0x01A41A74 ) =  0x0000009F;
		*mkptr( dfe_cfg_base, 0x01A41A78 ) =  0x0000009F;
		*mkptr( dfe_cfg_base, 0x01A41A7C ) =  0x0000009E;
		*mkptr( dfe_cfg_base, 0x01A41A80 ) =  0x0000009E;
		*mkptr( dfe_cfg_base, 0x01A41A84 ) =  0x0000009E;
		*mkptr( dfe_cfg_base, 0x01A41A88 ) =  0x0000009D;
		*mkptr( dfe_cfg_base, 0x01A41A8C ) =  0x0000009D;
		*mkptr( dfe_cfg_base, 0x01A41A90 ) =  0x0000009D;
		*mkptr( dfe_cfg_base, 0x01A41A94 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A41A98 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A41A9C ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A41AA0 ) =  0x0000009B;
		*mkptr( dfe_cfg_base, 0x01A41AA4 ) =  0x0000009B;
		*mkptr( dfe_cfg_base, 0x01A41AA8 ) =  0x0000009B;
		*mkptr( dfe_cfg_base, 0x01A41AAC ) =  0x0000009A;
		*mkptr( dfe_cfg_base, 0x01A41AB0 ) =  0x0000009A;
		*mkptr( dfe_cfg_base, 0x01A41AB4 ) =  0x0000009A;
		*mkptr( dfe_cfg_base, 0x01A41AB8 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x01A41ABC ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x01A41AC0 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x01A41AC4 ) =  0x00000099;
		*mkptr( dfe_cfg_base, 0x01A41AC8 ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A41ACC ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A41AD0 ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A41AD4 ) =  0x00000097;
		*mkptr( dfe_cfg_base, 0x01A41AD8 ) =  0x00000097;
		*mkptr( dfe_cfg_base, 0x01A41ADC ) =  0x00000097;
		*mkptr( dfe_cfg_base, 0x01A41AE0 ) =  0x00000096;
		*mkptr( dfe_cfg_base, 0x01A41AE4 ) =  0x00000096;
		*mkptr( dfe_cfg_base, 0x01A41AE8 ) =  0x00000096;
		*mkptr( dfe_cfg_base, 0x01A41AEC ) =  0x00000095;
		*mkptr( dfe_cfg_base, 0x01A41AF0 ) =  0x00000095;
		*mkptr( dfe_cfg_base, 0x01A41AF4 ) =  0x00000095;
		*mkptr( dfe_cfg_base, 0x01A41AF8 ) =  0x00000095;
		*mkptr( dfe_cfg_base, 0x01A41AFC ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A41B00 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A41B04 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A41B08 ) =  0x00000093;
		*mkptr( dfe_cfg_base, 0x01A41B0C ) =  0x00000093;
		*mkptr( dfe_cfg_base, 0x01A41B10 ) =  0x00000093;
		*mkptr( dfe_cfg_base, 0x01A41B14 ) =  0x00000093;
		*mkptr( dfe_cfg_base, 0x01A41B18 ) =  0x00000092;
		*mkptr( dfe_cfg_base, 0x01A41B1C ) =  0x00000092;
		*mkptr( dfe_cfg_base, 0x01A41B20 ) =  0x00000092;
		*mkptr( dfe_cfg_base, 0x01A41B24 ) =  0x00000092;
		*mkptr( dfe_cfg_base, 0x01A41B28 ) =  0x00000091;
		*mkptr( dfe_cfg_base, 0x01A41B2C ) =  0x00000091;
		*mkptr( dfe_cfg_base, 0x01A41B30 ) =  0x00000091;
		*mkptr( dfe_cfg_base, 0x01A41B34 ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41B38 ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41B3C ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41B40 ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41B44 ) =  0x0000008F;
		*mkptr( dfe_cfg_base, 0x01A41B48 ) =  0x0000008F;
		*mkptr( dfe_cfg_base, 0x01A41B4C ) =  0x0000008F;
		*mkptr( dfe_cfg_base, 0x01A41B50 ) =  0x0000008F;
		*mkptr( dfe_cfg_base, 0x01A41B54 ) =  0x0000008E;
		*mkptr( dfe_cfg_base, 0x01A41B58 ) =  0x0000008E;
		*mkptr( dfe_cfg_base, 0x01A41B5C ) =  0x0000008E;
		*mkptr( dfe_cfg_base, 0x01A41B60 ) =  0x0000008E;
		*mkptr( dfe_cfg_base, 0x01A41B64 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x01A41B68 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x01A41B6C ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x01A41B70 ) =  0x0000008D;
		*mkptr( dfe_cfg_base, 0x01A41B74 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41B78 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41B7C ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41B80 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41B84 ) =  0x0000008B;
		*mkptr( dfe_cfg_base, 0x01A41B88 ) =  0x0000008B;
		*mkptr( dfe_cfg_base, 0x01A41B8C ) =  0x0000008B;
		*mkptr( dfe_cfg_base, 0x01A41B90 ) =  0x0000008B;
		*mkptr( dfe_cfg_base, 0x01A41B94 ) =  0x0000008A;
		*mkptr( dfe_cfg_base, 0x01A41B98 ) =  0x0000008A;
		*mkptr( dfe_cfg_base, 0x01A41B9C ) =  0x0000008A;
		*mkptr( dfe_cfg_base, 0x01A41BA0 ) =  0x0000008A;
		*mkptr( dfe_cfg_base, 0x01A41BA4 ) =  0x00000089;
		*mkptr( dfe_cfg_base, 0x01A41BA8 ) =  0x00000089;
		*mkptr( dfe_cfg_base, 0x01A41BAC ) =  0x00000089;
		*mkptr( dfe_cfg_base, 0x01A41BB0 ) =  0x00000089;
		*mkptr( dfe_cfg_base, 0x01A41BB4 ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41BB8 ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41BBC ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41BC0 ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41BC4 ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41BC8 ) =  0x00000087;
		*mkptr( dfe_cfg_base, 0x01A41BCC ) =  0x00000087;
		*mkptr( dfe_cfg_base, 0x01A41BD0 ) =  0x00000087;
		*mkptr( dfe_cfg_base, 0x01A41BD4 ) =  0x00000087;
		*mkptr( dfe_cfg_base, 0x01A41BD8 ) =  0x00000086;
		*mkptr( dfe_cfg_base, 0x01A41BDC ) =  0x00000086;
		*mkptr( dfe_cfg_base, 0x01A41BE0 ) =  0x00000086;
		*mkptr( dfe_cfg_base, 0x01A41BE4 ) =  0x00000086;
		*mkptr( dfe_cfg_base, 0x01A41BE8 ) =  0x00000086;
		*mkptr( dfe_cfg_base, 0x01A41BEC ) =  0x00000085;
		*mkptr( dfe_cfg_base, 0x01A41BF0 ) =  0x00000085;
		*mkptr( dfe_cfg_base, 0x01A41BF4 ) =  0x00000085;
		*mkptr( dfe_cfg_base, 0x01A41BF8 ) =  0x00000085;
		*mkptr( dfe_cfg_base, 0x01A41BFC ) =  0x00000084;
		*mkptr( dfe_cfg_base, 0x01A41C00 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A41C04 ) =  0x000000FC;
		*mkptr( dfe_cfg_base, 0x01A41C08 ) =  0x000078D7;
		*mkptr( dfe_cfg_base, 0x01A41C0C ) =  0x000000F8;
		*mkptr( dfe_cfg_base, 0x01A41C10 ) =  0x00007215;
		*mkptr( dfe_cfg_base, 0x01A41C14 ) =  0x000000F4;
		*mkptr( dfe_cfg_base, 0x01A41C18 ) =  0x00006BB3;
		*mkptr( dfe_cfg_base, 0x01A41C1C ) =  0x000000F0;
		*mkptr( dfe_cfg_base, 0x01A41C20 ) =  0x000065AD;
		*mkptr( dfe_cfg_base, 0x01A41C24 ) =  0x000000EC;
		*mkptr( dfe_cfg_base, 0x01A41C28 ) =  0x00005FFD;
		*mkptr( dfe_cfg_base, 0x01A41C2C ) =  0x000000E8;
		*mkptr( dfe_cfg_base, 0x01A41C30 ) =  0x00005A9E;
		*mkptr( dfe_cfg_base, 0x01A41C34 ) =  0x000000E4;
		*mkptr( dfe_cfg_base, 0x01A41C38 ) =  0x0000558C;
		*mkptr( dfe_cfg_base, 0x01A41C3C ) =  0x000000E0;
		*mkptr( dfe_cfg_base, 0x01A41C40 ) =  0x000050C3;
		*mkptr( dfe_cfg_base, 0x01A41C44 ) =  0x000000DC;
		*mkptr( dfe_cfg_base, 0x01A41C48 ) =  0x00004C3F;
		*mkptr( dfe_cfg_base, 0x01A41C4C ) =  0x000000D8;
		*mkptr( dfe_cfg_base, 0x01A41C50 ) =  0x000047FB;
		*mkptr( dfe_cfg_base, 0x01A41C54 ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x01A41C58 ) =  0x000043F4;
		*mkptr( dfe_cfg_base, 0x01A41C5C ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A41C60 ) =  0x00004027;
		*mkptr( dfe_cfg_base, 0x01A41C64 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A41C68 ) =  0x00003C90;
		*mkptr( dfe_cfg_base, 0x01A41C6C ) =  0x000000C8;
		*mkptr( dfe_cfg_base, 0x01A41C70 ) =  0x0000392D;
		*mkptr( dfe_cfg_base, 0x01A41C74 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A41C78 ) =  0x000035FA;
		*mkptr( dfe_cfg_base, 0x01A41C7C ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A41C80 ) =  0x000032F5;
		*mkptr( dfe_cfg_base, 0x01A41C84 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A41C88 ) =  0x0000301B;
		*mkptr( dfe_cfg_base, 0x01A41C8C ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A41C90 ) =  0x00002D6B;
		*mkptr( dfe_cfg_base, 0x01A41C94 ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A41C98 ) =  0x00002AE0;
		*mkptr( dfe_cfg_base, 0x01A41C9C ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A41CA0 ) =  0x0000287A;
		*mkptr( dfe_cfg_base, 0x01A41CA4 ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A41CA8 ) =  0x00002637;
		*mkptr( dfe_cfg_base, 0x01A41CAC ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A41CB0 ) =  0x00002413;
		*mkptr( dfe_cfg_base, 0x01A41CB4 ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A41CB8 ) =  0x0000220F;
		*mkptr( dfe_cfg_base, 0x01A41CBC ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A41CC0 ) =  0x00002027;
		*mkptr( dfe_cfg_base, 0x01A41CC4 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A41CC8 ) =  0x00001E5B;
		*mkptr( dfe_cfg_base, 0x01A41CCC ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A41CD0 ) =  0x00001CA8;
		*mkptr( dfe_cfg_base, 0x01A41CD4 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A41CD8 ) =  0x00001B0D;
		*mkptr( dfe_cfg_base, 0x01A41CDC ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41CE0 ) =  0x0000198A;
		*mkptr( dfe_cfg_base, 0x01A41CE4 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41CE8 ) =  0x0000181C;
		*mkptr( dfe_cfg_base, 0x01A41CEC ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41CF0 ) =  0x000016C3;
		*mkptr( dfe_cfg_base, 0x01A41CF4 ) =  0x00000084;
		*mkptr( dfe_cfg_base, 0x01A41CF8 ) =  0x0000157D;
		*mkptr( dfe_cfg_base, 0x01A41CFC ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01A41D00 ) =  0x00001449;
		*mkptr( dfe_cfg_base, 0x01A41D04 ) =  0x0000007C;
		*mkptr( dfe_cfg_base, 0x01A41D08 ) =  0x00001327;
		*mkptr( dfe_cfg_base, 0x01A41D0C ) =  0x00000078;
		*mkptr( dfe_cfg_base, 0x01A41D10 ) =  0x00001215;
		*mkptr( dfe_cfg_base, 0x01A41D14 ) =  0x00000074;
		*mkptr( dfe_cfg_base, 0x01A41D18 ) =  0x00001112;
		*mkptr( dfe_cfg_base, 0x01A41D1C ) =  0x00000070;
		*mkptr( dfe_cfg_base, 0x01A41D20 ) =  0x0000101D;
		*mkptr( dfe_cfg_base, 0x01A41D24 ) =  0x0000006C;
		*mkptr( dfe_cfg_base, 0x01A41D28 ) =  0x00000F36;
		*mkptr( dfe_cfg_base, 0x01A41D2C ) =  0x00000068;
		*mkptr( dfe_cfg_base, 0x01A41D30 ) =  0x00000E5D;
		*mkptr( dfe_cfg_base, 0x01A41D34 ) =  0x00000064;
		*mkptr( dfe_cfg_base, 0x01A41D38 ) =  0x00000D8F;
		*mkptr( dfe_cfg_base, 0x01A41D3C ) =  0x00000060;
		*mkptr( dfe_cfg_base, 0x01A41D40 ) =  0x00000CCD;
		*mkptr( dfe_cfg_base, 0x01A41D44 ) =  0x0000005C;
		*mkptr( dfe_cfg_base, 0x01A41D48 ) =  0x00000C15;
		*mkptr( dfe_cfg_base, 0x01A41D4C ) =  0x00000058;
		*mkptr( dfe_cfg_base, 0x01A41D50 ) =  0x00000B68;
		*mkptr( dfe_cfg_base, 0x01A41D54 ) =  0x00000054;
		*mkptr( dfe_cfg_base, 0x01A41D58 ) =  0x00000AC5;
		*mkptr( dfe_cfg_base, 0x01A41D5C ) =  0x00000050;
		*mkptr( dfe_cfg_base, 0x01A41D60 ) =  0x00000A2B;
		*mkptr( dfe_cfg_base, 0x01A41D64 ) =  0x0000004C;
		*mkptr( dfe_cfg_base, 0x01A41D68 ) =  0x00000999;
		*mkptr( dfe_cfg_base, 0x01A41D6C ) =  0x00000048;
		*mkptr( dfe_cfg_base, 0x01A41D70 ) =  0x00000910;
		*mkptr( dfe_cfg_base, 0x01A41D74 ) =  0x00000044;
		*mkptr( dfe_cfg_base, 0x01A41D78 ) =  0x0000088E;
		*mkptr( dfe_cfg_base, 0x01A41D7C ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x01A41D80 ) =  0x00000814;
		*mkptr( dfe_cfg_base, 0x01A41D84 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x01A41D88 ) =  0x000007A0;
		*mkptr( dfe_cfg_base, 0x01A41D8C ) =  0x00000038;
		*mkptr( dfe_cfg_base, 0x01A41D90 ) =  0x00000733;
		*mkptr( dfe_cfg_base, 0x01A41D94 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x01A41D98 ) =  0x000006CC;
		*mkptr( dfe_cfg_base, 0x01A41D9C ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01A41DA0 ) =  0x0000066A;
		*mkptr( dfe_cfg_base, 0x01A41DA4 ) =  0x0000002C;
		*mkptr( dfe_cfg_base, 0x01A41DA8 ) =  0x0000060E;
		*mkptr( dfe_cfg_base, 0x01A41DAC ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x01A41DB0 ) =  0x000005B8;
		*mkptr( dfe_cfg_base, 0x01A41DB4 ) =  0x00000024;
		*mkptr( dfe_cfg_base, 0x01A41DB8 ) =  0x00000566;
		*mkptr( dfe_cfg_base, 0x01A41DBC ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01A41DC0 ) =  0x00000519;
		*mkptr( dfe_cfg_base, 0x01A41DC4 ) =  0x0000001C;
		*mkptr( dfe_cfg_base, 0x01A41DC8 ) =  0x000004D0;
		*mkptr( dfe_cfg_base, 0x01A41DCC ) =  0x00000018;
		*mkptr( dfe_cfg_base, 0x01A41DD0 ) =  0x0000048B;
		*mkptr( dfe_cfg_base, 0x01A41DD4 ) =  0x00000014;
		*mkptr( dfe_cfg_base, 0x01A41DD8 ) =  0x0000044A;
		*mkptr( dfe_cfg_base, 0x01A41DDC ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41DE0 ) =  0x0000040C;
		*mkptr( dfe_cfg_base, 0x01A41DE4 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01A41DE8 ) =  0x000003D2;
		*mkptr( dfe_cfg_base, 0x01A41DEC ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01A41DF0 ) =  0x0000039C;
		*mkptr( dfe_cfg_base, 0x01A41DF4 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A41DF8 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x01A41E00 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A41E04 ) =  0x000000FC;
		*mkptr( dfe_cfg_base, 0x01A41E08 ) =  0x000078D7;
		*mkptr( dfe_cfg_base, 0x01A41E0C ) =  0x000000F8;
		*mkptr( dfe_cfg_base, 0x01A41E10 ) =  0x00007215;
		*mkptr( dfe_cfg_base, 0x01A41E14 ) =  0x000000F4;
		*mkptr( dfe_cfg_base, 0x01A41E18 ) =  0x00006BB3;
		*mkptr( dfe_cfg_base, 0x01A41E1C ) =  0x000000F0;
		*mkptr( dfe_cfg_base, 0x01A41E20 ) =  0x000065AD;
		*mkptr( dfe_cfg_base, 0x01A41E24 ) =  0x000000EC;
		*mkptr( dfe_cfg_base, 0x01A41E28 ) =  0x00005FFD;
		*mkptr( dfe_cfg_base, 0x01A41E2C ) =  0x000000E8;
		*mkptr( dfe_cfg_base, 0x01A41E30 ) =  0x00005A9E;
		*mkptr( dfe_cfg_base, 0x01A41E34 ) =  0x000000E4;
		*mkptr( dfe_cfg_base, 0x01A41E38 ) =  0x0000558C;
		*mkptr( dfe_cfg_base, 0x01A41E3C ) =  0x000000E0;
		*mkptr( dfe_cfg_base, 0x01A41E40 ) =  0x000050C3;
		*mkptr( dfe_cfg_base, 0x01A41E44 ) =  0x000000DC;
		*mkptr( dfe_cfg_base, 0x01A41E48 ) =  0x00004C3F;
		*mkptr( dfe_cfg_base, 0x01A41E4C ) =  0x000000D8;
		*mkptr( dfe_cfg_base, 0x01A41E50 ) =  0x000047FB;
		*mkptr( dfe_cfg_base, 0x01A41E54 ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x01A41E58 ) =  0x000043F4;
		*mkptr( dfe_cfg_base, 0x01A41E5C ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A41E60 ) =  0x00004027;
		*mkptr( dfe_cfg_base, 0x01A41E64 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A41E68 ) =  0x00003C90;
		*mkptr( dfe_cfg_base, 0x01A41E6C ) =  0x000000C8;
		*mkptr( dfe_cfg_base, 0x01A41E70 ) =  0x0000392D;
		*mkptr( dfe_cfg_base, 0x01A41E74 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A41E78 ) =  0x000035FA;
		*mkptr( dfe_cfg_base, 0x01A41E7C ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A41E80 ) =  0x000032F5;
		*mkptr( dfe_cfg_base, 0x01A41E84 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A41E88 ) =  0x0000301B;
		*mkptr( dfe_cfg_base, 0x01A41E8C ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A41E90 ) =  0x00002D6B;
		*mkptr( dfe_cfg_base, 0x01A41E94 ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A41E98 ) =  0x00002AE0;
		*mkptr( dfe_cfg_base, 0x01A41E9C ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A41EA0 ) =  0x0000287A;
		*mkptr( dfe_cfg_base, 0x01A41EA4 ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A41EA8 ) =  0x00002637;
		*mkptr( dfe_cfg_base, 0x01A41EAC ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A41EB0 ) =  0x00002413;
		*mkptr( dfe_cfg_base, 0x01A41EB4 ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A41EB8 ) =  0x0000220F;
		*mkptr( dfe_cfg_base, 0x01A41EBC ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A41EC0 ) =  0x00002027;
		*mkptr( dfe_cfg_base, 0x01A41EC4 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A41EC8 ) =  0x00001E5B;
		*mkptr( dfe_cfg_base, 0x01A41ECC ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A41ED0 ) =  0x00001CA8;
		*mkptr( dfe_cfg_base, 0x01A41ED4 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A41ED8 ) =  0x00001B0D;
		*mkptr( dfe_cfg_base, 0x01A41EDC ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A41EE0 ) =  0x0000198A;
		*mkptr( dfe_cfg_base, 0x01A41EE4 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A41EE8 ) =  0x0000181C;
		*mkptr( dfe_cfg_base, 0x01A41EEC ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A41EF0 ) =  0x000016C3;
		*mkptr( dfe_cfg_base, 0x01A41EF4 ) =  0x00000084;
		*mkptr( dfe_cfg_base, 0x01A41EF8 ) =  0x0000157D;
		*mkptr( dfe_cfg_base, 0x01A41EFC ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01A41F00 ) =  0x00001449;
		*mkptr( dfe_cfg_base, 0x01A41F04 ) =  0x0000007C;
		*mkptr( dfe_cfg_base, 0x01A41F08 ) =  0x00001327;
		*mkptr( dfe_cfg_base, 0x01A41F0C ) =  0x00000078;
		*mkptr( dfe_cfg_base, 0x01A41F10 ) =  0x00001215;
		*mkptr( dfe_cfg_base, 0x01A41F14 ) =  0x00000074;
		*mkptr( dfe_cfg_base, 0x01A41F18 ) =  0x00001112;
		*mkptr( dfe_cfg_base, 0x01A41F1C ) =  0x00000070;
		*mkptr( dfe_cfg_base, 0x01A41F20 ) =  0x0000101D;
		*mkptr( dfe_cfg_base, 0x01A41F24 ) =  0x0000006C;
		*mkptr( dfe_cfg_base, 0x01A41F28 ) =  0x00000F36;
		*mkptr( dfe_cfg_base, 0x01A41F2C ) =  0x00000068;
		*mkptr( dfe_cfg_base, 0x01A41F30 ) =  0x00000E5D;
		*mkptr( dfe_cfg_base, 0x01A41F34 ) =  0x00000064;
		*mkptr( dfe_cfg_base, 0x01A41F38 ) =  0x00000D8F;
		*mkptr( dfe_cfg_base, 0x01A41F3C ) =  0x00000060;
		*mkptr( dfe_cfg_base, 0x01A41F40 ) =  0x00000CCD;
		*mkptr( dfe_cfg_base, 0x01A41F44 ) =  0x0000005C;
		*mkptr( dfe_cfg_base, 0x01A41F48 ) =  0x00000C15;
		*mkptr( dfe_cfg_base, 0x01A41F4C ) =  0x00000058;
		*mkptr( dfe_cfg_base, 0x01A41F50 ) =  0x00000B68;
		*mkptr( dfe_cfg_base, 0x01A41F54 ) =  0x00000054;
		*mkptr( dfe_cfg_base, 0x01A41F58 ) =  0x00000AC5;
		*mkptr( dfe_cfg_base, 0x01A41F5C ) =  0x00000050;
		*mkptr( dfe_cfg_base, 0x01A41F60 ) =  0x00000A2B;
		*mkptr( dfe_cfg_base, 0x01A41F64 ) =  0x0000004C;
		*mkptr( dfe_cfg_base, 0x01A41F68 ) =  0x00000999;
		*mkptr( dfe_cfg_base, 0x01A41F6C ) =  0x00000048;
		*mkptr( dfe_cfg_base, 0x01A41F70 ) =  0x00000910;
		*mkptr( dfe_cfg_base, 0x01A41F74 ) =  0x00000044;
		*mkptr( dfe_cfg_base, 0x01A41F78 ) =  0x0000088E;
		*mkptr( dfe_cfg_base, 0x01A41F7C ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x01A41F80 ) =  0x00000814;
		*mkptr( dfe_cfg_base, 0x01A41F84 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x01A41F88 ) =  0x000007A0;
		*mkptr( dfe_cfg_base, 0x01A41F8C ) =  0x00000038;
		*mkptr( dfe_cfg_base, 0x01A41F90 ) =  0x00000733;
		*mkptr( dfe_cfg_base, 0x01A41F94 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x01A41F98 ) =  0x000006CC;
		*mkptr( dfe_cfg_base, 0x01A41F9C ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01A41FA0 ) =  0x0000066A;
		*mkptr( dfe_cfg_base, 0x01A41FA4 ) =  0x0000002C;
		*mkptr( dfe_cfg_base, 0x01A41FA8 ) =  0x0000060E;
		*mkptr( dfe_cfg_base, 0x01A41FAC ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x01A41FB0 ) =  0x000005B8;
		*mkptr( dfe_cfg_base, 0x01A41FB4 ) =  0x00000024;
		*mkptr( dfe_cfg_base, 0x01A41FB8 ) =  0x00000566;
		*mkptr( dfe_cfg_base, 0x01A41FBC ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01A41FC0 ) =  0x00000519;
		*mkptr( dfe_cfg_base, 0x01A41FC4 ) =  0x0000001C;
		*mkptr( dfe_cfg_base, 0x01A41FC8 ) =  0x000004D0;
		*mkptr( dfe_cfg_base, 0x01A41FCC ) =  0x00000018;
		*mkptr( dfe_cfg_base, 0x01A41FD0 ) =  0x0000048B;
		*mkptr( dfe_cfg_base, 0x01A41FD4 ) =  0x00000014;
		*mkptr( dfe_cfg_base, 0x01A41FD8 ) =  0x0000044A;
		*mkptr( dfe_cfg_base, 0x01A41FDC ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41FE0 ) =  0x0000040C;
		*mkptr( dfe_cfg_base, 0x01A41FE4 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01A41FE8 ) =  0x000003D2;
		*mkptr( dfe_cfg_base, 0x01A41FEC ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01A41FF0 ) =  0x0000039C;
		*mkptr( dfe_cfg_base, 0x01A41FF4 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A41FF8 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x01A42000 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A42004 ) =  0x000000FC;
		*mkptr( dfe_cfg_base, 0x01A42008 ) =  0x000078D7;
		*mkptr( dfe_cfg_base, 0x01A4200C ) =  0x000000F8;
		*mkptr( dfe_cfg_base, 0x01A42010 ) =  0x00007215;
		*mkptr( dfe_cfg_base, 0x01A42014 ) =  0x000000F4;
		*mkptr( dfe_cfg_base, 0x01A42018 ) =  0x00006BB3;
		*mkptr( dfe_cfg_base, 0x01A4201C ) =  0x000000F0;
		*mkptr( dfe_cfg_base, 0x01A42020 ) =  0x000065AD;
		*mkptr( dfe_cfg_base, 0x01A42024 ) =  0x000000EC;
		*mkptr( dfe_cfg_base, 0x01A42028 ) =  0x00005FFD;
		*mkptr( dfe_cfg_base, 0x01A4202C ) =  0x000000E8;
		*mkptr( dfe_cfg_base, 0x01A42030 ) =  0x00005A9E;
		*mkptr( dfe_cfg_base, 0x01A42034 ) =  0x000000E4;
		*mkptr( dfe_cfg_base, 0x01A42038 ) =  0x0000558C;
		*mkptr( dfe_cfg_base, 0x01A4203C ) =  0x000000E0;
		*mkptr( dfe_cfg_base, 0x01A42040 ) =  0x000050C3;
		*mkptr( dfe_cfg_base, 0x01A42044 ) =  0x000000DC;
		*mkptr( dfe_cfg_base, 0x01A42048 ) =  0x00004C3F;
		*mkptr( dfe_cfg_base, 0x01A4204C ) =  0x000000D8;
		*mkptr( dfe_cfg_base, 0x01A42050 ) =  0x000047FB;
		*mkptr( dfe_cfg_base, 0x01A42054 ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x01A42058 ) =  0x000043F4;
		*mkptr( dfe_cfg_base, 0x01A4205C ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A42060 ) =  0x00004027;
		*mkptr( dfe_cfg_base, 0x01A42064 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A42068 ) =  0x00003C90;
		*mkptr( dfe_cfg_base, 0x01A4206C ) =  0x000000C8;
		*mkptr( dfe_cfg_base, 0x01A42070 ) =  0x0000392D;
		*mkptr( dfe_cfg_base, 0x01A42074 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A42078 ) =  0x000035FA;
		*mkptr( dfe_cfg_base, 0x01A4207C ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A42080 ) =  0x000032F5;
		*mkptr( dfe_cfg_base, 0x01A42084 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A42088 ) =  0x0000301B;
		*mkptr( dfe_cfg_base, 0x01A4208C ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A42090 ) =  0x00002D6B;
		*mkptr( dfe_cfg_base, 0x01A42094 ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A42098 ) =  0x00002AE0;
		*mkptr( dfe_cfg_base, 0x01A4209C ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A420A0 ) =  0x0000287A;
		*mkptr( dfe_cfg_base, 0x01A420A4 ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A420A8 ) =  0x00002637;
		*mkptr( dfe_cfg_base, 0x01A420AC ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A420B0 ) =  0x00002413;
		*mkptr( dfe_cfg_base, 0x01A420B4 ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A420B8 ) =  0x0000220F;
		*mkptr( dfe_cfg_base, 0x01A420BC ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A420C0 ) =  0x00002027;
		*mkptr( dfe_cfg_base, 0x01A420C4 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A420C8 ) =  0x00001E5B;
		*mkptr( dfe_cfg_base, 0x01A420CC ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A420D0 ) =  0x00001CA8;
		*mkptr( dfe_cfg_base, 0x01A420D4 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A420D8 ) =  0x00001B0D;
		*mkptr( dfe_cfg_base, 0x01A420DC ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A420E0 ) =  0x0000198A;
		*mkptr( dfe_cfg_base, 0x01A420E4 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A420E8 ) =  0x0000181C;
		*mkptr( dfe_cfg_base, 0x01A420EC ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A420F0 ) =  0x000016C3;
		*mkptr( dfe_cfg_base, 0x01A420F4 ) =  0x00000084;
		*mkptr( dfe_cfg_base, 0x01A420F8 ) =  0x0000157D;
		*mkptr( dfe_cfg_base, 0x01A420FC ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01A42100 ) =  0x00001449;
		*mkptr( dfe_cfg_base, 0x01A42104 ) =  0x0000007C;
		*mkptr( dfe_cfg_base, 0x01A42108 ) =  0x00001327;
		*mkptr( dfe_cfg_base, 0x01A4210C ) =  0x00000078;
		*mkptr( dfe_cfg_base, 0x01A42110 ) =  0x00001215;
		*mkptr( dfe_cfg_base, 0x01A42114 ) =  0x00000074;
		*mkptr( dfe_cfg_base, 0x01A42118 ) =  0x00001112;
		*mkptr( dfe_cfg_base, 0x01A4211C ) =  0x00000070;
		*mkptr( dfe_cfg_base, 0x01A42120 ) =  0x0000101D;
		*mkptr( dfe_cfg_base, 0x01A42124 ) =  0x0000006C;
		*mkptr( dfe_cfg_base, 0x01A42128 ) =  0x00000F36;
		*mkptr( dfe_cfg_base, 0x01A4212C ) =  0x00000068;
		*mkptr( dfe_cfg_base, 0x01A42130 ) =  0x00000E5D;
		*mkptr( dfe_cfg_base, 0x01A42134 ) =  0x00000064;
		*mkptr( dfe_cfg_base, 0x01A42138 ) =  0x00000D8F;
		*mkptr( dfe_cfg_base, 0x01A4213C ) =  0x00000060;
		*mkptr( dfe_cfg_base, 0x01A42140 ) =  0x00000CCD;
		*mkptr( dfe_cfg_base, 0x01A42144 ) =  0x0000005C;
		*mkptr( dfe_cfg_base, 0x01A42148 ) =  0x00000C15;
		*mkptr( dfe_cfg_base, 0x01A4214C ) =  0x00000058;
		*mkptr( dfe_cfg_base, 0x01A42150 ) =  0x00000B68;
		*mkptr( dfe_cfg_base, 0x01A42154 ) =  0x00000054;
		*mkptr( dfe_cfg_base, 0x01A42158 ) =  0x00000AC5;
		*mkptr( dfe_cfg_base, 0x01A4215C ) =  0x00000050;
		*mkptr( dfe_cfg_base, 0x01A42160 ) =  0x00000A2B;
		*mkptr( dfe_cfg_base, 0x01A42164 ) =  0x0000004C;
		*mkptr( dfe_cfg_base, 0x01A42168 ) =  0x00000999;
		*mkptr( dfe_cfg_base, 0x01A4216C ) =  0x00000048;
		*mkptr( dfe_cfg_base, 0x01A42170 ) =  0x00000910;
		*mkptr( dfe_cfg_base, 0x01A42174 ) =  0x00000044;
		*mkptr( dfe_cfg_base, 0x01A42178 ) =  0x0000088E;
		*mkptr( dfe_cfg_base, 0x01A4217C ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x01A42180 ) =  0x00000814;
		*mkptr( dfe_cfg_base, 0x01A42184 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x01A42188 ) =  0x000007A0;
		*mkptr( dfe_cfg_base, 0x01A4218C ) =  0x00000038;
		*mkptr( dfe_cfg_base, 0x01A42190 ) =  0x00000733;
		*mkptr( dfe_cfg_base, 0x01A42194 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x01A42198 ) =  0x000006CC;
		*mkptr( dfe_cfg_base, 0x01A4219C ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01A421A0 ) =  0x0000066A;
		*mkptr( dfe_cfg_base, 0x01A421A4 ) =  0x0000002C;
		*mkptr( dfe_cfg_base, 0x01A421A8 ) =  0x0000060E;
		*mkptr( dfe_cfg_base, 0x01A421AC ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x01A421B0 ) =  0x000005B8;
		*mkptr( dfe_cfg_base, 0x01A421B4 ) =  0x00000024;
		*mkptr( dfe_cfg_base, 0x01A421B8 ) =  0x00000566;
		*mkptr( dfe_cfg_base, 0x01A421BC ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01A421C0 ) =  0x00000519;
		*mkptr( dfe_cfg_base, 0x01A421C4 ) =  0x0000001C;
		*mkptr( dfe_cfg_base, 0x01A421C8 ) =  0x000004D0;
		*mkptr( dfe_cfg_base, 0x01A421CC ) =  0x00000018;
		*mkptr( dfe_cfg_base, 0x01A421D0 ) =  0x0000048B;
		*mkptr( dfe_cfg_base, 0x01A421D4 ) =  0x00000014;
		*mkptr( dfe_cfg_base, 0x01A421D8 ) =  0x0000044A;
		*mkptr( dfe_cfg_base, 0x01A421DC ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A421E0 ) =  0x0000040C;
		*mkptr( dfe_cfg_base, 0x01A421E4 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01A421E8 ) =  0x000003D2;
		*mkptr( dfe_cfg_base, 0x01A421EC ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01A421F0 ) =  0x0000039C;
		*mkptr( dfe_cfg_base, 0x01A421F4 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A421F8 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x01A42200 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A42204 ) =  0x000000FC;
		*mkptr( dfe_cfg_base, 0x01A42208 ) =  0x000078D7;
		*mkptr( dfe_cfg_base, 0x01A4220C ) =  0x000000F8;
		*mkptr( dfe_cfg_base, 0x01A42210 ) =  0x00007215;
		*mkptr( dfe_cfg_base, 0x01A42214 ) =  0x000000F4;
		*mkptr( dfe_cfg_base, 0x01A42218 ) =  0x00006BB3;
		*mkptr( dfe_cfg_base, 0x01A4221C ) =  0x000000F0;
		*mkptr( dfe_cfg_base, 0x01A42220 ) =  0x000065AD;
		*mkptr( dfe_cfg_base, 0x01A42224 ) =  0x000000EC;
		*mkptr( dfe_cfg_base, 0x01A42228 ) =  0x00005FFD;
		*mkptr( dfe_cfg_base, 0x01A4222C ) =  0x000000E8;
		*mkptr( dfe_cfg_base, 0x01A42230 ) =  0x00005A9E;
		*mkptr( dfe_cfg_base, 0x01A42234 ) =  0x000000E4;
		*mkptr( dfe_cfg_base, 0x01A42238 ) =  0x0000558C;
		*mkptr( dfe_cfg_base, 0x01A4223C ) =  0x000000E0;
		*mkptr( dfe_cfg_base, 0x01A42240 ) =  0x000050C3;
		*mkptr( dfe_cfg_base, 0x01A42244 ) =  0x000000DC;
		*mkptr( dfe_cfg_base, 0x01A42248 ) =  0x00004C3F;
		*mkptr( dfe_cfg_base, 0x01A4224C ) =  0x000000D8;
		*mkptr( dfe_cfg_base, 0x01A42250 ) =  0x000047FB;
		*mkptr( dfe_cfg_base, 0x01A42254 ) =  0x000000D4;
		*mkptr( dfe_cfg_base, 0x01A42258 ) =  0x000043F4;
		*mkptr( dfe_cfg_base, 0x01A4225C ) =  0x000000D0;
		*mkptr( dfe_cfg_base, 0x01A42260 ) =  0x00004027;
		*mkptr( dfe_cfg_base, 0x01A42264 ) =  0x000000CC;
		*mkptr( dfe_cfg_base, 0x01A42268 ) =  0x00003C90;
		*mkptr( dfe_cfg_base, 0x01A4226C ) =  0x000000C8;
		*mkptr( dfe_cfg_base, 0x01A42270 ) =  0x0000392D;
		*mkptr( dfe_cfg_base, 0x01A42274 ) =  0x000000C4;
		*mkptr( dfe_cfg_base, 0x01A42278 ) =  0x000035FA;
		*mkptr( dfe_cfg_base, 0x01A4227C ) =  0x000000C0;
		*mkptr( dfe_cfg_base, 0x01A42280 ) =  0x000032F5;
		*mkptr( dfe_cfg_base, 0x01A42284 ) =  0x000000BC;
		*mkptr( dfe_cfg_base, 0x01A42288 ) =  0x0000301B;
		*mkptr( dfe_cfg_base, 0x01A4228C ) =  0x000000B8;
		*mkptr( dfe_cfg_base, 0x01A42290 ) =  0x00002D6B;
		*mkptr( dfe_cfg_base, 0x01A42294 ) =  0x000000B4;
		*mkptr( dfe_cfg_base, 0x01A42298 ) =  0x00002AE0;
		*mkptr( dfe_cfg_base, 0x01A4229C ) =  0x000000B0;
		*mkptr( dfe_cfg_base, 0x01A422A0 ) =  0x0000287A;
		*mkptr( dfe_cfg_base, 0x01A422A4 ) =  0x000000AC;
		*mkptr( dfe_cfg_base, 0x01A422A8 ) =  0x00002637;
		*mkptr( dfe_cfg_base, 0x01A422AC ) =  0x000000A8;
		*mkptr( dfe_cfg_base, 0x01A422B0 ) =  0x00002413;
		*mkptr( dfe_cfg_base, 0x01A422B4 ) =  0x000000A4;
		*mkptr( dfe_cfg_base, 0x01A422B8 ) =  0x0000220F;
		*mkptr( dfe_cfg_base, 0x01A422BC ) =  0x000000A0;
		*mkptr( dfe_cfg_base, 0x01A422C0 ) =  0x00002027;
		*mkptr( dfe_cfg_base, 0x01A422C4 ) =  0x0000009C;
		*mkptr( dfe_cfg_base, 0x01A422C8 ) =  0x00001E5B;
		*mkptr( dfe_cfg_base, 0x01A422CC ) =  0x00000098;
		*mkptr( dfe_cfg_base, 0x01A422D0 ) =  0x00001CA8;
		*mkptr( dfe_cfg_base, 0x01A422D4 ) =  0x00000094;
		*mkptr( dfe_cfg_base, 0x01A422D8 ) =  0x00001B0D;
		*mkptr( dfe_cfg_base, 0x01A422DC ) =  0x00000090;
		*mkptr( dfe_cfg_base, 0x01A422E0 ) =  0x0000198A;
		*mkptr( dfe_cfg_base, 0x01A422E4 ) =  0x0000008C;
		*mkptr( dfe_cfg_base, 0x01A422E8 ) =  0x0000181C;
		*mkptr( dfe_cfg_base, 0x01A422EC ) =  0x00000088;
		*mkptr( dfe_cfg_base, 0x01A422F0 ) =  0x000016C3;
		*mkptr( dfe_cfg_base, 0x01A422F4 ) =  0x00000084;
		*mkptr( dfe_cfg_base, 0x01A422F8 ) =  0x0000157D;
		*mkptr( dfe_cfg_base, 0x01A422FC ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01A42300 ) =  0x00001449;
		*mkptr( dfe_cfg_base, 0x01A42304 ) =  0x0000007C;
		*mkptr( dfe_cfg_base, 0x01A42308 ) =  0x00001327;
		*mkptr( dfe_cfg_base, 0x01A4230C ) =  0x00000078;
		*mkptr( dfe_cfg_base, 0x01A42310 ) =  0x00001215;
		*mkptr( dfe_cfg_base, 0x01A42314 ) =  0x00000074;
		*mkptr( dfe_cfg_base, 0x01A42318 ) =  0x00001112;
		*mkptr( dfe_cfg_base, 0x01A4231C ) =  0x00000070;
		*mkptr( dfe_cfg_base, 0x01A42320 ) =  0x0000101D;
		*mkptr( dfe_cfg_base, 0x01A42324 ) =  0x0000006C;
		*mkptr( dfe_cfg_base, 0x01A42328 ) =  0x00000F36;
		*mkptr( dfe_cfg_base, 0x01A4232C ) =  0x00000068;
		*mkptr( dfe_cfg_base, 0x01A42330 ) =  0x00000E5D;
		*mkptr( dfe_cfg_base, 0x01A42334 ) =  0x00000064;
		*mkptr( dfe_cfg_base, 0x01A42338 ) =  0x00000D8F;
		*mkptr( dfe_cfg_base, 0x01A4233C ) =  0x00000060;
		*mkptr( dfe_cfg_base, 0x01A42340 ) =  0x00000CCD;
		*mkptr( dfe_cfg_base, 0x01A42344 ) =  0x0000005C;
		*mkptr( dfe_cfg_base, 0x01A42348 ) =  0x00000C15;
		*mkptr( dfe_cfg_base, 0x01A4234C ) =  0x00000058;
		*mkptr( dfe_cfg_base, 0x01A42350 ) =  0x00000B68;
		*mkptr( dfe_cfg_base, 0x01A42354 ) =  0x00000054;
		*mkptr( dfe_cfg_base, 0x01A42358 ) =  0x00000AC5;
		*mkptr( dfe_cfg_base, 0x01A4235C ) =  0x00000050;
		*mkptr( dfe_cfg_base, 0x01A42360 ) =  0x00000A2B;
		*mkptr( dfe_cfg_base, 0x01A42364 ) =  0x0000004C;
		*mkptr( dfe_cfg_base, 0x01A42368 ) =  0x00000999;
		*mkptr( dfe_cfg_base, 0x01A4236C ) =  0x00000048;
		*mkptr( dfe_cfg_base, 0x01A42370 ) =  0x00000910;
		*mkptr( dfe_cfg_base, 0x01A42374 ) =  0x00000044;
		*mkptr( dfe_cfg_base, 0x01A42378 ) =  0x0000088E;
		*mkptr( dfe_cfg_base, 0x01A4237C ) =  0x00000040;
		*mkptr( dfe_cfg_base, 0x01A42380 ) =  0x00000814;
		*mkptr( dfe_cfg_base, 0x01A42384 ) =  0x0000003C;
		*mkptr( dfe_cfg_base, 0x01A42388 ) =  0x000007A0;
		*mkptr( dfe_cfg_base, 0x01A4238C ) =  0x00000038;
		*mkptr( dfe_cfg_base, 0x01A42390 ) =  0x00000733;
		*mkptr( dfe_cfg_base, 0x01A42394 ) =  0x00000034;
		*mkptr( dfe_cfg_base, 0x01A42398 ) =  0x000006CC;
		*mkptr( dfe_cfg_base, 0x01A4239C ) =  0x00000030;
		*mkptr( dfe_cfg_base, 0x01A423A0 ) =  0x0000066A;
		*mkptr( dfe_cfg_base, 0x01A423A4 ) =  0x0000002C;
		*mkptr( dfe_cfg_base, 0x01A423A8 ) =  0x0000060E;
		*mkptr( dfe_cfg_base, 0x01A423AC ) =  0x00000028;
		*mkptr( dfe_cfg_base, 0x01A423B0 ) =  0x000005B8;
		*mkptr( dfe_cfg_base, 0x01A423B4 ) =  0x00000024;
		*mkptr( dfe_cfg_base, 0x01A423B8 ) =  0x00000566;
		*mkptr( dfe_cfg_base, 0x01A423BC ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01A423C0 ) =  0x00000519;
		*mkptr( dfe_cfg_base, 0x01A423C4 ) =  0x0000001C;
		*mkptr( dfe_cfg_base, 0x01A423C8 ) =  0x000004D0;
		*mkptr( dfe_cfg_base, 0x01A423CC ) =  0x00000018;
		*mkptr( dfe_cfg_base, 0x01A423D0 ) =  0x0000048B;
		*mkptr( dfe_cfg_base, 0x01A423D4 ) =  0x00000014;
		*mkptr( dfe_cfg_base, 0x01A423D8 ) =  0x0000044A;
		*mkptr( dfe_cfg_base, 0x01A423DC ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A423E0 ) =  0x0000040C;
		*mkptr( dfe_cfg_base, 0x01A423E4 ) =  0x0000000C;
		*mkptr( dfe_cfg_base, 0x01A423E8 ) =  0x000003D2;
		*mkptr( dfe_cfg_base, 0x01A423EC ) =  0x00000008;
		*mkptr( dfe_cfg_base, 0x01A423F0 ) =  0x0000039C;
		*mkptr( dfe_cfg_base, 0x01A423F4 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A423F8 ) =  0x00000368;
		*mkptr( dfe_cfg_base, 0x01A41400 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41440 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41480 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A414C0 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A41500 ) =  0x000003E8;
		*mkptr( dfe_cfg_base, 0x01A41508 ) =  0x000007D0;
		*mkptr( dfe_cfg_base, 0x01A006EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A006F0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A006FC ) =  0x00003000;
		*mkptr( dfe_cfg_base, 0x01A00A0C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A00A14 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A0020C ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01A40220 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A402A0 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A40320 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A403A0 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A40020 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A400A0 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A40120 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A401A0 ) =  0x00007FFF;
		*mkptr( dfe_cfg_base, 0x01A0021C ) =  0x00000011;
		*mkptr( dfe_cfg_base, 0x01A00220 ) =  0x000007B0;
		*mkptr( dfe_cfg_base, 0x01A00224 ) =  0x000000F0;
		*mkptr( dfe_cfg_base, 0x01A00228 ) =  0x00001280;
		*mkptr( dfe_cfg_base, 0x01A002D0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01A002D8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01A002C0 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01A002C8 ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01A40800 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40804 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40808 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A4080C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40810 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40814 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40818 ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A4081C ) =  0x00000005;
		*mkptr( dfe_cfg_base, 0x01A40820 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40824 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40828 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4082C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40830 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40834 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40838 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4083C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40840 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40844 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40848 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4084C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40850 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40854 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40858 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4085C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40860 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40864 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40868 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4086C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40870 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40874 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40878 ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A4087C ) =  0x00000004;
		*mkptr( dfe_cfg_base, 0x01A40880 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40884 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40888 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4088C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40890 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40894 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40898 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4089C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408A0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408A4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408A8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408AC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408B0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408B4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408B8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408BC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408C4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408C8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408CC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408D0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408D4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408D8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408DC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408E0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408E4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408E8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408EC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408F0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408F4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408F8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A408FC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40900 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40904 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40908 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4090C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40910 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40914 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40918 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4091C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40920 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40924 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40928 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4092C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40930 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40934 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40938 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4093C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40940 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40944 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40948 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4094C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40950 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40954 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40958 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4095C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40960 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40964 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40968 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4096C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40970 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40974 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40978 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4097C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40980 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40984 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40988 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4098C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40990 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40994 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40998 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A4099C ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409A0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409A4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409A8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409AC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409B0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409B4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409B8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409BC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409C0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409C4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409C8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409CC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409D0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409D4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409D8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409DC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409E0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409E4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409E8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409EC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409F0 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409F4 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409F8 ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A409FC ) =  0x00000003;
		*mkptr( dfe_cfg_base, 0x01A40A00 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A04 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A08 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A0C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A10 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A14 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A18 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A1C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A20 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A24 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A28 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A2C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A30 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A34 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A38 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A3C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A40 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A44 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A48 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A4C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A50 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A54 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A58 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A5C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A60 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A64 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A68 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A6C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A70 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A74 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A78 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A7C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A80 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A84 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A88 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A8C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A90 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A94 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A98 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40A9C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AA0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AA4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AA8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AAC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AB0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AB4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AB8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40ABC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AC0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AC4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AC8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40ACC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AD0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AD4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AD8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40ADC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AE0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AE4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AE8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AEC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AF0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AF4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AF8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40AFC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B00 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B04 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B08 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B0C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B10 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B14 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B18 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B1C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B20 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B24 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B28 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B2C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B30 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B34 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B38 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B3C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B40 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B44 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B48 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B4C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B50 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B54 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B58 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B5C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B60 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B64 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B68 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B6C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B70 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B74 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B78 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B7C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B80 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B84 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B88 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B8C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B90 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B94 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B98 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40B9C ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BA0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BA4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BA8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BAC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BB0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BB4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BB8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BBC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BC0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BC4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BC8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BCC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BD0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BD4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BD8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BDC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BE0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BE4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BE8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BEC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BF0 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BF4 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BF8 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A40BFC ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01A003EC ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A003F0 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01A003FC ) =  0x00003000;
		*mkptr( dfe_cfg_base, 0x01A0023C ) =  0x00005110;
		*mkptr( dfe_cfg_base, 0x01E80700 ) =  0x00001100;
		*mkptr( dfe_cfg_base, 0x01E80704 ) =  0x0000300C;
		*mkptr( dfe_cfg_base, 0x01E80708 ) =  0x0000300C;
		*mkptr( dfe_cfg_base, 0x01E80728 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01E8073C ) =  0x00000016;
		*mkptr( dfe_cfg_base, 0x01EC0000 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0040 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0080 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC00C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0100 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0140 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0180 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC01C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0200 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01EC0240 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01E80258 ) =  0x0000FA00;
		*mkptr( dfe_cfg_base, 0x01E80278 ) =  0x0000F400;
		*mkptr( dfe_cfg_base, 0x01E80288 ) =  0x00007100;
		*mkptr( dfe_cfg_base, 0x01E80210 ) =  0x0000001F;
		*mkptr( dfe_cfg_base, 0x01E80204 ) =  0x0000001F;
		*mkptr( dfe_cfg_base, 0x01E80218 ) =  0x00001F00;
		*mkptr( dfe_cfg_base, 0x01EC0800 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0804 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0808 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC080C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0810 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0814 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0818 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC081C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0820 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0824 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0828 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC082C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0830 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0834 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0838 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC083C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0840 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0844 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0848 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC084C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0850 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0854 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0858 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC085C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0860 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0864 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0868 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC086C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0870 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0874 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0878 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC087C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0880 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0884 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0888 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC088C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0890 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0894 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0898 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC089C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC08FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0900 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0904 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0908 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC090C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0910 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0914 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0918 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC091C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0920 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0924 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0928 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC092C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0930 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0934 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0938 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC093C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0940 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0944 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0948 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC094C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0950 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0954 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0958 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC095C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0960 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0964 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0968 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC096C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0970 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0974 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0978 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC097C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0980 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0984 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0988 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC098C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0990 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0994 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC0998 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC099C ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09A0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09A4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09A8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09AC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09B0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09B4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09B8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09BC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09C0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09C4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09C8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09CC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09D0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09D4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09D8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09DC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09E0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09E4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09E8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09EC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09F0 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09F4 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09F8 ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01EC09FC ) =  0x00008000;
		*mkptr( dfe_cfg_base, 0x01E802B0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01E802C0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01E802D0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01E802E0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01E802F0 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01C00200 ) =  0x00000002;
		*mkptr( dfe_cfg_base, 0x01C00208 ) =  0x00001111;
		*mkptr( dfe_cfg_base, 0x01C0020C ) =  0x000002FD;
		*mkptr( dfe_cfg_base, 0x01C00210 ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01C00214 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01C00218 ) =  0x00000080;
		*mkptr( dfe_cfg_base, 0x01C0021C ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01C00220 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01C00224 ) =  0x00002000;
		*mkptr( dfe_cfg_base, 0x01C00228 ) =  0x00000020;
		*mkptr( dfe_cfg_base, 0x01C0022C ) =  0x00004000;
		*mkptr( dfe_cfg_base, 0x01C00234 ) =  0x00003210;
		*mkptr( dfe_cfg_base, 0x01C00238 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C0023C ) =  0x00003210;
		*mkptr( dfe_cfg_base, 0x01C00240 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C00244 ) =  0x00100000;
		*mkptr( dfe_cfg_base, 0x01C00248 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C0024C ) =  0x00003210;
		*mkptr( dfe_cfg_base, 0x01C00250 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C00254 ) =  0x10100000;
		*mkptr( dfe_cfg_base, 0x01C00258 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C0025C ) =  0x32100000;
		*mkptr( dfe_cfg_base, 0x01C00260 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C00264 ) =  0x10100000;
		*mkptr( dfe_cfg_base, 0x01C00268 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C0026C ) =  0x32100000;
		*mkptr( dfe_cfg_base, 0x01C00270 ) =  0x0000000F;
		*mkptr( dfe_cfg_base, 0x01C00274 ) =  0x00000010;
		*mkptr( dfe_cfg_base, 0x01C00280 ) =  0x00000001;
		*mkptr( dfe_cfg_base, 0x01C00328 ) =  0x0000FFFF;
		*mkptr( dfe_cfg_base, 0x01C003E4 ) =  0x01010101;
#endif
}
#endif
#endif
///////////////////////////////////////////////////////
